#!/bin/bash

SOURCE="${BASH_SOURCE[0]}"
while [ -h "$SOURCE" ]; do # resolve $SOURCE until the file is no longer a symlink
  DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
  SOURCE="$(readlink "$SOURCE")"
  [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE" # if $SOURCE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
done
SCRIPT_DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"

LESS_BIN=$(type less 2>/dev/null)
MORE_BIN=$(type more 2>/dev/null)

ML_SERVER_ROOT="${SCRIPT_DIR}/../.."
ML_SERVER_VERSION=$(basename $(dirname ${SCRIPT_DIR}))

R_RXLIB_DIR=$(readlink -m ${ML_SERVER_ROOT}/libraries/RServer/RevoScaleR/rxLibs/x64)
PY_RXLIB_DIR=$(readlink -m ${ML_SERVER_ROOT}/libraries/PythonServer/revoscalepy/rxLibs)

R_SKU=${R_RXLIB_DIR}/sku.txt
PYTHON_SKU=${PY_RXLIB_DIR}/sku.txt

HASH="39850e113fd075bbbb7f578b97318730702d2b8c04e1a73ed77b6ba9b70017f6"

ACTIVATE=false
DEACTIVATE=false
LICENSE_ACCEPTED=false

R_INSTALLED=false
PYTHON_INSTALLED=false

function print_help {
cat <<EOF
  Usage: activate.sh
    -a, --activate         Activate the R Server installation. This is the default action.
    -d, --deactivate       Deactivate the R Server installation.
    -l, --license-accept   Accept the license, for use in automated scripts
    -h, --help             Print this help  
EOF
}

function parse_options {
  while [ $# -gt 0 ]; do
    case "$1" in
      -a|--activate)        ACTIVATE=true            ; shift ;;
      -d|--deactivate)      DEACTIVATE=true          ; shift ;;
      -l|--license-accept)  LICENSE_ACCEPTED=true    ; shift ;;
      -h|--help)            print_help               ; exit  ;;
      *)                    EXTRA_OPTIONS+=("$1")    ; shift ;;
    esac
  done
}


function check_root {
  if [[ "$(id -u)" != "0" ]]; then
      echo  "This script must be run with sudo or as root."
      exit 1
  fi
}

function detect_installations {
  if [[ -d ${R_RXLIB_DIR} ]]; then
    echo "R rxLibs detected at ${R_RXLIB_DIR}"
    R_INSTALLED=true
  fi

  if [[ -d ${PY_RXLIB_DIR} ]]; then
    echo "Python rxLibs detected at ${PY_RXLIB_DIR}"
    PYTHON_INSTALLED=true
  fi
}

function activate {
  echo "Activating Microsoft ML Server"
  if [[ ${R_INSTALLED} == true ]]; then
    echo "  - Activating R components"
    echo ${HASH} > ${R_SKU}
  fi

  if [[ ${PYTHON_INSTALLED} == true ]]; then
    echo "  - Activating Python components"
    echo ${HASH} > ${PYTHON_SKU}
  fi
}

function deactivate {
  if [[ -f ${R_SKU} ]]; then
    echo "Deactivating R Components."
    rm -f ${R_SKU}
  fi

  if [[ -f ${PYTHON_SKU} ]]; then
    echo "Deactivating Python components"
    rm -f ${PYTHON_SKU}
  fi
}

function display_eula {

  if [[ ${LICENSE_ACCEPTED} == true ]]; then
    return 0
  fi

  local eulaPath=$1
  local licenseName=$2

  echo ""
  read -p "Press [Enter] key to display the ${licenseName} license. When finished reading, press q to continue: "

  if [[ -n ${LESS_BIN} ]]; then
    less ${eulaPath}
  elif [[ -n ${MORE_BIN} ]]; then
    more ${eulaPath}
  else
    echo ""
    echo "We could not detect the less or more utilities to display the license."
    echo "Please read the license files in this directory and rerun this script"
    echo "with the --license-accept flag if you agree to the licenses."
    echo ""
    exit 1
  fi

  echo "Do you agree to the terms of the previously displayed license"
  REPLY=""
  while [[ $REPLY != y ]] && read -n1 -r -p "choose [y]es|[n]o"; do
  case $REPLY in
      y) printf "\n";;
      n) printf "\nThe EULA must be accepted before installing.\n" && exit 1;;
      *) printf "\nInvalid Input\n";;
    esac
  done
  printf "\n"
}

parse_options "$@"
check_root

if [[ ${ACTIVATE} == true && ${DEACTIVATE} == true ]]; then
  echo "Ambiguous parameter set. Activate and Deactivate are not compatable."
  exit 1
fi

if [[ ${ACTIVATE} == false && ${DEACTIVATE} == false ]]; then
  ACTIVATE=true
fi

detect_installations

if [[ ${ACTIVATE} == true ]]; then
  display_eula ${SCRIPT_DIR}/MRS_EULA.txt "Microsoft Machine Learning Server"
  activate
fi

if [[ ${DEACTIVATE} == true ]]; then
  deactivate
fi
