
# Gradient descent base class; inherit from this for explicit models such as logistic regression;
# the "fn" (objective function) and "gradientFn" (gradient of objective w/- parameters) methods must be 
# specified in the child classes
"PemaGradDescent" <- 
	setPemaClass("PemaGradDescent", contains="PemaBaseClass", 
	
	# list the fields (member variables)
	fields = list(
		formula = "ANY",
		stepLen = "numeric",	# step length
		tol = "numeric",
		gradientSum = "numeric",
		b = "numeric",			# estimated coefficients
		bPrev = "numeric",			
		fnVal = "numeric",
		fnValPrev = "numeric",
		X = "matrix",
		y = "ANY",
		adjStepLen = "logical",
		totalObs = "numeric"		# total observations processed
		),
	
	# list the methods (member functions)
	methods = list(
		"initialize" = function(formula = NULL, maxIters = 2000L, stepLen = 1.0, adjStepLen = TRUE, tol = 1e-6,...) 
		{
			"'fnValPrev' is initialized to 0"
			callSuper(...)	# always put this first, so that following code can override initializations		
			usingMethods(.pemaMethods) # always call this to provide access to parent methods
			maxIters <<- as.integer(maxIters)
			stepLen <<- stepLen
			tol <<- tol
			if (!is.null(formula))
			{
				formula <<- as.formula(formula)
			}
			else
			{
				formula <<- NULL
			}
			
			fnValPrev <<- 0.0
			adjStepLen <<- adjStepLen
			X <<- matrix()
			bPrev <<- b <<- as.numeric(NA)
			y <<- vector(mode = "numeric")
			totalObs <<- 0L
			# gradientSum, fnVal, iter and totalObs are initialized in initIter
			initIteration()
		},
		"initIteration" = function(iter = 0L)
		{
			"'fnVal', 'gradientSum', and 'totalObs' are initialized to 0"
			outputTrace(paste("Entering PemaGradDescent$initIteration iteration:", iter, "\n"), outTraceLevel = 7)
			iter <<- iter
			if ((length(X) == 1) && is.na(X))
			{
				gradientSum <<- 0
			}
			else
			{
				gradientSum <<- rep(0, ncol(X))
			}
	
			fnVal <<- 0.0
			totalObs <<- 0L
			outputTrace("Leaving PemaGradDescent$initIteration\n", outTraceLevel = 7)
		},
		"setMatrixData" = function(y, X)
		{
			'If data is already in matrix form, it can be set using this method'
			outputTrace("Entering PemaGradDescent$setMatrixData\n", outTraceLevel = 7)
			X <<- X
			y <<- y
			#b <<- as.matrix(rep(0, ncol(X))) 
			bPrev <<- b <<- rep(0, ncol(X))
			.dataInMemory <<- TRUE
			.dataInMemoryPrepared <<- TRUE # the data is in memory and is prepared
			outputTrace("Leaving PemaGradDescent$setMatrixData\n", outTraceLevel = 7)
			},
		"fn" = function() 
		{
			'Objective (loss) function for gradient descent'
			stop("'fn' must be specified for each model.")
		},
		"gradientFn" = function() 
		{
			'Gradient function for gradient descent'
			stop("'gradientFn' must be specified for each model.")
		},
		"prepareData" = function(dataList)
		{
			'Use a formula to convert from a list of data to the model matrix and response variable.'
			outputTrace("Entering PemaGradDescent$prepareData\n", outTraceLevel = 7)
			
			modelDf <- model.frame(formula, data = as.data.frame(dataList))
			X <<- model.matrix(formula, data = modelDf)
			y <<- model.response(data = modelDf)
			
			# Initialize b and gradientSum if needed
			if ((length(b) == 1) || is.na(b)) b <<- rep(0, ncol(X))
						
			# If all the data is in memory, set the .dataInMemoryPrepared flag so we 
			# don't have to prepare it again
			if (.dataInMemory)
 				.dataInMemoryPrepared <<- TRUE
				
			outputTrace("Leaving PemaGradDescent$prepareData\n", outTraceLevel = 7)
			invisible(NULL)
			
		},	
		"processData" = function(dataList) 
		{
			'Process a chunk of data, computing the gradient vector and the function value.'
			outputTrace("Entering PemaGradDescent$processData\n", outTraceLevel = 7)
			if (!.dataInMemoryPrepared) prepareData(dataList)
			gradientSum <<- gradientSum + gradientFn()
			fnVal <<- fnVal + fn()
			totalObs <<- totalObs + length(y)
			outputTrace("Leaving PemaGradDescent$processData\n", outTraceLevel = 7)
			invisible(NULL)
		},
		"updateResults" = function(gradDescentObject)	
		{
			'Update results for this object from results in another object.'
			outputTrace("Entering PemaGradDescent$updateResults\n", outTraceLevel = 7)
			
			gradientSum <<- gradientSum + gradDescentObject$gradientSum
			fnVal <<- fnVal + gradDescentObject$fnVal
			totalObs <<- totalObs + gradDescentObject$totalObs
			outputTrace("Leaving PemaGradDescent$updateResults\n", outTraceLevel = 7)
			return( NULL )
		},
		"processResults" = function()
		{
			'Given the current gradient vector, take a step to get the new coefficients'
			outputTrace("Entering PemaGradDescent$processResults\n", outTraceLevel = 7)
			
			bPrev <<- b
			obs <- max(totalObs,1)
			b <<- b - stepLen*(gradientSum/obs)
	
			if (reportProgress)	{cat("iter =", iter, "bPrev =", bPrev, "obs = ", obs, "b = ", b, "\n")}
			outputTrace("Leaving PemaGradDescent$processResults\n", outTraceLevel = 7)
			invisible(b)
		},
		"hasConverged" = function()
		{
			'Checks to see if convergence criteria has been met'
			outputTrace("Entering PemaGradDescent$hasConverged\n", outTraceLevel = 7)
		
			if (reportProgress)	{cat("iter =", iter, "fnVal =", fnVal, "fnValPrev = ", fnValPrev, "\n")}
			
			converged <- FALSE
			
			if (!is.infinite(fnVal))
			{
				# One convergence criterion is to stop when the values of the parameters (coefficients) have
				# stopped changing
				# An alternative convergence criterion is to stop when the value of the objective function
				# has stopped changing
				bDiff <- max(abs(b-bPrev))/max(max(abs(b)),1.0)
				fnDiff <- abs(fnVal - fnValPrev)/(0.1 + abs(fnVal))
				
				if (reportProgress)	{cat("bDiff = ", bDiff, "fnDiff = ", fnDiff, "\n")}
				
				converged <- ( (bDiff < tol) || (fnDiff < tol) )
				
				if (is.na(converged)) converged <- FALSE
			}
			
			if (!converged) 
			{
				# Simple handling of overstepping
				if (adjStepLen && (fnValPrev != 0) && (fnVal > fnValPrev))
				{
					stepLen <<- stepLen/2
					
					outputTrace(paste("In PemaGradDescent$hasConverged: Overstepped, new stepLen:", stepLen, "\n"), 
						outTraceLevel = 4)
					
					if (!identical(b,bPrev))
					{
						b <<- bPrev # start from previous set of coefficients
						processResults() # use the new stepLen to compute new coefficients using current gradients
					}					
				}
				fnValPrev <<- fnVal
			}
			outputTrace(paste("Leaving PemaGradDescent$hasConverged: ", converged, "\n"), outTraceLevel = 7)
			return(converged)
		}
	)
)

