#############################################################################################
# Reference Class Generator Function for PemaPopularWords - to identify popular words
# in a character column of a data set
#############################################################################################

PemaPopularWords <- setPemaClass("PemaPopularWords", 
	contains = "PemaBaseClass",
	# list the fields (member variables)
	fields = list(
		popularWords = "data.frame",
		varName = "character",
        chunkCutOff = "integer",
        minWordSize = "integer",
        omitWords = "character",
        minWordCount = "integer",
        ignoreCase = "logical",
		totalObs = "numeric"
		),
	
	# list the methods (member functions)
	methods = list(
		"initialize" = function( varName="", minWordSize = 4L,
             chunkCutOff = 1L, ignoreCase = TRUE, minWordCount = 2L,
             omitWords = c("also", "come", "does", "from", "have", "havent",
                "than", "that", "their", "them", 
                "then", "there", "these", "they", "this", 
                "what", "will", "with", "would", "your" ),
            ...) 
		{
            "'popularWords' initalized to a data frame containing only NA"
            # callSuper calls the method of the parent class
            callSuper(...)			
			usingMethods(.pemaMethods)
			popularWords <<- as.data.frame(NA)
			varName <<- varName
            chunkCutOff <<- as.integer(chunkCutOff)
            minWordSize <<- as.integer(minWordSize)
            omitWords <<- omitWords
            ignoreCase <<- ignoreCase
            minWordCount <<- as.integer(minWordCount)
			totalObs <<- 0
			outputTrace("In PemaPopularWords$initialize\n") # Call after fields are initialized
		},
		"processData" = function(dataList) 
		{
			'Computes the popular words for the current chunk of data.'
			outputTrace("Starting PemaPopularWords$processData\n") 
			totalObs <<- totalObs + length(dataList[[varName]])
            chunkVec <- dataList[[varName]]
			outputTrace(paste("  In PemaPopularWords$processData length(chunkVec): ", 
				length(chunkVec),"\n")) 
            # Remove single quote
            chunkVec <- gsub("'", "", chunkVec)
            # Replace other punctuation with space
            chunkVec <-  gsub("[[:punct:]]"," ", chunkVec) 
            # Lower case
            if (ignoreCase)
            {
                chunkVec <- tolower(chunkVec)
            }
            
            # Get all words
            allWords <- unlist(sapply(chunkVec, FUN = strsplit, split = " "))
            names(allWords) <- NULL
            # Remove small words
            allWords <- allWords[nchar(allWords) >= minWordSize ]
            # Remove omitted words
            allWords <- allWords[!(allWords %in% omitWords)]
            # Get word counts using table
            allWordsTable <- table(allWords)
			outputTrace(paste("  In PemaPopularWords$processData length(allWords): ", 
				length(allWordsTable),"\n")) 
			
            # Sort and remove those with low count
            allWordsFinal <- sort(allWordsTable[allWordsTable >= chunkCutOff], decreasing = TRUE)

			outputTrace(paste("  In PemaPopularWords$processData length(allWordsFinal): ", 
				length(allWordsFinal),"\n")) 		

            # Put in data frame
            chunkWords <- data.frame(words = names(allWordsFinal),  
                counts = as.vector(allWordsFinal), stringsAsFactors = FALSE)
			
			outputTrace(paste("  In PemaPopularWords$processData nrow(chunkWords): ", 
				nrow(chunkWords),"\n")) 			
            
            # Update the results for this thread
			if (nrow(chunkWords) > 0)
			{
				# Update the existing object with the new information from this chunk

				if ( (nrow(popularWords) == 1) && is.na(popularWords))
				{
					popularWords <<- chunkWords
				}
				else
				{
					dfCombine <- rbind(popularWords, chunkWords)
					# Find the counts of the words
					tableCombine <- sort(xtabs(counts~words, data = dfCombine), decreasing = TRUE)
					# Reset the original data frame with new words/counts
					popularWords <<- data.frame(words = names(tableCombine),                                
						counts = as.vector(tableCombine), stringsAsFactors = FALSE) 
				}
										
			}
			callSuper( dataList )  
			outputTrace("Leaving PemaPopularWords$processData \n")	    			
            invisible(NULL)
		},
		"updateResults" = function(pemaPopularWordsObj)
		{
		   'Updates the results from one PemaPopularWords object to another.'
			
			outputTrace("In PemaPopularWords$updateResults \n")
			totalObs <<- totalObs + pemaPopularWordsObj$totalObs
            if ((nrow(popularWords) == 1) && is.na(popularWords))
            {
                # If not yet initialized, set popularWords
				if (!is.null(pemaPopularWordsObj$popularWords) &&
					!((nrow(pemaPopularWordsObj$popularWords) == 1) && 
						is.na(pemaPopularWordsObj$popularWords)))
				{
					popularWords <<- pemaPopularWordsObj$popularWords
 				}	
            }
            else if (!(is.null(pemaPopularWordsObj$popularWords) ||
				       ((nrow(pemaPopularWordsObj$popularWords) == 1) && 
				       is.na(pemaPopularWordsObj$popularWords))))
            {
				outputTrace(paste("  In PemaPopularWords$updateResults nrow(popularWords): ", 
					nrow(popularWords),"\n")) 
				outputTrace(paste("  In PemaPopularWords$updateResults nrow(pemaPopularWordsObj$popularWords): ", 
					nrow(pemaPopularWordsObj$popularWords),"\n")) 				
				if (nrow(pemaPopularWordsObj$popularWords) > 0)
				{				
                    # Combine the existing and new data frames
					dfCombine <- rbind(popularWords, pemaPopularWordsObj$popularWords)
					# Find the counts of the words
					tableCombine <- sort(xtabs(counts~words, data = dfCombine), decreasing = TRUE)
					# Reset the original data frame with new words/counts
					popularWords <<- data.frame(words = names(tableCombine),                                
						counts = as.vector(tableCombine), stringsAsFactors = FALSE)  
   				}
            }
            callSuper( pemaPopularWordsObj)
			outputTrace("Leaving PemaPopularWords$updateResults \n")
			invisible(NULL)
		},
		"processResults" = function()
        {
           'Processes the final results, removing words with counts less than minWordCount.'			
            outputTrace("In PemaPopularWords$processResults \n")
			if ((length(popularWords) == 1) && is.na(popularWords))
			{
				outputTrace("In PemaPopularWords$processResults: empty word data frame. \n")
				return(0)
			}
			
            if (minWordCount > 1)
            {
                popularWords <<- popularWords[popularWords$counts >= minWordCount,]             
            }
            # Return a named vector of counts
            returnVec <- popularWords$counts
            names(returnVec) <- popularWords$words
			outputTrace("Leaving PemaPopularWords$processResults \n")
		    return( returnVec )
		},
        "getVarToUse" = function()
        {
            return( varName )
        }
	)
)

