#############################################################################################
# Reference Class Generator Function for PemaWordCount - to count specified words
# in a character column of a data set
#############################################################################################

PemaWordCount <- setPemaClass("PemaWordCount", 
	contains = "PemaBaseClass",
	# list the fields (member variables)
	fields = list(
		wordCounts = "data.frame",
		dictionary = "character",
		varName = "character",
        groupVarName = "character",
        groupObs = "numeric",
        ratioMultiple = "numeric",
        ignoreCase = "logical",
		totalObs = "numeric"
		),
	
	# list the methods (member functions)
	methods = list(
		"initialize" = function(dictionary = "", varName="", groupVarName = "", ignoreCase = TRUE,
             ratioMultiple = 0, ...) 
		{
            "'wordCounts' initalized to a data frame containing only NA, 'groupObs' set to 0"
            # callSuper calls the method of the parent class
            callSuper(...)			
			usingMethods(.pemaMethods)
						
			wordCounts <<- as.data.frame(NA)
			dictionary <<- dictionary
			varName <<- varName
            groupVarName <<- groupVarName
            groupObs <<- 0.0
            ratioMultiple <<- ratioMultiple
            ignoreCase <<- ignoreCase
			totalObs <<- 0
            
		},
		"processData" = function(dataList) 
		{
			'Updates the word counts for the current chunk of data.'
            hasGroups <- FALSE
            chunkGroupObs <- 0.0
			totalObs <<- totalObs + length(dataList[[varName]])
            if (nchar(groupVarName) > 0)
            {
                groupVar <- dataList[[groupVarName]]
                if (!is.null(groupVar) && is.factor(groupVar))
                {
                    groups <- levels(groupVar)
                    hasGroups <- TRUE
                }
                else
                {
                    stop('The grouping variable must be a factor.')
                }  
                # Compute group counts
                chunkGroupObs <- sapply(levels(groupVar), FUN = function(x){ 
                    sum(groupVar == x, na.rm = TRUE)
                    })      
            }
            chunkCounts <- sapply(dictionary, FUN = function(x)
            {
                if (hasGroups)
                {           
                    dictionaryWord <- x
                    res <- sapply(groups, FUN = function(x)
                        {
                            sum(grepl(dictionaryWord, dataList[[varName]][dataList[[groupVarName]] == x], 
                                ignore.case = ignoreCase), na.rm = TRUE)
                    })
                  }
                else
                {          
                    res <- sum(grepl(x, dataList[[varName]], ignore.case = ignoreCase))
                }
                return( res )
            })  
            if (is.matrix(chunkCounts))
            {
                chunkCounts <- t(chunkCounts)
            }
            
            chunkCounts <- as.data.frame(chunkCounts)
            if (ncol(chunkCounts) == 1)
            {
                names(chunkCounts) <- "ContainsWord"
            }   
            else
            {
                names(chunkCounts) <- make.names(names(chunkCounts), unique = TRUE, allow_ = FALSE)
            } 
            
            if ((length(wordCounts) == 1) && is.na(wordCounts))
            {
                wordCounts <<- chunkCounts
            }
            else
            {
                wordCounts <<- wordCounts + chunkCounts
            }
            groupObs <<- groupObs + chunkGroupObs
			invisible(NULL)
		},
		"updateResults" = function(pemaWordCountObj)
		{
			'Updates one PemaWordCount object from another.'
			totalObs <<- totalObs + pemaWordCountObj$totalObs
            if ((nrow(wordCounts) == 1) && is.na(wordCounts))
            {
                wordCounts <<- pemaWordCountObj$wordCounts
            }
            else if ((nrow(pemaWordCountObj$wordCounts) > 1) || 
                (!is.na(pemaWordCountObj$wordCounts)))
            {
            	wordCounts <<- wordCounts + pemaWordCountObj$wordCounts
            }
			invisible(NULL)
		},
		"processResults" = function()
        {
			'Processes the final results, computing ratios is requested.'
			if (ratioMultiple > 0)
			{
				if (length(groupObs) > 1)
				{
					sweep(wordCounts, 2, groupObs, `/`)*ratioMultiple
				}
				else
				{
					ratioMultiple*wordCounts/totalObs
				}
			}
			else
			{
				wordCounts
			}
		},
        "getVarToUse" = function()
        {
			'Returns the varName and, if specified, the groupVarName.'
            varsToUse <- varName
            if (nchar(groupVarName) > 0)
            {
                varsToUse <- c(varsToUse, groupVarName)
            }
            return( varsToUse )
        }
	)
)

