###################################################################################
# Sample code for importing finefoods data
###################################################################################
# The finefoods.txt data set can be downloaded from
# http://snap.stanford.edu/data/web-FineFoods.html
# This dataset consists of reviews of fine foods from amazon. 
# The data span a period of more than 10 years, including all ~500,000 reviews 
# up to October 2012. Reviews include product and user information, ratings, 
# and a plaintext review.
# Number of reviews 568,454 
# Number of users 256,059 
# Number of products 74,258 
# Users with > 50 reviews 260 
# Median no. of words per review 56 
# Timespan Oct 1999 - Oct 2012 


# SAMPLE ENTRY:
#product/productId: B001E4KFG0
#review/userId: A3SGXH7AUHUQQW
#review/profileName: argmartian
#review/helpfulness: 1/1
#review/score: 5.0
#review/time: 1303862400
#review/summary: Good Quality Dog Food
#review/text:

#########################################################################################
# Step 1:
# Read the data into an xdf file with two columns: label number and contents
# Remove all lines without id's at the beginning
#########################################################################################

splitText <- function(dataList)
{
	# These are the labels used at the beginnings of lines in the file
	productIdLabel <- "product/productId:"          # 1
    userIdLabel <- "review/userId:"                 # 2
    profileNameLabel  <- "review/profileName:"      # 3
    helpfulnessLabel <- "review/helpfulness:"       # 4
    reviewScoreLabel <- "review/score:"             # 5
    reviewTimeLabel <-"review/time:"                # 6
    reviewSummaryLabel <- "review/summary:"         # 7
    reviewTextLabel <- "review/text:"               # 8
	labelVec <- c(productIdLabel, userIdLabel, profileNameLabel,
		helpfulnessLabel, reviewScoreLabel, reviewTimeLabel,
		reviewSummaryLabel, reviewTextLabel)
	
	# Create a vector with labelType numbers
	labelType <- ifelse(grepl(productIdLabel, dataList[[1]] ), 1, NA)
	labelType <- ifelse(grepl(userIdLabel, dataList[[1]] ), 2, labelType)
	labelType <- ifelse(grepl(profileNameLabel, dataList[[1]] ), 3, labelType)
	labelType <- ifelse(grepl(helpfulnessLabel, dataList[[1]] ), 4, labelType)
	labelType <- ifelse(grepl(reviewScoreLabel, dataList[[1]] ), 5, labelType)
	labelType <- ifelse(grepl(reviewTimeLabel, dataList[[1]] ), 6, labelType)
	labelType <- ifelse(grepl(reviewSummaryLabel, dataList[[1]]), 7, labelType)
	labelType <- ifelse(grepl(reviewTextLabel, dataList[[1]]), 8, labelType)
	
	# Remove any rows without valid labels
	dataList[[1]] <- dataList[[1]][!is.na(labelType)]
	dataList$labelType <- labelType[!is.na(labelType)]
	
	# Get the repeating order of the labels for this chunk of data
	firstLabelType <- labelType[1]
	labelOrder <- 1:8
	if (firstLabelType > 1)
	{
		labelOrder <- c(firstLabelType:8, 1:(firstLabelType-1))
	}
	
	labelVecOrder <- labelVec[labelOrder]

	# Use strsplit to split the labels from the text context
	dataList$text <- sapply(strsplit(dataList[[1]], labelVecOrder), FUN = function(x)
		{
			if (length(x) <= 1)
			{
				return("")
			}
			else
			{
				val <- substring(text= x[2], first = 2 )			
				return(val)
			}
		})
		
	# Remove the original text strings
	dataList[[1]] <- NULL
	return(dataList)
}

# Specify the location for the downloaded data
textData <- RxTextData(file = "D:/Revolution/Data/finefoods/foods.txt",
  colInfo = list(entry = list(start= 1, width = 2000)), useFastRead = FALSE)

# Specify the location of the intermediate file that will be created
tempFile <- "D:/Revolution/Data/finefoods/finefoodsClean.xdf"

rxImport(inData = textData, outFile = tempFile, rowsPerRead = 20000,
   transformFunc = splitText, overwrite = TRUE)	

#########################################################################################
# Step 2:  Read the temporary file and reorganize it so that each "row" is
# a complete observation with all of the variables.  Perform transformations
# to create integer, factor, and Date variables
########################################################################################

reorgDataToRows <- function(dataList)
{		
	# Create the output data frame with productId as the first variable
	newData <- data.frame(productId = dataList[["text"]][dataList[["labelType"]] == 1], stringsAsFactors = FALSE)
	
	# Add the userId
	newData$userId <- dataList[["text"]][dataList[["labelType"]] == 2]
	
	# Add the profileName
	newData$profileName <- dataList[["text"]][dataList[["labelType"]] == 3]
	
	# Create two variables on helpfulness: the ratio and number
	helpfulExpr <- parse(text = dataList[["text"]][dataList[["labelType"]] ==4])
	newData$helpfulnessRatio <- sapply(helpfulExpr, FUN = eval)
	# Get number after /
	helpNumStr <- unlist(strsplit(x = dataList[["text"]][dataList[["labelType"]] ==4], split = "/"))
	newData$helpfulnessNum <- as.integer(helpNumStr[rep(c(FALSE, TRUE), times = nrow(newData))])
	
	# Convert the score to a factor
	scoreExpr <- parse(text = dataList[["text"]][dataList[["labelType"]] == 5])
	newData$reviewScore <- factor( sapply(scoreExpr, FUN = eval), levels = 1:5)

	# Convert the reviewTime string to a date
	newData$reviewDate <- as.Date(as.POSIXct(as.integer(dataList[["text"]][dataList[["labelType"]] == 6]), origin = "1970-01-01"))
	
	# Store the reviewSummary and reviewText
	newData$reviewSummary <- dataList[["text"]][dataList[["labelType"]] == 7]	
	newData$reviewText <- dataList[["text"]][dataList[["labelType"]] == 8]	
	
	return( newData )
}
	
# Specify the location of the intermediate file created above
tempFile <- "D:/Revolution/Data/finefoods/finefoodsClean.xdf"
# Specify the location of the final xdf file to be created
outFile <- "D:/Revolution/Data/finefoods/finefoods.xdf"

rxDataStep(inData = tempFile, outFile = outFile, rowsPerRead = 80000, # Must be multiple of 8
   transformFunc = reorgDataToRows, overwrite = TRUE)

####################################################################################
# Note:  The included sample data set was created with the following code
####################################################################################
xdfOut <- "D:/Revolution/Data/finefoods/foodrev.xdf"
rxDataStep(inData = outFile, outFile = xdfOut,
  varsToKeep = c("productId", "helpfulnessRatio", "helpfulnessNum", "reviewScore",
	"reviewDate", "reviewSummary"), numRows = 1000)

