############################################################################################
# Copyright (C) 2015 Microsoft Corporation
############################################################################################
#
# FUNCTIONS
#    Unit tests for PemaLogitGD and its parent class PemaGradDescent 
#      - exercising PemaBaseClass and pemaCompute
#
############################################################################################
library(RUnit)
"test.PemaLogitGD.aaa.setup" <- function()
{

    "GenerateLogisticData" <- function(numRows, numParams, a, b)
	{
		X <- matrix(rnorm(numRows*numParams), numRows, numParams)
		# To generate data, include the intercept
		X1 <- cbind(1, X)
		b1 <- rbind(a, b)
		#b0 <- b1*0.0
		err <- 0.02*matrix(rnorm(numRows), numRows, 1)
		z <- X1 %*% b1 + err
		y <- as.vector((plogis(z) <= runif(numRows)) + 0)
		k = numParams + 1 
		xNames <- paste("X", seq.int(1, numParams), sep="")
		colnames(X) <- xNames
		xy = cbind(X, y)
		dfxy <- as.data.frame(xy)
		return(dfxy)
	}

	set.seed(12345)
	numRows <- 5000
	numParams <- 10
	a <- -1.0 # intercept
	b <- matrix(rnorm(numParams), numParams, 1)
	b <- matrix(rep(-.1, numParams), numParams, 1)

    pemaLogitTestData <<- GenerateLogisticData(numRows, numParams, a, b)
	pkgDir <- path.package("RevoPemaR")
	
	gradDescentCode <- file.path(pkgDir, "/demoScripts/PemaGradDescent.R")
	source(gradDescentCode)
	
	logitCode <- file.path(pkgDir, "/demoScripts/PemaLogitGD.R")
	source(logitCode)

}



"test.PemaLogitGD.pemaCompute.numeric" <- function()
{

	logitGdObj <- PemaLogitGD()
	
	logitFormula <- as.formula(paste("y ~", paste("X", 1:(ncol(pemaLogitTestData)-1), collapse = " + ", sep = "")))
	
	result1 <- pemaCompute(pemaObj =logitGdObj, data = pemaLogitTestData, formula = logitFormula, reportProgress = 0)

    result2 <- coef(glm(formula = logitFormula, data = pemaLogitTestData, family = binomial()))
	RUnit::checkEquals(result1, result2, tolerance = .005)

}

"test.PemaLogitGD.method.initialize" <- function()
{
    logitPemaObj <- PemaLogitGD()
    # Get the values of all the fields at instantiation
    fieldListOrig <- logitPemaObj$getFieldList()
    
    # Run an pemaCompute to modify 
	logitFormula <- as.formula(paste("y ~", paste("X", 1:(ncol(pemaLogitTestData)-1), collapse = " + ", sep = "")))
	result1 <- pemaCompute(pemaObj = logitPemaObj, data = pemaLogitTestData, 
		formula = logitFormula, reportProgress = 0)
		
    # Re-intialize
    logitPemaObj$initialize()
    # Get the field values
    fieldListInitialize <- logitPemaObj$getFieldList()
    
    # Check to make sure everything is reset to initial value
    checkIdentical(fieldListOrig, fieldListInitialize)   
}

"test.PemaLogitGD.method.processData" <- function()
{
    logitPemaObj <- PemaLogitGD()
	logitPemaObj$formula <- as.formula(paste("y ~", paste("X", 1:(ncol(pemaLogitTestData)-1), collapse = " + ", sep = "")))

    logitPemaObj$processData( pemaLogitTestData )    

    # Get the field values
    fieldList <- logitPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, nrow(pemaLogitTestData))
    RUnit::checkEquals(as.vector(fieldList$y), as.vector(pemaLogitTestData$y)) 
    RUnit::checkEquals(fieldList$b, rep(0,times = 11))  # b is not updated in processData
    
    # Now process again.  This should update the results on this thread
    logitPemaObj$processData( pemaLogitTestData )    

    # Get the field values
    fieldList2 <- logitPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList2$totalObs, 2*nrow(pemaLogitTestData))
	RUnit::checkEquals(fieldList2$fnVal, 2*fieldList$fnVal)
	RUnit::checkEquals(fieldList2$fnValPrev, 2*fieldList$fnValPrev)
    RUnit::checkEquals(as.vector(fieldList2$y), as.vector(pemaLogitTestData$y)) 
    RUnit::checkEquals(fieldList2$b, rep(0,times = 11))   
}

"test.PemaLogitGD.method.updateResults" <- function()
{
	logitFormula <- as.formula(paste("y ~", paste("X", 1:(ncol(pemaLogitTestData)-1), collapse = " + ", sep = "")))
	
    # Create an PemaLogitGD object and process data 
    logitPemaObj1 <- PemaLogitGD()
    logitPemaObj1$formula <- logitFormula
    logitPemaObj1$processData( pemaLogitTestData ) 	 

    # Create another PemaLogitGD object and process data    
    logitPemaObj2 <- PemaLogitGD()
    logitPemaObj2$formula = logitFormula
    logitPemaObj2$processData( pemaLogitTestData )        
    
	fieldList <- logitPemaObj1$getFieldList()
	
    # This is used to update results from one node to another
    logitPemaObj1$updateResults( logitPemaObj2 )    

    # Get the field values
    fieldList2 <- logitPemaObj1$getFieldList()
    
    RUnit::checkEquals(fieldList2$totalObs, 2*nrow(pemaLogitTestData))
	RUnit::checkEquals(fieldList2$fnVal, 2*fieldList$fnVal)
	RUnit::checkEquals(fieldList2$fnValPrev, 2*fieldList$fnValPrev)
    RUnit::checkEquals(as.vector(fieldList$y), as.vector(pemaLogitTestData$y)) 
	RUnit::checkEquals(fieldList$b, rep(0,times = 11))
}

"test.PemaLogitGD.method.processResults" <- function()
{
    # Create an PemaLogitGD object and process data 
	logitFormula <- as.formula(paste("y ~", paste("X", 1:(ncol(pemaLogitTestData)-1), collapse = " + ", sep = "")))
	
    # Create an PemaLogitGD object and process data 
    logitPemaObj1 <- PemaLogitGD()
    logitPemaObj1$formula <- logitFormula
    logitPemaObj1$processData( pemaLogitTestData ) 

    # Create another PemaLogitGD object and process data    
    logitPemaObj2 <- PemaLogitGD()
    logitPemaObj2$formula <- logitFormula
    logitPemaObj2$processData( pemaLogitTestData )        
    
    # This is used to update results from one node to another
    logitPemaObj1$updateResults( logitPemaObj2 ) 
    
    # We're done now, so process the final results
    logitPemaObj1$processResults()   

    # We're done with the first iteration; we should have non-0 betas
    checkTrue(logitPemaObj1$b[1] > 0)  
	checkTrue(logitPemaObj1$fnVal > 0)
	checkTrue(logitPemaObj1$fnValPrev == 0)
	RUnit::checkEquals(logitPemaObj1$iter, 0) # This is updated in the compute method
	fieldList1 <- logitPemaObj1$getFieldList()
	
	# Check for convergence
	hasConverged <- logitPemaObj1$hasConverged() # This updates fnValPrev

	# Now process the data again anc check for convergence
	logitPemaObj1$processData( pemaLogitTestData )
	logitPemaObj1$processResults()

	fieldList2 <- logitPemaObj1$getFieldList()
	
	checkTrue(fieldList2$fnValPrev == fieldList1$fnVal)
	checkTrue(fieldList2$b[1] != fieldList1$b[1]) # b has changed
}

"test.PemaLogitGD.zzz.cleanup" <- function()
{
    rm(pemaLogitTestData, inherits = TRUE)
}

