############################################################################################
# Copyright (C) 2015 Microsoft Corporation
############################################################################################
#
# FUNCTIONS
#    Unit tests for PemaWordCount - exercising PemaBaseClass and pemaCompute
#
############################################################################################
library(RUnit)
"test.PemaWordCount.aaa.setup"  <- function()
{
    # Create a simple data frame with character data and a factor
   pemaTestData <<- data.frame(comment = 
       c("Today there will be scattered showers", 
         "Looks like mist, followed by drizzle to me", 
         "I see isolated showers to the west",
         "Sprinkles here",
         "No rain today",
         "Nope, don't see rain.",
         "Hardly ever rains this time of year",
         "I'd like a little rain."),
         state = factor(rep(c("Washington", "Arizona"), each = 4)), stringsAsFactors = FALSE)
	
		pkgDir <- path.package("RevoPemaR")
		wordCountCode <- file.path(pkgDir, "/demoScripts/PemaWordCount.R")
		source(wordCountCode)
}




"test.PemaWordCount.method.initialize" <- function()
{
    wordCountPemaObj <- PemaWordCount()
    # Get the values of all the fields at instantiation
    fieldListOrig <- wordCountPemaObj$getFieldList()
    
    dictionary = c("rain", "showers", "sprinkles", "mist", "drizzle")
    # Run an pemaCompute to modify fields
    out1 <- pemaCompute(pemaObj = wordCountPemaObj, data = pemaTestData, 
        varName = "comment", dictionary = dictionary)
    
    # Re-intialize
    wordCountPemaObj$initialize()
    # Get the field values
    fieldListInitialize <- wordCountPemaObj$getFieldList()
    
    # Check to make sure everything is reset to initial value
    checkIdentical(fieldListOrig, fieldListInitialize)   
}

"test.PemaWordCount.method.processData" <- function()
{
    wordCountPemaObj <- PemaWordCount()
    wordCountPemaObj$varName <- "comment"
    wordCountPemaObj$dictionary <- c("rain", "showers")
    wordCountPemaObj$processData( pemaTestData )    

    # Get the field values
    fieldList <- wordCountPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, nrow(pemaTestData))
    RUnit::checkEquals(fieldList$wordCounts["rain",1], 4) 
    RUnit::checkEquals(fieldList$wordCounts["showers",1], 2) 

    
    # Now process again.  This should update the results on this thread
    wordCountPemaObj$processData( pemaTestData)    

    # Get the field values
    fieldList <- wordCountPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(pemaTestData))
    RUnit::checkEquals(fieldList$wordCounts["rain",1], 8) 
    RUnit::checkEquals(fieldList$wordCounts["showers",1], 4) 

}

"test.PemaWordCount.method.updateResults" <- function()
{
    # Create an PemaWordCount object and process data 
    wordCountPemaObj <- PemaWordCount()
    wordCountPemaObj$varName <- "comment"
    wordCountPemaObj$dictionary <- c("rain", "showers")
    wordCountPemaObj$processData( pemaTestData )    

    # Create another PemaWordCount object and process data    
    wordCountPemaObj2 <- PemaWordCount()
    wordCountPemaObj2$varName <- "comment"
    wordCountPemaObj2$dictionary <- c("rain", "showers")
    wordCountPemaObj2$processData( pemaTestData )      
    
    # This is used to update results from one node to another
    wordCountPemaObj$updateResults( wordCountPemaObj2 )    

    # Get the field values  
    fieldList <- wordCountPemaObj$getFieldList()
    
    RUnit::checkEquals(fieldList$totalObs, 2*nrow(pemaTestData))
    RUnit::checkEquals(fieldList$wordCounts["rain",1], 8) 
    RUnit::checkEquals(fieldList$wordCounts["showers",1], 4) 
}

"test.PemaWordCount.method.processResults" <- function()
{
    # Create an PemaWordCount object and process data 
    wordCountPemaObj <- PemaWordCount()
    wordCountPemaObj$varName <- "comment"
    wordCountPemaObj$dictionary <- c("rain", "showers")
	wordCountPemaObj$ratioMultiple <- 1
    wordCountPemaObj$processData( pemaTestData )    

    # Create another PemaWordCount object and process data    
    wordCountPemaObj2 <- PemaWordCount()
    wordCountPemaObj2$varName <- "comment"
    wordCountPemaObj2$dictionary <- c("rain", "showers")
    wordCountPemaObj2$processData( pemaTestData )      
    
    # This is used to update results from one node to another
    wordCountPemaObj$updateResults( wordCountPemaObj2 )   
    
    # We're done now, so process the final results
    out1 <- wordCountPemaObj$processResults()   

    # Check the mean
    RUnit::checkEquals(out1["rain",1], .5) 
    RUnit::checkEquals(out1["showers",1], .25) 
}

"test.PemaWordCount.pemaCompute" <- function()
{
    # Instantiate an PemaWordCount reference class
    wordCountPemaObj <- PemaWordCount()
    
    # Compute the word count for the test data set
    # Call pemaCompute, specifying the custom analyis object, the data, and additional arg
    out1 <- pemaCompute(pemaObj = wordCountPemaObj, data = pemaTestData, varName = "comment",
      ratioMultiple = 1, dictionary = c("rain", "showers", "sprinkles", "mist", "hello"))

    RUnit::checkEquals(wordCountPemaObj$totalObs, nrow(pemaTestData))
    RUnit::checkEquals(out1["rain", 1], .5) 
    RUnit::checkEquals(out1["showers", 1], .25) 
    RUnit::checkEquals(out1["sprinkles", 1], .125) 
    RUnit::checkEquals(out1["mist", 1], .125) 
    RUnit::checkEquals(out1["hello", 1], .0) 
    RUnit::checkEquals(wordCountPemaObj$wordCounts["rain", 1], 4)
    RUnit::checkEquals(wordCountPemaObj$wordCounts["showers", 1], 2)
    RUnit::checkEquals(wordCountPemaObj$wordCounts["sprinkles", 1], 1)
    RUnit::checkEquals(wordCountPemaObj$wordCounts["mist", 1], 1)
    RUnit::checkEquals(wordCountPemaObj$wordCounts["hello", 1], 0)   
}

"test.PemaWordCount.zzz.cleanup" <- function()
{
    rm(pemaTestData, inherits = TRUE)
}
