##############################################################################
# RevoScaleR RxTeradata Getting Started Guide Examples
##############################################################################

##############################################################################
# General setup/cleanup: Run this code before proceeding
##############################################################################

# To follow the examples in this guide, you will need the following 
# comma-delimited text data files  loaded into your Teradata database: 
#  ccFraud.csv as RevoTestDB.ccFraud10
#  ccFraudScore.csv as RevoTestDB.ccFraudScore10
# These data files are available from Revolution FTP site. 
# You can use the 'fastload' Teradata command to load the data sets into your 
# data base. 

# Modify and uncomment the following code for your connection to Teradata,
# and other information about your Teradata setup:

# Note: for improved security, read connection string from a file, such as
# tdConnString <- readLines("tdConnString.txt")

# tdConnString <- "DRIVER=Teradata;DBCNAME=machineNameOrIP;UID=myUserID;PWD=myPassword;"
# tdShareDir = paste("c:\\AllShare\\", Sys.getenv("USERNAME"), sep="")

# Make sure that the local share dir exists:
if (!file.exists(tdShareDir)) dir.create(tdShareDir, recursive = TRUE)

# If your DATABASE is not named RevoTestDB, you will need
# to modify the SQL queries in the following code to your
# DATABASE name


##########################################################
# Using a Teradata Data Source and Compute Context
##########################################################  
 
  
# Creating an RxTeradata Data Source
  
tdQuery <- "SELECT * FROM RevoTestDB.ccFraud10"	
teradataDS <- RxTeradata(connectionString = tdConnString, 
sqlQuery = tdQuery, rowsPerRead = 50000)
  
# Extracting basic information about your data
  
rxGetVarInfo(data = teradataDS)
  
# Specifying column information in your data source
  
stateAbb <- c("AK", "AL", "AR", "AZ", "CA", "CO", "CT", "DC",
    "DE", "FL", "GA", "HI","IA", "ID", "IL", "IN", "KS", "KY", "LA",
    "MA", "MD", "ME", "MI", "MN", "MO", "MS", "MT", "NB", "NC", "ND",
    "NH", "NJ", "NM", "NV", "NY", "OH", "OK", "OR", "PA", "RI","SC",
    "SD", "TN", "TX", "UT", "VA", "VT", "WA", "WI", "WV", "WY")


ccColInfo <- list(		
    gender = list(
    type = "factor", 
        levels = c("1", "2"),
      newLevels = c("Male", "Female")),		
    cardholder = list(
      type = "factor", 
      levels = c("1", "2"),	
          newLevels = c("Principal", "Secondary")),
  state = list(
          type = "factor", 
          levels = as.character(1:51),
      newLevels = stateAbb)
  )
teradataDS <- RxTeradata(connectionString = tdConnString, 
    sqlQuery = tdQuery, colInfo = ccColInfo, rowsPerRead = 50000)

rxGetVarInfo(data = teradataDS)


# Using rxImport to Extract a Subsample
 
teradataProbDS <- RxTeradata(
  sqlQuery = paste(
  "Select * FROM ccScoreOutput2",
  "WHERE (ccFraudProb > .99)"),
  connectionString = tdConnString)
highRisk <- rxImport(teradataProbDS)
orderedHighRisk <- highRisk[order(-highRisk$ccFraudProb),]
row.names(orderedHighRisk) <- NULL  # Reset row numbers
head(orderedHighRisk)
  
# Using rxDataStep to Create a Teradata Table

xdfAirDemo <- RxXdfData(file.path(rxGetOption("sampleDataDir"),
    "AirlineDemoSmall.xdf"))
rxGetVarInfo(xdfAirDemo)

teradataAirDemo <- RxTeradata(table = "AirDemoSmallTest",
  connectionString = tdConnString)
if (rxTeradataTableExists("AirDemoSmallTest",
    connectionString = tdConnString))
    rxTeradataDropTable("AirDemoSmallTest", 
    connectionString = tdConnString)

rxDataStep(inData = xdfAirDemo, outFile = teradataAirDemo, 
  transforms = list(
    DayOfWeek = as.integer(DayOfWeek),
    rowNum = .rxStartRow : (.rxStartRow + .rxNumRows - 1)
  ), overwrite = TRUE
)

   
# Clean-up
  
rxTeradataDropTable("models")
rxTeradataDropTable("scores")

##########################################################
# Analyzing Your Data Alongside Teradata
##########################################################  

tdQuery <- "SELECT * FROM RevoTestDB.ccFraudScore10"  
  
# Perform an rxSummary using a local compute context
  
teradataDS1 <- RxTeradata(connectionString = tdConnString, 
    sqlQuery = tdQuery, colInfo = ccColInfo, rowsPerRead = 500000)

rxSummary(formula = ~gender + balance + numTrans + numIntlTrans +
creditLine, data = teradataDS1)
statesToKeep <- sapply(c("CA", "OR", "WA"), grep, stateAbb)
statesToKeep
  
# Fast import of iata from a Teradata Database
  
importQuery <- paste("SELECT gender,cardholder,balance,state",
  "FROM RevoTestDB.ccFraud10",
  "WHERE (state = 5 OR state = 38 OR state = 48)")
importColInfo <- list(		
    gender = list(
    type = "factor", 
        levels = c("1", "2"),
      newLevels = c("Male", "Female")),		
    cardholder = list(
      type = "factor", 
      levels = c("1", "2"),	
          newLevels = c("Principal", "Secondary")),
  state = list(
          type = "factor", 
          levels = as.character(statesToKeep),
        newLevels = names(statesToKeep))
  )

teradataImportDS <- RxTeradata(connectionString = tdConnString, 
      sqlQuery = importQuery, colInfo = importColInfo)

localDS <- rxImport(inData = teradataImportDS,
      outFile = "ccFraudSub.xdf", 
      overwrite = TRUE)
  
# Using the imported data
  
rxGetVarInfo(data = localDS)
rxSummary(~gender + cardholder + balance + state, data = localDS)	
