# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# These tests assume that the default, standard state exists on Linux systems (that is,
# that the thread pools are off, and we're using ad hoc threading).  If this is not the
# state when this file is run, that state will be created, and the original state restored
# when we're done.  Obviously, this pre-assumes that these functions work.
#
# FUNCTIONS
#    Unit tests for ad hoc threading:
#		rxSetEnableThreadPool
#		rxGetEnableThreadPool
#		RevoScaleR:::RxCreateThreadPool
#		RevoScaleR:::RxDeleteThreadPool
#
############################################################################################

".RxCreateThreadPool" <- function()
{
    if (exists("RxCreateThreadPool", globalenv()))
        get("RxCreateThreadPool", globalenv())
    else
        RevoScaleR:::RxCreateThreadPool
}

".RxDeleteThreadPool" <- function()
{
    if (exists("RxDeleteThreadPool", globalenv()))
        get("RxDeleteThreadPool", globalenv())
    else
        RevoScaleR:::RxDeleteThreadPool
}

".rxIsWindowsTest" <- if(exists(".rxIsWindows", globalenv())) get(".rxIsWindows", globalenv()) else RevoScaleR:::.rxIsWindows

.rxGlobalAdHocThreadingStartStateSet <- FALSE
.rxGlobalAdHocThreadingStartState <- TRUE

".rxSetDefaultThreadPoolTestState" <- function()
{
  if ( !.rxIsWindowsTest() )
  {
        retVal <- rxGetEnableThreadPool()
        if ( retVal )
        {
            rxSetEnableThreadPool( FALSE )
      
      # this line ensures that we only set the start state once (the first time)
      # if we set it again, we'll be setting it to the value that we set the
      # environment to on the last call to this function
            if ( !.rxGlobalAdHocThreadingStartStateSet )
            {
                .rxGlobalAdHocThreadingStartState <- retVal
                .rxGlobalAdHocThreadingStartStateSet <- TRUE
            }
        }
  }
}

".rxRestorDefaultThreadPoolTestState" <- function()
{
  if ( !.rxIsWindowsTest() && .rxGlobalAdHocThreadingStartStateSet )
  {
        suppressWarnings( rxSetEnableThreadPool( .rxGlobalAdHocThreadingStartState ) )
    }
}    

".setUp" <- function()
{
    .rxSetDefaultThreadPoolTestState()
}

".tearDown" <- function()
{
    .rxRestorDefaultThreadPoolTestState()
}


"test.AdHocThreading.rxSetEnableThreadPool.rxGetEnableThreadPool" <- function()
{
    .rxSetDefaultThreadPoolTestState()
  
  if ( .rxIsWindowsTest() )
  {
    # make sure that we're not actually changing state of thread pool in windows,
    # and that we're getting the expected return values
    suppressWarnings( checkEquals( rxSetEnableThreadPool( TRUE ), TRUE ) )
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )
    suppressWarnings( checkEquals( rxSetEnableThreadPool( FALSE ), TRUE ) )
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )
    
    # each of these calls should return a warning in windows.  Make sure we're getting it.
    oldWarn <- options(warn = 2L)
    on.exit(options(oldWarn), add = TRUE)
    
    checkException( rxSetEnableThreadPool( TRUE ) )
    checkException( rxGetEnableThreadPool() )
    checkException( rxSetEnableThreadPool( FALSE ) )
    checkException( rxGetEnableThreadPool() )
  }
  else  # Linux (boost threads)
  {
    oldWarn <- options(warn = 2L)
    on.exit(options(oldWarn), add = TRUE)

    checkEquals( rxSetEnableThreadPool( TRUE ), FALSE )
    checkEquals( rxGetEnableThreadPool(), TRUE )

    checkException( checkEquals( rxSetEnableThreadPool( TRUE ), TRUE ) )
    checkEquals( rxGetEnableThreadPool(), TRUE )

    checkEquals( rxSetEnableThreadPool( FALSE ), TRUE )
    checkEquals( rxGetEnableThreadPool(), FALSE )

    checkException( checkEquals( rxSetEnableThreadPool( FALSE ), FALSE ) )
    checkEquals( rxGetEnableThreadPool(), FALSE )
  }

    .rxSetDefaultThreadPoolTestState()
}


"test.AdHocThreading.RxCreateThreadPool.RxDeleteThreadPool" <- function()
{

  .rxSetDefaultThreadPoolTestState()

  RxDeleteThreadPool <- .RxDeleteThreadPool()
  RxCreateThreadPool <- .RxCreateThreadPool()
  
  if ( .rxIsWindowsTest() )
  {
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )
    
    checkEquals( RxDeleteThreadPool(), FALSE )
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )

    checkEquals( RxDeleteThreadPool(), FALSE )
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )
    
    checkEquals( RxCreateThreadPool(), FALSE )
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )

    checkEquals( RxCreateThreadPool(), FALSE )
    suppressWarnings( checkEquals( rxGetEnableThreadPool(), TRUE ) )
}
  else  # Linux (boost threading)
  {
    # check with thread pool disabled (default start state for the test)
    
    checkEquals( rxGetEnableThreadPool(), FALSE )
    
      checkEquals( RxDeleteThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), FALSE )
    
    checkEquals( RxCreateThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), FALSE )
    
    checkEquals( RxCreateThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), FALSE )
    
    checkEquals( RxDeleteThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), FALSE )
    
    checkEquals( RxDeleteThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), FALSE )

        # Now do the same checks, but with the thread pool enabled
    # note that thread pool enabled will always be TRUE - this is by design
    # these are special functions only for use internally to DeployR, and are
    # very dangerous black magic.
        checkEquals( rxSetEnableThreadPool( TRUE ), FALSE )

    checkEquals( rxGetEnableThreadPool(), TRUE )

        checkEquals( RxDeleteThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), TRUE )
    
    checkEquals( RxCreateThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), TRUE )
    
    checkEquals( RxCreateThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), TRUE )
    
    ##RA-12100: temporarily commented out the following 2 lines
    ##	which seem to cause the 2-process code to fail on Linux
    ##	if the test suite is run more than once. It's now fixed.
    checkEquals( RxDeleteThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), TRUE )
    
    checkEquals( RxDeleteThreadPool(), TRUE )
    checkEquals( rxGetEnableThreadPool(), TRUE )

  }

    .rxSetDefaultThreadPoolTestState()
}

# LAZY DEV BLOCK

# test.AdHocThreading.rxSetEnableThreadPool.rxGetEnableThreadPool()
# test.AdHocThreading.RxCreateThreadPool.RxDeleteThreadPool()

