# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for RxFileSystem, RxHdfsFileSystem, and RxNativeFileSystem and related functions
#
############################################################################################

"test.RxFileSystem.args.fileSystem" <- function()
{
  # Native file system
  testFS1 <- RxNativeFileSystem()
  testFS2 <- RxFileSystem("RxNativeFileSystem" )
  testFS3 <- RxFileSystem( "native" )
  testFS4 <- RxFileSystem( testFS1 )
  checkEquals( testFS1, testFS2 )
  checkEquals( testFS1, testFS3 )
  checkEquals( testFS1, testFS4 )
  checkTrue( is( testFS1, "RxFileSystem") )
  checkTrue( is( testFS1, "RxNativeFileSystem") ) 
  
  # HDFS file system
  testFS1 <- RxHdfsFileSystem(hostName = "sandbox-01", port = 8020)
  testFS2 <- RxFileSystem("RxHdfsFileSystem", hostName = "sandbox-01", port = 8020)
  testFS3 <- RxFileSystem( "hdfs" , hostName = "sandbox-01", port = 8020)
  testFS4 <- RxFileSystem( testFS1 )
  checkEquals( testFS1, testFS2 )
  checkEquals( testFS1, testFS3 )
  checkEquals( testFS1, testFS4 )
  checkTrue( is( testFS1, "RxFileSystem") )
  checkTrue( is( testFS1, "RxHdfsFileSystem") ) 	
  
  # Change arguments with HDFS
  testFS5 <- RxHdfsFileSystem( hostName = "sandbox-01", port = 8030)
  oldValue <- rxGetOption("hdfsHost")
  rxOptions( hdfsHost = "sandbox-01" )
  testFS6 <- RxHdfsFileSystem(testFS1,  port = 8030) # hostName will be determined by rxGetOption("hdfsHost")
  rxOptions( hdfsHost = oldValue )
  testFS7 <- RxFileSystem(fileSystem = testFS1, port = 8030) # hostName value will be that of testFS1
  checkEquals( testFS5, testFS6 )
  checkEquals( testFS5, testFS7 )	
  
  testFS5 <- RxHdfsFileSystem( hostName = "sandbox-02", port = 8020)
  oldValue <- rxGetOption("hdfsPort")
  rxOptions( hdfsPort = 8020 )
  testFS6 <- RxHdfsFileSystem(testFS1,  hostName = "sandbox-02") # port will be determined by rxGetOption("hdfsPort")
  rxOptions( hdfsPort = oldValue )
  testFS7 <- RxFileSystem(fileSystem = testFS1, hostName = "sandbox-02") # port value will be that of testFS1
  checkEquals( testFS5, testFS6 )
  checkEquals( testFS5, testFS7 )	
  
}
 
"test.RxFileSystem.method.print" <- function()  
{
  testFS1 <- RxFileSystem( "native" )
  testFS2 <- RxFileSystem( "hdfs" , hostName = "sandbox-01", port = 8020)
  
  out1 <- capture.output(print(testFS1))
  checkTrue(grepl("native", out1[1]))
  
  out2 <- capture.output(print(testFS2))
  checkTrue(grepl("hdfs", out2[1]))
  checkTrue(grepl("sandbox", out2[2]))
  checkTrue(grepl("8020", out2[3]))
}

"test.rxSetFileSystem.args.all" <- function()  
{
  originalFS <- rxGetFileSystem()
  testFS1 <- RxFileSystem( "native" )
  testFS2 <- RxFileSystem( "hdfs" , hostName = "sandbox-01", port = 8020)
  oldFS1 <- rxSetFileSystem(testFS2)
  oldFS2 <- rxSetFileSystem(testFS1)
  checkEquals(testFS2, oldFS2)
  
  # Now change an argument
  oldFS <- rxSetFileSystem(testFS2, hostName = "playpen-01")
  newFS <- rxGetFileSystem()
  checkEquals(newFS$hostName, "playpen-01")
  
  # Make sure changing another option doesn't cause problems
  oldBlocksPerRead <- rxGetOption("blocksPerRead")
  rxOptions(blocksPerRead = 2)
  currentFS <- rxGetFileSystem()
  checkEquals(newFS, currentFS)
  rxOptions(blocksPerRead = oldBlocksPerRead)
  rxSetFileSystem(originalFS)
      
}

"test.rxGetFileSystem.args.x" <- function()
{
    hdfsFS1 <- RxHdfsFileSystem( hostName = "sandbox-01", port = 8020)
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    textDS <- RxTextData(file = inFile, fileSystem = hdfsFS1)
    
    # Retrieve the file system information
    hdfsFS2 <- rxGetFileSystem(textDS)
  checkEquals(hdfsFS1, hdfsFS2)
  
  # Compute context has file system
  hdfsFS3 <- RxHdfsFileSystem( hostName = "sandbox-02", port = 8020)

    computeContext <- RxSpark( fileSystem = hdfsFS3)
  oldComputeContext <- rxSetComputeContext(computeContext)
  on.exit(rxSetComputeContext(oldComputeContext), add = TRUE)
  
  hdfsFS4 <- rxGetFileSystem()
  checkEquals(hdfsFS3, hdfsFS4)
  
  # Make sure input object still gets priority
  hdfsFS5 <- rxGetFileSystem(textDS)
  checkEquals(hdfsFS1, hdfsFS2)
  
  
}

"test.rxSetFileSystem.otherOptions" <- function()
{
    initialContext <- rxGetComputeContext()
    on.exit(rxSetComputeContext(initialContext), add = TRUE)
    
    rxSetComputeContext(RxHpcServer()) 
    context1 <- rxGetComputeContext() 
    rxSetFileSystem("native") 
    context2 <- rxGetComputeContext() 
    checkEquals(context1, context2)
}

".rxCreateDir" <- function()
{
    if (exists("rxCreateDir", globalenv()))
        get("rxCreateDir", globalenv())
    else
        RevoScaleR:::rxCreateDir
}

".rxRemoveDir" <- function()
{
    if (exists("rxRemoveDir", globalenv()))
        get("rxRemoveDir", globalenv())
    else
        RevoScaleR:::rxRemoveDir
}

".rxSetUsingRunAsUserMode" <- function()
{
    if (exists("rxSetUsingRunAsUserMode", globalenv()))
        get("rxSetUsingRunAsUserMode", globalenv())
    else
        RevoScaleR:::rxSetUsingRunAsUserMode
}

".rxGetUsingRunAsUserMode" <- function()
{
    if (exists("rxGetUsingRunAsUserMode", globalenv()))
        get("rxGetUsingRunAsUserMode", globalenv())
    else
        RevoScaleR:::rxGetUsingRunAsUserMode
}


# note that CxPath create and remove dir functionality is heavily unit tested in the C++
# code; this is only to test the RxLink interface
"test.RxCreatAndRemoveDir" <- function()
{
  rxCreateDir <- .rxCreateDir()
  rxRemoveDir <- .rxRemoveDir()
  
  myBaseDir <- tempdir()
  if ( toupper( Sys.info()[["sysname"]]) == "WINDOWS" )
  {
    myTempDir <- paste( myBaseDir, "\\foo", sep="" )
    myOs = "windows"
  }
  else
  {
    myTempDir <- paste( myBaseDir, "/foo", sep="" )
    myOs = "linux"
  }

  checkTrue( rxCreateDir( myTempDir, "local", myOs ) )
  checkTrue ( file.exists( myTempDir ) )
  checkTrue( rxRemoveDir( myTempDir, "local", myOs ) )
  
  # note: deleted directory not flushed out until process leaves on windows
  checkTrue( !isTRUE( TRUE %in% as.logical( grep( "foo", dir( myBaseDir ) ) ) ) )
}

"test.RxCreateAndRemoveRelativeDir" <- function()
{
    rxCreateDir <- .rxCreateDir()
    rxRemoveDir <- .rxRemoveDir()

  myWorkingDir <- getwd()
  on.exit( setwd( myWorkingDir ), add = TRUE )

    myBaseDir <- tempdir()
  setwd( myBaseDir )
  is32Bit <- FALSE

    if ( toupper( Sys.info()[["sysname"]]) == "WINDOWS" )
    {
    myNewDir <- "foo\\bar"
        myOs = "windows"
    is32Bit <- toupper( Sys.info()[["machine"]]) == "X86"
    }
    else
    {
    myNewDir <- "foo/bar"
        myOs = "linux"
    }

  if (!is32Bit)
  {
    checkTrue( rxCreateDir( myNewDir, "local", myOs ) )
    checkTrue ( file.exists( myNewDir ) )
    checkTrue( rxRemoveDir( "foo", "local", myOs ) )

    # note: deleted directory not flushed out until process leaves on windows
    checkTrue( !isTRUE( TRUE %in% as.logical( grep( "foo", dir( myBaseDir ) ) ) ) )
  }
}


"test.UsingRunAsUserMode" <- function()
{
  rxGetUsingRunAsUserMode <- .rxGetUsingRunAsUserMode()
  rxSetUsingRunAsUserMode <- .rxSetUsingRunAsUserMode()
  
  oldVal <- rxGetUsingRunAsUserMode()
  on.exit( rxSetUsingRunAsUserMode( oldVal ) )
  
  rxSetUsingRunAsUserMode ( TRUE )
  checkTrue( rxGetUsingRunAsUserMode() )
  
  rxSetUsingRunAsUserMode ( FALSE )
  checkTrue( !rxGetUsingRunAsUserMode() )
}
