# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for RxSasData
#
############################################################################################

"test.RxSasData.args.varsToKeep" <- function()
{
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    
    varsToKeep <- c("Y", "XFAC1")

    mySasData <- RxSasData( sasFileName, varsToKeep = varsToKeep)
     
    myData <- rxImport(inData = mySasData )
    checkEquals( names(myData), varsToKeep)  
    
    # Reverse order
    varsToKeep <- c("XFAC1", "Y")
    mySasData <- RxSasData( sasFileName, varsToKeep = varsToKeep)
     
    myData <- rxImport(inData = mySasData )
    checkEquals( names(myData), varsToKeep)     
      
}

"test.RxSasData.args.varsToDrop" <- function()
{
    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    
    varsToDrop<- c("Y", "XFAC1")

    mySasData <- RxSasData( sasFileName, varsToDrop = varsToDrop)
    
    allVarNames <- rxGetVarNames( RxSasData(sasFileName) )
    varsToKeep = allVarNames[!(allVarNames %in% varsToDrop)]
     
    myData <- rxImport(inData = mySasData )
    checkEquals( names(myData), varsToKeep)    
}

"test.RxSasData.function.rxCube" <- function()
{
  # An example with unspecified factor levels
  inFileSas <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  inFileXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 40)
  cubeOutSas <- rxResultsDF(rxCube(cost~age, data = sasDS))
  cubeOutXdf <- rxResultsDF(rxCube(cost~age, data = inFileXdf))
  checkEquals(cubeOutSas, cubeOutXdf)
  
  # An example with F
  inFileSas <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
  inFileXdf <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 5)
  cubeOutSas <- rxResultsDF(rxCube(~F(XINT1), data = sasDS))
  cubeOutXdf <- rxResultsDF(rxCube(~F(xint1), data = inFileXdf))
  checkEquals(cubeOutSas$XINT1, cubeOutXdf$xint1)
  checkEquals(cubeOutSas$Counts, cubeOutXdf$Counts)
}

"test.RxSasData.function.rxLinMod" <- function()
{
  # An example with unspecified factor levels
  inFileSas <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  inFileXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 40)
  linModOutSas <- rxLinMod(cost~age, data = sasDS)
  linModOutXdf <- rxLinMod(cost~age, data = inFileXdf)
  checkEquals(coef(linModOutSas), coef(linModOutXdf))
  checkEquals(linModOutSas$coef.p.value, linModOutXdf$coef.p.value)
  checkEquals(linModOutSas$df , linModOutXdf$df)
  rm(linModOutSas)
  rm(linModOutXdf)
  
  # An example with F
  inFileSas <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
  inFileXdf <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 5)
  linModOutSas <- rxLinMod(XNUM2~F(XINT1), data = sasDS)
  linModOutXdf <- rxLinMod(xnum2~F(xint1), data = inFileXdf)
  checkEquals(as.vector(coef(linModOutSas)), as.vector(coef(linModOutXdf)),
    tolerance=1e-7)
  checkEquals(as.vector(linModOutSas$coef.p.value), as.vector(linModOutXdf$coef.p.value),
    tolerance=1e-7)
  checkEquals(as.vector(linModOutSas$df) , as.vector(linModOutXdf$df))
}

"test.RxSasData.function.rxLogit" <- function()
{
  # An example with unspecified factor levels
  inFileSas <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  inFileXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 50)
  logitOutSas <- rxLogit(old~cost, data = sasDS,
       transforms = list(old = car_age %in% c("8-9", "10+")))

  logitOutXdf <- rxLogit(old~cost, data = inFileXdf,
    transforms = list(old = car.age %in% c("8-9", "10+")))
  checkEquals(coef(logitOutSas), coef(logitOutXdf), tolerance=1e-6)
  checkEquals(logitOutSas$coef.p.value, logitOutXdf$coef.p.value, tolerance=1e-6)
  checkEquals(logitOutSas$df , logitOutXdf$df)
  rm(logitOutSas)
  rm(logitOutXdf)
  
  # An example with F
  inFileSas <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
  inFileXdf <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 5)
  logitOutSas <- rxLogit(YLOGICAL~F(XINT1), initialValues=NA, data = sasDS)
  logitOutXdf <- rxLogit(ylogical~F(xint1), initialValues=NA, data = inFileXdf)
  checkEquals(as.vector(coef(logitOutSas)), as.vector(coef(logitOutXdf)),
    tolerance=1e-6)
  checkEquals(as.vector(logitOutSas$coef.p.value), as.vector(logitOutXdf$coef.p.value),
    tolerance=1e-6)
  checkEquals(as.vector(logitOutSas$df) , as.vector(logitOutXdf$df))
}

"test.RxSasData.function.rxGlm" <- function()
{
  # An example with unspecified factor levels
  inFileSas <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  inFileXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 50)
  logitOutSas <- rxGlm(old~cost, data = sasDS,  family = binomial(link = "logit"),
       transforms = list(old = car_age %in% c("8-9", "10+")))

  logitOutXdf <- rxGlm(old~cost, data = inFileXdf, family = binomial(link = "logit"),
    transforms = list(old = car.age %in% c("8-9", "10+")))
  checkEquals(coef(logitOutSas), coef(logitOutXdf), tolerance=1e-6)
  checkEquals(logitOutSas$coef.p.value, logitOutXdf$coef.p.value, tolerance=1e-6)
  checkEquals(logitOutSas$df , logitOutXdf$df)
  rm(logitOutSas)
  rm(logitOutXdf)
  
  # An example with F
  inFileSas <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
  inFileXdf <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b.xdf")
  sasDS <- RxSasData(file = inFileSas, stringsAsFactors = TRUE, rowsPerRead = 5)
  logitOutSas <- rxGlm(YLOGICAL~F(XINT1), initialValues=NA, data = sasDS,  family = binomial(link = "logit"))
  logitOutXdf <- rxGlm(ylogical~F(xint1), initialValues=NA, data = inFileXdf,  family = binomial(link = "logit"))
  checkEquals(as.vector(coef(logitOutSas)), as.vector(coef(logitOutXdf)),
    tolerance=1e-6)
  checkEquals(as.vector(logitOutSas$coef.p.value), as.vector(logitOutXdf$coef.p.value),
    tolerance=1e-6)
  checkEquals(as.vector(logitOutSas$df) , as.vector(logitOutXdf$df))
}

"test.RxSasData.method.head" <- function()
{
   sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    
    sasVarsToKeep <- c("Y", "XFAC1")
    
  xdfFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r1b.xdf")
  
  xdfDS <- RxXdfData(xdfFileName)
  headXdf <- head(xdfDS)
  varsToKeep <- c("y", "xfac1")
  headXdf <- subset(headXdf, select = varsToKeep) 
  
    sasDS <- RxSasData( sasFileName, varsToKeep = sasVarsToKeep, stringsAsFactors = TRUE) 

  headSas <- head(sasDS)

  # The SPSS data sources will have fewer factor levels and capitalized var names
  checkEquals(headSas, headXdf, checkNames = FALSE, check.attributes = FALSE, tolerance = 10E-7)
}

"test.RxSasData.method.names" <- function()
{
   sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")

  sasDS <- RxSasData( sasFileName) 
  importNames <- names(rxImport(sasDS, numRows = 3))
  methodNames <- names(sasDS)
  checkEquals(importNames, methodNames)
}


