# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxCompressXdf
#
############################################################################################

"test.rxCompressXdf.args.inFile" <- function()
{
  # Case 1: A single file
  
  fileDir <- tempdir()
  origXdf1 <- tempfile(pattern = ".rxCompress1", tmpdir = fileDir, fileext = ".xdf")
  newXdf1 <- tempfile(pattern = ".rxCompressNew1", tmpdir = fileDir, fileext = ".xdf")
    
    # Import a file without no compression
    testCsv1 <- file.path(rxGetOption("unitTestDataDir"), "richTest200rComma.txt")
    rxImport(inData = testCsv1, outFile = origXdf1, xdfCompressionLevel = 0)
        
  
  newDS <- rxCompressXdf(inFile = origXdf1, outFile = newXdf1, xdfCompressionLevel = 1)
  
  origFileInfo1 <- file.info(origXdf1)
  newFileInfo1 <- file.info(newXdf1)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  file.remove(newXdf1)
  
  # Case 2: RxXdfData objects
  
  origXdfDS1 <- RxXdfData(origXdf1)
  newXdfDS1 <- RxXdfData(newXdf1)
  newDS <- rxCompressXdf(inFile = origXdfDS1, outFile = newXdfDS1, xdfCompressionLevel = 1)
  
  origFileInfo1 <- file.info(origXdf1)
  newFileInfo1 <- file.info(newXdf1)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  file.remove(newXdf1)
  
  # Case 3: Mixture of RxXdfData and file names
  newXdfDS1 <- RxXdfData(newXdf1)
  newDS <- rxCompressXdf(inFile = origXdf1, outFile = newXdfDS1, xdfCompressionLevel = 1)
  
  origFileInfo1 <- file.info(origXdf1)
  newFileInfo1 <- file.info(newXdf1)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  file.remove(newXdf1)
  
  # Case 4: A vector of file names
  
  origXdf2 <- tempfile(pattern = ".rxCompress2", tmpdir = fileDir, fileext = ".xdf")
  newXdf2 <- tempfile(pattern = ".rxCompressNew2", tmpdir = fileDir, fileext = ".xdf")
    
    # Import a file without no compression
    testCsv2 <- file.path(rxGetOption("unitTestDataDir"), "richTest300rComma.txt")
    rxImport(inData = testCsv2, outFile = origXdf2, xdfCompressionLevel = 0)
    
  
  newDS <- rxCompressXdf(inFile = c(origXdf1, origXdf2), outFile = c(newXdf1, newXdf2), xdfCompressionLevel = 2)
  origFileInfo2 <- file.info(origXdf2)
  newFileInfo2<- file.info(newXdf2)
  newFileInfo1a<- file.info(newXdf1)
  checkTrue(newFileInfo1$size > newFileInfo1a$size)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  checkTrue(origFileInfo2$size > newFileInfo2$size)
  checkEquals(rxXdfFileName(newDS[[1]]), normalizePath(newXdf1))
  checkEquals(rxXdfFileName(newDS[[2]]), normalizePath(newXdf2))
  file.remove(newXdf1)
  file.remove(newXdf2)
  
  # Case 5: A list of RxXdfData objects
  origXdfDS1 <- RxXdfData(origXdf1)
  newXdfDS1 <- RxXdfData(newXdf1)
  origXdfDS2 <- RxXdfData(origXdf2)
  newXdfDS2 <- RxXdfData(newXdf2)
  newDS <- rxCompressXdf(inFile = c(origXdfDS1, origXdfDS2), outFile = c(newXdfDS1, newXdfDS2), xdfCompressionLevel = 2)
  origFileInfo2 <- file.info(origXdf2)
  newFileInfo2 <- file.info(newXdf2)
  newFileInfo1a <- file.info(newXdf1)
  checkTrue(newFileInfo1$size > newFileInfo1a$size)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  checkTrue(origFileInfo2$size > newFileInfo2$size)
  checkEquals(rxXdfFileName(newDS[[1]]), normalizePath(newXdf1))
  checkEquals(rxXdfFileName(newDS[[2]]), normalizePath(newXdf2))	
  file.remove(newXdf1)
  file.remove(newXdf2)
  
  # Case 6: A directory containing .xdf files, specifying a new directory for compressed files
  subDir <- "testDir"
  newFileDir <- file.path(fileDir, subDir)
  if (!file.exists(newFileDir))
  {
    dir.create(newFileDir)
  }
  newDS <- rxCompressXdf(inFile = fileDir, outFile = newFileDir, xdfCompressionLevel = 1, overwrite = TRUE)
  origFiles <- list.files(path = fileDir, pattern = ".xdf", all.files = TRUE, ignore.case = TRUE)
  newFiles <- file.path(newFileDir, origFiles)
    print(rxXdfFileName(newDS[[1]]))
    print(normalizePath(newFiles[1]))
    
  checkEquals(normalizePath(rxXdfFileName(newDS[[1]])), normalizePath(newFiles[1]))
  checkEquals(normalizePath(rxXdfFileName(newDS[[2]])), normalizePath(newFiles[2]))
  newFileInfo1 <- file.info(newFiles[1])
  newFileInfo2 <- file.info(newFiles[2])
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  checkTrue(origFileInfo2$size > newFileInfo2$size)	
  file.remove(newFiles[1])
  file.remove(newFiles[2])
  
  # Case 7: This will overwrite the existing temporary 'orig' files in the directory
  newDS <- rxCompressXdf(inFile = fileDir, xdfCompressionLevel = 1, overwrite = TRUE)
  overwriteFileInfo1 <- file.info(rxXdfFileName(newDS[[1]]))
  overwriteFileInfo2 <- file.info(rxXdfFileName(newDS[[2]]))
  checkTrue(origFileInfo1$size > overwriteFileInfo1$size)
  checkTrue(origFileInfo2$size > overwriteFileInfo2$size)	
  
  # Final clean-up
  # This removes all .xdf files from temp directory, if desired
  #origFiles <- list.files(path = fileDir, pattern = ".xdf", all.files = TRUE,
    #ignore.case = TRUE, full.names = TRUE, recursive = TRUE)
  #if (length(origFiles) > 0)
  #{
    #file.remove(origFiles)
  #}

  # This should be sufficient clean up for a single run
  file.remove(origXdf1)
  file.remove(origXdf2)
    
    if (file.exists(newFileDir)) unlink(newFileDir, recursive = TRUE)      
}

"test.rxCompressXdf.args.outFile" <- function()
{
  fileDir <- tempdir()
  origXdf1 <- tempfile(pattern = ".rxCompress1", tmpdir = fileDir, fileext = ".xdf")
  newXdf1 <- tempfile(pattern = ".rxCompressNew1", tmpdir = fileDir, fileext = ".xdf")

    # Import a file without no compression
    testCsv1 <- file.path(rxGetOption("unitTestDataDir"), "richTest200rComma.txt")
    rxImport(inData = testCsv1, outFile = origXdf1, xdfCompressionLevel = 0)    

  # Check for error when outFile is a mismatch	
  res <- try(
    newDS <- rxCompressXdf(inFile = origXdf1, outFile = c(newXdf1, "nothing"), xdfCompressionLevel = 1)
    )
  checkTrue(is(res,"try-error"))
  checkTrue(grepl("are of different lengths", attr(res, "condition")$message))
  
  # Omit outFile and make sure file is overwritten
  origFileInfo1 <- file.info(origXdf1)
  newDS <- rxCompressXdf(inFile = origXdf1, xdfCompressionLevel = 1, overwrite = TRUE)
  newFileInfo1 <- file.info(origXdf1)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  
  file.remove(origXdf1)
  
  # Same with with an input RxXdfData 
  rxImport(inData = testCsv1, outFile = origXdf1, xdfCompressionLevel = 0)  
    origFileInfo1 <- file.info(origXdf1)
  newDS <- rxCompressXdf(inFile = RxXdfData(origXdf1), xdfCompressionLevel = 1, overwrite = TRUE)
  newFileInfo1 <- file.info(origXdf1)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
    
  # Try input list RxXdfData and output vector of file names
    
    file.remove(origXdf1)
    rxImport(inData = testCsv1, outFile = origXdf1, xdfCompressionLevel = 0)  
  origXdf2 <- tempfile(pattern = ".rxCompress2", tmpdir = fileDir, fileext = ".xdf")
  newXdf2 <- tempfile(pattern = ".rxCompressNew2", tmpdir = fileDir, fileext = ".xdf")
    
    # Import a file with no compression
    testCsv2 <- file.path(rxGetOption("unitTestDataDir"), "richTest300rComma.txt")
    rxImport(inData = testCsv2, outFile = origXdf2, xdfCompressionLevel = 0)        
    
  origXdfDS1 <- RxXdfData(origXdf1)
  origXdfDS2 <- RxXdfData(origXdf2)
    newFiles <- c(newXdf1, newXdf2)
  newDS <- rxCompressXdf(inFile = c(origXdfDS1, origXdfDS2), outFile = newFiles, xdfCompressionLevel = 2, overwrite = TRUE)
    origFileInfo1 <- file.info(origXdf1)
  origFileInfo2 <- file.info(origXdf2)
    newFileInfo1 <- file.info(newXdf1)
  newFileInfo2 <- file.info(newXdf2)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
  checkTrue(origFileInfo2$size > newFileInfo2$size)
  print(newDS[[1]])
  print(newXdf1)
  checkEquals(rxXdfFileName(newDS[[1]]), normalizePath(newXdf1))
  checkEquals(rxXdfFileName(newDS[[2]]), normalizePath(newXdf2))	
  file.remove(newXdf1)
  file.remove(newXdf2)   
    
    file.remove(origXdf1)
    file.remove(origXdf2) 
}

"test.rxCompressXdf.args.xdfCompressionLevel" <- function()	
{ 
    origXdf1 <- tempfile(pattern = ".rxCompress",  fileext = ".xdf")
  newXdf1 <- tempfile(pattern = ".rxCompressNew", fileext = ".xdf")
    
    # Import a file without no compression
    testCsv1 <- file.path(rxGetOption("unitTestDataDir"), "richTest200rComma.txt")
    rxImport(inData = testCsv1, outFile = origXdf1, xdfCompressionLevel = 0)      
    
    origFileInfo1 <- file.info(origXdf1)
    
    oldOptions <- rxOptions(xdfCompressionLevel = 1)
    oldCompression <- oldOptions$xdfCompressionLevel
    on.exit(rxOptions(xdfCompressionLevel = oldCompression), add = TRUE)
    
    rxCompressXdf(inFile = origXdf1, outFile = newXdf1)
    
    newFileInfo1 <- file.info(newXdf1)
  checkTrue(origFileInfo1$size > newFileInfo1$size)
    
    file.remove(newXdf1) 
    
    rxOptions(xdfCompressionLevel = 3)
    
  rxCompressXdf(inFile = origXdf1, outFile = newXdf1, xdfCompressionLevel = 3)
    newFileInfo3 <- file.info(newXdf1)
    checkTrue(newFileInfo1$size > newFileInfo3$size) 
    
    file.remove(newXdf1)
    
    rxOptions(xdfCompressionLevel = 0)
    rxCompressXdf(inFile = origXdf1, outFile = newXdf1, xdfCompressionLevel = 1)
    newFileInfo1a <- file.info(newXdf1)
    checkEquals(newFileInfo1$size, newFileInfo1a$size)
    
    file.remove(newXdf1)
    file.remove(origXdf1)
    
    # Make sure no other options were modified
    oldOptions1 <- rxOptions(xdfCompressionLevel = oldCompression)
    oldOptions1$xdfCompressionLevel <- oldCompression
    oldOptions1$reset <- NULL
    oldOptions$reset <- NULL
    checkEquals(oldOptions, oldOptions1)
}

"test.rxCompressXdf.args.overwrite" <- function()	
{ 
    origXdf1 <- tempfile(pattern = ".rxCompress", fileext = ".xdf")
  newXdf1 <- tempfile(pattern = ".rxCompressNew", fileext = ".xdf")
    
    # Import a file without no compression
    testCsv1 <- file.path(rxGetOption("unitTestDataDir"), "richTest200rComma.txt")
    rxImport(inData = testCsv1, outFile = origXdf1, xdfCompressionLevel = 0)  
        
    rxCompressXdf(inFile = origXdf1, outFile = newXdf1, xdfCompressionLevel = 1)
    
    # Second try should fail without overwrite
    res <- try(
    newDS <- rxCompressXdf(inFile = origXdf1, outFile = newXdf1, xdfCompressionLevel = 2)
    )
  checkTrue(is(res,"try-error"))
  checkTrue(grepl("already exists", attr(res, "condition")$message))
    
    newDS <- rxCompressXdf(inFile = origXdf1, outFile = newXdf1, xdfCompressionLevel = 2, overwrite = TRUE)
    checkTrue(inherits(newDS, "RxXdfData"))
    
    file.remove(origXdf1)
    file.remove(newXdf1)
}

"test.rxCompressXdf.args.reportProgress" <- function()
{
    DEACTIVATED("Bug 74527: Unit test test.rxCompressXdf.args.reportProgress and RevoIOQ test test.rxCompressXdf.args.reportProgress failing")
    
    newXdf1 <- tempfile(pattern = ".rxCompressNew", fileext = ".xdf")
    sampleXdf1 <- tempfile(pattern = ".rxCompressOrig", fileext = ".xdf")
    
    # Import a file without no compression
    testCsv1 <- file.path(rxGetOption("unitTestDataDir"), "richTest200rComma.txt")
    rxImport(inData = testCsv1, outFile = sampleXdf1, rowsPerRead = 50, xdfCompressionLevel = 0)      
    
    out0 <- capture.output(newDS <- rxCompressXdf(inFile = sampleXdf1, outFile = newXdf1, xdfCompressionLevel = 1, reportProgress = 0))
    out1 <- capture.output(newDS <- rxCompressXdf(inFile = sampleXdf1, outFile = newXdf1, xdfCompressionLevel = 1, reportProgress = 1, overwrite = TRUE))
    out2 <- capture.output(newDS <- rxCompressXdf(inFile = sampleXdf1, outFile = newXdf1, xdfCompressionLevel = 1, reportProgress = 2, overwrite = TRUE))
    out3 <- capture.output(newDS <- rxCompressXdf(inFile = sampleXdf1, outFile = newXdf1, xdfCompressionLevel = 1, reportProgress = 3, overwrite = TRUE)) 
    
    checkEquals(length(out0), 0)
    checkEquals(length(out1), 1)
    if (rxIsExpressEdition())
    {
        checkEquals(length(out2), 1)
    }   else 
    {
        checkEquals(length(out2), 4)
    }
    checkTrue(length(out3) > 8)
    
    file.remove(newXdf1) 
    file.remove(sampleXdf1)
}


