# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for:
#      rxCovCoef
#      rxCovData
#      rxCorCoef
#
############################################################################################

"test.rxCovRegression.rxVCToMatrix" <- function()
{
    "..rxVCToMatrix" <- function()
    {
        if (exists(".rxVCToMatrix", globalenv()))
            get(".rxVCToMatrix", globalenv())
        else
            RevoScaleR:::.rxVCToMatrix
    }

    ".rxGenVCList" <- function(cubeLength, varLength)
    {
        parts <- vector("list", length=3)
        K <- cubeLength
        M <- varLength
        
        # form diagonal corresponding to KxK cube portion of the matrix 
        parts[[1]] <- matrix(seq(K))
        
        # form upper rhs of cov matrix which is a KxM matrix
        parts[[2]] <- matrix(seq(K*M), K, M)
        
        # form lower rhs corner of cov matrix, which is an MxM symmetric matrix
        z <- diag(seq(M))/2
        urz <- upper.tri(z)
        z[urz] <- seq(from=M+1, length.out=length(which(urz)))
        parts[[3]] <- z + t(z)
        
        parts
    }

    K <- 4 
    M <- 3
    parts <- .rxGenVCList(K,M)
    VC <- ..rxVCToMatrix()(parts)
    checkEquals(rep(K+M,2), dim(VC))
    checkEquals(diag(VC[seq(K),seq(K)]), as.vector(parts[[1]]))
    VCRHS <- VC[-seq(K),-seq(K)]
    checkEquals(VC[-seq(K+1,length.out=M),-seq(K)], parts[[2]])
    checkEquals(VC[-seq(K),-seq(K)], parts[[3]])
}

###
# Methods
###

"test.rxCovCoef.rxLinMod" <- function()
{
    # acceptable case
    form <- Sepal.Width ~ Sepal.Length + Petal.Length + Petal.Width
  z <- rxLinMod(form, data = iris, covCoef = TRUE)
  zCovCoef <- rxCovCoef(z)
    checkTrue(is.matrix(zCovCoef))
    checkTrue(is.numeric(zCovCoef))
    checkTrue(nrow(zCovCoef) == ncol(zCovCoef))
    
    # covCoef or cube set incorrectly
  z <- rxLinMod(form, data = iris, covCoef = FALSE, cube = FALSE)
  checkException(rxCovCoef(z),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because covCoef = FALSE")
  checkException(
        rxCovCoef(rxLinMod(Sepal.Width ~ Species + Sepal.Length, data = iris, covCoef = TRUE, cube = TRUE)),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because cube = TRUE")

    # zero degrees of freedom error
    z$df[2] <- 0 # force faux condition
    checkException(rxCovCoef(z), msg="Caught exception: zero degrees of freedom")    
}

"test.rxCovCoef.rxLogit" <- function()
{
    # acceptable case
  form <- case ~ education + age + parity + spontaneous + induced
    z <- rxLogit(form, data=infert, covCoef=TRUE)
  zCovCoef <- rxCovCoef(z)
    checkTrue(is.matrix(zCovCoef))
    checkTrue(is.numeric(zCovCoef))
    checkTrue(nrow(zCovCoef) == ncol(zCovCoef))

    # covCoef or cube set incorrectly
    z <- rxLogit(form, data=infert, covCoef = FALSE)
  checkException(rxCovCoef(z), msg="Caught exception: missing covCoef data in return object")	
  checkException(rxCovCoef(rxLogit(form, data=infert, covCoef = TRUE, cube = TRUE)),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because cube = TRUE")	
}

"test.rxCovCoef.rxGlm" <- function()
{
    # acceptable case
  form <- case ~ education + age + parity + spontaneous + induced
    z <- rxGlm(form, data=infert, covCoef=TRUE)
  zCovCoef <- rxCovCoef(z)
    checkTrue(is.matrix(zCovCoef))
    checkTrue(is.numeric(zCovCoef))
    checkTrue(nrow(zCovCoef) == ncol(zCovCoef))

    # covCoef or cube set incorrectly
    z <- rxGlm(form, data=infert, covCoef = FALSE)
  checkException(rxCovCoef(z), msg="Caught exception: missing covCoef data in return object")	
  checkException(rxCovCoef(rxLogit(form, data=infert, covCoef = TRUE, cube = TRUE)),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because cube = TRUE")	
}

"test.rxCovData.rxLinMod" <- function()
{
  form <- score ~ division + state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), 
        state=sample(state.name, 5, replace=TRUE), score=runif(5), 
        height=rnorm(5), weight=rnorm(5))
  
    z <- rxLinMod(form, data=DF, covData=TRUE)
  zCovData <- rxCovData(z)
    checkTrue(is.matrix(zCovData))
    checkTrue(is.numeric(zCovData))
    checkTrue(nrow(zCovData) == ncol(zCovData))

    # covData or cube set incorrectly
  z <- rxLinMod(form, data=DF, covData=FALSE, cube=FALSE)
  checkException(rxCovData(z), msg="Caught exception: missing covData data in return object")	
  checkException(rxCovData(rxLinMod(form, data=DF, covData=TRUE, cube=TRUE)),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because cube = TRUE")
    
    # check values
  z <- rxLinMod(score ~ height + weight, data=DF, covData=TRUE, cube=FALSE)
    checkEqualsNumeric(diag(rxCovData(z)), c(var(DF$height), var(DF$weight)))
    
    z <- rxLinMod(Sepal.Width ~ Sepal.Length + Petal.Length + Petal.Width, data = iris, covData = TRUE)
    checkEqualsNumeric(cov(iris[,c("Sepal.Length","Petal.Length", "Petal.Width")]), rxCovData(z))
}

"test.rxCovData.rxLogit" <- function()
{
  form <- case ~ education +age + parity + spontaneous + induced
  
    # test various combinations of covData and covData
    z <- rxLogit(form, data=infert, covData=TRUE)
  zCovData <- rxCovData(z)
    checkTrue(is.matrix(zCovData))
    checkTrue(is.numeric(zCovData))
    checkTrue(nrow(zCovData) == ncol(zCovData))
    
    # covData or cube set incorrectly
  z <- rxLogit(form, data=infert, covData=FALSE, cube=FALSE)
  checkException(rxCovData(z), msg="Caught exception: missing covData data in return object")	
  checkException(rxCovData(rxLogit(form, data=infert, covData=TRUE, cube=TRUE)),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because cube = TRUE")

    # check values
  z <- rxLogit(form, data=infert, covData=TRUE, cube=FALSE)
    checkEqualsNumeric(diag(rxCovData(z))["age"], var(infert["age"]))
}


####

"test.rxCorCoef.rxLinMod" <- function()
{
    # acceptable case
    form <- Sepal.Width ~ Sepal.Length + Petal.Length + Petal.Width
  z <- rxLinMod(form, data = iris, covCoef = TRUE)
  zCorCoef <- rxCorCoef(z)
    checkTrue(is.matrix(zCorCoef))
    checkTrue(is.numeric(zCorCoef))
    checkTrue(nrow(zCorCoef) == ncol(zCorCoef))
    
    # corCoef or cube set incorrectly
  z <- rxLinMod(form, data = iris, covCoef = FALSE)
  checkException(rxCorCoef(z),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because covCoef=FALSE")
    checkException(rxLinMod(Sepal.Width ~ Species + Sepal.Length, data = iris, covCoef = TRUE, cube = TRUE),
        msg="Caught exception: VC matrix of regression coefficients nonexistent because cube=TRUE")
  z <- rxLinMod(Sepal.Width ~ Species + Sepal.Length, data = iris, covData = TRUE, cube = FALSE)
  checkException(rxCorCoef(z),        
        msg="Caught exception: VC matrix of regression coefficients nonexistent because covCoef=FALSE")

    # zero degrees of freedom error
    DF <- data.frame(division=c("i","g","o","b","m"), score=1:5)
    z <- rxLinMod(score ~ division, data=DF, covCoef=TRUE, cube=FALSE)
    checkException(rxCorCoef(z), msg="Caught exception: zero degrees of freedom")

    # check values
    z <- rxLinMod(form, data = iris, covData = TRUE)
    checkEqualsNumeric(cor(iris[,c("Sepal.Length","Petal.Length", "Petal.Width")]), rxCorData(z))
}


"test.rxCorCoef.rxLogit" <- function()
{
    # acceptable case
  form <- case ~ education + age + parity + spontaneous + induced
    z <- rxLogit(form, data=infert, covCoef=TRUE)
  zCorCoef <- rxCorCoef(z)
    checkTrue(is.matrix(zCorCoef))
    checkTrue(is.numeric(zCorCoef))
    checkTrue(nrow(zCorCoef) == ncol(zCorCoef))

    # corCoef or cube set incorrectly
    z <- rxLogit(form, data=infert, covCoef = FALSE)
  checkException(rxCorCoef(z), msg="Caught exception: missing corCoef data in return object")	
    checkException(rxLogit(form, data=infert, covCoef = TRUE, cube = TRUE),
        msg="Caught exception: no VC matrix calculated because cube = TRUE")
    z <- rxLogit(form, data=infert, covData = TRUE, cube = FALSE)
  checkException(rxCorCoef(z), msg="Caught exception: missing covCoef data in return object")	
}

"test.rxCorCoef.rxGlm" <- function()
{
    # acceptable case
  form <- case ~ education + age + parity + spontaneous + induced
    z <- rxGlm(form, data=infert, covCoef=TRUE)
  zCorCoef <- rxCorCoef(z)
    checkTrue(is.matrix(zCorCoef))
    checkTrue(is.numeric(zCorCoef))
    checkTrue(nrow(zCorCoef) == ncol(zCorCoef))

    # corCoef or cube set incorrectly
    z <- rxGlm(form, data=infert, covCoef = FALSE)
  checkException(rxCorCoef(z), msg="Caught exception: missing corCoef data in return object")	
    checkException(rxGlm(form, data=infert, covCoef = TRUE, cube = TRUE),
        msg="Caught exception: no VC matrix calculated because cube = TRUE")
}
"test.rxCorData.rxLinMod" <- function()
{
  form <- score ~ division + state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), 
        state=sample(state.name, 5, replace=TRUE), score=runif(5), 
        height=rnorm(5), weight=rnorm(5))
  
    z <- rxLinMod(form, data=DF, covData=TRUE)
  zCorData <- rxCorData(z)
    checkTrue(is.matrix(zCorData))
    checkTrue(is.numeric(zCorData))
    checkTrue(nrow(zCorData) == ncol(zCorData))

    # corData or cube set incorrectly
  z <- rxLinMod(form, data=DF, covData=FALSE, cube=FALSE)
  checkException(rxCorData(z), msg="Caught exception: missing corData data in return object")	
  checkException(rxLinMod(form, data=DF, covData=TRUE, cube=TRUE),
        msg="Caught exception: no VC matrix calculated because cube = TRUE")
  z <- rxLinMod(form, data=DF, covCoef=TRUE, cube=FALSE)
  checkException(rxCorData(z), msg="Caught exception: missing covData data in return object")
    
    # check values
  z <- rxLinMod(score ~ height + weight, data=DF, covData=TRUE, cube=FALSE)
    checkEqualsNumeric(diag(rxCorData(z)), rep(1,2))
    
    z <- rxLinMod(Sepal.Width ~ Sepal.Length + Petal.Length + Petal.Width, data = iris, covData = TRUE)
    checkEqualsNumeric(cor(iris[,c("Sepal.Length","Petal.Length", "Petal.Width")]), rxCorData(z))    
}

"test.rxCorData.rxLogit" <- function()
{
  form <- case ~ education + age + parity + spontaneous + induced
  
    # test various combinations of corData and corData
    z <- rxLogit(form, data=infert, covData=TRUE)
  zCorData <- rxCorData(z)
    checkTrue(is.matrix(zCorData))
    checkTrue(is.numeric(zCorData))
    checkTrue(nrow(zCorData) == ncol(zCorData))
    
    # corData or cube set incorrectly
  z <- rxLogit(form, data=infert, covData=FALSE, cube=FALSE)
  checkException(rxCorData(z), msg="Caught exception: missing corData data in return object")	
  checkException(rxLogit(form, data=infert, covData=TRUE, cube=TRUE),
        msg="Caught exception: no VC matrix calculated because cube = TRUE")
  z <- rxLogit(form, data=infert, covCoef=TRUE, cube=FALSE)
  checkException(rxCorData(z), msg="Caught exception: missing covData data in return object")

    # check values
  z <- rxLogit(form, data=infert, covData=TRUE, cube=FALSE)
    checkEqualsNumeric(diag(rxCorData(z))["age"], 1)
}


