# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxCreateColInfo
#
############################################################################################

"test.rxDataStep.args.data" <- function()
{
    # Xdf file name
    airXdf <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
    colInfo1 <- rxCreateColInfo(data = airXdf)
    checkEquals(colInfo1$CRSDepTime$type, "float32")
    
    # Xdf data source
    airXdfDS <- RxXdfData(file = airXdf)
    colInfo2 <- rxCreateColInfo(data = airXdfDS)
    
    checkIdentical(colInfo1, colInfo2)
    
    # Xdf varInfo
    colInfo3 <- rxCreateColInfo(data = rxGetVarInfo(airXdfDS))
    checkIdentical(colInfo1, colInfo3)
    
    # Data frame:  no float32
    airDF <- rxDataStep(inData = airXdf)
    colInfo4 <- rxCreateColInfo(airDF)
    checkEquals(colInfo4$CRSDepTime$type, "numeric")
    checkEquals(colInfo1$DayOfWeek, colInfo4$DayOfWeek)
    
    # CSV data source (
    airCsv <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.csv")
    airCsvDS <- RxTextData(airCsv, missingValueString = "M", colClasses = c(DayOfWeek = "factor"))  
    colInfo5 <- rxCreateColInfo(airCsvDS)
    checkIdentical(colInfo1, colInfo5)
    
    # Date data
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDatesData.txt")
  
    tinyDS <- RxTextData(tinyFileName, defaultDecimalColType = "numeric", centuryCutoff = 0
        , readDateFormat = "%Y/%m/%d"
    )
    colInfo6 <- rxCreateColInfo(tinyDS)
    checkEquals(colInfo6$NewDates$type, "Date")
    outData1 <- rxImport(tinyDS)
    outData2 <- rxImport(inData = RxTextData(file = tinyFileName, colInfo = colInfo6, 
        readDateFormat = "%Y/%m/%d"))
    checkEquals(outData1$NewDates, outData2$NewDates)
    
}

"test.rxDataStep.args.includeLowHigh" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    
    colInfo0 <- rxCreateColInfo(claimsXdf, includeLowHigh = FALSE)
    checkTrue( is.null(colInfo0$number$low))
    checkTrue( is.null(colInfo0$number$high))
    
    colInfo1 <- rxCreateColInfo(claimsXdf, includeLowHigh = TRUE)
    checkEquals(colInfo1$number$low, 0)
    checkEquals(colInfo1$number$high, 434)
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDS <- RxTextData(file = claimsTxt)
    colInfo2 <- rxCreateColInfo(claimsTxtDS, includeLowHigh = TRUE)
    # Non-factor variables should be the same
    checkIdentical(colInfo1$number, colInfo2$number)
    checkIdentical(colInfo1$cost, colInfo2$cost)
    checkIdentical(colInfo1$RowNum, colInfo2$RowNum)
     
}

"test.rxDataStep.args.factorsOnly" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    
    colInfo0 <- rxCreateColInfo(claimsXdf, factorsOnly = FALSE)
    checkTrue( !is.null(colInfo0$number))
    checkTrue( !is.null(colInfo0$RowNum))
    
    colInfo1 <- rxCreateColInfo(claimsXdf, factorsOnly = TRUE)
    checkTrue( is.null(colInfo1$number))
    checkTrue( is.null(colInfo1$RowNum))
    
    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDS <- RxTextData(file = claimsTxt, colClasses = c(age = "factor"))
    colInfo2 <- rxCreateColInfo(claimsTxtDS, factorsOnly = TRUE)
    
    # Variable specified as factor should be the same
    checkIdentical(colInfo1$age, colInfo2$age)    
}

"test.rxDataStep.args.sortLevels" <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf")
    
    sortInteger <- function( x )
    {
        x <- as.integer(x)
        return( as.character(sort(x)))
    }    
    
    colInfo0 <- rxCreateColInfo(testXdf, sortLevels = FALSE)
    checkTrue(colInfo0$xfac1$levels[1] != sortInteger(colInfo0$xfac1$levels)[1])          
    checkTrue(colInfo0$xfac2$levels[1] != sort(colInfo0$xfac2$levels)[1])    
    
    colInfo1 <- rxCreateColInfo(testXdf, sortLevels = TRUE)
    checkEquals(colInfo1$xfac1$levels, as.character(5:14))
    checkEquals(colInfo1$xfac2$levels, sort(colInfo1$xfac2$levels))  
    
    testTxt <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    testTxtDS <- RxTextData(file = testTxt, colClasses = c(type = "factor", ylogical = "factor", 
        xfac1 = "factor", xfac2 = "factor"))
    colInfo2 <- rxCreateColInfo(testTxtDS, sortLevels = TRUE)
    
    # Variable specified as factor should be the same
    checkIdentical(colInfo1, colInfo2)  
    
    # Make sure we're not converting decimals to integers
    myData <- data.frame(x = as.character(10:1), y = as.character(10:1 + .5)) 
    
    colInfo3 <- rxCreateColInfo(myData, sortLevels = TRUE) 
    checkEquals(colInfo3$x$levels, as.character(1:10))
    checkEquals(colInfo3$y$levels, sort(as.character(10:1 + .5)))
}

"test.rxDataStep.args.computeInfo" <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r1b.xdf") 
    xdfVarInfo <- rxGetVarInfo(testXdf)
    testTxt <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    testTxtDS <- RxTextData(file = testTxt, colClasses = c(type = "factor", ylogical = "factor", 
        xfac1 = "factor", xfac2 = "factor"))
    colInfo0 <- rxCreateColInfo(testTxtDS, computeInfo = TRUE)
    checkEquals(colInfo0$xfac2$levels, xdfVarInfo$xfac2$levels)
    
    colInfo1 <- rxCreateColInfo(testTxtDS, computeInfo = FALSE)
    checkTrue(is.null(colInfo1$xfac2$levels))
    
    # Combine with includeLowHigh
    colInfo2 <- rxCreateColInfo(testTxtDS, computeInfo = TRUE, includeLowHigh = TRUE)
    checkEquals(colInfo2$xint1$low, 5)
    checkEquals(colInfo2$xint1$high, 14)
    
    colInfo3 <- rxCreateColInfo(testTxtDS, computeInfo = FALSE, includeLowHigh = TRUE)
    checkTrue(is.na(colInfo3$xint1$low))
    checkTrue(is.na(colInfo3$xint1$high))   
}
