# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxCrossTabs
#
############################################################################################# rxCrossTabs RUnit test file

library(MASS)

sampleDataDir <- rxGetOption("sampleDataDir")
CensusPath <- file.path(sampleDataDir, "CensusWorkers.xdf")

###
# local function definition
###

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

"compareXtabs" <- function(formula, data, 
     msg=paste("FORMULA:", deparse(substitute(formula)), ", DATA:", 
         deparse(substitute(data))))
{
  xtabsFormula <- formula(gsub(":","+", deparse(formula)))
  z <- xtabs(formula=xtabsFormula, data=data)
  zz <- rxCrossTabs(formula=formula, data=data)
  output <- if (zz$ndepvars == 0) "counts" else "sums"
  zz <- zz[[output]]

  if (length(zz) == 1)
  {
    checkTrue(all.equal(unclass(z), unclass(zz[[1]]), check.attributes=FALSE), msg)
  }
  else
  {
    checkTrue(all.equal(as.vector(unlist(z)), as.vector(unlist(zz))), msg)
  }
}

"rxCrossTabsStressTest" <- function(formula, transformFunc=NULL, transformVars=NULL, data=NULL,pweights=NULL, blocksPerRead=-1,...)
{		

    # run the crosstab
  res <- try(do.call("rxCrossTabs", list(formula=formula,data=data, pweights=pweights, 
        transformFunc=transformFunc, transformVars=transformVars, blocksPerRead=blocksPerRead)))
    
    # return TRUE for no errors encountered during execution
  return (!is(res, "try-error"))
}

"rxCrossTabsStressTestDf" <- function(formula, data, pweights=NULL, ...)
{
    # run the crosstab
  res <- try(do.call("rxCrossTabs", list(formula=formula,data=data, pweights=pweights, ...)))
    
    # return TRUE for no errors encountered during execution
  return (!is(res, "try-error"))
}

#################################################################################

###
# rxCrosstabs and xtabs output comparison
###

"test.rxCrossTabs.xtabs.comparison.basic.df" <- function()
{
  # basic example
  DF <- as.data.frame(UCBAdmissions)
  compareXtabs(Freq ~ Gender : Dept : Admit, DF)
  #TODO: uncomment when we get dot exapansion support in ExaStat
  #compareXtabs(Freq ~., DF)
}

"test.rxCrossTabs.xtabs.comparison.multiple.response.variables.df" <- function()
{
  # example where multiple dependent variables are specified
  compareXtabs(ncontrols ~ agegp : alcgp : tobgp, esoph)
  compareXtabs(ncases ~ agegp : alcgp : tobgp, esoph)
  compareXtabs(cbind(ncases, ncontrols) ~ agegp : alcgp : tobgp, esoph)
  #TODO: uncomment when we get dot exapansion support in ExaStat
  #compareXtabs(cbind(ncases, ncontrols) ~., esoph)
}

"test.rxCrossTabs.xtabs.comparison.factor.conversion.df" <- function()
{
  # small example where age must be a factor. the conversion is done internally
  # in the rxCrossTabs.formula code when the input data source is a local data.frame.
  # in this case, all independent variables are converted to factors so that exastat

  # define local data.frame objects
  myDF1 <- data.frame(sex=c("Male","Male","Female", "Male"), age=as.factor(c(20,20,12,15)), score=(1:4)+0.1)
  myDF2 <- data.frame(sex=c("Male","Male","Female", "Male"), age=c(20,20,12,15), score=(1:4)+0.1)

  # will return comparable results to xtabs
  z1 <- rxCrossTabs(~ sex : age, myDF1)
  compareXtabs(~ sex : age, myDF1)

  z2 <- rxCrossTabs(~ sex : F(age), myDF2)
  checkTrue(all.equal(z1$sums, z2$sums, check.attributes=FALSE))

  z3 <- rxCrossTabs(~ sex : F(age), myDF2)
  checkTrue(all.equal(z1$sums, z3$sums, check.attributes=FALSE))
  
  zx <- xtabs(~ sex + age, myDF2)
  xform <- function(z)
  {
  z$ageF <- factor(z$age)
  z
  }
  z4 <- rxCrossTabs(~ sex : ageF, myDF2, transformFunc=xform, transformVars="age")
  checkTrue(all.equal(unclass(zx), unclass(z4$counts[[1]]), check.attributes=FALSE))

  compareXtabs(score ~ sex : age, myDF1)
  compareXtabs(score ~ sex : factor(age), myDF2)

  # compare output for multiple response variables specified
  # in formula and where some of those are expressions
  
  #TODO: uncomment when we get dot exapansion support in ExaStat
  #compareXtabs(cbind(sin(ncases), ncases, ncontrols, log(ncontrols)) ~., esoph)
}

"test.rxCrossTabs.comparison.factored.depvar.xdf" <- function()
{

  myDataFrame <- data.frame(x=runif(20), y=factor(rep(1:5, times=4)), z=rep(1:5, each=4))
  myDataFrame$zfac <- as.factor(myDataFrame$z)  
  out1 <- rxCrossTabs(x ~ y : F(z), data=myDataFrame)$sum
  out2 <- rxCrossTabs(x ~ y : zfac, data=myDataFrame)$sum
  checkTrue(all.equal(out1,out2, check.attributes=FALSE))
}

###
# output list elemental name checks
###

"test.rxCrossTabs.single.depvar.xdf.output.names" <- function()
{
  z <- rxCrossTabs(wkswork1 ~ sex : F(age), data=CensusPath, pweights="perwt", blocksPerRead=3)
  checkTrue(names(z$sums) == "wkswork1")
}

"test.rxCrossTabs.multiple.depvar.xdf.output.names" <- function()
{
  z <- rxCrossTabs(cbind(wkswork1, age) ~ sex : F(age), data=CensusPath, pweights="perwt", 
    blocksPerRead=3)
  checkTrue(all(names(z$sums) == c("wkswork1", "age")))
}

"test.rxCrossTabs.no.depvar.df.output.names" <- function()
{
  deblank <- function(x) gsub("[ ]*","", x)
  myDF <- data.frame(sex=c("Male","Male","Female", "Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1)

  # will return comparable results to xtabs
  z <- rxCrossTabs(~ sex : age, myDF)
  checkTrue(is.null(z$sums))
  checkTrue(is.null(z$means))
  checkTrue(deblank(names(z$counts)) == "sex:age")
 
  xform <- function(z)
  {
    z$logage <- factor(log(as.integer(as.character(z$age))))
  return(z)
  }
  z <- rxCrossTabs(~ sex : logage, data=myDF, transformFunc=xform, transformVars="age")

  checkTrue(is.null(z$sums))
  checkTrue(is.null(z$means))
  checkTrue(deblank(names(z$counts)) == "sex:logage")
}

###
# independent variable order tests
###

"test.rxCrossTabs.indvar.order.df" <- function()
{
    myDF <- data.frame(sex=c("Male","Male","Female", "Male"), 
        age=factor(c(20,20,12,15)), score=(1:4)+0.1)
    form1 <- score ~ sex : age
    form2 <- score ~ age : sex

    # will return comparable results to xtabs
    z <- rxCrossTabs(form1, myDF)
    zt <- rxCrossTabs(form2, myDF)

    checkTrue(all.equal(t(z$sums$score), zt$sums$score, check.attributes=FALSE))
    compareXtabs(form1, myDF)
    compareXtabs(form2, myDF)
}

###
# rxCrossTabs stress test functions
###

"test.rxCrossTabs.single.depvar.xdf.stress" <- function()
{
    checkTrue(rxCrossTabsStressTest(wkswork1 ~ sex : F(age), data=CensusPath, 
        pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCrossTabs.multiple.depvar.xdf.stress" <- function()
{
    checkTrue(rxCrossTabsStressTest(cbind(wkswork1, age) ~ sex : F(age), data=CensusPath, pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCrossTabs.single.depvar.with.factorization.xdf.stress" <- function()
{
    checkTrue(rxCrossTabsStressTest(wkswork1 ~ sex : F(age), data=CensusPath, pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCrossTabs.single.depvar.with.expression.xdf.stress" <- function()
{
    checkTrue(rxCrossTabsStressTest(log(wkswork1) ~ sex , transformVars="incwage", data=CensusPath, pweights ="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCrossTabs.single.depvar.with.expression.and.factorization.xdf.stress" <- function()
{
    checkTrue(rxCrossTabsStressTest(log(wkswork1) ~ F(age), transformVars="incwage", 
      data=CensusPath, pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCrossTabs.Rcallback.df.stress" <- function()
{
    data <- data.frame(sex=c("Male","Male","Female", "Male"), age=c(20,20,12,15), score=(1:4)+0.1)
      
    xform <- function(x)
    {
      x$revage <- rev(x$age)
    x$frevage <- as.factor(x$revage)
      x$cow <- as.factor(1:4)
      x$sport <- as.factor(c('tennis','golf','tennis','football'))

      return(x)								
    }

    # NOTE: for formula = cbind(score, log(score)) ~ sin(revage) + sex,
    # if transformVars is NULL, ExaStat processes all variables that it parses
    # from the formula: score, revage, sex. However, revage is defined within the 
    # user-defined transformFunc as 'x$revage <- rev(x$age)', which uses the primary
    # variable age in its definition. ExaStat doesn't have any direct knowledge of 'age'
    # and so it forces us to use transformVars to specify which variables we should use
    # from the original data source for the R transform functions. In doing so, we have
    # to specify all the primary variables that are needed for the transformation function,
    # and not just 'age', for example.
   
    # both user-defined and hidden transform 
    checkTrue(rxCrossTabsStressTestDf(cbind(score, log(score)) ~ frevage : sex, data=data, 
      transformFunc=xform, transformVars=c("age","score")))

    # user-defined transform only
    checkTrue(rxCrossTabsStressTestDf(revage ~ cow : sport, data=data, transformFunc=xform, transformVars=c("age")))

    # hidden transform only
    checkTrue(rxCrossTabsStressTestDf(log(score) ~ F(age) : sex, data=data))

    # no user-defined or hidden transform
    checkTrue(rxCrossTabsStressTestDf(score ~ F(age) : sex, data=data))
}

###
# N(var) in expressions
###

"test.rxCrossTabs.N.formula.expression.conversion.df" <- function()
{
    myDF <- data.frame(sex=c("Male","Male","Female", "Male"), 
        age=as.factor(c(20,20,12,15)), score=(1:4)+0.1)

    # Given that 
    #
    #   > as.numeric(as.factor(c(20,20,12,15))) - 1
    #   [1] 2 2 0 1
    #
    # the following result makes sense because that conversion is what ExaStat currently does
    checkTrue(all(as.vector(rxCrossTabs(N(age) ~ sex : F(score), myDF)$age) == c(0, 2, 0, 2, 0, 0, 0, 1)))

    # this should break because age is a categorical variable here
    checkException(rxCrossTabs(age ~ sex : F(score), myDF),
    msg="Exception caught in rxCrossTabs: age dependent variable cannot be categorical")
}

"test.rxCrossTabs.twoWay.xdf" <- function()
{
    myFile <- file.path(sampleDataDir, "AirlineDemoSmall")
    myTab <- rxCrossTabs(ArrDelay~DayOfWeek, data=myFile)
    out1 <- print(myTab)
    checkEqualsNumeric(out1$ArrDelay[1], 1146016)
    out2 <- summary(myTab, output="means")
    checkEqualsNumeric(out2$ArrDelay[1], 12.025604)
  
  # Repeat with uncompressed .xdf file
    myFileC <- file.path(sampleDataDir, "AirlineDemoSmallUC")
    myTabC <- rxCrossTabs(ArrDelay~DayOfWeek, data=myFileC)
    out1C <- print(myTabC)
  checkEquals(out1, out1C)	
}

###
# Summary method
###

"test.rxCrosstabs.method.summary" <- function()
{
    # create gladiator data.frame
    set.seed(100)
    divs <- letters[1:5]
    glads <- c("spartacus", "crixus")
    DF <- data.frame(division=rep(divs,5), score=runif(25,min=0,max=10), 
        gladiator=c(rep(glads[1],12),rep(glads[2],13)))
    
    # calculate cross-tabs and summarize based on the means
    z <- rxCrossTabs(score ~ division : gladiator, data=DF)
    # TODO: fix the following line of code
    zsum <- summary(z, output="means", type="%") 
 
    # calculate marginal means explicitly from data.frame
    rowmeans <- unlist(lapply(divs, function(x, DF) mean(DF$score[DF$division==x]), DF=DF))
    names(rowmeans) <- divs
    colmeans <- unlist(lapply(glads, function(x, DF) mean(DF$score[DF$gladiator==x]), DF=DF))
    names(colmeans) <- glads
    grandmean <- mean(DF$score)
    
    # compare explicitly calculated marginal means from data.frame to those of the summary method 
    checkEqualsNumeric(zsum$score["a","Row Mean"], rowmeans["a"])
    checkEqualsNumeric(zsum$score["b","Row Mean"], rowmeans["b"])
    checkEqualsNumeric(zsum$score["c","Row Mean"], rowmeans["c"])
    checkEqualsNumeric(zsum$score["d","Row Mean"], rowmeans["d"])
    checkEqualsNumeric(zsum$score["e","Row Mean"], rowmeans["e"])
    checkEqualsNumeric(zsum$score["Col Mean","spartacus"], colmeans["spartacus"])
    checkEqualsNumeric(zsum$score["Col Mean","crixus"], colmeans["crixus"])
    checkEqualsNumeric(zsum$score["Grand Mean",1], grandmean)
    
    # calculate data means explicitly from source data.frame
    matmeans <- matrix(NA, nrow=length(divs), ncol=length(glads))
    for (idiv in seq_along(divs))
    {
        for (iglad in seq_along(glads))
        {
            matmeans[idiv, iglad] <- 
                mean(DF$score[DF$division == divs[idiv] & DF$gladiator == glads[iglad]])
        }
    }
    dimnames(matmeans) <- list(divs, glads)
    
    # compare means
    checkTrue(all.equal(matmeans, zsum$score[divs,glads]))
    
    # check summary object data structure
    checkTrue(is.list(zsum))
    checkTrue(names(zsum) == "score")
    
    # simple two-way cross tab with summary
    z <- rxCrossTabs(ArrDelay~DayOfWeek, data=file.path(sampleDataDir, "AirlineDemoSmall"))
    res <- try(summary(z, output="means"), silent=TRUE)
    checkTrue(!is(res,"try-error"))
}

###
# Mean method
###

"test.rxCrosstabs.method.mean" <- function()
{
    # create gladiator data.frame
    set.seed(100)
    divs <- letters[1:5]
    glads <- c("spartacus", "crixus")
    DF <- data.frame(division=rep(divs,5), score=runif(25,min=0,max=10), 
        gladiator=c(rep(glads[1],12),rep(glads[2],13)))
    
    # calculate cross-tabs and summarize based on the means
    z <- rxCrossTabs(score ~ division : gladiator, data=DF)
    zmean <- mean(z, marginals=TRUE)
    marginals <- attr(zmean$score, "marginals")
 
    # calculate marginal means explicitly from data.frame
    rowmeans <- unlist(lapply(divs, function(x, DF) mean(DF$score[DF$division==x]), DF=DF))
    names(rowmeans) <- divs
    colmeans <- unlist(lapply(glads, function(x, DF) mean(DF$score[DF$gladiator==x]), DF=DF))
    names(colmeans) <- glads
    grandmean <- mean(DF$score)
    
    # compare explicitly calculated marginal means from data.frame to those of the mean method 
    checkEqualsNumeric(marginals$row["a"], rowmeans["a"])
    checkEqualsNumeric(marginals$row["b"], rowmeans["b"])
    checkEqualsNumeric(marginals$row["c"], rowmeans["c"])
    checkEqualsNumeric(marginals$row["d"], rowmeans["d"])
    checkEqualsNumeric(marginals$row["e"], rowmeans["e"])
    checkEqualsNumeric(marginals$column["spartacus"], colmeans["spartacus"])
    checkEqualsNumeric(marginals$column["crixus"], colmeans["crixus"])
    checkEqualsNumeric(marginals$grand, grandmean)
    
    # calculate data means explicitly from source data.frame
    matmeans <- matrix(NA, nrow=length(divs), ncol=length(glads))
    for (idiv in seq_along(divs))
    {
        for (iglad in seq_along(glads))
        {
            matmeans[idiv, iglad] <- 
                mean(DF$score[DF$division == divs[idiv] & DF$gladiator == glads[iglad]])
        }
    }
    dimnames(matmeans) <- list(divs, glads)
    
    # compare means
    checkTrue(all.equal(matmeans[,glads], zmean$score[,glads], check.attributes = FALSE))
    
    # check summary object data structure
    checkTrue(is.list(zmean))
    checkTrue(names(zmean) == "score")
    
    ###
    # mean existence checks
    ###
    
    # means exist, no marginals
    form <- score ~ division : gladiator
    z <- rxCrossTabs(form, data=DF, means=TRUE, marginals=FALSE)
    checkTrue(!is.null(z$means))
    checkTrue(!is.null(z$means$score))
    checkTrue(is.null(attr(z$means$score, "marginals")))
    
    # means and marginals exist
    z <- rxCrossTabs(form, data=DF, means=TRUE, marginals=TRUE)
    checkTrue(!is.null(z$means))
    checkTrue(!is.null(z$means$score))
    checkTrue(!is.null(attr(z$means$score, "marginals")))
    
    # no means, no forced marginals
    z <- rxCrossTabs(form, data=DF)
    checkTrue(is.null(z$means))
    zmean <- mean(z, marginals=FALSE)
    checkTrue(!is.null(zmean$score))
    checkTrue(is.null(attr(zmean$score, "marginals")))
    
    # no means, forced marginals
    zmean <- mean(z, marginals=TRUE)
    checkTrue(!is.null(zmean$score))
    checkTrue(!is.null(attr(zmean$score, "marginals")))
    
    # simple two-way cross tab with mean
    z <- rxCrossTabs(ArrDelay~DayOfWeek, data=file.path(sampleDataDir, "AirlineDemoSmall"))
    zmean <- mean(z, marginals=TRUE)
    checkTrue(!is.null(zmean$ArrDelay))
    checkTrue(!is.null(attr(zmean$ArrDelay, "marginals")))
    zmean <- mean(z, marginals=FALSE)
    checkTrue(!is.null(zmean$ArrDelay))
    checkTrue(is.null(attr(zmean$ArrDelay, "marginals")))
  
  # Compare results with uncompressed file
  zC <- rxCrossTabs(ArrDelay~DayOfWeek, data=file.path(sampleDataDir, "AirlineDemoSmallUC"))
  z$params$File <- NULL
  zC$params$File <- NULL
  z$params$DataSource <- NULL
  zC$params$DataSource <- NULL    
  z$call <- NULL
  zC$call <- NULL
  checkEquals(z, zC)
}

"test.rxCrosstabs.marginals" <- function()
{
    UCBADF <- as.data.frame(UCBAdmissions)
    z <- rxCrossTabs(Freq ~ Gender : Admit, data=UCBADF, marginals=TRUE)
    checkTrue(!is.null(attr(z$sums$Freq,"marginals")))
    checkTrue(!is.null(attr(z$counts$Freq,"marginals")))
    z <- rxCrossTabs(Freq ~ Gender : Admit, data=UCBADF, marginals=FALSE)
    checkTrue(is.null(attr(z$sums$Freq,"marginals")))
    checkTrue(is.null(attr(z$counts$Freq,"marginals")))
    
    set.seed(100)
    divs <- letters[1:5]
    glads <- c("spartacus", "crixus")
    DF <- data.frame(division=rep(divs,5), score=runif(25,min=0,max=10), 
        gladiator=c(rep(glads[1],12),rep(glads[2],13)))
    z <- rxCrossTabs(score ~ division : gladiator, data=DF, marginals=TRUE, means=TRUE)

    checkTrue(!is.null(attr(z$sums$score,"marginals")))
    checkTrue(!is.null(attr(z$counts$score,"marginals")))
    checkTrue(!is.null(attr(z$means$score,"marginals")))

    checkTrue(all.equal(rowSums(z$sums$score), attr(z$sums$score,"marginals")$row))
    checkTrue(all.equal(rowSums(z$counts$score), attr(z$counts$score,"marginals")$row))
        
    checkTrue(all.equal(colSums(z$sums$score), attr(z$sums$score,"marginals")$column))
    checkTrue(all.equal(colSums(z$counts$score), attr(z$counts$score,"marginals")$column))
        
    checkTrue(all.equal(sum(z$sums$score), attr(z$sums$score,"marginals")$grand))
    checkTrue(all.equal(sum(z$counts$score), attr(z$counts$score,"marginals")$grand))
}

###
# argument checks
###

"test.rxCrossTabs.args.useSparseCube.mixed" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- cost ~ age:car.age:type:F(number)    #mixed continuous and discrete case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxCrossTabs.args.useSparseCube.continuous" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- cost ~ F(number)    #pure continuous case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxCrossTabs.args.useSparseCube.discrete" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- cost ~ age:car.age:type    #pure discrete case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxCrossTabs( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxCrossTabs.args.blocksPerRead" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  form <- ~ F(y) : F(xnum2miss)
  zCrossTabsBlocksDefault <- rxCrossTabs(form, data=test100r5b, blocksPerRead=-1)
  zCrossTabsBlocks1 <- rxCrossTabs(form, data=test100r5b, blocksPerRead=1)
  nocomp <- which(names(zCrossTabsBlocks1) %in% c("call","params"))
  checkTrue(all.equal(zCrossTabsBlocksDefault[-nocomp], zCrossTabsBlocks1[-nocomp], check.attributes=FALSE))
  
  zCrossTabsBlocks5 <- rxCrossTabs(form, data=test100r5b, blocksPerRead=5)
  checkTrue(all.equal(zCrossTabsBlocksDefault[-nocomp], zCrossTabsBlocks5[-nocomp], check.attributes=FALSE))
}

# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxCrossTabs.args.cppInterp" <- function()
#{
  #DF <- infert
  #DF1 <- subset(infert, parity > 3)
  #form <- stratum ~ F(age) : F(parity) : F(education) : F(spontaneous) : F(induced)
#
  ## The line below should be changed to have a single & when the R interpreter is used
  #zA <- rxCrossTabs(form, cppInterp=list(rowSelection="parity > 3"), data=DF)	
  #zB <- rxCrossTabs(form, data=DF1)
  #checkTrue(all.equal(coef(zA), coef(zB)))
#}

"test.rxCrossTabs.args.cube" <- function()
{
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  zCrossTabsFalse <- rxCrossTabs(form, data=DF, cube=FALSE)
  zCrossTabsTrue <- rxCrossTabs(form, data=DF, cube=TRUE)
  nocomp <- which(names(zCrossTabsTrue) %in% c("call","params"))
  checkTrue(all.equal(zCrossTabsFalse[-nocomp], zCrossTabsTrue[-nocomp], check.attributes=FALSE))
}

"test.rxCrossTabs.args.data" <- function()
{
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  XDF <- tempfile( pattern = ".rxCT", fileext = ".xdf")
  rxDataStep(DF, XDF)
  
  zCrossTabsDF <- rxCrossTabs(form, data=DF)
  zCrossTabsXDF <- rxCrossTabs(form, data=XDF)
  nocomp <- which(names(zCrossTabsXDF) %in% c("call","params"))
  checkTrue(all.equal(zCrossTabsDF[-nocomp], zCrossTabsXDF[-nocomp], check.attributes=FALSE))
    file.remove(XDF)
}

"test.rxCrossTabs.args.formula" <- function()
{
  form <- ~ division : state
  formStr <- deparse(form)
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  zCrossTabsFormula <- rxCrossTabs(form, data=DF)
  zCrossTabsString <- rxCrossTabs(formStr, data=DF)
  comparisons <- c("sums","counts","means")
  checkTrue(all.equal(zCrossTabsFormula[comparisons], zCrossTabsString[comparisons], check.attributes=FALSE))
}

"test.rxCrossTabs.args.means" <- function()
{
  form <- score ~ division : state
  set.seed(10, kind="Mersenne-Twister")
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  zCrossTabsMeansFalse <- rxCrossTabs(form, data=DF, means=FALSE)
  zCrossTabsMeansTrue <- rxCrossTabs(form, data=DF, means=TRUE)
  zCrossTabsMeansTrue$means[[1]]
  
  checkTrue(is.null(zCrossTabsMeansFalse$means))
  checkEqualsNumeric(zCrossTabsMeansTrue$means[[1]]["f","North Carolina"], 50.7, tol=0.1)
}

"test.rxCrossTabs.args.pweights.fweights" <- function()
{
    fwts <- 1:4
    sex <- c("Male","Male","Female", "Male")
    age <- c(20,20,12,15)
    score <- (1:4)+0.1
    
    myDF1 <- data.frame(sex=sex, age=age, score=factor(score), pWeights=fwts)    
    myDF2 <- data.frame(sex=factor(rep(sex,fwts)), age=rep(age,fwts), score=factor(rep(score,fwts)))
  form <- age ~ sex : score
    
    z1 <- rxCrossTabs(form, data=myDF1, pweights="pWeights")
    z2 <- rxCrossTabs(form, data=myDF2)
  fw <- "pWeights"
  z3 <- rxCrossTabs(form, data=myDF1, fweights = fw)
    
  nocomp <- which(names(z1) %in% c("call","params","validObs"))
    checkTrue(all.equal(z1[-nocomp],z2[-nocomp], check.attributes=FALSE))
  checkTrue(all.equal(z3[-nocomp],z2[-nocomp], check.attributes=FALSE))
    checkTrue(all(as.vector(z1$score) == score))
    checkTrue(all(as.vector(z1$sex) == sex))
    checkTrue(all(as.vector(z1$age) == age))
    checkTrue(all(as.vector(z1$Counts) == fwts))
}

"test.rxCrossTabs.args.reportProgress" <- function()
{
  # reportProgress argument stress tests
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  #reportProgress:  integer: 
  #  0: no progress is reported. 
  #  1: the number of processed rows is printed and updated. 
  #  2: rows processed and timings are reported. 
  #  3: rows processed and all timings are reported. 


  variations <- list(0,1,2,3)	
  results <- printOutput <- vector("list", length=length(variations))
  
  for (i in seq_along(variations))
  {
    printOutput[[i]] <- capture.output(results[[i]] <- 
            rxCrossTabs(form=form, data=DF, reportProgress=variations[[i]]))
  }

  # check that all values are the same
  nocomp <- which(names(results[[1]]) %in% c("call","params"))
  checkTrue(all(unlist(lapply(results[-1], 
    function(x,y,nocomp){all.equal(x[-nocomp], y[-nocomp], check.attributes=FALSE)}, 
    y=results[[1]], nocomp=nocomp))))

  names(printOutput) <- as.character(variations)
  
  noResponseExpected <- c("0")
  equalLengthExpected <- c("2")
  equalLengthExpected1 <- c("1")
  
  checkTrue(length(unlist(printOutput[noResponseExpected])) == 0)
  
  # all these outputs should be the same
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected],length)) == 2))
  
  # checking rows should just print one line
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1],length)) == 1))
  
  # details gives us the goods
  checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=5, TotalRowsProcessed=5, ReadTime", 
    printOutput[["3"]][1])) > 0)
  
  # time
  checkTrue(length(grep("^Computation time:", printOutput[["2"]][2])) > 0)
}

"test.rxCrossTabs.args.removeZeroCounts" <- function()
{
  # define local data source
  
  myDF <- data.frame(sex = c("Male", "Male", "Female", "Male"), age = c(20, 20, 12, 15), score = 1.1:4.1) 

  res1 <- rxCrossTabs(log(score) ~ F(age) : sex, data = myDF, removeZeroCounts=TRUE)
  res2 <- rxCrossTabs(log(score) ~ F(age) : sex, data = myDF, removeZeroCounts=FALSE)
  checkEquals(length(res1$counts[[1]]), 6)
  checkEquals(length(res2$counts[[1]]), 18)
    
  # With means set to TRUE
  res3 <- rxCrossTabs(log(score) ~ F(age) : sex, data = myDF, means=TRUE, removeZeroCounts=TRUE)
  res4 <- rxCrossTabs(log(score) ~ F(age) : sex, data = myDF, means=TRUE, removeZeroCounts=FALSE)
  checkEquals(length(res3$sums[[1]]), 6)
  checkEquals(length(res4$sums[[1]]), 18)
  
  # No dependent variable
  res5 <- rxCrossTabs( ~ F(age) : sex : F(score), data = myDF, removeZeroCounts=TRUE)
  res6 <- rxCrossTabs( ~ F(age) : sex : F(score), data = myDF, removeZeroCounts=FALSE)
  checkEquals(length(res5$counts[[1]]), 2)
  checkEquals(length(res6$counts[[1]]), 18)
  
}

"test.rxCrossTabs.args.rowSelection" <- function()
{
  DF <- as.data.frame(UCBAdmissions)
  DF1 <- subset(DF, subset = Gender=='Male' & Freq > 200)
    form <- Freq ~ Gender : Admit
  zA <- rxCrossTabs(form, rowSelection=Gender=='Male' & Freq > 200, data= DF)
  zB <- rxCrossTabs(form, data= DF1)
  checkTrue(all.equal(zA$sums, zB$sums))
}

"test.rxCrossTabs.args.transformFunc" <- function()
{
  # reportProgress argument stress tests
  set.seed(10)
  DF <- data.frame(score=(1:5)*0.1,
    division=c("a","a","z","t","l"), 
    state=sample(state.name, 5, replace=TRUE))
  xform <- function(dataList)
  {
    set.seed(10)
    dataList$STATE <- factor(sample(datasets::state.name, 5, replace=TRUE))
    dataList	
  }

  z1 <- rxCrossTabs(score ~ division : STATE, data=DF, transformFunc=xform)
  z2 <- rxCrossTabs(score ~ division : state, data=DF)
  nocomp <- which(names(z1) %in% c("call","params","formula"))
  checkTrue(all.equal(z1[-nocomp],z2[-nocomp],check.attributes=FALSE))
}

"test.rxCrossTabs.args.transformVars" <- function()
{
  # reportProgress argument stress tests
  set.seed(10)
  DF <- data.frame(score=(1:5)*0.1,
    division=c("a","a","z","t","l"), 
    state=factor(sample(state.name, 5, replace=TRUE)))
  xform <- function(dataList)
  {
    dataList$stateCopy <- dataList$state
    dataList	
  }

  z1 <- rxCrossTabs(score ~ division : stateCopy, data=DF, transformFunc=xform, transformVars="state")
  z2 <- rxCrossTabs(score ~ division : state, data=DF)
  nocomp <- which(names(z1) %in% c("call","params","formula"))
  checkTrue(all.equal(z1[-nocomp],z2[-nocomp],check.attributes=FALSE))
}

"test.rxCrossTabs.args.verbose" <- function()
{
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  
  zCrossTabsVerboseFalse <- capture.output(z <- rxCrossTabs(form, data=DF, verbose=0))
  zCrossTabsVerboseTrue <- capture.output(z <- rxCrossTabs(form, data=DF, verbose=1))
  checkTrue(length(zCrossTabsVerboseTrue) > length(zCrossTabsVerboseFalse))
}

"test.rxCrossTabs.args.verbose" <- function()
{
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  
  zCrossTabsVerboseFalse <- capture.output(z <- rxCrossTabs(form, data=DF, verbose=0))
  zCrossTabsVerboseTrue <- capture.output(z <- rxCrossTabs(form, data=DF, verbose=1))
  checkTrue(length(zCrossTabsVerboseTrue) > length(zCrossTabsVerboseFalse))
}

"test.rxCrossTabs.methods.rxResultsDF" <- function()
{
  
  crossOut <- rxCrossTabs(~Species:F(Sepal.Length), data=iris) 
  countsDF <- rxResultsDF(crossOut)
  varInfo <- rxGetVarInfo(countsDF)
  checkEquals(names(varInfo), c("Species", "Sepal.Length.4",
    "Sepal.Length.5", "Sepal.Length.6", "Sepal.Length.7"))
  checkEquals(varInfo$Species$varType, "factor")
  checkEquals(varInfo$Sepal.Length.4$varType, "integer")
  rm(varInfo)
  rm(countsDF)	
  
  crossOut <- rxCrossTabs(~F(Sepal.Length):Species, data=iris) 
  countsDF <- rxResultsDF(crossOut, integerLevels=TRUE)
  varInfo <- rxGetVarInfo(countsDF)
  checkEquals(names(varInfo), c("Sepal.Length", "Species.setosa",
    "Species.versicolor", "Species.virginica"))
  checkEquals(varInfo$Sepal.Length$varType, "integer")
  checkEquals(varInfo$Species.setosa$varType, "integer")
  rm(varInfo)
  rm(countsDF)
  
  crossOut <- rxCrossTabs(~F(Sepal.Length):Species, data=iris) 
  countsDF <- rxResultsDF(crossOut, integerLevels = FALSE)
  varInfo <- rxGetVarInfo(countsDF)
  checkEquals(names(varInfo), c("Sepal.Length", "Species.setosa",
    "Species.versicolor", "Species.virginica"))
  checkEquals(varInfo$Sepal.Length$varType, "factor")
  checkEquals(varInfo$Sepal.Length$levels, c("4", "5", "6", "7"))
  checkEquals(varInfo$Species.setosa$varType, "integer")
  rm(varInfo)
  rm(countsDF)
  
  admissions <- as.data.frame(UCBAdmissions)
  crossOut2 <- rxCrossTabs(~Gender, data=admissions)
  countsDF <- rxResultsDF(crossOut2)
  varInfo <- rxGetVarInfo(countsDF)
  checkEquals( names(varInfo), c("Gender", "Counts"))
  checkEquals( varInfo$Gender$varType, "factor")
  checkEquals( varInfo$Counts$varType, "integer")
  rm(varInfo)
  rm(countsDF)
  
  crossOut <- rxCrossTabs(Freq~Admit, data=admissions)
  sumsDF <- rxResultsDF(crossOut, output="sums")
  varInfo <- rxGetVarInfo(sumsDF)
  checkEquals( names(varInfo), c("Admit", "Freq"))
  rm(varInfo)
  rm(sumsDF)
  
  crossOut <- rxCrossTabs(Freq~Admit:Gender, data=admissions)
  sumsDF <- rxResultsDF(crossOut, output="sums")
  varInfo <- rxGetVarInfo(sumsDF)
  checkEquals( names(varInfo), c("Admit", "Gender.Male", "Gender.Female"))
  countsDF <- rxResultsDF(crossOut, output="counts")
  varInfo <- rxGetVarInfo(countsDF)
  checkEquals( names(varInfo), c("Admit", "Gender.Male", "Gender.Female"))
  
  meansDF <- rxResultsDF(crossOut, output="means")
  varInfo <- rxGetVarInfo(meansDF)
  checkEquals( names(varInfo), c("Admit", "Gender.Male", "Gender.Female"))
    
  cubeOut1 <- as.data.frame(rxCube(Freq~Admit:Gender, data = admissions))
  checkEquals( sumsDF[1,2], round(cubeOut1[1,3]*cubeOut1[1,4]))
  checkEquals( sumsDF[2,2], round(cubeOut1[2,3]*cubeOut1[2,4]))
  checkEquals( sumsDF[1,3], round(cubeOut1[3,3]*cubeOut1[3,4]))
  checkEquals( sumsDF[2,3], round(cubeOut1[4,3]*cubeOut1[4,4]))
  
  checkEquals( meansDF[1,2], cubeOut1[1,3] )
  checkEquals( meansDF[2,2], cubeOut1[2,3] )
  checkEquals( meansDF[1,3], cubeOut1[3,3] )
  checkEquals( meansDF[2,3], cubeOut1[4,3] )
  
  # Check factor levels with odd labels
  crossOut1 <- rxCrossTabs(ncases ~ agegp : alcgp, data=esoph)
  sumsDF <- rxResultsDF(crossOut1, output="sums")
  varInfo <- rxGetVarInfo(sumsDF)
  checkEquals(names(varInfo), c("agegp", "alcgp.0.39g.day", "alcgp.40.79","alcgp.80.119" ,   
    "alcgp.120." ))
  
  # Check factor levels with spaces
    sex <- c("This is a Male","This is a Male","This is a Female", "This is a Male")
    age <- c(20,20,12,15)
    myDF1 <- data.frame(sex=sex, age=age)    
    
    countsDF <- rxResultsDF(rxCrossTabs(~F(age):sex, data=myDF1))
  checkEquals(names(countsDF), c("age", "sex.This.is.a.Female", "sex.This.is.a.Male"))
}



########	rxCrossTabs Analysis on SAS datasource 	 OK 9/19
"test.rxCrossTabs.dataSource.SAS" <- function()
{		
  dsSAS <- RxSasData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.sas7bdat"),
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))
            )
    result <- rxCrossTabs(creditScore~default, data = dsSAS)
        
    testFile <- tempfile(pattern = ".rxCross1TempTest", fileext = ".xdf")                
  rxImport(dsSAS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxCrossTabs(creditScore~default, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}
  
########	rxCrossTabs Analysis on Text datasource 	 
"test.rxCrossTabs.dataSource.Text" <- function()
{			
  # useFastRead = TRUE
  dsText <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.csv"),
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes"))),
             useFastRead = TRUE
            )
          
    result <- rxCrossTabs(creditScore~default, data=dsText)
    testFile <- tempfile(pattern = ".rxCross2TempTest", fileext = ".xdf")	
  rxImport(dsText, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxCrossTabs(creditScore~default, testFile)	
  
    checkEquals(result$sums, result2$sums)
  checkEquals(result$counts, result2$counts)
    file.remove(testFile)
  
  # Use FastRead = FALSE
  dsTextFast <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.csv"),
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes"))),
             firstRowIsColNames = TRUE,
             useFastRead = FALSE
            )
          
    result3 <- rxCrossTabs(creditScore~default, data=dsTextFast)
  checkEquals(result$sums, result3$sums)
  checkEquals(result$counts, result3$counts)
  
  # Use FastRead = FALSE
  # Fixed format
  fixedColInfo <- list(
    creditScore = list( start = 1, width = 3, type = "integer"),
    houseAge = list( start = 5, width = 2, type = "integer"),
    yearsEmploy = list( start = 8, width = 1, type = "integer"),
    ccDebt = list(start = 10, width = 4, type = "integer"),
    year = list(start = 15, width = 4, type = "integer"),
    default = list( start = 20, width = 3, type = "factor", levels = c("No", "Yes")))
    
  dsFixed <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.dat"),
             colInfo = fixedColInfo,
            firstRowIsColNames = FALSE,
             useFastRead = FALSE
            )
          
    result4 <- rxCrossTabs(creditScore~default, data=dsFixed)
  checkEquals(result$sums, result4$sums)
  checkEquals(result$counts, result4$counts)	
  
  # Use FastRead = TRUE
  # Fixed format
    
  dsFixedFast <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.dat"),
             colInfo = fixedColInfo,
             useFastRead = TRUE
            )
          
    result5 <- rxCrossTabs(creditScore~default, data=dsFixedFast)
  checkEquals(result$sums, result5$sums)
  checkEquals(result$counts, result5$counts)		

}

"test.rxCrossTabs.dataSource.Text.interactions" <- function()
{			
    x <- rep(c("a", "b", "c", "d"), times=5)
    y <- c(rep(c("One", "Two", "Three", "Four"), each = 4), c("One", "One", "One", "Two"))
    z <- 1:20
    myData <- data.frame(x,y,z)
    resultDF <- rxCrossTabs(~x:y, data = myData) 
   
    # Write data to a temp file 
    textFile <- tempfile(pattern = ".rxCross3TempTest", fileext = ".csv")	
    write.csv(myData, file = textFile, row.names = FALSE)
    
    # Create a text data source with multiple blocks
    dsText <- RxTextData(textFile, stringsAsFactors = TRUE,
    rowsPerRead = 4)
     
    # Redo computations using text data source
    resultDS <- rxCrossTabs(~x:y, data = dsText)  
    
    checkEquals(resultDF$counts[[1]]["a","One"], resultDS$counts[[1]]["a","One"])    	
    checkEquals(resultDF$counts[[1]]["b","One"], resultDS$counts[[1]]["b","One"])  
  checkEquals(resultDF$counts[[1]]["c","One"], resultDS$counts[[1]]["c","One"])  
  checkEquals(resultDF$counts[[1]]["d","One"], resultDS$counts[[1]]["d","One"])  
  checkEquals(resultDF$counts[[1]]["a","Two"], resultDS$counts[[1]]["a","Two"])    	
    checkEquals(resultDF$counts[[1]]["b","Two"], resultDS$counts[[1]]["b","Two"])  
  checkEquals(resultDF$counts[[1]]["c","Two"], resultDS$counts[[1]]["c","Two"])  
  checkEquals(resultDF$counts[[1]]["d","Two"], resultDS$counts[[1]]["d","Two"])  						
    file.remove(textFile)
}

######## rxCrossTabs Analysis on ODBC datasource	
"test.rxCrossTabs.dataSource.ODBC.ST" <- function()
{			
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.sqlite"),
                        useFastRead = FALSE),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  dsODBC <- RxOdbcData(
        sqlQuery = "SELECT * FROM mortDef",
                connectionString = conString,
                useFastRead = FALSE,
         colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                  "ccDebt" = "integer", "year" = "integer",  "isDefault" = "factor"),
         colInfo = list(  isDefault = list(type = "factor", levels = c("No","Yes")))
        )					
    result <- rxCrossTabs(creditScore~isDefault, data=dsODBC)
  
    testFile <- tempfile(pattern = ".rxCross4TempTest", fileext = ".xdf")    
  rxImport(dsODBC, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxCrossTabs(creditScore~isDefault, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}

######## rxCrossTabs Analysis on Direct ODBC datasource	
"test.rxCrossTabs.dataSource.fastODBC" <- function()
{		
  # Fix for bug RA-4631

  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()	
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.sqlite")),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  dsODBC <- RxOdbcData(
        sqlQuery = "SELECT * FROM mortDef",
                connectionString = conString,
        useFastRead = TRUE,						
        colInfo = list(  isDefault = list(type = "factor", levels = c("No","Yes")))
        )	
                  
    result <- rxCrossTabs(creditScore~isDefault, data=dsODBC)
  
    testFile <- tempfile(pattern = ".rxCross5TempTest", fileext = ".xdf")    
  rxImport(dsODBC, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxCrossTabs(creditScore~isDefault, testFile)	
  
    checkEquals(result$counts, result2$counts)
    file.remove(testFile)
}

# rxCrossTabs comparision to xtabs using the returnXtabs flag
"test.rxCrossTabs.arg.returnXtabs" <- function()
{
    # define local function to compare xtabs to rxCrossTabs(..., returnXtabs = TRUE) output
    "rxCrossTabsEqualsXtabsSansCall" <- function(xtabsObject, rxCrossTabsObject)
    {
        attr(xtabsObject, "call") <- attr(rxCrossTabsObject, "call") <- NULL
        checkEquals(xtabsObject, rxCrossTabsObject)
    }
    
    # compare xtabs and rxCrossTabs(..., returnXtabs = TRUE) results for 3-way interaction, one dependent variable
    set.seed(100)
    divs <- letters[1:5]
    glads <- c("spartacus", "crixus")
    romeDF <- data.frame( division = rep(divs, 5L), 
                          score = runif(25, min = 0, max = 10), 
                          rank = runif(25, min = 1, max = 100), 
                          gladiator = c(rep(glads[1L], 12L), rep(glads[2L], 13L)),
                          arena = sample(c("colosseum", "ludus", "market"), 25L, replace = TRUE))

    z1 <- rxCrossTabs(score ~ division : gladiator : arena, data = romeDF, returnXtabs = TRUE)
    z2 <- xtabs(score ~ division + gladiator + arena, romeDF)
    rxCrossTabsEqualsXtabsSansCall(z1, z2)

    # compare xtabs and rxCrossTabs(..., returnXtabs = TRUE) results for 3-way interaction, multiple dependent variable
    z1 <- rxCrossTabs(cbind(score, rank) ~ division : gladiator : arena, data = romeDF, returnXtabs = TRUE, means = TRUE)
    z2 <- xtabs(cbind(score, rank) ~ division + gladiator + arena, romeDF)
    rxCrossTabsEqualsXtabsSansCall(z1, z2)
    
    # compare xtabs and rxCrossTabs(..., returnXtabs = TRUE) results for 3-way interaction, no dependent variable
    z1 <- rxCrossTabs( ~ division : gladiator : arena, data = romeDF, returnXtabs = TRUE, means = TRUE)
    z2 <- xtabs(~ division + gladiator + arena, romeDF)
    rxCrossTabsEqualsXtabsSansCall(z1, z2)

    # compare xtabs and rxCrossTabs(..., returnXtabs = TRUE) results for 2-way interaction, no dependent variable
    DF <- data.frame(sex=c("Male","Male","Female","Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1)
    z1 <- rxCrossTabs(~ sex : age, DF, returnXtabs = TRUE, means = TRUE)
    z2 <- xtabs(~ sex + age, DF)
    rxCrossTabsEqualsXtabsSansCall(z1, z2)
    
    # compare xtabs and rxCrossTabs(..., returnXtabs = TRUE) results for a single factor variable, no dependent variable
    DF <- data.frame(sex=c("Male","Male","Female","Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1)
    z1 <- rxCrossTabs(~ sex, DF, returnXtabs = TRUE, means = TRUE)
    z2 <- xtabs(~ sex, DF)
    rxCrossTabsEqualsXtabsSansCall(z1, z2)
}

"test.rxCrossTabs.method.as.xtabs" <- function()
{
    # define local function to compare xtabs to as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) output
    "rxCrossTabsEqualsXtabsSansCall" <- function(xtabsObject, rxCrossTabsObject)
    {
        attr(xtabsObject, "call") <- attr(rxCrossTabsObject, "call") <- NULL
        checkEquals(xtabsObject, rxCrossTabsObject)
    }
    
    # compare xtabs and as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) results for 3-way interaction, one dependent variable
    set.seed(100)
    divs <- letters[1:5]
    glads <- c("spartacus", "crixus")
    romeDF <- data.frame( division = rep(divs, 5L), 
                          score = runif(25, min = 0, max = 10), 
                          rank = runif(25, min = 1, max = 100), 
                          gladiator = c(rep(glads[1L], 12L), rep(glads[2L], 13L)),
                          arena = sample(c("colosseum", "ludus", "market"), 25L, replace = TRUE))

    z1 <- rxCrossTabs(score ~ division : gladiator : arena, data = romeDF, returnXtabs = FALSE)
    z2 <- xtabs(score ~ division + gladiator + arena, romeDF)
    rxCrossTabsEqualsXtabsSansCall(as.xtabs(z1), z2)

    # compare xtabs and as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) results for 3-way interaction, multiple dependent variable
    z1 <- rxCrossTabs(cbind(score, rank) ~ division : gladiator : arena, data = romeDF, returnXtabs = FALSE, means = TRUE)
    z2 <- xtabs(cbind(score, rank) ~ division + gladiator + arena, romeDF)
    rxCrossTabsEqualsXtabsSansCall(as.xtabs(z1), z2)
    
    # compare xtabs and as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) results for 3-way interaction, no dependent variable
    z1 <- rxCrossTabs( ~ division : gladiator : arena, data = romeDF, returnXtabs = FALSE, means = TRUE)
    z2 <- xtabs(~ division + gladiator + arena, romeDF)
    rxCrossTabsEqualsXtabsSansCall(as.xtabs(z1), z2)

    # compare xtabs and as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) results for 2-way interaction, no dependent variable
    DF <- data.frame(sex=c("Male","Male","Female","Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1)
    z1 <- rxCrossTabs(~ sex : age, DF, returnXtabs = FALSE, means = TRUE)
    z2 <- xtabs(~ sex + age, DF)
    rxCrossTabsEqualsXtabsSansCall(as.xtabs(z1), z2)
 
    # compare xtabs and as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) results for 2-way interaction, multiple dependent variablea
    DF <- data.frame(sex=c("Male","Male","Female","Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1, rank = rnorm(4), size = runif(4))
    z1 <- rxCrossTabs(cbind(score, rank, size) ~ sex : age, DF, returnXtabs = FALSE, means = TRUE)
    z2 <- xtabs(cbind(score, rank, size) ~ sex + age, DF)
    rxCrossTabsEqualsXtabsSansCall(as.xtabs(z1), z2)
   
    # compare xtabs and as.xtabs(rxCrossTabs(..., returnXtabs = FALSE)) results for a single factor variable, no dependent variable
    DF <- data.frame(sex=c("Male","Male","Female","Male"))
    z1 <- rxCrossTabs(~ sex, DF, returnXtabs = FALSE, means = TRUE)
    z2 <- xtabs(~ sex, DF)
    rxCrossTabsEqualsXtabsSansCall(as.xtabs(z1), z2)
}
