# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxCube
#
############################################################################################

###
# required libraries and paths
###

sampleDataDir <- rxGetOption("sampleDataDir")
CensusPath <- file.path(sampleDataDir, "CensusWorkers.xdf")

###
# local function definition
###

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

"rxCubeStressTest" <- function(formula, transformFunc=NULL, transformVars=NULL, data=NULL,pweights=NULL, blocksPerRead=-1,...)
{		
    # run the crosstab
  res <- try(do.call("rxCube", list(formula=formula,data=data, pweights=pweights, 
        transformFunc=transformFunc, transformVars=transformVars, blocksPerRead=blocksPerRead)))
    
    # return TRUE for no errors encountered during execution
  return (!is(res, "try-error"))
}

"rxCubeStressTestDf" <- function(formula, data, pweights=NULL, ...)
{
    # run the crosstab
  res <- try(do.call("rxCube", list(formula=formula,data=data, pweights=pweights, ...)))
    
    # return TRUE for no errors encountered during execution
  return (!is(res, "try-error"))
}

###
# Output comparison of rxCube, rxCrossTabs, xtabs, etc.
###

"test.rxCube.rxCrossTabs.xtabs" <- function()
{
    # Basic data.frame source example
    DF <- as.data.frame(UCBAdmissions)
    form <- Freq ~ Gender : Admit

    # calculate rxCrossTabs and convert to data.frame
    zcross <- rxCrossTabs(form, DF)$sums$Freq
    class(zcross) <- "table"
    zcross <- as.data.frame(zcross)

    # calculate rxCube data.frame and convert mean column to sums
    zcube <- rxCube(form, DF, returnDataFrame=TRUE)
    zcube$Freq <- zcube$Freq * zcube$Counts
    zcube$Counts <- NULL

    # calculate xtabs data.frame
    zxtabs <- as.data.frame(xtabs(Freq ~ Gender + Admit, DF))

    # compare to each other (2 should be sufficient if TRUE but let's be complete)
    checkTrue(all.equal(zcross, zcube, check.attributes=FALSE))
    checkTrue(all.equal(zcross, zxtabs, check.attributes=FALSE))
    checkTrue(all.equal(zcube, zxtabs, check.attributes=FALSE))   
    checkTrue(all.equal(dimnames(zcross), dimnames(zcube)))
    checkTrue(all.equal(dimnames(zcross), dimnames(zxtabs)))
    checkTrue(all.equal(dimnames(zcube), dimnames(zxtabs))) 
  
    # verify column names (only need to verify one if above are true)
    colnms <- sort(c("Gender","Admit","Freq"))
    checkTrue(all(sort(names(zcube)) == colnms))
    
    # check dimensions (only need to verify one if above are true)
    checkTrue(all(dim(zcube) == c(4,3)))
}

"test.rxCube.output.class" <- function()
{
    # Basic data.frame source example
    DF <- as.data.frame(UCBAdmissions)
    form <- Freq ~ Gender : Admit

    # calculate rxCube data.frame
    zdf <- rxCube(form, DF, returnDataFrame=TRUE)
    zlt <- rxCube(form, DF, returnDataFrame=FALSE)
    
    checkTrue(is.data.frame(zdf))
    checkTrue(!is.data.frame(zlt) && is.list(zlt))
}

"test.rxCube.method.df" <- function()
{
  # Basic data.frame source example
  DF <- as.data.frame(UCBAdmissions)
  form <- Freq ~ Gender : Admit

  # calculate rxCube data.frame
  z <- rxCube(form, DF)

  # verify as.data.frame.rxCube output
  checkTrue(is.data.frame(as.data.frame(z)))
}

"test.rxCube.comparison.factored.depvar.xdf" <- function()
{
  myDataFrame <- data.frame(x=runif(20), y=factor(rep(1:5, times=4)), z=rep(1:5, each=4))
  myDataFrame$zfac <- as.factor(myDataFrame$z)  
  out1 <- rxCube(x ~ y : F(z), data=myDataFrame)
  out2 <- rxCube(x ~ y : zfac, data=myDataFrame)
  checkTrue(all.equal(out1,out2, check.attributes=FALSE))
}

###
# rxCube and xtabs output comparison
###

"test.rxCube.xtabs.comparison.factor.conversion.df" <- function()
{
 # small example where age must be a factor. the conversion is done internally
 # in the rxCube.formula code when the input data source is a local data.frame.
 # in this case, all independent variables are converted to factors

 # define local data.frame objects
 myDF1 <- data.frame(sex=c("Male","Male","Female", "Male"), age=as.factor(c(20,20,12,15)), score=(1:4)+0.1)
 myDF2 <- data.frame(sex=c("Male","Male","Female", "Male"), age=c(20,20,12,15), score=(1:4)+0.1)

 # will return comparable results to xtabs
 z1 <- rxCube(~ sex : age, myDF1, returnDataFrame=TRUE)
 xform <- function(z)
 {
  z$ageF <- factor(z$age)
  z
 }
 zz1 <- rxCube(~ sex : ageF, myDF1, transformFunc=xform, transformVars="age", returnDataFrame=TRUE)
 checkTrue(all.equal(z1, zz1, check.attributes=FALSE))

 z2 <- rxCube(~ sex : factor(age), myDF2, returnDataFrame=TRUE)
 checkTrue(all.equal(z1, z2, check.attributes=FALSE))
 
 "compareXtabs" <- function(formula, data, cubeVar,
     msg=paste("FORMULA:", deparse(substitute(formula)), ", DATA:", 
         deparse(substitute(data))))
 {  
  # calculate cross tabulations and compare values
    xtabsFormula <- formula(gsub(":","+", deparse(formula)))
  zxtabs <- xtabs(formula=xtabsFormula, data=data)
  zcubes <- rxCube(formula=formula, data=data, means=FALSE)[[cubeVar]]
  checkEqualsNumeric(sort(as.numeric(zxtabs)), sort(as.numeric(zcubes)))
 }

 compareXtabs(score ~ sex : age, myDF1, "score")
 compareXtabs(sin(score) ~ sex : factor(age), myDF2, "sin(score)")
}


###
# output list elemental name checks
###

"test.rxCube.single.depvar.xdf.output.names" <- function()
{
  z <- rxCube(wkswork1 ~ sex : F(age), data=CensusPath, pweights="perwt", blocksPerRead=3)
  checkTrue(all(names(z) %in% c("wkswork1","sex","F_age","Counts")))
}

"test.rxCube.no.depvar.df.output.names" <- function()
{
  sex <- c("Male","Male","Female", "Male")
  counts <- c(20,20,12,15)
  DF1 <- data.frame(sex=sex, Counts=counts)
  DF2 <- data.frame(sex=sex, Counts=factor(counts))
  DF3 <- data.frame(sex=sex, foo=factor(counts))

  checkTrue(all(names(rxCube(~ sex : Counts, DF2)) %in% c("sex","Counts",".rxCounts")))  
  checkTrue(all(names(rxCube(~ sex : foo, DF3)) %in% c("sex","foo","Counts")))  
  checkTrue(all(names(rxCube(~ sex : F(Counts,1.3,20.5,TRUE), DF1)) %in% c("sex","F_Counts_1.3_20.5_T","Counts")))  
  checkTrue(all(names(rxCube(~ sex : F(Counts), DF1)) %in% c("sex","F_Counts","Counts")))  
  checkTrue(all(names(rxCube(Counts ~ sex, DF1)) %in% c("sex","Counts",".rxCounts")))  
  checkTrue(all(names(rxCube(N(Counts) ~ sex, DF2)) %in% c("sex","N_Counts","Counts")))  
}

###
# rxCube stress test functions
###

"test.rxCube.single.depvar.xdf.stress" <- function()
{
    checkTrue(rxCubeStressTest(wkswork1 ~ sex, data=CensusPath, pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCube.single.depvar.with.factorization.xdf.stress" <- function()
{
    checkTrue(rxCubeStressTest(wkswork1 ~ sex : F(age), data=CensusPath, pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCube.single.depvar.with.expression.xdf.stress" <- function()
{
    checkTrue(rxCubeStressTest(log(wkswork1) ~ sex , transformVars="incwage", data=CensusPath, pweights ="perwt", 
        blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCube.single.depvar.with.expression.and.factorization.xdf.stress" <- function()
{
    checkTrue(rxCubeStressTest(log(wkswork1) ~ F(age) , transformVars="incwage", 
      data=CensusPath, pweights="perwt", blocksPerRead=3, rowDisplayMax=300))
}

"test.rxCube.Rcallback.df.stress" <- function()
{
    data <- data.frame(sex=c("Male","Male","Female", "Male"), age=c(20,20,12,15), score=(1:4)+0.1)
      
    xform <- function(x)
    {
      x$revage <- rev(x$age)
    x$frevage <- as.factor(x$revage)
      x$cow <- as.factor(1:4)
      x$sport <- as.factor(c('tennis','golf','tennis','football'))
      return(x)								
    }

    # NOTE: for formula = cbind(score, log(score)) ~ sin(revage) + sex,
    # if transformVars is NULL, ExaStat processes all variables that it parses
    # from the formula: score, revage, sex. However, revage is defined within the 
    # user-defined transformFunc as 'x$revage <- rev(x$age)', which uses the primary
    # variable age in its definition. ExaStat doesn't have any direct knowledge of 'age'
    # and so it forces us to use transformVars to specify which variables we should use
    # from the original data source for the R transform functions. In doing so, we have
    # to specify all the primary variables that are needed for the transformation function,
    # and not just 'age', for example.
   
    # both user-defined and hidden transform 
    checkTrue(rxCubeStressTestDf(log(score) ~ frevage :sex, data=data, 
      transformFunc=xform, transformVars=c("age","score")))

    # user-defined transform only
    checkTrue(rxCubeStressTestDf(revage ~ cow : sport, data=data, transformFunc=xform, transformVars="age"))

    # hidden transform only
    checkTrue(rxCubeStressTestDf(log(score) ~ F(age) : sex, data=data))

    # no user-defined or hidden transform
    checkTrue(rxCubeStressTestDf(score ~ F(age) : sex, data=data))
}

###
# N(var) in expressions
###

"test.rxCube.N.formula.expression.conversion.df" <- function()
{
    myDF <- data.frame(sex=c("Male","Male","Female", "Male"), age=as.factor(c(20,20,12,15)), score=factor((1:4)+0.1))

    # Given that 
    #
    #   > as.numeric(as.factor(c(20,20,12,15))) - 1
    #   [1] 2 2 0 1
    #
    # the following result makes sense because that conversion is what ExaStat currently does
    checkTrue(all(as.vector(rxCube(N(age) ~ sex : score, myDF)$age) == c(2, 2, 0, 1)))

    # this should break because age is a categorical variable here
    checkException(rxCube(age ~ sex : score, myDF), 
    msg="Exception caught in rxCube: age dependent variable should be categorical")
}

# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxCube.rowSelection.df" <- function()
#{
  #DF <- as.data.frame(UCBAdmissions)
  #DF1 <- subset(DF, subset = Gender=='Male' & Freq > 200)
    #form <- Freq ~ Gender : Admit
  ## The line below should be changed to have a single & when the R interpreter is used
  #zA <- rxCube(form, cppInterp=list(rowSelection="Gender=='Male' && Freq > 200"), data=DF)
  #zB <- rxCube(form, data= DF1)
  #checkTrue(all.equal(zA, zB, check.attributes=FALSE))
  #checkTrue(all.equal(dimnames(zA), dimnames(zB)))
#}

"test.rxCube.twoWay.xdf" <- function()
{
    myFile <- file.path(sampleDataDir, "AirlineDemoSmall")
    myTab <- rxCube(ArrDelay~DayOfWeek, data=myFile)
    out1 <- print(myTab)
    checkEqualsNumeric(out1$ArrDelay * out1$Counts, c(1146016,  835866,  779880,  685238, 1186449,  995758,  964939))
  
    # Repeat with uncompressed file
  myFileC <- file.path(sampleDataDir, "AirlineDemoSmallUC")
    myTabC <- rxCube(ArrDelay~DayOfWeek, data=myFileC)
    out1C <- print(myTabC)	
  
  checkEquals(out1, out1C)
  
}

"test.rxCube.sums.means" <- function()
{
  # define local data source
  sex <- c("Male","Male","Female", "Male")
  age <- c(20,20,12,15)
  score <- (1:4)+0.1
  DF <- data.frame(sex, age=factor(age), score=score)
  
  # depvar, switch on means
  zDepNoMeans <- rxCube(score ~ sex : age, DF, means=FALSE, returnDataFrame=TRUE, removeZeroCounts=TRUE)
  zDepMeans <- rxCube(score ~ sex : age, DF, means=TRUE, returnDataFrame=TRUE, removeZeroCounts=TRUE)
  checkEqualsNumeric(zDepNoMeans$score/zDepNoMeans$Counts, zDepMeans$score)
  checkEqualsNumeric(zDepMeans$score, c(3.1, 4.1, 1.6))
  
  # depvar, switch on means; repeat with removeZeroCounts set to FALSE
  zDepNoMeans <- rxCube(score ~ sex : age, DF, means=FALSE, returnDataFrame=TRUE, removeZeroCounts=FALSE)
  zDepMeans <- rxCube(score ~ sex : age, DF, means=TRUE, returnDataFrame=TRUE, removeZeroCounts=FALSE)
  checkEqualsNumeric(zDepNoMeans$score/zDepNoMeans$Counts, zDepMeans$score)
  
  # no depvar, switch on means (only sums returned for score)
  zNoDepNoMeans <- rxCube(~ sex : age, DF, means=FALSE, returnDataFrame=TRUE)	
  zNoDepMeans <- rxCube(~ sex : age, DF, means=TRUE, returnDataFrame=TRUE)
  checkTrue(all.equal(zNoDepNoMeans, zNoDepMeans, check.attributes = FALSE))
  
  zNoDepNoMeans <- rxCube(~ sex : age, DF, means=FALSE, returnDataFrame=TRUE, removeZeroCounts=TRUE)	
  zNoDepMeans <- rxCube(~ sex : age, DF, means=TRUE, returnDataFrame=TRUE, removeZeroCounts=TRUE)
  checkTrue(all.equal(zNoDepNoMeans, zNoDepMeans, check.attributes = FALSE))
}

"test.rxCube.formula.multiple.depvars" <- function()
{
  # create data
  set.seed(100)
  divs <- letters[1:5]
  glads <- c("spartacus", "crixus")
  DF <- data.frame(wins=sample(0:1,25, replace=T), losses=sample(0:1,25, replace=T), tosses=sample(0:1,25, replace=T), 
    rome=factor(sample(c(T,F),25,replace=T)), division=factor(rep(divs,5)), 
    score=runif(25,min=0,max=10), gladiator=factor(c(rep(glads[1],12),rep(glads[2],13))))

  # use formulas with various combinations of dependent variables
  f1 <- rxCube(cbind(wins, score) ~ division : gladiator : rome, data=DF)
  f2 <- rxCube(cbind(wins/2, score*3) ~ division : gladiator : rome, data=DF, means=FALSE)
  f3 <- rxCube(score*3 ~ division : gladiator : rome, data=DF)
  f4 <- rxCube(~ division : gladiator : rome, data=DF)
  f5 <- rxCube(~ division, data=DF)
  f6 <- rxCube(cbind(wins, score, losses, tosses) ~ division : gladiator : rome, data=DF)
  
  # develop transformation functions needed for string form of formulas above
  xform <- function(z)
  {
    z$wins2 <- z$wins / 2
    z$score3 <- z$score * 3
    z
  }
  
  # use formula strings commensurate with the formula combinations
  s1 <- rxCube("wins ~ division : gladiator : rome;score ~ division : gladiator : rome", data=DF)
  s2 <- rxCube("wins2 ~ division : gladiator : rome; score3 ~ division : gladiator : rome", 
        data=DF, means=FALSE,
    transformFunc=xform, transformVars=c("wins","score"))
  s3 <- rxCube("score3 ~ division : gladiator : rome", data=DF,
    transformFunc=xform, transformVars=c("wins","score"))
  s4 <- rxCube("~ division : gladiator : rome", data=DF)
  s5 <- rxCube("~ division", data=DF)
  s6 <- rxCube(paste(c("wins", "score", "losses", "tosses"),
        "~ division : gladiator : rome", collapse=";"), data=DF)

  # compare the outputs
  checkTrue(all.equal(f1, s1, check.attributes=FALSE))
  checkTrue(all.equal(f2, s2, check.attributes=FALSE))
  checkTrue(all.equal(f3, s3, check.attributes=FALSE))
  checkTrue(all.equal(f4, s4, check.attributes=FALSE))
  checkTrue(all.equal(f5, s5, check.attributes=FALSE))
  checkTrue(all.equal(f6, s6, check.attributes=FALSE))
}

###
# argument stress tests
###

"test.rxCube.args.useSparseCube.mixed" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- cost ~ age:car.age:type:F(number)    #mixed continuous and discrete case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxCube.args.useSparseCube.continuous" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- cost ~ F(number)    #pure continuous case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxCube.args.useSparseCube.discrete" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- cost ~ age:car.age:type    #pure discrete case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

#bigAnalytics RA-12455 
#rxCube() reports the wrong cell counts when many categorical cross-classifying factors are used
"test.rxCube.args.useSparseCube.discrete.bigCube" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "CensusWorkers.xdf") 
    info <- rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)
    #rxSummary(~., myData)  # make sure there is no missings
    
    allvars <- names(info$varInfo)
    allvars <- paste("F(", allvars, ")", sep = "")  #convert all vars to factors
    form.summ <- formula(paste("~", paste(allvars, collapse = " + ")))
    summ <- rxSummary(form.summ, myData)
    alllvls <- sapply(summ$categorical, nrow)
    #sapply(summ$categorical, names)
    
    alllvls.order <- order(alllvls)
    allvars <- allvars[alllvls.order]
    alllvls <- alllvls[alllvls.order]
    
    ##NOTE: sizeof.long is 4 on Windows but 8 on Linux
    #uint.max <- 2^(.Machine$sizeof.long * 8) - 1
    uint.max <- 2^32 - 1
    #alllvls.cumprod <- cumprod(alllvls)
    #alllvls.cumprod > uint.max
    
    ###################################################################
    ## too big to compute 
    
    xvars.id <- c(1, 4:6)
    checkTrue(prod(alllvls[xvars.id]) > uint.max)
    myForm <- paste("~", paste(allvars[xvars.id], collapse = ":"))
    
    ###############################
    checkException(useSparseCubeF.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        ))   
    checkException(useSparseCubeT.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        ))
    
    ###############################
    checkException(useSparseCubeF.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        ))
    checkException(useSparseCubeT.removeZeroCountsF <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        ))
    
    ###################################################################
    ## big but still able to compute
    
    xvars.id <- -5
    checkTrue(prod(alllvls[xvars.id]) < uint.max)
    myForm <- paste("~", paste(allvars[xvars.id], collapse = ":"))
    
    ###############################
    useSparseCubeT.removeZeroCountsT <- rxCube( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

    checkEquals(summ$categorical[[4]][["Counts"]][summ$categorical[[4]][["F_sex"]] == "Male"],
        sum(useSparseCubeT.removeZeroCountsT[["Counts"]][useSparseCubeT.removeZeroCountsT[["F_sex"]] == "Male"]))
    checkEquals(summ$categorical[[4]][["Counts"]][summ$categorical[[4]][["F_sex"]] == "Female"],
        sum(useSparseCubeT.removeZeroCountsT[["Counts"]][useSparseCubeT.removeZeroCountsT[["F_sex"]] == "Female"]))
}

"test.rxCube.args.blocksPerRead" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  form <- ~ F(y) : F(xnum2miss)
  zCubeBlocksDefault <- rxCube(form, data=test100r5b, blocksPerRead=-1)
  zCubeBlocks1 <- rxCube(form, data=test100r5b, blocksPerRead=1)
  checkTrue(all.equal(zCubeBlocksDefault, zCubeBlocks1, check.attributes=FALSE))
  
  zCubeBlocks5 <- rxCube(form, data=test100r5b, blocksPerRead=5)
  checkTrue(all.equal(zCubeBlocksDefault, zCubeBlocks5, check.attributes=FALSE))
}

# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxCube.args.cppInterp" <- function()
#{
  #DF <- infert
  #DF1 <- subset(infert, parity > 3)
  #form <- stratum ~ F(age) : F(parity) : education : F(spontaneous) : F(induced)
#
  ## The line below should be changed to have a single & when the R interpreter is used
  #zA <- rxCube(form, cppInterp=list(rowSelection="parity > 3"), data=DF)	
  #zB <- rxCube(form, data=DF1)
  #checkTrue(all.equal(coef(zA), coef(zB)))
#}

"test.rxCube.args.cube" <- function()
{
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  zCubeFalse <- rxCube(form, data=DF, cube=FALSE)
  zCubeTrue <- rxCube(form, data=DF, cube=TRUE)
  checkTrue(all.equal(zCubeFalse, zCubeTrue, check.attributes=FALSE))
}

"test.rxCube.args.data" <- function()
{
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  XDF <- tempfile( pattern = ".rxCube", fileext = ".xdf")
  rxDataStep(DF, XDF)
  
  zCubeDF <- rxCube(form, data=DF)
  zCubeXDF <- rxCube(form, data=XDF)
  checkTrue(all.equal(zCubeDF, zCubeXDF, check.attributes=FALSE))
    file.remove(XDF)
}

"test.rxCube.args.formula" <- function()
{
  form <- ~ division : state
  formStr <- deparse(form)
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  zCubeFormula <- rxCube(form, data=DF)
  zCubeString <- rxCube(formStr, data=DF)
  checkTrue(all.equal(zCubeFormula, zCubeString, check.attributes=FALSE))
}

"test.rxCube.args.formula.F" <- function()
{
  DF <- data.frame(x=rep(c(10,100),4), y=1:8, xfac=as.factor(rep(c(10,100),4)),
    xlogfac=as.factor(log10(rep(c(10,100),4))))
  cubeOut1 <- rxCube(~xfac, data=DF)
  cubeOut2 <- rxCube(~F(x), data=DF)
  checkEquals(cubeOut1$data, cubeOut2$data, check.attributes=FALSE)
}

"test.rxCube.args.means" <- function()
{
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  zCubeMeansFalse <- rxCube(form, data=DF, means=FALSE)
  zCubeMeansTrue <- rxCube(form, data=DF, means=TRUE)
  comparativeMean <- zCubeMeansFalse$score / zCubeMeansFalse$Counts
  checkEqualsNumeric(zCubeMeansTrue$score, comparativeMean)
}

"test.rxCube.args.fweights" <- function()
{
    fwts <- c((1:5), (1:5))
    sex <- c("Male","Male","Female", "Male", "Female", "Male", "Female", "Male", "Female", "Male") 
    age <- c(NA,20,12,15, NA, 17, 16, 13, 19, 12) 

    myDF1 <- data.frame(sex = sex, age=age, weightVar = fwts) 
    myDF2 <- data.frame(sex = rep(sex,fwts), age = rep(age, fwts)) 
    fweightsObj <- rxCube(age ~ sex , data = myDF1, fweights = "weightVar") 
    baseObj <- rxCube(age ~ sex , data = myDF2) 
     # Number of missings will be unweighted for fweightsObj
    attr(fweightsObj, "missingObs") <- NULL
    attr(baseObj, "missingObs") <- NULL 
    attr(fweightsObj, "call") <- NULL
    attr(baseObj, "call") <- NULL
    attr(fweightsObj, "params") <- NULL
    attr(baseObj, "params") <- NULL
    checkEquals(fweightsObj, baseObj)     
}

"test.rxCube.args.pweights.fweights" <- function()
{
    fwts <- 1:4
    sex <- c("Male","Male","Female", "Male")
    age <- c(20,20,12,15)
    score <- (1:4)+0.1
    
    myDF1 <- data.frame(sex=sex, age=age, score=factor(score), pWeights=fwts)    
    myDF2 <- data.frame(sex=factor(rep(sex,fwts)), age=rep(age,fwts), score=factor(rep(score,fwts)))
    
    z1 <- rxCube(age ~ sex : score, data=myDF1, pweights="pWeights", removeZeroCounts=TRUE)
  z1a <- rxCube(cbind(age,age/2) ~ sex : score, data=myDF1, pweights="pWeights", removeZeroCounts=TRUE)
    z2 <- rxCube(age ~ sex : score, data=myDF2, removeZeroCounts=TRUE)
  pw <- "pWeights"
  z3 <- rxCube(age ~ sex : score, data=myDF1, fweights = pw, removeZeroCounts=TRUE)
    
    checkTrue(all.equal(z1,z2, check.attributes=FALSE))
  checkTrue(all.equal(z3,z2, check.attributes=FALSE))
    checkTrue(all(as.vector(z1$score) == score))
    checkTrue(all(as.vector(z1$sex) == sex))
    checkTrue(all(as.vector(z1$age) == age))
    checkTrue(all(as.vector(z1$Counts) == fwts))
}

"test.rxCube.args.reportProgress" <- function()
{
  # reportProgress argument stress tests
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  #reportProgress: logical, character string or integer: 
  #   0: no progress is reported. 
  #   1: the number of processed rows is printed and updated. 
  #   2: rows processed and timings are reported. 
  #  3: rows processed and all timings are reported. 

  variations <- list(0,1,2,3)	
  results <- printOutput <- vector("list", length=length(variations))
  
  for (i in seq_along(variations))
  {
    printOutput[[i]] <- capture.output(results[[i]] <- 
            rxCube(form=form, data=DF, reportProgress=variations[[i]]))
  }

  # check that all values are the same
  checkTrue(all(unlist(lapply(results[-1], 
    function(x,y){all.equal(x, y, check.attributes=FALSE)}, y=results[[1]]))))

  names(printOutput) <- as.character(variations)
  

  noResponseExpected <- c("0")
  equalLengthExpected <- c("2")
  equalLengthExpected1 <- c("1")

  checkTrue(length(unlist(printOutput[noResponseExpected])) == 0)
  
  # just row numbers
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1],length)) == 1))
  
  # all these outputs should be the same
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected],length)) == 2))
  
  # details gives us the goods
  checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=5, TotalRowsProcessed=5, ReadTime", 
    printOutput[["3"]][1])) > 0)
  
  # time
  checkTrue(length(grep("^Computation time:", printOutput[["2"]][2])) > 0)
}

"test.rxCube.args.returnDataFrame" <- function()
{
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  zDFFalse <- rxCube(form, data=DF, returnDataFrame=FALSE)
  zDFTrue <- rxCube(form, data=DF, returnDataFrame=TRUE)

  checkTrue(!is.data.frame(zDFFalse) && is.list(zDFFalse))
  checkTrue(is.data.frame(zDFTrue) && is.list(zDFTrue))
  checkTrue(all.equal(as.data.frame(zDFFalse), zDFTrue, check.attributes=FALSE))
}

"test.rxCube.args.returnDataFrame.subset" <- function()
{
  form <- score ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(score=runif(N,0,100),
    division=sample(letters, N, replace=TRUE), 
    state=sample(state.name, N, replace=TRUE))
  
  zDFFalse <- rxCube(form, data=DF, returnDataFrame=FALSE)
  zDFTrue <- rxCube(form, data=DF, returnDataFrame=TRUE)
    
    zDFsubset <- zDFFalse[,c("division", "state")]
    checkTrue(!("rxCube" %in% class(zDFsubset)))
    checkTrue(is.data.frame(zDFsubset))
    
    rm(zDFsubset)
    
    zDFsubset <- zDFTrue[,c("division", "state")]
    checkTrue(!("rxCube" %in% class(zDFsubset)))
    checkTrue(is.data.frame(zDFsubset))
    
    rm(zDFsubset)
    
    zDFsubsetF <- subset(zDFFalse, score > 5, select=c(division, state))
    checkTrue(!("rxCube" %in% class(zDFsubsetF)))
    checkTrue(is.data.frame(zDFsubsetF))
    
    zDFsubsetT <- subset(zDFTrue, score > 5, select=c(division, state))
    
    checkEquals(zDFsubsetF, zDFsubsetT)
}

"test.rxCube.args.rowSelection" <- function()
{
  DF <- as.data.frame(UCBAdmissions)
  DF1 <- subset(DF, subset = Gender=='Male' & Freq > 200)
    form <- Freq ~ Gender : Admit
  zA <- rxCube(form, rowSelection=Gender=='Male' & Freq > 200, data= DF)
  zB <- rxCube(form, data= DF1)
  checkTrue(all.equal(zA$sums, zB$sums))
}

"test.rxCube.args.transformFunc" <- function()
{
  # reportProgress argument stress tests
  set.seed(10)
  DF <- data.frame(score=(1:5)*0.1,
    division=c("a","a","z","t","l"), 
    state=sample(state.name, 5, replace=TRUE))
  xform <- function(dataList)
  {
    set.seed(10)
    dataList$STATE <- factor(sample(datasets::state.name, 5, replace=TRUE))
    dataList	
  }

  z1 <- rxCube(score ~ division : STATE, data=DF, transformFunc=xform)
  z2 <- rxCube(score ~ division : state, data=DF)
  checkTrue(all.equal(z1,z2,check.attributes=FALSE))
}

"test.rxCube.args.transformVars" <- function()
{
  # reportProgress argument stress tests
  set.seed(10)
  DF <- data.frame(score=(1:5)*0.1,
    division=c("a","a","z","t","l"), 
    state=factor(sample(state.name, 5, replace=TRUE)))
  xform <- function(dataList)
  {
    dataList$stateCopy <- dataList$state
    dataList	
  }

  z1 <- rxCube(score ~ division : stateCopy, data=DF, transformFunc=xform, transformVars="state")
  z2 <- rxCube(score ~ division : state, data=DF)
  checkTrue(all.equal(z1,z2,check.attributes=FALSE))
}

"test.rxCube.args.verbose" <- function()
{
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  
  zCubeVerboseFalse <- capture.output(z <- rxCube(form, data=DF, verbose=0))
  zCubeVerboseTrue <- capture.output(z <- rxCube(form, data=DF, verbose=1))
  checkTrue(length(zCubeVerboseTrue) > length(zCubeVerboseFalse))
}

"test.rxCube.args.tranforms.transformFunc.rowSelection" <- function()
{
    # transforms, transformFunc, and rowSelection    
    xform <- function(dataList)
    {
        # add actor variable which is equivalent to existing
        # Species variable, only with levels converted to upper case 
        dataList$mySpecies <- factor(toupper(dataList$Species)) 
        dataList
    }

    transformVars <- c("Species")
    
    form1 <- I(sqrt(mySepalLength)) ~ mySpecies : F(Petal.Length)

    z1 <- rxCube(formula=form1, data=iris,
        transforms = list(
            mySepalLength = Sepal.Length^2, 
            Sepal.Length = NULL),
        transformFunc = xform,
        transformVars = transformVars,
        rowSelection=mySepalLength > 6^2, removeZeroCounts=TRUE)

    # transformFunc and rowSelection
    form2 <- Sepal.Length ~ mySpecies : F(Petal.Length)
    z2 <- rxCube(formula=form2, data=iris,
        transformFunc = xform,
        transformVars = transformVars,
        rowSelection=Sepal.Length > 6, removeZeroCounts=TRUE)

    # rowSelection
    data3 <- iris
    data3$mySpecies <- factor(toupper(iris$Species))
    z3 <- rxCube(formula=form2, data=data3,
        rowSelection=Sepal.Length > 6, removeZeroCounts=TRUE)

    # No data transformations
    data4 <- iris
    data4$mySpecies <- factor(toupper(iris$Species))
    data4 <- subset(data3, Sepal.Length > 6)
    z4 <- rxCube(formula=form2, data=data4, removeZeroCounts=TRUE)

    checkEquals(z1,z2, check.attributes=FALSE)
    checkEquals(z2,z3, check.attributes=FALSE)
    checkEquals(z3,z4, check.attributes=FALSE)
}

"test.rxCube.methods.rxResultsDF" <- function()
{
  y = rep(1:2, times=5)
  myData <- data.frame(x = rnorm(10), y = y, yfac = factor(y))
  myCube1 <- rxCube(x~yfac, data=myData)
  resultsDF1 <- rxResultsDF(myCube1)
  checkTrue(is.integer(resultsDF1$Counts))
  
  myCube2 <- rxCube(x~F(y), data=myData)
  resultsDF2 <- rxResultsDF(myCube2)
  checkTrue(is.integer(resultsDF2$y))
  
  checkEquals(resultsDF1$x, resultsDF2$x)
  
  myData$y1 <- rep(1:5, times=2)
  myCube3 <- rxCube(x~yfac:F(y1), data=myData)
  resultsDF3 <- rxResultsDF(myCube3)
  checkTrue(is.integer(resultsDF3$y1))
  checkTrue(is.integer(resultsDF3$Counts))
  
}

########	rxCube Analysis on Text datasource	
#		All independent variables must be factors for rxCube and rxCrossTabs
#		Dependent variables must be continuous variables in the analysis
"test.rxCube.dataSource.Text" <- function()
{		
    refDate <- as.Date(ISOdate(2008,8,26))
  
  inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.csv")
    colClasses <- c("ArrDelay" = "integer", "CRSDepTime" = "integer", "UniqueCarrier" = "factor")	        
  colInfo <- list(  UniqueCarrier = list(type = "factor", levels = c("9E","AA", "AS","B6", "CO", "DH", "DL", "EA", "EV", "F9", "FL", 
              "HP", "MQ", "NW", "OH", "OO", "PA(1)", "PI", "PS", "TW", "TZ", "UA", "US", "WN","XE", "YV")))
                
    dsText <- RxTextData(file = inFile, colClasses = colClasses,
        colInfo = colInfo, useFastRead = FALSE)

    result <- rxCube(ArrDelay~UniqueCarrier, data=dsText, removeZeroCounts = TRUE, 
          rowSelection = Date > refDate,  transformObjects = list(refDate = refDate), returnDataFrame = TRUE)
  
    result2 <- rxCube(ArrDelay ~ UniqueCarrier, file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf"), removeZeroCounts = TRUE, 
          rowSelection = Date > refDate,  transformObjects = list(refDate = refDate), returnDataFrame = TRUE)
        
    checkEquals(result$coefficients, result2$coefficients)
}

"test.rxCube.dataSource.Text.factor" <- function()
{
  xLabels <- c("a", "b", "c")
  x1 <- rep(xLabels, each = 4)
  x2 <- 1:12
  testFile <- tempfile(pattern = ".rxCube1TempTest", fileext = ".csv")  
  write.csv( data.frame(x1, x2), testFile, row.names = FALSE)

  textDS <- RxTextData( testFile , colClasses = c(x1 = "factor"), rowsPerRead = 3 )

  cubeOut <- rxResultsDF(rxCube(~x1, data = textDS))
  checkEquals(cubeOut$x1, as.factor(xLabels))
  checkEquals(cubeOut$Counts, as.integer(c(4,4,4)))
  file.remove(testFile)
}
"test.rxCube.dataSource.Text.F" <- function()
{
  x1 <- rep(1:3, each = 4)
  x2 <- 1:12
  testFile <- tempfile(pattern = ".rxCube2TempTest", fileext = ".csv") 
  write.csv( data.frame(x1, x2), testFile, row.names = FALSE)

  textDS <- RxTextData(testFile, rowsPerRead = 3 )

  cubeOut <- rxResultsDF(rxCube(~F(x1), data = textDS))
  checkEquals(cubeOut$Counts, as.integer(c(4,4,4)))
  checkEquals(cubeOut$x1, as.integer(1:3)) 
  
  # Now try a data transformation
  cubeOut <- rxResultsDF(rxCube(~F(x1), data = textDS,
    transforms = list(myNewX = x2*2)))
  
  checkEquals(cubeOut$Counts, as.integer(c(4,4,4)))
  checkEquals(cubeOut$x1, as.integer(1:3))
  
  # Change the order
  x3 <- as.integer(rep(1:4, each = 3))
  write.csv( data.frame(x1, x2, x3), testFile, row.names = FALSE)

  textDS <- RxTextData(testFile, rowsPerRead = 3 )	
  cubeOut <- rxResultsDF(rxCube(~F(x3), data = textDS,
    transforms = list(myNewX = x1*2)))
  
  checkEquals(cubeOut$Counts, as.integer(c(3,3,3,3)))
  checkEquals(cubeOut$x3, as.integer(1:4))
  
  # Use a data transformation in the F()
  cubeOut <- rxResultsDF(rxCube(~F(myNewX, low=1, high=24), data = textDS,
    transforms = list(myNewX = x2*2)))
  
  checkEquals(cubeOut$Counts, as.integer(rep(0:1, times=12)))
  checkEquals(cubeOut$myNewX, as.integer(1:24))
  file.remove( testFile )
}

######## rxCube Analysis on SAS datasource   
"test.rxCube.dataSource.SAS" <- function()
{		
  dsSAS <- RxSasData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.sas7bdat"),
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))
            )
    result <- rxCube(creditScore~default, data = dsSAS)
      
    testFile <- tempfile(pattern = ".rxCube3TempTest", fileext = ".xdf")            
  rxImport(dsSAS, testFile, overwrite = TRUE, reportProgress = 0)
    result2 <- rxCube(creditScore~default, testFile)
        
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}

######## rxCube Analysis on ODBC datasource 	
"test.rxCube.dataSource.ODBC.ST" <- function()
{			
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.sqlite"),
                        useFastRead = FALSE),
          silent = TRUE)
    
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  dsODBC <- RxOdbcData(
        sqlQuery = "SELECT * FROM mortDef",
                connectionString = conString, useFastRead = FALSE,
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "isDefault" = "factor"),
             colInfo = list(  isDefault = list(type = "factor", levels = c("No","Yes")))
            )
          
    result <- rxCube(creditScore~isDefault, data=dsODBC)
  
    testFile <- tempfile(pattern = ".rxCube4TempTest", fileext = ".xdf")    
  rxImport(dsODBC, testFile, overwrite = TRUE, reportProgress = 0, verbose=1)
  result2 <- rxCube(creditScore~isDefault, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}

######## rxCube Analysis on Direct ODBC datasource 	
"test.rxCube.dataSource.useFastRead" <- function()
{		
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.sqlite")),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  dsODBC <- RxOdbcData(
        sqlQuery = "SELECT * FROM mortDef",
                connectionString = conString,
        useFastRead = TRUE,
        colInfo = list(  isDefault = list(type = "factor", levels = c("No","Yes")))
            )
          
    result <- rxCube(creditScore~isDefault, data=dsODBC)
  
    testFile <- tempfile(pattern = ".rxCube5TempTest", fileext = ".xdf")    
  rxImport(dsODBC, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxCube(creditScore~isDefault, testFile)	
  
    checkEquals(rxResultsDF(result), rxResultsDF(result2))
    file.remove(testFile)
}

"test.rxCube.method.as.xtabs" <- function()
{
    # define local function to compare xtabs to as.xtabs(rxCube(...)) output
    "rxCubeEqualsXtabsSansCall" <- function(xtabsObject, rxCubeObject)
    {
        attr(xtabsObject, "call") <- attr(rxCubeObject, "call") <- NULL
        checkEquals(xtabsObject, rxCubeObject)
    }
    
    # compare xtabs and as.xtabs(rxCube(...)) results for 3-way interaction, one dependent variable
    set.seed(100)
    divs <- letters[1:5]
    glads <- c("spartacus", "crixus")
    romeDF <- data.frame( division = rep(divs, 5L), 
                          score = runif(25, min = 0, max = 10), 
                          rank = runif(25, min = 1, max = 100), 
                          gladiator = c(rep(glads[1L], 12L), rep(glads[2L], 13L)),
                          arena = sample(c("colosseum", "ludus", "market"), 25L, replace = TRUE))

    z1 <- rxCube(score ~ division : gladiator : arena, data = romeDF)
    z2 <- xtabs(score ~ division + gladiator + arena, romeDF)
    rxCubeEqualsXtabsSansCall(as.xtabs(z1), z2)

    # compare xtabs and as.xtabs(rxCube(...)) results for 3-way interaction, multiple dependent variable
    z1 <- rxCube(cbind(score, rank) ~ division : gladiator : arena, data = romeDF, means = TRUE)
    z2 <- xtabs(cbind(score, rank) ~ division + gladiator + arena, romeDF)
    rxCubeEqualsXtabsSansCall(as.xtabs(z1), z2)
    
    # compare xtabs and as.xtabs(rxCube(...)) results for 3-way interaction, no dependent variable
    z1 <- rxCube( ~ division : gladiator : arena, data = romeDF, means = FALSE)
    z2 <- xtabs(~ division + gladiator + arena, romeDF)
    rxCubeEqualsXtabsSansCall(as.xtabs(z1), z2)

    # compare xtabs and as.xtabs(rxCube(...)) results for 2-way interaction, no dependent variable
    DF <- data.frame(sex=c("Male","Male","Female","Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1)
    z1 <- rxCube(~ sex : age, DF, means = FALSE)
    z2 <- xtabs(~ sex + age, DF)
    rxCubeEqualsXtabsSansCall(as.xtabs(z1), z2)
 
    # compare xtabs and as.xtabs(rxCube(...)) results for 2-way interaction, multiple dependent variablea
    DF <- data.frame(sex=c("Male","Male","Female","Male"), age=factor(c(20,20,12,15)), score=(1:4)+0.1, rank = rnorm(4), size = runif(4))
    z1 <- rxCube(cbind(score, rank, size) ~ sex : age, DF, means = TRUE)
    z2 <- xtabs(cbind(score, rank, size) ~ sex + age, DF)
    rxCubeEqualsXtabsSansCall(as.xtabs(z1), z2)
   
    # compare xtabs and as.xtabs(rxCube(...)) results for a single factor variable, no dependent variable
    DF <- data.frame(sex=c("Male","Male","Female","Male"))
    z1 <- rxCube(~ sex, DF, means = TRUE) # means should be ignored
    z2 <- xtabs(~ sex, DF)
    rxCubeEqualsXtabsSansCall(as.xtabs(z1), z2)
    
    # compare xtabs and as.xtabs(rxCube(...)) results for a single factor variable, no dependent variable
    # and with all zero counts removed. Here, we should expect an error because we cannot repopulate the
    # rows that were removed having zero counts.
    z1 <- rxCube( ~ division : gladiator : arena, data = romeDF, removeZeroCounts = TRUE)
    checkException(as.xtabs(z1), silent = TRUE)
}

"test.rxCube.data.missings" <- function()
{
    set.seed(17)
    x <- as.factor(as.integer(runif(100, min = 1, max = 5)))
    y <- as.factor(as.integer(runif(100, min = 1, max = 5)))
  naIdx <- c(12, 19, 57, 88, 89)
    x[naIdx] <- NA
    myData <- data.frame(x, y)
    x[naIdx] <- "1"
    myDataNoNA <- data.frame(x, y)
    myFunc <- function(data)
  {
        data$x[is.na(data$x)] <- "1"
        print(length(data$x))
    return(data)
  }
    cubeObj <- rxCube( ~y:x, data=myData,
        transformFunc = myFunc)
    cubeObjNoNA <- rxCube(~y:x, data=myDataNoNA)
    checkEquals(rxResultsDF(cubeObj), rxResultsDF(cubeObjNoNA))
    
}

"test.rxCube.args.outFile" <- function()
{
  inFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall")
  cubeXdf <- tempfile( pattern = ".rxCubeOut", fileext = ".xdf")
    cubeXdfDS <- rxCube(ArrDelay~DayOfWeek, data=inFile, outFile = cubeXdf, overwrite = TRUE, rowsPerBlock = 5)
  
  cubeDFfromXdf<- rxReadXdf(cubeXdfDS)
  cubeDF <- as.data.frame(rxCube(ArrDelay~DayOfWeek, data=inFile))
  
  checkEquals(cubeDFfromXdf$DayOfWeek, cubeDF$DayOfWeek)
  checkEquals(cubeDFfromXdf$ArrDelay, cubeDF$ArrDelay)
  checkEquals(cubeDFfromXdf$Counts, cubeDF$Counts)
  
  fileInfo <- rxGetInfo(cubeXdf, getBlockSizes = TRUE)
  checkEquals(fileInfo$rowsPerBlock[1], 5)
    checkEquals(fileInfo$numBlocks, 2)
  
  if (file.exists(cubeXdf)) file.remove(cubeXdf)
  
  # Just counts
  cubeXdfDS <- rxCube(~DayOfWeek, data=inFile, outFile = cubeXdf, overwrite = TRUE, rowsPerBlock = 5)
  
  cubeDFfromXdf<- rxReadXdf(cubeXdfDS)
  cubeDF <- as.data.frame(rxCube(~DayOfWeek, data=inFile))
  
  checkEquals(cubeDFfromXdf$DayOfWeek, cubeDF$DayOfWeek)
  checkEquals(cubeDFfromXdf$Counts, cubeDF$Counts)
  
  fileInfo <- rxGetInfo(cubeXdf, getBlockSizes = TRUE)
  checkEquals(fileInfo$rowsPerBlock[1], 5)
  
  if (file.exists(cubeXdf)) file.remove(cubeXdf)
  
  # Xdf data source
  cubeXdfOutSource <- RxXdfData(cubeXdf)
    cubeXdfDS <- rxCube(ArrDelay~DayOfWeek, data=inFile, outFile = cubeXdfOutSource, overwrite = TRUE, rowsPerBlock = 5)
  cubeDF <- as.data.frame(rxCube(ArrDelay~DayOfWeek, data=inFile))
  cubeDFfromXdf<- rxReadXdf(cubeXdfDS)
  
  checkEquals(cubeDFfromXdf$DayOfWeek, cubeDF$DayOfWeek)
  checkEquals(cubeDFfromXdf$ArrDelay, cubeDF$ArrDelay)
  checkEquals(cubeDFfromXdf$Counts, cubeDF$Counts)
  
  fileInfo <- rxGetInfo(cubeXdfOutSource, getBlockSizes = TRUE)
  checkEquals(fileInfo$rowsPerBlock[1], 5)
    checkEquals(fileInfo$numBlocks, 2)
            
  if (file.exists(cubeXdf)) file.remove(cubeXdf)
}

"test.rxCube.args.overwrite" <- function()
{
  inFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall")
  cubeXdf <- tempfile( pattern = ".rxCubeOut", fileext = ".xdf")
  if (file.exists(cubeXdf)) file.remove(cubeXdf)
    cubeXdfDS <- rxCube(ArrDelay~DayOfWeek, data=inFile, outFile = cubeXdf, overwrite = FALSE, rowsPerBlock = 5)
  
  fileInfo <- rxGetInfo(cubeXdf, getBlockSizes = TRUE)
  checkEquals(fileInfo$rowsPerBlock[1], 5)
  
  # Should fail
  
  res <- try(
    cubeXdfDS <- rxCube(ArrDelay~DayOfWeek, data=inFile, outFile = cubeXdf, overwrite = FALSE, rowsPerBlock = 5)
    )
  
  checkTrue(is(res, "try-error"))
  
  # Should succeed
  cubeXdfDS <- rxCube(ArrDelay~DayOfWeek, data=inFile, outFile = cubeXdf, overwrite = TRUE, rowsPerBlock = 3)
  
  fileInfo <- rxGetInfo(cubeXdf, getBlockSizes = TRUE)
  checkEquals(fileInfo$rowsPerBlock[1], 3)
  
  if (file.exists(cubeXdf)) file.remove(cubeXdf)
  
}

"test.rxCube.args.rowsPerBlock" <- function()
{
  inFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
  cubeXdf <- tempfile( pattern = ".rxCubeOut", fileext = ".xdf")
    cubeXdfDS <- rxCube(incwage~F(wkswork1):sex, data=inFile, outFile = cubeXdf, overwrite = TRUE, rowsPerBlock = 20)
  
  cubeDFfromXdf<- rxReadXdf(cubeXdfDS)
  cubeDF <- as.data.frame(rxCube(incwage~F(wkswork1):sex, data=inFile))
  
  checkEquals(cubeDFfromXdf$incwage, cubeDF$incwage)
  checkEquals(cubeDFfromXdf$F_wkswork1, cubeDF$F_wkswork1)
  checkEquals(cubeDFfromXdf$Counts, cubeDF$Counts)
  
  fileInfo <- rxGetInfo(cubeXdf, getBlockSizes = TRUE)
  checkEquals(fileInfo$rowsPerBlock[1], 20)
  
  file.remove(cubeXdf)
}

"test.rxCube.args.outFile.Counts" <- function()
{

    #RA-11396
    testCubeXdf <- tempfile(pattern = "rxCubeCnt", fileext = ".xdf") 

    x <- data.frame(X1 = sample(LETTERS[1:4], size = 20, replace = TRUE), 
        X2 = sample(LETTERS[5:6], size = 20, replace = TRUE)) 

    rxCube(~ X1:X2, data = x, outFile = testCubeXdf, removeZeroCounts = TRUE, overwrite = TRUE) 
    
    cubeOut <- rxCube(Counts ~ X2, data = testCubeXdf, removeZeroCounts = TRUE) 
    
    checkEquals(cubeOut$.rxCounts, c(4,3))
    
    unlink(testCubeXdf)
}