# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxDForest
#
############################################################################################

.setUp <- function()
{
    #require(rpart)
  #THIS MUST ALWAYS BE SET TO FALSE ON CHECKIN
  .check.cptable <<- FALSE
    
    .deactivate.rxDForest.tests <<- FALSE
    #.maxRowsInMemory <<- 1e6
    
    rxOptions(reportProgress = 0) # by default, don't report progress
}

.tearDown <- function()
{
    .deactivate.rxDForest.tests <<- FALSE
  if(exists(".maxRowsInMemory", where = ".GlobalEnv")) rm(".maxRowsInMemory", pos = ".GlobalEnv")

  rxOptions(TRUE)
}

rtsCheck <- function(regular.model, binary.model, data, type)
{
    # Convert factors to strings for the real-time scoring. Real-time scoring takes care of the factorization.
    rtsData <- data
    factorColumns <- sapply(rtsData, is.factor)
    rtsData[factorColumns] <- lapply(rtsData[factorColumns], as.character)
    
    pred.regular <- rxPredict(regular.model, data = data, type = type)
    oo <- capture.output(pred.rts <- rxPredict(binary.model, data = rtsData, type = type))
    
    # Verify we went through the native scoring path    
    checkEquals(1, sum(grepl("Native prediction succeeded", oo)), paste0("Verification that native scoring was used [type = ", type, "]."))
    # Rename the columns - the name are different
    colnames(pred.regular) <- paste0("col", 1:NCOL(pred.regular))
    colnames(pred.rts) <- paste0("col", 1:NCOL(pred.rts))
    # Verify the predictions are the same
    checkEquals(pred.regular, pred.rts, paste0("RTS results verification [type = ", type, "]."))
}

#######################################################################
# testing scope:
#
#	+ methods: classification vs regression
#	+ randomization: randomize rows/obs vs randomize cols/vars
#		- randomize rows: no sampling, w/ and w/o replacement, stratefied sampling
#		- randomize cols: no sampling, w/o replacement (only), sampling all cols
#	+ data source: data.frame vs xdf file
#		- data.frame: single-thread vs multi-thread
#		- xdf file: single-block vs multi-block, multi-core vs multi-computer

##NOTE: the *.dataframe.xdffile.* tests are expected to fail (not error) 
##	with multiple compute nodes even though the xdf file has only one block.
##	This is because while we use only one node (the client) to process data.frame,
##	we use the server to process the xdf file. The client and server use different RNGs.

#######################################################################
## classification

###################################
## data.frame v.s. single-block xdf

"test.rxDForest.classification.dataframe.xdffile.weight" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestA", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
    
  ## xdf file with only 1 block
    rxDataStep(DF, XDF, overwrite = TRUE)    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#NULL		#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  computeOobError <- -1
    
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
    blocksPerRead = 1,	#numBlocks,	#
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  dontmatch <- c("params", "call")
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
    
    checkEquals(rxTreeDepth(fit.dforest.xdf),  rxTreeDepth(fit.dforest.df))
    checkEquals(rxLeafSize(fit.dforest.xdf),  rxLeafSize(fit.dforest.df))

  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  if (computeOobError == 1) plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  if (computeOobError == 1) plot(fit.dforest.xdf)
  
  ######################################################################
  ## rxPredict.rxForest
  
  ## use data.frame as input
  XDFtmp <- tempfile( pattern = ".rxDForestB", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
    on.exit(if(file.exists(XDFtmp)) unlink(XDFtmp), add = TRUE)
  
  pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  pred.dforest.vote.df <- rxPredict(fit.dforest.df, data = DF, type = "vote", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  pred.dforest.prob.df <- rxPredict(fit.dforest.df, data = DF, type = "prob", computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp) #residType = c("usual", "pearson", "deviance")
  pred.dforest.freq.df <- rxPredict(fit.dforest.df, data = DF, type = "freq", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  
  ## use xdf file as input
  XDFout <- tempfile( pattern = ".rxDForestC", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
    on.exit(if(file.exists(XDFout)) unlink(XDFout), add = TRUE)
  
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.resp.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "vote", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.vote.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "prob", computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, overwrite = TRUE) #residType = c("usual", "pearson", "deviance")
  pred.dforest.prob.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "freq", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.freq.xdf <- rxDataStep(XDFout)

  ## compare data.frame and xdf results
  checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf)
  checkEquals(pred.dforest.vote.df, pred.dforest.vote.xdf)
  checkEquals(pred.dforest.prob.df, pred.dforest.prob.xdf)
  checkEquals(pred.dforest.freq.df, pred.dforest.freq.xdf)
  
  ##############
  ## Real-Time scoring
  fit.binary <- fit.dforest.df
  fit.binary$binary <- rxSerializeModel(fit.binary)
  
  for(tt in c("response", "vote", "prob", "freq"))
  {
      rtsCheck(fit.dforest.df, fit.binary, data, type = tt)
  }
}

"test.rxDForest.classification.dataframe.xdffile.weight.multithread" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestD", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    on.exit(if(file.exists(XDF)) file.remove(XDF), add = TRUE)
    
  ## xdf file with only 1 block
    rxDataStep(DF, XDF, overwrite = TRUE)    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#NULL		#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  computeOobError <- 1
    
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
    blocksPerRead = 1,	#numBlocks,	#
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  dontmatch <- c("params", "call")
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

    checkEquals(rxTreeDepth(fit.dforest.xdf),  rxTreeDepth(fit.dforest.df))
    checkEquals(rxLeafSize(fit.dforest.xdf),  rxLeafSize(fit.dforest.df))
    
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  if (computeOobError == 1) plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  if (computeOobError == 1) plot(fit.dforest.xdf)
  
  ######################################################################
  ## rxPredict.rxForest
  
  ## use data.frame as input
  XDFtmp <- tempfile( pattern = ".rxDForestE", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
    on.exit(if(file.exists(XDFtmp)) file.remove(XDFtmp), add = TRUE)
    
  pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  pred.dforest.vote.df <- rxPredict(fit.dforest.df, data = DF, type = "vote", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  pred.dforest.prob.df <- rxPredict(fit.dforest.df, data = DF, type = "prob", computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp) #residType = c("usual", "pearson", "deviance")
  pred.dforest.freq.df <- rxPredict(fit.dforest.df, data = DF, type = "freq", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  
  ## use xdf file as input
  XDFout <- tempfile( pattern = ".rxDForestF", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
    on.exit(if(file.exists(XDFout)) file.remove(XDFout), add = TRUE)
    
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.resp.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "vote", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.vote.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "prob", computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, overwrite = TRUE) #residType = c("usual", "pearson", "deviance")
  pred.dforest.prob.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "freq", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.freq.xdf <- rxDataStep(XDFout)

  ## compare data.frame and xdf results
  checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf)
  checkEquals(pred.dforest.vote.df, pred.dforest.vote.xdf)
  checkEquals(pred.dforest.prob.df, pred.dforest.prob.xdf)
  checkEquals(pred.dforest.freq.df, pred.dforest.freq.xdf)
  
  ##############
  ## Real-Time scoring
  fit.binary <- fit.dforest.df
  fit.binary$binary <- rxSerializeModel(fit.binary)
  
  for(tt in c("response", "vote", "prob", "freq"))
  {
      rtsCheck(fit.dforest.df, fit.binary, data, type = tt)
  }
}

"test.rxDForest.classification.dataframe.weight.multithread.randomForest" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")
    
    Sys <- Sys.info()
    if ( ( Sys[ "sysname" ] == "Windows" ) && ( Sys[ "machine" ] == "x86" ) ) {
        DEACTIVATED( "Disable on Win32 pending investigation: RA-10676." )
    }
    #Executing test function test.rxDForest.classification.dataframe.weight.multithread.randomForest  ... Timing stopped at: 3.11 0.09 3 
    #Error in checkEquals(vote.rxPredict[c("absent_vote", "present_vote")],  : 
      #Component 1: 'current' is not a factor
    #Component 2: 'current' is not a factor
    #In addition: Warning messages:
    #1: In FUN(X[[2L]], ...) :
      #When using 'factor' in a formula, results are unreliable if 'levels' is not specified.
    #2: In FUN(X[[3L]], ...) :
      #When using 'factor' in a formula, results are unreliable if 'levels' is not specified.
     #done successfully.

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)

    suppressWarnings(randomForest.installed <- require(randomForest, quietly = TRUE))
  if (!randomForest.installed)
  {
        DEACTIVATED("This optional test requires the third-party package 'randomForest', which is not installed on this system. To activate the test, run install.packages('randomForest') and then re-run the test.")
        #install.packages("randomForest")
  }
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    ##NOTE: rxDForest and randomForest deal with missing values differently:
    ##  rxDForest: predicts using surrogate splits and majority rule for splitting
    ##  randomForest: predicts NA if any predictor is missing.
    #kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    #kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    #kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    #kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    #kyphosis.with.weights$Start[7]      <- NA   # 
    #kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a xdf file
    DF <- kyphosis.with.weights

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#NULL		#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  
  ######################################################################
  ## rxPredict.rxForest
    transformVars <- attr(terms(form), "term.labels")
    myRxPredict <- function( dataList )
    {
        myData <- as.data.frame(dataList)
        rxPredict(modelObject = modelObject, data = myData, type = type)
    }
    resp.rxPredict <- rxDataStep(inData = DF, 
        transformFunc = myRxPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "response"))
    vote.rxPredict <- rxDataStep(inData = DF, 
        transformFunc = myRxPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "vote"))
    prob.rxPredict <- rxDataStep(inData = DF, 
        transformFunc = myRxPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "prob"))
    freq.rxPredict <- rxDataStep(inData = DF, 
        transformFunc = myRxPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "freq"))
    
  ## predict.randomForest    
    myPredict <- function( dataList )
    {
        myData <- as.data.frame(dataList)
        object <- as.randomForest(modelObject)
        as.data.frame(predict(object = object, newdata = myData, type = type, norm.votes = FALSE))
    }
    resp.predict <- rxDataStep(inData = DF, 
        transformFunc = myPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "response"))
    vote.predict <- rxDataStep(inData = DF, 
        transformFunc = myPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "vote"))
    prob.predict <- rxDataStep(inData = DF, 
        transformFunc = myPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "prob"))

  ## compare data.frame and xdf results
  checkEquals(vote.rxPredict[c("absent_vote", "present_vote")], vote.predict[c("absent", "present")], checkNames = FALSE)
    ##NOTE: the class predictions might be different for type = "response" in case of ties of votes.
    ##  predict.randomForest breaks ties at random but rxPredict.rxDForest uses the first one for now. 
  checkEquals(resp.rxPredict["Kyphosis_Pred"], resp.predict[ncol(resp.predict)], checkNames = FALSE) 
       
    ##NOTE: prob is computed differently in rxDForest and randomForest:
    ##  rxDForest: the prob is computed as the sum of the predicted tree probs.
    ##  randomForest: the prob is computed as the ratio of the predicted tree votes. 
  #checkEquals(prob.rxPredict[c("absent_prob", "present_prob")], prob.predict[c("absent", "present")], checkNames = FALSE)
  #checkEquals(freq.rxPredict, freq.predict)  # no type = "freq" in randomForest

    ##############
    ## Real-Time scoring
    fit.binary <- fit.dforest.df
    fit.binary$binary <- rxSerializeModel(fit.binary)
    oo <- capture.output(pred.rts <- rxPredict(fit.binary, data = DF))
    pred.default <- rxPredict(fit.dforest.df, DF)

    # Verify we went through the native scoring path    
    checkEquals(1, sum(grepl("Native prediction succeeded", oo)), "Verification that native scoring was used.")

    # Verify the predictions are the same
    checkEquals(pred.rts, pred.default)
}


"test.rxDForest.transformFunc.negative" <- function(){
  
  xform <- function(dataList)
  {
    ret = data.frame(dataList)
    ret = ret[ret$age < 40,,drop=FALSE]
    ret    
  }
  
  infert_mod = tryCatch({
    rxDForest(case ~ parity + education + spontaneous + induced, 
            data=infert, transformFunc=xform, transformVars="age")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxDForest"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxDForest"))
    }
  })
  
}


"test.rxDForest.classification.dataframe.xdffile.noweight" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestG", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    on.exit(if(file.exists(XDF)) file.remove(XDF), add = TRUE)

  ## xdf file with only 1 block
    rxDataStep(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    #rxDataFrameToXdf(DF[1:20,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[21:40,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[41:60,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[61:81,], XDF, append = "rows")
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#NULL		#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  computeOobError <- -1
    
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, #pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, #pweights = "Age",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
    blocksPerRead = 1,	#numBlocks,	#
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  dontmatch <- c("params", "call")
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

    checkEquals(rxTreeDepth(fit.dforest.xdf),  rxTreeDepth(fit.dforest.df))
    checkEquals(rxLeafSize(fit.dforest.xdf),  rxLeafSize(fit.dforest.df))
    
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  if (computeOobError == 1) plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  if (computeOobError == 1) plot(fit.dforest.xdf)
  
  ######################################################################
  ## rxPredict.rxForest
  
  ## use data.frame as input
  XDFtmp <- tempfile( pattern = ".rxDForestH", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
    on.exit(if(file.exists(XDFtmp)) file.remove(XDFtmp), add = TRUE)
        
  pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  pred.dforest.vote.df <- rxPredict(fit.dforest.df, data = DF, type = "vote", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  pred.dforest.prob.df <- rxPredict(fit.dforest.df, data = DF, type = "prob", computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp) #residType = c("usual", "pearson", "deviance")
  pred.dforest.freq.df <- rxPredict(fit.dforest.df, data = DF, type = "freq", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  
  ## use xdf file as input
  XDFout <- tempfile( pattern = ".rxDForestI", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
    on.exit(if(file.exists(XDFout)) file.remove(XDFout), add = TRUE)
    
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.resp.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "vote", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.vote.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "prob", computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, overwrite = TRUE) #residType = c("usual", "pearson", "deviance")
  pred.dforest.prob.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "freq", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.freq.xdf <- rxDataStep(XDFout)

  ## compare data.frame and xdf results
  checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf)
  checkEquals(pred.dforest.vote.df, pred.dforest.vote.xdf)
  checkEquals(pred.dforest.prob.df, pred.dforest.prob.xdf)
  checkEquals(pred.dforest.freq.df, pred.dforest.freq.xdf)
  
  ##############
  ## Real-Time scoring
  fit.binary <- fit.dforest.df
  fit.binary$binary <- rxSerializeModel(fit.binary)
  
  for(tt in c("response", "vote", "prob", "freq"))
  {
      rtsCheck(fit.dforest.df, fit.binary, data, type = tt)
  }
}

###################################
## multi-block xdf

"test.rxDForest.classification.singleThread.singleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestJ", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    }else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#NULL		#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = TRUE, verbose = 0,
    blocksPerRead = 1,	#numBlocks,	#
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

"test.rxDForest.classification.singleThread.multipleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")
  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestK", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

"test.rxDForest.classification.multipleThread.singleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestL", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {   
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
  }
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

"test.rxDForest.classification.multipleThread.multipleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestM", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:20,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[21:40,], outFile = XDF, append = "rows")
        rxDataStep(inData = DF[41:60,], outFile = XDF, append = "rows")
        rxDataStep(inData = DF[61:81,], outFile = XDF, append = "rows")
    }
    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

#######################################################################
## regression

###################################
## data.frame v.s. single-block xdf

"test.rxDForest.regression.dataframe.xdffile.weight" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestN", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
    
  ## xdf file with only 1 block
    rxDataStep(DF, XDF, overwrite = TRUE)    
  ## xdf file with 4 blocks
    #rxDataFrameToXdf(DF[1:20,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[21:40,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[41:60,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[61:81,], XDF, append = "rows")
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
    computeOobError <- 1

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )

  ## compare the forests built on data.frame and xdf file respectively
  dontmatch <- c("params", "call")
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

    checkEquals(rxTreeDepth(fit.dforest.xdf),  rxTreeDepth(fit.dforest.df))
    checkEquals(rxLeafSize(fit.dforest.xdf),  rxLeafSize(fit.dforest.df))
    
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  if (computeOobError == 1) plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  if (computeOobError == 1) plot(fit.dforest.xdf)
  
  ######################################################################
  ## rxPredict.rxForest
  
  ## use data.frame as input
  XDFtmp <- tempfile( pattern = ".rxDForestO", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
  on.exit(if(file.exists(XDFtmp)) unlink(XDFtmp), add = TRUE)

  pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  
  ## use xdf file as input
  XDFout <- tempfile( pattern = ".rxDForestP", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
  on.exit(if(file.exists(XDFout)) unlink(XDFout), add = TRUE)

  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.resp.xdf <- rxDataStep(XDFout)

  ## compare data.frame and xdf results
  checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf)
  
  ##############
  ## Real-Time scoring
  fit.binary <- fit.dforest.df
  fit.binary$binary <- rxSerializeModel(fit.binary)
  
  for(tt in c("response"))
  {
      rtsCheck(fit.dforest.df, fit.binary, data, type = tt)
  }
}

"test.rxDForest.regression.dataframe.xdffile.weight.multithread" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestQ", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)

  ## xdf file with only 1 block
    rxDataStep(DF, XDF, overwrite = TRUE)    
  ## xdf file with 4 blocks
    #rxDataFrameToXdf(DF[1:20,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[21:40,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[41:60,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[61:81,], XDF, append = "rows")
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
    computeOobError <- -1

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )

  ## compare the forests built on data.frame and xdf file respectively
  dontmatch <- c("params", "call")
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

    checkEquals(rxTreeDepth(fit.dforest.xdf),  rxTreeDepth(fit.dforest.df))
    checkEquals(rxLeafSize(fit.dforest.xdf),  rxLeafSize(fit.dforest.df))
    
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  if (computeOobError == 1) plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  if (computeOobError == 1) plot(fit.dforest.xdf)
  
  ######################################################################
  ## rxPredict.rxForest
  
  ## use data.frame as input
  XDFtmp <- tempfile( pattern = ".rxDForestR", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
  on.exit(if(file.exists(XDFtmp)) unlink(XDFtmp), add = TRUE)

  pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  
  ## use xdf file as input
  XDFout <- tempfile( pattern = ".rxDForestS", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
  on.exit(if(file.exists(XDFout)) unlink(XDFout), add = TRUE)

  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
  pred.dforest.resp.xdf <- rxDataStep(XDFout)

  ## compare data.frame and xdf results
  checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf)
  
  ##############
  ## Real-Time scoring
  fit.binary <- fit.dforest.df
  fit.binary$binary <- rxSerializeModel(fit.binary)
  
  for(tt in c("response"))
  {
      rtsCheck(fit.dforest.df, fit.binary, data, type = tt)
  }
}

"test.rxDForest.regression.dataframe.weight.multithread.randomForest" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    suppressWarnings(randomForest.installed <- require(randomForest, quietly = TRUE))
  if (!randomForest.installed)
  {
        DEACTIVATED("This optional test requires the third-party package 'randomForest', which is not installed on this system. To activate the test, run install.packages('randomForest') and then re-run the test.")
        #install.packages("randomForest")
  }    
    
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    ##NOTE: rxDForest and randomForest deal with missing values differently:
    ##  rxDForest: predicts using surrogate splits and majority rule for splitting
    ##  randomForest: predicts NA if any predictor is missing.
    #kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    #kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    #kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    #kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    #kyphosis.with.weights$Start[7]      <- NA   # 
    #kyphosis.with.weights$Age[9]		 <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
    
    ## create a xdf file
    DF <- kyphosis.with.weights
    
    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  
    ######################################################################
    ## rxPredict.rxForest
    transformVars <- attr(terms(form), "term.labels")
    myRxPredict <- function( dataList )
    {
        myData <- as.data.frame(dataList)
        rxPredict(modelObject = modelObject, data = myData, type = type)
    }
    resp.rxPredict <- rxDataStep(inData = DF, 
        transformFunc = myRxPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "response"))
    
    ## predict.randomForest    
    myPredict <- function( dataList )
    {
        myData <- as.data.frame(dataList)
        object <- as.randomForest(modelObject)
        as.data.frame(predict(object = object, newdata = myData, type = type, norm.votes = FALSE))
    }
    resp.predict <- rxDataStep(inData = DF, 
        transformFunc = myPredict, transformVars = transformVars,
        transformObjects = list(modelObject = fit.dforest.df, type = "response"))

    ## compare data.frame and xdf results
    checkEquals(resp.rxPredict["Age_Pred"], resp.predict[ncol(resp.predict)], checkNames = FALSE)

    ##############
    ## Real-Time scoring
    fit.binary <- fit.dforest.df
    fit.binary$binary <- rxSerializeModel(fit.binary)
    oo <- capture.output(pred.rts <- rxPredict(fit.binary, data = DF))
    pred.default <- rxPredict(fit.dforest.df, DF)

    # Verify we went through the native scoring path    
    checkEquals(1, sum(grepl("Native prediction succeeded", oo)), "Verification that native scoring was used.")

    # Verify the predictions are the same
    checkEquals(pred.rts, pred.default)
}

"test.rxDForest.regression.dataframe.xdffile.noweight" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestT", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
    
  ## xdf file with only 1 block
    rxDataStep(DF, XDF, overwrite = TRUE)    
  ## xdf file with 4 blocks
    #rxDataFrameToXdf(DF[1:20,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[21:40,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[41:60,], XDF, append = "rows")
    #rxDataFrameToXdf(DF[61:81,], XDF, append = "rows")
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
    computeOobError <- 1

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, #pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, #pweights = "Kyphosis",
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0, computeOobError = computeOobError,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )

    ## compare the forests built on data.frame and xdf file respectively
    dontmatch <- c("params", "call")
    domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
    sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

    checkEquals(rxTreeDepth(fit.dforest.xdf),  rxTreeDepth(fit.dforest.df))
    checkEquals(rxLeafSize(fit.dforest.xdf),  rxLeafSize(fit.dforest.df))
    
    ######################################################################
    ## {print, plot}.rxDForest
    print(fit.dforest.df)
    if (computeOobError == 1) plot(fit.dforest.df)
  
    print(fit.dforest.xdf)
    if (computeOobError == 1) plot(fit.dforest.xdf)
  
    ######################################################################
    ## rxPredict.rxForest
  
    ## use data.frame as input
    XDFtmp <- tempfile( pattern = ".rxDForestU", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
    on.exit(if(file.exists(XDFtmp)) unlink(XDFtmp), add = TRUE)

    pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp)
  
    ## use xdf file as input
    XDFout <- tempfile( pattern = ".rxDForestV", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
    on.exit(if(file.exists(XDFout)) unlink(XDFout), add = TRUE)

    rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.resp.xdf <- rxDataStep(XDFout)

    ## compare data.frame and xdf results
    checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf)

    ##############
    ## Real-Time scoring
    fit.binary <- fit.dforest.df
    fit.binary$binary <- rxSerializeModel(fit.binary)
    
    for(tt in c("response"))
    {
        rtsCheck(fit.dforest.df, fit.binary, data, type = tt)
    }
}

###################################
## multi-block xdf

"test.rxDForest.regression.singleThread.singleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestW", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Kyphosis",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

"test.rxDForest.regression.singleThread.multipleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestX", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Kyphosis",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = 1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

"test.rxDForest.regression.multipleThread.singleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestY", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Kyphosis",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

"test.rxDForest.regression.multipleThread.multipleFile" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]		<- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestZ", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
  on.exit(if(file.exists(XDF)) unlink(XDF), add = TRUE)
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else 
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
    
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Age ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#1			#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0

    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
    fit.dforest.xdf <- rxDForest(
        formula = form, data = XDF, pweights = "Kyphosis",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = TRUE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = 1,	#numBlocks,	#
    )
    
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))
  
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
}

#######################################################################
## bug fixes

#bigAnalytics RA-11308
#When HPA functions that need to write to disk run in local parallel under rxExec, 
#they can try to write to the same file causing R to crash.
"test.rxDForest.classification.RA-11308.scheduleOnce.localpar" <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  
    ## setup
    claims.dir <- rxGetOption("sampleDataDir")
    claims.xdf <- file.path(claims.dir, "claims4blocks.xdf")
    #rxGetInfo(claims.xdf, getVarInfo = TRUE, getBlockSizes = TRUE)
    
    oldCC <- rxSetComputeContext("localpar")
    on.exit(rxSetComputeContext(oldCC), add = TRUE)
    
    ## train
    seed <- 0
    claims.form <- age ~ car.age + type + cost + number    # classification

    ##NOTE: Rng from MKL is used for both sampling observations and breaking ties when computing OOB error.
    ##  So if computeOobError is set differently, the following results will not match when there are ties. 
    claims.dforest.allowDiskWriteT <- rxDForest(claims.form, data = claims.xdf, 
        allowDiskWrite = TRUE, computeOobError = 1,
        scheduleOnce = TRUE, seed = seed
        #, reportProgress = 1, verbose = 2
        )
    claims.dforest.allowDiskWriteF <- rxDForest(claims.form, data = claims.xdf, 
        allowDiskWrite = FALSE, computeOobError = 1,
        scheduleOnce = TRUE, seed = seed
        #, reportProgress = 1, verbose = 2
        )
    
    ## compare results
    checkEquals(claims.dforest.allowDiskWriteT$forest, claims.dforest.allowDiskWriteF$forest)
    
    ## do prediction
    tstamp <- as.numeric(Sys.time())
    XDFtmp <- tempfile( pattern = paste0("rxDForestB", tstamp), fileext = ".xdf")
    if (file.exists(XDFtmp)) unlink(XDFtmp, recursive = TRUE, force = TRUE)
    on.exit(if(file.exists(XDFtmp)) unlink(XDFtmp, recursive = TRUE, force = TRUE), add = TRUE)
    
    ## use xdf file as input
    XDF <- claims.xdf
    XDFout <- tempfile( pattern = paste0("rxDForestC", tstamp), fileext = ".xdf")
    if (file.exists(XDFout)) unlink(XDFout, recursive = TRUE, force = TRUE)
    on.exit(if(file.exists(XDFout)) unlink(XDFout, recursive = TRUE, force = TRUE), add = TRUE)

    # tree-by-tree prediction 
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "response", 
        scheduleOnce = FALSE,
        computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.resp.xdf <- rxDataStep(XDFout)
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "vote", 
        scheduleOnce = FALSE,
        computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.vote.xdf <- rxDataStep(XDFout)
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "prob", 
        scheduleOnce = FALSE,
        computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, overwrite = TRUE) #residType = c("usual", "pearson", "deviance")
    pred.dforest.prob.xdf <- rxDataStep(XDFout)
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "freq", 
        scheduleOnce = FALSE,
        computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.freq.xdf <- rxDataStep(XDFout)
    
    # single-pass prediction
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "response", 
        scheduleOnce = 2,
        computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.resp.xdf2 <- rxDataStep(XDFout)
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "vote", 
        scheduleOnce = 2,
        computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.vote.xdf2 <- rxDataStep(XDFout)
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "prob", 
        scheduleOnce = 2,
        computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, overwrite = TRUE) #residType = c("usual", "pearson", "deviance")
    pred.dforest.prob.xdf2 <- rxDataStep(XDFout)
    rxPredict(claims.dforest.allowDiskWriteT, data = XDF, outData = XDFout, type = "freq", 
        scheduleOnce = 2,
        computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE)
    pred.dforest.freq.xdf2 <- rxDataStep(XDFout)
    
    ## compare data.frame and xdf results
    checkEquals(pred.dforest.resp.xdf, pred.dforest.resp.xdf2)
    checkEquals(pred.dforest.vote.xdf, pred.dforest.vote.xdf2)
    checkEquals(pred.dforest.prob.xdf, pred.dforest.prob.xdf2)
    checkEquals(pred.dforest.freq.xdf, pred.dforest.freq.xdf2)
}

#bigAnalytics RA-9803
#rxDTree with outFile specified throws an error when rxOptions() outDataPath is set
"test.rxDForest.RA-9803.outDataPath.outFile" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  iris.xdf <- tempfile("iris", , ".xdf")
  if( file.exists( iris.xdf ) ) file.remove( iris.xdf ) 
  on.exit(if (file.exists(iris.xdf)) file.remove(iris.xdf), add = TRUE)

    if (rxIsExpressEdition())
    {
        rxDataStep(inData = iris, outFile = iris.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(iris[1:100,], iris.xdf, overwrite=TRUE) 
      rxDataStep(iris[101:150,], iris.xdf, append="rows", overwrite=TRUE)
    }
  
  tmpDir <- tempdir()
  tmpFile <- tempfile("rxDForest", tmpDir, ".xdf")
  outFile <- basename(tmpFile)

  if (file.exists(tmpFile)) file.remove(tmpFile)
  on.exit(if (file.exists(tmpFile)) file.remove(tmpFile), add = TRUE)

  outDataPath.old <- rxGetOption("outDataPath")
  rxOptions(outDataPath = c(tmpDir, outDataPath.old))
  on.exit(rxOptions(outDataPath = outDataPath.old), add = TRUE)

  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width
  iris.dforest <- rxDForest(form, data = iris.xdf, outFile = outFile, nTree = 3, maxDepth = 2,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  ##NOTE: if data is a data.frame, we will not output any file even if outFile is specified.

  # The following indicate that the outFile gets written to the proper place 
  checkTrue(file.exists(tmpFile))
}

#bigAnalytics RA-9718
#rxDTree is overwriting rather than appending to the contents of its outFile
"test.rxDForest.outFile.bSameInandOutFile" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    }  else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if(file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- data.xdf

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.xdf, outFile = outFile,
    maxDepth = 10, nTree = 3,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 3 more cols (i.e., ".rxFWeight", "___nodeid", ".rxRowSelection") will be generated.
  predColNames <- c(paste("Votes", levels(iris$Species), sep = ""), "Predicted", "OOBTimes")
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), predColNames) %in% names(outFile.df)))
}

"test.rxDForest.outFile.nonempty" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
      rxDataStep(data.df[1:150, ], data.xdf, overwrite = TRUE) 
    }  else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)  
    }
     
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  rxDataStep( data.xdf, outFile)
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.xdf, outFile = outFile,
    maxDepth = 10, nTree = 3,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 3 more cols (i.e., ".rxFWeight", "___nodeid", ".rxRowSelection") will be generated.	
  predColNames <- c(paste("Votes", levels(iris$Species), sep = ""), "Predicted", "OOBTimes")
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), predColNames) %in% names(outFile.df)))
}

"test.rxDForest.outFile.empty" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.xdf, outFile = outFile,
    maxDepth = 10, nTree = 3,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 3 more cols (i.e., ".rxFWeight", "___nodeid", ".rxRowSelection") will be generated.
  predColNames <- c(paste("Votes", levels(iris$Species), sep = ""), "Predicted", "OOBTimes")
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(predColNames %in% names(outFile.df)))
}

#bigAnalytics RA-10012
#rxDForest is ignoring "writeModelVars = TRUE" and not writing the model variables to its output file
"test.rxDForest.outFile.bSameInandOutFile.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  
  on.exit(if(file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- data.xdf

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.xdf, outFile = outFile,
    maxDepth = 10, nTree = 3, writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 3 more cols (i.e., ".rxFWeight", "___nodeid", ".rxRowSelection") will be generated.
  predColNames <- c(paste("Votes", levels(iris$Species), sep = ""), "Predicted", "OOBTimes")
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), predColNames) %in% names(outFile.df)))
}

"test.rxDForest.outFile.nonempty.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile)
  varsToKeep <- c("x1", "x2")
  outdata.df <- as.data.frame(matrix(runif(150 * 2), ncol = 2, dimnames = list(NULL, varsToKeep)))
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = outdata.df, outFile = outFile, overwrite = TRUE)
    } else
    {
      rxDataStep(outdata.df[1:100, ], outFile, overwrite = TRUE) 
      rxDataStep(outdata.df[101:150, ], outFile, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.xdf, outFile = outFile,
    maxDepth = 1, nTree = 3, writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = 1e6,  #if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 3 more cols (i.e., ".rxFWeight", "___nodeid", ".rxRowSelection") will be generated.	
  modelVars <- as.character(attr(terms(form), "variables"))[-1]
  predColNames <- c(paste("Votes", levels(iris$Species), sep = ""), "Predicted", "OOBTimes")
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(c(varsToKeep, modelVars, predColNames), predColNames) %in% names(outFile.df)))
}

"test.rxDForest.outFile.empty.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.xdf, outFile = outFile,
    maxDepth = 10, nTree = 3, writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 3 more cols (i.e., ".rxFWeight", "___nodeid", ".rxRowSelection") will be generated.
  modelVars <- as.character(attr(terms(form), "variables"))[-1]
  predColNames <- c(paste("Votes", levels(iris$Species), sep = ""), "Predicted", "OOBTimes")
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(modelVars, predColNames) %in% names(outFile.df)))
}

#bigAnalytics RA-10017
#rxVarImpPlot( rxDForestObject ) does not show the variable names in the plot
"test.rxDForest.importance" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  
  ## create input data
  data.df <- iris

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dforest <- rxDForest(form, data = data.df,
    maxDepth = 10, nTree = 3, importance = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  #library(randomForest)
  #iris.rforest <- randomForest(form, data = data.df,
    #ntree = 3, importance = TRUE
    #)
  #iris.importance.rforest <- iris.rforest$importance
  
  ## check results
  iris.importance <- rxVarImpPlot( iris.dforest )
  checkTrue(all(names( iris.importance ) %in% names(data.df)))
}

#bigAnalytics RA-9419
#rxPredict not playing nicely with rxDTree and rxDForest
"test.rxDForest.rxPredict.rxDTree" <- function()
{
    #Sys <- Sys.info()
    #if ( ( Sys[ "sysname" ] == "Windows" ) && ( Sys[ "machine" ] == "x86" ) ) {
        #DEACTIVATED( "Disable on Win32 pending further info as to why it is breaking the build." )
    #}

  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf")
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE)
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.dtree <- rxDTree(form, data = data.xdf, control = control,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  iris.dforest <- rxDForest(form, data = data.xdf, maxDepth = 10, nTree = 3, 
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## do prediction and checking
  predColNames <- c(names(data.df), "Species_Pred")
  nrow.data.df <- nrow(data.df)
    
  rxPredict(iris.dtree, data = data.xdf, type = "class", overwrite = TRUE)
  pred.dtree <- rxDataStep(data.xdf)
  checkEquals(names(pred.dtree), predColNames)
    checkEquals(nrow(pred.dtree), nrow.data.df)

  rxPredict(iris.dforest, data = data.xdf, type = "class", overwrite = TRUE)
  pred.dforest <- rxDataStep(data.xdf)
  checkEquals(names(pred.dforest), predColNames)
    checkEquals(nrow(pred.dforest), nrow.data.df)
}

## rxPredict.rxDForest(writeModelVars = TRUE) for multiblock xdf input files. 
"test.rxDForest.classification.RA-9514.writeModelVars.multiblock.xdf" <- function()
{
    if(exists(".deactivate.rxDForest.tests") && .deactivate.rxDForest.tests) DEACTIVATED("rxDForest not complete")

    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
  
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
  
    ## introduce unordered factors
    kyphosis.with.weights$Strata <- factor(kyphosis$Age >= 7*12, labels = c("yrs7-", "yrs7+"))
  
    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDForestW", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    on.exit(if (file.exists(XDF)) file.remove(XDF), add = TRUE)
    
  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
  
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
    #DF.my <- rxDataStep(XDF)

    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 2, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
  ## forest args
  nTree		<- 3		#1			#
  mTry		<- 2		#NULL		#
  sampRate	<- 0.5		#1			#
  replace		<- FALSE	#TRUE		#
  strata		<- NULL		#"Strata"	#
  cutoff		<- NULL
  seed		<- 0
  
    ## build the models
    fit.dforest.df <- rxDForest(
        formula = form, data = DF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  ## NOTE: Depending on the input data type (data.frame vs. xdf file), different threading model will be used.
  ## If RNG is used on threads to break ties at random (for classification), the results might not be identical.
    fit.dforest.xdf <- rxDForest(
        #formula = form, data = XDF, pweights = "Age",
        formula = form, data = DF, pweights = "Age",
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100, importance = TRUE,
    nTree = nTree, mTry = mTry, sampRate = sampRate, replace = replace, strata = strata, cutoff = cutoff, seed = seed,
    numThreads = -1, oneFilePerTree = FALSE, verbose = 0,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    blocksPerRead = numBlocks,
    )
  
  ## compare the forests built on data.frame and xdf file respectively
  ##NOTE: the forest built on data.frame and multiblock will be different
  ##	unless we set blocksPerRead to the number of blocks in the xdf file
  ##	(i.e., we read the entire multi-block xdf all at once into memory)
  ##	as we sample only within the blocks that we read into the memory.
  dontmatch <- c("params", "call")
  #dontmatch <- c(dontmatch, c("forest", "oob.err", "confusion", "importance"))	# extra dontmatchs due to randomoization within blocks read into memory
  domatch <- setdiff(names(fit.dforest.xdf), dontmatch)
    # This seems to timeout on some machines
  sapply(domatch, function(nm) checkIdentical(fit.dforest.xdf[[nm]], fit.dforest.df[[nm]]))

    # Try taking it out of sapply
  #for (i in 1:length(domatch))
  #{
  #    checkIdentical(fit.dforest.xdf[domatch[i]],fit.dforest.df[domatch[i]])
#
  #}
    #print("DEBUGGING FAILING UNIT TEST")
  #for (i in 1:length(domatch))
  #{
      #print(paste("Element: ", i))
        #print(fit.dforest.xdf[domatch[i]])
      #print(fit.dforest.df[domatch[i]])
  #}
  #
  ######################################################################
  ## {print, plot}.rxDForest
  print(fit.dforest.df)
  plot(fit.dforest.df)
  
  print(fit.dforest.xdf)
  plot(fit.dforest.xdf)
  
  ######################################################################
  ## rxPredict.rxForest
  
  ## use data.frame as input
  XDFtmp <- tempfile( pattern = ".rxDForestZ", fileext = ".xdf")
    if (file.exists(XDFtmp)) file.remove(XDFtmp)
    on.exit(if (file.exists(XDFtmp)) file.remove(XDFtmp), add = TRUE)

  pred.dforest.resp.df <- rxPredict(fit.dforest.df, data = DF, type = "response", 
    computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, writeModelVars = TRUE)
  pred.dforest.vote.df <- rxPredict(fit.dforest.df, data = DF, type = "vote", 
    computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, writeModelVars = TRUE)
  pred.dforest.prob.df <- rxPredict(fit.dforest.df, data = DF, type = "prob", 
    computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, writeModelVars = TRUE)
  pred.dforest.freq.df <- rxPredict(fit.dforest.df, data = DF, type = "freq", 
    computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, writeModelVars = TRUE)
  
  ## use xdf file as input
  XDFout <- tempfile( pattern = ".rxDForest1A", fileext = ".xdf")
    if (file.exists(XDFout)) file.remove(XDFout)
    on.exit(if (file.exists(XDFout)) file.remove(XDFout), add = TRUE)

  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "response", 
    computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE, writeModelVars = TRUE)
  pred.dforest.resp.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "vote", 
    computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE, writeModelVars = TRUE)
  pred.dforest.vote.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "prob", 
    computeResiduals = TRUE, residType = "pearson", tempFileName = XDFtmp, overwrite = TRUE, writeModelVars = TRUE)
  pred.dforest.prob.xdf <- rxDataStep(XDFout)
  rxPredict(fit.dforest.xdf, data = XDF, outData = XDFout, type = "freq", 
    computeResiduals = TRUE, residType = "usual", tempFileName = XDFtmp, overwrite = TRUE, writeModelVars = TRUE)
  pred.dforest.freq.xdf <- rxDataStep(XDFout)

  ## compare data.frame and xdf results
  checkEquals(pred.dforest.resp.df, pred.dforest.resp.xdf[colnames(pred.dforest.resp.df)])
  checkEquals(pred.dforest.vote.df, pred.dforest.vote.xdf[colnames(pred.dforest.vote.df)])
  checkEquals(pred.dforest.prob.df, pred.dforest.prob.xdf[colnames(pred.dforest.prob.df)])
  checkEquals(pred.dforest.freq.df, pred.dforest.freq.xdf[colnames(pred.dforest.freq.df)])
}

# manage expression-like input arguments: transforms and rowSelection.
"test.rxDForest.classification.RA-9532.transforms.rowSelection" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  seed <- 777
  
  ###############################
  ## transforms
  
  ## without transform
  kyphosisLogical <- kyphosis
  kyphosisLogical$KyphosisLogical <- unclass(kyphosis$Kyphosis)==2

  formLogical <- KyphosisLogical ~ Age + Number + Start
  logical.dforest <- rxDForest(formLogical, data = kyphosisLogical, method = "class", seed = seed,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## with transforms
  formFactor <- KyphosisFactor ~ Age + Number + Start
  factor.dforest <- rxDForest(formFactor, data = kyphosisLogical, seed = seed,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    transforms = list(KyphosisFactor = factor(as.integer(KyphosisLogical), levels = c("0","1"))))
  
  checkEqualsNumeric(logical.dforest$confusion, factor.dforest$confusion)
  
  ###############################
  ## rowSelection
  
  kyphosisBinary <- kyphosis
  kyphosisBinary$KyphosisBinary <- as.integer(unclass(kyphosis$Kyphosis) - 1)
  formBinary <- KyphosisBinary ~ Age + Number + Start
  
  ## with rowSelection
  set.seed(seed)
  dforest.rowSelection <- rxDForest(formBinary, method = "class",
    data = kyphosisBinary, rowSelection = (Age > 10),
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## with subsetting
  set.seed(seed)
  dforest.subset <- rxDForest(formBinary, method = "class",
    data = kyphosisBinary[kyphosisBinary$Age > 10, ],
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  #checkTrue(dforest.subset$oob.err$num.oob[1] >= dforest.rowSelection$oob.err$num.oob[1])
}

"test.rxDForest.classification.RA-10491.transforms.rxPredict" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  
  ## setup
  data.dir <- rxGetOption("sampleDataDir")
  data.ds <- file.path(data.dir, "claims4blocks.xdf")
  #rxGetInfo(data.ds, getVarInfo = TRUE, getBlockSizes = TRUE)
  
  ## train
  formLogical <- ageLogical ~ car.age + type + cost + number    # classification
  logical.dforest <- rxDForest(formLogical, data = data.ds, #method = "class", #regression
    seed = 777, nTree = 2, maxDepth = 2,
    transforms = list( ageLogical = (unclass(age)==2) ),
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## predict
  outFile <- file.path(tempdir(), "logical.dforest.predict.xdf")
  if ( file.exists( outFile ) ) file.remove( outFile )
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)
  
  rxPredict( logical.dforest, data = data.ds, outData = outFile, 
    overwrite = TRUE, writeModelVars = TRUE) 
  outFile.info <- rxGetInfo(outFile, getVarInfo = TRUE, getBlockSizes = TRUE, numRows = 5)

  ## compare results
  checkTrue(outFile.info$numVars == 7)
}

# RA-9533: classification (method="class") with logical (FALSE/TRUE) and binary (integer 0/1) response
# RA-9610: rxPredict.rxDForest(type = "class") should return a factor for logical or binary response
# RA-10119 and RA-10120: rxPredict.rxDForest fails due to integral factor levels (and thus integral column names)
"test.rxDForest.classification.RA-9533.logical.binary.response" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  seed <- 777
  
  ## logical (FALSE/TRUE) response
  kyphosisLogical <- kyphosis
  kyphosisLogical$KyphosisLogical <- unclass(kyphosis$Kyphosis)==2

  formLogical <- KyphosisLogical ~ Age + Number + Start
  set.seed(seed)
  logical.dforest <- rxDForest(formLogical, data = kyphosisLogical, method = "class")
  logical.dtree <- rxDTree(formLogical, data = kyphosisLogical, method = "class",
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## binary (integer 0/1) response
  kyphosisBinary <- kyphosis
  kyphosisBinary$KyphosisBinary <- as.integer(unclass(kyphosis$Kyphosis) - 1)
  formBinary <- KyphosisBinary ~ Age + Number + Start
  
  DF <- kyphosisBinary
    XDF <- tempfile( pattern = ".rxDForest", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    on.exit(if (file.exists(XDF)) file.remove(XDF), add = TRUE) 
       
  XDFpred <- tempfile( pattern = ".rxDForest.Pred", fileext = ".xdf")
    if (file.exists(XDFpred)) file.remove(XDFpred)
    on.exit(if (file.exists(XDFpred)) file.remove(XDFpred), add = TRUE)

  ## xdf file with only 1 block
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    
  ### xdf file with 2 blocks
    #rxDataFrameToXdf(DF[1:40,], XDF, overwrite = TRUE)
    #rxDataFrameToXdf(DF[41:81,], XDF, append = "rows")
  ## xdf file with 4 blocks
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:20,], XDF, overwrite = TRUE)
        rxDataStep(DF[21:40,], XDF, append = "rows")
        rxDataStep(DF[41:60,], XDF, append = "rows")
        rxDataStep(DF[61:81,], XDF, append = "rows")
    }
      
    numBlocks <- rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)$numBlocks
  
  set.seed(seed)
  ## NOTE: Depending on the input data type (data.frame vs. xdf file), different threading model will be used.
  ## If RNG is used on threads to break ties at random (for classification), the results might not be identical.
  #binary.dforest <- rxDForest(formBinary, data = XDF, method = "class",
  binary.dforest <- rxDForest(formBinary, data = kyphosisBinary, method = "class",
    blocksPerRead = numBlocks,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  binary.dtree <- rxDTree(formBinary, data = XDF, method = "class", 
    blocksPerRead = numBlocks,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## compare results
  checkEqualsNumeric(logical.dforest$confusion, binary.dforest$confusion)
  dontmatch <- c("where", "call", "cptable", "params", "formula")
  domatch <- setdiff(names(logical.dtree), dontmatch)
  sapply(domatch, function(nm) checkIdentical(logical.dtree[[nm]], binary.dtree[[nm]]))
  
  ## rxPredict
  pred.dforest.logical <- rxPredict(logical.dforest, data = kyphosisLogical, type = "class")
  rxPredict(binary.dforest, data = XDF, outData = XDFpred, type = "class", overwrite = TRUE, writeModelVars = TRUE)
  pred.dforest.binary <- rxDataStep(XDFpred)
  checkTrue(is.factor(pred.dforest.logical$KyphosisLogical_Pred) && is.factor(pred.dforest.binary$KyphosisBinary_Pred))
  checkIdentical(as.numeric(pred.dforest.logical$KyphosisLogical_Pred), as.numeric(pred.dforest.binary$KyphosisBinary_Pred))
  
  pred.dtree.logical <- rxPredict(logical.dtree, data = kyphosisLogical, type = "class")
  rxPredict(binary.dtree, data = XDF, outData = XDFpred, type = "class", overwrite = TRUE, writeModelVars = TRUE)
  pred.dtree.binary <- rxDataStep(XDFpred)
  checkTrue(is.factor(pred.dtree.logical$KyphosisLogical_Pred) && is.factor(pred.dtree.binary$KyphosisBinary_Pred))
  checkIdentical(as.numeric(pred.dtree.logical$KyphosisLogical_Pred), as.numeric(pred.dtree.binary$KyphosisBinary_Pred))

    ## cleanup
    file.remove(XDF)
    file.remove(XDFpred)
}

#bigAnalytics RA-9927
#Suspected difference in parallel random number generator on MKL between Windows and Linux versions (as exhibited in rxDForest tests)

## MKL RNG Bug 6000037558: The random number generators in MKL/VSL are platform-dependent when the seed >= 2^31.
"test.rxDForest.cross.platform.seed" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    mydata <- file.path(rxGetOption("sampleDataDir"),"claims4blocks.xdf")
    #mydata <- rxDataStep(mydata)
    
    #mydata <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    # Express edition should be the same as a single block file
    
    form <- age ~ cost + number # classification
    
    #NOTE: results will most likely be different for different numThreads
    fit.rxDForest <- rxDForest(form, data = mydata, 
        minSplit = 5, maxDepth = 2, maxNumBins = 200, 
        seed = -17, nTree = 2, numThreads = 1,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        findSplitsInParallel = FALSE,    # hidden arg
        )   
    res <- fit.rxDForest$confusion
    
    #ans <- structure(c(4, 5, 1, 5, 1, 0, 2, 3, 6, 0, 0, 0, 0, 1, 0, 0, 0,
        #0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 4, 4, 4, 2, 6,
        #5, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 3, 3, 4, 2, 3, 2, 0, 0, 0,
        #0, 0, 0, 0, 0, 0.666666666666667, 1, 1, 1, 0.714285714285714,
        #1, 0.7, 1
        #), .Dim = 8:9, .Dimnames = structure(list(
            #age = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"),
            #Predicted = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+", "class.error")
            #), .Names = c("age", "Predicted"))
        #)
    
    ans <- structure(c(5, 1, 1, 0, 1, 0, 2, 1, 4, 1, 1, 1, 1, 0, 1, 2, 0,
        0, 0, 0, 0, 0, 0, 0, 1, 4, 4, 6, 4, 5, 5, 1, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 3, 3, 0, 1, 0, 1, 4, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 2,
        1, 2, 4, 1, 1, 0.545454545454545, 0.9, 1, 0.25, 1, 1, 0.909090909090909, 0.888888888888889
        ), .Dim = 8:9, .Dimnames = structure(list(
            age = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"),
            Predicted = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+", "class.error")
            ), .Names = c("age", "Predicted"))
        )
    
    ansExpress <- structure(c(4, 5, 1, 4, 1, 1, 1, 1, 
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0,
        2, 1, 4, 4, 4, 6, 5, 11,
        0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 
        0, 3, 6, 3, 0, 1, 3, 1,
        0.3333333333, 1, 1, 1, .2, 1, 1, .923076923077      
        ), .Dim = 8:9, .Dimnames = structure(list(
            age = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"),
            Predicted = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+", "class.error")
            ), .Names = c("age", "Predicted"))
       )
    if (rxIsExpressEdition())
    {
        checkEquals(res, ansExpress)
    }
    else
    {
        checkEquals(res, ans)
    }
    
    
    ###################################################################
    fit.rxDForest.findSplitsInParallel <- rxDForest(form, data = mydata, 
        minSplit = 5, maxDepth = 2, maxNumBins = 200, 
        seed = -17, nTree = 2, numThreads = 1,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        findSplitsInParallel = TRUE,    # hidden arg
        )   
    res.findSplitsInParallel <- fit.rxDForest.findSplitsInParallel$confusion
        
    ans.findSplitsInParallel <- structure(c(2, 1, 1, 0, 1, 0, 2, 1, 2, 1, 1, 1, 1, 0, 1, 1, 0,
        0, 0, 0, 0, 0, 0, 0, 4, 5, 4, 5, 4, 4, 5, 5, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 2, 3, 0, 1, 0, 1, 2, 2, 0, 0, 0, 0, 0, 0, 0, 1, 1, 2,
        2, 2, 6, 2, 0, 0.818181818181818, 0.9, 1, 0.375, 1, 1, 1, 1
        ), .Dim = 8:9, .Dimnames = structure(list(
            age = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"),
            Predicted = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+", "class.error")
            ), .Names = c("age", "Predicted"))
        )
    
    ansExpress.findSplitsInParallel <- structure(c(
       5, 6, 3, 4, 4, 3, 2, 7,
       0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 1, 2, 0, 4, 3, 4,
       0, 0, 0, 0, 0, 0, 0, 0,  
       0, 0, 0, 0, 0, 0, 0, 0,
       1, 3, 7, 5, 1, 1, 4, 2,
       0.166666666667, 1, 1, 1, 1, 1, 1, 0.846153846154
        ), .Dim = 8:9, .Dimnames = structure(list(
            age = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"),
            Predicted = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+", "class.error")
            ), .Names = c("age", "Predicted"))
        )    
    
    if (rxIsExpressEdition())
    {
        checkEquals(res.findSplitsInParallel, ansExpress.findSplitsInParallel)
    }
    else
    {
        checkEquals(res.findSplitsInParallel, ans.findSplitsInParallel)
    }
    
}

## MKL RNG Bug 6000037527: The random number generators in MKL/VSL are platform-dependent when a vector of random numbers are produced.
"test.rxDForest.cross.platform.vector" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

    airDataFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
    form <- ArrDelay ~ CRSDepTime + DayOfWeek
    
    # For Express, should be identical to
    # airDataFile <- rxDataStep(airDataFile)
    
  # NOTE: batchLength must be in [1, 15] for the current test to get reproducible behavior on Windows and Linux
  batchLength <- 1    #sample.int(15, 1)    
    fit.rxDForest <- rxDForest(form, data = airDataFile,
        seed = 0, nTree = 2, maxDepth = 2, numThreads = 1, batchLength = batchLength,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        ) 
    oob.err.res <- fit.rxDForest$oob.err
    
    oobExpress.err.ans <- structure(list(
        oob.err = c(1669.46423340, 1662.13659668),
        num.oob = c(214294, 349822),
        oob.var = c(1671.87243652, 1664.58081055),
        oob.rsq = c(0.00144042276934, 0.00146836600044)
        ), .Names = c("oob.err", "num.oob", "oob.var", "oob.rsq"), row.names = c(NA, -2L), class = "data.frame")
    
    
    oob.err.ans <- structure(list(
        oob.err = c(1649.2939453125, 1640.09802246094),
        num.oob = c(214351, 349264),
        oob.var = c(1652.00354003906, 1642.98022460938),
        oob.rsq = c(0.00164018699772184, 0.00175425248902361)
        ), .Names = c("oob.err", "num.oob", "oob.var", "oob.rsq"), row.names = c(NA, -2L), class = "data.frame")
    
    if (rxIsExpressEdition())
    {
        checkEquals(oob.err.res, oobExpress.err.ans)
    }
    else
    {
        checkEquals(oob.err.res, oob.err.ans)
    }
    
}

"test.rxPredict.rxDForest.parameters" <- function()
{
    predVarNames <- "mtCars_out"
    checkEquals(rxGetVarNames(rxPredict(rxDForest(am ~ cyl + mpg, data = mtcars), mtcars, predVarNames = predVarNames)), predVarNames)
}