# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxExecBy
#
############################################################################################

".setUp" <- function()
{
}

".tearDown" <- function()
{
}

# Test base case
"test.rxExecBy.base" <- function() {
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
    
    # Setup the input and output data paths.
    inFile <- "claims.txt"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

    # create input data source
    inTxtDataSource <- RxTextData(file = inFile, delimiter = ",")

    # user function
    ".linMod" <- function(data, keys)
    {
      result <- rxLinMod(formula = cost ~ number, data = data)
      return(result$coefficients[[1]])
    }

    #local compute context
    rxSetComputeContext("localpar")
    rxOptions(numCoresToUse = 4)

    varsToPartition <- c("car.age", "type")
    localCCResults <- rxExecBy(inData = inTxtDataSource, keys = varsToPartition, func = .linMod)

    checkEquals(length(localCCResults), 16)
    results <- do.call(rbind, lapply(localCCResults, unlist))

    checkEquals(max(as.integer(results[, 3])), 553)
    checkEquals(min(as.integer(results[, 3])), 94)
}


# function to verify the total number of rows
".verifyRowCount" <- function(partList, expectedNumParts, expectedTotalRows)
{
  checkEquals(length(partList), expectedNumParts)

  oldReportProgress <- rxGetOption("reportProgress")
  rxOptions(reportProgress = 0)
  on.exit(rxOptions(reportProgress = oldReportProgress), add=TRUE)

  results <- lapply(partList, unlist)
  totalrows <- 0
  for (i in 1:length(results))
  {
    partDataSet <- rxImport(inData = results[[i]])
    nrowPart <- nrow(partDataSet)
    totalrows <- totalrows + nrowPart
  }
  checkEquals(totalrows, expectedTotalRows)
}


"test.rxExecBy.rxPartition" <- function() {
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
  # Setup the input and output data paths.
  inFile <- "claims.xdf"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)

  # Remove any artifacts from a pervious test.
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE) , add = TRUE)

  # create input and output Xdf data source
  inXdfDataSource <- RxXdfData(file = inFile)
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

  # user function
  ".linMod" <- function(data, keys)
  {
    result <- rxLinMod(formula = cost ~ number, data = data)
    return(result$coefficients[[1]])
  }

  #local compute context
  rxSetComputeContext("localpar")
  rxOptions(numCoresToUse = 4)

  # create a partitioned Xdf with rxPartition
  varsToPartition <- c("car.age", "type")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, overwrite = TRUE)

  .verifyRowCount(partDF$DataSource, 16, 128)

  # feed the partitioned Xdf Data Source for rxExecBy
  localCCResults <- rxExecBy(inData = outXdfDataSource, keys = varsToPartition, func = .linMod)

  checkEquals(length(localCCResults), 16)
  results <- do.call(rbind, lapply(localCCResults, unlist))

  checkEquals(max(as.integer(results[, 3])), 553)
  checkEquals(min(as.integer(results[, 3])), 94)

  # call rxGetPartitions and verify that outXdfDataSource didn't get changed by rxExecBy
  partDF <- rxGetPartitions(inData = outXdfDataSource)
  .verifyRowCount(partDF$DataSource, 16, 128)

  # user function
  ".Count" <- function(keys, data)
  {
    myDF <- rxImport(inData = data)
    return(nrow(myDF))
  }

  # call rxExecBy again with a different user function
  localCCResults <- rxExecBy(inData = outXdfDataSource, keys = varsToPartition, func = .Count)
  checkEquals(length(localCCResults), 16)
  results <- do.call(rbind, lapply(localCCResults, unlist))

  checkEquals(max(as.integer(results[, 3])), 8)
  checkEquals(min(as.integer(results[, 3])), 8)

  # call rxExecBy again with a different set of keys, rxExecBy should fail
  try(localCCResults <- rxExecBy(inData = outXdfDataSource, keys = c("age", "type"), func = .Count), silent = TRUE)

  # call rxExecBy again with keys = NULL
  localCCResults <- rxExecBy(inData = outXdfDataSource, keys = NULL, func = .Count)
  checkEquals(length(localCCResults), 16)
  results <- do.call(rbind, lapply(localCCResults, unlist))

  checkEquals(max(as.integer(results[, 3])), 8)
  checkEquals(min(as.integer(results[, 3])), 8)

  # delete outFile and rxExecBy should fail
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  err <- try(localCCResults <- rxExecBy(inData = outXdfDataSource, keys = varsToPartition, func = .Count), silent = TRUE)
  checkEquals(inherits(err, "try-error"), TRUE)
}

"test.rxExecBy.TextDataSource" <- function() {
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
  
  # setup output data source
  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)

  # user function
  ".linMod" <- function(data, keys)
  {
    result <- rxLinMod(formula = ArrDelayNew ~ DayOfWeek, data = data)
    return(result$coefficients[[1]])
  }

  # cleanup
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE) , add = TRUE)

  # Txt data source
  inFile <- "AirlineDemoSmall.csv"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
  inTxtDS <- RxTextData(file = inFile, colInfo = list(ArrDelay = list(type = "numeric", newName = "ArrDelayNew"), DayOfWeek = list(type = "factor")))

  localCCResults <- rxExecBy(inData = inTxtDS, keys = c("DayOfWeek"), func = .linMod)

  checkEquals(length(localCCResults), 7)
  results <- do.call(rbind, lapply(localCCResults, unlist))
  checkEquals(max(as.integer(results[, 2])), 14)
  checkEquals(min(as.integer(results[, 2])), 8)
}


"test.rxExecBy.filterFunc" <- function() {
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
  # Setup the input and output data paths.
  inFile <- "claims.xdf"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)

  # Remove any artifacts from a pervious test.
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE) , add = TRUE)

  # create input and output Xdf data source
  inXdfDataSource <- RxXdfData(file = inFile)
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

  # user function
  ".linMod" <- function(data, keys)
  {
    result <- rxLinMod(formula = cost ~ number, data = data)
    return(result$coefficients[[1]])
  }

  #local compute context
  rxSetComputeContext("localpar")
  rxOptions(numCoresToUse = 4)

  # filter function
  ".carFilter" <- function(partDF)
  {
    subset(partDF, car.age != "10+" & type == "A")
  }

  # rxExecBy with no filterFunction
  varsToPartition <- c("car.age", "type")
  localCCResults <- rxExecBy(inData = inXdfDataSource, keys = varsToPartition, func = .linMod)

  checkEquals(length(localCCResults), 16)
  results <- do.call(rbind, lapply(localCCResults, unlist))

  checkEquals(max(as.integer(results[, 3])), 553)
  checkEquals(min(as.integer(results[, 3])), 94)

  # call rxExecBy with filterFunction
  localCCResults <- rxExecBy(inData = inXdfDataSource, keys = varsToPartition, func = .linMod, filterFunc = .carFilter)

  checkEquals(length(localCCResults), 3)
  results <- do.call(rbind, lapply(localCCResults, unlist))

  checkEquals(max(as.integer(results[, 3])), 281)
  checkEquals(min(as.integer(results[, 3])), 94)
}


"test.rxExecBy.ComputeContext" <- function() {
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
  on.exit(rxSetComputeContext("local"), add = TRUE)
  ccBefore = rxGetComputeContext()
  x <- rxExecBy(mtcars, "am", function(keys, data) {tempdir()})
  ccAfter = rxGetComputeContext()
  checkTrue(identical(ccBefore, ccAfter))
}

"test.rxExecBy.varsToKeep" <- function() {
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
  checkIt <- function(res1, res2, varsToCheck) {
    checkEquals(length(res1), length(res2))
    for (i in 1:length(res1)) {
      pname = paste("P", i, sep = "")
      val1 = get(pname, res1)
      val2 = get(pname, res2)
      checkEquals(val1$keys, val2$keys)
      checkEquals(val1$status, val2$status)
      for (name in varsToCheck) {
        checkEquals(val1$result$name, val2$result$name)
      }
      names = colnames(val1$result)
      checkEquals(names, varsToCheck)
    }
  }

  compare <- function(varsToKeep, varsToPartition) {
    inFile <- "claims.xdf"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
    mt1 <- RxXdfData(inFile, varsToKeep = varsToKeep)
    mt2 <- RxXdfData(inFile)
    keepResult <- rxExecBy(mt1, varsToPartition, function(keys, data) head(data))
    normalResult <- rxExecBy(mt2, varsToPartition, function(keys, data) head(data))
    checkIt(keepResult, normalResult, varsToCheck = varsToKeep)
  }

  compare(varsToKeep = c("car.age"), varsToPartition = c("car.age"))
  compare(varsToKeep = c("car.age", "type"), varsToPartition = c("car.age"))
  compare(varsToKeep = c("RowNum", "age", "car.age", "type", "number"), varsToPartition = c("car.age", "number"))
}


"test.rxExecBy.varsToDrop" <- function() {
  checkIt <- function(res1, res2, varsToPart, varsToDrop) {
    checkEquals(length(res1), length(res2))
    for (i in 1:length(res1)) {
      pname = paste("P", i, sep = "")
      val1 = get(pname, res1)
      val2 = get(pname, res2)
      checkEquals(val1$keys, val2$keys)
      checkEquals(val1$status, val2$status)
      names = colnames(val1$result)
      for (name in names) {
        checkEquals(val1$result$name, val2$result$name)
      }
      for (name in varsToDrop) {
        checkEquals(name %in% names, FALSE)
      }
    }
  }

  compare <- function(varsToDrop, varsToPartition) {
    inFile <- "claims.xdf"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
    mt1 <- RxXdfData(inFile, varsToDrop = varsToDrop)
    mt2 <- RxXdfData(inFile)
    keepResult <- rxExecBy(mt1, varsToPartition, function(keys, data) head(data))
    normalResult <- rxExecBy(mt2, varsToPartition, function(keys, data) head(data))
    checkIt(keepResult, normalResult, varsToPart = varsToPartition, varsToDrop = varsToDrop)
  }

  compare(varsToDrop = c("RowNum"), varsToPartition = c("car.age"))
  compare(varsToDrop = c("RowNum", "age"), varsToPartition = c("car.age"))
  compare(varsToDrop = c("RowNum", "age"), varsToPartition = c("car.age", "number"))
}
