# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for fuzzy match code
#
############################################################################################    

"test.rxGetFuzzyKeys.args.Levenshtein" <- function()
{

     cityDictionary <- c("Dublin", "Cork", "Limerick", "Galway", "Waterford", "Drogheda", 
      "Kilkenny", "Wexford", "Sligo", "Clonmel", "Bundalk", "Bray", "Athy", "Kilrush", 
      "Skibbereen", "Bundoran", "Templemore", "Clones", "Newbridge", "Mullingar", 
      "Balgriggan", "Greystones", "Leixlip", "Boyle", "Ballyshannon",
      "Cootehill", "Ballybay", "Belturbet", "Lismore", "Kilkee", "Granard")

    cityNames <- c("Dublin", "Dubline", "Corke", "DUBLIN", "COrk")
    correctNames <- c("Dublin", "Dublin", "Cork", "Dublin", "Cork")

    # Large minimum distance should result in no change
    cityKey1 <- rxGetFuzzyKeys(stringsIn = cityNames, matchMethod = "lv", dictionary = cityDictionary, minDistance = .9)
    checkEquals(cityKey1, cityNames)

    # Smaller minimum distance should fix matched cases, but not mixed case issues
    cityKey2 <- rxGetFuzzyKeys(stringsIn = cityNames, matchMethod = "lv", dictionary = cityDictionary, minDistance = .8)
    checkEquals(cityKey2, c("Dublin", "Dublin", "Cork",  "DUBLIN", "COrk"  ))

    # Small minimum distance should fix everything, even mixed case issues
    cityKey3 <- rxGetFuzzyKeys(stringsIn = cityNames, matchMethod = "lv", dictionary = cityDictionary, minDistance = .1)
    checkEquals(cityKey3, correctNames)

}


"test.rxGetFuzzyKeys.args.Levenshtein.soundex" <- function()
{
    
    cityDictionary <- c("Dublin", "Cork", "Limerick", "Galway", "Waterford", "Drogheda", 
      "Kilkenny", "Wexford", "Sligo", "Clonmel", "Bundalk", "Bray", "Athy", "Kilrush", 
      "Skibbereen", "Bundoran", "Templemore", "Clones", "Newbridge", "Mullingar", 
      "Balgriggan", "Greystones", "Leixlip", "Boyle", "Ballyshannon",
      "Cootehill", "Ballybay", "Belturbet", "Lismore", "Kilkee", "Granard")

    cityNames <- c("Dublin", "Dubline", "Corke", "DUBLIN", "COrk")
    correctNames <- c("Dublin", "Dublin", "Cork", "Dublin", "Cork")

    # Basic soundex will always match
    cityKey1 <- rxGetFuzzyKeys(stringsIn = cityNames, matchMethod = "lv", dictionary = cityDictionary, 
        keyType = "soundex", minDistance = .9)
    checkEquals(cityKey1, correctNames)
    
}

"test.rxGetFuzzyKeys.args.JaroWinkler"
{
    
    # MARTHA MARHTA: Jaro .944 JaroWinkler .961, Levenshtein = 1 - (2/6) = .6667
    # DWAYNE DUANE: Jaro .822 JaroWinkler .84, Levenshtein = 1 - (2/6) = .667
    myDictionary <- c("MARTHA", "DWAYNE")
    myNames = c("MARHTA", "DUANE")
    
    convertMartha <- c("MARTHA", "DUANE")
    # At .95, JaroWinkler should convert Martha, but not Jaro.  Lev. will convert neither
    nameKey1 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "jw", dictionary = myDictionary, minDistance = .95)
    checkEquals(nameKey1, convertMartha)   
    
    nameKey2 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "j", dictionary = myDictionary, minDistance = .95) 
    checkEquals(nameKey2, myNames) 
    
    nameKey3 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "lv", dictionary = myDictionary, minDistance = .95) 
    checkEquals(nameKey3, myNames) 
     
    # At .83, JaroWinkler should convert both, but Jaro just Martha.  Lev. will convert neither
    nameKey4 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "jw", dictionary = myDictionary, minDistance = .83)
    checkEquals(nameKey4, myDictionary)       
    
    nameKey5 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "j", dictionary = myDictionary, minDistance = .83)
    checkEquals(nameKey5, convertMartha) 
        
    nameKey6 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "lv", dictionary = myDictionary, minDistance = .83)
    checkEquals(nameKey6, myNames)  
    
    # At .65, Lev. should convert both
    nameKey7 <- rxGetFuzzyKeys(stringsIn = myNames, matchMethod = "lv", dictionary = myDictionary, minDistance = .65)
    checkEquals(nameKey7, myDictionary)       
  
}

"test.rxGetFuzzyKeys.args.keyType.phonetic"
{      
    origStr <- c("Phorensics", "forensics", "Nicholas", "Nicolas", "Nikolas",
         "Knight", "Night", "Stephen", "Steven", "Matthew", "Matt", "Shuan", "Shawn",
         "McDonald", "MacDonald", "Schwarzenegger", "Shwardseneger", "Ellen", "Elena", "Allen")
    
    soundexCode <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "soundex")
    data.frame(origStr, soundexCode)
    resMphone3 <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "mphone3")
    resMphone3Vowels <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "mphone3Vowels")
    resMphone3Exact <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "mphone3Exact")
    resSoundex <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "soundex")
    ressoundexAll <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "soundexAll")
    resSoundexNum <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "soundexNum")
    
    showData1 <- data.frame(origStr, resMphone3, resMphone3Vowels, resMphone3Exact)
    showData2 <- data.frame(origStr, resSoundex, ressoundexAll, resSoundexNum)
    
    checkEquals(showData2[18, "resSoundex"], showData2[19, "resSoundex"])
    checkEquals(showData2[18, "ressoundexAll"], showData2[19, "ressoundexAll"])
 
}

"test.rxGetFuzzyKeys.args.keyType.alpha" <- function()
{     
    string1 <- c("100 Maple St", "Maple St.", "17 Cherry Ave.", "Cherry 17 Ave.")
    
    resAlpha<- rxGetFuzzyKeys(stringsIn = string1, keyType = "alpha")
    resAlphaNum <- rxGetFuzzyKeys(stringsIn = string1, keyType = "alphaNum")
    checkEquals(resAlphaNum, c("100MapleSt", "MapleSt", "17CherryAve", "Cherry17Ave"))
    checkEquals(resAlpha, c("MapleSt", "MapleSt", "CherryAve", "CherryAve"))
 
}

"test.rxGetFuzzyKeys.args.Metaphone3.ignoreNumbers" <- function()
{
   
    string1 <- c("100 Maple Street", "200 Maple Street")
    res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "none", keyType = "mphone3",
          ignoreNumbers = FALSE)
    checkTrue(res1[1] != res1[2])
    
    res2 <- rxGetFuzzyKeys(stringsIn = string1, keyType = "mphone3", ignoreNumbers = TRUE)
    checkEquals(res2[1], res2[2]) # Should be the same if numbers are ignored
    
    checkTrue(nchar(res1[1])  > nchar(res2[1]))
    checkTrue(nchar(res1[2]) >  nchar(res2[2]))
}

"test.rxGetFuzzyKeys.args.Soundex.default" <- function()
{
    
    string1 <- c("Smith", "Smythe", "Katie", "Katy", 'tech on the net', 'TECH ON THE NET', 'apples', 'apples are great','applus',
      "Washington", "Lee", "Johnson", "Jahnsoon", "Fan", "Fann", "FLASHCARD", "Chris", "Cris", "Kris", ".Kris",
      "WILLIAMS", "BARAGWANATH", "DONNELL", "LLOYD", "WOOLCOCK", "ASHCROFT", "Hong Kong")

    res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "none", keyType =  "soundex")
    checkEquals(res1[1], "S530") # "Smith"
    checkEquals(res1[2], "S530") # "Smythe",
    checkEquals(res1[3], "K300") # "Katie",
    checkEquals(res1[4], "K300") # "Katy"
    checkEquals(res1[5], 'T253') # 'tech on the net'
    checkEquals(res1[6], 'T253') # 'TECH ON THE NET
    checkEquals(res1[7], 'A142') # 'apples'
    checkEquals(res1[8], 'A142') # 'apples are great
    checkEquals(res1[9], 'A142') # 'applus',
    checkEquals(res1[10], 'W252') # "Washington",
    checkEquals(res1[11], 'L000') # 'Lee'
    checkEquals(res1[12], 'J525')
    checkEquals(res1[13], 'J525')
    checkEquals(res1[14], 'F500')
    checkEquals(res1[15], 'F500')
    checkEquals(res1[16], 'F422') # "FLASHCARD" 
    checkEquals(res1[17], 'C620')
    checkEquals(res1[18], 'C620')
    checkEquals(res1[19], 'K620')
    checkEquals(res1[20], 'K620')
    checkEquals(res1[21], 'W452') # WILLIAMS
    checkEquals(res1[22], 'B625')
    checkEquals(res1[23], 'D540')
    checkEquals(res1[24], 'L300')
    checkEquals(res1[25], 'W422')
    checkEquals(res1[26], 'A226')
    checkEquals(res1[27], 'H525')# Hong Kong

}

"test.rxGetFuzzyKeys.args.AmericanSoundex.default" <- function()
{
    
    string1 <- c("Smith", "Smythe", "Katie", "Katy", 'tech on the net', 'TECH ON THE NET', 'apples', 'apples are great','applus',
      "Washington", "Lee", "Johnson", "Jahnsoon", "Fan", "Fann", "FLASHCARD", "Chris", "Cris", "Kris", ".Kris",
      "WILLIAMS", "BARAGWANATH", "DONNELL", "LLOYD", "WOOLCOCK", "ASHCROFT")

    testString = c("ASHCROFT")
    testOut <- rxGetFuzzyKeys(stringsIn = testString, keyType = "soundexAm")
    
    res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "none", keyType =  "soundexAm")
    checkEquals(res1[1], "S530") # "Smith"
    checkEquals(res1[2], "S530") # "Smythe",
    checkEquals(res1[3], "K300") # "Katie",
    checkEquals(res1[4], "K300") # "Katy"
    checkEquals(res1[5], 'T253') # 'tech on the net'
    checkEquals(res1[6], 'T253') # 'TECH ON THE NET
    checkEquals(res1[7], 'A142') # 'apples'
    checkEquals(res1[8], 'A142') # 'apples are great
    checkEquals(res1[9], 'A142') # 'applus',
    checkEquals(res1[10], 'W252') # "Washington",
    checkEquals(res1[11], 'L000') # 'Lee'
    checkEquals(res1[12], 'J525')
    checkEquals(res1[13], 'J525')
    checkEquals(res1[14], 'F500')
    checkEquals(res1[15], 'F500')
    checkEquals(res1[16], 'F426') # "FLASHCARD" - different from Simplified Soundex
    checkEquals(res1[17], 'C620')
    checkEquals(res1[18], 'C620')
    checkEquals(res1[19], 'K620')
    checkEquals(res1[20], 'K620')
    checkEquals(res1[21], 'W452') # WILLIAMS
    checkEquals(res1[22], 'B625')
    checkEquals(res1[23], 'D540')
    checkEquals(res1[24], 'L300')
    checkEquals(res1[25], 'W422')
    checkEquals(res1[26], 'A261') # Different from simplified soundex

}

"test.rxGetFuzzyKeys,args.soundexAll" <- function()
{
    
    string1 <- c("Washington", "Hong Kong", "Jackson",
    "Goerge", "greg", "Thmas", "Thomnas", " TANK",
     "Washnigton", "Wa.sh", "Jefrson", "Wasshington",
    " JEEPERS")

    res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "none", keyType = "soundexAll")
    checkEquals(res1[1], "W25235")
    checkEquals(res1[2], "H52252")
    checkEquals(res1[3], "J25")
    checkEquals(res1[4], "G62")
    checkEquals(res1[5], "G62")
    checkEquals(res1[6], "T52")
    checkEquals(res1[7], "T52")
    checkEquals(res1[8], "T52")
    checkEquals(res1[9], "W25235")
    checkEquals(res1[10], "W2")
    checkEquals(res1[11], "J1625")
    checkEquals(res1[12], "W25235")
    checkEquals(res1[13], "J162")
    
}

"test.rxGetFuzzyKeys.args.SoundexNum" <- function()
{
    origStr <- c("Phorensics", "forensics", "Knight", "Night", "Karet", "Carrot")
    soundexNumCode <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "soundexNum")
    checkEquals(soundexNumCode[1], soundexNumCode[2])
    checkEquals(soundexNumCode[5], soundexNumCode[6])
    
    string1 <- c("Washington", "Hong Kong", "Jackson" , "Michael")
    res1 <- rxGetFuzzyKeys(stringsIn = string1, keyType = "soundexNum")
    checkEquals(res1[1], "2523")
    checkEquals(res1[2], "5252") 
    checkEquals(res1[3], "2250") 
    checkEquals(res1[4], "5240")

}

"test.rxGetFuzzyKeys.args.Soundex.dictionary" <- function()
{   
    nameDictionary <- c("Washington", "Jefferson", "George", "Thomas")
    string1 <- c("Washington", "Jackson",
    "Goerge", "Thmas", "Thomnas",
     "Washnigton", "Wa.sh", "Jefrson", "Wasshington",
    " JEEPERS")
   # res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "soundex", dictionary = nameDictionary)
     res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "exact", keyType = "soundex", dictionary = nameDictionary)
    
    checkEquals(res1[1], "Washington")
    checkEquals(res1[2], "Jackson") # No match
    checkEquals(res1[3], "George")
    checkEquals(res1[4], "Thomas")
    checkEquals(res1[5], "Thomas")
    checkEquals(res1[6], "Washington") 
    checkEquals(res1[7], "Wa.sh") # No match
    checkEquals(res1[8], "Jefferson")
    checkEquals(res1[9], "Washington") 
    checkEquals(res1[10], "Jefferson")
    
}

"test.rxGetFuzzyKeys.args.Metaphone3.dictionary" <- function()
{   
    nameDictionary <- c("Washington", "Jefferson", "George", "Thomas")
    string1 <- c("Washington", "Jackson",
    "Goerge", "Thmas", "Thomnas",
     "Washnigton", "Wa.sh", "Jefrson", "Wasshington",
    " JEEPERS")
    dictionaryCodes <- rxGetFuzzyKeys(stringsIn = nameDictionary, keyType = "mphone3")
    stringCodes <- rxGetFuzzyKeys(stringsIn = string1, keyType = "mphone3")

   # res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "mphone3", dictionary = nameDictionary)
     res1 <- rxGetFuzzyKeys(stringsIn = string1, matchMethod = "lv", keyType = "mphone3", minDistance = .55,
        dictionary = nameDictionary)
    checkEquals(res1[1], "Washington")
    checkEquals(res1[2], "Jefferson") 
    checkEquals(res1[3], "George")
    checkEquals(res1[4], "Thomas")
    checkEquals(res1[5], "Thomnas")
    checkEquals(res1[6], "Washington") 
    checkEquals(res1[7], "Wa.sh") # No match
    checkEquals(res1[8], "Jefferson")
    checkEquals(res1[9], "Washington") 
    checkEquals(res1[10], "Jefferson")
    
}

"test.rxGetFuzzyKeys.args.Metaphone3.ignoreNumbers" <- function()
{

    origStr <- c("100 Maple Apt 410", "200 Maple Apt 300", "200 Maple Apt 502")

    mphone3Code <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "mphone3")
    mphone3NumCode <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "mphone3", ignoreNumbers = FALSE, ignoreSpaces = TRUE)
    mphone3NumSpCode <- rxGetFuzzyKeys(stringsIn = origStr, keyType = "mphone3", ignoreNumbers = FALSE, ignoreSpaces = FALSE)
    outData <- data.frame(origStr, mphone3Code, mphone3NumCode, mphone3NumSpCode)
    checkEquals(mphone3Code[1], mphone3Code[1])
    checkEquals(mphone3NumCode[2], mphone3NumCode[2])
    checkTrue(nchar(mphone3NumSpCode[1]) > nchar(mphone3NumCode[1]))
}


"test.rxGetFuzzyKeys.args.bagOfWords" <- function()
{
    institutionDictionary = c("University of Maryland", "Michigan State University", "University of Michigan", 
        "University of Washington", "Washington State University" )
    
    origCol1 <- c("Univ of Maryland", "Michigan State", "Michigan State University", "Michigan University of",
        "WASHINGTON UNIVERSITY OF", "Washington State Univ")
    
    
    newCol <- rxGetFuzzyKeys(stringsIn = origCol1, 
        dictionary = institutionDictionary, matchMethod = "bag", minDistance = .75, 
        ignoreCase = TRUE )

    outCol1 <- c("Univ of Maryland", "Michigan State", "Michigan State University", "University of Michigan",
        "University of Washington", "Washington State Univ")
  
    checkEquals(newCol, outCol1)
    
    outCol2 <- rxGetFuzzyKeys(stringsIn = origCol1,
        dictionary = institutionDictionary, matchMethod = "bag", minDistance = .75, 
        ignoreCase = TRUE, ignoreWords = c("University", "Univ") )
  
  # TODO Add test
}

"test.rxGetFuzzyKeys.data.xdf" <- function()
{

    # Create a temporary xdf file with educational insitution names
    inData1 <- data.frame(institution = c("Dublin City Univ",
        "DUBLIN COLLEGE UNIV.", "Cork College",  "University of Ireland Maynooth",
        "Dublin Technology Institute", "Limerick University", "Trinity College Dublin",
        "Dublin City Univ.", "University College Dublin", "Univ of Limerick"),
        stringsAsFactors = FALSE)
    inData1$id <- as.integer(1:10)
    inData2 <- data.frame(institution = c(
        "City Univ Dublin", "CORK UNIVERSITY", "Univ Ireland Galway",
        "Galway University", "Dublin Inst for Advanced Studies",
        "Univ of Limrick", "University College Cork", "Maynooth National Univ",
        "Univ. College Dublin", "Trinity College Dubline"),
        stringsAsFactors = FALSE)
    inData2$id <- as.integer(11:20)
   
    inData3 <- data.frame(institution = c(
        "Dublin Advanced Studies", "Dublin Technology Institute", 
        "University of Dublin Trinity",
        "Limrick Univ", "Maynooth Ireland University", "Dublin College Univ",
        "National Univ of Ireland Maaynoth", "Dublin Cities Univ",
        "Dublin Institutute of Technology", "National Ireland Galaway"),
        stringsAsFactors = FALSE)
    inData3$id <- as.integer(21:30)
        
    tempInFile <- tempfile(pattern = "fuzzyKeysIn", fileext = ".xdf")        
    rxDataStep(inData = inData1, outFile = tempInFile, overwrite = TRUE) 
    rxDataStep(inData = inData2, outFile = tempInFile, append = "rows") 
    rxDataStep(inData = inData3, outFile = tempInFile, append = "rows")  
    
   
    # Specify a dictionary
    uDictionary = c("Dublin City University", "University College Cork",
     "University College Dublin", "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "University of Dublin, Trinity College", "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")

    tempOutFile <- tempfile(pattern = "fuzzyKeysOut", fileext = ".xdf")

    outDataSource <- rxGetFuzzyKeys(stringsIn = "institution",
            data = tempInFile, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData <- rxDataStep(outDataSource) 
     inDF <- rxDataStep(tempInFile)
     outDataDirect <- rxGetFuzzyKeys(stringsIn = inDF$institution,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3") 
        
    checkEquals(outData$institution.key, outDataDirect)
    
    # Repeat using an input data frame but output data file
    outDataSource2 <- rxGetFuzzyKeys(stringsIn = "institution",
            data = inDF, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE) 
        

    outData2 <- rxDataStep(outDataSource2)            
    checkEquals(outData2$institution.key, outDataDirect)
    
    # Repeat using an input string but output data file
    outDataSource3 <- rxGetFuzzyKeys(stringsIn = inDF$institution,
            data = NULL, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE) 
        

    outData3 <- rxDataStep(outDataSource3)            
    checkEquals(outData3$stringsIn.key, outDataDirect)  
    
    # Repeat one more time without outFile
    outData4 <- rxGetFuzzyKeys(stringsIn = "institution",
            data = tempInFile, outFile = NULL,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE)        
    checkEquals(outData4$institution.key, outDataDirect)  
    unlink(tempInFile)
    unlink(tempOutFile)       
    
}

"test.rxGetFuzzyKeys.data.factor" <- function()
{

    # Create a temporary xdf file with educational insitution names
    inData1 <- data.frame(institution = c("Dublin City Univ",
        "DUBLIN COLLEGE UNIV.", "Cork College",  "University of Ireland Maynooth",
        "Dublin Technology Institute", "Limerick University", "Trinity College Dublin",
        "Dublin City Univ.", "University College Dublin", "Univ of Limerick"),
        stringsAsFactors = FALSE)
    inData1$id <- as.integer(1:10)
    inData2 <- data.frame(institution = c(
        "City Univ Dublin", "CORK UNIVERSITY", "Univ Ireland Galway",
        "Galway University", "Dublin Inst for Advanced Studies",
        "Univ of Limrick", "University College Cork", "Maynooth National Univ",
        "Univ. College Dublin", "Trinity College Dubline"),
        stringsAsFactors = FALSE)
    inData2$id <- as.integer(11:20)
   
    inData3 <- data.frame(institution = c(
        "Dublin Advanced Studies", "Dublin Technology Institute", 
        "University of Dublin Trinity",
        "Limrick Univ", "Maynooth Ireland University", "Dublin College Univ",
        "National Univ of Ireland Maaynoth", "Dublin Cities Univ",
        "Dublin Institutute of Technology", "National Ireland Galaway"),
        stringsAsFactors = FALSE)
    inData3$id <- as.integer(21:30)
    
    factorLevels <- sort(unique(c(inData1$institution, inData2$institution, inData3$institution)))
    inData1$institution <- factor(inData1$institution, levels = factorLevels)
    inData2$institution <- factor(inData2$institution, levels = factorLevels)
    inData3$institution <- factor(inData3$institution, levels = factorLevels) 
       
    tempInFile <- tempfile(pattern = "fuzzyKeysIn", fileext = ".xdf")        
    rxDataStep(inData = inData1, outFile = tempInFile, overwrite = TRUE) 
    rxDataStep(inData = inData2, outFile = tempInFile, append = "rows") 
    rxDataStep(inData = inData3, outFile = tempInFile, append = "rows")  
    
   
    # Specify a dictionary
    uDictionary = c("Dublin City University", "University College Cork",
     "University College Dublin", "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "University of Dublin, Trinity College", "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")

    tempOutFile <- tempfile(pattern = "fuzzyKeysOut", fileext = ".xdf")

    outDataSource <- rxGetFuzzyKeys(stringsIn = "institution",
            data = tempInFile, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData <- rxDataStep(outDataSource) 
     inDF <- rxDataStep(tempInFile)
     outDataDirect <- rxGetFuzzyKeys(stringsIn = inDF$institution,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3") 
        
    checkEquals(outData$institution.key, outDataDirect)
    
    # Repeat using an input data frame but output data file
    outDataSource2 <- rxGetFuzzyKeys(stringsIn = "institution",
            data = inDF, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE) 
        

    outData2 <- rxDataStep(outDataSource2)            
    checkEquals(outData2$institution.key, outDataDirect)
    
    # Repeat using an input string but output data file
    outDataSource3 <- rxGetFuzzyKeys(stringsIn = inDF$institution,
            data = NULL, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE) 
        

    outData3 <- rxDataStep(outDataSource3)            
    checkEquals(outData3$stringsIn.key, outDataDirect)  
    
    # Repeat one more time without outFile
    outData4 <- rxGetFuzzyKeys(stringsIn = "institution",
            data = tempInFile, outFile = NULL,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .2, 
            keyType = "mphone3", overwrite = TRUE)        
    checkEquals(outData4$institution.key, outDataDirect)  
    unlink(tempInFile)
    unlink(tempOutFile)       
    
}

"test.rxGetFuzzyKeys.args.dictionary.rxFactors" <- function()
{

 origData <- data.frame(
    year = rep(c(2010:2014), each = 5),
    cities = c("Dublin", "Cork", "Limerick", "Dubline", "Corke",
             "Cork", "Limrick", "Dublin", "Dubln", "Cork",
             "Limerick", "Dubline", "Dublin", "Crk", "Dubln",
             "Dublin", "Corke", "Dublin", "Limerick", "Dubln",
             "Dublin", "Dublin", "Cork", "Limerick", "Cork"),
     stringsAsFactors = FALSE)

  recentData <- rxDataStep(inData = origData, rowSelection = year == 2014)
  factoredData <- rxFactors(inData = recentData, factorInfo = c("cities"), sortLevels = TRUE)
  cityDictionary <- rxGetVarInfo(factoredData, varsToKeep = "cities")$cities$levels
  keyData <- rxGetFuzzyKeys(stringsIn = "cities", data = origData, 
    keyType = "mphone3",
    matchMethod = "lv", 
    dictionary = cityDictionary,
    minDistance = .2)
  checkEquals(sum(keyData$cities.key %in% cityDictionary), 25)       
    
}

"test.rxGetFuzzyKeys.args.dictionary.rxSort" <- function()
{

 origData <- data.frame(
    year = rep(c(2010:2014), each = 5),
    cities = c("Dublin", "CORK", "Limerick", "Dubline", "Corke",
             "Cork", "Limrick", "Dublin", "Dubln", "Cork",
             "Limerick", "Dubline", "DUBLIN", "Crk", "Dubln",
             "Dublin", "Corke", "Dublin", "Limerick", "Dubln",
             "Dublin", "Dublin", "Cork", "Limerick", "Cork"),
     stringsAsFactors = FALSE)

  recentData <- rxDataStep(inData = origData, rowSelection = year == 2014)
  dedupData <- rxSort(inData = recentData, sortByVars = "cities", 
    varsToKeep = "cities", removeDupKeys = TRUE)
  cityDictionary <- dedupData$cities
  keyData <- rxGetFuzzyKeys(stringsIn = "cities", data = origData, 
    keyType = "mphone3",
    matchMethod = "lv", 
    dictionary = cityDictionary,
    minDistance = .2,
    keyVarName = "NewCities")
  checkEquals(sum(keyData$NewCities %in% cityDictionary), 25)       
    
}

"test.rxGetFuzzyKeys.args.extraVars" <- function()
{
    cityDictionary <- c("Dublin", "Cork", "Limerick", "Corky")

    cityNames <- c("Dublin", "Dubline", "Corke", "DUBLIN", "COrk")
    correctNames <- c("Dublin", "Dublin", "Cork", "Dublin", "Cork")

    # Large minimum distance should result in no change; 1 match for first
    cityKey1 <- rxGetFuzzyKeys(stringsIn = cityNames, matchMethod = "lv", 
        dictionary = cityDictionary, minDistance = .9, numMatchVarName = "numMatches",
        hasMatchVarName = "hasMatch", matchDistVarName = "matchDist")
    checkEquals(cityKey1$numMatches, c(1, 0, 0, 0, 0))
    checkEquals(cityKey1$hasMatch, c(TRUE, FALSE, FALSE, FALSE, FALSE))
    checkEquals(cityKey1$matchDist, c(1, NA, NA, NA, NA))

    # Smaller minimum distance should fix matched cases, but not mixed case issues
    cityKey2 <- rxGetFuzzyKeys(stringsIn = cityNames, matchMethod = "lv", 
        dictionary = cityDictionary, minDistance = .8, numMatchVarName = "numMatches",
        hasMatchVarName = "hasMatch", matchDistVarName = "matchDist")
    checkEquals(cityKey2$numMatches, c(1,1,2,0,0 ))
    checkEquals(cityKey2$hasMatch, c(TRUE, TRUE, TRUE, FALSE, FALSE))
    checkEquals(cityKey2$matchDist, c(1, .8571429, .8, NA, NA), tolerance = 1e-7)
    
}

"test.rxGetFuzzyKeys.args.Levenshtein.weights" <- function()
{
   
    adistOut <- adist(x = "hi", y = "bi", costs = c(insertions = 1,
       deletions = 1, substitutions = 1), counts = TRUE)
    adistOutT = as.numeric(1 - (adistOut/nchar("hi")))
    fuzzyOut <- rxGetFuzzyKeys(stringsIn = "hi", dictionary = "bi",
      matchMethod = "lv", minDistance = .1, matchDistVarName = "matchDist",
      costs = c(insert = 1, delete = 1, subst = 1))
    checkEquals(adistOutT, fuzzyOut$matchDist)
    
    adistOut <- adist(x = "hi", y = "bi", costs = c(insertions = .6,
       deletions = 1.2, substitutions = 1.2), counts = TRUE)
    adistOutT = as.numeric(1 - (adistOut/nchar("hi")))
    fuzzyOut <- rxGetFuzzyKeys(stringsIn = "hi", dictionary = "bi",
      matchMethod = "lv", minDistance = .1, matchDistVarName = "matchDist",
      costs = c(insert = .6, delete = 1.2, subst = 1.2))
    checkEquals(adistOutT, fuzzyOut$matchDist, tolerance = 1e-7)
    
    adistOut <- adist(x = "hi", y = "bi", costs = c(insertions = .75,
       deletions = .75, substitutions = 1.5), counts = TRUE)
    adistOutT = as.numeric(1 - (adistOut/nchar("hi"))) # Has 1 insertion & 1 deletion
    fuzzyOut <- rxGetFuzzyKeys(stringsIn = "hi", dictionary = "bi",
      matchMethod = "lv", minDistance = .1, matchDistVarName = "matchDist",
      costs = c(insert = .75, delete = .75, subst = 1.5))
    checkEquals(adistOutT, fuzzyOut$matchDist, tolerance = 1e-7) 
    
    adistOut <- adist(x = "GUMBO", y = "GAMBOL", costs = c(insertions = 1,
       deletions = 1, substitutions = 1), counts = TRUE)
    adistOutT = as.numeric(1 - (adistOut/nchar("GAMBOL"))) # 1 insert, 1 subst
    fuzzyOut <- rxGetFuzzyKeys(stringsIn = "GUMBO", dictionary = "GAMBOL",
      matchMethod = "lv", minDistance = .1, matchDistVarName = "matchDist",
      costs = c(insert = 1, delete = 1, subst = 1))
    checkEquals(adistOutT, fuzzyOut$matchDist, tolerance = 1e-7)
    
    adistOut <- adist(x = "GUMBO", y = "GAMBOL", costs = c(insertions = .6,
       deletions = 1.2, substitutions = 1.2), counts = TRUE)
    adistOutT = as.numeric(1 - (adistOut/nchar("GAMBOL")))  # 1 insert, 1 subst
    fuzzyOut <- rxGetFuzzyKeys(stringsIn = "GUMBO", dictionary = "GAMBOL",
      matchMethod = "lv", minDistance = .1, matchDistVarName = "matchDist",
      costs = c(insert = .6, delete = 1.2, subst = 1.2))
    checkEquals(adistOutT, fuzzyOut$matchDist, tolerance = 1e-7)
    
    adistOut <- adist(x = "GUMBO", y = "GAMBOL", costs = c(insertions = .75,
       deletions = .75, substitutions = 1.5), counts = TRUE)
    adistOutT = as.numeric(1 - (adistOut/nchar("GAMBOL"))) # Has 1 insertion & 1 deletion
    fuzzyOut <- rxGetFuzzyKeys(stringsIn = "GUMBO", dictionary = "GAMBOL",
      matchMethod = "lv", minDistance = .1, matchDistVarName = "matchDist",
      costs = c(insert = .75, delete = .75, subst = 1.5))
    checkEquals(adistOutT, fuzzyOut$matchDist, tolerance = 1e-7)     
 }

"test.rxGetFuzzyDist.args.basic" <- function()
{
    
    myWords <- c("Allen", "Ellen", "Elena")
    myDictionary <- c("Allen", "Helen", "Harry")
    lvRes <- adist(myWords, myDictionary)
    lvRes <- 1 - (lvRes/5)
    outDist1 <- rxGetFuzzyDist(stringsIn = myWords, dictionary = myDictionary, minDistance = .5, returnString = FALSE)
    checkEquals(outDist1$distance[1], lvRes[outDist1$stringIndex[1], outDist1$dictionaryIndex[1]])
    checkEquals(outDist1$distance[2], lvRes[outDist1$stringIndex[2], outDist1$dictionaryIndex[2]], tolerance = 1e-7 )
    checkEquals(outDist1$distance[3], lvRes[outDist1$stringIndex[3], outDist1$dictionaryIndex[3]], tolerance = 1e-7)
    checkEquals(outDist1$distance[4], lvRes[outDist1$stringIndex[4], outDist1$dictionaryIndex[4]], tolerance = 1e-7)

    outDist2 <- rxGetFuzzyDist(stringsIn = c("Univ of Ontario"), 
      dictionary = c("Univ of Michgan", "Univ of Ontario", "Univ Michigan", "Michigan State", "Ontario Univ of"),
      matchMethod = "bag", minDistance = 0)
    checkEquals(outDist2$matchWords[1], "Univ of")
    checkEquals(outDist2$matchWords[2], "Univ of Ontario")
    checkEquals(outDist2$matchWords[3], "Univ")
    checkEquals(outDist2$matchWords[4], as.character(NA))
    checkEquals(outDist2$matchWords[5], "Univ of Ontario")
    checkEquals(outDist2$distance, c(0.6666667, 1, .3333333, 0, 1), tolerance = 1e-7)

    outDist3 <- rxGetFuzzyDist(stringsIn = c("Univ of Ontario"), 
      dictionary = c("Univ of Michgan", "Univ of Ontario", "Univ Michigan", "Michigan State", "Ontario Univ of"),
      matchMethod = "bag", ignoreWords = c("of"), minDistance = 0)

    checkEquals(outDist3$matchWords[1], "Univ")
    checkEquals(outDist3$matchWords[2], "Univ Ontario")
    checkEquals(outDist3$matchWords[3], "Univ")
    checkEquals(outDist3$matchWords[4], as.character(NA))
    checkEquals(outDist3$matchWords[5], "Univ Ontario")

    outDist4 <- rxGetFuzzyDist(stringsIn = c("Univ of Ontario"), 
      dictionary = c("Univ of Michgan", "Univ of Ontario", "Univ Michigan", "Michigan State", "Ontario Univ of"),
      matchMethod = "bag",  keyType = "soundex", ignoreSpaces = FALSE,  minDistance = 0)    
    
    soundKeys <- rxGetFuzzyKeys(stringsIn =  c("Univ of", "Univ of Ontario", "Univ", "", "Univ of Ontario"), 
        keyType = "soundex", ignoreSpaces = FALSE)
    
    checkEquals(outDist4$matchWords[1], soundKeys[1])
    checkEquals(outDist4$matchWords[2], soundKeys[2])
    checkEquals(outDist4$matchWords[3], soundKeys[3])
    checkEquals(outDist4$matchWords[4], as.character(NA))
    checkEquals(outDist4$matchWords[5], soundKeys[5])
 
}

"test.rxGetFuzzyDist.output.noMatches" <- function()
{
    
    # Specify a dictionary
    uDictionary = c("Dublin City University", "University College Cork",
     "University College Dublin", "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "University of Dublin, Trinity College", "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")
    
    fuzzyOut <- rxGetFuzzyDist(stringsIn = "Galway University",
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3") 
        
    checkEquals(nrow(fuzzyOut), 0L)
}

"test.rxGetFuzzyDist.args.bagOfWords.partial" <- function()
{
     #rxGetFuzzyDist with bag-of-words doesn't remove partial words", 
    
    # Specify a dictionary
    uDictionary = c("Dublin City Univ.", 
     "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")
    
    fuzzyOut <- rxGetFuzzyDist(stringsIn = "Galway University",
            dictionary = uDictionary,
            ignoreWords = c( "Univ", "University" ),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .1, 
           ) 
        
    checkEquals(nrow(fuzzyOut), 1L)
    
    fuzzyOut <- rxGetFuzzyDist(stringsIn = "Dublin Univ.",
            dictionary = uDictionary,
            ignoreWords = c( "Univ", "University" ),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            keyType = "alphanum",
            ignoreSpaces = FALSE,  
            minDistance = .1, 
           )   
    
    checkEquals(fuzzyOut$matchWords[1], "DUBLIN")      
}

"test.rxGetFuzzyDist.rxDataStep" <- function()
{
  # rxGetFuzzyDist computes works inside rxDataStep

    # Create a temporary xdf file with educational insitution names
    inData1 <- data.frame(institution = c("Dublin City Univ",
        "DUBLIN COLLEGE UNIV.", "Cork College",  "University of Ireland Maynooth",
        "Dublin Technology Institute", "Limerick University", "Trinity College Dublin",
        "Dublin City Univ.", "University College Dublin", "Univ of Limerick"),
        stringsAsFactors = FALSE)
    inData1$id <- as.integer(1:10)
    inData2 <- data.frame(institution = c(
        "City Univ Dublin", "CORK UNIVERSITY", "Univ Ireland Galway",
        "Galway University", "Dublin Inst for Advanced Studies",
        "Univ of Limrick", "University College Cork", "Maynooth National Univ",
        "Univ. College Dublin", "Trinity College Dubline"),
        stringsAsFactors = FALSE)
    inData2$id <- as.integer(11:20)
   
    inData3 <- data.frame(institution = c(
        "Dublin Advanced Studies", "Dublin Technology Institute", 
        "University of Dublin Trinity",
        "Limrick Univ", "Maynooth Ireland University", "Dublin College Univ",
        "National Univ of Ireland Maaynoth", "Dublin Cities Univ",
        "Dublin Institutute of Technology", "National Ireland Galaway"),
        stringsAsFactors = FALSE)
    inData3$id <- as.integer(21:30)
        
    tempInFile <- tempfile(pattern = "fuzzyDistIn", fileext = ".xdf")        
    rxDataStep(inData = inData1, outFile = tempInFile, overwrite = TRUE) 
    rxDataStep(inData = inData2, outFile = tempInFile, append = "rows") 
    rxDataStep(inData = inData3, outFile = tempInFile, append = "rows")  
    
    # Specify a transformation function
    getAllDistance <- function( dataList )
    {
        thisDictionary <- .rxGet("tDictionary")
        thisMinDistance <- .rxGet("tMinDistance")
        thisMatchMethod <- .rxGet("tMatchMethod")
        thisKeyType <- .rxGet("tKeyType")
        thisIgnoreWords <- .rxGet("tIgnoreWords")
        thisIgnoreSpaces <- .rxGet("tIgnoreSpaces")
        thisIgnoreCase <- .rxGet("tIgnoreCase")
        
        thisStringsIn = dataList[[1]]
        outData <- rxGetFuzzyDist(stringsIn = thisStringsIn,
            dictionary = thisDictionary,
            ignoreWords = thisIgnoreWords,
            ignoreCase = thisIgnoreCase,
            matchMethod = thisMatchMethod,  
            ignoreSpaces = thisIgnoreSpaces,  
            minDistance = thisMinDistance, 
            keyType = thisKeyType)   
        
        return( outData )
    }

    # Specify a dictionary
    uDictionary = c("Dublin City University", "University College Cork",
     "University College Dublin", "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "University of Dublin, Trinity College", "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")

    tempOutFile <- tempfile(pattern = "fuzzyDistOut", fileext = ".xdf")
    rxDataStep(inData = tempInFile, outFile = tempOutFile, 
        transformFunc = getAllDistance, varsToKeep = "institution",
        transformObjects =
        list( tDictionary = uDictionary, tMinDistance = .5, 
            tMatchMethod = "bag", tKeyType = "mphone3", 
            tIgnoreWords = c("University", "Univ", "Institute", "Inst","of"),
            tIgnoreSpaces = FALSE,
            tIgnoreCase = TRUE,
            rxGetFuzzyDist = rxGetFuzzyDist), overwrite = TRUE)  
        
     outData <- rxDataStep(tempOutFile) 
     inDF <- rxDataStep(tempInFile)
     outDataDirect <- rxGetFuzzyDist(stringsIn = inDF$institution,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3") 
        
    checkEquals(outData, outDataDirect)
    
    unlink(tempInFile)
    unlink(tempOutFile)       
    
}

"test.rxGetFuzzyDist.data.xdf" <- function()
{

    # Create a temporary xdf file with educational insitution names
    inData1 <- data.frame(institution = c("Dublin City Univ",
        "DUBLIN COLLEGE UNIV.", "Cork College",  "University of Ireland Maynooth",
        "Dublin Technology Institute", "Limerick University", "Trinity College Dublin",
        "Dublin City Univ.", "University College Dublin", "Univ of Limerick"),
        stringsAsFactors = FALSE)
    inData1$id <- as.integer(1:10)
    inData2 <- data.frame(institution = c(
        "City Univ Dublin", "CORK UNIVERSITY", "Univ Ireland Galway",
        "Galway University", "Dublin Inst for Advanced Studies",
        "Univ of Limrick", "University College Cork", "Maynooth National Univ",
        "Univ. College Dublin", "Trinity College Dubline"),
        stringsAsFactors = FALSE)
    inData2$id <- as.integer(11:20)
   
    inData3 <- data.frame(institution = c(
        "Dublin Advanced Studies", "Dublin Technology Institute", 
        "University of Dublin Trinity",
        "Limrick Univ", "Maynooth Ireland University", "Dublin College Univ",
        "National Univ of Ireland Maaynoth", "Dublin Cities Univ",
        "Dublin Institutute of Technology", "National Ireland Galaway"),
        stringsAsFactors = FALSE)
    inData3$id <- as.integer(21:30)
        
    tempInFile <- tempfile(pattern = "fuzzyDistIn", fileext = ".xdf")        
    rxDataStep(inData = inData1, outFile = tempInFile, overwrite = TRUE) 
    rxDataStep(inData = inData2, outFile = tempInFile, append = "rows") 
    rxDataStep(inData = inData3, outFile = tempInFile, append = "rows")  
    
   
    # Specify a dictionary
    uDictionary = c("Dublin City University", "University College Cork",
     "University College Dublin", "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "University of Dublin, Trinity College", "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")

    tempOutFile <- tempfile(pattern = "fuzzyDistOut", fileext = ".xdf")
    
    outDataSource <- rxGetFuzzyDist(stringsIn = "institution",
            data = tempInFile, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData <- rxDataStep(outDataSource) 
     inDF <- rxDataStep(tempInFile)
     outDataDirect <- rxGetFuzzyDist(stringsIn = inDF$institution,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3") 
        
    checkEquals(outData$stringsIn, outDataDirect$stringsIn)
    checkEquals(outData$matchWords, outDataDirect$matchWords)
    
    multEntries <- duplicated(outData$id) # No match for 14
    checkEquals(outData$id[!multEntries], c(1:13, 15:30))
    
    # Redo with input data frame and output file
    outDataSource2 <- rxGetFuzzyDist(stringsIn = "institution",
            data = inDF, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData2 <- rxDataStep(outDataSource2) 
     checkEquals(outData2, outData) 
    
    
     # Redo with input string and output file
     outDataSource3 <- rxGetFuzzyDist(stringsIn = inDF$institution,
            data = NULL, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData3 <- rxDataStep(outDataSource3) 
     checkEquals(outData3, outDataDirect) 
      
     # Redo one more time with no output xdf
     outData4 <- rxGetFuzzyDist(stringsIn = "institution",
            data = tempInFile, outFile = NULL,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)      
    checkEquals(outData4, outData) 
    
    unlink(tempInFile)
    unlink(tempOutFile)       
    
}

"test.rxGetFuzzyDist.data.factor" <- function()
{

    # Create a temporary xdf file with educational insitution names
    inData1 <- data.frame(institution = c("Dublin City Univ",
        "DUBLIN COLLEGE UNIV.", "Cork College",  "University of Ireland Maynooth",
        "Dublin Technology Institute", "Limerick University", "Trinity College Dublin",
        "Dublin City Univ.", "University College Dublin", "Univ of Limerick"),
        stringsAsFactors = FALSE)
    inData1$id <- as.integer(1:10)
    inData2 <- data.frame(institution = c(
        "City Univ Dublin", "CORK UNIVERSITY", "Univ Ireland Galway",
        "Galway University", "Dublin Inst for Advanced Studies",
        "Univ of Limrick", "University College Cork", "Maynooth National Univ",
        "Univ. College Dublin", "Trinity College Dubline"),
        stringsAsFactors = FALSE)
    inData2$id <- as.integer(11:20)
   
    inData3 <- data.frame(institution = c(
        "Dublin Advanced Studies", "Dublin Technology Institute", 
        "University of Dublin Trinity",
        "Limrick Univ", "Maynooth Ireland University", "Dublin College Univ",
        "National Univ of Ireland Maaynoth", "Dublin Cities Univ",
        "Dublin Institutute of Technology", "National Ireland Galaway"),
        stringsAsFactors = FALSE)
    inData3$id <- as.integer(21:30)
    
    factorLevels <- sort(unique(c(inData1$institution, inData2$institution, inData3$institution)))
    inData1$institution <- factor(inData1$institution, levels = factorLevels)
    inData2$institution <- factor(inData2$institution, levels = factorLevels)
    inData3$institution <- factor(inData3$institution, levels = factorLevels)     
        
    tempInFile <- tempfile(pattern = "fuzzyDistIn", fileext = ".xdf")        
    rxDataStep(inData = inData1, outFile = tempInFile, overwrite = TRUE) 
    rxDataStep(inData = inData2, outFile = tempInFile, append = "rows") 
    rxDataStep(inData = inData3, outFile = tempInFile, append = "rows")  
    
   
    # Specify a dictionary
    uDictionary = c("Dublin City University", "University College Cork",
     "University College Dublin", "National University of Ireland, Galway",
     "National University of Ireland, Maynooth", "University of Limerick",
     "University of Dublin, Trinity College", "Dublin Institute of Technology",
     "Dublin Institute for Advanced Studies")

    tempOutFile <- tempfile(pattern = "fuzzyDistOut", fileext = ".xdf")
    
    outDataSource <- rxGetFuzzyDist(stringsIn = "institution",
            data = tempInFile, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData <- rxDataStep(outDataSource) 
     inDF <- rxDataStep(tempInFile)
     outDataDirect <- rxGetFuzzyDist(stringsIn = inDF$institution,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3") 
        
    checkEquals(outData, outDataDirect)
    
    # Redo with input data frame and output file
    outDataSource2 <- rxGetFuzzyDist(stringsIn = "institution",
            data = inDF, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData2 <- rxDataStep(outDataSource2) 
     checkEquals(outData2, outDataDirect) 
    
    
     # Redo with input string and output file
     outDataSource3 <- rxGetFuzzyDist(stringsIn = inDF$institution,
            data = NULL, outFile = tempOutFile,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)     
        
     outData3 <- rxDataStep(outDataSource3) 
     checkEquals(outData3, outDataDirect) 
      
     # Redo one more time with no output xdf
     outData4 <- rxGetFuzzyDist(stringsIn = "institution",
            data = tempInFile, outFile = NULL,
            dictionary = uDictionary,
            ignoreWords = c("University", "Univ", "Institute", "Inst","of"),
            ignoreCase = TRUE,
            matchMethod = "bag",  
            ignoreSpaces = FALSE,  
            minDistance = .5, 
            keyType = "mphone3", overwrite = TRUE)      
    checkEquals(outData4, outDataDirect) 
    
    unlink(tempInFile)
    unlink(tempOutFile)       
    
}



