# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxGetInfo and functionality for defunct rxGetInfoXdf
#
############################################################################################

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

"test.rxGetInfo.data" <- function()
{
  # Use rxGetInfo with data frame
    myDF <- data.frame(x=1:10, y=21:30)
    infoObj <- rxGetInfo(myDF)
  # Should have 2 variables 
  checkEquals(infoObj$numVars, 2)
  checkEquals(infoObj$numRows, 10)

  # Create .xdf file and use rxGetInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    infoObjXdf <- rxGetInfo(myFileXdf)
  
  checkEquals(infoObj$numRows, infoObjXdf$numRows)
  checkEquals(infoObj$numVars, infoObjXdf$numVars)
  checkEquals(infoObj$numBlocks, 0)
  checkEquals(infoObjXdf$numBlocks, 1)

  # Create .xdf data source
  myXdfDS <- RxXdfData( myFileXdf)
  infoObjDS <- rxGetInfo( myXdfDS )
  checkEquals(infoObjXdf, infoObjDS)
  
  file.remove(myFileXdf)
  
  infoObj1 <- rxGetInfo(data = myDF, getVarInfo =TRUE, startRow = 2, numRows = 2)
  checkEquals(infoObj1$varInfo$x$varType, "integer")
  checkEquals(infoObj1$data$x, c(2,3))
  checkEquals(infoObj1$data$y, c(22,23))
  
  # Try a list
  myList <- list(item1 = c(1:10), item2 = "hello", item3 = list(a=1, b=2))
  infoObjList <- rxGetInfo(myList)
  checkEquals(infoObjList$objName, "myList")
  checkEquals(infoObjList$class, "list")
  checkEquals(infoObjList$length, 3)
    
    # SAS file
    claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  claimsSasSource <- RxSasData(claimsSasFileName)
    infoObjSas <- rxGetInfo( claimsSasSource,  numRows = 2 )   
    checkEquals(infoObjSas$numRows, 128)
  
    # Try an SPSS file
    claimsSpssFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
  claimsSpssSource <- RxSpssData(claimsSpssFileName)
    infoObjSpss <- rxGetInfo( claimsSpssSource, getVarInfo = TRUE, numRows = 5 )
  checkEquals(infoObjSpss$numRows, 128)
    checkEquals(nrow(infoObjSpss$data), 5)
    
    # Fixed format
    claimsFixedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sts")
  claimsFixedSource <- RxTextData(claimsFixedFileName, firstRowIsColNames = FALSE, useFastRead = FALSE)
    infoObjFixed1 <- rxGetInfo( claimsFixedSource, getVarInfo = TRUE, numRows = 5 )
    checkEquals(nrow(infoObjFixed1$data), 5)    
    
    # Fast Fixed format
  claimsFixedSource <- RxTextData(claimsFixedFileName, useFastRead = TRUE)
    infoObjFixed2 <- rxGetInfo( claimsFixedSource, getVarInfo = TRUE, numRows = 5 )
  
  checkEquals(infoObjFixed1, infoObjFixed2)
    
    # Delimited format
    claimsDelimitedFileName <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
  claimsDelimitedSource <- RxTextData(claimsDelimitedFileName, firstRowIsColNames = TRUE, useFastRead = FALSE)
    infoObjDelimited1 <- rxGetInfo(claimsDelimitedSource, getVarInfo = TRUE, numRows = 5)
  checkEquals(nrow(infoObjDelimited1$data), 5)  
    
  claimsDelimitedSource <- RxTextData(claimsDelimitedFileName, useFastRead = TRUE)
    infoObjDelimited2 <- rxGetInfo(claimsDelimitedSource, getVarInfo = TRUE, numRows = 5)
  
  checkEquals(infoObjDelimited1, infoObjDelimited2)
    
    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE) 
    
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
    odbcDS <-  RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString)
    infoObjSqlite <- rxGetInfo(odbcDS, computeInfo = TRUE, getVarInfo = TRUE,numRows = 5)
    
    checkEquals(nrow(infoObjSqlite$data), 5) 
    checkEquals(length(infoObjSqlite$varInfo), 6)
    checkEquals(infoObjSqlite$numRows,128)
    checkEquals(infoObjSqlite$numVars,6)
   
}

"test.rxGetInfo.args.numRows" <- function()
{
  # Use rxGetInfo with data frame
    myDF <- data.frame(x=1:10, y=21:30)
    infoObj <- rxGetInfo(myDF, numRows=2)
  checkEquals(infoObj$data, myDF[1:2,])

  # Create .xdf file and use rxGetInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    infoObjXdf <- rxGetInfo(myFileXdf, numRows=2)
  
  checkEquals(infoObj$data, infoObjXdf$data)
  file.remove(myFileXdf)
}

"test.rxGetInfo.args.startRow" <- function()
{
  # Use rxGetInfo with data frame
    myDF <- data.frame(x=1:10, y=21:30)
    infoObj <- rxGetInfo(myDF, numRows = 2, startRow = 5)
  
  # Subsetted data frame will have old row names
  checkEquals(infoObj$data, myDF[5:6,], check.attributes=FALSE)

  # Create .xdf file and use rxGetInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    infoObjXdf <- rxGetInfo(myFileXdf, numRows=2, startRow = 5)
  
  checkEquals(infoObj$data, infoObjXdf$data)
  file.remove(myFileXdf)
}

"test.rxGetInfo.args.getVarInfo" <- function()
{
  # Use rxGetInfo with data frame
    myDF <- data.frame(x=1:10, y=21:30 )
    infoObj <- rxGetInfo(myDF, getVarInfo=TRUE)

  # Create .xdf file and use rxGetInfo with .xdf file
  myFileXdf <-  file.path(tempdir(),".rxTempfoXdf.xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf)
    infoObjXdf <- rxGetInfo(myFileXdf, getVarInfo=TRUE)
  
  checkEquals(infoObj$varInfo, infoObjXdf$varInfo)
  file.remove(myFileXdf)
}

"test.rxGetInfo.args.computeInfo" <- function()
{
  # Text data source
    claimsFileTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt") 
    claimsTxt <- RxTextData(file = claimsFileTxt)
    txtInfoCompute <- rxGetInfo(claimsTxt, computeInfo = TRUE)	
    checkEquals(txtInfoCompute$numVars, 6)
    txtInfoNoCompute <- rxGetInfo(claimsTxt, computeInfo = FALSE)
    checkTrue(is.null(txtInfoNoCompute$numVars))
    
    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE) 
    
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
    odbcDS <-  RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString)
    odbcInfoCompute <- rxGetInfo(odbcDS, computeInfo = TRUE)   
    checkEquals(odbcInfoCompute$numVars, 6)
    odbcInfoNoCompute <- rxGetInfo(odbcDS, computeInfo = FALSE) 
    checkTrue(is.null(odbcInfoNoCompute$numVars))   
}

"test.rxGetInfoXdf.data.dataframe" <- function()
{
    myDF <- data.frame(x=1:10, y=21:30)
  myFileXdf <- tempfile(pattern =  ".rxTempXdf", fileext = ".xdf")
    if (file.exists(myFileXdf)) file.remove(myFileXdf)
  rxDataStep(myDF, myFileXdf, rowVarName = "rowNames")
    infoObj <- rxGetInfo(myFileXdf)
  # Should have 3 variables (including rownames)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numRows, 10)
  file.remove(myFileXdf)
}

"test.rxGetInfoXdf.data.xdf" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    infoObj <- rxGetInfo(fileName)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
  
}

"test.rxGetInfoXdf.print.xdf" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    out1 <- capture.output(rxGetInfo(fileName, getBlockSizes=TRUE))
  checkTrue(grepl("File name:", out1[1]))
  checkTrue(grepl("CensusWorkers", out1[1]))
  checkTrue(grepl("Number of observations: 351121", out1[2]))
  checkTrue(grepl("Number of variables: 6", out1[3]))
  checkTrue(grepl("Number of blocks: 6", out1[4]))
  checkTrue(grepl("Rows per block: 95420 42503 1799 131234 34726 45439", out1[5]))
 
}

            
"test.rxGetInfoXdf.args.getVarInfo" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    infoObj <- rxGetInfo(fileName, getVarInfo=TRUE)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
        varInfo <- infoObj$varInfo
    checkEquals(length(varInfo), infoObj$numVars)
    # Spot check variable information
    checkEquals(varInfo[[6]]$varType, "factor")
    checkEquals(varInfo[[5]]$description, "Weeks worked last year")
    checkEquals(varInfo[[4]]$levels, c("Male", "Female"))
    checkEquals(varInfo[[3]]$high, 168)
    checkEquals(varInfo[[2]]$storage, "int32")
    checkEquals(varInfo[[1]]$low, 20)  
}

"test.rxGetInfoXdf.args.varsToKeep" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    varsToKeep <- c("age", "wkswork1", "perwt")

    infoObj <- rxGetInfo(fileName, getVarInfo=TRUE, varsToKeep=varsToKeep)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    varInfo <- infoObj$varInfo
    checkEquals(length(varInfo), 3)
    checkEquals(names(varInfo) %in% varsToKeep, c(TRUE, TRUE, TRUE))
    # Spot check variable information
    checkEquals(varInfo[[1]]$low, 20) 
    checkEquals(varInfo[[2]]$description, "Weeks worked last year")
    checkEquals(varInfo[[3]]$high, 168)
}

"test.rxGetInfoXdf.args.varsToDrop" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    varsToKeep <- c("age", "wkswork1", "perwt")
    varsToDrop <- c("incwage", "sex", "state")

    infoObj <- rxGetInfo(fileName, getVarInfo=TRUE, varsToDrop=varsToDrop)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    varInfo <- infoObj$varInfo
    checkEquals(length(varInfo), 3)
    checkEquals(names(varInfo) %in% varsToKeep, c(TRUE, TRUE, TRUE))
    # Spot check variable information
    checkEquals(varInfo$age$low, 20) 
    checkEquals(varInfo$wkswork1$description, "Weeks worked last year")
    checkEquals(varInfo$perwt$high, 168)
}

"test.rxGetInfoXdf.args.getBlockSizes" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    infoObj <- rxGetInfo(fileName, getBlockSizes=TRUE)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    blockSizes <- c(95420, 42503, 1799, 131234, 34726, 45439)
    checkEquals(infoObj$rowsPerBlock, blockSizes)
  
}

"test.rxGetInfoXdf.args.getValueLabels" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")

    infoObj <- rxGetInfo(fileName, getVarInfo=TRUE, getValueLabels=TRUE)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    varInfo <- infoObj$varInfo
    checkEquals(length(varInfo), 6)
   
    # Check levels information
    checkEquals(varInfo$state$levels, c("Connecticut", "Indiana", "Washington")) 
    checkEquals(varInfo$sex$levels, c("Male", "Female"))
    
    infoObj <- rxGetInfo(fileName, getVarInfo=TRUE, getValueLabels=FALSE)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    varInfo <- infoObj$varInfo
    checkEquals(length(varInfo), 6)
   
    # Check levels information
    checkEquals(is.null(varInfo$state$levels), TRUE) 
    checkEquals(is.null(varInfo$sex$levels), TRUE)
}

"test.rxGetInfoXdf.args.numRows" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    dfSubSamp <- rxDataStep(inData =fileName, numRows=2)
    
    infoObj <- rxGetInfo(fileName,  numRows=2)
    checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    checkEquals(infoObj$data, dfSubSamp)
    
    # BUG RA-1416
  
    infoObj <- rxGetInfo(fileName, getBlockSizes=TRUE, numRows=2)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    blockSizes <- c(95420, 42503, 1799, 131234, 34726, 45439)
    checkEquals(infoObj$rowsPerBlock, blockSizes)
    checkEquals(infoObj$data, dfSubSamp)
}

"test.rxGetInfoXdf.args.startRow" <- function()
{
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    dfSubSamp <- rxDataStep(inData =fileName, numRows=2, startRow=100)
    
    infoObj <- rxGetInfo(fileName,  numRows=2, startRow=100)
    checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    checkEquals(infoObj$data, dfSubSamp)
    
    # BUG RA-1416
  
    infoObj <- rxGetInfo(fileName, getBlockSizes=TRUE, numRows=2, startRow=100)

  checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    blockSizes <- c(95420, 42503, 1799, 131234, 34726, 45439)
    checkEquals(infoObj$rowsPerBlock, blockSizes)
    checkEquals(infoObj$data, dfSubSamp)
}

"test.rxGetInfoXdf.args.verbose" <- function()
{
    # Verbose = 1
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
      
    o1 <- capture.output(infoObj <- rxGetInfo(fileName, verbose=1))
    checkEquals(infoObj$numVars, 6)
  checkEquals(infoObj$numRows, 351121)
  checkEquals(infoObj$numBlocks, 6)
    checkTrue(length(grep("Number of rows: 351121", o1, fixed=TRUE)) > 0)
    checkTrue(length(grep("Number of variables: 6", o1, fixed=TRUE)) > 0)
    checkTrue(length(grep("Number of blocks: 6", o1, fixed=TRUE)) > 0)
    checkTrue(length(grep("Col 1: 'age', ('Age'), Int (Min/Max=20,65)", o1, fixed=TRUE)) > 0)
    checkTrue(length(grep("Col 6: 'state', UInt (Min/Max/PadForMissings=0,2,0)  Labels", o1, fixed=TRUE)) > 0)
}


"test.rxGetInfoXdf.output.print" <- function()
{
    # Check printed output
  fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
      
    o1 <- capture.output(rxGetInfo(fileName))
    checkTrue(length(grep("Number of observations: 351121", o1, fixed=TRUE)) > 0)
    checkTrue(length(grep("Number of variables: 6", o1, fixed=TRUE)) > 0)
    checkTrue(length(grep("Number of blocks: 6", o1, fixed=TRUE)) > 0)
}

"test.rxGetInfo.output.print.compressionType" <- function()
{
    myData <- data.frame(x = 1:100, y = 1:100/10)
    outFileNone <- tempfile(pattern = ".rxGetInfo1", fileext = ".xdf")
    outFileCompress <- tempfile(pattern = ".rxGetInfo2", fileext = ".xdf")
    rxDataStep(inData = myData, outFile = outFileNone, xdfCompressionLevel = 0)
    rxDataStep(inData = myData, outFile = outFileCompress, xdfCompressionLevel = 1)
    o1 <- capture.output(rxGetInfo(outFileNone))
    checkTrue(length(grep("Compression type: none", o1, fixed=TRUE)) > 0)
    
    o2 <- capture.output(rxGetInfo(outFileCompress))
    checkTrue(length(grep("Compression type: zlib", o2, fixed=TRUE)) > 0)
    
    file.remove(outFileNone)
    file.remove(outFileCompress)
 }
    
"test.rxGetVarNames.args.data" <- function()
{
    fileName <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers")
    varInfo <- rxGetVarInfo(fileName)
    varNames <- rxGetVarNames(fileName)
    checkEquals(names(varInfo), varNames)
    
    varNames <- rxGetVarNames(iris)
    checkEquals(names(iris), varNames)
    
    claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  claimsSasSource <- RxSasData(claimsSasFileName)

    varNamesDS <- rxGetVarNames( claimsSasSource )
    
  # Import the data into a data frame in memory
  claimsIn <- rxImport(claimsSasSource)
  varNamesDF <- rxGetVarNames(claimsIn)
    
    checkEquals(varNamesDS, varNamesDF )
}

"test.rxSetInfo.args.description" <- function()
{    
    # Specify name and location of output file
    outFile <- tempfile(pattern= ".rxTempFile", fileext = ".xdf")
    if (file.exists(outFile)) file.remove(outFile)
    
    # Create a sample data frame and .xdf file
    origData <- data.frame(x=1:10)
    rxDataStep(inData = origData, outFile = outFile, rowsPerRead = 5)
    
    # Set a description in the data file
    myDescription <- "Hello Data File!"
    rxSetInfo( data = outFile, description = myDescription)
    fileInfo <- rxGetInfo(outFile)
    
    checkEquals( fileInfo$description, myDescription)
    myData <- rxReadXdf( outFile )
    
    # Make sure the data is still OK
    checkEquals(myData$x, 1:10)
    
    # Does the description get passed to the data frame?
    checkEquals(attr(myData, ".rxDescription"), myDescription )
    
    file.remove(outFile)
    
    # Does the description go from the data frame to the .xdf?
    rxDataStep(inData = myData, outFile = outFile )
    fileInfo <- rxGetInfo( outFile )
    dfInfo <- rxGetInfo(myData)
    checkEquals(fileInfo$description, myDescription)
    checkEquals(dfInfo$description, myDescription)
    
    myNewDescription = "Good-by data."
    myData <- rxSetInfo(data=myData, description = myNewDescription)
    checkEquals(rxGetInfo(myData)$description, myNewDescription)
       
    file.remove(outFile)
 }
    
###
# Multiple data sources

"test.rxGetInfo.args.data.multiple.sources" <- function()
{   
    DEACTIVATED("Cannot run in parallel on shipR. Filed TFS bug 1745")
  #####
  # Combinations of data sources using iris, airquality, and attitude data frames
  
  # list of data frames
  df.only.list <- rxGetInfo(list(iris, airquality, attitude))

  # list of .xdf paths
  irisXDF <- tempfile(pattern = "iris", fileext = ".xdf")
  airXDF <- tempfile(pattern = "airquality", fileext = ".xdf")
  attXDF <- tempfile(pattern = "attitude", fileext = ".xdf")
  irisXDF <- "iris.xdf"
  airXDF <- "air.xdf"
  attXDF <- "att.xdf"
  rxDataStep(iris, irisXDF, overwrite = TRUE)
  rxDataStep(airquality, airXDF, overwrite = TRUE)
  rxDataStep(attitude, attXDF, overwrite = TRUE)

  xdf.only.list <- rxGetInfo(list(irisXDF, airXDF, attXDF))
  xdf.only.c <- rxGetInfo(c(irisXDF, airXDF, attXDF))

  # list of RxXdfData objects
  irisDS <- RxXdfData(irisXDF)
  airDS <- RxXdfData(airXDF)
  attDS <- RxXdfData(attXDF)

  ds.only.list <- rxGetInfo(list(irisDS, airDS, attDS))
  ds.only.c <- rxGetInfo(c(irisDS, airDS, attDS))

  # list of heterogenous data sources
  ds.xdf.df <- rxGetInfo(list(irisDS, airXDF, attitude))

  # variable assignment calls
  x <- iris
  y <- airquality
  z <- attitude
  df.only.list.assigned.vars <- rxGetInfo(list(x, y, z))

  x <- irisDS
  y <- airXDF
  z <- attitude
  ds.xdf.df.assigned.vars <- rxGetInfo(list(x, y, z))
  
  # define standards for comparison
  iris.df.std <- rxGetInfo(iris)
  air.df.std <- rxGetInfo(airquality)
  att.df.std <- rxGetInfo(attitude)

  iris.xdf.std <- rxGetInfo(irisXDF)
  air.xdf.std <- rxGetInfo(airXDF)
  att.xdf.std <- rxGetInfo(attXDF)
    
  iris.ds.std <- rxGetInfo(irisDS)
  air.ds.std <- rxGetInfo(airDS)
  att.ds.std <- rxGetInfo(attDS)

  # Test and compare the results
  checkEquals(iris.df.std, df.only.list[[1L]])
  checkEquals(air.df.std, df.only.list[[2L]])
  checkEquals(att.df.std, df.only.list[[3L]])
  checkEquals(names(df.only.list), c("iris","airquality","attitude"))
    
  checkEquals(iris.xdf.std, xdf.only.list[[1L]])
  checkEquals(air.xdf.std, xdf.only.list[[2L]])
  checkEquals(att.xdf.std, xdf.only.list[[3L]])
  checkEquals(names(xdf.only.list), c("irisXDF","airXDF","attXDF"))
  
  checkEquals(iris.xdf.std, xdf.only.c[[1L]])
  checkEquals(air.xdf.std, xdf.only.c[[2L]])
  checkEquals(att.xdf.std, xdf.only.c[[3L]])
  checkEquals(names(xdf.only.c), c("iris.xdf","air.xdf","att.xdf"))

  checkEquals(iris.ds.std, xdf.only.list[[1L]])
  checkEquals(air.ds.std, xdf.only.list[[2L]])
  checkEquals(att.ds.std, xdf.only.list[[3L]])
  checkEquals(names(ds.only.list), c("irisDS","airDS","attDS"))
  
  checkEquals(iris.ds.std, xdf.only.c[[1L]])
  checkEquals(air.ds.std, xdf.only.c[[2L]])
  checkEquals(att.ds.std, xdf.only.c[[3L]])
  checkEquals(names(ds.only.c), c("irisDS","airDS","attDS"))

  checkEquals(iris.ds.std, ds.xdf.df[[1L]])
  checkEquals(air.xdf.std, ds.xdf.df[[2L]])
  checkEquals(att.df.std, ds.xdf.df[[3L]])
  checkEquals(names(ds.xdf.df), c("irisDS","airXDF","attitude"))

  nocomp <- which(names(iris.df.std) %in% c("objName"))
  checkEquals(iris.df.std[-nocomp], df.only.list.assigned.vars[[1L]][-nocomp])
  checkEquals(air.df.std[-nocomp], df.only.list.assigned.vars[[2L]][-nocomp])
  checkEquals(att.df.std[-nocomp], df.only.list.assigned.vars[[3L]][-nocomp])
  checkEquals(names(df.only.list.assigned.vars), c("x","y","z"))

  checkEquals(iris.ds.std, ds.xdf.df.assigned.vars[[1L]])
  checkEquals(air.xdf.std, ds.xdf.df.assigned.vars[[2L]])
  nocomp <- which(names(att.df.std) %in% c("objName"))
  checkEquals(att.df.std[-nocomp], ds.xdf.df.assigned.vars[[3L]][-nocomp])
  checkEquals(names(ds.xdf.df.assigned.vars), c("x","y","z"))
  
  #####
  # rxSplitXdf case
  splitPaths <- rxSplit(iris, splitByFactor = "Species", outFilesBase = file.path(tempdir(),"iris"), 
      reportProgress = 0, verbose = 0, overwrite = TRUE)

  rxSplit.df <- rxGetInfo(splitPaths)
  
  # check values
  checkTrue(grep("iris.Species.setosa.xdf", rxSplit.df[[1L]][["fileName"]]) > 0L)
    checkEquals(rxSplit.df[[1L]][["numRows"]], 50L)
    checkEquals(rxSplit.df[[1L]][["numVars"]], 5L)
    checkEquals(rxSplit.df[[1L]][["numBlocks"]], 1L)
  
  checkTrue(grep("iris.Species.versicolor.xdf", rxSplit.df[[2L]][["fileName"]]) > 0L)
    checkEquals(rxSplit.df[[2L]][["numRows"]], 50L)
    checkEquals(rxSplit.df[[2L]][["numVars"]], 5L)
    checkEquals(rxSplit.df[[2L]][["numBlocks"]], 1L)

  checkTrue(grep("iris.Species.virginica.xdf", rxSplit.df[[3L]][["fileName"]]) > 0L)
    checkEquals(rxSplit.df[[3L]][["numRows"]], 50L)
    checkEquals(rxSplit.df[[3L]][["numVars"]], 5L)
    checkEquals(rxSplit.df[[3L]][["numBlocks"]], 1L)

  #####
  # Direct inline definition
  
  direct.list <- rxGetInfo(list(data.frame(cow = 1:5, horse = 2:6), 
                  data.frame(pig = letters, goat = 1:26, dog = 26:1)))
  
  # check values
    checkEquals(direct.list[[1L]][["numRows"]], 5L)
    checkEquals(direct.list[[1L]][["numVars"]], 2L)
  
    checkEquals(direct.list[[2L]][["numRows"]], 26L)
    checkEquals(direct.list[[2L]][["numVars"]], 3L)

  #####
  # Clean up
  if (file.exists(irisXDF)) file.remove(irisXDF)
  if (file.exists(airXDF)) file.remove(airXDF)
  if (file.exists(attXDF)) file.remove(attXDF)
  invisible(sapply(splitPaths, function(x) if (file.exists(x@file)) file.remove(x@file)))
}

"test.rxGetInfoXdf.args.data.multiple.sources" <- function()
{   
        DEACTIVATED("Test fails when run in parallel with other tests. Created VSO work item bug 1745")
  #####
  # Combinations of data sources using iris, airquality, and attitude data frames
  
  # list of .xdf paths
  irisXDF <- tempfile(pattern = "iris", fileext = ".xdf")
  airXDF <- tempfile(pattern = "airquality", fileext = ".xdf")
  attXDF <- tempfile(pattern = "attitude", fileext = ".xdf")
  irisXDF <- "iris.xdf"
  airXDF <- "air.xdf"
  attXDF <- "att.xdf"
  rxDataStep(iris, irisXDF, overwrite = TRUE)
  rxDataStep(airquality, airXDF, overwrite = TRUE)
  rxDataStep(attitude, attXDF, overwrite = TRUE)

  xdf.only.list <- rxGetInfo(list(irisXDF, airXDF, attXDF))
  xdf.only.c <- rxGetInfo(c(irisXDF, airXDF, attXDF))

  # list of RxXdfData objects
  irisDS <- RxXdfData(irisXDF)
  airDS <- RxXdfData(airXDF)
  attDS <- RxXdfData(attXDF)

  ds.only.list <- rxGetInfo(list(irisDS, airDS, attDS))
  ds.only.c <- rxGetInfo(c(irisDS, airDS, attDS))

  # list of heterogenous data sources
  ds.xdf.ds <- rxGetInfo(list(irisDS, airXDF, attDS))

  # variable assignment calls
  x <- irisDS
  y <- airXDF
  z <- attDS
  ds.xdf.df.assigned.vars <- rxGetInfo(list(x, y, z))
  
  # define standards for comparison
  iris.xdf.std <- rxGetInfo(irisXDF)
  air.xdf.std <- rxGetInfo(airXDF)
  att.xdf.std <- rxGetInfo(attXDF)
    
  iris.ds.std <- rxGetInfo(irisDS)
  air.ds.std <- rxGetInfo(airDS)
  att.ds.std <- rxGetInfo(attDS)

  # Test and compare the results
  checkEquals(iris.xdf.std, xdf.only.list[[1L]])
  checkEquals(air.xdf.std, xdf.only.list[[2L]])
  checkEquals(att.xdf.std, xdf.only.list[[3L]])
  checkEquals(names(xdf.only.list), c("irisXDF","airXDF","attXDF"))
  
  checkEquals(iris.xdf.std, xdf.only.c[[1L]])
  checkEquals(air.xdf.std, xdf.only.c[[2L]])
  checkEquals(att.xdf.std, xdf.only.c[[3L]])
  checkEquals(names(xdf.only.c), c("iris.xdf","air.xdf","att.xdf"))

  checkEquals(iris.ds.std, xdf.only.list[[1L]])
  checkEquals(air.ds.std, xdf.only.list[[2L]])
  checkEquals(att.ds.std, xdf.only.list[[3L]])
  checkEquals(names(ds.only.list), c("irisDS","airDS","attDS"))
  
  checkEquals(iris.ds.std, xdf.only.c[[1L]])
  checkEquals(air.ds.std, xdf.only.c[[2L]])
  checkEquals(att.ds.std, xdf.only.c[[3L]])
  checkEquals(names(ds.only.c), c("irisDS","airDS","attDS"))

  checkEquals(iris.ds.std, ds.xdf.ds[[1L]])
  checkEquals(air.xdf.std, ds.xdf.ds[[2L]])
  checkEquals(att.ds.std, ds.xdf.ds[[3L]])
  checkEquals(names(ds.xdf.ds), c("irisDS","airXDF","attDS"))
  
  #####
  # rxSplitXdf case
  splitPaths <- rxSplit(iris, splitByFactor = "Species", outFilesBase = file.path(tempdir(),"iris"), 
      reportProgress = 0, verbose = 0, overwrite = TRUE)

  rxSplit.df <- rxGetInfo(splitPaths)
  
  # check values
  checkTrue(grep("iris.Species.setosa.xdf", rxSplit.df[[1L]][["fileName"]]) > 0L)
    checkEquals(rxSplit.df[[1L]][["numRows"]], 50L)
    checkEquals(rxSplit.df[[1L]][["numVars"]], 5L)
    checkEquals(rxSplit.df[[1L]][["numBlocks"]], 1L)
  
  checkTrue(grep("iris.Species.versicolor.xdf", rxSplit.df[[2L]][["fileName"]]) > 0L)
    checkEquals(rxSplit.df[[2L]][["numRows"]], 50L)
    checkEquals(rxSplit.df[[2L]][["numVars"]], 5L)
    checkEquals(rxSplit.df[[2L]][["numBlocks"]], 1L)

  checkTrue(grep("iris.Species.virginica.xdf", rxSplit.df[[3L]][["fileName"]]) > 0L)
    checkEquals(rxSplit.df[[3L]][["numRows"]], 50L)
    checkEquals(rxSplit.df[[3L]][["numVars"]], 5L)
    checkEquals(rxSplit.df[[3L]][["numBlocks"]], 1L)

  #####
  # Clean up
  if (file.exists(irisXDF)) file.remove(irisXDF)
  if (file.exists(airXDF)) file.remove(airXDF)
  if (file.exists(attXDF)) file.remove(attXDF)
  invisible(sapply(splitPaths, function(x) if (file.exists(x@file)) file.remove(x@file)))
}
