# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxGlm
#
############################################################################################
.setUp <- function()
{
    require(stats)
    RNGkind("Mersenne-Twister", "Inversion")
}

.tearDown <- function()
{
}

# Define global functions
# Here we copy the rtweedie frunction from the tweedie package
# so that we don't need to rely on it for the unit tests

"rtweedie" <- function (n, xi = power, mu, phi, power = NULL) 
{
    if (is.null(power) & is.null(xi)) 
        stop("Either xi or power must be given\n")
    xi.notation <- TRUE
    if (is.null(power)) 
        power <- xi
    if (is.null(xi)) {
        xi <- power
        xi.notation <- FALSE
    }
    if (xi != power) {
        cat("Different values for xi and power given; the value of xi used.\n")
        power <- xi
    }
    index.par <- ifelse(xi.notation, "xi", "p")
    if (power < 1) 
        stop("power must be greater than 1.\n")
    if (any(phi <= 0)) 
        stop("phi must be positive.")
    if (n < 1) 
        stop("n must be a positive integer.\n")
    if (any(mu <= 0)) 
        stop("mu must be positive.\n")
    if (length(mu) > 1) {
        if (length(mu) != n) 
            stop("mu must be scalar, or of length n.\n")
    }
    else {
        mu <- array(dim = n, mu)
    }
    if (length(phi) > 1) {
        if (length(phi) != n) 
            stop("phi must be scalar, or of length n.\n")
    }
    else {
        phi <- array(dim = n, phi)
    }
    if (power == 1) {
        rt <- phi * rpois(n, lambda = mu/(phi))
    }
    if (power == 2) {
        alpha <- (2 - power)/(1 - power)
        gam <- phi * (power - 1) * mu^(power - 1)
        rt <- rgamma(n, shape = 1/phi, scale = gam)
    }
    if (power > 2) {
        rt <- qtweedie(runif(n), mu = mu, phi = phi, power = power)
    }
    if ((power > 1) & (power < 2)) {
        rt <- array(dim = n, NA)
        lambda <- mu^(2 - power)/(phi * (2 - power))
        alpha <- (2 - power)/(1 - power)
        gam <- phi * (power - 1) * mu^(power - 1)
        N <- rpois(n, lambda = lambda)
        for (i in (1:n)) {
            rt[i] <- sum(rgamma(N[i], shape = -alpha, scale = gam[i]))
        }
    }
    as.vector(rt)
}


"test.rxGlm.family.binomial.glm.rxGlm.comparison.link.logit" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
  # This test compares the outputs of glm, rxGlm, and rxGlm for the binomial case,
  # Calculations for binomial(link = "logit") family-link in rxGlm is currently performed in C++.
  
  # Ingest small airline data and convert to .xdf file
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall1st10kOf6.csv") # Has some missing data
    XDF <- file.path(tempdir(), "ADS.xdf")
    rxTextToXdf(inFile, XDF, transforms = list(ArrOnTime = ArrDelay < 2),
                colInfo=list(DayOfWeek = list(type = "factor")), overwrite = TRUE)
  DF <- rxDataStep(XDF)
  
  # Compare the outputs of glm, rxGlm, and rxGlm(..., family = binomial(link = "logit"))
  form <- ArrOnTime ~ CRSDepTime
    zGLM <- glm(form, data = DF, family = binomial(link = "logit"))
    zRXLOGIT <- rxLogit(form,  data = XDF)
    zRXGLM <- rxGlm(form, data = XDF, family = binomial(link = "logit"))

    # Compare regression coefficients
    coefGLM <- coef(zGLM)
    coefRXLOGIT <- coef(zRXLOGIT)
    coefRXGLM <- coef(zRXGLM)

  tol <- 1e-7
    checkEqualsNumeric(coefGLM, coefRXLOGIT, tolerance = tol)
    checkEqualsNumeric(coefRXLOGIT, coefRXGLM, tolerance = tol)
  
  ###
  # Repeat test with the inclusion of DayOfWeek (a factor)
  form <- ArrOnTime ~ CRSDepTime + DayOfWeek
    zGLM <- glm(form, data = DF, family = binomial(link = "logit"), contrasts=list(DayOfWeek = "contr.SAS"))
    zRXLOGIT <- rxLogit(form,  data = XDF)
    zRXGLM <- rxGlm(form, data = XDF, family = binomial(link = "logit"))

    # Compare regression coefficients
    coefGLM <- coef(zGLM)
    coefRXLOGIT <- coef(zRXLOGIT)
    coefRXGLM <- coef(zRXGLM)
    checkEqualsNumeric(coefGLM, coefRXLOGIT[!is.na(coefRXLOGIT)], tolerance = tol)
    checkEqualsNumeric(coefRXLOGIT, coefRXGLM, tolerance = tol)
  file.remove(XDF)
}

"test.rxGlm.family.binomial.glm.comparison.link.probit" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
  # This test compares the outputs of rxGlm, and rxGlm for the binomial case,
  # Calculations for binomial(link = "probit") family-link in rxGlm is currently performed in R.
  
  # Ingest small airline data and convert to .xdf file
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall1st10kOf6.csv") # Has some missing data
    XDF <- file.path(tempdir(), "ADS.xdf")
    rxTextToXdf(inFile, XDF, transforms = list(ArrOnTime = ArrDelay < 2),
                colInfo=list(DayOfWeek = list(type = "factor")), overwrite = TRUE)
  DF <- rxDataStep(XDF)
  
  # Compare the outputs of rxGlm, and rxGlm(..., family = binomial(link = "probit"))
  form <- ArrOnTime ~ CRSDepTime
    zGLM <- glm(form, data = DF, family = binomial(link = "probit"))
    zRXGLM <- rxGlm(form, data = XDF, family = binomial(link = "probit"))

    # Compare regression coefficients
    coefGLM <- coef(zGLM)
    coefRXGLM <- coef(zRXGLM)

  tol <- 1e-7
    checkEqualsNumeric(coefGLM, coefRXGLM, tolerance = tol)
  
  ###
  # Repeat test with the inclusion of DayOfWeek (a factor)
  form <- ArrOnTime ~ CRSDepTime + DayOfWeek
    zGLM <- glm(form, data = DF, family = binomial(link = "probit"), contrasts=list(DayOfWeek = "contr.SAS"))
    zRXGLM <- rxGlm(form, data = XDF, family = binomial(link = "probit"))

    # Compare regression coefficients
    coefGLM <- coef(zGLM)
    coefRXGLM <- coef(zRXGLM)
    checkEqualsNumeric(coefGLM, coefRXGLM[!is.na(coefRXGLM)], tolerance = tol)
}

###
# Tweedie Distributions in Generalized Linear Models
#
# The variance power p characterizes the distribution of the responses y. 
# The following are some special cases:
#
# p	Response distribution
# -----------------------
# 0         Normal
# 1         Poisson
# (1, 2)    Compound Poisson, non-negative with mass at zero
# 2	        Gamma
# 3         Inverse-Gaussian
# > 2       Stable, with support on the positive reals

"test.rxGlm.family.tweedie.poisson.link.identity" <- function()
{
  # Generate faux data
  set.seed(100)
    nobs <- 20L
    DF <- data.frame(y = rgamma(nobs, shape = 5), x = seq(nobs))

    # Define model components
    form <- y ~ x
    var.power <- 1  # Poisson
    link.power <- 1 # Identity
    
    # The following was calculated using the tweedie package via:
    #
  #  zGLM <- glm(form, data = DF, family = tweedie(var.power = var.power, link.power = link.power))
  #  glmCoefs <- coef(zGLM)
    glmCoefs <- c("(Intercept)" = 3.91099702, x = 0.08004823)

    # Compare to rxGlm
    zRXGLM <- rxGlm(form, data = DF, family = rxTweedie(var.power = var.power, link.power = link.power))
    rxglmCoefs <- coef(zRXGLM)
    #(Intercept)           x 
    #3.90714939  0.08041467 
    checkEqualsNumeric(glmCoefs, rxglmCoefs, tol = 1e-7) #Failing 1e-02 works
}

"test.rxGlm.family.tweedie.inverse-gaussian.link.log" <- function()
{
  # Generate faux data
  set.seed(100)
    nobs <- 20L
    DF <- data.frame(y = rgamma(nobs, shape = 5), x = seq(nobs))

    # Define model components
    form <- y ~ x
    var.power <- 3  # Inverse-Gaussian
    link.power <- 0 # Log
    
    # The following was calculated using the tweedie package via:
    #
    #zGLM <- glm(form, data = DF, family = tweedie(var.power = var.power, link.power = link.power))
    #glmCoefs <- coef(zGLM)
    glmCoefs <- c("(Intercept)" = 1.37235317, x = 0.01729206)

    # Compare to rxGlm
    zRXGLM <- rxGlm(form, data = DF, family = rxTweedie(var.power = var.power, link.power = link.power))
    rxglmCoefs <- coef(zRXGLM)
    checkEqualsNumeric(glmCoefs, rxglmCoefs, tol = 1e-7)
}

"test.rxGlm.family.tweedie.normal.link.identify" <- function()
{
  # Generate faux data
  set.seed(100)
    nobs <- 20L
    DF <- data.frame(y = rnorm(nobs), x = seq(nobs))

    # Define model components
    form <- y ~ x
    var.power <- 0  # Gaussian
    link.power <- 1 # Identity
    
    # The following was calculated using the tweedie package via:
    #
    #zGLM <- glm(form, data = DF, family = tweedie(var.power = var.power, link.power = link.power))
    #glmCoefs <- coef(zGLM)
    glmCoefs <- c("(Intercept)" = -0.14830307, x = 0.02439716)

    # Compare to rxGlm
    zRXGLM <- rxGlm(form, data = DF, family = rxTweedie(var.power = var.power, link.power = link.power))
    rxglmCoefs <- coef(zRXGLM)
    checkEqualsNumeric(glmCoefs, rxglmCoefs, tol = 1e-7)
}

"test.rxGlm.family.tweedie.compound-poisson.link.identify" <- function()
{
  # Generate faux data
  set.seed(100)
    
    nobs <- 2000L
    power <- 1.2
    DF <- data.frame(y = rtweedie(nobs, power = power, mu = 1, phi = 1), x = seq(nobs))

    # Define model components
    form <- y ~ x
    var.power <- power  # Compound-Poisson (exact zeros + Poisson for values > 0)
    link.power <- 1 # Identity
    
    # The following was calculated using the tweedie package via:
    #
    #zGLM <- glm(form, data = DF, family = tweedie(var.power = var.power, link.power = link.power))
    #glmCoefs <- coef(zGLM)
    glmCoefs <- c("(Intercept)" = 1.057892e+00, x = -3.189217e-05)

    # Compare to rxGlm
    zRXGLM <- rxGlm(form, data = DF, family = rxTweedie(var.power = var.power, link.power = link.power))
    rxglmCoefs <- coef(zRXGLM)
    print(tail(DF$y))
    print(rxglmCoefs)
    print(glmCoefs)
    print(RNGkind())
    checkEqualsNumeric(glmCoefs, rxglmCoefs, tol = 1e-6)
}

"test.rxGlm.family.tweedie.gamma.link.identify" <- function()
{
  # Generate faux data
  set.seed(100)
    nobs <- 20L
    DF <- data.frame(y = rgamma(nobs, shape = 5), x = seq(nobs))

    # Define model components
    form <- y ~ x
    var.power <- 2  # Gamma
    link.power <- 1 # Identity
    
    # The following was calculated using the tweedie package via:
    #
    #zGLM <- glm(form, data = DF, family = tweedie(var.power = var.power, link.power = link.power))
    #glmCoefs <- coef(zGLM)
    glmCoefs <- c("(Intercept)" = 3.89354203, x = 0.08180512)

    # Compare to rxGlm
    zRXGLM <- rxGlm(form, data = DF, family = rxTweedie(var.power = var.power, link.power = link.power))
    rxglmCoefs <- coef(zRXGLM)
    checkEqualsNumeric(glmCoefs, rxglmCoefs, tol = 1e-6)
}

"test.rxGlm.na" <- function()
{
    # Create a copy of the 'infert' dataset
    infert.na <- infert
    
    # Randomly add missing values to the dataset
    infert.na[17,1] <- NA
    infert.na[16,2] <- NA
    infert.na[7,4] <- NA
    infert.na[16,5] <- NA
    infert.na[9,6] <- NA
    infert.na[12,7] <- NA

    form <- case ~ age + parity + spontaneous + induced
    infert.glm.NA <- glm(form, data=infert.na, na.action = na.omit, family = binomial())
    infert.rxGlm.NA <- rxGlm(form, data = infert.na, family = binomial())

    # Check that coefficient estimates are the same
    mysum.glm <- summary(infert.glm.NA)
    mycoef.glm <- mysum.glm$coefficients[,1L]
    names(mycoef.glm) <- NULL
    myrxGlm.sum <- summary(infert.rxGlm.NA)
    checkEquals(mycoef.glm,as.vector(myrxGlm.sum$coefficients[,1L]),tol=.001)

    # Check covariance matrices for equality
    #mysum.glm$cov.values <- c(mysum.glm$cov.unscaled[1,1],mysum.glm$cov.unscaled[2,2])
    #checkEqualsNumeric(as.vector(mysum.glm$cov.values),diag(infert.rxGlm.NA$covCoef)[1:2],tol=.001)

    # Check for equal degrees of freedom
    checkEquals(mysum.glm$df.residual, myrxGlm.sum$df[2])
}


###
# Argument Tests
###

"test.rxGlm.args.formula" <- function()
{
    form <- case ~ age + parity + education + spontaneous + induced
    formStr <- deparse(form)
    zGlmFormula <- rxGlm(form, data=infert)
    zGlmString <- rxGlm(formStr, data=infert)
    nocomp <- which(names(zGlmString) %in% c("call","params","formula"))
    checkTrue(all.equal(zGlmFormula[-nocomp], zGlmString[-nocomp], check.attributes=FALSE))
    checkEqualsNumeric(coef(zGlmFormula)[["induced"]], 0.2196706, tol = 1e-6)
}

"test.rxGlm.args.data" <- function()
{
    form <- case ~ age + parity + education + spontaneous + induced
    XDF <- tempfile(pattern = ".rxGlmInfert", fileext = ".xdf")
    on.exit(unlink(XDF), add = TRUE)
    rxDataStep(infert, XDF)
    
    zGlmDF <- rxGlm(form, data = infert)
    zGlmXDF <- rxGlm(form, data = XDF, verbose = 0L)
    nocomp <- which(names(zGlmXDF) %in% c("call","params"))
    checkTrue(all.equal(zGlmDF[-nocomp], zGlmXDF[-nocomp], check.attributes=FALSE))
}

"test.rxGlm.args.family" <- function()
{
    # Define new family: Logistical Exposure Link Function
    # Reference: Shaffer, T. 2004. A unifying approach to analyzing nest success.
    # See also: http://www.npwrc.usgs.gov/resource/birds/nestsurv/download/CreateLogisticExposureFamily.R
    "logisticExposure" <- function(days = 1)
    {
        linkfun <- function(mu) qlogis(mu^(1/days))
        linkinv <- function(eta) plogis(eta)^days
    if (compareVersion(paste(R.version$major, R.version$minor, sep="."), "3.0.1") < 0) {
      mu.eta <- function(eta) days * plogis(eta)^(days-1) *
        .Call("logit_mu_eta", eta, PACKAGE="stats")
    }
    else
    {
      mu.eta <- function(eta) days * plogis(eta)^(days-1) *
        .Call(stats:::C_logit_mu_eta, eta)			
    }
        valideta <- function(eta) TRUE
        link <- paste("logexp(", days, ")", sep="")
        structure(list(linkfun = linkfun, linkinv = linkinv,
                       mu.eta = mu.eta, valideta = valideta, name = link),
                  class = "link-glm")
    }
    
    #nestSurvivalDF <- read.table("http://data.prbo.org/tools/NestSurvival/chat.txt")
    nestSurvivalDF <- rxDataStep(file.path(rxGetOption("unitTestDataDir"), "NestSurvival.xdf"))
    
    form <- survive / trials ~ parastat + nest_ht * patsize
    family <- binomial(logisticExposure(days = nestSurvivalDF[["expos"]]))
    
    # glm() works as expected
    chat.glm.logexp <- glm(form, family = family, data = nestSurvivalDF, contrasts=list(patsize = "contr.SAS"))
    
    # rxGlm() crashes the RPE
    oldNumCores <- rxGetOption("numCoresToUse")
    rxOptions(numCoresToUse = 1L)
    chat.rxGlm.logexp <- rxGlm(form, family = family, data = nestSurvivalDF)
    rxOptions(numCoresToUse = oldNumCores)
    
    # Compare the results
    coefGLM <- coef(chat.glm.logexp)
    coefRXGLM <- coef(chat.rxGlm.logexp)
    checkEqualsNumeric(coefGLM, coefRXGLM[!is.na(coefRXGLM)])
}

"test.rxGlm.args.pweights" <- function()
{
   set.seed(100)
   test.y <- factor(c(rep(rbinom(50,1,.5),2), rbinom(50,1,.5)))
   test.x <- c(rep(runif(50),2),runif(50))

   # now weight the first 50 cases by a factor of 2 and then
   # eliminate the next 50 cases
   test.y2 <- test.y[c(1:50,101:150)]
   test.x2 <- test.x[c(1:50,101:150)]
   wts <- rep(c(2,1), c(50,50))
   fam <- binomial()
   form <- test.y2 ~ test.x2
   DF <- data.frame(test.y2, test.x2, wts)

   glmSummary <- summary(glm(form, data = DF, family = fam, weights = wts), cor = FALSE)
   rxGlmSummary <- summary(rxGlm(form, data = DF, family = fam, pweights = "wts", covCoef = TRUE))

   # Check that coefficient estimates are the same
   mycoef.glm <- glmSummary[["coefficients"]][, 1L]
   names(mycoef.glm) <- NULL
   checkEquals(mycoef.glm,as.vector(rxGlmSummary[["coefficients"]][, 1L]), tol = .001)

   # Check covariance matrices for equality
   glmSummary[["cov.values"]] <- c(glmSummary$cov.unscaled[1L, 1L], glmSummary[["cov.unscaled"]][2L, 2L])
   checkEqualsNumeric(as.vector(glmSummary[["cov.values"]]), diag(rxGlmSummary[["covCoef"]]),tol = 1e-3)

   # Check for equal degrees of freedom
   checkEquals(glmSummary[["df.residual"]], rxGlmSummary[["df"]][2L])
}

"test.rxGlm.args.fweights" <- function()
{
    # create frequency weighting vector whose length is equal to the
    # number of rows in our test data frame. select weight randomly from 
    # the set of integers on the interval [1,4]
    set.seed(10)
    DF <- infert
    DF[["fwts"]] <- sample(1:4, nrow(DF), replace = TRUE)
    
    # create new data frame where the rows are replictaed explicitly according
    # to the number frequency weighting vector
    DF2 <- as.data.frame(lapply(DF, rep, times = DF[["fwts"]]))
    DF2[["fwts"]] <- NULL

    # define common formula    
    form <- case ~ age + parity + education + spontaneous + induced

    # calculate logisitc regression coefficients for the two cases and compare
    z1 <- coef(rxGlm(form, data = DF, family = binomial, fweights = "fwts", reportProgress = 0))
    z2 <- coef(rxGlm(form, data = DF2, family = binomial, reportProgress = 0))
    checkTrue(all.equal(z1, z2))    
}

"test.rxGlm.args.cube" <- function()
{
    # Create a simple data set with 50 values each of a factor set to -1 and 1
    set.seed(100)
    x <- rep(c(-1,1), times=50)
    ytemp <- x + rnorm(100)
    y <- ytemp > 0
    myData <- data.frame(xfac = factor(x), x, y)
    logRes <- rxLogit(y ~ xfac, data = myData, cube = TRUE, verbose = 0L)
    checkEquals(logRes$countDF$Counts, c(50, 50), check.attributes = FALSE)
    glmRes <- glm(y ~ xfac + 0, data = myData, family = binomial())
    checkEquals(coef(logRes), coef(glmRes), check.attributes=FALSE)
    rxGlmRes <- rxGlm(y ~ xfac + 0, data = myData, family = binomial(), cube = TRUE)
    checkEquals(coef(rxGlmRes), coef(glmRes), check.attributes=FALSE)
}

# No current plans to implement this functionality 3/22/12
#"test.rxGlm.args.cubePredictions" <- function()
#{
    #DEACTIVATED("Pending fix for RA-5552")
    ## logical flag. If TRUE and cube is TRUE the estimated model is evaluated (predicted) 
    ## for each cell in the cube, fixing the non-cube variables in the model at their mean 
    ## values, and these predictions are included in the countDF component of the returned 
    ## value. This may be time and memory intensive for large models.
    #set.seed(100)
    #x <- rep(c(-1,1), times=50)
    #x1 <- rnorm(100)
    #ytemp <- x + 2 * x1 + rnorm(100)
    #y <- ytemp > 0
    #form <- y ~ xfac + x1
    #DF <- data.frame(xfac = factor(x), x, x1, y)
    #
    #rxGlmTrue <- rxGlm(form, data = DF, family = binomial, cube = TRUE, cubePredictions = TRUE, reportProgress = 0)
    #checkTrue(all(names(rxGlmTrue[["countDF"]]) == c("xfac", "y", "Counts")))
    #
    #rxGlmFalse <- rxGlm(form, data = DF, family = binomial, cube = TRUE, cubePredictions = FALSE, reportProgress = 0)
    #checkTrue(all(names(rxGlmFalse[["countDF"]]) == c("xfac", "Counts")))
    #checkTrue(is.null(rxGlmFalse[["countDF"]][["y"]]))
#}

"test.rxGlm.args.rowSelection" <- function()
{
    form <- case ~ age + parity + education + spontaneous + induced
    zGlmRowSelectionImplicit <- rxGlm(form, data=infert, rowSelection=pooled.stratum > 20 & pooled.stratum < 50)
    zGlmRowSelectionExplicit <- rxGlm(form, data=subset(infert, pooled.stratum > 20 & pooled.stratum < 50))
    nocomp <- which(names(zGlmRowSelectionExplicit) %in% c("call","params","nMissingObs"))
    checkTrue(all.equal(zGlmRowSelectionImplicit[-nocomp], zGlmRowSelectionExplicit[-nocomp], check.attributes=FALSE))
}

"test.rxGlm.args.transforms" <- function()
{
    zGlm1 <- rxGlm(case ~ Age + parity, data = infert, transforms = list(HalfAge = age / 2, Age = HalfAge * 2))
    zGlm2 <- rxGlm(case ~ age + parity, data = infert)
    nocomp <- which(names(zGlm1) %in% c("call","params", "formula"))
    checkEquals(zGlm1[-nocomp], zGlm2[-nocomp], check.attributes = FALSE)
    checkException(rxGlm(case ~ Age + parity, data = infert), silent = TRUE)
}

"test.rxGlm.args.transformObjects" <- function()
{
    "xform" <- function(dataList)
    {
        dataList[["HalfAge"]] <- Half(dataList[["age"]])
        dataList[["Age"]] <- Double(dataList[["HalfAge"]])
        dataList
    }

    transformObjects <- list(Half <- function(x) x / 2,
                             Double <- function(x) x * 2)
                        
    zGlm1 <- rxGlm(case ~ Age + parity, data = infert, transformFunc = xform, transformVars = "age")
    zGlm2 <- rxGlm(case ~ age + parity, data = infert)
    nocomp <- which(names(zGlm1) %in% c("call","params", "formula"))
    checkEquals(zGlm1[-nocomp], zGlm2[-nocomp], check.attributes = FALSE)
}

"test.rxGlm.args.transformFunc" <- function()
{
    "xform" <- function(dataList)
    {
        dataList[["HalfAge"]] <- dataList[["age"]] / 2 
        dataList[["Age"]] <- dataList[["HalfAge"]] * 2
        dataList
    }
    
    zGlm1 <- rxGlm(case ~ Age + parity, data = infert, transformFunc = xform, transformVars = "age")
    zGlm2 <- rxGlm(case ~ age + parity, data = infert)
    nocomp <- which(names(zGlm1) %in% c("call","params", "formula"))
    checkEquals(zGlm1[-nocomp], zGlm2[-nocomp], check.attributes = FALSE)
    checkException(rxGlm(case ~ Age + parity, data = infert), silent = TRUE)
}

"test.rxGlm.args.transformVars" <- function()
{
    "xform" <- function(dataList)
    {
        dataList[["HalfAge"]] <- dataList[["age"]] / 2 
        dataList[["Age"]] <- dataList[["HalfAge"]] * 2
        dataList
    }
    
    checkException(rxGlm(case ~ Age + parity, data = infert, transformFunc = xform), silent = TRUE)
}

"test.rxGlm.args.transformEnvir" <- function()
{
    if (exists("Half")) rm("Half")
    if (exists("Double")) rm("Double")
        
    envir <- new.env(hash = TRUE, parent = baseenv())
    
    "xform" <- function(dataList)
    {
        dataList[["HalfAge"]] <- Half(dataList[["age"]])
        dataList[["Age"]] <- Double(dataList[["HalfAge"]])
        dataList
    }

    assign("Half", function(x) x / 2, envir = envir)
    assign("Double", function(x) x * 2, envir = envir)
    zGlm1 <- rxGlm(case ~ Age + parity, data = infert, 
                   transformFunc = xform, 
                   transformVars = "age",
                   transformEnvir = envir)

    zGlm2 <- rxGlm(case ~ age + parity, data = infert)
    nocomp <- which(names(zGlm1) %in% c("call","params", "formula"))
    checkEquals(zGlm1[-nocomp], zGlm2[-nocomp], check.attributes = FALSE)
    checkException(rxGlm(case ~ Age + parity, data = infert, 
                   transformFunc = xform, 
                   transformVars = "age"), silent = TRUE)
}

"test.rxGlm.args.dropFirst" <- function()
{
    zDropFirstFalse <- rxGlm(Sepal.Width ~ Species, iris, family = gaussian(), dropFirst = FALSE)
    zDropFirstTrue <- rxGlm(Sepal.Width ~ Species, iris, family = gaussian(), dropFirst = TRUE)
    checkTrue(is.na(coef(zDropFirstFalse)[["Species=virginica"]]))
    checkTrue(is.na(coef(zDropFirstTrue)[["Species=setosa"]]))
}

"test.rxGlm.args.covCoef" <- function()
{
    .isNumericMatrix <- function(x) is.matrix(x) && is.numeric(x)
    
    form <- case ~ age + parity + education + spontaneous + induced
    
    # test various combinations of covCoef and covData
    z <- rxGlm(form, data=infert, family = binomial(link = "logit"), covCoef = TRUE)
    checkTrue(.isNumericMatrix(z[["covCoef"]]))
 
    z <- rxGlm(form, data=infert, family = binomial(link = "logit"), covCoef = FALSE)
    checkTrue(is.null(z[["covCoef"]]))
}

"test.rxGlm.args.initialValues" <- function()
{
    ###
    #NA : The initial values are based upon the 'mustart' values generated by the family$initialize expression if 
    #an R family object is used, or the equivalent for a family coded in C++.
    form <- case ~ age + parity + education + spontaneous + induced
    fam <- binomial(link = "logit")
    zNA <- rxGlm(form, infert, family = fam, initialValues = NA)
    
    ###
    #Scalar : A numeric scalar that will be replicated once for each of the coefficients in the model to form the 
    #initial values. Zeros may be good starting values, depending upon the model.
    zZeroScalar <- rxGlm(form, data=infert, family = fam, initialValues = 0)
    
    ###
    #Vector : A numeric vector of length K, where K is the number of model coefficients, including the intercept 
    #if any and including any that might be dropped from sets of dummies. This argument is most useful when a set 
    #of estimated coefficients is available from a similar set of data. Note that K can be found by running the model 
    #on a small number of observations and obtaining the number of coefficients from the output, say z, via 
    #length(z$coefficients).
    zZeroVector <- rxGlm(form, data=infert, family = fam, initialValues = rep(0, length(coef(zNA))))
    
    nocomp <- which(names(zNA) %in% c("call","params","iterations"))
    checkEquals(zNA[-nocomp], zZeroScalar[-nocomp], check.attributes = FALSE, tol = 1e-5)
    checkEquals(zZeroScalar[-nocomp], zZeroVector[-nocomp], check.attributes = FALSE)
    
    checkException(rxGlm(form, data=infert, family = fam, initialValues = rep(0, length(coef(zNA)) + 1L)), silent = TRUE)
    checkException(rxGlm(form, data=infert, family = fam, initialValues = Inf), silent = TRUE)
    checkException(rxGlm(form, data=infert, family = fam, initialValues = "pi"), silent = TRUE)
}

"test.rxGlm.args.blocksPerRead" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    form <- ybinint ~ xnum2miss
    zGlmBlocksDefault <- rxGlm(form, data=test100r5b, blocksPerRead=-1)
    zGlmBlocks1 <- rxGlm(form, data=test100r5b, blocksPerRead=1)
    nocomp <- which(names(zGlmBlocks1) %in% c("call","params"))
    checkTrue(all.equal(zGlmBlocksDefault[-nocomp], zGlmBlocks1[-nocomp], check.attributes=FALSE))
    zGlmBlocks5 <- rxGlm(form, data=test100r5b, blocksPerRead=5)
    checkTrue(all.equal(zGlmBlocksDefault[-nocomp], zGlmBlocks5[-nocomp], check.attributes=FALSE))
}

"test.rxGlm.args.maxIterations" <- function()
{
    # See RA-5550
    
    # Define common arguments
    require(stats)
    form <- ArrOnTime ~ CRSDepTime + DayOfWeek
    fam <- binomial(link = "logit")
    sasContrast <- list(DayOfWeek = "contr.SAS")

    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
    airlineDF <- rxDataStep(inData = inFile)
    DF <- cbind(airlineDF, ArrOnTime =((airlineDF$ArrDelay > -2) & (airlineDF$ArrDelay < 2)))

    # With no limitation on max iterations  
    zGLM <- glm(form, data = DF, contrasts = sasContrast, family = fam)
    zRXGLM <- rxGlm(form, data = DF, family = fam, initialValues = NA)

    # Show the number of iterations to convergence: these currently differ by one
    checkEquals(zGLM$iter, zRXGLM$iterations)

    # However, if we limit the max iterations then all is well
    maxit <- 3L
    glmControl <- glm.control(maxit = maxit)
    zGLM <- glm(form, data = DF, contrasts = sasContrast, control = glmControl, family = fam)
    zRXGLM <- rxGlm(form, data = DF, family = fam, initialValues = NA, maxIterations = maxit)

    checkEquals(zGLM$iter, 3L)
    checkEquals(zRXGLM$iterations, 3L)
}

#"test.rxGlm.args.coeffTolerance" <- function()
#{
#}
#
#"test.rxGlm.args.gradientTolerance" <- function()
#{
#}
#
#"test.rxGlm.args.objectiveFunctionTolerance" <- function()
#{
#}

"test.rxGlm.args.offset" <- function()
{
    utils::data(anorexia, package = "MASS")
    glmOffset <- glm(Postwt ~ Prewt + Treat + offset(Prewt), family = gaussian, data = anorexia, contrast = list(Treat = "contr.SAS"))
    rxGlmOffsetFormula <- rxGlm(Postwt ~ Prewt + Treat + offset(Prewt), family = gaussian, data = anorexia)
    rxGlmOffsetFormulaString <- rxGlm("Postwt ~ Prewt + Treat + offset(Prewt)", family = gaussian, data = anorexia)
    rxGlmOffsetDirect <- rxGlm(Postwt ~ Prewt + Treat, family = gaussian, data = anorexia, offset = "Prewt")
    checkException(rxGlm(Postwt ~ Prewt + Treat + offset(Prewt), family = gaussian, data = anorexia, offset = "Prewt"))
    
    checkEqualsNumeric(coef(glmOffset), na.omit(coef(rxGlmOffsetFormula)))
    checkEqualsNumeric(coef(rxGlmOffsetFormulaString), coef(rxGlmOffsetFormula))
    checkEqualsNumeric(coef(rxGlmOffsetDirect), coef(rxGlmOffsetFormula))
}

"test.rxGlm.args.offset.formula.arithmetic.expressions" <- function()
{
    utils::data(cats, package = "MASS")
    utils::data(Insurance, package = "MASS")

  form <- log(Hwt) ~ Sex*log(Bwt) + offset(log(Bwt))
  zglm <- glm(form, data = cats, contrast = list(Sex = "contr.SAS")) 
  zrxglm <- rxGlm(form, data = cats, reportProgress = 0L)
  checkEqualsNumeric(coef(zglm), na.omit(coef(zrxglm)))

  form <- Claims ~ District + Group + Age + offset(log(Holders)/2 + 3)
  zglm <- glm(form, data = Insurance, family = poisson(link = "log"), 
    contrasts = list(District = "contr.SAS", Age = "contr.SAS", Group = "contr.SAS"))
  zrxglm <- rxGlm(form, data = Insurance, family = poisson(link = "log"), reportProgress = 0L)
  checkEqualsNumeric(coef(zglm), na.omit(coef(zrxglm)))
}

"test.rxGlm.args.reportProgress" <- function()
{
    # reportProgress argument stress tests
    form <- case ~ age + parity + education + spontaneous + induced

    #reportProgress: logical, character string or integer: 
    #  0: no progress is reported. 
    #  1: the number of processed rows is printed and updated. 
    #  2: rows processed and timings are reported. 
    #  3: rows processed and all timings are reported. 

    variations <- 0:3
    results <- printOutput <- vector("list", length = length(variations))

    for (i in seq_along(variations))
    {
        printOutput[[i]] <- capture.output(results[[i]] <- 
            rxGlm(form = form, data = infert, famil = binomial, reportProgress = variations[i]))
    }

    # check that all values are the same
    nocomp <- which(names(results[[1L]]) %in% c("call", "params"))
    checkTrue(all(unlist(lapply(results[-1L], function(x)
              all.equal(x[- nocomp], results[[1L]][- nocomp],
                        check.attributes=FALSE)))))

    names(printOutput) <- as.character(variations)

    noResponseExpected <- "0"
    equalLengthExpected <- "2"
    equalLengthExpected1 <- "1"

    checkTrue(length(unlist(printOutput[noResponseExpected])) == 0L)

    # just row numbers
    checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1], length)) == 6L))

    # details gives us the goods
    checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=248, TotalRowsProcessed=248, ReadTime", 
              printOutput[["3"]][1L])) > 0)

    # time
    checkTrue(length(grep("^Elapsed computation time:", printOutput[["2"]][17L])) > 0)
}

"test.rxGlm.args.verbose" <- function()
{
    form <- case ~ spontaneous + induced
    o1 <- capture.output(t1 <- rxGlm(form, data=infert, family = binomial, reportProgress=0))
    o2 <- capture.output(t2 <- rxGlm(form, data=infert, family = binomial, verbose=0, reportProgress=0))
    o3 <- capture.output(t3 <- rxGlm(form, data=infert, family = binomial, verbose=1))  
    o4 <- capture.output(t4 <- rxGlm(form, data=infert, family = binomial, verbose=1))
    o5 <- capture.output(t5 <- rxGlm(form, data=infert, family = binomial, verbose=2))

    # No output if verbose is 0(the default)
    checkEquals(length(o1), 0)
    checkEquals(length(o2), 0)
    checkTrue(length(o3) > 1)
    checkTrue(nchar(o4[1]) > 1)
    checkTrue(length(o3) < length(o5))
    
    o7 <- capture.output(t7 <- rxGlm(form, data=infert, family = binomial, reportProgress=1))
    checkTrue(length(o7) > 0)
    
    # Check to make sure there is no output if cube = TRUE
    form <- case ~ education + spontaneous + induced
    o8 <- capture.output(t1 <- rxGlm(form, data=infert, family = binomial, cube = TRUE, reportProgress = 0L))
    checkEquals(length(o8), 0)
    
    o9 <- capture.output(t1 <- rxGlm(form, data=infert, family = binomial, cube = TRUE, reportProgress = 2L))
    checkTrue(length(o9) > 0)
    
    o10 <- capture.output(t1 <- rxGlm(form, data=infert, family = binomial, cube = TRUE,
        reportProgress = 0L, verbose = 1L))
    checkTrue(length(o10) > 0)
    
    o11 <- capture.output(t1 <- rxGlm(form, data=infert, family = binomial, cube = TRUE,
        reportProgress = 0L, verbose = 2L))
    checkTrue(length(o11) > length(10))
}

