# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxGlm with variableSelection, i.e., stepwise generalized linear regression
#
############################################################################################

.setUp <- function()
{
    ## NOTE: specifying contrasts in options changes the way how factors are labeled!
    #.op.old <<- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
  
  ## various test modes
  # R		-- mostly compatible with R
  # SAS	-- SAS features such as stepCriterion = "SigLevel"
  # SWEEP	-- perform variable selection with the SSCP matrix
  # REVO	-- the "correct" way but might be different from R 
    .test.mode <<- "R" #"SAS" #"SWEEP"	#"REVO"	#
  
    rxOptions(reportProgress = 0) # by default, don't report progress
}

.tearDown <- function()
{
    #options(.op.old)
    if (exists(".test.mode", where = ".GlobalEnv")) remove(".test.mode", pos = ".GlobalEnv")
    rxOptions(TRUE) # reset all options
}

#######################################################################
## local utility functions

compareGlmStepResults <- function(rlm, rxlm, 
    tolerance.aic = .Machine$double.eps^0.5, tolerance.coef = .Machine$double.eps^0.5, tolerance.anova = .Machine$double.eps^0.5)
{
    ## anova
    rlm.anova <- rlm$anova
    rxlm.anova <- rxlm$anova
    
    #remove rows with 0 df as they can be dropped in any order
    zeroDf <- (rlm.anova$Df == 0)
    rlm.anova <- rlm.anova[!zeroDf, ]
    rxlm.anova <- rxlm.anova[!zeroDf, ]
    
    checkEquals(rlm.anova[-1], rxlm.anova[-1], check.attributes = FALSE, tolerance = tolerance.anova)
    checkEquals(as.character(rlm.anova[-1, 1]), as.character(rxlm.anova[-1, 1]))
    
    ## coef
    rlm.coef <- coef(rlm)
    rxlm.coef <- coef(rxlm)
    
    ## workaround for PR#7842. terms.formula may have flipped interactions
    names.sort <- function(names.old)
    {
        names.new <- sapply(strsplit(names.old, ":", fixed = TRUE),
            function(x) paste(sort(x), collapse = ":"))
        names.new
    }

    names(rlm.coef) <- names.sort(names(rlm.coef))
    # convert "Revo" labels to "R" labels
    names.rxlm.coef <- gsub("=", "", names(rxlm.coef))
    names.rxlm.coef <- gsub(" for |[*]", ":", names.rxlm.coef)
    names(rxlm.coef) <- names.sort(names.rxlm.coef)
    
  checkEqualsNumeric(rlm.coef, rxlm.coef[names(rlm.coef)], tolerance = tolerance.coef)
    
    ## aic
    checkEqualsNumeric(rlm$aic, rxlm$aic, tolerance = tolerance.aic)
}

#######################################################################
## NOTE: the following tests are organized by families and
## each test generally tests the correctness of 3 things: 
## * rxGlm
## * rxGlm with variableSelection
## * rxPredict on the result of rxGlm with variableSelection

#######################################################################
# binomial

"test.rxGlmStep.binomial.logit.infert.no.intercept" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial(link = "logit")    #implemented in C++
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = formula(drop.terms(form.terms, dropx = 2:5, keep.response = FALSE)),
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.binomial.logit.infert.no.intercept.keepStepCoefs" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial(link = "logit")    #implemented in C++
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = formula(drop.terms(form.terms, dropx = 2:5, keep.response = FALSE)),
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection with keepStepCoefs = FALSE
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test, keepStepCoefs = FALSE)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step.keepStepCoefsF <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## rxGlm/variableSelection with keepStepCoefs = TRUE
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test, keepStepCoefs = TRUE)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step.keepStepCoefsT <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
  
  rxStepPlot(rxglm.step.keepStepCoefsT)
  dev.off()
  
    ## compare rxglm.step.keepStepCoefsF, rxglm.step.keepStepCoefsT, and glm.step
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step.keepStepCoefsF)
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step.keepStepCoefsT)
  
    dont.compare <- c("stepCoefs", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.step.keepStepCoefsF), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.step.keepStepCoefsF[[nm]], rxglm.step.keepStepCoefsT[[nm]])))))
  
  ## compare the coefficients of the start and final models
  glm.coef.start <- glm.obj$coefficients
  glm.coef.final <- glm.step$coefficients
  
  stepCoefs <- rxglm.step.keepStepCoefsT$stepCoefs
  rxglm.stepCoefs.names <- gsub("=", "", rownames(stepCoefs))
  
  rxglm.stepCoefs.start <- stepCoefs[, 1]
  names(rxglm.stepCoefs.start) <- rxglm.stepCoefs.names
  
  rxglm.stepCoefs.final <- stepCoefs[, ncol(stepCoefs)]
  names(rxglm.stepCoefs.final) <- rxglm.stepCoefs.names
  
  rxglm.coef.final <- drop(rxglm.step.keepStepCoefsT$coefficients)
  names(rxglm.coef.final) <- gsub("=", "", names(rxglm.coef.final))
  
  checkEquals(rxglm.stepCoefs.start[names(glm.coef.start)], glm.coef.start)
  checkEquals(rxglm.stepCoefs.final[names(rxglm.coef.final)], rxglm.coef.final)
  
  ## compare the number of steps
  checkEquals(ncol(stepCoefs), nrow(rxglm.step.keepStepCoefsT$anova))
}

"test.rxGlmStep.binomial.logit.infert" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial(link = "logit")    #implemented in C++
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.binomial.logit.infert.no.valid.observations" <- function()
{
  Sys <- Sys.info()
  #CH: Will remove deactivation since RA-10687 is resolved, if passes tomorrow will remove altogether
    #if ( ( Sys[ "sysname" ] == "Windows" ) && ( Sys[ "machine" ] == "x86" ) ) {
    #    DEACTIVATED("Deactivated on WIN32 pending investigation: RA-10687" )
    #}
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    dataset$induced <- NA   # introduce all missing var
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial(link = "logit")    #implemented in C++
    
    #glm.full <- glm(form.full, data = dataset, family = fam,
        #contrasts = list(education = contr.SAS))   
    #
    #rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        #computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    #compareGlmStepResults(glm.full, rxglm.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    #glm.obj <- glm(form.base, data = dataset, family = fam,
        #contrasts = list(education = contr.SAS))
    #glm.step <- step(glm.obj, direction = direction, scope = scope,
        #steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    checkException( rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R") )
    
    ## compare
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
}

"test.rxGlmStep.binomial.probit.birthwt" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    attach(birthwt)
    ethnicity <- factor(race, labels = c("white", "black", "other"))
    ptd <- factor(ptl > 0)
    ftv <- factor(ftv)
    levels(ftv)[-(1:2)] <- "2+"
    bwt <- data.frame(low = factor(low), age, lwt, ethnicity,
       smoke = (smoke > 0), ptd, ht = (ht > 0), ui = (ui > 0), ftv)
    detach()
    rm(ethnicity, ptd, ftv)

    dataset <- bwt
    #summary(dataset)
    
    ################################### basic model
    form.full <- low ~ .
    form.terms <- terms(form.full, data = bwt)
    # expand the dot in formula for rxLinMod
    (form.full <- formula(form.terms))    #, simplify = TRUE
    
    fam <- binomial(link = "probit")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(ethnicity = contr.SAS, ptd = contr.SAS, ftv = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    ##coef labels for logicals are different!!! fixed
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 5:8, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(ethnicity = contr.SAS, ptd = contr.SAS, ftv = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.binomial.cauchit.bacteria" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- bacteria
    #summary(dataset)
    
    ################################### basic model
    (form.full <- y ~ trt * week)
    form.terms <- terms(form.full)
    fam <- binomial(link = "cauchit")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(trt = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(trt = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.binomial.cloglog.birthwt" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    attach(birthwt)
    ethnicity <- factor(race, labels = c("white", "black", "other"))
    ptd <- factor(ptl > 0)
    ftv <- factor(ftv)
    levels(ftv)[-(1:2)] <- "2+"
    bwt <- data.frame(low = factor(low), age, lwt, ethnicity,
       smoke = (smoke > 0), ptd, ht = (ht > 0), ui = (ui > 0), ftv)
    detach()
    rm(ethnicity, ptd, ftv)

    dataset <- bwt
    #summary(dataset)
    
    ################################### basic model
    form.full <- low ~ .
    form.terms <- terms(form.full, data = bwt)
    # expand the dot in formula for rxLinMod
    (form.full <- formula(form.terms))    #, simplify = TRUE
    
    fam <- binomial(link = "cloglog")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(ethnicity = contr.SAS, ptd = contr.SAS, ftv = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    ##coef labels for logicals are different!!! fixed
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 5:8, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(ethnicity = contr.SAS, ptd = contr.SAS, ftv = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

# Gamma
"test.rxGlmStep.Gamma.inverse.claims" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    claims.xdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claims.df <- rxDataStep(claims.xdf)
    dataset <- claims.df
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
        
    ################################### basic model
    (form.full <- cost ~ age + car.age + type)
    form.terms <- terms(form.full)
    fam <- Gamma(link = "inverse")

    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full, tolerance.aic = 1e-6)  
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #1  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)    #NOTE: there seems a bug in R: missing one row in the anova component
  
  ## glm/stepAIC:  agrees with glm/step
  #require(MASS)
    #glm.stepAIC <- stepAIC(glm.obj, direction = direction, scope = scope,
        #steps = steps, scale = scale, k = k, test = test, trace = trace)    #NOTE: there seems a bug in R: missing one row in the anova component
  
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
  ## the above test failed when .test.mode = "REVO", which seems due to bugs in R.
  #Error in checkEqualsNumeric(rlm.coef, rxlm.coef[names(rlm.coef)], tolerance = tolerance.coef) :
  #Mean relative difference: 0.03652121
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.Gamma.identity.claims" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    claims.xdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claims.df <- rxDataStep(claims.xdf)
    dataset <- claims.df
    dataset <- na.omit(dataset)
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
        
    ################################### basic model
    (form.full <- cost ~ age + car.age + type)
    form.terms <- terms(form.full)
    fam <- Gamma(link = "identity")

    glm.full <- glm(form.full, data = dataset, family = fam,    #form.base generate different aic
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)   
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## test for the convergence of deviance
    glm.base <- glm(form.base, data = dataset, family = fam,    #form.base generate different aic
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxglm.base <- rxGlm(form.base, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.base, rxglm.base)   
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.Gamma.log.claims" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    claims.xdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claims.df <- rxDataStep(claims.xdf)
    dataset <- claims.df
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
        
    ################################### basic model
    (form.full <- cost ~ age + car.age + type)
    form.terms <- terms(form.full)
    fam <- Gamma(link = "log")    #implemented in C++

    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.Gamma.log.claims.weights" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    claims.xdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claims.df <- rxDataStep(claims.xdf)
    dataset <- claims.df
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
        
    ################################### basic model
    (form.full <- cost ~ age + car.age + type)
    form.terms <- terms(form.full)
    fam <- Gamma(link = "log")    #implemented in C++
    weights <- "number"

    glm.full <- glm(form.full, data = dataset, family = fam, 
        weights = dataset[[weights]],
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        fweights = weights,
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        weights = dataset[[weights]],
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        fweights = weights,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        fweights = weights,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}


"test.rxGlmStep.Gamma.log.claims.weights.start" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    claims.xdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claims.df <- rxDataStep(claims.xdf)
    dataset <- claims.df
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
        
    ################################### basic model
    (form.full <- cost ~ age + car.age + type)
    form.terms <- terms(form.full)
    fam <- Gamma(link = "log")    #implemented in C++
    weights <- "number"
    start <- c(5, 0, 0, 0, 0, 0, 0, 0, NA, 1, 1, 1, NA, 0, 0, 0, NA)

    glm.full <- glm(form.full, data = dataset, family = fam, 
        weights = dataset[[weights]], start = start[!is.na(start)],
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        pweights = weights, initialValues = start,
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
    #start <- unname(round(coef(rxglm.full)))

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    ## NOTE: glm() does not return the "start" argument,
    ##  which is thus not used at all in functionstep().
}

# poisson
"test.rxGlmStep.poisson.log.d.AD" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    d.AD <- data.frame(treatment, outcome, counts)
    dataset <- d.AD
    #summary(dataset)
    
    ################################### basic model
    (form.full <- counts ~ outcome + treatment)
    form.terms <- terms(form.full)
    fam <- poisson(link = "log")   #implemented in C++
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.poisson.identity.housing" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- housing
    #summary(dataset)
    
    ################################### basic model
    (form.full <- Freq ~ Infl*Type*Cont + Sat)
    form.terms <- terms(form.full)
    fam <- poisson(link = "identity")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Infl = contr.SAS, Type = contr.SAS, Cont = contr.SAS, Sat = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 5:8, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Infl = contr.SAS, Type = contr.SAS, Cont = contr.SAS, Sat = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.poisson.sqrt.quine" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- quine
    #summary(dataset)
    
    ################################### basic model
    form.full <- Days ~ .^4
    # expand the dot in formula
    (form.full <- formula(terms(form.full, data = quine)))    #, simplify = TRUE
    form.terms <- terms(form.full)
    fam <- poisson(link = "sqrt")

    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 9:15, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") 
        compareGlmStepResults(glm.step, rxglm.step, tolerance.coef = 1e-4, tolerance.anova = 1e-7)
    #Error in checkEqualsNumeric(rlm.coef, rxlm.coef[names(rlm.coef)]) :
      #Mean relative difference: 4.844165e-05

    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]], tolerance = 1e-5)
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]], tolerance = 1e-5)
    }
    #Error in checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]]) :
      #Mean relative difference: 5.596007e-06

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.poisson.log.epil" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- epil
    #summary(dataset)
    
    ################################### basic model
    (form.full <- y ~ lbase*trt + lage + V4)
    form.terms <- terms(form.full)
    fam <- poisson(link = "log")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(trt = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 3:5, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(trt = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

# gaussian
"test.rxGlmStep.gaussian.identity.anorexia" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- anorexia
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- Postwt ~ Prewt + Treat + offset(Prewt))
    form.terms <- terms(form.full)
    fam <- gaussian(link = "identity")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Treat = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Treat = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.gaussian.log.anorexia.offset" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- anorexia
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- Postwt ~ Prewt + Treat)
    form.terms <- terms(form.full)
    fam <- gaussian(link = "log")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Treat = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    form.base <- update(form.base, ~. + offset(Prewt))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Treat = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.gaussian.inverse.anorexia" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- anorexia
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
    
    ################################### basic model
    (form.full <- Postwt ~ Prewt + Treat)
    form.terms <- terms(form.full)
    fam <- gaussian(link = "inverse")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Treat = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))

    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    scope$lower <- update(scope$lower, ~. + offset(Prewt))    
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Treat = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

## inverse.gaussian
"test.rxGlmStep.inverse.gaussian.'1/mu^2'.d.AD" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    d.AD <- data.frame(treatment, outcome, counts)
    dataset <- d.AD
    #summary(dataset)
    
    ################################### basic model
    (form.full <- counts ~ outcome + treatment)
    form.terms <- terms(form.full)
    fam <- inverse.gaussian(link = "1/mu^2")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.inverse.gaussian.inverse.d.AD" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    d.AD <- data.frame(treatment, outcome, counts)
    dataset <- d.AD
    #summary(dataset)
    
    ################################### basic model
    (form.full <- counts ~ outcome + treatment)
    form.terms <- terms(form.full)
    fam <- inverse.gaussian(link = "inverse")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.inverse.gaussian.inverse.d.AD.weights" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    weights <- seq_along(counts)
    d.AD <- data.frame(treatment, outcome, counts, weights)
    dataset <- d.AD
    #summary(dataset)
    
    ################################### basic model
    (form.full <- counts ~ outcome + treatment)
    form.terms <- terms(form.full)
    fam <- inverse.gaussian(link = "inverse")
    
    glm.full <- glm(form.full, data = dataset, family = fam, 
        weights = dataset$weights,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        fweights = "weights",
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.inverse.gaussian.inverse.d.AD.weights.start" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    weights <- seq_along(counts)
    d.AD <- data.frame(treatment, outcome, counts, weights)
    dataset <- d.AD
    #summary(dataset)
    
    ################################### basic model
    (form.full <- counts ~ outcome + treatment)
    form.terms <- terms(form.full)
    fam <- inverse.gaussian(link = "inverse")
    start <- c(0.07, -0.02, 0.01, NA, -0.01, 0, NA)
    
    glm.full <- glm(form.full, data = dataset, family = fam, 
        weights = dataset$weights, start = start[!is.na(start)],
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        fweights = "weights",  initialValues = start,
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
    #start <- unname(round(coef(rxglm.full), digits = 2))

    compareGlmStepResults(glm.full, rxglm.full)   
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.inverse.gaussian.identity.d.AD" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    counts <- c(18,17,15,20,10,20,25,13,12)
    outcome <- gl(3,1,9)
    treatment <- gl(3,3)
    d.AD <- data.frame(treatment, outcome, counts)
    dataset <- d.AD
    #summary(dataset)
    
    ################################### basic model
    (form.full <- counts ~ outcome + treatment)
    form.terms <- terms(form.full)
    fam <- inverse.gaussian(link = "identity")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(outcome = contr.SAS, treatment = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

#negative.binomial
"test.rxGlmStep.negative.binomial.log.quine" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- quine
    #summary(dataset)
    
    ################################### basic model
    form.full <- Days ~ .^4
    # expand the dot in formula
    (form.full <- formula(terms(form.full, data = quine)))    #, simplify = TRUE
    form.terms <- terms(form.full)
    fam <- negative.binomial(theta = 2, link = "log")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 9:15, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
  ## the above test failed when .test.mode = "REVO", which seems due to bugs in R.
  #Error in checkEquals(rlm.anova[-1], rxlm.anova[-1],
  #Component 1: Mean relative difference: 2
  #Component 2: Mean relative difference: 1.115431
  #Component 3: Mean relative difference: 0.02255639
  #Component 4: Mean relative difference: 0.01215058
  #Component 5: Mean relative difference: 0.0004248719
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.negative.binomial.sqrt.quine" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- quine
    #summary(dataset)
    
    ################################### basic model
    form.full <- Days ~ .^4
    # expand the dot in formula
    (form.full <- formula(terms(form.full, data = quine)))    #, simplify = TRUE
    form.terms <- terms(form.full)
    fam <- negative.binomial(theta = 2, link = "sqrt")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:15, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #0  #
        
    ## glm/step
    glm.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    glm.step <- step(glm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxGlm/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

"test.rxGlmStep.negative.binomial.identity.quine" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- quine
    #summary(dataset)
    
    ################################### basic model
    form.full <- Days ~ .^4
    # expand the dot in formula
    (form.full <- formula(terms(form.full, data = quine)))    #, simplify = TRUE
    form.terms <- terms(form.full)
    fam <- negative.binomial(theta = 2, link = "identity")
    
    glm.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))   
    
    rxglm.full <- rxGlm(form.full, data = dataset, family = fam, 
        computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareGlmStepResults(glm.full, rxglm.full)    
    
    ################################### stepwise
    ##TODO: step() for glm failed
    #direction <- "both"
    #(form.base <- formula(drop.terms(form.terms, dropx = 3:15, keep.response = TRUE)))
    #(scope <- list(
      #lower = ~ 1,
        #upper = formula(delete.response(form.terms)) ))
    #steps <- 1000
    #scale <- 0  #1  #
    #k <- 2
    #test <- NULL
    #trace <- 0  #1  #
        #
    ### glm/step
    #glm.obj <- glm(form.base, data = dataset, family = fam,
        #contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    #glm.step <- step(glm.obj, direction = direction, scope = scope,
        #steps = steps, scale = scale, k = k, test = test, trace = trace)
    #
    ### rxGlm/variableSelection
    #varsel <- list(#dropPolicy = "Competitive",
        #method = if (direction == "both") "stepwise" else direction, scope = scope,
    #maxSteps = steps, scale = scale, k = k, test = test)
    #if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
  #if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
  #if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    #rxglm.step <- rxGlm(form.base, data = dataset, family = fam, variableSelection = varsel, 
        #computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    #
    ### compare
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareGlmStepResults(glm.step, rxglm.step)
    #
    #################################### predict
    ##rxOptions(numCoresToUse = 1)
    #rxglm.obj <- rxGlm(form = rxglm.step$formula, data = dataset, family = fam,
        #computeAIC = TRUE, verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
#
    #pred.rxglm.obj <- rxPredict(rxglm.obj, data = dataset, checkFactorLevels = FALSE)
    #pred.rxglm.step <- rxPredict(rxglm.step, data = dataset, checkFactorLevels = FALSE)
    #pred.glm.step <- predict(glm.step, newdata = dataset, type = "response")
        #
    #checkEquals(pred.rxglm.obj, pred.rxglm.step)        
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        #checkEqualsNumeric(pred.glm.step, pred.rxglm.step[[1]])
        #checkEqualsNumeric(pred.glm.step, pred.rxglm.obj[[1]])
    #}
#
    #dont.compare <- c("call", "params")
    #checkTrue(all(sapply(setdiff(names(rxglm.obj), dont.compare), 
        #function(nm) isTRUE(all.equal(rxglm.obj[[nm]], rxglm.step[[nm]])))))
}

#######################################################################
#mortDefault    ## for benchmark?

#"test.rxLogitStep.mortDefault" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #mortDefault.xdf <- file.path("C:/Revolution/Data/mortDefault", "mortDefault.xdf")
    ##rxGetInfo(mortDefault.xdf, getVarInfo = T)
    ##mortDefault.df <- rxDataStep(mortDefault.xdf)
    ##Warning message:
    ##In rxReadXdf(outFile, maxRowsByCols = maxRowsByCols, reportProgress = 0) :
    ##The number of rows (1e+07) times the number of columns (6) exceeds the maxRowsByCols argument value (3e+06).
    ##Only 500000 rows will be read.
    #dataset.xdf <- mortDefault.xdf
    ##dataset.df <- mortDefault.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #form.full <- creditScore ~ default + houseAge + yearsEmploy + ccDebt + year
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    ##fam <- Gamma(link = "log")  #implemented in C++
    ##fam <- Gamma(link = "inverse")  #implemented in R
    #fam <- Gamma(link = "identity")  #implemented in R
    #
    ##fam <- poisson(link = "log")  #implemented in C++
    ##fam <- poisson(link = "sqrt")  #implemented in R
    #fam <- poisson(link = "identity")  #implemented in R
        #
    ##glm.full <- glm(form.full, data = dataset.df, family = fam)   
    #
    #system.time(
        #rxglm.full <- rxGlm(form.full, data = dataset.xdf, family = fam,
            #computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
    #)
#
    #system.time(
        #rxglm.full <- rxGlm(form.full, data = dataset.xdf, family = fam, computeAIC = FALSE,
            #computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
    #)
#
    ##compareGlmStepResults(glm.full, rxlogit.full)
#}

## Airline2000  ## for benchmark

#"test.rxLogitStep.Airline2000" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #Airline2000.xdf <- file.path("C:/Revolution/Data/Airline2000", "Airline2000.xdf")
    ##rxGetInfo(Airline2000.xdf, getVarInfo = T)
    ##Airline2000.df <- rxDataStep(Airline2000.xdf)
    ##Warning message:
    ##In rxReadXdf(outFile, maxRowsByCols = maxRowsByCols, reportProgress = 0) :
    ##The number of rows (5683047) times the number of columns (29) exceeds the maxRowsByCols argument value (3e+06).
    ##Only 103448 rows will be read.
    #
    #dataset.xdf <- Airline2000.xdf
    ##dataset.df <- Airline2000.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #yvar <- "AirTime" #"ArrDelay"    #"Cancelled" # "Diverted"    # 
    #xvars <- c("Year", "Month", "DayofMonth", "DayOfWeek", 
        #"DepTime", "CRSDepTime", "ArrTime", "CRSArrTime", "UniqueCarrier", 
        #"ActualElapsedTime", "CRSElapsedTime", "DepDelay", #"AirTime", 
        #"Origin", "Dest", "Distance", "TaxiIn", "TaxiOut")
    #
    #form.full <- formula(paste(yvar, paste(xvars, collapse = " + "), sep = " ~ "))
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    #fam <- Gamma(link = "log")
    #
    ##glm.full <- glm(form.full, data = dataset.df, family = fam)   
    #
    #system.time(
    #rxglm.full <- rxGlm(form.full, data = dataset.xdf, family = fam, 
        #computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
    #)
#
    ##compareGlmStepResults(glm.full, rxlogit.full)
#}

### AirlineData87to08  ## for benchmark
#
#"test.rxLogitStep.AirlineData87to08" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #AirlineData87to08.xdf <- file.path("C:/Revolution/Data/AirlineData87to08", "AirlineData87to08.xdf")
    ##rxGetInfo(Airline2000.xdf, getVarInfo = T)
    #AirlineData87to08.df <- rxDataStep(AirlineData87to08.xdf)
    ##Warning message:
    ##In rxReadXdf(outFile, maxRowsByCols = maxRowsByCols, reportProgress = 0) :
    ##The number of rows (5683047) times the number of columns (29) exceeds the maxRowsByCols argument value (3e+06).
    ##Only 103448 rows will be read.
    #
    #dataset.xdf <- AirlineData87to08.xdf
    #dataset.df <- AirlineData87to08.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #yvar <- "Cancelled" # "Diverted"    # "ArrDelay"    #
    #xvars <- c("Year", "Month", "DayofMonth", "DayOfWeek", 
        #"DepTime", "CRSDepTime", "ArrTime", "CRSArrTime", "UniqueCarrier", 
        #"ActualElapsedTime", "CRSElapsedTime", "AirTime", "DepDelay", 
        #"Origin", "Dest", "Distance", "TaxiIn", "TaxiOut")
    #
    #form.full <- formula(paste(yvar, paste(xvars, collapse = " + "), sep = " ~ "))
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    #fam <- binomial
    #
    #glm.full <- glm(form.full, data = dataset.df, family = fam)   
    #
    #rxlogit.full <- rxLogit(form.full, data = dataset.xdf,
        #computeAIC = TRUE, dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    ##compareGlmStepResults(glm.full, rxlogit.full)
#}
