# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxImport
#
############################################################################################

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

"test.rxImport.hugeFile" <- function()
{
    # Code to deactivate test if less than 2 GB of memory on a Linux system.
    platform <- R.version$platform
    if (platform == "x86_64-pc-linux-gnu")
    {
        meminfo <- system("cat /proc/meminfo", intern = TRUE)
        mem.avail <- meminfo[2]
        kb.pos <- gregexpr("k", mem.avail)[[1]][1]
        colon.pos <- gregexpr(":", mem.avail)[[1]][1]
        mem.avail <- substring(mem.avail, colon.pos+1, kb.pos-2)
        mem.avail <- as.integer(mem.avail)
        mem.avail <- mem.avail / 1000
    } else {
        meminfo <- system("systeminfo", intern = TRUE)
        mem.avail <- meminfo[26]
        mb.pos <-gregexpr("MB", mem.avail)[[1]][1]
        colon.pos <- gregexpr(":", mem.avail)[[1]][1]
        mem.avail <- substring(mem.avail, colon.pos+1, mb.pos-2)
        mem.avail <- strsplit(mem.avail, ",", fixed = TRUE)
        mem.avail <- paste(mem.avail[[1]][1], mem.avail[[1]][2], sep = "")
        mem.avail <- as.integer(mem.avail) 
    }

    if (mem.avail < 2500)
        DEACTIVATED("Not enough free system memory to run test.\n")
     
    # Create a dataframe that is larger than 500 MB to test chunking 
    df <- data.frame(replicate(10,sample(0:1,25000000,rep=TRUE)))
    df2 <- rxImport(df, reportProgress = 0)
    checkEquals(df,df2)
}

"test.rxImport.args.outFile.NULL" <- function()
{
  claimsSasFileName <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  claimsSasSource <- RxSasData(claimsSasFileName)

  # Import the data into a data frame in memory
  claimsIn <- rxImport(claimsSasSource)

  checkEquals( claimsIn$cost[1:3], c(289, 282, 133), checkNames = FALSE)
  checkEquals( claimsIn$number[1:3], c(8, 8, 4), checkNames = FALSE)

    # Use an SPSS file that is a subset of claims
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")
  spssDS <- RxSpssData(spssFileName)

  # Import data; notice that factor variables are created for any
  # SPSS variables with value labels

    claimsInt <- rxImport(spssDS, reportProgress = 0)
    # claimsInt is a subset of claims with 120 rows
  checkEquals(nrow(claimsInt), 120)
  checkEquals(ncol(claimsInt), 6)
    checkEquals(levels(claimsInt$type), LETTERS[1:4])
}

"test.rxImport.args.transforms" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
  # Import into data frame
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  myDataImport <- rxImport(inData=inFile,
    transforms = list(xint2 = xint1 + ybinint))

  myDataImport$xint2test <- myDataImport$xint1 + myDataImport$ybinint
  checkIdentical(myDataImport$xint2test, myDataImport$xint2)

  # Import into xdf file
  outFile <-  file.path(tempdir(),".rxTempImport.xdf")
  if (file.exists(outFile)) file.remove(outFile)

  rxImport(inData=inFile, outFile=outFile,
    transforms = list(xint2 = xint1 + ybinint))

  myDataXdf <- rxDataStep( outFile)
  checkIdentical(myDataXdf$xint2, myDataImport$xint2)

  file.remove( outFile )
}

"test.rxImport.args.rowSelection" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")
  # Import into data frame
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  myDataImport <- rxImport(inData=inFile, rowSelection = xnum1 > 0,
    transforms = list(xint2 = xint1 + ybinint))

  myDataImportFull <- rxImport(inData = inFile)

  myDataImportSubset <- subset(myDataImportFull, xnum1 > 0)

  checkIdentical(myDataImport$xint1, myDataImport$xint1)

  # Import into xdf file
  outFile <-  file.path(tempdir(),".rxTempImport.xdf")
  if (file.exists(outFile)) file.remove(outFile)

  rxImport(inData=inFile, outFile=outFile, rowSelection = xnum1 > 0,
    transforms = list(xint2 = xint1 + ybinint))

  myDataXdf <- rxDataStep( outFile)
  checkIdentical(myDataXdf$xint1, myDataImport$xint1)

  file.remove( outFile )
}

#	Fix: Bug 11919. Import from xdf with rowSelection
"test.rxImport.Xdf.rowSelection" <- function()
{
  airlineSmall <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
  df <- rxImport(airlineSmall, rowSelection = DayOfWeek=="Monday")
  df2 <- head(df)	# Bug: only one column: DayOfWeek
  checkEquals(ncol(df2), 3)
}

#	Fix: Bug 12086. Import long long integer
"test.rxImport.LargeInteger" <- function()
{
  # Import into data frame
  inFile <- file.path(rxGetOption("unitTestDataDir"), "SmallSample.txt")
  myDataImport <- rxImport(inData=inFile)
  checkEquals(myDataImport$id_tech_cdr[1], 8068000001467185185, tolerance=1e-6 )
  checkEquals(myDataImport$id_tech_cdr[2], 8079000001732603619, tolerance=1e-6 )
  checkEquals(myDataImport$id_tech_cdr[3], 8074000001440841703, tolerance=1e-6 )
  checkEquals(myDataImport$id_tech_cdr[4], 8067000001491074094, tolerance=1e-6 )

}


"test.rxImport.RxSasData" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))
    whichLogical <- which(sapply(expect, is.logical))
    for (i in whichLogical)
        expect[[i]] <- as.character(expect[[i]])
    expect[[9]] <- as.character(expect[[9]])
    expect[[12]][expect[[12]] == " "] <- NA


    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")

    rxImport(inData=sasFileName, outFile = xdfFileName)

    received <- rxDataStep(xdfFileName)
    checkEquals( received, expect)

}

"test.rxImport.RxSasData.args.colClasses" <- function()
{
    expect <-
      read.csv(file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt"),
               stringsAsFactors = FALSE)
    colnames(expect) <- toupper(gsub(".", "_", colnames(expect), fixed = TRUE))

    for (i in c(9, 10))
        expect[[i]] <- factor(expect[[i]], levels = unique(expect[[i]]))
    expect[[12]] <-
      factor(expect[[12]], levels = c("Washington", "Alaska", "North Carolina"))

    sasFileName <-
      file.path(rxGetOption("unitTestDataDir"), "richTest100r.sas7bdat")
    xdfFileName <- file.path(tempdir(), "richTest.xdf")

    sasColClasses =
                c(Y = "float64", Y2 = "float64", YLOGICAL = "logical",
                  YBININT = "int16", XNUM1 = "float64", XNUM2 = "float64",
                  XNUM2MISS = "float64", XINT1 = "integer", XFAC1 = "factor",
                  XFAC2 = "factor", ZFAC = "factor")

     rxImport(sasFileName, xdfFileName, colClasses = sasColClasses,
        overwrite = TRUE, reportProgress = 0)

     received <- rxDataStep(xdfFileName, reportProgress = 0)
    checkEquals(received, expect)
    file.remove(xdfFileName)
}

"test.rxImport.RxSpssData.args.elipses" <- function()
{
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")

    testData <- rxImport(spssFileName, overwrite = TRUE, reportProgress = 0, labelsAsLevels = FALSE)

  varInfo <- rxGetVarInfo( testData )
  checkEquals(varInfo$type$varType, "numeric")
  checkEquals(varInfo$car_age$varType, "numeric")

  # Since labelsAsInfo is by default true, that information should be in varInfo
  checkEquals(varInfo$type$valueInfoCodes, as.character(1:4))
  typeLabels <- LETTERS[1:4]
  checkEquals(varInfo$type$valueInfoLabels, typeLabels)

    # Create an XDF data file path
    xdfFileName <- file.path(tempdir(), "ClimsIntImport.xdf")
    if (file.exists(xdfFileName)) file.remove(xdfFileName)

    # Import data; notice that factor variables are created for any
    # SPSS variables with value labels
    rxImport(spssFileName, xdfFileName, overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNum$varType, "numeric")
    checkEquals(varInfo$car_age$varType, "factor")

    # Reimport SPSS data, storing some variables as integer instead of
    # numeric

    spssColClasses <- c(RowNum = "integer", cost = "integer",
      number = "integer")
    rxImport(spssFileName, xdfFileName, colClasses = spssColClasses,
       overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNum$varType, "integer")
    checkEquals(varInfo$number$varType, "integer")

    # Reimport SPSS data, changing re-specifing the strings used
    # for factor levels

    spssColInfo <- list(
      RowNum = list(type="integer", newName = "RowNumber"),
      cost = list(type="integer", newName = "TotalCost", description = "Cost of Auto"),
      number = list(type="integer", newName = "Num", description = "Number of claims"),
      type = list(
      levels = as.character(1:4),
      newLevels = c("AA", "BB", "CC", "DD")))
    rxImport(spssFileName, xdfFileName, colInfo = spssColInfo, overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNumber$varType, "integer")
    checkEquals(varInfo$TotalCost$varType, "integer")
    checkEquals(varInfo$Num$description, "Number of claims")
    checkEquals(varInfo$type$levels, c("AA", "BB", "CC", "DD"))
    if (file.exists(xdfFileName)) file.remove(xdfFileName)
}

"test.rxImport.RxSpssData.args.RxDataSource" <- function()
{

  # Test that information in data source object is being respected
    spssFileName <- file.path(rxGetOption("unitTestDataDir"), "claimsInt.sav")
    spssDS <- RxSpssData(spssFileName)
    testData <- rxImport(spssDS, overwrite = TRUE, reportProgress = 0, labelsAsLevels = FALSE)

  varInfo <- rxGetVarInfo( testData )
  checkEquals(varInfo$car_age$varType, "numeric")
  checkEquals(varInfo$type$varType, "numeric")

    # Set inside of a function
    myTestFunction <- function( useLabelsAsLevels, spssDS )
    {
        rxImport(spssDS, overwrite = TRUE, reportProgress = 0, labelsAsLevels = useLabelsAsLevels)
    }

    myLabels <- FALSE
    testData <- myTestFunction(useLabelsAsLevels = myLabels, spssDS)
  varInfo <- rxGetVarInfo( testData )
  checkEquals(varInfo$car_age$varType, "numeric")
  checkEquals(varInfo$type$varType, "numeric")


  # Since labelsAsInfo is by default true, that information should be in varInfo
  checkEquals(varInfo$type$valueInfoCodes, as.character(1:4))
  typeLabels <- LETTERS[1:4]
  checkEquals(varInfo$type$valueInfoLabels, typeLabels)

    # Create an XDF data file path
    xdfFileName <- file.path(tempdir(), "ClaimsIntImport2.xdf")
    if (file.exists(xdfFileName)) file.remove(xdfFileName)

    # Import data; notice that factor variables are created for any
    # SPSS variables with value labels
    rxImport(spssDS, xdfFileName, overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$type$varType, "factor")
    checkEquals(varInfo$RowNum$varType, "numeric")
    checkEquals(varInfo$cost$varType, "numeric")


    # Reimport SPSS data, storing some variables as integer instead of
    # numeric

    spssColClasses <- c(RowNum = "integer", cost = "integer",
      number = "integer")
  spssDS <- RxSpssData( spssFileName, colClasses = spssColClasses)
    rxImport(spssDS, xdfFileName,
       overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    checkEquals(varInfo$type$varType, "factor")
    checkEquals(varInfo$RowNum$varType, "integer")
    checkEquals(varInfo$cost$varType, "integer")

    # Reimport SPSS data, changing re-specifing the strings used
    # for factor levels

    spssColInfo <- list(
      RowNum = list(type="integer", newName = "RowNumber"),
      cost = list(type="integer", newName = "TotalCost", description = "Cost of Auto"),
      number = list(type="integer", newName = "Num", description = "Number of claims"),
      type = list(
      levels = as.character(1:4),
      newLevels = c("AA", "BB", "CC", "DD")))
  spssDS <- RxSpssData(spssFileName, colInfo = spssColInfo)
    rxImport(spssDS, xdfFileName, overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )

    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNumber$varType, "integer")
    checkEquals(varInfo$TotalCost$varType, "integer")
    checkEquals(varInfo$Num$description, "Number of claims")
    checkEquals(varInfo$type$levels, c("AA", "BB", "CC", "DD"))


    varsToKeepVector <- c("RowNum", "cost", "number", "type", "age")
    spssDS <- RxSpssData(spssFileName, colInfo = spssColInfo)
    rxImport(spssDS, xdfFileName, varsToKeep = varsToKeepVector,
    overwrite = TRUE, reportProgress = 0)
    varInfo <- rxGetVarInfo( xdfFileName )
    renamedVarsToKeepVector <- c("RowNumber","TotalCost","Num", "type", "age")
    checkEquals(names(varInfo), renamedVarsToKeepVector)
    checkEquals(varInfo$age$varType, "factor")
    checkEquals(varInfo$RowNumber$varType, "integer")
    checkEquals(varInfo$TotalCost$varType, "integer")
    checkEquals(varInfo$Num$description, "Number of claims")
    checkEquals(varInfo$type$levels, c("AA", "BB", "CC", "DD"))

    if (file.exists(xdfFileName)) file.remove(xdfFileName)
}

"test.rxImport.args.delimiter.User" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyHash.txt") # hash delimited
  outFile <- file.path(tempdir(), ".rxTempTiny.xdf")

    if (file.exists(outFile)) file.remove(outFile)
    rxImport(inFile, outFile, delimiter = "#")
    infoObj <- rxGetInfo(outFile, numRows = 0, startRow = 1)
  varInfoObj <- rxGetVarInfo(outFile)
  checkEquals(infoObj$fileName, normalizePath(outFile))
  checkEquals(infoObj$numRows, 3)
  checkEquals(infoObj$numVars, 3)
  checkEquals(infoObj$numBlocks, 1)
  checkEquals(varInfoObj[["a"]], structure(list(varType = "integer", storage = "int32", low = 1, high = 3), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["b"]], structure(list(varType = "numeric", storage = "float32", low = 4.09999990463257, high = 6.30000019073486), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
  checkEquals(varInfoObj[["c"]], structure(list(varType = "character", storage = "string", low = NA, high = NA), .Names = c("varType", "storage", "low", "high"), class = "rxVarInfo"))
    if (file.exists(outFile)) file.remove(outFile)
}
"test.rxImport.args.delimiter.User.fixedWidth" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyHash.txt") # hash delimited
  colInfo <- list(c = list(type = "character", width = 5))
  myData1 <- rxImport(inFile, colInfo = colInfo, delimiter = "#")
  myData2 <- rxImport(inFile, colInfo = colInfo, type = "text", delimiter = "#")
  checkEquals(myData1, myData2)
}

"test.rxImport.args.rowsPerRead.FixedFormat" <- function()
{
    claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
    claimsXdf <- file.path(tempdir(), "importedClaims.xdf")

    # Specify column information about the input file
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

    # Import the data into the xdf file
    rxImport(inData = claimsFF, outFile = claimsXdf,
      colInfo = claimsFFColInfo,
    type = "fixed",
    firstRowIsColNames = FALSE,
      rowsPerRead = 65, # rowsPerRead will determine block size
      overwrite = TRUE)

    # Look at information about the new file
    fileInfo <- rxGetInfo( claimsXdf)
    if (rxIsExpressEdition())
  {
    checkEquals( fileInfo$numBlocks, 1)
  } else
  {
    checkEquals( fileInfo$numBlocks, 2)
  }

  # Redo with fast fixed
  rxImport(inData = claimsFF, outFile = claimsXdf,
      colInfo = claimsFFColInfo,
    type = "fixedFast",
      rowsPerRead = 65, # rowsPerRead will determine block size
      overwrite = TRUE)

    # Look at information about the new file
    fileInfo2 <- rxGetInfo( claimsXdf)
  checkEquals(fileInfo, fileInfo2)

    # Clean-up: delete the new file
    file.remove( claimsXdf)
}

#	 Test non-ST Fixed format text:	No datasource object
"test.rxImport.args.FastFixedFormat" <- function()
{
    claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
    claimsXdf <- file.path(tempdir(), "nonST_FixedFormatClaims.xdf")

    # Specify column information about the input file
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

    # Import the data into the xdf file
    rxImport(inData = claimsFF, outFile = claimsXdf,
      colInfo = claimsFFColInfo,
      type = "fixedFast",
    rowsPerRead = 65,
      overwrite = TRUE)

    # Look at information about the new file
    fileInfo <- rxGetInfo( claimsXdf)
  if (rxIsExpressEdition())
  {
     checkEquals( fileInfo$numBlocks, 1)
  } else
  {
        checkEquals( fileInfo$numBlocks, 2)
  }

    # Clean-up: delete the new file
    file.remove( claimsXdf)
}


"test.rxImport.value" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    outFile2 <- tempfile(pattern = ".rxTest2", fileext = ".xdf")
    outFile3 <- tempfile(pattern = ".rxTest3", fileext = ".xdf")
    outDS3 <- RxXdfData( outFile3 )

    # Import to data frame
    outData1 <- rxImport( inData = inFile )
    # Import to xdf
    outData2 <- rxImport( inData = inFile, outFile = outFile2,
        rowsPerRead = 30, overwrite = TRUE)
    # Import to xdf data source
    outData3 <- rxImport( inData = inFile, outFile = outDS3, overwrite = TRUE )

    outSum1 <- rxResultsDF(rxSummary(~y + ybinint + xnum2miss + xint1, data = outData1))

    outSum2 <- rxResultsDF(rxSummary(~y + ybinint + xnum2miss + xint1, data = outData2))

    outSum3 <- rxResultsDF(rxSummary(~y + ybinint + xnum2miss + xint1, data = outData3))

    checkEquals(outSum1, outSum2)
    checkEquals(outSum1, outSum3)

    file.remove(outFile2)
    file.remove(outFile3)
}
"test.rxImport.Date.type" <- function()
{
    # Fixed format
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDateTimeData.dat")

    fixedTextDS <- RxTextData(file=tinyFileName, firstRowIsColNames = TRUE,
    readPOSIXctFormat = "%Y/%m/%d %H:%M:%S",
    readDateFormat = "%Y/%m/%d",
    colInfo = list(
           MyDateTime = list(type = "POSIXct", start=1, width=19),
             MyDateTime2 = list(type = "POSIXct", start=21, width=16)))


    myDataFixed <- rxImport(fixedTextDS, reportProgress = 0, verbose=1)
    varInfo <- rxGetVarInfo(myDataFixed)
    checkEquals( varInfo$MyDateTime$varType, "POSIXct")
    checkEquals( varInfo$MyDateTime2$varType, "POSIXct")

    firstVal <- as.numeric(myDataFixed[[1]][1])
    secondVal <- as.numeric(myDataFixed[[1]][2])
    thirdVal <- as.numeric(myDataFixed[[2]][1])

    myTestVal1 <- as.numeric(as.POSIXct("1962/01/02 09:10:11", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))
    myTestVal2 <- as.numeric(as.POSIXct("1963/01/03 10:11:12", format = "%Y/%m/%d %H:%M:%S", tz = "GMT"))

    # First value in second column
    myTestVal3 <- as.numeric(as.POSIXct("1962/01/02 09:10", format = "%Y/%m/%d %H:%M", tz = "GMT"))

    checkEquals(firstVal, myTestVal1)
    checkEquals(secondVal, myTestVal2)
    checkEquals(thirdVal, myTestVal3)

    # Delimited
    tinyFileName <- file.path(rxGetOption("unitTestDataDir"), "tinyDateTimeData.csv")
    myDataDelimited <- rxImport(tinyFileName,
        colClasses = c(MyDateTime = "POSIXct", MyDateTime2 = "POSIXct"),
    readPOSIXctFormat = "%Y/%m/%d %H:%M:%S",
        firstRowIsColNames = TRUE, verbose = 1)
    varInfo <- rxGetVarInfo(myDataDelimited)
    checkEquals( varInfo$MyDateTime$varType, "POSIXct")
    checkEquals( varInfo$MyDateTime2$varType, "POSIXct")

    # Again, sniffing data types
    myDataDelimited <- rxImport(tinyFileName,
        firstRowIsColNames = TRUE, useFastRead = FALSE)
    varInfo <- rxGetVarInfo(myDataDelimited)
    checkEquals( varInfo$MyDateTime$varType, "POSIXct")
    checkEquals( varInfo$MyDateTime2$varType, "POSIXct")
}

"test.rxImport.data.Date.integer.string" <- function()
{
  x <- c(20111004, 20100107, 19990227)
  y <- c("2011-10-4", "2010-1-7", "1999-02-27")

  txtFile1 = tempfile(pattern = "date", fileext = ".csv")
    write.table(x = data.frame(x=x, y=y), file = txtFile1, row.names = FALSE, sep=",", na="NA", quote=FALSE )
  outData1 <- rxImport(inData = txtFile1, firstRowIsColNames = TRUE, readDateFormat = "%Y-%m-%d")
  varInfo1 <- rxGetVarInfo(outData1)
    checkEquals(varInfo1$x$varType, "integer")
    checkEquals(varInfo1$y$varType, "Date")

  outData1a <- rxImport(inData = txtFile1, firstRowIsColNames = TRUE, readDateFormat = "%Y-%m-%d",
      colClasses = c(x = "Date", y = "Date"))
  varInfo1a <- rxGetVarInfo(outData1a)
    # Cannot currently convert integer to Date
    checkEquals(varInfo1a$x$varType, "integer")
    checkEquals(varInfo1a$y$varType, "Date")

  file.remove(txtFile1)

    txtFile2 = tempfile(pattern = "date", fileext = ".csv")
    write.table(x = data.frame(x=x, y=y), file = txtFile2, row.names = FALSE, sep=",", na="NA", quote=TRUE )

    outData2 <- rxImport(inData = txtFile2, firstRowIsColNames = TRUE, readDateFormat = "%Y-%m-%d",
    colClasses = c(x = "Date", y = "Date"))
  varInfo2 <- rxGetVarInfo(outData2)

    file.remove(txtFile2)

}

"test.rxImport.args.verbose" <- function()
{
    # Delimited text
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0 ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = TRUE, isFixedFormat = FALSE")
    checkEquals(myOut[2], "Import type: textFast(delimited)" )

  ## TODO: Add Fast Fixed-format test
    # Fixed format text
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

    myOut <- capture.output(outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, verbose = 1, reportProgress = 0, useFastRead = FALSE ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = FALSE, isFixedFormat = TRUE")
    checkEquals(myOut[2], "Import type: text(fixed format)"  )

    myOut <- capture.output(outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, verbose = 1, reportProgress = 0, useFastRead = TRUE ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = TRUE, isFixedFormat = TRUE")
    checkEquals(myOut[2], "Import type: textFast(fixed format)"  )
    # SAS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0 ))
    checkEquals(myOut[1], "Import type: sas" )

    # SPSS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0 ))
    checkEquals(myOut[1], "Import type: spss" )

    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE)

   if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

    myOutOdbc <- capture.output(odbcDS <-
      RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString, verbose = 1))
    checkEquals(myOutOdbc[1], "Data source type: RxOdbcData, useFastRead = TRUE")

    myOutImport <- capture.output(myData <- rxImport(odbcDS, reportProgress = 0, verbose = 1))
    checkEquals(myOutImport[1], "Import type: odbcFast")
}

"test.rxImport.args.rowSelection.excludeFirst10" <- function()
{
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(
        file.path(rxGetOption("sampleDataDir"), "claims.sqlite"),
        useFastRead = FALSE),
          silent = TRUE)

    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

    odbcDSSlow <- RxOdbcData(sqlQuery = "SELECT * FROM claims",
       connectionString = conString,  useFastRead = FALSE)

    myDataSlow <- rxImport(odbcDSSlow, reportProgress = 0,
        rowSelection = (RowNum <= 50) & (RowNum > 24),
        transforms = list(RowNum2 = 2*RowNum))

    odbcDSFast <- RxOdbcData(sqlQuery = "SELECT * FROM claims",
       connectionString = conString,  useFastRead = TRUE)

    myDataFast <- rxImport(odbcDSFast, reportProgress = 0,
        rowSelection = (RowNum <= 50) & (RowNum > 24),
        transforms = list(RowNum2 = 2*RowNum))

    checkEquals(nrow(myDataSlow), nrow(myDataFast))

    inFileSAS <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
    # RowNum is character in the SAS data set
    myDataSAS <- rxImport( inData = inFileSAS,  reportProgress = 0,
         rowSelection = (as.integer(RowNum) <= 50) & (as.integer(RowNum) > 24),
         transforms = list(RowNum2 = 2*as.integer(RowNum)))

    checkEquals(nrow(myDataSlow), nrow(myDataSAS))

}

"test.rxImport.args.rowSelection.varsToKeep" <- function()
{
  # tests rowSelection with varsToKeep

  df1<-rxDataStep(mtcars, rowSelection=mpg > 20 )
  df2<-rxImport(mtcars, rowSelection=mpg > 20)

  checkTrue(identical(df1,df2), "rxImport != rxDataStep using rowSelection")

  df1<-rxDataStep(mtcars, rowSelection=mpg > 20, varsToKeep = c("mpg","cyl", "disp"))
  df2<-rxImport(mtcars, rowSelection=mpg > 20, varsToKeep = c("mpg","cyl", "disp"))

  checkTrue(identical(df1,df2), "rxImport != rxDataStep using rowSelection & varsToKeep")
}

"test.rxImport.args.type" <- function()
{
    # Delimited text
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0, type = "textFast" ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = TRUE, isFixedFormat = FALSE")
    checkEquals(myOut[2], "Import type: textFast(delimited)" )

    # Fixed format text
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

    myOut <- capture.output(outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, verbose = 1, reportProgress = 0, type = "fixed" , firstRowIsColNames = FALSE ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = FALSE, isFixedFormat = TRUE")
    checkEquals(myOut[2], "Import type: text(fixed format)"  )

  # Fixed fast
    myOut <- capture.output(outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, verbose = 1, reportProgress = 0, type = "fixedFast" ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = TRUE, isFixedFormat = TRUE")
    checkEquals(myOut[2], "Import type: textFast(fixed format)"  )

    # SAS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0, type = "sas" ))
    checkEquals(myOut[1], "Import type: sas" )

    # SPSS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0, type = "spss" ))
    checkEquals(myOut[1], "Import type: spss" )

    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE)

   if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

    myOutOdbc <- capture.output(odbcDS <-
      RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString, verbose = 1))
    checkEquals(myOutOdbc[1], "Data source type: RxOdbcData, useFastRead = TRUE")

    myOutImport <- capture.output(myData <- rxImport(odbcDS, reportProgress = 0, verbose = 1, type = "odbcFast"))
    checkEquals(myOutImport[1], "Import type: odbcFast")
}

"test.rxImport.data.logical" <- function()
{
  # Test to make sure valid non-zero values are set to TRUE
  # Get values for comparison
  sampleDir <- rxGetOption("sampleDataDir")
  inFile <- file.path(sampleDir, "claims.xdf")
  outData <- rxDataStep(inFile)
  costL <- as.logical(outData$cost)
  numberL <- as.logical(outData$number)

  testColClasses = c(cost = "logical", number = "logical")

    # Delimited text
  inFile <- file.path(sampleDir, "claims.txt")

  outData1 <- rxImport(inFile, colClasses = testColClasses, type = "textFast")
  checkEquals(outData1$cost, costL)
  checkEquals(outData1$number, numberL)

  outData2 <- rxImport(inFile, colClasses = testColClasses, type = "text")
  checkEquals(outData2$cost, costL)
  checkEquals(outData2$number, numberL)

    # Fixed format text
    inFile <- file.path(sampleDir, "claims.dat")
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "logical", start = 13, width = 6),
           number = list(type = "logical", start = 19, width = 3))

    outData3 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, reportProgress = 0, type = "fixed", firstRowIsColNames = FALSE )
  checkEquals(outData3$cost, costL)
  checkEquals(outData3$number, numberL)

  # Fixed fast
    outData4 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, reportProgress = 0, type = "fixedFast" )
  checkEquals(outData4$cost, costL)
  checkEquals(outData4$number, numberL)

    # SAS
    inFile <- file.path(sampleDir, "claims.sas7bdat")
  outData5 <- rxImport(inFile, colClasses = testColClasses)
  checkEquals(outData5$cost, costL)
  checkEquals(outData5$number, numberL)

    # SPSS
    inFile <- file.path(sampleDir, "claims.sav")
  outData6 <- rxImport(inFile, colClasses = testColClasses)
  checkEquals(outData6$cost, costL)
  checkEquals(outData6$number, numberL)

    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(
    file.path(rxGetOption("sampleDataDir"),"claims.sqlite"), useFastRead = TRUE),
          silent = TRUE)

   if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

     odbcDS <-
      RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString)

  outData7 <- rxImport(inData = odbcDS, colClasses = testColClasses)
  checkEquals(outData7$cost, costL)
  checkEquals(outData7$number, numberL)

}

"test.rxImport.rowsToSniff" <- function()
{

    tinyFileName <- outXDF <-  tempfile( pattern = ".rxImport1", fileext = ".txt")
    writeLines(c("a,b,c",
                 "1,1,1",
                 "2,NA,NA",
                 "3,3, three"), tinyFileName)

    # Just sniff first two rows - should be integer
    myData1 <- rxImport(tinyFileName, firstRowIsColNames = TRUE,
        rowsToSniff = 2, defaultDecimalColType = "numeric", overwrite = TRUE,
        type = "textFast", reportProgress = 0)

    varInfo1 <- rxGetVarInfo(myData1)
    checkEquals(varInfo1$a$varType, "integer")
    checkEquals(varInfo1$b$varType, "integer")
    checkEquals(varInfo1$c$varType, "integer")

    # Redo with StatTran text import
    myData2 <- rxImport(tinyFileName, firstRowIsColNames = TRUE,
        rowsToSniff = 2, defaultDecimalColType = "numeric", overwrite = TRUE,
        type = "text", reportProgress = 0)

    varInfo2 <- rxGetVarInfo(myData2)
    checkEquals(varInfo2$a$varType, "integer")
    checkEquals(varInfo2$b$varType, "integer")
    checkEquals(varInfo2$c$varType, "integer")

    # Now sniff 4 rows - third col should be character
    myData3 <- rxImport(tinyFileName, firstRowIsColNames = TRUE,
        rowsToSniff = 4, defaultDecimalColType = "numeric", overwrite = TRUE,
        type = "textFast", reportProgress = 0)

    varInfo3 <- rxGetVarInfo(myData3)
    checkEquals(varInfo3$a$varType, "integer")
    checkEquals(varInfo3$b$varType, "integer")
    checkEquals(varInfo3$c$varType, "character")

    # Redo with StatTran text import
    myData4 <- rxImport(tinyFileName, firstRowIsColNames = TRUE,
        rowsToSniff = 4, defaultDecimalColType = "numeric", overwrite = TRUE,
        type = "text", reportProgress = 0)

    varInfo4 <- rxGetVarInfo(myData4)
    checkEquals(varInfo4$a$varType, "integer")
    checkEquals(varInfo4$b$varType, "integer")
    checkEquals(varInfo4$c$varType, "character")

    file.remove(tinyFileName)
}

"test.rxImport.args.varsToKeep" <- function()
{
    # Setup
    inPath <- rxGetOption("sampleDataDir")
    varsToKeep = c("age", "type", "cost")
    varsToDrop = c("RowNum", "car.age", "number")

    # Delimited text
    inFile <- file.path(inPath, "claims.txt")
    outData1 <- rxImport( inData = inFile, varsToKeep = varsToKeep,
        reportProgress = 0, type = "textFast" )
    outData2 <- rxImport( inData = inFile, varsToDrop = varsToDrop,
        reportProgress = 0, type = "textFast" )
    dataInfo1 <- rxGetInfo(outData1)
    checkEquals( dataInfo1$numVars, 3)
    checkEquals(outData1, outData2)

    inFile <- file.path(inPath, "claims.txt")
    outData1 <- rxImport( inData = inFile, varsToKeep = varsToKeep,
        reportProgress = 0, type = "text")
    outData2 <- rxImport( inData = inFile, varsToDrop = varsToDrop,
        reportProgress = 0, type = "text" )
    dataInfo1 <- rxGetInfo(outData1)
    checkEquals( dataInfo1$numVars, 3)
    checkEquals(outData1, outData2)

    # SAS
    inFile <- file.path(inPath, "claims.sas7bdat")
        varsToDrop = c("RowNum", "car_age", "number")
    outData1 <- rxImport( inData = inFile, varsToKeep = varsToKeep, reportProgress = 0 )
    outData2 <- rxImport( inData = inFile, varsToDrop = varsToDrop, reportProgress = 0 )
    dataInfo1 <- rxGetInfo(outData1)
    checkEquals( dataInfo1$numVars, 3)
    checkEquals(outData1, outData2)

    # SPSS
    inFile <- file.path(inPath, "claims.sav")
    outData1 <- rxImport( inData = inFile, varsToKeep = varsToKeep, reportProgress = 0 )
    outData2 <- rxImport( inData = inFile, varsToDrop = varsToDrop, reportProgress = 0 )
    dataInfo1 <- rxGetInfo(outData1)
    checkEquals( dataInfo1$numVars, 3)
    checkEquals(outData1, outData2)

    # Set warnings to trigger errors
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)

    # Fixed format text: not yet implemented
    inFile <- file.path(inPath, "claims.dat")

    claimsFFColInfo <-
      list(RowNum = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

    res <- try(outData1 <- rxImport( inData = inFile, varsToKeep = varsToKeep,
        colInfo = claimsFFColInfo, verbose = 1, type = "fixed", reportProgress = 0 ))
    warningExpr <- "'varsToDrop' not supported"
    checkTrue(inherits(res,"try-error"))
    checkTrue(length(grep(warningExpr, as.character(res))) > 0)

    res <- try(outData1 <- rxImport( inData = inFile, varsToKeep = varsToKeep,
        colInfo = claimsFFColInfo, verbose = 1, type = "fixedFast", reportProgress = 0 ))
    warningExpr <- "'varsToDrop' not supported"
    checkTrue(inherits(res,"try-error"))
    checkTrue(length(grep(warningExpr, as.character(res))) > 0)

    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE)

   if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

    odbcDS <-
      RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString)
     # Won't get warning if no rowSelection or transformation
    res <- try(outData1 <- rxImport( inData = odbcDS, varsToKeep = varsToKeep,
        rowSelection = cost > 100,
        reportProgress = 0 ))
    warningExpr <- "'varsToDrop' not supported"
    checkTrue(inherits(res,"try-error"))
    checkTrue(length(grep(warningExpr, as.character(res))) > 0)
}

"test.rxImport.args.inData.blank" <- function()
{
    # RA-4612
    inFile <- ""
    res <- try(
        myData <- rxImport( inData = inFile, stringsAsFactors = FALSE)
    )

    # check that error is thrown
  checkTrue(is(res, "try-error"))

    # Check that the text of the error is reasonable
    checkTrue(grepl("must be a character string containing a file name or", res))
}

"test.rxImport.args.rowsPerRead" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt")
    outFile <- file.path(tempdir(), ".rxTempTest300r.xdf")
    rxImport(inData=fileName, outFile=outFile, overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
    checkEquals(fileInfo$numBlocks, 1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 100, overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 3)
  }

    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 120, overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars,9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 3)
  }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 200,
        defaultReadBufferSize=100, overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 2)
  }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    # Repeat with different file
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test200rSemiColon.txt")
    outFile <- file.path(tempdir(), ".rxTempTest200r.xdf")
    rxImport(inData=fileName, outFile=outFile, delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
    checkEquals(fileInfo$numBlocks, 1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 20, delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 10)
  }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 12,
        delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars,9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 17)
  }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 24,
        defaultReadBufferSize=5, delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows,200)
    checkEquals(fileInfo$numVars, 9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 9)
  }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 10,
        defaultReadBufferSize=100, delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 20)
  }
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 200,
     defaultReadBufferSize = 100, delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 200)
    checkEquals(fileInfo$numVars, 9)
    checkEquals(fileInfo$numBlocks, 1)
    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)

     # Repeat with a bigger file
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300rSemiColon.txt")
    outFile <- file.path(tempdir(), ".rxTempTest300r.xdf")
    rxImport(inData=fileName, outFile=outFile, rowsPerRead = 200,
     defaultReadBufferSize = 100, delimiter=";", overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
    checkEquals(fileInfo$numRows, 300)
    checkEquals(fileInfo$numVars, 9)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 2)
  }
    # The text file was created by replicated 100 rows 3 times
    # Make sure the first 100 and last 100 match
    myDF1 <- rxDataStep(inData=outFile, numRows=100)
    myDF2 <- rxDataStep(inData=outFile, numRows=100, startRow=201)
    checkEquals(myDF1, myDF2)

    # xdf to xdf, resetting rowsPerRead
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    #outFile <- file.path(tempdir(), ".rxTempTest200r.xdf")
    rxImport(inData = test100r5b, outFile = outFile, rowsPerRead = 10, overwrite=TRUE)
    fileInfo <- rxGetInfo(outFile)
  if (rxIsExpressEdition())
  {
    checkEquals(fileInfo$numBlocks, 1)
  } else
  {
        checkEquals(fileInfo$numBlocks, 10)
  }

    if (file.exists(outFile)) file.remove(outFile)
    rm(fileInfo)
}

"test.rxImport.args.rowsPerRead.odbc.useFastRead.TRUE" <- function()
{
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"),
                        useFastRead = TRUE),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

  # Temporary conString
     #conString <- paste("Driver=SQLite3 ODBC Driver;Database=", file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), sep = "")

  claimsOdbc <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                useFastRead = TRUE,
                connectionString = conString,
                rowsPerRead = 32)

  myDataFast <- rxImport(claimsOdbc)

  res <- try(checkEquals(nrow(myDataFast), 128), silent=TRUE)
  if (inherits(res, "try-error"))
  {
    message("Incorrect number of rows returned. Verify that you have a compatible SQLite ODBC driver (!=.92).")
  }
  res
}
"test.rxImport.args.rowsPerRead.odbc.useFastRead.FALSE" <- function()
{
  # Compares FALSE with TRUE
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"),
                        useFastRead = FALSE),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

  # Temporary conString
     #conString <- paste("Driver=SQLite3 ODBC Driver;Database=", file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), sep = "")

  claimsOdbc <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                useFastRead = TRUE,
                connectionString = conString,
                rowsPerRead = 32)

  myDataFast <- rxImport(claimsOdbc)


  claimsOdbcSlow <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                useFastRead = FALSE,
                connectionString = conString,
                rowsPerRead = 32)
  myDataSlow <- rxImport(claimsOdbcSlow)

  checkEquals(myDataFast, myDataSlow)
  checkEquals(nrow(myDataFast), 128)
}


"test.rxImport.args.verbose2" <- function()
{
    # Delimited text
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0 ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = TRUE, isFixedFormat = FALSE")
    checkEquals(myOut[2], "Import type: textFast(delimited)" )

    # Fixed format text
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

    myOut <- capture.output(outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, verbose = 1, reportProgress = 0, useFastRead = FALSE ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = FALSE, isFixedFormat = TRUE")
    checkEquals(myOut[2], "Import type: text(fixed format)"  )

    myOut <- capture.output(outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, verbose = 1, reportProgress = 0, useFastRead = TRUE ))
    checkEquals(myOut[1], "Data source type: RxTextData, useFastRead = TRUE, isFixedFormat = TRUE")
    checkEquals(myOut[2], "Import type: textFast(fixed format)"  )
    # SAS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0 ))
    checkEquals(myOut[1], "Import type: sas" )

    # SPSS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
    myOut <- capture.output(outData1 <- rxImport( inData = inFile, verbose = 1, reportProgress = 0 ))
    checkEquals(myOut[1], "Import type: spss" )

    # SQLITE
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
    conString <-
      try(rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"),
                                                    "claims.sqlite")),
          silent = TRUE)

   if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])

    myOutOdbc <- capture.output(odbcDS <-
      RxOdbcData(sqlQuery = "SELECT * FROM claims",
                 connectionString = conString, verbose = 1))
    checkEquals(myOutOdbc[1], "Data source type: RxOdbcData, useFastRead = TRUE")

    myOutImport <- capture.output(myData <- rxImport(odbcDS, reportProgress = 0, verbose = 1))
    checkEquals(myOutImport[1], "Import type: odbcFast")
}

"test.rxImport.args.rowsPerRead.nonxdf" <- function()
{

    # Fixed format text
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
    claimsFFColInfo <-
      list(rownames = list(start = 1, width = 3),
           age = list(type = "factor", start = 4, width = 5),
           car.age = list(type = "factor", start = 9, width = 3),
           type = list(type = "factor", start = 12, width = 1),
           cost = list(type = "numeric", start = 13, width = 6),
           number = list(type = "integer", start = 19, width = 3))

  outData1 <- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, reportProgress = 0, type = "fixed", firstRowIsColNames = FALSE, rowsPerRead = 35 )


  # Fixed fast
    outData2<- rxImport( inData = inFile,
        colInfo = claimsFFColInfo, reportProgress = 0, type = "fixedFast", rowsPerRead = 35 )

  checkEquals(outData1, outData2)
  checkEquals(nrow(outData1), 128)

    # SAS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat")
  outData3 <- rxImport( inData = inFile, reportProgress = 0, type = "sas" , rowsPerRead = 35)


    # SPSS
    inFile <- file.path(rxGetOption("sampleDataDir"), "claims.sav")
    outData4 <- rxImport( inData = inFile, reportProgress = 0, type = "spss", rowsPerRead = 35)
    checkEquals(outData3, outData4 )
   # ODBC in separate file
}

 "test.rxImport.args.colInfo.newLevels.newName.append" <- function()
{
    # Delimited text
  outFile <- tempfile(pattern = ".rxTestAppend", fileext = ".xdf")
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")

    # Specify 'newLevels' for one column;  'newName' for another
    testColInfo <- list(
        xint1 = list(type= "factor", levels = as.character(5:15), newLevels = letters[1:11]),
        xfac1 = list(type = "factor", newName = "newXFac1"))
  inDS <- RxTextData(file = inFile,
    colInfo = testColInfo
    )
    outData1 <- rxImport( inData = inDS, outFile = outFile,
       varsToKeep = c("xint1", "xfac1"),
    overwrite = TRUE )
  varInfo1 <- rxGetVarInfo(outData1)
  xint1Levels1 <- varInfo1$xint$levels

    # Import with append
  outData2 <- rxImport( inData = inDS, outFile = outFile,
        varsToKeep = c("xint1", "xfac1"),
        append = "rows" )

    varInfo2 <- rxGetVarInfo(outData2)
  xint1Levels2 <- varInfo2$xint$levels
  checkEquals(xint1Levels2, xint1Levels1)
    file.remove(outFile)
}

"test.rxImport.data.RxTextData.quotedDelimiters.transforms" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 77131: rxTextToXdf and rxImport deadlocks in SPEES with transform function")

  tempCsv <- tempfile(pattern = ".rxTestAppend", fileext = ".csv")
    myData <- data.frame( x = 1:3,
        text= c("hello","\"Hi, there, \"\"", "goodby"),
    stringsAsFactors = FALSE)

    write.csv(myData, file = tempCsv, row.names = FALSE, quote = FALSE)

    textDS <- RxTextData(file = tempCsv,
        firstRowIsColNames = TRUE,
        colInfo = list(x = list(type = "integer")), delimiter = ",", quotedDelimiters = TRUE)

    myData <- rxImport(inData = textDS,
        transforms = list(
        RowNum = .rxStartRow : (.rxStartRow + .rxNumRows - 1)),
        transformVars = "x"
    )
    varInfo <- rxGetVarInfo(myData)
    checkEquals(varInfo$RowNum$high, 3)
    file.remove(tempCsv)
}


"test.rxImport.args.colInfo.index" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt")

    myData1 <- rxImport(inData=fileName, colInfo = list(
        ylogical = list(type = "logical", newName = "reallyYLogical")) )
    varInfo1 <- rxGetVarInfo(myData1 )
    checkEquals(varInfo1$reallyYLogical$varType, "logical")

    myData2 <- rxImport(inData=fileName, colInfo = list(
       list(index = 2, type = "logical", newName = "reallyYLogical")) )
    varInfo2 <- rxGetVarInfo(myData2 )
    checkEquals(varInfo1, varInfo2)

    myData3 <- rxImport(inData=fileName, colInfo = list(
        ylogical = list(type = "logical", newName = "reallyYLogical"),
        xnum1 = list(type = "integer", newName = "xnum1int")) )
    varInfo3 <- rxGetVarInfo(myData3 )
    checkEquals(varInfo3$xnum1int$varType, "integer")

    # Try a mix of indices and names
    myData4 <- rxImport(inData=fileName, colInfo = list(
       list(index = 2, type = "logical", newName = "reallyYLogical"),
        xnum1 = list(type = "integer", newName = "xnum1int")
    ) )
    varInfo4 <- rxGetVarInfo(myData4 )
    checkEquals(varInfo3, varInfo4)

 }
"test.rxImport.args.useFastRead_FALSE.colInfo.index" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt")
  useFastRead <- FALSE

    myData1 <- rxImport(inData=fileName, colInfo = list(
        ylogical = list(type = "logical", newName = "reallyYLogical")),
    useFastRead = useFastRead, firstRowIsColNames = TRUE)
    varInfo1 <- rxGetVarInfo(myData1 )
    checkEquals(varInfo1$reallyYLogical$varType, "logical")

    myData2 <- rxImport(inData=fileName, colInfo = list(
       list(index = 2, type = "logical", newName = "reallyYLogical")),
    useFastRead = useFastRead, firstRowIsColNames = TRUE )
    varInfo2 <- rxGetVarInfo(myData2 )
    checkEquals(varInfo1, varInfo2)

    myData3 <- rxImport(inData=fileName, colInfo = list(
        ylogical = list(type = "logical", newName = "reallyYLogical"),
        xnum1 = list(type = "integer", newName = "xnum1int")),
    useFastRead = useFastRead, firstRowIsColNames = TRUE)
    varInfo3 <- rxGetVarInfo(myData3 )
    checkEquals(varInfo3$xnum1int$varType, "integer")

    # Try a mix of indices and names
    myData4 <- rxImport(inData=fileName, colInfo = list(
       list(index = 2, type = "logical", newName = "reallyYLogical"),
        xnum1 = list(type = "integer", newName = "xnum1int")),
    useFastRead = useFastRead, firstRowIsColNames = TRUE )
    varInfo4 <- rxGetVarInfo(myData4 )
    checkEquals(varInfo3, varInfo4)

 }

"test.rxImport.args.xdfCompressionLevel" <- function()
{
    fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt")
    outFile1 <- tempfile(pattern = ".rxTestCompress1", fileext = ".xdf")
    outFile2 <- tempfile(pattern = ".rxTestCompress2", fileext = ".xdf")

    myData1 <- rxImport(inData=fileName, outFile = outFile1,
        xdfCompressionLevel = 2,
    useFastRead = TRUE)

    myData2 <- rxImport(inData=fileName, outFile = outFile2,
        xdfCompressionLevel = -1,
    useFastRead = TRUE)

    # TODO - put a test here to compare output file sizes
    file.remove(outFile1)
    file.remove(outFile2)

    myData1 <- rxImport(inData=fileName, outFile = outFile1,
        xdfCompressionLevel = 2,
    useFastRead = FALSE)

    myData1 <- rxImport(inData=fileName, outFile = outFile2,
        xdfCompressionLevel = 7,
    useFastRead = FALSE)

    # TODO - put a test here to compare output file sizes
    file.remove(outFile1)
    file.remove(outFile2)
}

"test.rxImport.data.badRow" <- function()
{

    # Create a text data file with a "bad" row
    txtFileName <- tempfile(pattern = ".rxTestBadRow", fileext = ".txt")

  rxTiny <-  data.frame(a = 1:3, b = c(4.1, 5.2, 6.3), c = c("one", "two", "three"),
           stringsAsFactors = FALSE)
    write.table(rxTiny, file = txtFileName, row.names = FALSE, sep = ",")
    rxBad <-  data.frame(a = 1)
    write.table(rxBad, file = txtFileName, row.names = FALSE, col.names = FALSE, append = TRUE, sep = ",")
    write.table(rxTiny, file = txtFileName, row.names = FALSE, col.names = FALSE, append = TRUE, sep = ",")

    importedData <- rxImport(inData = txtFileName, defaultDecimalColType = "numeric")

  rxBadCorrected <-data.frame(a = 1, b = as.numeric(NA), c = as.character(NA))
  checkData <- rbind(rxTiny, rxBadCorrected)
  checkData <- rbind(checkData, rxTiny)
  checkEquals(importedData, checkData)

  file.remove(txtFileName)

}

## TODO: Add Fast Fixed-format test
"test.rxImport.data.stringFixedWidth" <- function()
{
    fixedFileName <- file.path(rxGetOption("unitTestDataDir"), "richTest100rFW.dat")
    outXdfName <- tempfile(pattern = ".rxFWS", fileext = ".xdf")
    iXdfCompress <- rxGetOption("xdfCompressionLevel")
    for (iXdfCompress in 0:1)
    {
        rxImport(inData = fixedFileName, outFile = outXdfName, rowsPerRead = 30,
                overwrite = TRUE,
                colInfo =
                 list(y = list(type = "numeric", start = 1, width = 8, newName="NEWy"),
            z.char1 = list( start = 73, width = 15)),
                 missingValueString = "",
                 xdfCompressionLevel = iXdfCompress)

        varInfo <- rxGetVarInfo( outXdfName )
      checkEquals(varInfo$z.char1$colWidth, 15)
      inData <- rxDataStep(outXdfName)
      # Quick sanity check on character data
      checkEquals(inData$z.char1[1:5], c("Washington", "Washington", "Washington", "Washington", "North Carolina"))
  }
  # Re-import, specifying character data type
    rxImport(inData = fixedFileName, outFile = outXdfName, rowsPerRead = 30,
        firstRowIsColNames = FALSE,
                overwrite = TRUE,
                colInfo =
                 list(y = list(type = "numeric", start = 1, width = 8, newName="NEWy"),
            z.char1 = list(type = "character", start = 73, width = 15)),
                 missingValueString = "")

  varInfo1 <- rxGetVarInfo( outXdfName )
  checkEquals(varInfo1$z.char1$colWidth, 15)

  # Do a round trip on fixed width character data
  myData <- rxDataStep(inData = outXdfName, varsToKeep = "z.char1")
  outXdfSmall <- tempfile(pattern = ".rxFWS2", fileext = ".xdf")
  rxDataStep(inData = myData, outFile = outXdfSmall, overwrite = TRUE)
  varInfoSmall <- rxGetVarInfo(outXdfSmall)
    print(varInfoSmall)
  checkEquals(varInfoSmall$z.char1$colWidth, 15)

  # Make sure the data is the same
  myDataSmall <- rxDataStep(inData = outXdfSmall, varsToKeep = "z.char1")
    print(myData$z.char1)
    print(myDataSmall$z.char1)

  checkEquals(myData$z.char1, myDataSmall$z.char1)

  # Now try with comma delimited data, both 'text' and 'textFast'
  delimitedFileName <- file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")
  rxImport(inData = delimitedFileName, outFile = outXdfName, type = "text",
      rowsPerRead = 30, overwrite = TRUE,
      varsToKeep = c("y", "z.char1"),
            colInfo =
                 list(y = list(type = "numeric", newName="NEWy"),
            z.char1 = list(type = "character", width = 15)),
            missingValueString = "")

  varInfo2 <- rxGetVarInfo( outXdfName )
  checkEquals(varInfo2$z.char1$colWidth, 15)

  # Make sure the data is the same
  myDataDelim <- rxDataStep(inData = outXdfName, varsToKeep = "z.char1")
  checkEquals(myData$z.char1, myDataDelim$z.char1)

  # Now try textFast
  rxImport(inData = delimitedFileName, outFile = outXdfName, type = "textFast",
      rowsPerRead = 30, overwrite = TRUE,
      varsToKeep = c("y", "z.char1"),
            colInfo =
                 list(y = list(type = "numeric", newName="NEWy"),
            z.char1 = list(type = "character", width = 15)),
            missingValueString = "")

  varInfo3 <- rxGetVarInfo( outXdfName )
  checkEquals(varInfo3$z.char1$colWidth, 15)

  # Make sure the data is the same
  myDataDelim3 <- rxDataStep(inData = outXdfName, varsToKeep = "z.char1")
  checkEquals(myData$z.char1, myDataDelim3$z.char1)

  # Now try appending
    rxImport(inData = delimitedFileName, outFile = outXdfName, type = "textFast",
      rowsPerRead = 30,
      varsToKeep = c("y", "z.char1"),
            colInfo =
                 list(y = list(type = "numeric", newName="NEWy"),
            z.char1 = list(type = "character", width = 15)),
            missingValueString = "", append = "rows")

  varInfo3a <- rxGetVarInfo( outXdfName )
  checkEquals(varInfo3a$z.char1$colWidth, 15)

    myDataDelim3a <- rxDataStep(inData = outXdfName, varsToKeep = "z.char1")
    totalRows <- nrow(myDataDelim3a)
  checkEquals(as.vector(myData$z.char1), as.vector(myDataDelim3a$z.char1[((totalRows/2)+1):totalRows]))

    # Now try appending regular character data

    # This should give warning that character data will be fixed width
    rxImport(inData = delimitedFileName, outFile = outXdfName, type = "textFast",
      rowsPerRead = 30,
      varsToKeep = c("y", "z.char1"),
            colInfo =
                 list(y = list(type = "numeric", newName="NEWy"),
            z.char1 = list(type = "character")),
            missingValueString = "", append = "rows")

  varInfo3b <- rxGetVarInfo( outXdfName )
  checkEquals(varInfo3b$z.char1$colWidth, 15)

    myDataDelim3b <- rxDataStep(inData = outXdfName, varsToKeep = "z.char1")
    totalRows <- nrow(myDataDelim3b)
  checkEquals(as.vector(myData$z.char1), as.vector(myDataDelim3b$z.char1[201:totalRows]))

    # Try importing with a fixed width that is too small
    rxImport(inData = delimitedFileName, outFile = outXdfName, type = "textFast",
      rowsPerRead = 30,
      varsToKeep = c("y", "z.char1"),
            colInfo =
                 list(y = list(type = "numeric", newName="NEWy"),
            z.char1 = list(type = "character", width = 8)),
            missingValueString = "", overwrite = TRUE)
  myDataTrunc <- rxDataStep(inData = outXdfName, varsToKeep = "z.char1")
    testTrunc <- as.vector(substring(myData$z.char1, first = 1, last = 8))
    checkEquals(as.vector(myDataTrunc$z.char1), testTrunc)

   # Now try with SPSS
  spssFileName <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
  rxImport(inData = spssFileName, outFile = outXdfName,
      rowsPerRead = 30, overwrite = TRUE,
      varsToKeep = c("Y", "Z_CHAR1"),
            colInfo =
                 list(Y = list(type = "numeric", newName="NEWy"),
            Z_CHAR1 = list(type = "character", width = 15))
            )

  varInfo4 <- rxGetVarInfo( outXdfName )
  checkEquals(varInfo4$Z_CHAR1$colWidth, 15)

  # Make sure the data is the same
  myDataSpss <- rxDataStep(inData = outXdfName, varsToKeep = "Z_CHAR1")
  names(myDataSpss) <- "z.char1"
  checkEquals(myData$z.char1, myDataSpss$z.char1)

  file.remove(outXdfName)
  file.remove(outXdfSmall)

}

"test.rxImport.RxTextData.fixed.stringsAsFactors" <- function()
{
    dataFile.sts <- file.path(rxGetOption("sampleDataDir"),"claims.sts")

    claimsTextSource.sts <- RxTextData(dataFile.sts, stringsAsFactors=TRUE)
    claimsDF <- rxImport(inData=claimsTextSource.sts)

    claimsVarInfo <- rxGetVarInfo(claimsDF)
    checkEquals(claimsVarInfo$age$varType, "factor")
    checkEquals(claimsVarInfo$car.age$varType, "factor")
    checkEquals(claimsVarInfo$type$varType, "factor")
}

###  Test non-ST Fixed format text:	Use datasource object
"test.rxImport.RxTextData.FastFixed" <- function()
{
    dataFile.sts <- file.path(rxGetOption("sampleDataDir"),"claims.sts")

    ffDS_nonST <- RxTextData(dataFile.sts, missingValueString = "",
                  useFastRead = TRUE,
                  stringsAsFactors=TRUE)
    xdfFile_nonST <- tempfile(pattern = "nonST_FixedFormatClaims", fileext = ".xdf")

    xdfDS_nonST <- RxXdfData(xdfFile_nonST)
    rxImport(ffDS_nonST, xdfDS_nonST, overwrite = TRUE)
    DF_nonST <- rxDataStep(xdfFile_nonST)

    ffDS <- RxTextData(dataFile.sts, missingValueString = "",
                  useFastRead = FALSE,
                  firstRowIsColNames = FALSE,
                  stringsAsFactors=TRUE)


    xdfFile <- tempfile(pattern = "ST_FixedFormatClaims", fileext = ".xdf")
    xdfDS <- RxXdfData(xdfFile)
    rxImport(ffDS, xdfDS, overwrite = TRUE)
    DF_ST <- rxDataStep(xdfFile)

    checkEquals(DF_nonST, DF_ST)
  file.remove(xdfFile_nonST)
  file.remove(xdfFile)
}

###  Test non-ST Fixed format text:	Use datasource object
"test.rxImport.RxTextData.FastFixed.stringsAsFactors" <- function()
{
    dataFile.sts <- file.path(rxGetOption("sampleDataDir"),"claims.sts")

    ffDS <- RxTextData(dataFile.sts,
                  useFastRead = TRUE,
                  stringsAsFactors=TRUE)
    claimsDF <- rxImport(inData=ffDS)

    claimsVarInfo <- rxGetVarInfo(claimsDF)
    checkEquals(claimsVarInfo$age$varType, "factor")
    checkEquals(claimsVarInfo$car.age$varType, "factor")
    checkEquals(claimsVarInfo$type$varType, "factor")
}

"test.rxImport.args.quotedDelimiters" <- function()
{
  tempCsv <- tempfile(pattern = "rxImportTest", fileext = ".csv")
  # quoted delimters
  writeLines(c("1,\"Jones, Mary\",A",
         "2,\"James, John\",B",
         "3,\"Lamp, Harold\",C"),
        tempCsv)
  tempData <- rxImport(tempCsv, quotedDelimiters = TRUE, firstRowIsColNames = FALSE)
  checkEquals(as.vector(tempData$V1), 1:3)
  checkEquals(tempData$V2[2], "James, John")

  # Check to see that StatTransfer import always supports quoted delimiters
  tempData2 <- rxImport(tempCsv, quotedDelimiters = FALSE, useFastRead = FALSE,
    firstRowIsColNames = FALSE)
  checkEquals(as.vector(tempData2$V1), 1:3)
  checkEquals(tempData2$V2[2], "James, John")

  #RA-10675 quotes within quotes
  writeLines(c("1,\"Toys \"R\" Us\",A",
         "2,\"Walmart\",B",
         "3,\"Sears\",C"),
        tempCsv)

  tempData <- rxImport(tempCsv, firstRowIsColNames = FALSE)
  checkEquals(as.vector(tempData$V1), 1:3)
  checkEquals(tempData$V2[1], "Toys \"R\" Us")

  file.remove(tempCsv)

}


"test.rxImport.args.colInfo.width" <- function()
{
    # If width is specified for character data, it should be fixed width
  fileName <- file.path(rxGetOption("unitTestDataDir"), "test300r.txt")
  useFastRead <- FALSE

    myData1 <- rxImport(inData=fileName, colInfo = list(
        ylogical = list(type = "logical", newName = "reallyYLogical")),
    useFastRead = useFastRead)
    varInfo1 <- rxGetVarInfo(myData1 )
    checkEquals(varInfo1$reallyYLogical$varType, "logical")
}

"test.rxImport.args.colInfo.newLevels" <- function()
{
  outFile <- tempfile(pattern = "levelsTest", fileext = ".xdf")
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
    if (file.exists(outFile)) file.remove(outFile)

    origLevels <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
    newLevels <- c("aa","bb", "cc", "dd", "ee", "ff", "gg", "hh", "ii", "jj")
    testColInfo <- list(
    xfac2=list(type="factor",
          levels=origLevels,
            newLevels=newLevels),
        xfac1 = list(type = "factor")
   )

    inDataSource <- RxTextData(inFile, colInfo=testColInfo)
    rxImport(inData = inDataSource, outFile = outFile, overwrite = TRUE)
    levels1 <- rxGetVarInfo(outFile)[["xfac2"]]$levels

    rxImport(inData = inDataSource, outFile = outFile, append = "rows")
    levels2 <- rxGetVarInfo(outFile)[["xfac2"]]$levels
    checkEquals(levels1, levels2)
  file.remove(outFile)

}


"test.rxImport.args.rowsPerRead" <- function()
{
  claimsFF <- file.path(rxGetOption("sampleDataDir"), "claims.dat")
  claimsXdf <- file.path(tempdir(), "importedClaims.xdf")

  # Create an RxTextData data source

  claimsDS <- RxTextData( file = claimsFF,
    colInfo =  list(
       rownames = list(start = 1, width = 3),
       age = list(type = "factor", start = 4, width = 5),
       car.age = list(type = "factor", start = 9, width = 3),
       type = list(type = "factor", start = 12, width = 1),
       cost = list(type = "numeric", start = 13, width = 6),
       number = list(type = "integer", start = 19, width = 3)),
       rowsPerRead = 50 # rowsPerRead will determine block size
       )

  # Import the data specified in the data source into an xdf file
    rowsPerRead <- 65
  rxImport(inData = claimsDS, outFile = claimsXdf,
    rowsPerRead = rowsPerRead,
    overwrite = TRUE)

  fileInfo <- rxGetInfo(claimsXdf, getBlockSizes = TRUE)

    if (rxIsExpressEdition())
    {
        rowsPerRead <- 128
    }
  checkEquals(fileInfo$rowsPerBlock[1], rowsPerRead)

}

"test.rxImport.args.inData.data.frame" <- function()
{
  # Test to see if factor levels can be appended with new data
  outFile <- tempfile(pattern = "importDF", fileext = ".xdf")
  labels1 <- c("One", "Two")
  labels2 <- c(labels1, "Three", "Four")
  myData1 <- data.frame(x = 1:100, y = factor(rep(1:2, times = 50), levels = 1:2, labels = labels1))
  myData2 <- data.frame(x = 1:100, y = factor(rep(1:4, times = 25), levels = 1:4, labels = labels2))

  if (file.exists(outFile)) file.remove(outFile)

  rxImport(inData = myData1, outFile = outFile)
  varInfo1 <- rxGetVarInfo(outFile)
  checkEquals(varInfo1$y$levels, labels1)

  rxImport(inData = myData2, outFile = outFile, append = "rows")
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(varInfo2$y$levels, labels2)

    file.remove(outFile)

    myData1 <- data.frame(x = 1:100, y = rep(c("One", "Two"), times = 50), stringsAsFactors = FALSE)
    myData2 <- data.frame(x = 1:100, y = rep(c( "Two", "Three"), times = 50), stringsAsFactors = FALSE)

    rxImport(inData = myData1, outFile = outFile, stringsAsFactors = TRUE)
  varInfo1 <- rxGetVarInfo(outFile)
  checkEquals(varInfo1$y$levels, labels1)

  rxImport(inData = myData2, outFile = outFile, append = "rows", stringsAsFactors = TRUE)
  varInfo2 <- rxGetVarInfo(outFile)
    labels3 <- c("One", "Two", "Three")
  checkEquals(varInfo2$y$levels, labels3 )

  file.remove(outFile)

    # Test with colInfo
    y1 <- rep(c("One", "Two"), times = 50)
    y2 <- rep(c( "Two", "Three"), times = 50)
    labels4 <- c("One", "Two", "Three", "Four")
    myData1 <- data.frame(x = y1, y = y1, stringsAsFactors = FALSE)
    myData2 <- data.frame(x = y2, y = y2, stringsAsFactors = FALSE)
    rxImport(inData = myData1, outFile = outFile, stringsAsFactors = TRUE,
      colInfo = list(x = list(type = "factor", levels = c("One", "Two", "Three", "Four")))
      )
  varInfo1 <- rxGetVarInfo(outFile)
  checkEquals(varInfo1$x$levels, labels4)

  rxImport(inData = myData2, outFile = outFile, append = "rows", stringsAsFactors = TRUE,
          colInfo = list(x = list(type = "factor", levels = c("One", "Two", "Three", "Four")))
     )
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(varInfo2$x$levels, labels4)
    checkEquals(varInfo2$y$levels, labels3)
    file.remove(outFile)

    # Factors in data frame
    x1 <- rep(c("A", "B"), times = 50)
    x2 <- rep(c( "B", "C"), times = 50)
    labels4 <- c("A", "B", "C", "D")
    y <- rep(labels4, times = 25)

    myData1 <- data.frame(x = x1, y = y, stringsAsFactors = TRUE)
    myData2 <- data.frame(x = x2, y = y, stringsAsFactors = TRUE)
    rxImport(inData = myData1, outFile = outFile, stringsAsFactors = TRUE,
      colInfo = list(x = list(type = "factor", levels = labels4))
      )
  varInfo1 <- rxGetVarInfo(outFile)
  checkEquals(varInfo1$x$levels, labels4)
    checkEquals(varInfo1$x$levels, labels4)

  rxImport(inData = myData2, outFile = outFile, append = "rows", stringsAsFactors = TRUE,
          colInfo = list(x = list(type = "factor", levels = labels4))
     )
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(varInfo2$x$levels, labels4)
    checkEquals(varInfo2$y$levels, labels4)

    file.remove(outFile)

    # Factors in data frame with full colInfo
    x1 <- rep(c("A", "B"), times = 50)
    x2 <- rep(c( "B", "C"), times = 50)
    labels4 <- c("A", "B", "C", "D")
    labels5 <- c("A", "B", "C", "D", "E")
    y <- rep(labels4, times = 25)

    myData1 <- data.frame(x = x1, y = y, stringsAsFactors = TRUE)
    myData2 <- data.frame(x = x2, y = y, stringsAsFactors = TRUE)
    rxImport(inData = myData1, outFile = outFile, stringsAsFactors = TRUE,
      colInfo = list(
        x = list(type = "factor", levels = labels5),
        y = list(type = "factor", levels = labels5)
        )
      )
  varInfo1 <- rxGetVarInfo(outFile)
  checkEquals(varInfo1$x$levels, labels5)
    checkEquals(varInfo1$x$levels, labels5)

  rxImport(inData = myData2, outFile = outFile, append = "rows", stringsAsFactors = TRUE,
        colInfo = list(
        x = list(type = "factor", levels = labels5),
        y = list(type = "factor", levels = labels5)
        )
      )
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(varInfo2$x$levels, labels5)
    checkEquals(varInfo2$y$levels, labels5)

    file.remove(outFile)
}


"test.rxImport.data.extra.lf" <- function()
{
  textFile <- tempfile(pattern = "extralf", fileext = ".csv")
    textFile1 <- tempfile(pattern = "extralf1a", fileext = ".csv")
  writeLines(c("a,b,c","1,2,3","4,5,6", "7,8,9"), textFile)
    print("After writing first file.")
  myData1 <- rxImport(textFile)
    print("After first import.")
  myData2 <- rxImport(textFile, type = "text", firstRowIsColNames = TRUE)
    print("After second import.")
  myData3 <- read.table(textFile, sep=",", header = TRUE)
    print ("After read.table.")
  checkEquals(myData1, myData2)
  checkEquals(myData2, myData3)
  file.remove(textFile)

    # Line feed in middle of file
  writeLines(c("a,b,c","1,2,3","", "4,5,6", "7,8,9", "" ), textFile1)
    print("After writing second file.")
  myData1 <- rxImport(textFile1)
    print ("After third import.")
  # myData2 <- rxImport(textFile, type = "text") # Doesn't work with StatTran
  myData3 <- read.table(textFile1, sep=",", header = TRUE)

  checkEquals(myData1, myData3)
  file.remove(textFile1)
}

"test.rxImport.utf8" <- function()
{
  # Import into data frame
  inFile <- file.path(rxGetOption("unitTestDataDir"), "testUtf8.txt")
  myDataImport <- rxImport(inData=inFile, useFastRead=FALSE, firstRowIsColNames=TRUE)

  asianMoonChar <- "\u6708" # unicode asian moon character (creates a UTF-8 character)
  checkEquals("UTF-8", Encoding(myDataImport[1,1])) # confirm it is UTF-8
  checkEquals(asianMoonChar, myDataImport[1,1]) # and has correct contents
  checkEquals("UTF-8", Encoding(colnames(myDataImport))) # do the same for column name
  checkEquals(asianMoonChar, colnames(myDataImport))

  # Import into xdf file
  outFile <-  file.path(tempdir(),".rxTempImport.xdf")
  if (file.exists(outFile)) file.remove(outFile)

  rxImport(inData=inFile, outFile=outFile, useFastRead=FALSE, firstRowIsColNames=TRUE)

  myDataXdf <- rxDataStep(outFile)
  checkIdentical(myDataXdf[1], myDataImport[1])

  file.remove( outFile )
}

"test.rxImport.gb18030" <- function()
{
  # Import into data frame
  inFile <- file.path(rxGetOption("unitTestDataDir"), "testGb18030.txt")
  myDataImport <- rxImport(inData=inFile, useFastRead=FALSE, firstRowIsColNames=TRUE, inputEncoding="gb18030")

  asianMoonChar <- "\u6708" # unicode asian moon character (creates a UTF-8 character)
  checkEquals("UTF-8", Encoding(myDataImport[1,1])) # confirm it is UTF-8
  checkEquals(asianMoonChar, myDataImport[1,1]) # and has correct contents
  checkEquals("UTF-8", Encoding(colnames(myDataImport))) # do the same for column name
  checkEquals(asianMoonChar, colnames(myDataImport))

  # Import into xdf file
  on.exit(file.remove(outFile))
  outFile <-  file.path(tempdir(),".rxTempImport.xdf")
  if (file.exists(outFile)) file.remove(outFile)

  rxImport(inData=inFile, outFile=outFile, useFastRead=FALSE, firstRowIsColNames=TRUE, inputEncoding="gb18030")

  myDataXdf <- rxDataStep(outFile)
  checkIdentical(myDataXdf[1], myDataImport[1])
}


"test.rxImport.gb18030.against.utf8" <- function()
{
  # rxImport: import gb18030 data with conversion
  # read.csv2: read utf8 data
  inFile <- file.path(rxGetOption("unitTestDataDir"), "gb18030.txt")
  inData <- rxImport(inData=inFile, useFastRead=FALSE, firstRowIsColNames=FALSE,
                     inputEncoding="gb18030")

  cmpFile <-  file.path(rxGetOption("unitTestDataDir"), "gb18030.as-utf8.txt")
  cmpData <- read.csv2(cmpFile, sep = "\t", header = FALSE, stringsAsFactors = FALSE, encoding = "UTF-8")

  # imported and converted needs to be the same as pre-converted and read
  checkEquals(inData, cmpData)
}

"test.rxImport.transformFunc.negative" <- function(){
  
  xform <- function(dataList)
  {
    ret = data.frame(dataList)
    ret = ret[ret$age < 40,,drop=FALSE]
    ret    
  }
  
  infert_mod = tryCatch({
    rxImport(inData=infert, transformFunc=xform, transformVars="age")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxImport"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxImport"))
    }
  })
  
}

"test.rxImport.transformFunc.transformVars" <- function(){
  sampleDataDir <- rxGetOption("sampleDataDir")
  inputFile <- file.path(sampleDataDir, "claims.xdf")
  
  claims_df <- rxDataStep(inputFile)
  
  cutoff <- 300
  tf <- function(dataList){
    data = dataList
    data$high <- data$cost > limit
    data
  }
  
  import_result <- rxImport(inData=inputFile, transformFunc = tf, transformVars = c("cost"), 
                            transformObjects = list(limit=cutoff))
  claims_df$high <- claims_df$cost > cutoff

  checkIdentical(import_result, claims_df)  
}
