# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxKmeans
#
############################################################################################

.randomRowSample <- function(data, numRows, varsToKeep = NULL, seed = 100)
{
  if (!is.null(varsToKeep))
  {
    if (inherits(varsToKeep, "formula")) varsToKeep <- attr(terms(varsToKeep), "term.labels")
    if (is.data.frame(data)) data <- data[varsToKeep]
  }
  
  if (!is.data.frame(data))
  {
    data <- rxReadXdf(data, varsToKeep = varsToKeep, numRows = 5L * numRows, returnDataFrame = TRUE)
    data <- data[which(apply(data, MARGIN = 1, function(x){!any(is.na(x))})),]		
  }
  
    set.seed(17)
  # this ensures that the row indices are unique
  irow <- unique(sample.int(nrow(data), numRows * 2, replace = TRUE))[seq(numRows)]
  data[irow,, drop = FALSE]
}

"test.rxKmeans.args.algorithm" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width
  centers <- .randomRowSample(iris, 4L, form)
    res <- try(rxKmeans(form, data = iris, centers = centers, algorithm = "lLoyD"))
  checkTrue(!is(res, "try-error"))
    checkEquals(res$params$Algorithm, "lloyd")
   
    # check for unsupported type
    checkException(rxKmeans(form, data = iris, numClusters = 4L, algorithm = "unsupported"), 
        silent = TRUE)
}

"test.rxKmeans.args.blocksPerRead" <- function()
{

    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  form <- ~ xnum1 + xnum2 + xnum2miss
  centers <- .randomRowSample(test100r5b, 3, form)
  zKmeansBlocksDefault <- rxKmeans(form, data = test100r5b, centers = centers, blocksPerRead = -1)
    
  zKmeansBlocks1 <- rxKmeans(form, data = test100r5b, centers = centers, blocksPerRead = 1)
    zKmeansBlocks1$totss
    zKmeansBlocks1$betweenss
    
  nocomp <- which(names(zKmeansBlocks1) %in% c("call","params"))
  checkEquals(zKmeansBlocksDefault[-nocomp], zKmeansBlocks1[-nocomp], check.attributes = FALSE)
    
    zKmeansBlocks2 <- rxKmeans(form, data = test100r5b, centers = centers, blocksPerRead = 2)
  checkEquals(zKmeansBlocksDefault[-nocomp], zKmeansBlocks2[-nocomp], check.attributes = FALSE)  
     
  zKmeansBlocks5 <- rxKmeans(form, data = test100r5b, centers = centers, blocksPerRead = 5)
  checkEquals(zKmeansBlocksDefault[-nocomp], zKmeansBlocks5[-nocomp], check.attributes = FALSE) 
    zKmeansBlocks5$totss
    zKmeansBlocks5$betweenss
    
    testDF <- rxDataStep(test100r5b)
    zKmeansDF <- rxKmeans(form, data = testDF, centers = centers)
    nocomp1 <- which(names(zKmeansBlocks1) %in% c("call","params", "cluster"))
  checkEquals(zKmeansBlocksDefault[-nocomp1], zKmeansDF[-nocomp1], check.attributes = FALSE)  
    
    # Check blocksPerRead when writing to an existing file
    data.df <- iris
    data.xdf <- tempfile(pattern = "iris", fileext = ".xdf") 

    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
        rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
        rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
    ## create outFile

    outFile <- tempfile(pattern = "rxKmeans",  fileext = ".xdf")
    if (file.exists(outFile)) file.remove(outFile) 
    rxDataStep( data.xdf, outFile)

    ## set up arguments
    form <- ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
    
    # These should give warnings and internally set blocksPerRead to 1
    iris.kmeans1 <- rxKmeans(form, data = data.xdf, outFile = outFile, 
      numClusters  = 3, blocksPerRead = 2, seed = 4
      )
    iris.kmeans2 <- rxKmeans(form, data = data.xdf, outFile = data.xdf, 
      numClusters  = 3, blocksPerRead = 2, seed = 4
      )   

    outFile.df <- rxDataStep(outFile)
    inFile.df <- rxDataStep(data.xdf) 
    checkEquals(outFile.df, inFile.df)    
    file.remove(outFile)
    file.remove(data.xdf)
      
}

"test.rxKmeans.args.centers" <- function()
{	
    # check mismatch of number of columns in 'centers' matrix 
    # with the number of terms in the formula
    set.seed(100)
    DF <- data.frame(x=rnorm(100), y=rnorm(100), z=rnorm(100))
    checkException(rxKmeans(~ (x + y + z)^3, data = DF, centers = matrix(rnorm(15), ncol=3)),
        silent = TRUE)
       
    # wrong type
  centers <- .randomRowSample(DF, 2)
    checkException(rxKmeans(~ x + y + z, data = DF, centers = as.list(centers)), silent = TRUE)
    
    # compare matrix to data frame input
    z1 <- rxKmeans(~ x + y + z, data = DF, centers = centers)
    z2 <- rxKmeans(~ x + y + z, data = DF, centers = as.matrix(centers))
  nocomp <- which(names(z1) %in% c("call","params"))
  checkEquals(z1[-nocomp], z2[-nocomp], check.attributes=FALSE)
    
    # compare to native k-means 
    zNative <- stats::kmeans(as.matrix(DF), centers = as.matrix(centers), 
        iter.max = 10, algorithm = "Lloyd", nstart = 1)
    zRevo <- rxKmeans(~ x + y + z, data = DF, centers = centers, 
        maxIterations = 10, algorithm = "Lloyd", numStarts = 1)
    comps <- c("cluster", "centers",  "withinss", "size")
    checkEquals(zNative[comps], zRevo[comps], check.attributes = FALSE)
} 

"test.rxKmeans.args.data" <- function()
{
  form <- ~ height + weight
  DF <- women
  XDF <-  tempfile( pattern = ".rxKmeans1", fileext = ".xdf")
    DS <- RxXdfData(XDF)
  rxDataStep(DF, XDF)
  centers <- .randomRowSample(DF, 2L, form)

  zKmeansDF <- rxKmeans(form, data = DF, centers = centers)
  zKmeansXDF <- rxKmeans(form, data = XDF, centers = centers)
    zKmeansDS <- rxKmeans(form, data = DS, centers = centers)
  nocomp <- which(names(zKmeansXDF) %in% c("call","params","cluster"))
  checkEquals(zKmeansDF[-nocomp], zKmeansXDF[-nocomp], check.attributes=FALSE)
  checkEquals(zKmeansDF[-nocomp], zKmeansDS[-nocomp], check.attributes=FALSE)
    
    if (file.exists(XDF)) file.remove(XDF)
}

"test.rxKmeans.args.formula" <- function()
{
    # check for unsupported dependent variable specification
    checkException(rxKmeans(Sepal.Length ~ Sepal.Width, data = iris, numClusters = 2), silent = TRUE)
    
    # check for unsupported stripped intercept
    checkException(rxKmeans(~ Sepal.Length - 1, data = iris, numClusters = 2), silent = TRUE)
    
    # check for unsupported dot expansion
    checkException(rxKmeans(~., data = iris, numClusters = 2), silent = TRUE)
    
    # check for unsupported mixed categorical and continuous variables in formula
    checkException(rxKmeans(~ Plant:Type + conc + uptake, data = CO2, numClusters = 2))
    
    # check for unsupported categorical only
    checkException(rxKmeans(~ Plant:Type:Treatment, data = CO2, numClusters = 2))

    # check for continuous only
    #cluster <- rxKmeans(~ conc + uptake, data = CO2, numClusters = 2)
}

"test.rxKmeans.args.maxIterations" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width
  centers <- .randomRowSample(iris, 2, form)
    
    # test illegal values
    checkException(rxKmeans(form, data = iris, maxIterations = 0L, centers = centers), silent = TRUE)
    checkException(rxKmeans(form, data = iris, maxIterations = -1L, centers = centers), silent = TRUE)
  
  # test limit
  z <- rxKmeans(form, data = iris, centers = centers)
  iterConverged <- z[["numIterations"]]
  maxIterations <- iterConverged - 2L
  z <- rxKmeans(form, maxIterations = maxIterations, data = iris, centers = centers)
  checkEquals(z[["numIterations"]], maxIterations, check.attributes = FALSE)
}

"test.rxKmeans.args.numClusters" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width
    
    # test illegal values
    checkException(rxKmeans(form, data = iris, numClusters = 0L), silent = TRUE)
    checkException(rxKmeans(form, data = iris, numClusters = -1L), silent = TRUE)
    
    # cannot specify numClusters and centers simultaneously
    checkException(rxKmeans(form, data = iris, numClusters = 2L, 
        centers=matrix(rnorm(4L), ncol = 2L)), silent = TRUE)
}

"test.rxKmeans.args.outFile.composite" <- function()
{ 
    
    centers <- matrix(c(220, 700), ncol = 1)
    xdfFile <- file.path(rxGetOption("sampleDataDir"),"claims.xdf")
    outFile1 <- tempfile(pattern = "rxKmeans1",  fileext = ".xdf")
    kmeansXdf <- rxKmeans(~cost, data=xdfFile, #numClusters = 2, numStarts = 4L, seed = 4,
        centers = centers,
        outFile = outFile1, writeModelVars = TRUE)
    
  
    xdfComposite <- file.path(rxGetOption("sampleDataDir"), "claimsXdf")
    compositeOut <- tempfile(pattern = "rxKmeansDir", fileext = "") 
    compositeOutDS <- RxXdfData(compositeOut, createCompositeSet = TRUE, blocksPerCompositeFile = 4L)
    kmeansComposite <- rxKmeans(~cost, data = xdfComposite, centers = centers, 
         writeModelVars = TRUE, outFile = compositeOutDS)
    getInfoOut <- rxGetInfo(compositeOutDS)
    getInfoIn <- rxGetInfo(xdfComposite)
    
    if (!rxIsExpressEdition())
    {
        checkEquals(getInfoOut$numCompositeFiles, getInfoIn$numCompositeFiles)
    }
    
    checkEquals(getInfoOut$numRows, getInfoIn$numRows)
    
    df1 <- rxDataStep(outFile1)
    df2 <- rxDataStep(compositeOutDS)
    checkEquals(df1$cost, df2$cost)
    checkEquals(df1$.rxCluster , df2$.rxCluster)
      
    unlink(compositeOut, recursive = TRUE)
    file.remove(outFile1)

    ## specify the number of clusters to create instead of the centers
    seed <- 123
    numClusters <- nrow(centers)
    kmeansComposite2 <- rxKmeans( ~ cost, data = xdfComposite,
         numClusters = numClusters, seed = seed)

    checkTrue(TRUE)
}


"test.rxKmeans.args.numStarts" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width
  numClusters <- 2L

    # test illegal values
    checkException(rxKmeans(form, data = iris, numClusters = numClusters, numStarts = -1L), silent = TRUE)    
  
  # test legal values
    # With the same seed, numStarts at 5 should be at least as good as numStarts at 2
    resStart1 <- rxKmeans(form, data = iris, numClusters = numClusters, numStarts = 1L, seed = 4)
    resStart5 <- rxKmeans(form, data = iris, numClusters = numClusters, numStarts = 5L, seed = 4)

    checkEquals(resStart1[["valid.obs"]],resStart5[["valid.obs"]])
    checkTrue(resStart1$tot.withinss >= resStart5$tot.withinss)
          
    # Check to make sure we are choosing the best of the estimated models    
    xdfFile <- file.path(rxGetOption("sampleDataDir"),"claims.xdf")
    out4 <- capture.output(
        resStart4 <- rxKmeans(~cost, data=xdfFile, numClusters = 2, numStarts = 4,
             verbose = 2)
        )
    # Extract all strings for Within cluster sum of Squares
    hasCluster1 <- grepl("Within cluster sum of squares\\[1\\]", out4)
    hasCluster2 <- grepl("Within cluster sum of squares\\[2\\]", out4)
    outSumOfSquares1 <- out4[hasCluster1]
    outSumOfSquares2 <- out4[hasCluster2]
    sumOfSquares1 <- as.double(substring(outSumOfSquares1, first = 34))
    sumOfSquares2 <- as.double(substring(outSumOfSquares2, first = 34))
    sumOfSquares = sumOfSquares1 + sumOfSquares2
    numVals <- length(sumOfSquares)
    checkEquals(min(sumOfSquares[1:(numVals-1)]), sumOfSquares[numVals])
    checkEquals(sumOfSquares[numVals], as.numeric(resStart4["tot.withinss"]))
    sumOut <- rxResultsDF(rxSummary(~cost, data=xdfFile))
    checkEquals(as.numeric(resStart4$missing.obs), as.numeric(sumOut$MissingObs))
    checkEquals(as.numeric(resStart4$valid.obs), as.numeric(sumOut$ValidObs))
   
}

"test.rxKmeans.args.outColName" <- function()
{
    # illegal length
    form <- ~ Sepal.Length + Sepal.Width
  centers <- .randomRowSample(iris, 2L, form)

  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  rxDataStep(iris, inXDF, overwrite = TRUE)
  
    # test illegal values
    checkException(rxKmeans(form, outFile = outXDF, outColName = c("x", "y"),
        data = iris, numClusters = 2L), silent = TRUE)
    checkException(rxKmeans(form, outFile = outXDF, outColName = "",
        data = iris, numClusters = 2L), silent = TRUE)
        
  ###
    # check ability to write to a file with a specified outColName
  ###

  outColName <- "myClusterIndices"

  # same I/O file
    
  zSameIO <- rxKmeans(form, data = inXDF, outFile = inXDF, centers = centers, outColName = outColName)
  checkTrue(file.exists(inXDF))
  zSameIOCluster <- rxReadXdf(inXDF, varsToKeep = outColName)

  # different I/O file
  zDiffIO <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, outColName = outColName)
  checkTrue(file.exists(outXDF))
  zDiffIOCluster <- rxReadXdf(outXDF, varsToKeep = outColName)
  checkEquals(zSameIOCluster, zDiffIOCluster)

  # this should create a new ".rxCluster" column in output 
  # and preserve the original column with different name
  zDiffIO2 <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers)
  checkTrue(file.exists(outXDF))
  zDiffIO2Cluster <- rxReadXdf(outXDF, varsToKeep = c(".rxCluster", outColName))
  checkEquals(zDiffIO2Cluster[[".rxCluster"]], zDiffIO2Cluster[[outColName]])	
  
  # check legal yet mangled output column name
  zDiffIO3 <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, outColName = "cat and dog")
  checkTrue(file.exists(outXDF))
  zDiffIO3Cluster <- rxReadXdf(outXDF, varsToKeep = c(".rxCluster", "cat.and.dog"))
  checkEquals(zDiffIO3Cluster[[".rxCluster"]], zDiffIO3Cluster[["cat.and.dog"]])
    
    # check existing output column name with overwrite=FALSE and overwrite=TRUE
  if (file.exists(outXDF)) file.remove(outXDF)	
  zDiffIO4 <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, outColName = outColName)
    zDiffIO4Indices <- rxReadXdf(outXDF)
    checkException(rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, outColName = outColName, 
        overwrite = FALSE), silent = TRUE)
  zDiffIO4Overwrite <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, outColName = outColName, 
        overwrite = TRUE)
    zDiffIO4OverwriteIndices <- rxReadXdf(outXDF)
    checkEquals(zDiffIO4OverwriteIndices, zDiffIO4Indices)
    checkEquals(zDiffIO4Indices, zDiffIO4OverwriteIndices)

    # check existing output column name but has different storage type
  if (file.exists(outXDF)) file.remove(outXDF)
    DF2 <- data.frame(letters)
    names(DF2) <- outColName
  rxDataStep(DF2, outXDF) 
    checkException(rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, outColName = outColName, 
        overwrite = TRUE), silent = TRUE)
        
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)	
}

"test.rxKmeans.args.outFile" <- function()
{
    # illegal length
    form <- ~ Sepal.Length + Sepal.Width
  centers <- .randomRowSample(iris, 2L, form)
    
    # test illegal values
    checkException(rxKmeans(form, outFile = "", data = iris, numClusters = 2L), silent = TRUE)
    checkException(rxKmeans(form, outFile = c("temp1","temp2"), data = iris, numClusters = 2L), silent = TRUE)
    checkException(rxKmeans(form, outFile = NA, data = iris, numClusters = 2L), silent = TRUE)
    
  ###
    # check ability to write to a file
  ###
  
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  rxDataStep(iris, inXDF, overwrite = TRUE)
  
  
  # same I/O file
  zSameIO <- rxKmeans(form, data = inXDF, outFile = inXDF, centers = centers)
  checkTrue(file.exists(inXDF))
  zSameIOCluster <- rxReadXdf(inXDF, varsToKeep = ".rxCluster")

  # different I/O file
  zDiffIO <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers)
  checkTrue(file.exists(outXDF))
  zDiffIOCluster <- rxReadXdf(outXDF, varsToKeep = ".rxCluster")
  checkEquals(zSameIOCluster, zDiffIOCluster)
    
    # different I/O xdf data source
    if (file.exists(outXDF)) file.remove(outXDF)
    outDS <- RxXdfData(outXDF)
  zDiffIO <- rxKmeans(form, data = inXDF, outFile = outDS, centers = centers)
  checkTrue(file.exists(outXDF))
  zDiffIOCluster2 <- rxReadXdf(outXDF, varsToKeep = ".rxCluster")
  checkEquals(zSameIOCluster, zDiffIOCluster2)    
  
    # Reverse formula and use centers matrix
    form <- ~  Sepal.Width + Sepal.Length
  if (file.exists(outXDF)) file.remove(outXDF)
  z1 <- rxKmeans(form, data = iris, centers=matrix(c(3,4,6,5), nrow=2),
      reportProgress=0)
  z2 <- rxKmeans(form, data = inXDF, outFile = outXDF, 
    outColName = "Cluster Indices", 
        centers=matrix(c(3,4,6,5), nrow=2),
      reportProgress=0)	
  
  z3 <- rxReadXdf(outXDF)
  
  checkEqualsNumeric(z3$Cluster.Indices, z1$cluster)
    
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)	

} 
"test.rxKmeans.args.writeModelVars" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width
  centers <- .randomRowSample(iris, 2L, form)
    
  ###
    # check ability to write centers to a multi-block file
  ###
  
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp.xdf")
    
    numCoresOld <- rxGetOption("numCoresToUse")
  
    testIO <- function( numCoresToUse = -1)
    {
        rxOptions(numCoresToUse = numCoresToUse)
      if (file.exists(inXDF)) file.remove(inXDF)
      if (file.exists(outXDF)) file.remove(outXDF)
        
        if (rxIsExpressEdition())
        {
            rxDataStep(inData = iris, outFile = inXDF, overwrite = TRUE)
        } else
        {
          rxDataStep(iris[1:50,], inXDF, overwrite = TRUE)
          rxDataStep(iris[51:100,], inXDF, append="rows")
          rxDataStep(iris[101:150,], inXDF, append="rows") 
        }
               
      # same I/O file
      zSameIO <- rxKmeans(form, data = inXDF, outFile = inXDF,
        centers = centers, overwrite=TRUE)
      
      # different I/O file
      zDiffIO <- rxKmeans(form, data = inXDF, outFile = outXDF, 
        centers = centers, writeModelVars=TRUE, overwrite=TRUE)
      checkTrue(file.exists(outXDF))
      varInfo <- rxGetVarInfo(outXDF)
      varNames <- c(".rxCluster", "Sepal.Length", "Sepal.Width")
      checkEquals( names(varInfo), varNames)
      zSameIOdf <- rxReadXdf( inXDF, varsToKeep = varNames)
      zDiffIOdf <- rxReadXdf( outXDF, varsToKeep = varNames)
      
      checkEquals(zSameIOdf, zDiffIOdf)
    }
    testIO(numCoresToUse = 1)
    testIO(numCoresToUse = 2)
    testIO(numCoresToUse = 4)
    
     rxOptions(numCoresToUse = numCoresOld)
  
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)	
} 

"test.rxKmeans.args.exraVarsToWrite" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width
  centers <- .randomRowSample(iris, 2L, form)

  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp.xdf")
    
    numCoresOld <- rxGetOption("numCoresToUse")
    
  if (file.exists(inXDF)) file.remove(inXDF)
    rxDataStep(inData = iris[1:50,], outFile = inXDF, overwrite = TRUE)
    rxDataStep(inData = iris[51:100,], outFile = inXDF, append="rows")
    rxDataStep(inData = iris[101:150,], outFile = inXDF, append="rows")    
  
    testIO <- function( numCoresToUse = -1)
    {
        rxOptions(numCoresToUse = numCoresToUse)
      
        if (file.exists(outXDF)) file.remove(outXDF)   
      # different I/O file
      kmeansOut1 <- rxKmeans(form, data = inXDF, outFile = outXDF, 
        centers = centers, writeModelVars = TRUE, overwrite=TRUE)
        varInfoBase <- rxGetVarInfo(outXDF)
        file.remove(outXDF)
        
        kmeansOut2 <- rxKmeans(form, data = inXDF, outFile = outXDF, 
        centers = centers, writeModelVars = TRUE, 
            extraVarsToWrite = "Petal.Length", overwrite=TRUE)
        varInfoExtra = rxGetVarInfo(outXDF)
        
        checkEquals(varInfoBase$.rxCluster, varInfoExtra$.rxCluster)
        checkEquals(varInfoBase$Sepal.Length, varInfoExtra$Sepal.Length)
        checkEquals(length(varInfoBase)+1, length(varInfoExtra))
        
        file.remove(outXDF)
        kmeansOut3 <- rxKmeans(form, data = inXDF, outFile = outXDF, 
        centers = centers, writeModelVars = FALSE, 
            extraVarsToWrite = "Petal.Length", overwrite=TRUE)
        varInfoExtraNoModel = rxGetVarInfo(outXDF)
        
        checkEquals(varInfoBase$.rxCluster, varInfoExtraNoModel$.rxCluster)
        checkEquals(varInfoExtra$Petal.Length, varInfoExtraNoModel$Petal.Length)
        checkEquals(length(varInfoExtraNoModel), 2)        
    }
    testIO(numCoresToUse = 1)
    testIO(numCoresToUse = 2)
    testIO(numCoresToUse = 4)
    
    rxOptions(numCoresToUse = numCoresOld)
  
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)	
} 


"test.rxKmeans.args.reportProgress" <- function()
{
  # reportProgress argument stress tests
  form <- ~ conc + uptake
  DF <- CO2
  centers <- .randomRowSample(DF, 2L, form)
  
  #reportProgress: logical, character string or integer: 
  #  0: no progress is reported. 
  #  1: the number of processed rows is printed and updated. 
  #  2: rows processed and timings are reported. 
  #  3: rows processed and all timings are reported. 

  variations <- list(0,1,2,3)	
  results <- printOutput <- vector("list", length=length(variations))
  
  for (i in seq_along(variations))
  {
    printOutput[[i]] <- capture.output(results[[i]] <- 
            rxKmeans(form = form, data = DF, centers = centers, 
                reportProgress = variations[[i]], verbose = 0L))
  }

  # check that all values are the same
  nocomp <- which(names(results[[1L]]) %in% c("call","params"))
  checkTrue(all(unlist(lapply(results[-1L], 
    function(x,y,nocomp){all.equal(x[-nocomp], y[-nocomp], check.attributes=FALSE)}, 
    y=results[[1L]], nocomp=nocomp))))

  names(printOutput) <- as.character(variations)
  noResponseExpected <- c("0")
  equalLengthExpected <- c("2")
  equalLengthExpected1 <- c("1")
  
  checkTrue(length(unlist(printOutput[noResponseExpected])) == 0L)
  
  # just rows
  #checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1],length)) == 1L))
  
  # all these outputs should be the same
  #assign("printOutput", printOutput, .GlobalEnv) 
  checkTrue(length(printOutput[equalLengthExpected]) > 0L)
  
  # details gives us the goods
  checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=84, TotalRowsProcessed=84, ReadTime", 
    printOutput[["3"]][1])) > 0L)
  
  # time
  checkTrue(length(grep("^Rows Read:", printOutput[["2"]][2L])) > 0L)
}

"test.rxKmeans.args.rowSelection" <- function()
{
  # initialization
    set.seed(100)
    DF <- data.frame(x = rnorm(20), y = rnorm(20))
  DF$selectVar <- DF$y > 0.0
    rowSelection <- expression(y > 0.0)
    form <- ~ x + y
  centers <- .randomRowSample(DF, 2L, form)
  
  # test various schemes for row selection and compare to results
  selectRowsObj1 <- rxKmeans(form, data = DF, centers = centers, rowSelection = rowSelection, algorithm = "Lloyd")
  selectRowsObj2 <- rxKmeans(form, data = DF, centers = centers, rowSelection = selectVar, algorithm = "Lloyd")
  selectRowsObj3 <- rxKmeans(form, data = DF, centers = centers, rowSelection = y > 0.0, algorithm = "Lloyd")
  DFcut <- subset(DF[c("x", "y")], y > 0)
  kmeansObj <- kmeans(DFcut, centers, algorithm = "Lloyd")
    checkEquals(kmeansObj[["centers"]], selectRowsObj1[["centers"]])
    nocomp <- which(names(selectRowsObj1) %in% c("call","params"))
  checkEquals(selectRowsObj1[-nocomp], selectRowsObj2[-nocomp], check.attributes=FALSE)
  checkEquals(selectRowsObj2[-nocomp], selectRowsObj3[-nocomp], check.attributes=FALSE)
}

"test.rxKmeans.args.rowSelection.outFile" <- function()
{
    # initialization
    form <- ~ Sepal.Length + Sepal.Width
    rowSelection <- expression(Sepal.Length > 5)
  centers <- .randomRowSample(iris, 2L, form)
  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF <- file.path(tempdir(), ".rxOutFileTemp.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
  rxDataStep(iris, inXDF, overwrite = TRUE)
  
    # cannot have rowSelection with outFile the same as the data source	
  checkException(rxKmeans(form, data = inXDF, outFile = inXDF, centers = centers, 
        rowSelection = rowSelection, algorithm = "Lloyd"), silent = TRUE)
    
    # Can output to new data file
  outKmeans <- rxKmeans(form, data = inXDF, outFile = outXDF, centers = centers, 
        rowSelection = rowSelection, algorithm = "Lloyd")
    clusterIDs <- rxReadXdf(outXDF)
    irisCut <- subset(iris, Sepal.Length > 5)
    checkEquals(names(clusterIDs), ".rxCluster")
    checkEquals(NROW(clusterIDs), NROW(irisCut))
    
    # Can't write to an existing outFile with rowSelection
  checkException(
        rxKmeans(form, data = inXDF, outFile = outXDF, outColName = ".rxCluster2",
            centers = centers, 
            rowSelection = rowSelection, algorithm = "Lloyd")
       )
   
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
}

"test.rxKmeans.args.transformFunc" <- function()
{
    xform <- function(x)
    {
      x$area <- x$Sepal.Length * x$Sepal.Width
      x
    }

    res <- try(rxKmeans(~ Petal.Length + area, data = iris, transformFunc = xform, 
      transformVars=c("Sepal.Length","Sepal.Width"), numClusters = 2L))
    
    # return for no errors encountered during execution
  checkTrue(!is(res, "try-error"))
  
    # TODO: need to test some values here
}

"test.rxKmeans.args.transforms" <- function()
{
    # Create local data source
    # Create new transform variables (copies of original).
    # Deleted variables: time, conc
    transforms <- expression(list(
        newTime = time,
        time = NULL,
        newConc = conc,
        conc = NULL))
  
  # set common parameters
  seed <- 100
  maxIterations <- 1000L
  numClusters <- 3L
  algorithm <- "Lloyd"
  DF <- Indometh

  z1 <- rxKmeans(~ newTime + newConc, data = DF, numClusters = numClusters, seed = seed, 
        transforms = transforms, algorithm = algorithm, maxIterations = maxIterations)
  z2 <- rxKmeans(~ time + conc, data = DF, numClusters = numClusters, seed = seed,
        algorithm = algorithm, maxIterations = maxIterations)

  nocomp <- which(names(z1) %in% c("call","params","formula"))
  checkEquals(z1[-nocomp], z2[-nocomp], check.attributes=FALSE)
}

"test.rxKmeans.args.transformVars" <- function()
{
    # Create local data source
    # Create transforms variables.
    # New variables: HalfTime, concDouble
    # Deleted variables: time, conc
    xform <- 
    function(dataList)
    {
        dataList$HalfTime <- dataList$time / 2
        dataList$time <- NULL
        dataList$concDouble <- dataList$conc * 2
        dataList$conc <- NULL
        dataList
    }

    DF <- Indometh[c("time", "conc")]
    form <- ~ HalfTime + concDouble
    centers <- DF[c(1,4),]
  algorithm <- "Lloyd"
  maxIterations <- 1000L

    kmeansRevoXform <- rxKmeans(form, 
      data = DF, centers = centers,
        transformFunc = xform, transformVars = c("time", "conc"), 
        algorithm = algorithm, maxIterations = maxIterations)
      
    xformDF <-  data.frame(xform(DF))
    kmeansRevo <- rxKmeans(form, data = xformDF, centers = centers, 
      algorithm = algorithm, maxIterations = maxIterations)
  
  kmeansR <- stats::kmeans(xformDF, centers = centers, algorithm = algorithm,
        iter.max = maxIterations)
   
    comps <- c("cluster", "centers",  "withinss", "size")
    checkEquals(kmeansRevoXform[comps], kmeansRevo[comps], check.attributes = FALSE)
    checkEquals(kmeansRevo[comps], kmeansR[comps], check.attributes = FALSE)
} 

"test.rxKmeans.args.verbose" <- function()
{
    form <- ~ Sepal.Width + Petal.Length + Petal.Width 
  centers <- iris[1:3, c("Sepal.Width", "Petal.Length", "Petal.Width")]
    o1 <- capture.output(rxKmeans.test1 <- rxKmeans(form, data=iris, centers = centers, reportProgress = 0L))
    o2 <- capture.output(rxKmeans.test2 <- rxKmeans(form, data=iris, centers = centers, verbose = 0L, reportProgress = 0L))
    o3 <- capture.output(rxKmeans.test3 <- rxKmeans(form, data=iris, centers = centers, verbose = 1L, reportProgress = 0L))

    checkEquals(length(o1), 0L)
    checkEquals(length(o2), 0L)
    checkTrue(length(o3) > 0L) 
}

"test.rxKmeans.data.compressedXdf" <- function()
{
    # Note that random sampling is different for compressed xdf files
    # Need to test on a data set with more than 5000 rows
    AirlineDemoSmall <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
    AirlineDemoSmallUC <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmallUC.xdf")
  kmeansUC <- rxKmeans( ~ CRSDepTime, AirlineDemoSmallUC, numClusters = 5, seed = 42, reportProgress = 0 )
    kmeansC <- rxKmeans( ~ CRSDepTime, AirlineDemoSmall, numClusters = 5, seed = 42, reportProgress = 0 )
    checkEquals(sort(kmeansUC$centers), sort(kmeansC$centers))
    checkEquals(sort(kmeansUC$size), sort(kmeansC$size), checkNames = FALSE)
    
}

"test.rxKmeans.data.compositeXdf" <- function()
{
    # Create composite xdf file
    airColInfo <- list(DayOfWeek = list(type = "factor", 
        levels = c("Monday", "Tuesday", "Wednesday", "Thursday", 
            "Friday", "Saturday", "Sunday")))
    
    airDemoTextLocalDS <-  RxTextData(file = file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.csv"),
        colInfo = airColInfo,
        rowsPerRead = 200000,
        missingValueString = "M", 
        fileSystem = "native")    

   airCompositePath <- file.path(tempdir(), "AirlineDemoSmallComposite")
   airDemoCompositeXdfDS <-  RxXdfData(airCompositePath,
        createCompositeSet = TRUE,
        fileSystem = "native") 
    
   rxImport(airDemoTextLocalDS, outFile = airDemoCompositeXdfDS,
        rowsPerRead = 200000, blocksPerCompositeFile = 1, overwrite = TRUE)           

    # Run kmeans with outFile using composite in and out data sources
    outFile1 <- file.path(tempdir(), "KmeansOut1")
    unlink(outFile1, recursive = TRUE)
    outDS1 <- RxXdfData(outFile1, createCompositeSet = TRUE, blocksPerCompositeFile = 1)
  kmeansCompositeXdf <- rxKmeans( ~ CRSDepTime, airDemoCompositeXdfDS, numClusters = 5, seed = 42, 
        reportProgress = 0, writeModelVars = TRUE, outFile = outDS1 )
    
    AirlineDemoSmall <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")
        
    # Repeat using single xdf files
    outFile2 <- file.path(tempdir(), "KmeansOut2.xdf")
    unlink(outFile2, recursive = TRUE)
    kmeansXdf <- rxKmeans( ~ CRSDepTime, AirlineDemoSmall, numClusters = 5, seed = 42, reportProgress = 0,
         writeModelVars = TRUE, outFile = outFile2)
    
    checkEquals(sort(kmeansXdf$centers), sort(kmeansCompositeXdf$centers))
    checkEquals(sort(kmeansXdf$size), sort(kmeansCompositeXdf$size), checkNames = FALSE)
    
    outInfoComposite <- rxGetInfo(outDS1, getVarInfo = TRUE)
    outInfoXdf <- rxGetInfo(outFile2, getVarInfo = TRUE)
    checkEquals(outInfoComposite$numRows, outInfoXdf$numRows)
    
    # Clean-up
    unlink(outFile1, recursive = TRUE)
    unlink(outFile2, recursive = TRUE)
    unlink(airCompositePath, recursive = TRUE)
    
}

"test.rxKmeans.data.spss" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "richTest100r1b.xdf")
    test100rSpss <- file.path(rxGetOption("unitTestDataDir"), "richTest100r.sav")
  form <- ~ xnum1 + xnum2 + xnum2miss
    centers <- data.frame(
    xnum1 = c( 0, .1, .7 ), 
    xnum2 = c( -0.5, -0.3, -0.4 ), 
    xnum2miss = c(-0.5, -0.3, -0.4))
  zKmeansXdf <- rxKmeans(form, data = test100r5b, centers = centers)
    mySpssData <- RxSpssData(file = test100rSpss)
    
    form <- ~ XNUM1 + XNUM2 + XNUM2MISS
    names(centers) <- toupper(names(centers)) 
    zKmeansSpss <- rxKmeans(form, data = mySpssData, centers = centers)
    
    checkEquals(zKmeansXdf$valid.obs, zKmeansSpss$valid.obs)
    
    # Handle the fact that SPSS variables are upper case
    colnames(zKmeansXdf$centers) <- toupper(colnames(zKmeansXdf$centers))

    checkEquals(zKmeansXdf$centers, zKmeansSpss$centers)
    checkEquals(zKmeansXdf$NumClusters, zKmeansSpss$NumCenters)
    
    # What happens when we try to write clusters to the spss file?
    res <- try(
        zKmeansSpss <- rxKmeans(form, data = mySpssData, outFile = mySpssData, centers = centers)
        )
  checkTrue(is(res, "try-error"))	
    
    # What about writing them to an .xdf file?
    testOutFile <- tempfile(pattern = ".rxKmeans1TempTest", fileext = ".xdf")
    zKmeansSpss <- rxKmeans(form, data = mySpssData, outFile = testOutFile, centers = centers)
    varInfo <- rxGetVarInfo( testOutFile )
    checkEquals(length(varInfo), 1)
    checkEquals(varInfo$.rxCluster$varType, "integer")
    fileInfo <- rxGetInfo( testOutFile )
    checkEquals(fileInfo$numRows, 100)
  
    # Now with writeModelVars  
    zKmeansSpss <- rxKmeans(form, data = mySpssData, outFile = testOutFile, centers = centers, 
        writeModelVars = TRUE, overwrite = TRUE)
    varInfo <- rxGetVarInfo( testOutFile )
    checkEquals(length(varInfo), 4)
    checkEquals(varInfo$.rxCluster$varType, "integer")
    checkEquals(varInfo$XNUM2$varType, "numeric")
    fileInfo <- rxGetInfo( testOutFile )
    checkEquals(fileInfo$numRows, 100)
        
    file.remove(testOutFile)
  
  # Example using claims data
  spssFile <- file.path(rxGetOption("sampleDataDir"),"claims.sav")                
  spssDS <- RxSpssData(spssFile, colClasses = c(cost = "integer"))                        
  resultSpss <- rxKmeans(~cost, data=spssDS, numClusters = 2, numStarts = 0, verbose = 2)	
    
    xdfFile <- file.path(rxGetOption("sampleDataDir"),"claims.xdf")
    resultXdf <- rxKmeans(~cost, data=xdfFile, numClusters = 2, numStarts = 0)	
  
    checkEquals(resultSpss$centers, resultXdf$centers)
    checkEquals(resultSpss$valid.obs, resultXdf$valid.obs)
    checkEquals(resultSpss$missing.obs, resultXdf$missing.obs)
    checkEquals(resultSpss$size, resultXdf$size)
    checkEquals(resultSpss$withinss, resultXdf$withinss)
    checkEquals(resultSpss$betweenss, resultXdf$betweenss)
}

"test.rxKmeans.data.text" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "richTest100r1b.xdf")
    test100rComma <- file.path(rxGetOption("unitTestDataDir"), "richTest100rComma.txt")
  form <- ~ xnum1 + xnum2 + xnum2miss
    centers <- data.frame(
    xnum1 = c( 0, .1, .7 ), 
    xnum2 = c( -0.5, -0.3, -0.4 ), 
    xnum2miss = c(-0.5, -0.3, -0.4))
  zKmeansXdf <- rxKmeans(form, data = test100r5b, centers = centers)
    myTextData <- RxTextData(file = test100rComma)
    
    form <- ~ xnum1 + xnum2 + xnum2miss
    names(centers) <- toupper(names(centers)) 
    zKmeansText <- rxKmeans(form, data = myTextData, centers = centers)
    
    checkEquals(zKmeansXdf$valid.obs, zKmeansText$valid.obs)
    
    # Handle the fact that SPSS variables are upper case
   # colnames(zKmeansXdf$centers) <- toupper(colnames(zKmeansXdf$centers))

    checkEquals(zKmeansXdf$centers, zKmeansText$centers)
    checkEquals(zKmeansXdf$NumClusters, zKmeansText$NumCenters)
    
    # What happens when we try to write clusters to the spss file?
    res <- try(
        zKmeansText <- rxKmeans(form, data = myTextData, outFile = myTextData, centers = centers)
        )
  checkTrue(is(res, "try-error"))	
    
    # What about writing them to an .xdf file?
    testOutFile <- tempfile(pattern = ".rxKmeans1TempTest", fileext = ".xdf")
    zKmeansSpss <- rxKmeans(form, data = myTextData, outFile = testOutFile, centers = centers)
    varInfo <- rxGetVarInfo( testOutFile )
    checkEquals(length(varInfo), 1)
    checkEquals(varInfo$.rxCluster$varType, "integer")
    fileInfo <- rxGetInfo( testOutFile )
    checkEquals(fileInfo$numRows, 100)
  
    # Now with writeModelVars  
    zKmeansText <- rxKmeans(form, data = myTextData, outFile = testOutFile, centers = centers, 
        writeModelVars = TRUE, overwrite = TRUE)
    varInfo <- rxGetVarInfo( testOutFile )
    checkEquals(length(varInfo), 4)
    checkEquals(varInfo$.rxCluster$varType, "integer")
    checkEquals(varInfo$xnum2$varType, "numeric")
    fileInfo <- rxGetInfo( testOutFile )
    checkEquals(fileInfo$numRows, 100)
        
    file.remove(testOutFile)
  
  # Example using claims data
  textFile <- file.path(rxGetOption("sampleDataDir"),"claims.txt")                
  textDS <- RxSpssData(textFile, colClasses = c(cost = "integer"))                        
  resultText <- rxKmeans(~cost, data=textDS, numClusters = 2, numStarts = 0, verbose = 2)	
    
    xdfFile <- file.path(rxGetOption("sampleDataDir"),"claims.xdf")
    resultXdf <- rxKmeans(~cost, data=xdfFile, numClusters = 2, numStarts = 0)	
  
    checkEquals(resultText$centers, resultXdf$centers)
    checkEquals(resultText$valid.obs, resultXdf$valid.obs)
    checkEquals(resultText$missing.obs, resultXdf$missing.obs)
    checkEquals(resultText$size, resultXdf$size)
    checkEquals(resultText$withinss, resultXdf$withinss)
    checkEquals(resultText$betweenss, resultXdf$betweenss)
}

    
"test.rxKmeans.args.numCores" <- function()
{
    xdfFileName <- tempfile( pattern = ".rxKmeans2", fileext = ".xdf")
    form <- ~ Sepal.Width + Petal.Length + Petal.Width 
    myIris <- iris
    myIris$Sepal.Width[2:5] <- NA
    myIris$Petal.Length[75:78] <- NA
    irisDS <- rxDataStep(inData = myIris, outFile = xdfFileName, rowsPerRead = 50) 
  centers <- iris[1:3, c("Sepal.Width", "Petal.Length", "Petal.Width")]
    
    mySum <- rxSummary(form, data=myIris, byTerm = FALSE)
        
    origNumCores <- rxGetOption("numCoresToUse")
    rxOptions(numCoresToUse = 1)
    rxKmeans.test1 <- rxKmeans(form, data=myIris, centers = centers,
        reportProgress = 0)
    
    checkEquals(mySum$nobs.valid, rxKmeans.test1$valid.obs)
    checkEquals(mySum$nobs.missing, rxKmeans.test1$missing.obs)
    withinSS <- rxKmeans.test1$withinss 
    checkEquals(sum(withinSS), rxKmeans.test1$tot.withinss)
    
    rxKmeans.test1xdf <- rxKmeans(form, data=irisDS, centers = centers,
        reportProgress = 0)  
    
    checkEquals(mySum$nobs.valid, rxKmeans.test1xdf$valid.obs)
    checkEquals(mySum$nobs.missing, rxKmeans.test1xdf$missing.obs)
    checkEquals(withinSS, rxKmeans.test1xdf$withinss )               
    
    rxOptions(numCoresToUse = 2)
    rxKmeans.test2 <- rxKmeans(form, data=myIris, centers = centers, 
        reportProgress = 0)
    
    checkEquals(mySum$nobs.valid, rxKmeans.test2$valid.obs)
    checkEquals(mySum$nobs.missing, rxKmeans.test2$missing.obs)
    checkEquals(withinSS, rxKmeans.test2$withinss )     
    
    rxKmeans.test2xdf <- rxKmeans(form, data=irisDS, centers = centers, 
        reportProgress = 0)
    
    checkEquals(mySum$nobs.valid, rxKmeans.test2xdf$valid.obs)
    checkEquals(mySum$nobs.missing, rxKmeans.test2xdf$missing.obs) 
    checkEquals(withinSS, rxKmeans.test2xdf$withinss )        
    
    rxOptions(numCoresToUse = 4)
    rxKmeans.test4 <- rxKmeans(form, data=myIris, centers = centers,
        reportProgress = 0)  
    checkEquals(mySum$nobs.valid, rxKmeans.test4$valid.obs)
    checkEquals(mySum$nobs.missing, rxKmeans.test4$missing.obs) 
    checkEquals(withinSS, rxKmeans.test4$withinss )      
    
    rxKmeans.test4xdf <- rxKmeans(form, data=irisDS, centers = centers,
        reportProgress = 0)  
    checkEquals(mySum$nobs.valid, rxKmeans.test4xdf$valid.obs)
    checkEquals(mySum$nobs.missing, rxKmeans.test4xdf$missing.obs) 
    checkEquals(withinSS, rxKmeans.test4xdf$withinss )             
    
    rxOptions(numCoresToUse = origNumCores) 
    file.remove( xdfFileName )      
    
}

"test.rxKmeans.args.xdfCompressionLevel" <- function()
{
    form <- ~ Sepal.Length + Sepal.Width
  centers <- .randomRowSample(iris, 2L, form)

  inXDF <- file.path(tempdir(), ".rxInFileTemp.xdf")
  outXDF0 <- file.path(tempdir(), ".rxOutFileTemp0.xdf")
  outXDF1 <- file.path(tempdir(), ".rxOutFileTemp1.xdf")
  if (file.exists(inXDF)) file.remove(inXDF)
  if (file.exists(outXDF0)) file.remove(outXDF0)
  if (file.exists(outXDF1)) file.remove(outXDF1)	
  rxDataStep(iris, inXDF, overwrite = TRUE)
  

  outColName <- "myClusterIndices"
  outDS0 <- rxKmeans(form, data = inXDF, outFile = outXDF0, centers = centers, 
    outColName = outColName, xdfCompressionLevel = 0)
  
  outDS1 <- rxKmeans(form, data = inXDF, outFile = outXDF1, centers = centers, 
    outColName = outColName, xdfCompressionLevel = 1)
  
  compressedSize0 <- file.info(outXDF0)$size
  compressedSize1 <- file.info(outXDF1)$size
  checkTrue(compressedSize0 > compressedSize1)	
  
  file.remove(inXDF)
  file.remove(outXDF0)
  file.remove(outXDF1)
}
