# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLinMod
#
############################################################################################

# rUnit should create these objects just before defining and executing the tests in this file,
# then clean them up automatically after all tests in the file are finished.

unitTestDataDir <- rxGetOption("unitTestDataDir")
# Instead use R's tempdir()
toutDir <- tempdir()
testOutputDir <- file.path(toutDir, "testOutput")
sampleDataDir <- rxGetOption("sampleDataDir")
require(MASS)

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

compareSingleDepvarLM <- function(rxLinModOutput, lmOutput)
{
   x <- if (is.matrix(rxLinModOutput)) rxLinModOutput[,1, drop=TRUE] else rxLinModOutput
   all.equal(x[!is.na(x)], lmOutput)
}

compareLinModResults <- function(linMod1, linMod2)
{
  checkEquals(linMod1$coefficients, linMod2$coefficients)
  checkEquals(linMod1$nValidObs, linMod2$nValidObs)
  checkEquals(linMod1$nMissingObs, linMod2$nMissingObs)
  checkEquals(linMod1$r.squared, linMod2$r.squared)
  checkEquals(linMod1$sigma, linMod2$sigma)
  checkEquals(linMod1$coef.t.value, linMod2$coef.t.value)
  checkEquals(linMod1$coef.std.error, linMod2$coef.std.error)
  checkEquals(linMod1$total.squares, linMod2$total.squares)

}

rxLinModRemoveNAs <- function(x) x[!is.na(x)]

.setUp <- function()
{
  if (file.exists(testOutputDir))
  {
    unlink(testOutputDir,recursive=TRUE)
  }
  dir.create(testOutputDir)
}

.tearDown <- function()
{
  unlink(testOutputDir,recursive=TRUE)
}

"test.rxLinMod.data.df.whiteside" <- function()
{
    gasLinMod.test <- rxLinMod(Gas ~ Temp, data=whiteside) 
    gaslm.test <- lm(Gas ~ Temp, data=whiteside)
    checkTrue(all.equal(as.numeric(coefficients(gasLinMod.test)), as.numeric(coefficients(gaslm.test))))
    checkTrue(!identical(gasLinMod.test, gaslm.test))
}

"test.rxLinMod.data.df.iris" <- function()
{
    #TODO: uncomment when we get dot exapansion support in ExaStat
    #irisLinMod.test <- rxLinMod(Sepal.Length~., data=iris)
    form <- Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species
    irisLinMod.test <- rxLinMod(form, data=iris)
    irislm.test <- lm(form, data=iris, contrasts=list(Species=contr.SAS))
    # remove NA entries in rxLinMod output, representing aliased coefficients
    rxLinModOutput <- rxLinModRemoveNAs(as.numeric(coefficients(irisLinMod.test)))
    checkTrue(all.equal(rxLinModOutput, as.numeric(coefficients(irislm.test))))
    #checkTrue(!identical(irisLinMod.test, irislm.test))
}

"test.rxLinMod.data.xdf.missings" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
    test100r5bStartMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bStartMiss")
    test100r5bEndMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bEndMiss")
    test77r1bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r1bNoMiss")
    test77r5bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r5bNoMiss")
    
    testDataFrame100 <- rxDataStep(inData =test100r1b)
    testDataFrame77 <- rxDataStep(inData =test77r1bNoMiss)
    
    # Use lm to compare data sets with and without missings
    lmObj100 <- lm(y~xnum1+xnum2miss+ xint1, data=testDataFrame100)
    lmObj77 <- lm(y~xnum1+xnum2miss+ xint1, data=testDataFrame77)
    checkTrue(compareSingleDepvarLM(coef(lmObj100),coef(lmObj77)))
    
    # Compare rxLinMod:  1 block with missings
    linModObj1 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r1b)
    checkTrue(compareSingleDepvarLM(coef(linModObj1),coef(lmObj77)))
    
    # Compare rxLinMod:  5 blocks with missings
    linModObj2 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5b)
    checkTrue(compareSingleDepvarLM(coef(linModObj2),coef(lmObj77)))
    
    # Compare rxLinMod: 5 blocks with missings first
    linModObj3 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5bStartMiss)
    checkTrue(all.equal(coef(linModObj3),coef(lmObj77)))
    
    # Compare rxLinMod: 5 blocks with missings last
    linModObj4 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5bEndMiss)
    checkTrue(compareSingleDepvarLM(coef(linModObj4),coef(lmObj77))) 
  }

###
# argument stress tests
###

"test.rxLinMod.args.blocksPerRead" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  form <- y ~ F(y) + xnum2miss
  zLinModBlocksDefault <- rxLinMod(form, data=test100r5b, blocksPerRead=-1)
  isExpressEdition <- rxIsExpressEdition()
    
    out1 <- capture.output(zLinModBlocks1 <- rxLinMod(form, data=test100r5b, blocksPerRead=1))
    if (isExpressEdition)
    {
        checkEquals(length(out1), 2)
    } else
    {
        checkEquals(length(out1), 6)
    }
    
        
  nocomp <- which(names(zLinModBlocks1) %in% c("call","params"))
  checkTrue(all.equal(zLinModBlocksDefault[-nocomp], zLinModBlocks1[-nocomp], check.attributes=FALSE))
  
  out5 <- capture.output(zLinModBlocks5 <- rxLinMod(form, data=test100r5b, blocksPerRead=5))
    checkEquals(length(out5), 2)
    
  checkTrue(all.equal(zLinModBlocksDefault[-nocomp], zLinModBlocks5[-nocomp], check.attributes=FALSE))
    
    checkTrue(all.equal(zLinModBlocksDefault[-nocomp], zLinModBlocks5[-nocomp], check.attributes=FALSE)) 
       
    # Try using a data source
    testDS <- RxXdfData(test100r5b)
    out5a <- capture.output(zLinModBlocks5DS <- rxLinMod(form, data=testDS, blocksPerRead=5))
    checkEquals(length(out5a), 2)
    
    testDS <- RxXdfData(test100r5b, blocksPerRead = 5)
    out5b <- capture.output(zLinModBlocks5DS <- rxLinMod(form, data=testDS))
    checkEquals(length(out5b), 2)
    
  
}

"test.rxLinMod.args.covCoef.covData" <- function()
{
    .isVCMatrix <- function(x) is.matrix(x) && is.numeric(x) && nrow(x) == ncol(x)
    form <- Sepal.Width ~ Sepal.Length + Petal.Length + Petal.Width
  z <- rxLinMod(form, data = iris, covCoef = TRUE)
  zCovCoef <- rxCovCoef(z)
    checkTrue(.isVCMatrix(zCovCoef))
  
    # test various combinations of covCoef and covData
    z <- rxLinMod(form, data=iris, covCoef=TRUE, covData=FALSE)
  checkTrue(.isVCMatrix(z$covCoef))
  checkTrue(is.null(z$covData))
 
    z <- rxLinMod(form, data=iris, covCoef=TRUE, covData=TRUE)
  checkTrue(.isVCMatrix(z$covCoef))
  checkTrue(.isVCMatrix(z$covData))

    z <- rxLinMod(form, data=iris, covCoef=FALSE, covData=FALSE)
  checkTrue(is.null(z$covCoef))
  checkTrue(is.null(z$covData))
       
    z <- rxLinMod(form, data=iris, covCoef=FALSE, covData=TRUE)
  checkTrue(is.null(z$covCoef))
  checkTrue(.isVCMatrix(z$covData))

 }


# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxLinMod.args.cppInterp" <- function()
#{
    ## testing transformation and newVars
    #w <- cbind (whiteside, rvar=runif(nrow(whiteside)))
#
    ## in place transformation
    #linModTrans.test <- rxLinMod("Gas ~ Insul:Temp+rvar", data=w, 
    #cppInterp=list(transforms="rvar=pow(rvar,2)"))
#
    ## introduce new variable as an additional column
    #linModVars.test <- rxLinMod("Gas ~ Insul:Temp+rvar1", data=w, 
    #cppInterp=list(newVars="rvar1=pow(rvar,2)"))
    #checkTrue(all.equal(as.numeric(coefficients(linModTrans.test)), as.numeric(coefficients(linModVars.test))))
#}

"test.rxLinMod.args.cube" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
  airlineDF <- rxReadXdf(file = inFile)
  airline2DF<-cbind(airlineDF, ArrOnTime=((airlineDF$ArrDelay > -2) & (airlineDF$ArrDelay < 2)))
  zLinModCubeTrue <- rxLinMod(ArrOnTime ~ DayOfWeek + CRSDepTime , data=airline2DF, blocksPerRead=100, 
        cube=TRUE, cubePredictions=TRUE)
  checkTrue(all(names(zLinModCubeTrue$countDF) == c("DayOfWeek", "ArrOnTime", "Counts")))

  zLinModCubeFalse <- rxLinMod(ArrOnTime ~ DayOfWeek + CRSDepTime , data=airline2DF, blocksPerRead=100, cube=FALSE)
  checkTrue(is.null(zLinModCubeFalse$countDF))
}

"test.rxLinMod.args.cubePredictions" <- function()
{
    # cubePredictions for dummy variables outside the cube
  inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
  airlineDF <- rxDataStep(inData = inFile)
  airline2DF<-cbind(airlineDF, ArrOnTime=((airlineDF$ArrDelay > -2) & (airlineDF$ArrDelay < 2)))
  zLinModCubePredTrue <- rxLinMod(ArrOnTime ~ DayOfWeek + CRSDepTime , data=airline2DF, blocksPerRead=100,
         cube=TRUE, cubePredictions = TRUE)
  checkTrue(all(names(zLinModCubePredTrue$countDF) == c("DayOfWeek", "ArrOnTime", "Counts")))

  zLinModCubePredFalse <- rxLinMod(ArrOnTime ~ DayOfWeek + CRSDepTime , data=airline2DF, blocksPerRead=100, cube=TRUE)
    checkTrue(all(names(zLinModCubePredFalse$countDF) == c("DayOfWeek", "Counts")))
  checkTrue(is.null(zLinModCubePredFalse$countDF$ArrOnTime))
    
    xfac1 <- factor(rep(c(1,2), times=50))
    xfac2 <- factor(rep(c(1, 2, 3, 4), each=25))
    x3 <- rep(c(5, 10, 15, 20, 25), times=20)
    z  <- rep(c(.5, .1, .3, .4, .5), each=20)
    y <- 1*(xfac1==1) + 2*(xfac1==2) + 3*(xfac2==1) + 2*(xfac2==2) + 3*(xfac2==3) + 4*(xfac2==4) + 1*x3 + z
    myData <- data.frame(y, xfac1, xfac2, x3)

    linMod1 <- rxLinMod(y~xfac1+x3+xfac2, data=myData, cube=TRUE, cubePredictions=TRUE)

    # Let's figure out what the predicted values should actually be using weighted coefficients
    # If xfac == 1
    coef1 <- coef(linMod1)
    yPred1 <- coef1["xfac1=1"] + coef1["x3"]*mean(x3) + .25*coef1["xfac2=1"] + .25*coef1["xfac2=2"] + .25*coef1["xfac2=3"]
    checkEquals(yPred1, linMod1$countDF$y[1], checkNames=FALSE)
    yPred2 <- coef1["xfac1=2"] + coef1["x3"]*mean(x3) + .25*coef1["xfac2=1"] + .25*coef1["xfac2=2"] + .25*coef1["xfac2=3"]
    checkEquals(yPred2, linMod1$countDF$y[2], checkNames=FALSE)
    
    # Make sure order of additional variables doesn't matter
    linMod2 <- rxLinMod(y~xfac1+xfac2+x3, data=myData, cube=TRUE, cubePredictions=TRUE)
    checkEquals(linMod1$countDF, linMod2$countDF)
    
}

"test.rxLinMod.args.data" <- function()
{
  form <- score ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE), score=runif(5))
  XDF <- tempfile( pattern = ".rxLinMod1", fileext = ".xdf")
  rxDataStep(DF, XDF)
  
  zLinModDF <- rxLinMod(form, data=DF)
  zLinModXDF <- rxLinMod(form, data=XDF)
  nocomp <- which(names(zLinModXDF) %in% c("call","params"))
  checkTrue(all.equal(zLinModDF[-nocomp], zLinModXDF[-nocomp], check.attributes=FALSE))   
    
  file.remove(XDF)
}

"test.rxLinMod.args.formula.df" <- function()
{
  # data frame data source
    set.seed(100)
    x <- runif(1:1000)
    y <- 1 + 2*log(x) + rnorm(1:1000)
    y2 <- 1 + 2*log(x+1) + rnorm(1:1000)
    myDF <- data.frame(x=x, y=y, y2=y2)
   
    linModObj <- rxLinMod(y~log(x), data=myDF)
    lmObj <- lm(y ~ log(x), data=myDF)
    checkEqualsNumeric(as.vector(coef(linModObj)), as.vector(coef(lmObj)), tolerance=1e-3)
    
    linModObj2 <- rxLinMod(y2~log(x+1), data=myDF)
    lmObj2 <- lm(y2 ~ log(x+1), data=myDF)
    checkEqualsNumeric(as.vector(coef(linModObj2)), as.vector(coef(lmObj2)), tolerance=1e-3)
}

"test.rxLinMod.args.formula.xdf" <- function()
{
  # XDF data source
  dataFile <- file.path(sampleDataDir, "mortDefaultSmall")
  myDF <- rxDataStep(inData =dataFile)    
    linModObj6 <- rxLinMod(creditScore~ log(ccDebt+1), data=dataFile)
  lmObj6 <- lm(creditScore ~ log(ccDebt+1), data=myDF)
  checkEqualsNumeric(as.vector(coef(linModObj6)), as.vector(coef(lmObj6)), tolerance=1e-3)
}

"test.rxLinMod.args.formula.multiple.depvars" <- function()
{
  # multiple dependent variables
    sampleDataDir <- rxGetOption("sampleDataDir")
    myFile <- file.path(sampleDataDir, "AirlineDemoSmall")
    z1 <- rxLinMod(ArrDelay ~ DayOfWeek, data=myFile)
    z2 <- rxLinMod(cbind(ArrDelay, CRSDepTime) ~ DayOfWeek, data=myFile)
    z3 <- rxLinMod(cbind(pow(ArrDelay,2), ArrDelay, CRSDepTime) ~ DayOfWeek, data=myFile)
    z4 <- rxLinMod(pow(ArrDelay,2) ~ DayOfWeek, data=myFile)
    
    # comparative regression coefficients checks
    checkTrue(all.equal(coef(z1), coef(z2)[,"ArrDelay", drop=TRUE]))
    checkTrue(all.equal(coef(z2)[,"CRSDepTime", drop=FALSE], coef(z3)[,"CRSDepTime", drop=FALSE]))
    
    # regression coefficient matrices checks
    checkTrue(NCOL(z1$coefficients) == 1)
    checkTrue(NCOL(z2$coefficients) == 2)
    checkTrue(NCOL(z3$coefficients) == 3)
    checkTrue(NCOL(z4$coefficients) == 1)
    
    # dependent variabel name checks in regression coefficients matrix
    checkIdentical(colnames(z1$coefficients), "ArrDelay")
    checkIdentical(colnames(z2$coefficients), c("ArrDelay","CRSDepTime"))
    checkIdentical(colnames(z3$coefficients), c("pow(ArrDelay,2)","ArrDelay","CRSDepTime"))
    checkIdentical(colnames(z4$coefficients), "pow(ArrDelay,2)")
    
    # basic summary structure    
    checkTrue(is.list(summary(z1)))
    checkTrue(is.list(summary(z2)))
    checkTrue(is.list(summary(z3)))
    checkTrue(is.list(summary(z4)))
    
    checkTrue(is(summary(z1),"summary.rxLinModList"))
    checkTrue(is(summary(z2),"summary.rxLinModList"))
    checkTrue(is(summary(z3),"summary.rxLinModList"))
    checkTrue(is(summary(z4),"summary.rxLinModList"))
    
    checkIdentical(names(summary(z1)), "ArrDelay")
    checkIdentical(names(summary(z2)), c("ArrDelay","CRSDepTime"))
    checkIdentical(names(summary(z3)), c("pow(ArrDelay,2)","ArrDelay","CRSDepTime"))
    checkIdentical(names(summary(z4)), "pow(ArrDelay,2)")
}

"test.rxLinMod.args.formula.power.whiteside" <- function()
{
    # RA-190, RA-334, RA-1727
    require(MASS)
    form <- Gas ~ Insul/(Temp + I(Temp^2)) - 1
    mylm <- rxLinMod(form, data=whiteside)
    mylm1 <- lm(form, data=whiteside) 
    checkEquals(coef(mylm), coef(mylm1), check.attributes=FALSE)
}

"test.rxLinMod.args.formula.power.census" <- function()
{
    # RA-900: Comparing pow specification in rxLinMod formula expression with
    #         equivalent lm formula expression does not produce the same
    #         coefficient matrix.

    censusFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    censusData <- rxReadXdf(file = censusFile, reportProgress = 0)

    linModPow <-
      rxLinMod(pow(incwage,2)~ state, data = censusData, reportProgress = 0)
    linModCaret <-
      rxLinMod(I(incwage^2)~ state, data = censusData, reportProgress = 0)
    lmCaret <- lm(I(incwage^2) ~ state, contrasts = list(state = "contr.SAS"),
                  data = censusData)

    checkEquals(coefficients(linModPow), coefficients(linModCaret))
    checkEqualsNumeric(head(coefficients(linModPow), -1), coefficients(lmCaret))
}

"test.rxLinMod.args.formula.cut.census" <- function()
{
    # RA-1011: Expressions need to accommodate cut() function.

    censusFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    censusData <- rxReadXdf(file = censusFile, reportProgress = 0)

    linModCut <-
      rxLinMod(log1p(incwage)~ cut(age, seq(20, 70, by = 10), include.lowest = TRUE) - 1,
               data = censusData, reportProgress = 0)
    lmCut <-
      lm(log1p(incwage)~ cut(age, seq(20, 70, by = 10), include.lowest = TRUE) - 1,
         data = censusData)

    checkEqualsNumeric(coefficients(linModCut), coefficients(lmCut))
}

"test.rxLinMod.args.formula.composite.transforms" <- function()
{
    # RA-1021: Composite formulas that contain arithmetic expressions sometimes
    #          fail

    test <- data.frame(x = 1:10, y = 1:10 + rep(c(.1, .2), 5),
                       row.names = head(letters, 10))
    linModComp <- rxLinMod(y ~ I(sqrt(x)), data = test, reportProgress = 0)
    lmComp <- lm(y ~ I(sqrt(x)), data = test)
    checkEqualsNumeric(coefficients(linModComp), coefficients(lmComp))
}

"test.rxLinMod.args.fweights" <- function()
{
    fwts <- c((1:5), (1:5))
    sex <- c("Male","Male","Female", "Male", "Female", "Male", "Female", "Male", "Female", "Male") 
    age <- c(NA,20,12,15, NA, 17, 16, 13, 19, 12) 
    score <- (1:10)+0.1 
    myDF1 <- data.frame(sex = sex, age=age, score = score, weightVar = fwts) 
    myDF2 <- data.frame(sex = rep(sex,fwts), age = rep(age, fwts), score = rep(score, fwts)) 
    fweightsObj <- rxLinMod(age ~ sex + score, data = myDF1, fweights = "weightVar") 
    baseObj <- rxLinMod(age ~ sex + score, data = myDF2) 
    fweightsObj$call <- NULL
    baseObj$call <- NULL
    fweightsObj$params <- NULL
    baseObj$params <- NULL
    # Number of missings will be unweighted for fweightsObj
    fweightsObj$nMissingObs <- NULL
    baseObj$nMissingObs <- NULL 
    checkEquals(fweightsObj, baseObj)     
}

"test.rxLinMod.args.pweights.fweights.df" <- function()
{
    fwts <- 1:4
    sex <- c("Male","Male","Female", "Male")
    age <- c(20,20,12,15)
    score <- (1:4)+0.1
    
    myDF1 <- data.frame(sex = sex, age=age, score = score, weightVar = fwts)    
    myDF2 <- data.frame(sex = rep(sex,fwts), age = rep(age, fwts), score = rep(score, fwts))
    
    pweightsObj <- rxLinMod(age ~ sex + score, data = myDF1, pw = "weightVar")
  pw <- "weightVar"
  pweightsObj1 <- rxLinMod(age ~ sex + score, data = myDF1, pweights = pw)

  fweightsObj <- rxLinMod(age ~ sex + score, data = myDF1, fweights = "weightVar")
  fw <- "weightVar"
  fweightsObj1 <- rxLinMod(age ~ sex + score, data = myDF1, fweights = fw)
    baseObj <- rxLinMod(age ~ sex + score, data = myDF2)
  lmObj <- lm(age ~ sex + score, data = myDF2, contrasts = contr.SAS)
      
  # Estimated coefficients should be the same
    checkTrue(all.equal(coef(pweightsObj),coef(baseObj))) 
  checkTrue(all.equal(coef(fweightsObj),coef(baseObj))) 
    checkTrue(all.equal(as.vector(coef(baseObj)),c(23.342857, -5.142857, NA, -2.000000)))
  
  # Degrees of freedom should be different for pweights
  checkTrue(pweightsObj$df[2] == 1)
  checkTrue(fweightsObj$df[2] == baseObj$df[2])
  
  # Results should be the same if the variable name is not quoted
  checkTrue(all.equal(coef(pweightsObj),coef(pweightsObj1)))
  checkTrue(all.equal(coef(fweightsObj),coef(fweightsObj1))) 
  checkTrue(all.equal(pweightsObj$df, pweightsObj1$df))
  checkTrue(all.equal(fweightsObj$df, fweightsObj1$df))
}

"test.rxLinMod.args.reportProgress" <- function()
{
  # reportProgress argument stress tests
  form <- weight ~ group
  ctl <- c(4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14)
  trt <- c(4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69)
  DF <- data.frame(group = gl(2,10,20, labels=c("Ctl","Trt")), weight = c(ctl, trt))
  
  #reportProgress: logical, character string or integer: 
  #  0: no progress is reported. 
  #  1: the number of processed rows is printed and updated. 
  #  2: rows processed and timings are reported. 
  #  3: rows processed and all timings are reported. 

  variations <- list(0,1,2,3)	
  results <- printOutput <- vector("list", length=length(variations))
  
  for (i in seq_along(variations))
  {
    printOutput[[i]] <- capture.output(results[[i]] <- 
            rxLinMod(form=form, data=DF, reportProgress=variations[[i]]))
  }

  # check that all values are the same
  nocomp <- which(names(results[[1]]) %in% c("call","params"))
  checkTrue(all(unlist(lapply(results[-1], 
    function(x,y,nocomp){all.equal(x[-nocomp], y[-nocomp], check.attributes=FALSE)}, 
    y=results[[1]], nocomp=nocomp))))

  names(printOutput) <- as.character(variations)
  

  noResponseExpected <- c("0")
  equalLengthExpected <- c("2")
  equalLengthExpected1 <- c("1")

  
  checkTrue(length(unlist(printOutput[noResponseExpected])) == 0)
  
  # just rows
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1],length)) == 1))
  
  # all these outputs should be the same
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected],length)) == 2))
  
  # details gives us the goods
  checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=20, TotalRowsProcessed=20, ReadTime", 
    printOutput[["3"]][1])) > 0)
  
  # time
  checkTrue(length(grep("^Computation time:", printOutput[["2"]][2])) > 0)
}

# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxLinMod.args.rowSelection" <- function()
#{
    ## testing rowSelection: RA-185
    #gas.cppInterp.test <- rxLinMod("Gas ~ Temp", data=whiteside, 
    #cppInterp=list(rowSelection="Insul=='Before'"))
  #gas.rInterp.test <- rxLinMod(Gas ~ Temp, data=whiteside, rowSelection=Insul=='Before') 
    #gas.lm.test <- lm(Gas ~ Temp, data=whiteside, subset=Insul=="Before")
    #checkTrue(all.equal(as.numeric(coefficients(gas.cppInterp.test)), as.numeric(coefficients(gas.lm.test))))
  #checkTrue(all.equal(as.numeric(coefficients(gas.rInterp.test)), as.numeric(coefficients(gas.lm.test))))
#}

"test.rxLinMod.args.rowSelection.df" <- function()
{
    myDF <- data.frame(x = rnorm(20), y = rnorm(20))
  myDF$selectVar = myDF$y > 0.0
    # Not yet implemented: specify a variable name for selection as character string
    #selectRowsObj1 <- rxLinMod(y ~ x, data=myDF, rowSelection="selectVar")
  selectRowsObj2 <- rxLinMod(y ~ x, data=myDF, rowSelection=selectVar)
  selectRowsObj3 <- rxLinMod(y ~ x, data=myDF, rowSelection= y > 0.0)
  
  # FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
  #selectRowsObj4 <- rxLinMod(y ~ x, data=myDF, cppInterp=list(rowSelection = "y > 0.0"))
  
  lmObj <- lm(y ~ x, data=myDF, subset=selectVar)
      
  # Estimated coefficients should be the same
    #checkTrue(all.equal(coef(selectRowsObj1),coef(lmObj))) 
  checkTrue(compareSingleDepvarLM(coef(selectRowsObj2),coef(lmObj))) 
  checkTrue(compareSingleDepvarLM(coef(selectRowsObj3),coef(lmObj))) 
  
  # FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
  #checkTrue(compareSingleDepvarLM(coef(selectRowsObj4),coef(lmObj))) 
}

"test.rxLinMod.args.rowSelection.dotsInName.iris" <- function()
{      
    res <- try(rxLinMod(Sepal.Length ~ Sepal.Width, data=iris, rowSelection = Petal.Length > 1.5))
    
    # check for no errors encountered during execution
  checkTrue(!is(res, "try-error"))
  
  iris1 <- subset(iris, Petal.Length > 1.5)
  lmres <- lm(Sepal.Length ~ Sepal.Width, data=iris1)
  checkEqualsNumeric(as.vector(coef(res)), as.vector(coef(lmres)), tolerance=1e-6)
}

"test.rxLinMod.args.rowSelection.xdf" <- function()
{
  dataFile <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall")
  #rxGetInfo(dataFile)
  myDF <- rxDataStep(inData =dataFile)
  
  # FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
  #linModObj1 <- rxLinMod(creditScore~yearsEmploy, data=dataFile, cppInterp=list(rowSelection="yearsEmploy > 5 && ccDebt > 1000"))
    linModObj2 <- rxLinMod(creditScore~yearsEmploy, data=dataFile, rowSelection=yearsEmploy > 5 & ccDebt > 1000)
  lmObj <- lm(creditScore~yearsEmploy, data=myDF, subset=yearsEmploy > 5 & ccDebt > 1000)
  
  # FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
    #checkTrue(compareSingleDepvarLM(coef(linModObj1),coef(lmObj)))
  
    checkTrue(compareSingleDepvarLM(coef(linModObj2),coef(lmObj)))
  
  transformFunc <- function(data)
  {
    data$selectVar <- data$yearsEmploy >= 5 & data$yearsEmploy <= 10 & data$ccDebt > 1000
    data$selectVar2<- data$yearsEmploy >= 7
    return(data)
  }
  
  linModObj3 <- rxLinMod(creditScore~yearsEmploy, data=dataFile, transformFunc=transformFunc,
    transformVars=c("ccDebt", "yearsEmploy"), rowSelection=selectVar)
  lmObj <- lm(creditScore~yearsEmploy, data=myDF, subset= yearsEmploy >=5 & yearsEmploy <= 10 & ccDebt > 1000)
  checkTrue(compareSingleDepvarLM(coef(linModObj3),coef(lmObj)))
    
    # Transformation in formula + transformFunc + selection
    # Bug RA-1727 fixed.
    linModObj4 <- rxLinMod(creditScore~ccDebt*ccDebt, data=dataFile, transformFunc=transformFunc,
      transformVars= c("yearsEmploy", "ccDebt"), rowSelection=selectVar)
  lmObj <- lm(creditScore~ccDebt*ccDebt, data=myDF, subset= yearsEmploy >=5 & yearsEmploy <= 10 & ccDebt > 1000)
  checkTrue(compareSingleDepvarLM(coef(linModObj4),coef(lmObj)))
}

"test.rxLinMod.args.transformFunc.ifelse.df" <- function()
{
  xform <- function(x)
  {
    x$Temp.sub <- ifelse(x$Temp>65,1,0)
    return(x)
  }
  res.lm <- try(lm(Ozone ~ Solar.R + (Temp > 65), data = airquality))
  res.linMod <- try(rxLinMod(Ozone ~ Solar.R + Temp.sub, transformFunc=xform, transformVars="Temp", data = airquality))
  checkEqualsNumeric(as.vector(coef(res.lm)),as.vector(coef(res.linMod)))
}

"test.rxLinMod.args.transformFunc.whiteside" <- function()
{
    # testing transformation and newVars
    w <- cbind (whiteside, rvar=runif(nrow(whiteside)))
    gaslm.test <- lm(Gas ~ Insul:Temp+I(rvar^2), data=w)
    #gasLinMod.test <- rxLinMod(Gas ~ Insul:Temp+I(rvar*rvar), data=w)
    #checkTrue(all.equal(as.numeric(coefficients(gaslm.test)), as.numeric(coefficients(gasLinMod.test))))
    #
    # in place transformation
    transformFunc <- function(data)
    {
        data$rvarsq <-data$rvar^2
        return(data)
    }
    
    gasLinMod.test <- rxLinMod(Gas ~ Insul:Temp + rvarsq, data=w, transformFunc=transformFunc, 
         transformVars=c("rvar"))
    # coefficients are returned in different orders
    checkEquals(coef(gasLinMod.test)[c(1,4,2,3)], coef(gaslm.test), check.attributes=FALSE)
    
    # FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
    #gasLinModNewVar.test <- rxLinMod(Gas ~ Insul:Temp+rvarsq, data=w, cppInterp=list(newVars="rvarsq = pow(rvar,2)", transformVars=c("rvar")))
    #checkTrue(all.equal(as.numeric(coefficients(gasLinMod.test)), as.numeric(coefficients(gasLinModNewVar.test))))
    #checkEquals(coef(gasLinMod.test), coef(gasLinModNewVar.test))
}

"test.rxLinMod.args.transformFunc.dotsInName.iris" <- function()
{
    xform <- function(x)
    {
      x$area <- x$Sepal.Length * x$Sepal.Width
      return(x)
    }
    
    res <- try(rxLinMod(Sepal.Length ~ area, data=iris, transformFunc=xform, 
      transformVars=c("Sepal.Length","Sepal.Width")))
    
    # return TRUE for no errors encountered during execution
  checkTrue(!is(res, "try-error"))
  checkEqualsNumeric(as.vector(coef(res)), c(2.8617369, 0.1672905), tolerance=1e-6)
}

"test.rxLinMod.args.transformVars.xdf" <- function()
{
  dataFile <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall")
  
  transformFunc1 <- function(data)
  {
    data$yearsEmploySq <- data$yearsEmploy * data$yearsEmploy
    return(data)
  }

  # If no transformVars are specified, all formula arguments should be passed to transformFunc
  linModObj1 <- try(rxLinMod(creditScore~yearsEmploy + yearsEmploySq, data=dataFile, transformFunc=transformFunc1))
  checkTrue(!is(linModObj1, "try-error"))
  
  # This should fail because the user has mis-specified the needed transformVars
  linModObj2 <- try(rxLinMod(creditScore~yearsEmploy + yearsEmploySq, data=dataFile, 
    transformFunc=transformFunc1, transformVars=c("creditScore")))
  checkTrue(is(linModObj2, "try-error"))
  
  transformFunc2 <- function(data)
  {
    data$creditScore10 <- data$creditScore*10
    return(data)
  }	

  # If no transformVars are specified, this should fail
  linModObj2 <- try(rxLinMod(creditScore10~yearsEmploy, data=dataFile, transformFunc=transformFunc2))
  checkTrue(is(linModObj2, "try-error"))	
  
  # If transformVars are specified, this should work
  linModObj3 <- try(rxLinMod(creditScore10~yearsEmploy, data=dataFile, transformFunc=transformFunc2,
    transformVars=c("creditScore")))
  checkTrue(!is(linModObj3, "try-error"))
    
  transformFunc3 <- function(data)
  {
    data$yearsEmployDum <- data$yearsEmploy >= 10
    return(data)
  }
    # Transformation in formula + transformFunc + selection
    linModObj4 <- try(rxLinMod(creditScore~ccDebt*ccDebt+yearsEmployDum, data=dataFile, 
    transformFunc=transformFunc3, rowSelection=yearsEmploy >= 5))
  # This should work because rowSelection uses yearsEmploy, which will be passed to transformFunc
  checkTrue(!is(linModObj4, "try-error"))
  
}

"test.rxLinMod.transformFunc.negative" <- function(){
  # Check if subsetting rows with transform function generate informative error message
  
  data_path = file.path(rxGetOption("sampleDataDir"), "claims.xdf")
  ds = RxXdfData(data_path)
  df = rxDataStep(ds)
  transform_func = function(data)
  {
    ret = data.frame(data)
    ret = ret[ret$cost < 300,,drop=FALSE]
    return(ret)
  }
  form = number ~ age + cost
  
  claim_mod = tryCatch({
    rxLinMod(form, df, transformFunc=transform_func, transformVars="cost")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxLinMod"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxLinMod"))
    }
  })
  
}


"test.rxLinMod.args.verbose.iris" <- function()
{
    #TODO: uncomment when we get dot exapansion support in ExaStat
    #linModObj <- rxLinMod(Sepal.Length~., data=myData)
    #o1 <- capture.output(irisLinMod.test1 <- rxLinMod(Sepal.Length~., data=iris, reportProgress=0))
    #o2 <- capture.output(irisLinMod.test2 <- rxLinMod(Sepal.Length~., data=iris, verbose=0, reportProgress=0))
    #o3 <- capture.output(irisLinMod.test3 <- rxLinMod(Sepal.Length~., data=iris, verbose=0, reportProgress=0))
    form <- Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species
    linModObj <- rxLinMod(form, data=iris)
    o1 <- capture.output(irisLinMod.test1 <- rxLinMod(form, data=iris, reportProgress=0))
    o2 <- capture.output(irisLinMod.test2 <- rxLinMod(form, data=iris, verbose=0, reportProgress=0))
    o3 <- capture.output(irisLinMod.test3 <- rxLinMod(form, data=iris, verbose=1, reportProgress=0))

    # No output if verbose is FALSE (the default)
    checkEquals(length(o1), 0)
    checkEquals(length(o2), 0)
    checkTrue(length(o3) > 1)
}

"test.rxLinMod.formula.long" <- function()
{
    if ( grepl( "SLES11|TERA14S11|TERA14S10", packageDescription("RevoScaleR", fields="RevoBuildId" ) ) )
        {
            DEACTIVATED("RA-10039: Long formulas not supported on SLES11 or TERADATA14-SLES11 platforms.")
       
        }

    # define very long formula
    set.seed(100)
    nVars <- 100
    nRows <- 100
    vars <- c("Weekday", unlist(lapply(seq(nVars), function(x) basename(tempfile(pattern = ".rxLinModFL")))))
    formStr <- paste("myDepVar ~ ", paste(vars, collapse=" + "), collapse="")
    form <- formula(formStr)

    dataList <- data.frame(lapply(seq_along(vars), function(x, n){rnorm(n)}, n=nRows))
    names(dataList) <- vars
    dataList$Weekday <- factor(
        sample(c("Saturday","Sunday","Monday","Tuesday","Wednesday","Thursday","Friday"),
        nRows, replace=TRUE))
    dataList$myDepVar <- runif(nRows)
    zFormula <- rxLinMod(form, data=dataList, cube=TRUE)
    zString <- rxLinMod(formStr, data=dataList, cube=TRUE)
    
    zFormulaSummary <- summary(zFormula)
    zStringSummary <- summary(zString)

    # compare formulas
    deblank <- function(x) gsub("[ ]+","",x)
    zFormulaStr <- deblank(zFormula$params$Formula)
    checkEquals(zFormulaStr, deblank(zString$params$Formula))
    
    # check that the summary method also contains the original formula
    checkEquals(zFormulaStr, deblank(paste(deparse(zStringSummary$myDepVar$formula), collapse=" ")))
    checkEquals(zFormulaStr, deblank(paste(deparse(zFormulaSummary$myDepVar$formula), collapse=" ")))
    
    # check that that print method for the summary outputs a multiline formula
    zStringSummaryPrint <- capture.output(print(zStringSummary))
    zFormulaSummaryPrint <- capture.output(print(zFormulaSummary))
    checkTrue(length(grep("^Cube Linear Regression Results for: myDepVar [~] Weekday [+]*", 
        zStringSummaryPrint[4])) > 0)
    checkTrue(length(grep("^Cube Linear Regression Results for: myDepVar [~] Weekday [+]*", 
        zFormulaSummaryPrint[4])) > 0)
    checkTrue(length(grep("^[ ]+.*[file].*", zStringSummaryPrint[6:8])) == 3)
    checkTrue(length(grep("^[ ]+.*[file].*", zFormulaSummaryPrint[6:8])) == 3)
}

"test.rxLinMod.args.transformVars.NULL" <- function()
{
  # Test combinations where a transformation function
  # is specified without transformVars, but all of
  # the transformVars are in the formula
  # See RA-1905
    #NOTE: it is required to specify levels when creating factors in transformFunc!!!
  inFile <- file.path(rxGetOption("sampleDataDir"),"kyphosis") 
  myTransformFunc <- function(data) 
  { 
    data$Age <- data$Age*2. 
    data$Number <- as.integer(data$Number) 
    data$Start <- factor(data$Start, levels = c(1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18)) 
    return(data) 
  }
    
  myTransforms <- substitute( list(
    Age = Age*2,
    Number = as.integer(Number),
    Start = factor(Start, levels = c(1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18))
    ))
    
  # Transformation function with rowSelection
  linMod1 <- rxLinMod(Age ~ Start + Kyphosis + Number, data=inFile, 
    transformFunc=myTransformFunc, rowSelection= Kyphosis=="absent") 
  linMod2 <- rxLinMod(Age ~ Start + Kyphosis + Number, data=inFile, 
    transforms=myTransforms, rowSelection = Kyphosis=="absent")
  compareLinModResults(linMod1, linMod2)
  rm(linMod1)
  rm(linMod2)
  
  myTransforms2 <- substitute( list(
    Age1 = Age*2,
    Number2 = as.integer(Number),
    Start2 = factor(Start, levels = c(1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18))
    ))
  
  # Transformation function with  additional transforms
  linMod1 <- rxLinMod(Age ~ Start + Kyphosis + Number, data=inFile, 
    transformFunc=myTransformFunc, transforms=myTransforms2) 
  linMod2 <- rxLinMod(Age ~ Start + Kyphosis + Number, data=inFile, 
    transforms=myTransforms)
  compareLinModResults(linMod1, linMod2)
  rm(linMod1)
  rm(linMod2)
  
  # Transformation function with formula transforms
  linMod1 <- rxLinMod(Age*2 ~ Start + Kyphosis + Number, data=inFile, 
    transformFunc=myTransformFunc) 
  linMod2 <- rxLinMod(Age*2 ~ Start + Kyphosis + Number, data=inFile, 
    transforms=myTransforms)
  compareLinModResults(linMod1, linMod2)
  
}

"test.rxLinMod.args.transformFunc.rxStartRow" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    
  myTransformFunc <- function( dataList)
  {
    rxDataSource <- .rxReadFileName
    rxStartRow <- .rxStartRow
    rxChunkNum <- .rxChunkNum
    rxIsTestChunk <- .rxIsTestChunk
    
    nrows <- length(dataList[[1]])
    dataList$startRow <- rep(.rxStartRow, nrows)
    dataList$chunkNum <- rep(.rxChunkNum, nrows)
    return ( dataList)
  }
  linModObj <- rxLinMod(y~xnum1+startRow, data=test100r5b, transformFunc=myTransformFunc)
  
  myData <- rxReadXdf(test100r5b)
    if (rxIsExpressEdition())
    {
      myData$startRow <- rep(1, each=100)
    }   else
    {
        myData$startRow <- rep(seq(1,by=20, length.out=5), each=20)
    }
    
  lmObj <- lm(y~xnum1+startRow, data=myData)	
  
  checkEquals(coef(lmObj), coef(linModObj))
  
}

"test.rxLinMod.args.cube.manyfactors" <- function()
{
    set.seed(100)
    x1 <- runif(1000, min=1, max=27)
    x2 <- runif(1000, min=1, max=14)
    x3 <- runif(1000, min=1, max=25)
    x4 <- runif(1000, min=1, max=21)
    x5 <- runif(1000, min=1, max=21)
    x9 <- runif(1000, min=1, max=20)
    x10 <- runif(1000, min=1, max=10)
    x11 <- runif(1000, min=1, max=4)
    y <- 1 + x1+x2+x3+x4+x5+x9+x10+x11+rnorm(1000)
    fac1 <- factor(as.integer(x1))
    fac2 <- factor(as.integer(x2))
    fac3 <- factor(as.integer(x3))
    fac4 <- factor(as.integer(x4))
    fac9 <- factor(as.integer(x9))
    fac10 <- factor(as.integer(x10))
    fac11 <- factor(as.integer(x11))
    myData <- data.frame(y, fac1, fac2, fac3, fac4, x5, fac9, fac10, fac11)
    tm1 <- system.time(
    linModObj1 <- rxLinMod(y~fac1+fac2+fac3+fac4+x5+fac9+fac10+fac11-1, data=myData)
    )

    tm2 <- system.time(
    linModObj2 <- rxLinMod(y~fac1+fac2+fac3+fac4+x5+fac9+fac10+fac11, data=myData, cube=TRUE)
    )

  checkEquals(coef(linModObj1), coef(linModObj2))
  
    # Do a rough time check to make sure that they are within 8 times the time of each other
  # Commented out because timings can be different depending on whatever else is going on
    #checkTrue(tm1["elapsed"] < 8*tm2["elapsed"])
    #checkTrue(tm2["elapsed"] < 8*tm1["elapsed"])
}

"test.rxLinMod.methods.rxResultsDF" <- function()
{
  
    xfac1 <- factor(rep(c(1,2), times=50))
    xfac2 <- factor(rep(c(1, 2, 3, 4), each=25))
    x3 <- rep(c(5, 10, 15, 20, 25), times=20)
    z  <- rep(c(.5, .1, .3, .4, .5), each=20)
    y <- 1*(xfac1==1) + 2*(xfac1==2) + 3*(xfac2==1) + 2*(xfac2==2) + 3*(xfac2==3) + 4*(xfac2==4) + 1*x3 + z
    myData <- data.frame(y, xfac1, xfac2, x3)

    linMod1 <- rxLinMod(y~xfac1+x3+xfac2, data=myData, cube=TRUE, cubePredictions=TRUE)
  
  countsDF1 <- rxResultsDF(linMod1, type="counts")
  
  checkEquals( as.vector(countsDF1$Counts), c(50,50))
  checkEquals(names(countsDF1), c("xfac1", "y", "Counts"))

  linMod2 <- rxLinMod(y~xfac1:F(x3), data=myData, cube=TRUE, cubePredictions=TRUE)
  countsDF2 <- rxResultsDF(linMod2, type="counts")
  checkTrue( is.integer(countsDF2$x3))
  
}

"test.rxLinMod.args.dropFirst" <- function()
{
    xfac1 <- factor(rep(c(1,2), times=50))
    xfac2 <- factor(rep(c(1, 2, 3, 4), each=25))
    x3 <- rep(c(5, 10, 15, 20, 25), times=20)
    z  <- rep(c(.5, .1, .3, .4, .5), each=20)
    y <- 1*(xfac1==1) + 2*(xfac1==2) + 3*(xfac2==1) + 2*(xfac2==2) + 3*(xfac2==3) + 4*(xfac2==4) + 1*x3 + z
    myData <- data.frame(y, xfac1, xfac2, x3)

    linMod1 <- rxLinMod(y~xfac1+x3+xfac2, data=myData, dropFirst = FALSE)
  coef1 <- coef(linMod1)
  checkTrue(is.na(coef1["xfac1=2"]))
  checkTrue(is.na(coef1["xfac2=4"]))
  linMod2 <- rxLinMod(y~xfac1+x3+xfac2, data=myData, dropFirst = TRUE)
  coef2 <- coef(linMod2)
  checkTrue(is.na(coef2["xfac1=1"]))
  checkTrue(is.na(coef2["xfac2=1"]))
}

"test.rxLinMod.args.data.dataSource.xdf" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")	
  
    xdfDS <- RxXdfData(test100r5b)
  
    linModObj1 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=xdfDS)
    linModObj2 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5b)
    checkEquals(coef(linModObj1),coef(linModObj2))
}

########################################################################
###			Analysis on non-xdf datasource							####
########################################################################

"test.rxLinMod.args.data.dataSource.delimited" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")	
  
  origLevels <- c("a", "b", "c", "d", "e", "f", "g", "h", "i", "j")
  newLevels <- c("aa","bb", "cc", "dd", "ee", "ff", "gg", "hh", "ii", "jj")
  
  testColInfo <- list(xfac2=list(type="factor",
    levels=origLevels,
        newLevels=newLevels))
  
    textDSuseFastRead <- RxTextData(inFile, colInfo=testColInfo, useFastRead = TRUE)
    textDSnotuseFastRead <- RxTextData(inFile, colInfo=testColInfo, useFastRead = FALSE)
  
   linModObj1 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=textDSuseFastRead)
   linModObj2 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=textDSnotuseFastRead)
   linModObj3 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5b)
   checkEquals(coef(linModObj1),coef(linModObj2))
   checkEquals(coef(linModObj2),coef(linModObj3))
}

#### Fixed-Format Text datasource
"test.rxLinMod.args.data.dataSource.Fixed" <- function()
{
  dataFile <- file.path(rxGetOption("unitTestDataDir"),"Airline2000Fixed20r.sts")
  dsText <- RxTextData(dataFile,
             #colClasses = c(	"ArrDelay" = "integer", "CRSDepTime" = "float64", "DayOfWeek" = "factor"),
             colInfo = list(  DayOfWeek = list(type = "factor", levels = as.character(1:7), 
                            newLevels = c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")))
            )
  result <- rxLinMod(ArrDelay~DayOfWeek, data=dsText)
    
    testFile <- tempfile(pattern = ".rxLinMod1TempTest", fileext = ".xdf")        
  rxImport(dsText, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLinMod(ArrDelay~DayOfWeek, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}
  
#### Fast Fixed-Format Text datasource	
####  TODO: Design decision on if colInfo override the sts file info
####		Can we import|rxLinMod just subset of columns by colInfo
"test.rxLinMod.args.data.dataSource.FastFixedCheck" <- function()
{
  dataFile <- file.path(rxGetOption("unitTestDataDir"),"Airline2000Fixed20r.sts")
  dsText <- RxTextData(dataFile,
             #colClasses = c(	"ArrDelay" = "integer", "CRSDepTime" = "float64", "DayOfWeek" = "factor"),
             colInfo = list(  DayOfWeek = list(type = "factor", levels = as.character(1:7), 
                            newLevels = c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"))), 
             useFastRead = TRUE
            )
              
    testFile <- tempfile(pattern = ".rxLinMod1TempTest", fileext = ".xdf")        
  rxImport(dsText, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLinMod(ArrDelay~DayOfWeek, testFile)	
  
  result <- rxLinMod(ArrDelay~DayOfWeek, data=dsText)
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}
  
####  Text datasource	
"test.rxLinMod.args.data.dataSource.delimited2" <- function()
{
  dataFile <- file.path(rxGetOption("unitTestDataDir"), "Airline2000r20.csv")
    xdfFile <- tempfile(pattern = ".rxText", fileext = ".xdf")
    colInfo <-  list(  DayOfWeek = list(type = "factor", levels = as.character(1:7), 
                            newLevels = c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")))
                        
  dsTextNotuseFastRead <- RxTextData(dataFile, colInfo = colInfo, useFastRead = FALSE)
    dsTextuseFastRead <- RxTextData(dataFile, colInfo = colInfo, useFastRead = TRUE)
    
  result1 <- rxLinMod(ArrDelay~DayOfWeek, data=dsTextNotuseFastRead)	
    result2 <- rxLinMod(ArrDelay~DayOfWeek, data=dsTextuseFastRead)	
    
  rxImport(dsTextNotuseFastRead, xdfFile, overwrite = TRUE, reportProgress = 0)
    
  result3 <- rxLinMod(ArrDelay~DayOfWeek, xdfFile)	
  
    checkEquals(result1$coefficients, result2$coefficients)
    checkEquals(result2$coefficients, result3$coefficients)
    
    dataFrame <- rxDataStep(xdfFile)
    result4 <- rxLinMod(ArrDelay~DayOfWeek, xdfFile)
    checkEquals(result2$coefficients, result4$coefficients)
    file.remove( xdfFile)
}

#### ODBC datasource 
"test.rxLinMod.dataSource.ODBC.ST" <- function()
{	
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
      rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), 
                useFastRead = FALSE),
      silent = TRUE)
        
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
  odbcDS <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString,
                useFastRead = FALSE,
                colClasses = c("RowNum" = "integer", "age" = "character",
                 "car_age" = "character", "type" = "character",
                 "cost" = "integer", "number" = "integer"))
  result = rxLinMod(cost~number, data=odbcDS)
    
    testFile <- tempfile(pattern = ".rxLinMod2TempTest", fileext = ".xdf")        
  rxImport(odbcDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLinMod(cost~number, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}

#### ODBC datasource 
"test.rxLinMod.dataSource.ODBC.useFastRead" <- function()
{	
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
      rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), 
                useFastRead = TRUE),
      silent = TRUE)
        
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
  odbcDS <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString,
                useFastRead = TRUE,
                colClasses = c("RowNum" = "integer", "age" = "character",
                 "car_age" = "character", "type" = "character",
                 "cost" = "integer", "number" = "integer"))
  result = rxLinMod(cost~number, data=odbcDS)
    
    testFile <- tempfile(pattern = ".rxLinMod3TempTest", fileext = ".xdf")         
  rxImport(odbcDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLinMod(cost~number, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}
  
####  SAS datasource 
"test.rxLinMod.dataSource.SAS" <- function()
{

  dataFile <- file.path(rxGetOption("unitTestDataDir"),"Airline2000r20.sas7bdat")
  sasDS <- RxSasData( dataFile,
             colInfo = list(  DayOfWeek = list(type = "factor", levels = as.character(1:7), 
              newLevels = c("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday")))
            )								
  result1 <- rxLinMod(ArrDelay~DayOfWeek, data=sasDS)
    
    testFile <- tempfile(pattern = ".rxLinMod4TempTest", fileext = ".xdf")     
    
  rxImport(sasDS, testFile, overwrite = TRUE, rowsPerRead = 15, reportProgress = 0)
  result1a <- rxLinMod(ArrDelay~DayOfWeek, testFile)
  
    checkEquals(result1$coefficients, result1a$coefficients)
  
  # Use a weight
  result2 <- rxLinMod(ArrDelay~DayOfWeek, data=sasDS, pweights = "Distance")
  result2a <- rxLinMod(ArrDelay~DayOfWeek, data=testFile, pweights = "Distance")
  checkEquals(result2$coefficients, result2a$coefficients)
  
  # Use a transform
  result3 <- rxLinMod(PosArrDelay~DayOfWeek, data=sasDS, 
    transforms = list(PosArrDelay = ifelse(ArrDelay > 0, ArrDelay, 0)))
  result3a <- rxLinMod(PosArrDelay~DayOfWeek, data=testFile,
    transforms = list(PosArrDelay = ifelse(ArrDelay > 0, ArrDelay, 0))) 
  checkEquals(result3$coefficients, result3a$coefficients)
  
  # Use a transformFunc with transformVars
  testTrans <- function( dataList)
  {
    dataList$PosArrDelay <- ifelse(dataList$ArrDelay > 0, dataList$ArrDelay, 0)
    return( dataList )
  }
  
  result4 <- rxLinMod(PosArrDelay~DayOfWeek, data=sasDS, 
    transformFunc = testTrans, transformVars = "ArrDelay")
  result4a <- rxLinMod(PosArrDelay~DayOfWeek, data=testFile,
    transformFunc = testTrans, transformVars = "ArrDelay") 
  checkEquals(result4$coefficients, result4a$coefficients)	
  
    file.remove(testFile)   
    
}
  
"test.rxLinMod.dataSource.SAS.factors" <- function()
{
    dataFile.sas <- file.path(rxGetOption("sampleDataDir"),"claims.sas7bdat") 

    # Specify factor without levels 
    DS <- RxSasData(dataFile.sas, colClasses=c(cost="numeric",age="factor",car_age="factor",type="factor"),
        rowsPerRead = 20)
 
    myData <- rxImport(inData=DS) 
    form <- "cost~age + car_age + type" 
    dfObj <- rxLinMod(eval(form), data=myData) 
    sasDsObj <- rxLinMod(eval(form), DS) 
    checkEquals(dfObj$coefficients, sasDsObj$coefficients)    
}

    
####  SAS datasource 	SPSS file
"test.rxLinMod.dataSource.SPSS" <- function()
{
  spssFile <- file.path(rxGetOption("sampleDataDir"),"claims.sav")
  spssDS <- RxSasData(spssFile,
            colClasses = c("RowNum" = "integer", "age" = "factor",
                     "car_age" = "factor", "type" = "factor",
                     "cost" = "integer", "number" = "integer"),
             colInfo = list(  age = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34","35-39","40-49","50-59","60+")),
                      car.age = list(type = "factor", levels = c("0-3", "4-7", "8-9","10+")),
                      type = list(type = "factor", levels = c("A", "B", "C", "D")))
            )		
  result <- rxLinMod(cost ~number, data=spssDS)	
  
    testFile <- tempfile(pattern = ".rxLinMod5TempTest", fileext = ".xdf")     
  rxImport(spssDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLinMod(cost ~number, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}

####  SAS datasource 	SPSS file with rowSelection
"test.rxLinMod.dataSource.SPSS.args.rowSelection" <- function()
{
  spssFile <- file.path(rxGetOption("sampleDataDir"),"claims.sav")
  spssDS <- RxSasData(spssFile,
            colClasses = c("RowNum" = "integer", "age" = "factor",
                     "car_age" = "factor", "type" = "factor",
                     "cost" = "integer", "number" = "integer"),
             colInfo = list(  age = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34","35-39","40-49","50-59","60+")),
                      car.age = list(type = "factor", levels = c("0-3", "4-7", "8-9","10+")),
                      type = list(type = "factor", levels = c("A", "B", "C", "D")))
            )		
  result <- rxLinMod(cost ~type, data=spssDS, rowSelection = type != "A" )	
  
    testFile <- tempfile(pattern = ".rxLinMod6TempTest", fileext = ".xdf")     
  rxImport(spssDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLinMod(cost ~type, testFile, rowSelection = type != "A" )	
    
    checkEquals(result$coefficients, result2$coefficients) 
       
    result3 <- rxLinMod(cost ~ number, data = spssDS, rowSelection = myVar > 0,
         transforms = list(myVar = type != "A")) 
    result4 <- rxLinMod(cost ~ number, data = spssDS, rowSelection = type != "A")
    result5 <- rxLinMod(cost ~ number, data = testFile, rowSelection = type != "A")
  checkEquals(result3$coefficients, result4$coefficients) 
    checkEquals(result3$coefficients, result5$coefficients)     
   
    file.remove(testFile)
}

"test.rxLinMod.data.missings" <- function()
{
    set.seed(17)
    x <- rnorm(100)
    y <- 2*x + rnorm(100)
    naIdx <- seq(from=5, to=95, by = 5)
    x[naIdx] <- NA
    myData <- data.frame(x, y)
    x[naIdx] <- 0
    myDataNoNA <- data.frame(x, y)
    myFunc <- function(data)
  {
        data$x[is.na(data$x)] <- 0
        print(length(data$x))
    return(data)
  }
    linModObj <- rxLinMod( y~x, data=myData,
        transformFunc = myFunc)
    linModObjNoNA <- rxLinMod(y~x, data=myDataNoNA)
    
    checkEquals(linModObj$nValidObs, linModObjNoNA$nValidObs)
    
    checkEquals(coef(linModObj), coef(linModObjNoNA))
    
    checkEquals(linModObj$coef.t.value, linModObjNoNA$coef.t.value)
    
}	

"test.rxLinMod.factors.returned" <- function() {
    # Read the data set in
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    df <- rxDataStep(inFile)

    # Generate some factor columns
    n.len <- NROW(df)
    month.int <- trunc(runif(n.len, 1, 12))
    month.names <- c("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December")
    df$Month <- factor(month.names[month.int], levels = month.names)
    dow.int <- trunc(runif(n.len, 1, 7))
    dow.names <- c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")
    df$DayOfWeek <- factor(dow.names[dow.int], levels = dow.names)

    model1 <- rxLinMod(ArrDelay ~ DayOfWeek:Origin + Month + CRSDepTime, df, cube = T)
    # Same variables, no interactions
    model2 <- rxLinMod(ArrDelay ~ Month + DayOfWeek + Origin + CRSDepTime, df, cube = T)

    xl1 <- attr(model1, "xlevels")
    xl2 <- attr(model2, "xlevels")

    checkTrue(all(!is.na(match(names(xl1),c("DayOfWeek","Origin","Month")))))
    checkTrue(all(!is.na(match(names(xl1), names(xl2)))))
    # The names are in different order, so we need to re-order the list
    checkIdentical(xl1[names(xl2)], xl2)
}
