# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLinMod with variableSelection, i.e., stepwise linear regression
#
############################################################################################

.setUp <- function()
{
    ## NOTE: specifying contrasts in options changes the way how factors are labeled!
    #.op.old <<- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
  
  ## various test modes
  # R		-- mostly compatible with R
  # SAS	-- SAS features such as stepCriterion = "SigLevel"
  # REFIT	-- refit at each step instead of using SSCP matrix
  # REVO	-- the "correct" way but might be different from R 
    .test.mode <<- "R" #"SAS" #"REFIT"	#"REVO"	#
  
    rxOptions(reportProgress = 0) # by default, don't report progress
}

.tearDown <- function()
{
    #options(.op.old)
    if (exists(".test.mode", where = ".GlobalEnv")) remove(".test.mode", pos = ".GlobalEnv")
    rxOptions(TRUE) # reset all options
}

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

#######################################################################
## local utility functions

compareLinModStepResults <- function(rlm, rxlm)
{
    ## anova
    rlm.anova <- rlm$anova
    rxlm.anova <- rxlm$anova
    
    #remove rows with 0 df as they can be dropped in any order
    zeroDf <- (rlm.anova$Df == 0)
    rlm.anova <- rlm.anova[!zeroDf, ]
    rxlm.anova <- rxlm.anova[!zeroDf, ]
    
    checkEquals(rlm.anova[-1], rxlm.anova[-1], check.attributes = FALSE)
    checkEquals(as.character(rlm.anova[-1, 1]), as.character(rxlm.anova[-1, 1]))
    
    ## coef
    rlm.coef <- coef(rlm)
    rxlm.coef <- coef(rxlm)
    
    ## workaround for PR#7842. terms.formula may have flipped interactions
    names.sort <- function(names.old)
    {
        names.new <- sapply(strsplit(names.old, ":", fixed = TRUE),
            function(x) paste(sort(x), collapse = ":"))
        names.new
    }

    names(rlm.coef) <- names.sort(names(rlm.coef))
    # convert "Revo" labels to "R" labels
    names.rxlm.coef <- gsub("=", "", names(rxlm.coef))
    names.rxlm.coef <- gsub(" for |[*]", ":", names.rxlm.coef)
    names(rxlm.coef) <- names.sort(names.rxlm.coef)
    
  checkEqualsNumeric(rlm.coef, rxlm.coef[names(rlm.coef)])
}

#######################################################################
# iris
"test.rxLinModStep.no.intercept" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    
    direction <- "both"
    form <- Sepal.Length ~ Sepal.Width + Petal.Length - 1
    scope <- list(
      lower = ~ Sepal.Width - 1,
        upper = ~ Sepal.Width + Petal.Length + Petal.Width + Species - 1)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Species = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
        
    checkEquals(pred.rxlm, pred.rxlm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.no.intercept.keepStepCoefs" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    
    direction <- "both"
    form <- Sepal.Length ~ Sepal.Width + Petal.Length - 1
    scope <- list(
      lower = ~ Sepal.Width - 1,
        upper = ~ Sepal.Width + Petal.Length + Petal.Width + Species - 1)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Species = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection with keepStepCoefs = FALSE
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test, keepStepCoefs = FALSE)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step.keepStepCoefsF <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## rxLinMod/variableSelection with keepStepCoefs = TRUE
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test, keepStepCoefs = TRUE)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step.keepStepCoefsT <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
  
  rxStepPlot(rxlm.step.keepStepCoefsT)
  dev.off()
  
    ## compare rxlm.step.keepStepCoefsF, rxlm.step.keepStepCoefsT, and rlm.step
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step.keepStepCoefsF)
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step.keepStepCoefsT)

    dont.compare <- c("stepCoefs", "params")
    checkTrue(all(sapply(setdiff(names(rxlm.step.keepStepCoefsF), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm.step.keepStepCoefsF[[nm]], rxlm.step.keepStepCoefsT[[nm]])))))
  
  ## compare the coefficients of the start and final models
  rlm.coef.start <- rlm.obj$coefficients
  rlm.coef.final <- rlm.step$coefficients
  
  stepCoefs <- rxlm.step.keepStepCoefsT$stepCoefs
  rxlm.stepCoefs.names <- gsub("=", "", rownames(stepCoefs))
  
  rxlm.stepCoefs.start <- stepCoefs[, 1]
  names(rxlm.stepCoefs.start) <- rxlm.stepCoefs.names
  
  rxlm.stepCoefs.final <- stepCoefs[, ncol(stepCoefs)]
  names(rxlm.stepCoefs.final) <- rxlm.stepCoefs.names
  
  rxlm.coef.final <- drop(rxlm.step.keepStepCoefsT$coefficients)
  names(rxlm.coef.final) <- gsub("=", "", names(rxlm.coef.final))
  
  checkEquals(rxlm.stepCoefs.start[names(rlm.coef.start)], rlm.coef.start)
  checkEquals(rxlm.stepCoefs.final[names(rxlm.coef.final)], rxlm.coef.final)
  
  ## compare the number of steps
  checkEquals(ncol(stepCoefs), nrow(rxlm.step.keepStepCoefsT$anova))
}

"test.rxLinModStep.interaction.num*num" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    
    direction <- "both"
    form <- Sepal.Length ~ Sepal.Width
    scope <- list(
      lower = ~ 1,
        upper = ~ Sepal.Width + Petal.Length * Petal.Width + Species)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Species = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
        
    checkEquals(pred.rxlm, pred.rxlm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.interaction.num*fac" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    
    direction <- "both"
    form <- Sepal.Length ~ Sepal.Width + Petal.Length
    scope <- list(
      lower = ~ Sepal.Width,
        upper = ~ Sepal.Width + Petal.Length + Petal.Width * Species)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Species = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") { 
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.interaction.fac*fac" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    dataset[["Petal.Width"]] <- cut(dataset[["Petal.Width"]], 
        unique(quantile(dataset[["Petal.Width"]], probs = seq(0, 1, length.out = 3))), 
        include.lowest = TRUE)
    
    direction <- "both"    
    form <- Sepal.Length ~ Sepal.Width + Petal.Length
    scope <- list(
      lower = ~ Sepal.Width,
        upper = ~ Sepal.Width + Petal.Length + Petal.Width * Species)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Petal.Width = contr.SAS, Species = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k,	test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}
    
"test.rxLinModStep.interaction.num*num*fac" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    
    direction <- "both"    
    form <- Sepal.Length ~ Sepal.Width
    scope <- list(
      lower = ~ 1,
        upper = ~ Sepal.Width + Petal.Length * Petal.Width * Species)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Species = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.interaction.num*fac*fac" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    dataset[["Petal.Width"]] <- cut(dataset[["Petal.Width"]], 
        unique(quantile(dataset[["Petal.Width"]], probs = seq(0, 1, length.out = 3))), 
        include.lowest = TRUE)
    
    direction <- "both"
    form <- Sepal.Length ~ Sepal.Width
    scope <- list(
      lower = ~ 1,
        upper = ~ Sepal.Width + Petal.Length * Petal.Width * Species)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Species = contr.SAS, Petal.Width = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

#######################################################################
# examples from package "stats"

"test.rxLinModStep.stats.drop1" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))    
    
    ## setup
    dataset <- swiss
    yvar <- "Fertility"
    xvar <- setdiff(colnames(dataset), yvar)
    
    direction <- "backward"    
    form <- paste(yvar, paste(xvar, collapse = " + "), sep = " ~ ")
    scope <- NULL
    steps <- 1
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
    
    ## lm/drop1
    rlm.obj <- lm(form, data = dataset)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.stats.add1" <- function()
{  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

    ## setup
    dataset <- swiss
    yvar <- "Fertility"
    xvar <- setdiff(colnames(dataset), yvar)
    
    direction <- "forward"    
    form <- paste(yvar, paste(xvar, collapse = " + "), sep = " ~ ")
    scope <- ~ I(Education^2) + .^2
    steps <- 1
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/add1
    ##NOTE: there seems to be a bug in step function which does not look for 'data' in parent
    #rlm.obj <- lm(form, data = dataset)    # does not work due to the bug in the step function
    rlm.obj <- lm(form, data = swiss)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step) 
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.stats.step" <- function()
{  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

    ## setup
    dataset <- swiss
    yvar <- "Fertility"
    xvar <- setdiff(colnames(dataset), yvar)
    
    direction <- "both"    
    form <- paste(yvar, paste(xvar, collapse = " + "), sep = " ~ ")
    scope <- NULL
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.stats.step.2" <- function()
{  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    ctl <- c(4.17, 5.58, 5.18, 6.11, 4.50, 4.61, 5.17, 4.53, 5.33, 5.14)
    trt <- c(4.81, 4.17, 4.41, 3.59, 5.87, 3.83, 6.03, 4.89, 4.32, 4.69)
    group <- gl(2, 10, 20, labels = c("Ctl", "Trt"))
    weight <- c(ctl, trt)
    dataset <- data.frame(weight = weight, group = group)
    yvar <- "weight"
    xvar <- setdiff(colnames(dataset), yvar)
    
    direction <- "both"    
    form <- paste(yvar, paste(xvar, collapse = " + "), sep = " ~ ")
    scope <- NULL
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

#######################################################################
# examples from package "MASS"

"test.rxLinModStep.MASS.stepAIC" <- function()
{  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    
    ## setup
    cpus1 <- cpus
    attach(cpus)
    for(v in names(cpus)[2:7])
      cpus1[[v]] <- cut(cpus[[v]], unique(quantile(cpus[[v]])), include.lowest = TRUE)
    detach()
    dataset <- cpus1[, 2:8]
    
    direction <- "both"    
    form <- log10(perf) ~ .
    # expand the dot in formula for rxLinMod
    form <- formula(terms(form, data = dataset))
    scope <- NULL
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/stepAIC
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(syct = contr.SAS, mmin = contr.SAS, mmax = contr.SAS, 
            cach = contr.SAS, chmin = contr.SAS, chmax = contr.SAS))
    rlm.step <- stepAIC(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.MASS.dropterm" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    
    ## setup
    dataset <- quine
    
    direction <- "backward"
    form <- log(Days + 2.5) ~ .^4
    # expand the dot in formula for rxLinMod
    form <- formula(terms(form, data = dataset))    #, simplify = TRUE
    scope <- NULL
    steps <- 1
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
    
    ## lm/dropterm
    rlm.obj <- lm(form, data = dataset,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    rlm.step <- stepAIC(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    #DEACTIVATED("RA-8417: the results of rxPredict/rxLinMod are different from predict/lm")
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)     
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    ## the order of the coefficients and related stuff might not be the same
    #dont.compare <- c("call", "params")
    #checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        #function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.MASS.dropterm.2" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    
    ## setup
    dataset <- quine
    
    direction <- "backward"   
    form <- log(Days + 2.5) ~ .^4
    # expand the dot in formula for rxLinMod
    form <- formula(terms(form, data = dataset))    #, simplify = TRUE    
    form <- update(form, . ~ . - Eth:Sex:Age:Lrn)    
    scope <- NULL
    steps <- 1
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
    
    rlm.obj <- lm(form, data = dataset,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    rlm.step <- stepAIC(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    #DEACTIVATED("RA-8417: the results of rxPredict/rxLinMod are different from predict/lm")
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    ## the order of the coefficients and related stuff might not be the same
    #dont.compare <- c("call", "params")
    #checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        #function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

#######################################################################

"test.rxLinModStep.interaction.fac^3" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    
    ## setup
    dataset <- quine
    
    direction <- "backward"
    form <- log(Days + 2.5) ~ .^3
    # expand the dot in formula for rxLinMod
    form <- formula(terms(form, data = dataset))    #, simplify = TRUE
    scope <- NULL
    steps <- 1
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
    
    ## lm/dropterm
    rlm.obj <- lm(form, data = dataset,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    #DEACTIVATED("RA-8417: the results of rxPredict/rxLinMod are different from predict/lm")
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)     
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    ## the order of the coefficients and related stuff might not be the same
    #dont.compare <- c("call", "params")
    #checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        #function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.interaction.fac^2" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    
    ## setup
    dataset <- quine
    
    direction <- "backward"
    form <- log(Days + 2.5) ~ .^2
    # expand the dot in formula for rxLinMod
    form <- formula(terms(form, data = dataset))    #, simplify = TRUE
    scope <- NULL
    steps <- 1
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
    
    ## lm/dropterm
    rlm.obj <- lm(form, data = dataset,
        contrasts = list(Eth = contr.SAS, Sex = contr.SAS, Age = contr.SAS, Lrn = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    ## the order of the coefficients and related stuff might not be the same
    #dont.compare <- c("call", "params")
    #checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        #function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

#######################################################################
## R bugs for step()

## PR 753 (step can't find variables)
"test.rxLinModStep.RBugs.PR753" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- data.frame(a = rnorm(10), b = rnorm(10), c = rnorm(10))
    
    direction <- "both"
    form <- a ~ 1
    scope <- list(
      lower = ~ 1,
        upper = ~ b + c)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

## model fitting with non-syntactic variable names in formula

#"test.rxLinModStep.RBugs.nonsyntactic.names" <- function()
#{  
    #DEACTIVATED("Pending implementation of RA-8115")
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
#
    ### setup
    #dataset <- swiss
    #names(dataset)[6] <- "Infant Mortality" #introduce a non-syntactic variable name
    #
    #direction <- "both"    
    #form <- Fertility ~ .
    #form <- formula(terms(form, data = dataset)) # expand the "." for ScaleR
    #
    #scope <- ~ I(Education^2) + .^2
    #steps <- 1000
    #scale <- 0
    #k <- 2
    #test <- NULL
    #trace <- 0  #1
        #
    ### lm/step
    #rlm.obj <- lm(form, data = dataset)
    #rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        #steps = steps, scale = scale, k = k, test = test, trace = trace)
    #
    ### rxLinMod/variableSelection
    #varsel <- list(#dropPolicy = "Competitive",
        #method = if (direction == "both") "stepwise" else direction, scope = scope,
    #maxSteps = steps, scale = scale, k = k, test = test)
    #rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        #verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    #
    ### compare
    #compareLinModStepResults(rlm.step, rxlm.step)
#}
    
#######################################################################
## transformFunc
"test.rxLinModStep.args.transformVars.NULL" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
    
    ## setup
    dataset <- kyphosis
    dataset$Age <- dataset$Age * 2
  dataset$Number <- as.integer(dataset$Number) 
  dataset$Start <- factor(dataset$Start, 
        levels = c(1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18))
    
    direction <- "both"
    form <- Age ~ Start + Kyphosis + Number
    scope <- NULL
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset, 
        contrasts = list(Kyphosis = contr.SAS, Start = contr.SAS))
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
  inFile <- file.path(rxGetOption("sampleDataDir"),"kyphosis") 
  myTransformFunc <- function(data) 
  { 
    data$Age <- data$Age*2. 
    data$Number <- as.integer(data$Number) 
    data$Start <- factor(data$Start, 
            levels = c(1, 2, 3, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18)) 
    return(data) 
  }    
    
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = inFile, variableSelection = varsel, 
        transformFunc = myTransformFunc, verbose = trace, 
        dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

#######################################################################

"test.rxLinModStep.step.stepwise" <- function()
{  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    inFile <- file.path(rxGetOption("unitTestDataDir"), "boston.Rdata")
    load(inFile)
    ##NOTE: there seems to be a bug in step function which does not look for 'data' in parent
    dataset <<- boston  # NOTE: step() looks for data only in the global env!
    yvar <- "crime"
    xvar <- setdiff(colnames(dataset), yvar)
    
    direction <- "both"    
    form <- paste(yvar, paste(xvar, collapse = " + "), sep = " ~ ")
    scope <- list(
        lower = ~ 1, 
        upper = formula( delete.response(terms(as.formula(form)))) ) 
    steps <- 1000
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.singularity" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    countiesXDF <- file.path( rxGetOption("unitTestDataDir"), "counties.xdf" )
    countiesDF <- rxDataStep( countiesXDF, removeMissings = TRUE )
    countiesDF$state <- as.factor( countiesDF$state )
    dataset <- countiesDF
    
    direction <- "both"
    form <- crime ~ republican 
    scope <- list(
        lower = ~ 1, 
        upper = ~ state + msa + pmsa + pop.density + pop + pop.change + 
            age6574 + age75 + college + income + farm + democrat + republican + 
            Perot + white + black + turnout ) 
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    op.old <- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
    #rlm.obj <- lm(form, data = dataset, contrasts = list(state = contr.SAS))   #coef labels are confusing
    rlm.obj <- lm(form, data = dataset)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    options(op.old)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = dataset)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.scope.wrong" <- function()
{  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    inFile <- file.path(rxGetOption("unitTestDataDir"), "boston.Rdata")
    load(inFile)
    ##NOTE: there seems to be a bug in step function which does not look for 'data' in parent
    dataset <<- boston  # NOTE: step() looks for data only in the global env!
    yvar <- "crime"
    xvar <- setdiff(colnames(dataset), yvar)
    
    direction <- "both"    
    form <- crime ~ value * town * black * residential  # NOTE: residential is not in the upper scope
    scope <- list(
        lower = ~ 1, 
        upper = ~ value * town * black ) 
    steps <- 1000
    scale <- 0
    k <- 2
    test <- "F"
    trace <- 0  #1
        
    ## lm/step
    rlm.obj <- lm(form, data = dataset)
    checkException( rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace) )
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    checkException( rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo") )
    
    ## compare
    #compareLinModStepResults(rlm.step, rxlm.step)
}

"test.rxLinModStep.no.valid.observations" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    dataset <- iris
    dataset$Petal.Width <- NA
    
    direction <- "both"
    form <- Sepal.Length ~ Sepal.Width
    scope <- list(
      lower = ~ 1,
        upper = ~ Sepal.Width + Petal.Length + Petal.Width + Species)
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    #rlm.obj <- lm(form, data = dataset, 
        #contrasts = list(Species = contr.SAS))
    #rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        #steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    checkException( rxlm.step <- rxLinMod(form, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R") )
    
    ### compare
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
}

#######################################################################
## weights

"test.rxLinModtep.claims.weights" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    claims.xdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claims.df <- rxDataStep(claims.xdf)
    dataset <- claims.df
    #summary(dataset)
    #rxOptions(numCoresToUse = 1)
        
    ################################### basic model
    (form.full <- cost ~ age + car.age + type)
    form.terms <- terms(form.full)
    weights <- "number"

    lm.full <- lm(form.full, data = dataset, weights = dataset[[weights]],
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))   
    
    rxlm.full <- rxLinMod(form.full, data = dataset, fweights = weights,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareLinModStepResults(lm.full, rxlm.full)    
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #1  #
    k <- 2
    test <- NULL
    trace <- 0  #1  #
        
    ## lm/step
    lm.obj <- lm(form.base, data = dataset, weights = dataset[[weights]],
        contrasts = list(age = contr.SAS, car.age = contr.SAS, type = contr.SAS))
    lm.step <- step(lm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form.base, data = dataset, variableSelection = varsel, 
    fweights = weights,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
    
    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(lm.step, rxlm.step)
    
    ################################### predict
    #rxOptions(numCoresToUse = 1)
    rxlm.obj <- rxLinMod(form = rxlm.step$formula, data = dataset, fweights = weights,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlm.obj <- rxPredict(rxlm.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = dataset, checkFactorLevels = FALSE)
    pred.lm.step <- predict(lm.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxlm.obj, pred.rxlm.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.lm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.lm.step, pred.rxlm.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm.obj[[nm]], rxlm.step[[nm]])))))
}

#######################################################################
## tests for various data sources

"test.rxLinModStep.datasource.xdf" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    sampleDataDir <- rxGetOption("sampleDataDir")
    claimsFileName <- file.path( sampleDataDir, "claims.xdf" )
    claimsXDF <- RxXdfData(claimsFileName)
    claimsDF <- rxDataStep( claimsXDF )
    
    datasetXDF <- claimsXDF
    datasetDF <- claimsDF
    
    direction <- "both"
    form <- cost ~ age 
    scope <- list(
        lower = ~ 1, 
        upper = ~ age + car.age + type + number ) 
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    op.old <- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
    #rlm.obj <- lm(form, data = datasetDF, contrasts = list(state = contr.SAS))   #coef labels are confusing
    rlm.obj <- lm(form, data = datasetDF)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    options(op.old)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = datasetXDF, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = datasetXDF,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = datasetDF, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = datasetDF, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = datasetDF)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.datasource.txt" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    sampleDataDir <- rxGetOption("sampleDataDir")
    claimsFileName <- file.path( sampleDataDir, "claims.txt" )
    claimsTXT <- RxTextData(claimsFileName, stringsAsFactors = TRUE)
    claimsDF <- rxImport( claimsTXT, rowsPerRead = 20, stringsAsFactors = TRUE )
    
    datasetTXT <- claimsTXT
    datasetDF <- claimsDF
    
    direction <- "both"
    form <- cost ~ age 
    scope <- list(
        lower = ~ 1, 
        upper = ~ age + car.age + type + number ) 
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    op.old <- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
    #rlm.obj <- lm(form, data = datasetDF, contrasts = list(state = contr.SAS))   #coef labels are confusing
    rlm.obj <- lm(form, data = datasetDF)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    options(op.old)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = datasetTXT, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = datasetTXT,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = datasetDF, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = datasetDF, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = datasetDF)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
        
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.datasource.sas" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    sampleDataDir <- rxGetOption("sampleDataDir")
    claimsFileName <- file.path( sampleDataDir, "claims.sas7bdat" )
    claimsSAS <- RxSasData(claimsFileName, stringsAsFactors = TRUE)
    claimsDF <- rxImport( claimsSAS, rowsPerRead = 20, stringsAsFactors = TRUE )
    
    datasetSAS <- claimsSAS
    datasetDF <- claimsDF
    
    direction <- "both"
    form <- cost ~ age 
    scope <- list(
        lower = ~ 1, 
        upper = ~ age + car_age + type + number ) # car_age instead of car.age
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    op.old <- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
    #rlm.obj <- lm(form, data = datasetDF, contrasts = list(state = contr.SAS))   #coef labels are confusing
    rlm.obj <- lm(form, data = datasetDF)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    options(op.old)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = datasetSAS, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = datasetSAS,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = datasetDF, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = datasetDF, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = datasetDF)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
        
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.datasource.spss" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    ## setup
    sampleDataDir <- rxGetOption("sampleDataDir")
    claimsFileName <- file.path( sampleDataDir, "claims.sav" )
    claimsSPSS <- RxSpssData(claimsFileName, stringsAsFactors = TRUE)
    claimsDF <- rxImport( claimsSPSS, rowsPerRead = 20, stringsAsFactors = TRUE )
    
    datasetSPSS <- claimsSPSS
    datasetDF <- claimsDF
    
    direction <- "both"
    form <- cost ~ age 
    scope <- list(
        lower = ~ 1, 
        upper = ~ age + car_age + type + number ) # car_age instead of car.age
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    op.old <- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
    #rlm.obj <- lm(form, data = datasetDF, contrasts = list(state = contr.SAS))   #coef labels are confusing
    rlm.obj <- lm(form, data = datasetDF)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    options(op.old)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = datasetSPSS, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = datasetSPSS,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = datasetDF, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = datasetDF, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = datasetDF)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
    
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}

"test.rxLinModStep.datasource.odbc" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
  ### Check for existing SQLite configuration - DEACTIVATE test if SQLite not found.
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite")),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
    ## setup
    sampleDataDir <- rxGetOption("sampleDataDir")
    claimsFileName <- file.path( sampleDataDir, "claims.sqlite" )
    connectionString <- paste("Driver=SQLite3 ODBC Driver;Database=", claimsFileName, sep = "")
    claimsODBC <- RxOdbcData(sqlQuery = "SELECT * FROM claims",
             connectionString = connectionString, stringsAsFactors = TRUE)
    claimsDF <- rxImport( claimsODBC, stringsAsFactors = TRUE ) #RA-8554: rowsPerRead = 20, 
    
    datasetODBC <- claimsODBC
    datasetDF <- claimsDF
    
    direction <- "both"
    form <- cost ~ age 
    scope <- list(
        lower = ~ 1, 
        upper = ~ age + car_age + type + number ) # car_age instead of car.age
    steps <- 1000
    scale <- 0
    k <- 2
    test <- NULL
    trace <- 0  #1
        
    ## lm/step
    op.old <- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
    #rlm.obj <- lm(form, data = datasetDF, contrasts = list(state = contr.SAS))   #coef labels are confusing
    rlm.obj <- lm(form, data = datasetDF)
    rlm.step <- step(rlm.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    options(op.old)
    
    ## rxLinMod/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "REFIT") varsel <- c(varsel, refitEachStep = TRUE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlm.step <- rxLinMod(form, data = datasetODBC, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    ## compare
    ##TODO: need to come up with a way to compare results for interactions
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") compareLinModStepResults(rlm.step, rxlm.step)
    
    ## predict
    #rxOptions(numCoresToUse = 1)
    rxlm <- rxLinMod(form = rxlm.step$formula, data = datasetODBC,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
    
    pred.rxlm <- rxPredict(rxlm, data = datasetDF, checkFactorLevels = FALSE)
    pred.rxlm.step <- rxPredict(rxlm.step, data = datasetDF, checkFactorLevels = FALSE)
    pred.rlm.step <- predict(rlm.step, newdata = datasetDF)
    
    checkEquals(pred.rxlm, pred.rxlm.step)    
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO" || .test.mode == "REFIT") {
        checkEqualsNumeric(pred.rlm.step, pred.rxlm.step[[1]])
        checkEqualsNumeric(pred.rlm.step, pred.rxlm[[1]])
    }
        
    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlm), dont.compare), 
        function(nm) isTRUE(all.equal(rxlm[[nm]], rxlm.step[[nm]])))))
}
