# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLogit
#
############################################################################################

unitTestDataDir <- rxGetOption("unitTestDataDir")
# Instead use R's tempdir()
toutDir <- tempdir()
testOutputDir <- file.path(toutDir, "testOutput")
sampleDataDir <- rxGetOption("sampleDataDir")

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

.setUp <- function()
{
  if (file.exists(testOutputDir))
  {
    unlink(testOutputDir,recursive=TRUE)
}
  dir.create(testOutputDir)
}

.tearDown <- function()
{
  unlink(testOutputDir,recursive=TRUE)
}

#===============================================================
# AirlineArrDelay
#

test.rxLogit.airlineArrOnTimeHasMissingData <- function()
{
    # Activated when Bug RA-264 was fixed
    require(stats)

    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineDemoSmall1st10kOf6.csv") # Has some missing data
    outFile <- file.path(testOutputDir, "ADS.xdf")

    rxDataStep(inData = RxTextData(file = inFile, rowsPerRead=1000,
            colInfo=list(DayOfWeek=list(type="factor")), missingValueString = "M"), 
            outFile = outFile, overwrite = TRUE)
  airlineDF <- rxReadXdf(file = outFile)
    # Arrivals before or within 1 minute of stated arrival time
    airline2DF <- cbind(airlineDF, ArrOnTime = (airlineDF$ArrDelay < 2))
  x1<-summary(airline2DF)
    try(na.fail(airline2DF)) # This will display an error string indicating there's some missing data.

    result2 <- glm(ArrOnTime ~ CRSDepTime, data=airline2DF, family = binomial())
    result1 <- rxLogit(ArrOnTime ~ CRSDepTime, data=airline2DF, blocksPerRead=100)

    # Verify using coefficients
    coefRxLogit <- coef(result1)
    coefGlm <- coef(result2)

    checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.00001)
    checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.0001)
}

test.rxLogit.airlineArrOnTimeNoMissingData <- function()
{
    require(stats)

    inFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemo1kNoMissing.csv") # ~1000 rows from AirlineDemoSmall.csv with all missing data removed

  outFile <- ".rxTempADS.xdf"
  if (file.exists(outFile)) file.remove(outFile)

    rxTextToXdf(inFile, outFile, rowsPerRead=1000,
                colInfo=list(DayOfWeek=list(type="factor")))
  airlineDF <- rxReadXdf(file = outFile)
    # Arrivals before or within 1 minute of stated arrival time
    airline2DF <- cbind(airlineDF, ArrOnTime = (airlineDF$ArrDelay < 2))
  x1<-summary(airline2DF)
    try(na.fail(airline2DF))

    result1 <- rxLogit(ArrOnTime ~ CRSDepTime, data=airline2DF, verbose=2)
    result2 <- glm(ArrOnTime ~ CRSDepTime, data=airline2DF, family = binomial())

    # Verify using coefficients
    coefRxLogit <- coef(result1)
    coefGlm <- coef(result2)

    checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.00001)
    checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.0001)

    file.remove(outFile)
}

#-----------------------------------------
# This test examines rxLogit() behavior when the dependent variable has an illegal datatype (other than TRUE/FALSE or 1/0)
#
test.rxLogit.dependentVariableNumeric.airline <- function()
{
    if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 74579: Unit test test.rxDataStep.args.append.differentNumCols fails to return ERROR in output when running inside SPEES")
    
    # rxLogit requires a variable that takes the values 0 and 1.
    # Bug RA-440 has been fixed to provide a reasonable error message

     require(stats)

    # Test:  Dependent variable is non-boolean numeric
    # Import airline data - with no missing information
    inFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemo1kNoMissing.csv") # ~1000 rows from AirlineDemoSmall.csv with all missing data removed
    outFile <- tempfile(pattern = ".rxTempLDM", fileext = ".xdf")
  if (file.exists(outFile)) file.remove(outFile)
    rxTextToXdf(inFile, outFile, rowsPerRead=1000,
                colInfo=list(DayOfWeek=list(type="factor")), reportProgress=FALSE)
    airlineDF <- rxReadXdf(file = outFile, reportProgress=FALSE)
    try(na.fail(airlineDF)) # Display an error if there's any missing data.

    # Run rxLogit with an inappropriate dependent variable
    oi <- capture.output(res <- try(result1 <- rxLogit(CRSDepTime ~ ArrDelay, data=airlineDF, blocksPerRead=100, reportProgress=FALSE)))

    #Bug 76414: capture.output fails to capture entire output when run inside SPEES
    if(!(  RevoScaleR:::rxInDatabaseHostPlatform())) checkEquals(oi[1], "The dependent variable in a logistic regression must be a binary variable," )

    # return TRUE if error is encountered
    checkTrue(is(res, "try-error"))

    file.remove(outFile)
}

test.rxLogit.dependentVariableCharacterYesNo.mortDefault <- function()
{
    require(stats)

    # Test 2: Dependent variable is character-valued (Yes/No)
    # Import mortDefaultTiny CSV data - with no missing information
    inFile <- file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.csv") # ~25 rows 
    outFile <-  tempfile(pattern = ".rxTempLogit1", fileext = ".xdf")
  if (file.exists(outFile)) file.remove(outFile)

    rxTextToXdf(inFile, outFile, rowsPerRead=1000)
    mortDefaultDF <- rxReadXdf(file = outFile, stringsAsFactors=TRUE)
    try(na.fail(mortDefaultDF)) # Display an error if there's any missing data.
    result2 <- glm(default ~ creditScore + yearsEmploy, data=mortDefaultDF, family = binomial())
    result1 <- rxLogit(default ~ creditScore + yearsEmploy, data=mortDefaultDF)

    # Verify using coefficients
    coefRxLogit <- coef(result1)
    coefGlm <- coef(result2)

    checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.00001)
    checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.00001)
    checkEqualsNumeric(coefRxLogit[3], coefGlm[3], tolerance=0.00001)

    file.remove(outFile)
}

test.rxLogit.ageParityEducSpontInduc.infert <- function()
{
  require(stats)
  # Use in-built data from datasets::infert 
  result1 <- rxLogit(case ~ age+parity+education+spontaneous+induced, data=infert)
  result2 <- glm(case ~ age+parity+education+spontaneous+induced, data=infert, contrasts = list(education="contr.SAS"), family = binomial())

  # Verify using summary 
  x1 <- summary(result1)
  checkEqualsNumeric(x1$rank, 7)
  checkEqualsNumeric(x1$coefficients[1,1], -2.55244, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[2,1], 0.039582, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[3,1], -0.82828, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[4,1], 1.40321, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[5,1], 0.35896, tolerance=0.00001)
  checkTrue(is.na(x1$coefficients[6,1]))
  checkEqualsNumeric(x1$coefficients[7,1], 2.04591, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[8,1], 1.28876, tolerance=0.00001)

  # Verify using coefficients
  coefRxLogit <- coef(result1)
  coefGlm <- coef(result2)

  checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.0001)
  checkEqualsNumeric(coefRxLogit[3], coefGlm[3], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[4], coefGlm[4], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[5], coefGlm[5], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[7], coefGlm[6], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[8], coefGlm[7], tolerance=0.00001)
}

test.rxLogit.spontaneousInduced.infert <- function()
{
  require(stats)
  # Use in-built data from datasets::infert 
  result1 <- rxLogit(case ~ spontaneous+induced, data=infert, covCoef=TRUE)
  result2 <- glm(case ~ spontaneous+induced, data=infert, family = binomial())

  # Verify using summary 
  x1 <- summary(result1)
  checkEqualsNumeric(x1$rank, 3)
  checkEqualsNumeric(x1$coefficients[1], -1.70786, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[2], 1.19721, tolerance=0.00001)
  checkEqualsNumeric(x1$coefficients[3], 0.41813, tolerance=0.00001)
  checkEqualsNumeric(diag(rxCovCoef(result1)), c(0.071668, 0.044793, 0.042283), tolerance=0.00001)
  
  x2 <- summary(result2)
  checkEqualsNumeric(x2$coefficients[1,1], -1.70786, tolerance=0.00001)
  checkEqualsNumeric(x2$coefficients[2,1], 1.19721, tolerance=0.00001)
  checkEqualsNumeric(x2$coefficients[3,1], 0.41813, tolerance=0.00001)

  # Verify using coefficients
  coefRxLogit <- coef(result1)
  coefGlm <- coef(result2)
  
  checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.00001)
  checkEqualsNumeric(coefRxLogit[3], coefGlm[3], tolerance=0.00001)
}

# Use in-built data from datasets::infert and run rxLogit with different coefTolerances
test.rxLogit.args.coeffTolerance.spontInduc.infert <- function()
{
  require(stats)

  # coeffTolerance = 0.0001
  result1 <- rxLogit(case ~ spontaneous+induced, data=infert, coeffTolerance = 0.0001)
  x1 <- summary(result1)
  checkEqualsNumeric(x1$coefficients[1], -1.7079, tolerance=0.0001)
  checkEqualsNumeric(x1$coefficients[2], 1.1972, tolerance=0.0001)
  checkEqualsNumeric(x1$coefficients[3], 0.4181, tolerance=0.0001)

  # coeffTolerance = 0.001
  result2 <- rxLogit(case ~ spontaneous+induced, data=infert, coeffTolerance = 0.001)
  x2 <- summary(result2)
  checkEqualsNumeric(x2$coefficients[1], -1.708, tolerance=0.001)
  checkEqualsNumeric(x2$coefficients[2], 1.197, tolerance=0.001)
  checkEqualsNumeric(x2$coefficients[3], 0.418, tolerance=0.001)

  # coeffTolerance = 0.01
  result3 <- rxLogit(case ~ spontaneous+induced, data=infert, coeffTolerance = 0.01)
  x3 <- summary(result3)
  checkEqualsNumeric(x3$coefficients[1], -1.71, tolerance=0.01)
  checkEqualsNumeric(x3$coefficients[2], 1.20, tolerance=0.01)
  checkEqualsNumeric(x3$coefficients[3], 0.42, tolerance=0.01)
}


#==============================================================
#
# Examples from ?esoph 
#

#
# Compare various outputs between rxLogit() and glm(binomial) regression runs.
# Outputs compared: coefficients, coefficient standard errors, degrees of freedom, residual deviance 
# Dataset: esoph
# 
test.rxLogit.compareOutputs.esoph <- function()
{
  require(stats)
  # Use in-built data from datasets::esoph 
  esoph2DF<-cbind(esoph, esophCase=as.numeric(esoph$ncases > 0) )

  # This test used to have much more complex model that was highly singular and unstable
  # that neither glm or rxLogit could accurately estimate (esophCase ~ agegp + tobgp * alcgp)
  # resulting in two bugs (RA-433 and RA-437 that have been closed as "not bugs"
  # Test 1: Use two factor variables (tobgp, alcgp). 
  result1 <- rxLogit(esophCase ~ tobgp + alcgp, data = esoph2DF, maxIterations=50, verbose=1)
  result2 <- glm(esophCase ~ tobgp + alcgp, data = esoph2DF, 
                 contrasts = list(tobgp="contr.SAS", alcgp="contr.SAS"), family = binomial())    
  keep <- which(!is.na(result1$coefficients))

  # Compare coefficients

  checkEquals(as.vector(coef(result1)[keep]),as.vector(coef(result2)), tol=.0001)

  # Compare coefficient standard errors
  # (GLM) Coefficient Std Error is inside summaryGlm$coefficients[,2]
  summaryGlm <- summary(result2)
  checkEqualsNumeric(result1$coef.std.error[keep], summaryGlm$coefficients[,2], tol=.0001)

  # Compare degrees of freedom
  checkEqualsNumeric(result1$df[2], result2$df.residual)
  checkEqualsNumeric(result2$df.residual, 81)
}

### Test the 'missing values' argument, 'removeMissings=' ###
test.rxLogit.na <- function()
{
    # Bug RA-513 fixed.
    # Create a copy of the 'infert' dataset
    infert.na <- infert
    # Randomly add missing values to the dataset
    infert.na[17,1] <- NA
    infert.na[16,2] <- NA
    infert.na[7,4] <- NA
    infert.na[16,5] <- NA
    infert.na[9,6] <- NA
    infert.na[12,7] <- NA

    infert.glm.NA <- glm(case ~ age+parity+spontaneous+induced, data=infert.na, na.action=na.omit, family = binomial())
    infert.rxLogit.NA <- rxLogit(case ~ age+parity+spontaneous+induced, data=infert.na, covCoef=TRUE)

    # Check that coefficient estimates are the same
    mysum.glm <- summary(infert.glm.NA)
    mycoef.glm <- mysum.glm$coefficients[,1]
    names(mycoef.glm) <- NULL
    myrxLog.sum <- summary(infert.rxLogit.NA)
    checkEquals(mycoef.glm,as.vector(myrxLog.sum$coefficients[,1]),tol=.001)

    # Check covariance matrices for equality
    mysum.glm$cov.values<- c(mysum.glm$cov.unscaled[1,1],mysum.glm$cov.unscaled[2,2])
    checkEqualsNumeric(as.vector(mysum.glm$cov.values),diag(infert.rxLogit.NA$covCoef)[1:2],tol=.001)

    # Check for equal degrees of freedom
    checkEquals(mysum.glm$df.residual,myrxLog.sum$df[2])
}

test.rxLogit.Rcallback.infert <- function()
{
    xform <- function(data)
    {
      data$myVar <- data$induced + 1
      return(data)
    }

    res <- try(infert.rxLogit <- rxLogit(case ~ spontaneous + myVar, data=infert, transformFunc=xform, transformVars=c("induced")))
        
    # return TRUE for no errors encountered during execution
    checkTrue(!is(res, "try-error"))

    # Bug RA-507 is fixed
    infert.glm <- glm(case ~ spontaneous + I(induced+1), data=infert, family = binomial())
    # Variable names are different, so use as.vector to strip attributes and just compare values
    checkTrue(all.equal(as.vector(coef(infert.glm)),as.vector(coef(infert.rxLogit)))) 
}

test.rxLogit.kyphosis <- function()
{
    # Compare results using a data frame and an .xdf file
    require(rpart)
    kyphosisXdf <- file.path(rxGetOption("sampleDataDir"), "kyphosis")
    logitObj1 <- rxLogit(Kyphosis ~ Age + Number + Start, data=kyphosis)
    logitObj2 <- rxLogit(Kyphosis ~ Age + Number + Start, data=kyphosisXdf)
    checkEquals(coef(logitObj1), coef(logitObj2))
}   

###
# argument stress tests
###

"test.rxLogit.args.blocksPerRead" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    form <- ybinint ~ xnum2miss
    zLogitBlocksDefault <- rxLogit(form, data=test100r5b, blocksPerRead=-1)
    zLogitBlocks1 <- rxLogit(form, data=test100r5b, blocksPerRead=1)
    nocomp <- which(names(zLogitBlocks1) %in% c("call","params"))
    checkTrue(all.equal(zLogitBlocksDefault[-nocomp], zLogitBlocks1[-nocomp], check.attributes=FALSE))
    zLogitBlocks5 <- rxLogit(form, data=test100r5b, blocksPerRead=5)
    checkTrue(all.equal(zLogitBlocksDefault[-nocomp], zLogitBlocks5[-nocomp], check.attributes=FALSE))
    
    o1 <- capture.output(
        zLogitBlocks5a <- rxLogit(form, data=test100r5b, blocksPerRead=5)
        )
    o2 <- capture.output(
        zLogitBlocks5a <- rxLogit(form, data=RxXdfData(test100r5b), blocksPerRead=5)
        )
    o3 <- capture.output(
        zLogitBlocks5a <- rxLogit(form, data=RxXdfData(test100r5b, blocksPerRead=5))
        )   
    o4 <- capture.output(
        zLogitBlocks5a <- rxLogit(form, data=RxXdfData(test100r5b, blocksPerRead=3), blocksPerRead = 5)
        )       
    
    checkEquals(length(o1), length(o2))
    checkEquals(length(o1), length(o3))
    checkEquals(length(o1), length(o4))
     
}

"test.rxLogit.args.covariance.covCoef.covData" <- function()
{
    .isNumericMatrix <- function(x) is.matrix(x) && is.numeric(x)
    
    form <- case ~ age + parity + education + spontaneous + induced
    
    # test various combinations of covCoef and covData
    z <- rxLogit(form, data=infert, covCoef=TRUE, covData=FALSE)
    checkTrue(.isNumericMatrix(z$covCoef))
    checkTrue(is.null(z$covData))
 
    z <- rxLogit(form, data=infert, covCoef=TRUE, covData=TRUE)
    checkTrue(.isNumericMatrix(z$covCoef))
    checkTrue(.isNumericMatrix(z$covData))

    z <- rxLogit(form, data=infert, covCoef=FALSE, covData=FALSE)
    checkTrue(is.null(z$covCoef))
    checkTrue(is.null(z$covData))
       
    z <- rxLogit(form, data=infert, covCoef=FALSE, covData=TRUE)
    checkTrue(is.null(z$covCoef))
    checkTrue(.isNumericMatrix(z$covData))

    # test covariance override of unspecified covCoef and covData values
    # 'covariance' now defunct
    #z <- rxLogit(form, data=infert, covCoef=TRUE, covData=TRUE, covariance=FALSE)
    #checkTrue(.isNumericMatrix(z$covCoef))
    #checkTrue(.isNumericMatrix(z$covData))
#
    #z <- rxLogit(form, data=infert, covCoef=FALSE, covData=FALSE, covariance=TRUE)
    #checkTrue(is.null(z$covCoef))
    #checkTrue(is.null(z$covData))

    #z <- rxLogit(form, data=infert, covCoef=FALSE, covariance=TRUE)
    #checkTrue(is.null(z$covCoef))
    #checkTrue(.isNumericMatrix(z$covData))
#
    #z <- rxLogit(form, data=infert, covData=FALSE, covariance=TRUE)
    #checkTrue(.isNumericMatrix(z$covCoef))
    #checkTrue(is.null(z$covData))
#
    #z <- rxLogit(form, data=infert, covariance=TRUE)
    #checkTrue(.isNumericMatrix(z$covCoef))
    #checkTrue(.isNumericMatrix(z$covData))
#
    #z <- rxLogit(form, data=infert, covariance=FALSE)
    #checkTrue(is.null(z$covCoef))
    #checkTrue(is.null(z$covData))
    
    z <- rxLinMod(form, data=infert)
    checkTrue(is.null(z$covCoef))
    checkTrue(is.null(z$covData))
}


"test.rxLogit.args.formula.noIntercept" <- function()
{ 
    glmInfert <- glm(case ~ education -1, data=infert, family = binomial())
    rxLogitInfert <- rxLogit(case ~ education - 1, data=infert)
    glmCoef <- coef(glmInfert)
    rxLogitCoef <- coef(rxLogitInfert)
    
    checkEquals( as.numeric(glmCoef[1]), as.numeric(rxLogitCoef[1]))
    checkEquals( as.numeric(glmCoef[2]), as.numeric(rxLogitCoef[2]))
    checkEquals( as.numeric(glmCoef[3]), as.numeric(rxLogitCoef[3]))
}
    
"test.rxLogit.args.cube" <- function()
{
    # Use the cube option in rxLogit; compare results with glm
    glmInfert <- glm(case ~ education, data=infert, family = binomial())
    rxLogitInfert <- rxLogit(case ~ education, data=infert, cube=TRUE)
    glmCoef <- coef(glmInfert)
    rxLogitCoef <- coef(rxLogitInfert)
    
    # Since rxLogit drops intercept
    checkEquals( as.numeric(glmCoef[1] + glmCoef[2]), as.numeric(rxLogitCoef[2]))
    checkEquals( as.numeric(glmCoef[1] + glmCoef[3]), as.numeric(rxLogitCoef[3]))
    
    infertMiss <- infert
    infertMiss$case[50] <- NA
    infertMiss$education[80] <- NA
    infertMiss$case[90] <- NA
    infertMiss$education[22] <- NA

    # Repeat after introducing missings
    glmInfertMiss <- glm(case ~ education, data=infertMiss, family = binomial())
    rxLogitInfertMiss <- rxLogit(case ~ education, data=infertMiss, cube=TRUE)
    glmCoef <- coef(glmInfertMiss)
    rxLogitCoef <- coef(rxLogitInfertMiss)
    
    # Since rxLogit drops intercept
    checkEquals( as.numeric(glmCoef[1] + glmCoef[2]), as.numeric(rxLogitCoef[2]))
    checkEquals( as.numeric(glmCoef[1] + glmCoef[3]), as.numeric(rxLogitCoef[3]))

    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
    airlineDF <- rxReadXdf(file = inFile)
    airline2DF<-cbind(airlineDF, ArrOnTime=((airlineDF$ArrDelay > -2) & (airlineDF$ArrDelay < 2)))
    zLogitCubeTrue <- rxLogit(ArrOnTime ~ DayOfWeek + CRSDepTime , data=airline2DF, 
        maxIterations=5, blocksPerRead=100, cube=TRUE, cubePredictions=TRUE)
    checkTrue(all(names(zLogitCubeTrue$countDF) == c("DayOfWeek", "ArrOnTime", "Counts")))

    zLogitCubeFalse <- rxLogit(ArrOnTime ~ DayOfWeek + CRSDepTime , data=airline2DF, 
        maxIterations=5, blocksPerRead=100, cube=FALSE)
    checkTrue(is.null(zLogitCubeFalse$countDF))
    
}    

"test.rxLogit.args.cube.counts" <- function()
{
    # Regression test for RA-1920 for Counts in rxLogit
    # Create a simple data set with 50 values each of a factor set to -1 and 1
    set.seed(100)
    x <- rep(c(-1,1), times=50)
    ytemp <- x + rnorm(100)
    y <- ytemp > 0
    myData <- data.frame(xfac=factor(x),x, y)
    logRes <- rxLogit(y~xfac, data=myData, cube=TRUE, verbose=1)
    checkEquals(logRes$countDF$Counts, c(50,50), check.attributes=FALSE)
    glmRes <- glm(y ~xfac + 0, data=myData, family = binomial())
    checkEquals(coef(logRes), coef(glmRes), check.attributes=FALSE)
}

"test.rxLogit.args.cubePredictions" <- function()
{

    set.seed(100)
    x <- rep(c(-1,1), times=50)
    x1 <- rnorm(100)
    ytemp <- x + 2*x1 + rnorm(100)
    y <- ytemp > 0
    myData <- data.frame(xfac=factor(x),x, x1, y)
    logResTrue <- rxLogit(y~xfac+x1, data=myData, cube=TRUE, cubePredictions=TRUE)
    checkTrue(all(names(logResTrue$countDF) == c("xfac", "y", "Counts")))
    
    logResFalse <- rxLogit(y~xfac+x1, data=myData, cube=TRUE, cubePredictions=FALSE)
    checkTrue(all(names(logResFalse$countDF) == c("xfac", "Counts")))
    checkTrue(is.null(logResFalse$countDF$y))
}

"test.rxLogit.args.data" <- function()
{
    form <- case ~ age + parity + education + spontaneous + induced
   
    XDF  <- tempfile(pattern = ".rxTempLogit", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    
    rxDataStep(infert, XDF, rowsPerRead = 50)
    
    zLogitDF <- rxLogit(form, data=infert)
    zLogitXDF <- rxLogit(form, data=XDF, verbose=1)
    nocomp <- which(names(zLogitXDF) %in% c("call","params"))
    checkTrue(all.equal(zLogitDF[-nocomp], zLogitXDF[-nocomp], check.attributes=FALSE))
    file.remove(XDF)
}

"test.rxLogit.args.formula" <- function()
{
    form <- case ~ age + parity + education + spontaneous + induced
    formStr <- deparse(form)
    zLogitFormula <- rxLogit(form, data=infert)
    zLogitString <- rxLogit(formStr, data=infert)
    nocomp <- which(names(zLogitString) %in% c("call","params","formula"))
    checkTrue(all.equal(zLogitFormula[-nocomp], zLogitString[-nocomp], check.attributes=FALSE))
    
}

"test.rxLogit.args.fweights" <- function()
{
    fwts <- c((1:5), (1:5))
    sex <- c("Male","Male","Female", "Male", "Female", "Male", "Female", "Male", "Female", "Male") 
    age <- c(NA,20,12,15, NA, 17, 16, 13, 19, 12) 
    score <- (1:10)+0.1 
    myDF1 <- data.frame(sex = sex, age=age, score = score, weightVar = fwts) 
    myDF2 <- data.frame(sex = rep(sex,fwts), age = rep(age, fwts), score = rep(score, fwts)) 
    fweightsObj <- rxLogit(age > 15 ~ sex + score, data = myDF1, fweights = "weightVar") 
    baseObj <- rxLogit(age > 15 ~ sex + score, data = myDF2) 
    fweightsObj$call <- NULL
    baseObj$call <- NULL
    fweightsObj$params <- NULL
    baseObj$params <- NULL
    # Number of missings will be unweighted for fweightsObj
    fweightsObj$nMissingObs <- NULL
    baseObj$nMissingObs <- NULL 
    checkEquals(fweightsObj, baseObj)     
}

"test.rxLogit.args.fweights.pweights" <- function()
{
    # create frequency weighting vector whose length is equal to the
    # number of rows inour test data frame. select weight randomly from 
    # the set of integers on the interval [1,4]
    set.seed(10)
    DF <- infert
    DF$fwts <- sample(1:4, nrow(DF), replace = TRUE)
    
    # create new data frame where the rows are replictaed explicitly according
    # to the number frequency weighting vector
    DF2 <- as.data.frame(lapply(DF, rep, times = DF$fwts))
    DF2$fwts <- NULL

    # define common formula    
    form <- case ~ age + parity + education + spontaneous + induced

    # calculate logisitc regression coefficients for the two cases and compare
  fweightsLogit <- rxLogit(form, data = DF, fweights = "fwts")
  noWeightsLogit <-rxLogit(form, data = DF2)
    z1 <- coef(fweightsLogit)
    z2 <- coef(noWeightsLogit)
    checkTrue(all.equal(z1, z2)) 
    checkEquals(fweightsLogit$df, noWeightsLogit$df)
  checkEquals(fweightsLogit$coef.std.error, noWeightsLogit$coef.std.error)
  checkEquals(fweightsLogit$fstatistics, noWeightsLogit$fstatistics)
  
  
}

"test.rxLogit.args.maxIterations" <- function()
{
    # Compare maxIterations parameter in rxLogit vs maxit parameter in glm.control.   
 
    # Activated when Bug RA-428 was fixed
    require(stats)

    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
    airlineDF <- rxReadXdf(file = inFile)
    airline2DF<-cbind(airlineDF, ArrOnTime=((airlineDF$ArrDelay > -2) & (airlineDF$ArrDelay < 2)))
    x1<-summary(airline2DF)
    try(na.fail(airline2DF)) # This passes clean (without displaying any error) indicating there's no missing data.

    # With no limitation on max iterations  
    result2 <- glm(ArrOnTime ~ CRSDepTime + DayOfWeek, data=airline2DF, 
        contrasts = list(DayOfWeek="contr.SAS"), family = binomial())
    result1 <- rxLogit(ArrOnTime ~ CRSDepTime + DayOfWeek, data=airline2DF, blocksPerRead=100)

    # Verify by comparing coefficients
    coefRxLogit <- coef(result1)
    coefGlm <- coef(result2)

    checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.0001)
    checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.0001)
    
    # Now limit the max iterations. There should be a match between the coefficients but with a higher error
    # tolerance.
    glmControl <- glm.control(maxit=5)
    result2 <- glm(ArrOnTime ~ CRSDepTime + DayOfWeek, data=airline2DF, 
        contrasts = list(DayOfWeek="contr.SAS"), control = glmControl, family = binomial())
    result1 <- rxLogit(ArrOnTime ~ CRSDepTime + DayOfWeek, data=airline2DF, maxIterations=5, blocksPerRead=100)

    coefRxLogit <- coef(result1)
    coefGlm <- coef(result2)

    checkEqualsNumeric(coefRxLogit[1], coefGlm[1], tolerance=0.1)
    checkEqualsNumeric(coefRxLogit[2], coefGlm[2], tolerance=0.1)
}

"test.rxLogit.args.formula.expression" <- function()
{
  inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineArrDelayLT5s.xdf")
  logit1 <- rxLogit(ArrDelay > 0 ~ DayOfWeek, data = inFile)
  logit2 <- rxLogit(Late ~ DayOfWeek, transforms = list(Late = ArrDelay > 0),
    data = inFile)
  checkEquals(as.numeric(coef(logit1)), as.numeric(coef(logit2)))
  
}

"test.rxLogit.args.objectiveFunctionTolerance" <- function()
{    
    form <- case ~ age + parity + education + spontaneous + induced
    zLogit1 <- rxLogit(form, data=infert, objectiveFunctionTolerance=1e-8)
  checkEquals(zLogit1$deviance, 257.7977, tolerance = 0.1)
  zLogit2 <- rxLogit(form, data=infert, objectiveFunctionTolerance=1e-4)
  checkTrue(zLogit2$iterations <= zLogit1$iterations)
}

"test.rxLogit.args.pweights" <- function()
{
   set.seed(100)
   test.y <- factor(c(rep(rbinom(50,1,.5),2),rbinom(50,1,.5)))
   test.x <- c(rep(runif(50),2),runif(50))

   # now weight the first 50 cases by a factor of 2 and then
   # eliminate the next 50 cases
   test.y2 <- test.y[c(1:50,101:150)]
   test.x2 <- test.x[c(1:50,101:150)]
   wts <- rep(c(2,1),c(50,50))
   mysum.glm <- summary(glm(test.y2~test.x2,family=binomial,weights=wts),cor=F)
   mydata <- data.frame(test.y2,test.x2,wts)
   myrxLog.sum <- summary(rxLogit(test.y2 ~ test.x2, pweights="wts",data=mydata, covCoef=TRUE))

   # Check that coefficient estimates are the same
   mycoef.glm <- mysum.glm$coefficients[,1]
   names(mycoef.glm) <- NULL
   checkEquals(mycoef.glm,as.vector(myrxLog.sum$coefficients[,1]),tol=.001)

   # Check covariance matrices for equality
   mysum.glm$cov.values<- c(mysum.glm$cov.unscaled[1,1],mysum.glm$cov.unscaled[2,2])
   checkEqualsNumeric(as.vector(mysum.glm$cov.values),diag(myrxLog.sum$covCoef),tol=.001)

   # Check for equal degrees of freedom
   checkEquals(mysum.glm$df.residual,myrxLog.sum$df[2])
}

"test.rxLogit.args.reportProgress" <- function()
{
    # reportProgress argument stress tests
    form <- case ~ age + parity + education + spontaneous + induced

    #reportProgress: logical, character string or integer: 
    #  0: no progress is reported. 
    #  1: the number of processed rows is printed and updated. 
    #  2: rows processed and timings are reported. 
    #  3: rows processed and all timings are reported. 

    variations <- 0:3
    results <- printOutput <- vector("list", length = length(variations))

    for (i in seq_along(variations))
    {
        printOutput[[i]] <- capture.output(results[[i]] <- 
            rxLogit(form = form, data = infert, reportProgress = variations[i]))
    }

    # check that all values are the same
    nocomp <- which(names(results[[1L]]) %in% c("call", "params"))
    checkTrue(all(unlist(lapply(results[-1L], function(x)
              all.equal(x[- nocomp], results[[1L]][- nocomp],
                        check.attributes=FALSE)))))

    names(printOutput) <- as.character(variations)

    noResponseExpected <- "0"
    equalLengthExpected <- "2"
    equalLengthExpected1 <- "1"

    checkTrue(length(unlist(printOutput[noResponseExpected])) == 0L)

    # just row numbers
    checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1], length)) == 6L))

    # all these outputs should be the same
    checkTrue(all(unlist(lapply(printOutput[equalLengthExpected], length)) == 17L))

    # details gives us the goods
    checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=248, TotalRowsProcessed=248, ReadTime", 
              printOutput[["3"]][1L])) > 0)

    # time
    checkTrue(length(grep("^Elapsed computation time:", printOutput[["2"]][17L])) > 0)
}

"test.rxLogit.args.rowSelection" <- function()
{
    form <- case ~ age + parity + education + spontaneous + induced
    zLogitRowSelectionImplicit <- rxLogit(form, data=infert, rowSelection=pooled.stratum > 20 & pooled.stratum < 50)
    zLogitRowSelectionExplicit <- rxLogit(form, data=subset(infert, pooled.stratum > 20 & pooled.stratum < 50))
    nocomp <- which(names(zLogitRowSelectionExplicit) %in% c("call","params","nMissingObs"))
    checkTrue(all.equal(zLogitRowSelectionImplicit[-nocomp], zLogitRowSelectionExplicit[-nocomp], check.attributes=FALSE))
}

"test.rxLogit.args.transformFunc" <- function()
{     
    xform <- function(dataList)
    {
        dataList$division <- as.logical(dataList$stratum > 10)
        dataList    
    }

    z1 <- rxLogit(division ~ age, data=infert, transformFunc=xform, transformVars="stratum")
    checkEqualsNumeric(coef(z1)[["age"]], -0.003967889, tol= 0.001)
}

"test.rxLogit.args.transformVars" <- function()
{     
    xform <- function(dataList)
    {
        dataList$ageCopy <- dataList$age
        dataList$age <- NULL
        dataList    
    }

    z1 <- rxLogit(case ~ ageCopy + parity + education + spontaneous + induced, data=infert, transformFunc=xform, transformVars="age")
    z2 <- rxLogit(case ~ age + parity + education + spontaneous + induced, data=infert)
    nocomp <- which(names(z1) %in% c("call","params","formula","fstatistics"))
    checkTrue(all.equal(z1[-nocomp],z2[-nocomp],check.attributes=FALSE))
}

"test.rxLogit.args.transformFunc.negative" <- function()
{     
  xform <- function(dataList)
  {
    ret = data.frame(dataList)
    ret = ret[ret$age < 40,,drop=FALSE]
    ret    
  }
  
  infert_mod = tryCatch({
    rxLogit(case ~ parity + education + spontaneous + induced, 
            data=infert, transformFunc=xform, transformVars="age")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxLogit"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxLogit"))
    }
  })
}

"test.rxLogit.args.verbose" <- function()
{
    o1 <- capture.output(t1 <- rxLogit(case ~ spontaneous+induced, data=infert, reportProgress=0))
    o2 <- capture.output(t2 <- rxLogit(case ~ spontaneous+induced, data=infert, verbose=0, reportProgress=0))
    o3 <- capture.output(t3 <- rxLogit(case ~ spontaneous+induced, data=infert, verbose=1))  
    o4 <- capture.output(t4 <- rxLogit(case ~ spontaneous+induced, data=infert, verbose=1))
    o5 <- capture.output(t5 <- rxLogit(case ~ spontaneous+induced, data=infert, verbose=2))

    # No output if verbose is 0(the default)
    checkEquals(length(o1), 0)
    checkEquals(length(o2), 0)
    checkTrue(length(o3) > 1)
    checkTrue(nchar(o4[1]) > 1)
    checkTrue(length(o3) < length(o5))
    # reportProgress = "rows" won't do anything because data is data frame in memory
    #o6 <- capture.output(t6 <- rxLogit(case ~ spontaneous+induced, data=infert, reportProgress="rows"))
    
    o7 <- capture.output(t7 <- rxLogit(case ~ spontaneous+induced, data=infert, reportProgress=1))
    checkTrue(length(o7) > 0)
    
    # Check to make sure there is no output if cube = TRUE (RA-1804)
    o8 <- capture.output(t1 <- rxLogit(case ~ education+spontaneous+induced, data=infert, cube=TRUE,reportProgress=0))
    checkEquals(length(o8), 0)
    
    o9 <- capture.output(t1 <- rxLogit(case ~ education+spontaneous+induced, data=infert, cube=TRUE,reportProgress=2))
    checkTrue(length(o9) > 0)
    
    
    o10 <- capture.output(t1 <- rxLogit(case ~ education+spontaneous+induced, data=infert, cube=TRUE,
        reportProgress=0, verbose=1))
    checkTrue(length(o10) > 0)
    
    o11 <- capture.output(t1 <- rxLogit(case ~ education+spontaneous+induced, data=infert, cube=TRUE,
    reportProgress=0, verbose=2))
    checkTrue(length(o11) > length(10))
}


"test.rxLogit.args.dropFirst" <- function()
{
    set.seed(100)
    x <- rep(1:4, times=50)
    ytemp <- x + rnorm(100)
    y <- ytemp > 0
    myData <- data.frame(xfac=factor(x),x, y)
    logRes1 <- rxLogit(y~xfac, data=myData, dropFirst = FALSE)
  logRes2 <- rxLogit(y~xfac, data=myData, dropFirst = TRUE)
  checkTrue( is.na(coef(logRes1)[5]))
  checkTrue( is.na(coef(logRes2)[2]))
}


#####  Text datasource
"test.rxLogit.dataSource.RxTextData.useFastRead.FALSE" <- function()
{
  dsText <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.csv"),
                         useFastRead = FALSE,
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))                                        
          )
                   
    result3 <- rxLogit(default ~ creditScore + yearsEmploy, data=dsText, verbose=3, reportProgress=3)
  summary(result3)
  
  inFile <- file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.csv")
    outFile <- tempfile(pattern = ".rxTempMDT", fileext = ".xdf")
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, stringsAsFactors=TRUE, overwrite=TRUE)
    result4 <- rxLogit(default ~ creditScore + yearsEmploy, data=outFile, verbose=3, reportProgress=3)
  summary(result4)	
    checkEquals(result3$coefficients, result4$coefficients)
    
    file.remove(outFile)
  
  # Fixed format
  fixedColInfo <- list(
    creditScore = list( start = 1, width = 3, type = "integer"),
    houseAge = list( start = 5, width = 2, type = "integer"),
    yearsEmploy = list( start = 8, width = 1, type = "integer"),
    ccDebt = list(start = 10, width = 4, type = "integer"),
    year = list(start = 15, width = 4, type = "integer"),
    default = list( start = 20, width = 3, type = "factor", levels = c("No", "Yes")))
    
  fixedFile <-file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.dat")
  
  dsFixed <- RxTextData(fixedFile, colInfo = fixedColInfo,
                         useFastRead = FALSE, firstRowIsColNames = FALSE)
          
    result5 <- rxLogit(default ~ creditScore + yearsEmploy, data=dsFixed, verbose=3, reportProgress=3)

    checkEquals(result3$coefficients, result5$coefficients)	
  
  dsFixedFast <- RxTextData(fixedFile, colInfo = fixedColInfo,
                         useFastRead = TRUE)
          
    result6 <- rxLogit(default ~ creditScore + yearsEmploy, data=dsFixedFast, verbose=3, reportProgress=3)

    checkEquals(result3$coefficients, result6$coefficients)												

}

"test.rxLogit.dataSource.RxTextData.useFastRead.TRUE" <- function()
{
  dsText <- RxTextData(file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.csv"),
                         useFastRead = TRUE,
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
             colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))
            )
    result3 <- rxLogit(default ~ creditScore + yearsEmploy, data=dsText, verbose=3, reportProgress=3)
  summary(result3)
  
  inFile <- file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.csv")
    outFile <- tempfile(pattern = ".rxTempMDT", fileext = ".xdf")
    rxTextToXdf(inFile, outFile, rowsPerRead=1000, stringsAsFactors=TRUE, overwrite=TRUE)
    result4 <- rxLogit(default ~ creditScore + yearsEmploy, data=outFile, verbose=3, reportProgress=3)
  summary(result4)	
    checkEquals(result3$coefficients, result4$coefficients)
    file.remove(outFile)
}

######## 	rxLogit Analysis on SAS datasource 	 
"test.rxLogit.dataSource.SAS" <- function()
{
  sasDS <- RxSasData(	file.path(rxGetOption("unitTestDataDir"),"mortDefaultTiny.sas7bdat"),
            colClasses = c("creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                     "ccDebt" = "integer", "year" = "integer",  "default" = "factor"),
            colInfo = list(  default = list(type = "factor", levels = c("No","Yes")))
            )
  result <- rxLogit(default ~ creditScore + yearsEmploy, data=sasDS)

    testFile <- tempfile(pattern = ".rxTempSAS", fileext = ".xdf") 
  rxImport(sasDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLogit(default ~ creditScore + yearsEmploy, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove( testFile )
}



######## rxLogit Analysis on ODBC datasource
"test.rxLogit.dataSource.ODBC.ST" <- function()
{			
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.sqlite"),
                    useFastRead = FALSE),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  dsODBC <- RxOdbcData(
        sqlQuery = "SELECT * FROM mortDef",
                useFastRead = FALSE,
                connectionString = conString,
             colClasses = c(	"creditScore" = "integer", "houseAge" = "integer", "yearsEmploy" = "integer", 
                      "ccDebt" = "integer", "year" = "integer",  "isDefault" = "factor"),
             colInfo = list(  isDefault = list(type = "factor", levels = c("No","Yes")))
            )
          
    result <- rxLogit(isDefault~creditScore + yearsEmploy, data=dsODBC)
    
  outFile <- tempfile(pattern = ".rxLogit1", fileext = ".xdf")
  rxImport(dsODBC, outFile = outFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLogit(isDefault~creditScore + yearsEmploy, data = outFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(outFile)
}


######## rxLogit Analysis on useFastRead ODBC datasource
"test.rxLogit.dataSource.ODBC.useFastRead" <- function()
{	
  # Fix for bug RA-4758
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()		
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "mortDefaultTiny.sqlite")),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  dsODBC <- RxOdbcData(
        sqlQuery = "SELECT * FROM mortDef",
                connectionString = conString,
        useFastRead = TRUE,
             colInfo = list(  isDefault = list(type = "factor", levels = c("No","Yes")))
            )
          
    result <- rxLogit(isDefault~creditScore + yearsEmploy, data=dsODBC)
  outFile <- tempfile(pattern = ".rxLogit2", fileext = ".xdf")	
  rxImport(dsODBC, outFile = outFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxLogit(isDefault~creditScore + yearsEmploy, data = outFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(outFile)
  
  ####### File with missing values #######################
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    form <- ybinint ~ xnum2
    logitOutXdf <- rxLogit(form, data=test100r5b)
  
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("unitTestDataDir"), "test100r.sqlite")),
          silent = TRUE)
    #conString <- "Driver=SQLite3 ODBC Driver;Database=C:/Revolution/R-Enterprise-7.1/R-3.0.2/library/RevoScaleR/unitTestData/test100r.sqlite"
                
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  test100rOdbc <- RxOdbcData(
        sqlQuery = "SELECT * FROM test100r",
                connectionString = conString,
        useFastRead = TRUE
            )
                          
    logitOutOdbc <- rxLogit(form, data=test100rOdbc)
  checkEquals(logitOutXdf$nValidObs, logitOutOdbc$nValidObs)					
}

"test.rxLogit.data.missings.dataSource" <- function()
{
    set.seed(17)
    x <- rnorm(100)
    y <- 2*x + rnorm(100) > 0
    naIdx <- seq(from=5, to=95, by = 5)
    x[naIdx] <- NA
    myData <- data.frame(x, y)
    logitObj <- rxLogit( y~x, data=myData)
    
    testFile <- tempfile(pattern = ".rxLogit1", fileext = ".csv") 
    write.csv(myData, file = testFile, row.names = FALSE)
    
    textDS = RxTextData(testFile, colClasses = c(y = "logical"),
        useFastRead = FALSE)

    logitObjDS <- rxLogit(y~x, data = textDS)
    checkEquals(coef(logitObjDS), coef(logitObj))
    checkEquals(logitObjDS$nValidObs, logitObj$nValidObs)
    # Missing obs not calculated for text data source
    preloadData <-rxGetOption("preloadData")
    if (!is.null(preloadData)&& preloadData)
    {
        checkEquals( logitObjDS$nMissingObs, 19)
    } else
    {
        checkEquals( logitObjDS$nMissingObs, as.numeric(NA))
    }
    checkEquals( logitObj$nMissingObs, 19)
    
    # Check to make sure no missings are still reported for a data frame
    infertLogit <- rxLogit(case ~ age + parity,
                       data = infert)
    checkEquals( infertLogit$nMissingObs, 0)
    file.remove(testFile)
    
}

#"test.rxLogit.data.missings" <- function()
#{
  #DEACTIVATED("RA-6604 Pending implementation of removing missings on read for rxLogit.")
    #set.seed(17)
    #x <- rnorm(100)
    #y <- 2*x + rnorm(100) > 0
    #naIdx <- seq(from=5, to=95, by = 5)
    #x[naIdx] <- NA
    #myData <- data.frame(x, y)
    #x[naIdx] <- 0
    #myDataNoNA <- data.frame(x, y)
    #myFunc <- function(data)
  #{
        #data$x[is.na(data$x)] <- 0
        #print(length(data$x))
    #return(data)
  #}
    #logitObj <- rxLogit( y~x, data=myData,
        #transformFunc = myFunc)
    #logitObjNoNA <- rxLogit(y~x, data=myDataNoNA)
    #
    #checkEquals(logitObj$nValidObs, logitObjNoNA$nValidObs)
    #
    #checkEquals(coef(logitObj), coef(logitObjNoNA))
    #
    #checkEquals(logitObj$coef.t.value, logitObjNoNA$coef.t.value)
    #
#}

"test.rxLogit.verbose.showsCoefficients" <- function()
{
  oi <- capture.output(obj <- rxLogit(case ~ age + parity + education + spontaneous + induced,data = infert, verbose = 1))
  # Check to make sure the "Coeffs." column is present
  checkEquals(1, sum(oi == "Row        Coeffs.                         Value      Std. Error         t Value        Pr(>|t|)"))
  # Make sure each of the labels is present
  expectedNames <- c(" (Intercept) "," age "," parity "," education=0-5yrs "," education=6-11yrs "," education=12+ yrs "," spontaneous "," induced ")
  for (name in expectedNames) checkEquals(1, length(grep(name, oi, fixed = TRUE)))
  # Verify that 8 rows are returned
  checkEquals(8, length(obj$aliased))
}

"test.rxLogit.factors.returned" <- function() {
    # Read the data set in
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    df <- rxDataStep(inFile)

    # Generate some factor columns
    n.len <- NROW(df)
    month.int <- trunc(runif(n.len, 1, 12))
    month.names <- c("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December")
    df$Month <- factor(month.names[month.int], levels = month.names)
    dow.int <- trunc(runif(n.len, 1, 7))
    dow.names <- c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")
    df$DayOfWeek <- factor(dow.names[dow.int], levels = dow.names)

    # The dependant variable - a boolean
    df$Late <- df$ArrDelay > 0

    model1 <- rxLogit(Late ~ DayOfWeek:Origin + Month + CRSDepTime, df, cube = T, maxIterations = 5)
    # Same variables, no interactions
    model2 <- rxLogit(Late ~ Month + DayOfWeek + Origin + CRSDepTime, df, cube = T, maxIterations = 5)

    xl1 <- attr(model1, "xlevels")
    xl2 <- attr(model2, "xlevels")

    checkTrue(all(!is.na(match(names(xl1),c("DayOfWeek","Origin","Month")))))
    checkTrue(all(!is.na(match(names(xl1), names(xl2)))))
    # The names are in different order, so we need to re-order the list
    checkIdentical(xl1[names(xl2)], xl2)
}
