# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxLogit with variableSelection, i.e., stepwise logistic regression
#
############################################################################################

.setUp <- function()
{
    ## NOTE: specifying contrasts in options changes the way how factors are labeled!
    #.op.old <<- options(contrasts = c(unordered = "contr.SAS", ordered = "contr.SAS"))
  
  ## various test modes
  # R		-- mostly compatible with R
  # SAS	-- SAS features such as stepCriterion = "SigLevel"
  # SWEEP	-- perform variable selection with the SSCP matrix
  # REVO	-- the "correct" way but might be different from R 
    .test.mode <<- "R" #"SAS" #"SWEEP"	#"REVO"	#
  
    rxOptions(reportProgress = 0) # by default, don't report progress
}

.tearDown <- function()
{
    #options(.op.old)
    if (exists(".test.mode", where = ".GlobalEnv")) remove(".test.mode", pos = ".GlobalEnv")
    rxOptions(TRUE) # reset all options
}

#######################################################################
## local utility functions

compareLogitStepResults <- function(rlm, rxlm, tolerance = .Machine$double.eps^0.5)
{
    ## anova
    rlm.anova <- rlm$anova
    rxlm.anova <- rxlm$anova
    
    #remove rows with 0 df as they can be dropped in any order
    zeroDf <- (rlm.anova$Df == 0)
    rlm.anova <- rlm.anova[!zeroDf, ]
    rxlm.anova <- rxlm.anova[!zeroDf, ]
    
    checkEquals(rlm.anova[-1], rxlm.anova[-1], check.attributes = FALSE)
    checkEquals(as.character(rlm.anova[-1, 1]), as.character(rxlm.anova[-1, 1]))
    
    ## coef
    rlm.coef <- coef(rlm)
    rxlm.coef <- coef(rxlm)
    
    ## workaround for PR#7842. terms.formula may have flipped interactions
    names.sort <- function(names.old)
    {
        names.new <- sapply(strsplit(names.old, ":", fixed = TRUE),
            function(x) paste(sort(x), collapse = ":"))
        names.new
    }

    names(rlm.coef) <- names.sort(names(rlm.coef))
    # convert "Revo" labels to "R" labels
    names.rxlm.coef <- gsub("=", "", names(rxlm.coef))
    names.rxlm.coef <- gsub(" for |[*]", ":", names.rxlm.coef)
    names(rxlm.coef) <- names.sort(names.rxlm.coef)
    
  checkEqualsNumeric(rlm.coef, rxlm.coef[names(rlm.coef)], tolerance = tolerance)
    
    ## aic
    checkEqualsNumeric(rlm$aic, rxlm$aic)
}

#######################################################################
## NOTE: the following tests are organized by families and
## each test generally tests the correctness of 3 things: 
## * rxLogit
## * rxLogit with variableSelection
## * rxPredict on the result of rxLogit with variableSelection

#######################################################################
# infert

"test.rxLogitStep.infert.no.intercept" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced - 1)
    form.terms <- terms(form.full)
    fam <- binomial
    
    logit.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))   
    
    rxlogit.full <- rxLogit(form.full, data = dataset,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "Revo")

    compareLogitStepResults(logit.full, rxlogit.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = formula(drop.terms(form.terms, dropx = 2:5, keep.response = FALSE)),
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2  #NOTE: there seems a bug in R when scale <- 2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    logit.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))
    logit.step <- step(logit.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")

    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step)
    
    #################################### predict
    #rxOptions(numCoresToUse = 1)
    rxlogit.obj <- rxLogit(form = rxlogit.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")

    pred.rxlogit.obj <- rxPredict(rxlogit.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxlogit.step <- rxPredict(rxlogit.step, data = dataset, checkFactorLevels = FALSE)
    pred.logit.step <- predict(logit.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxlogit.obj, pred.rxlogit.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.step[[1]])
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlogit.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxlogit.obj[[nm]], rxlogit.step[[nm]])))))
}

"test.rxLogitStep.infert.no.intercept.keepStepCoefs" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced - 1)
    form.terms <- terms(form.full)
    fam <- binomial
    
    logit.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))   
    
    rxlogit.full <- rxLogit(form.full, data = dataset,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "Revo")

    compareLogitStepResults(logit.full, rxlogit.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = formula(drop.terms(form.terms, dropx = 2:5, keep.response = FALSE)),
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2  #NOTE: there seems a bug in R when scale <- 2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    logit.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))
    logit.step <- step(logit.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection with keepStepCoefs = FALSE
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test, keepStepCoefs = FALSE)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step.keepStepCoefsF <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")

    ## rxLogit/variableSelection with keepStepCoefs = TRUE
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test, keepStepCoefs = TRUE)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step.keepStepCoefsT <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "Revo")
  
  rxStepPlot(rxlogit.step.keepStepCoefsT)
  dev.off()
  
    ## compare rxlogit.step.keepStepCoefsF, rxlogit.step.keepStepCoefsT, and logit.step
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step.keepStepCoefsF)
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step.keepStepCoefsT)

    dont.compare <- c("stepCoefs", "params")
    checkTrue(all(sapply(setdiff(names(rxlogit.step.keepStepCoefsF), dont.compare), 
        function(nm) isTRUE(all.equal(rxlogit.step.keepStepCoefsF[[nm]], rxlogit.step.keepStepCoefsT[[nm]])))))
  
  ## compare the coefficients of the start and final models
  logit.coef.start <- logit.obj$coefficients
  logit.coef.final <- logit.step$coefficients
  
  stepCoefs <- rxlogit.step.keepStepCoefsT$stepCoefs
  rxlogit.stepCoefs.names <- gsub("=", "", rownames(stepCoefs))
  
  rxlogit.stepCoefs.start <- stepCoefs[, 1]
  names(rxlogit.stepCoefs.start) <- rxlogit.stepCoefs.names
  
  rxlogit.stepCoefs.final <- stepCoefs[, ncol(stepCoefs)]
  names(rxlogit.stepCoefs.final) <- rxlogit.stepCoefs.names
  
  rxlogit.coef.final <- drop(rxlogit.step.keepStepCoefsT$coefficients)
  names(rxlogit.coef.final) <- gsub("=", "", names(rxlogit.coef.final))
  
  checkEquals(rxlogit.stepCoefs.start[names(logit.coef.start)], logit.coef.start)
  checkEquals(rxlogit.stepCoefs.final[names(rxlogit.coef.final)], rxlogit.coef.final)
  
  ## compare the number of steps
  checkEquals(ncol(stepCoefs), nrow(rxlogit.step.keepStepCoefsT$anova))
}

"test.rxLogitStep.infert" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial
    
    logit.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))   
    
    rxlogit.full <- rxLogit(form.full, data = dataset,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareLogitStepResults(logit.full, rxlogit.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2  #NOTE: there seems a bug in R when scale <- 2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    logit.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(education = contr.SAS))
    logit.step <- step(logit.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step)
    
    #################################### predict
    #rxOptions(numCoresToUse = 1)
    rxlogit.obj <- rxLogit(form = rxlogit.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlogit.obj <- rxPredict(rxlogit.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxlogit.step <- rxPredict(rxlogit.step, data = dataset, checkFactorLevels = FALSE)
    pred.logit.step <- predict(logit.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxlogit.obj, pred.rxlogit.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.step[[1]])
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlogit.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxlogit.obj[[nm]], rxlogit.step[[nm]])))))
}

"test.rxLogitStep.infert.weight" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    #summary(dataset)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial
    weights <- "stratum"    #"pooled.stratum"
    
    logit.full <- glm(form.full, data = dataset, family = fam,
        weights = dataset[[weights]],
        contrasts = list(education = contr.SAS))   
    
    rxlogit.full <- rxLogit(form.full, data = dataset,
        fweights = weights,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareLogitStepResults(logit.full, rxlogit.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2  #NOTE: there seems a bug in R when scale <- 2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    logit.obj <- glm(form.base, data = dataset, family = fam,
        weights = dataset[[weights]],
        contrasts = list(education = contr.SAS))
    logit.step <- step(logit.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        fweights = weights,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step)
    
    #################################### predict
    #rxOptions(numCoresToUse = 1)
    rxlogit.obj <- rxLogit(form = rxlogit.step$formula, data = dataset,
        fweights = weights,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlogit.obj <- rxPredict(rxlogit.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxlogit.step <- rxPredict(rxlogit.step, data = dataset, checkFactorLevels = FALSE)
    pred.logit.step <- predict(logit.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxlogit.obj, pred.rxlogit.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.step[[1]])
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlogit.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxlogit.obj[[nm]], rxlogit.step[[nm]])))))
}

"test.rxLogitStep.infert.no.valid.observations" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    dataset <- infert
    dataset$induced <- NA   # introduce all missing var
    #summary(dataset)
    
    ################################### basic model
    (form.full <- case ~ age + parity + education + spontaneous + induced)
    form.terms <- terms(form.full)
    fam <- binomial
    
    #logit.full <- glm(form.full, data = dataset, family = fam,
        #contrasts = list(education = contr.SAS))   
    #
    #rxlogit.full <- rxLogit(form.full, data = dataset,
        #dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    #compareLogitStepResults(logit.full, rxlogit.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2  #NOTE: there seems a bug in R when scale <- 2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    #logit.obj <- glm(form.base, data = dataset, family = fam,
        #contrasts = list(education = contr.SAS))
    #logit.step <- step(logit.obj, direction = direction, scope = scope,
        #steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    checkException( rxlogit.step <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R") )

    ## compare
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step)
}

# birthwt

"test.rxLogitStep.birthwt" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)

    attach(birthwt)
    ethnicity <- factor(race, labels = c("white", "black", "other"))
    ptd <- factor(ptl > 0)
    ftv <- factor(ftv)
    levels(ftv)[-(1:2)] <- "2+"
    bwt <- data.frame(low = factor(low), age, lwt, ethnicity,
        smoke = (smoke > 0), ptd, ht = (ht > 0), ui = (ui > 0), ftv)
    detach("birthwt")

    dataset <- bwt
    #summary(dataset)
    
    ################################### basic model
    form.full <- low ~ .
    form.terms <- terms(form.full, data = dataset)    
    # expand the dot in formula
    (form.full <- formula(form.terms))
    fam <- binomial
    
    logit.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(ethnicity = contr.SAS, ptd = contr.SAS, ftv = contr.SAS))   
    
    rxlogit.full <- rxLogit(form.full, data = dataset,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareLogitStepResults(logit.full, rxlogit.full, tolerance = 1e-7)    # Mean relative difference: 1.662366e-08

    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 5:8, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    logit.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(ethnicity = contr.SAS, ptd = contr.SAS, ftv = contr.SAS))
    logit.step <- step(logit.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step)
    
    #################################### predict
    #rxOptions(numCoresToUse = 1)
    rxlogit.obj <- rxLogit(form = rxlogit.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlogit.obj <- rxPredict(rxlogit.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxlogit.step <- rxPredict(rxlogit.step, data = dataset, checkFactorLevels = FALSE)
    pred.logit.step <- predict(logit.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxlogit.obj, pred.rxlogit.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.step[[1]])
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlogit.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxlogit.obj[[nm]], rxlogit.step[[nm]])))))
}

# bacteria

"test.rxLogitStep.bacteria" <- function()
{
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(MASS)
    dataset <- bacteria
    #summary(dataset)
    
    ################################### basic model
    form.full <- y ~ trt * week
    form.terms <- terms(form.full, data = dataset)    
    # expand the dot in formula
    (form.full <- formula(form.terms))
    fam <- binomial
    
    logit.full <- glm(form.full, data = dataset, family = fam,
        contrasts = list(trt = contr.SAS))   
    
    rxlogit.full <- rxLogit(form.full, data = dataset,
        dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")

    compareLogitStepResults(logit.full, rxlogit.full)
    
    ################################### stepwise
    direction <- "both"
    (form.base <- formula(drop.terms(form.terms, dropx = 2:3, keep.response = TRUE)))
    (scope <- list(
      lower = ~ 1,
        upper = formula(delete.response(form.terms)) ))
    steps <- 1000
    scale <- 0  #2
    k <- 2
    test <- "Chisq"  #"F" #NULL   #
    trace <- 0  #1  #
        
    ## glm/step
    logit.obj <- glm(form.base, data = dataset, family = fam,
        contrasts = list(trt = contr.SAS))
    logit.step <- step(logit.obj, direction = direction, scope = scope,
        steps = steps, scale = scale, k = k, test = test, trace = trace)
    
    ## rxLogit/variableSelection
    varsel <- list(#dropPolicy = "Competitive",
        method = if (direction == "both") "stepwise" else direction, scope = scope,
    maxSteps = steps, scale = scale, k = k, test = test)
    if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
    rxlogit.step <- rxLogit(form.base, data = dataset, variableSelection = varsel, 
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    ## compare
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step)
    
    #################################### predict
    #rxOptions(numCoresToUse = 1)
    rxlogit.obj <- rxLogit(form = rxlogit.step$formula, data = dataset,
        verbose = trace, dropMain = FALSE, coefLabelStyle = "R")

    pred.rxlogit.obj <- rxPredict(rxlogit.obj, data = dataset, checkFactorLevels = FALSE)
    pred.rxlogit.step <- rxPredict(rxlogit.step, data = dataset, checkFactorLevels = FALSE)
    pred.logit.step <- predict(logit.step, newdata = dataset, type = "response")
        
    checkEquals(pred.rxlogit.obj, pred.rxlogit.step)        
    if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.step[[1]])
        checkEqualsNumeric(pred.logit.step, pred.rxlogit.obj[[1]])
    }

    dont.compare <- c("call", "params")
    checkTrue(all(sapply(setdiff(names(rxlogit.obj), dont.compare), 
        function(nm) isTRUE(all.equal(rxlogit.obj[[nm]], rxlogit.step[[nm]])))))
}

# mortDefaultSmall
# Move to acceptance test?
#"test.rxLogitStep.mortDefaultSmall" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #mortDefaultSmall.xdf <- file.path(rxGetOption("sampleDataDir"), "mortDefaultSmall.xdf")
    ##rxGetInfo(mortDefaultSmall.xdf, getVarInfo = T)
    #mortDefaultSmall.df <- rxDataStep(mortDefaultSmall.xdf)
    #
    #dataset.xdf <- mortDefaultSmall.xdf
    #dataset.df <- mortDefaultSmall.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #form.full <- default ~ creditScore + houseAge + yearsEmploy + ccDebt + year
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    #fam <- binomial
    #
    #logit.full <- glm(form.full, data = dataset.df, family = fam)   
#
    #rxlogit.full <- rxLogit(form.full, data = dataset.xdf,
        #dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    #compareLogitStepResults(logit.full, rxlogit.full, tolerance = 1e-7)    # Mean relative difference: 2.119491e-08
    #
    #################################### stepwise
    #direction <- "both"
    #(form.base <- formula(drop.terms(form.terms, dropx = 4:5, keep.response = TRUE)))
    #(scope <- list(
      #lower = ~ 1,
        #upper = formula(delete.response(form.terms)) ))
    #steps <- 1000
    #scale <- 0  #2
    #k <- 2
    #test <- "Chisq"  #"F" #NULL   #
    #trace <- 0  #1  #
        #
    ### glm/step
    #system.time({
    #logit.obj <- glm(form.base, data = dataset.df, family = fam)
    #logit.step <- step(logit.obj, direction = direction, scope = scope,
      #steps = steps, scale = scale, k = k, test = test, trace = trace)
    #})
    #
    ### rxLogit/variableSelection
    #varsel <- list(#dropPolicy = "Competitive",
        #method = if (direction == "both") "stepwise" else direction, scope = scope,
    #maxSteps = steps, scale = scale, k = k, test = test)
    #if(exists(".test.mode") && .test.mode == "SAS") varsel <- c(varsel, stepCriterion = "SigLevel")
    #if(exists(".test.mode") && .test.mode == "SWEEP") varsel <- c(varsel, refitEachStep = FALSE)
    #if(exists(".test.mode") && .test.mode == "REVO") varsel <- c(varsel, matchRStep = FALSE)
  #system.time({
    #rxlogit.step <- rxLogit(form.base, data = dataset.df, variableSelection = varsel, 
      #verbose = trace, dropMain = FALSE, coefLabelStyle = "R", reportProgress = 0)
    #})
#
  ### rxGlm/variableSelection
  #system.time({
    #rxglm.step <- rxGlm(form.base, data = dataset.df, variableSelection = varsel, 
      #family = fam, computeAIC = TRUE,
      #verbose = trace, dropMain = FALSE, coefLabelStyle = "R", reportProgress = 0)
    #})
#
    ### compare
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxlogit.step, tolerance = 1e-7)	#Mean relative difference: 2.119491e-08
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") compareLogitStepResults(logit.step, rxglm.step, tolerance = 1e-7)	#Mean relative difference: 2.062153e-08
#
    ##################################### predict
    ##rxOptions(numCoresToUse = 1)
    #rxlogit.obj <- rxLogit(form = rxlogit.step$formula, data = dataset.xdf,
        #verbose = trace, dropMain = FALSE, coefLabelStyle = "R")
#
    #pred.rxlogit.obj <- rxPredict(rxlogit.obj, data = dataset.df, checkFactorLevels = FALSE)
    #pred.rxlogit.step <- rxPredict(rxlogit.step, data = dataset.df, checkFactorLevels = FALSE)
    #pred.logit.step <- predict(logit.step, newdata = dataset.df, type = "response")
        #
    #checkEquals(pred.rxlogit.obj, pred.rxlogit.step)        
    #if(!exists(".test.mode") || .test.mode == "R" || .test.mode == "REVO") {
        #checkEqualsNumeric(pred.logit.step, pred.rxlogit.step[[1]], tolerance = 1e-7) #Mean relative difference: 1.615453e-08
        #checkEqualsNumeric(pred.logit.step, pred.rxlogit.obj[[1]], tolerance = 1e-7)    #Mean relative difference: 1.615453e-08
    #}
#
    #dont.compare <- c("call", "params")
    #checkTrue(all(sapply(setdiff(names(rxlogit.obj), dont.compare), 
        #function(nm) isTRUE(all.equal(rxlogit.obj[[nm]], rxlogit.step[[nm]], tolerance = 1e-7)))))
#}

##mortDefault    ## for benchmark?
#
#"test.rxLogitStep.mortDefault" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #mortDefault.xdf <- file.path("C:/Revolution/Data/mortDefault", "mortDefault.xdf")
    ##rxGetInfo(mortDefault.xdf, getVarInfo = T)
    #mortDefault.df <- rxDataStep(mortDefault.xdf)
    ##Warning message:
    ##In rxReadXdf(outFile, maxRowsByCols = maxRowsByCols, reportProgress = 0) :
    ##The number of rows (1e+07) times the number of columns (6) exceeds the maxRowsByCols argument value (3e+06).
    ##Only 500000 rows will be read.
    #dataset.xdf <- mortDefault.xdf
    #dataset.df <- mortDefault.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #form.full <- default ~ creditScore + houseAge + yearsEmploy + ccDebt + year
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    #fam <- binomial
    #
    #logit.full <- glm(form.full, data = dataset.df, family = fam)   
    #
    #rxlogit.full <- rxLogit(form.full, data = dataset.xdf,
        #dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    ##compareLogitStepResults(logit.full, rxlogit.full)
#}
#
### Airline2000  ## for benchmark
#
#"test.rxLogitStep.Airline2000" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #Airline2000.xdf <- file.path("C:/Revolution/Data/Airline2000", "Airline2000.xdf")
    ##rxGetInfo(Airline2000.xdf, getVarInfo = T)
    #Airline2000.df <- rxDataStep(Airline2000.xdf)
    ##Warning message:
    ##In rxReadXdf(outFile, maxRowsByCols = maxRowsByCols, reportProgress = 0) :
    ##The number of rows (5683047) times the number of columns (29) exceeds the maxRowsByCols argument value (3e+06).
    ##Only 103448 rows will be read.
    #
    #dataset.xdf <- Airline2000.xdf
    #dataset.df <- Airline2000.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #yvar <- "Cancelled" # "Diverted"    # "ArrDelay"    #
    #xvars <- c("Year", "Month", "DayofMonth", "DayOfWeek", 
        #"DepTime", "CRSDepTime", "ArrTime", "CRSArrTime", "UniqueCarrier", 
        #"ActualElapsedTime", "CRSElapsedTime", "AirTime", "DepDelay", 
        #"Origin", "Dest", "Distance", "TaxiIn", "TaxiOut")
    #
    #form.full <- formula(paste(yvar, paste(xvars, collapse = " + "), sep = " ~ "))
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    #fam <- binomial
    #
    #logit.full <- glm(form.full, data = dataset.df, family = fam)   
    #
    #rxlogit.full <- rxLogit(form.full, data = dataset.xdf,
        #dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    ##compareLogitStepResults(logit.full, rxlogit.full)
#}
#
### AirlineData87to08  ## for benchmark
#
#"test.rxLogitStep.AirlineData87to08" <- function()
#{
    ##source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    #
    #AirlineData87to08.xdf <- file.path("C:/Revolution/Data/AirlineData87to08", "AirlineData87to08.xdf")
    ##rxGetInfo(Airline2000.xdf, getVarInfo = T)
    #AirlineData87to08.df <- rxDataStep(AirlineData87to08.xdf)
    ##Warning message:
    ##In rxReadXdf(outFile, maxRowsByCols = maxRowsByCols, reportProgress = 0) :
    ##The number of rows (5683047) times the number of columns (29) exceeds the maxRowsByCols argument value (3e+06).
    ##Only 103448 rows will be read.
    #
    #dataset.xdf <- AirlineData87to08.xdf
    #dataset.df <- AirlineData87to08.df
    ##summary(dataset.df)
    #
    #################################### basic model
    #yvar <- "Cancelled" # "Diverted"    # "ArrDelay"    #
    #xvars <- c("Year", "Month", "DayofMonth", "DayOfWeek", 
        #"DepTime", "CRSDepTime", "ArrTime", "CRSArrTime", "UniqueCarrier", 
        #"ActualElapsedTime", "CRSElapsedTime", "AirTime", "DepDelay", 
        #"Origin", "Dest", "Distance", "TaxiIn", "TaxiOut")
    #
    #form.full <- formula(paste(yvar, paste(xvars, collapse = " + "), sep = " ~ "))
    #form.terms <- terms(form.full)    
    #(form.full <- formula(form.terms))
    #fam <- binomial
    #
    #logit.full <- glm(form.full, data = dataset.df, family = fam)   
    #
    #rxlogit.full <- rxLogit(form.full, data = dataset.xdf,
        #dropFirst = FALSE, dropMain = FALSE, coefLabelStyle = "R")
#
    ##compareLogitStepResults(logit.full, rxlogit.full)
#}
