"test.rxSerializeModel.Raw.verify" <- function() {
    
    # An external, binary model
    bmodel <- charToRaw('binary.model')
    checkException(ww <- RevoScaleR:::rxSerializeModelRaw(bmodel))
    
    checkException(RevoScaleR:::rxSerializeModelRaw(bmodel, runtimeId="runtime.id"))
    
    # Negative tests for UUID
    checkException(RevoScaleR:::rxSerializeModelRaw(bmodel, runtimeId = "dddfc64b-cc25-4c6c-9725-5ac8r3f0724c9", metadata = charToRaw("meta.data")))
    checkException(RevoScaleR:::rxSerializeModelRaw(bmodel, runtimeId = "{dddfc64b-cc25-4c6c-9725-5ac83f0724c9", metadata = charToRaw("meta.data")))
    
    # With meta data
    rid <- "dddfc64b-cc25-4c6c-9725-5ac83f0724c9"
    ww <- RevoScaleR:::rxSerializeModelRaw(bmodel, runtimeId = rid, metadata = charToRaw("meta.data"))
    mm <- RevoScaleR:::rxUnserializeModel(ww)
    checkIdentical(bmodel, mm$Model)
    checkIdentical(rid, mm$RuntimeId)
    
    checkIdentical("meta.data", rawToChar(mm$Metadata))
    
    # With an R model (any object really)
    obj <- list(a = "rmodel", b = 5)
    
    ww <- RevoScaleR:::rxSerializeModelRaw(bmodel, runtimeId = rid, rmodel = obj, metadata = charToRaw("meta.data"))
    mm <- RevoScaleR:::rxUnserializeModel(ww)
    checkIdentical(bmodel, mm$Model)
    checkIdentical(rid, mm$RuntimeId)
    
    checkIdentical("meta.data", rawToChar(mm$Metadata))
    checkEquals(obj, mm$RModel)
}

"verify.RegressionModel" <- function(originalModel, unserializedModel )
{
    rtsModel <- unserializedModel$RTSModel
    xl1 <- attr(originalModel, "xlevels")
    xl2 <- attr(rtsModel, "xlevels")
    
    # Verify the factor levels
    checkTrue(all(!is.na(match(names(xl1),c("DayOfWeek","Origin","Month")))))
    checkTrue(all(!is.na(match(names(xl1), names(xl2)))))
    # It's ok for the names to be in different order, so we need to re-order the list
    checkIdentical(xl1[names(xl2)], xl2)
    
    # Verify the coefficients
    checkIdentical(as.numeric(as.vector(originalModel$coefficients)), as.numeric(as.vector(rtsModel$betas)))
    coeffNames <- names(originalModel$aliased)
    checkIdentical(coeffNames, rtsModel$coeffNames)
    
    # Verify the flag for the intercept
    flag <- coeffNames[1] == "(Intercept)"
    checkIdentical(flag, rtsModel$interceptIncluded)
    
    checkIdentical(originalModel$df[2], rtsModel$ResidualDF)
    
    checkEquals(originalModel, unserializedModel$Model)
}

"test.rxSerializeModel.RegressionModel.verify" <- function() {

    # Read the data set in
    inFile <- file.path(rxGetOption("unitTestDataDir"), "AirlineSampleDate.xdf")
    df <- rxDataStep(inFile)
    
    # Generate some factor columns
    n.len <- NROW(df)
    month.int <- trunc(runif(n.len, 1, 12))
    month.names <- c("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December")
    df$Month <- factor(month.names[month.int], levels = month.names)
    dow.int <- trunc(runif(n.len, 1, 7))
    dow.names <- c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")
    df$DayOfWeek <- factor(dow.names[dow.int], levels = dow.names)
    
    model <- rxLinMod(ArrDelay ~ DayOfWeek:Origin + Month + CRSDepTime, df, cube = T)

    bb <- rxSerializeModel(model)

    mm <- rxUnserializeModel(bb, includeRTSModel=T)
    
    verify.RegressionModel(model, mm);

    l1 <- rxLogit(case ~ age + parity + education + spontaneous + induced,
                  data = infert)
    l2 <- rxSerializeModel(l1)
    l3 <- rxUnserializeModel(l2)
    checkEquals(l1, l3)
}

"verify.ForestModel" <- function(originalModel, unserializedModel) {
    
    m2 <- unserializedModel$RTSModel
    
    # Compare the full model
    checkEquals(originalModel, unserializedModel$Model)
    
    m3 <- rxSerializeModel(originalModel, prepareModel = 2)
    
    checkEquals(m2$InputVars, m3$InputVars)
    checkEquals(m2$ReadVars, m3$ReadVars)
    checkEquals(m2$TargetVars, m3$TargetVars)
    checkEquals(m2$Method, m3$Method)
    checkEquals(m2$UseSurrogate, m3$UseSurrogate)
    
    # Verify the forest
    checkEquals(NROW(m3$FrameNodeInfo), NROW(m2$FrameNodeInfo))
    for(ii in 1:NROW(m3$FrameNodeInfo))
    {
        checkEquals(as.integer(m3$FrameNodeInfo[[ii]]), as.integer(m2$FrameNodeInfo[[ii]]))
        checkEquals(as.integer(m3$FrameNodeId[[ii]]), as.integer(m2$FrameNodeId[[ii]]))
        checkEquals(as.integer(m3$FrameYVal[[ii]]), as.integer(m2$FrameYVal[[ii]]))
        
        checkEquals(as.numeric(m3$Splits[[ii]]), as.numeric(m2$Splits[[ii]]))
        checkEquals(as.character(m3$SplitsVar[[ii]]), as.character(m2$SplitsVar[[ii]]))
        checkEquals(as.numeric(m3$CSplits[[ii]]), as.numeric(m2$CSplits[[ii]]))
    }
    checkTrue(is.null(m2$FrameYVal2))
}

"test.rxSerializeModel.ForestModel.verify" <- function() {
    # Read the data set in
    inFile <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    fullData <- rxDataStep(inFile)
    
    # Select 10K rows, randomly
    trainData <- fullData[sample(1:NROW(fullData), 10000),]
    
    formula <- incwage ~ age + sex + wkswork1
    m1 <- rxDTree(formula, pweights = "perwt", maxDepth = 3, minBucket = 30, data = trainData)
    
    bb <- rxSerializeModel(m1)
    mm <- rxUnserializeModel(bb, includeRTSModel=T)

    verify.ForestModel(m1, mm)

    # Now use a BTree
    m1 <- rxBTrees(formula, data = trainData)
    
    bb <- rxSerializeModel(m1)
    mm <- rxUnserializeModel(bb, includeRTSModel=T)

    verify.ForestModel(m1, mm)

    # Verify rxDForest model
    iris.sub <- c(sample(1:50, 25), sample(51:100, 25), sample(101:150, 25))
    df <- rxDForest(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
                    data = iris[iris.sub, ])
    df1 <- rxSerializeModel(df)
    df2 <- rxUnserializeModel(df1)
    checkEquals(df, df2)
}

"test.rxSerializeModel.metadata" <- function()
{
    form <- Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species
    irisLinMod <- rxLinMod(form, data = iris)
    metadata <- paste("Version:1.0, Date:", date())
    ser <- rxSerializeModel(irisLinMod, metadata = charToRaw(metadata))
    unser <- rxUnserializeModel(ser)
    checkEquals(rawToChar(unser$Metadata), metadata)
    
    # non-raw metadata
    checkException(rxSerializeModel(irisLinMod, metadata = metadata))
}

"test.rxSerializeModel.realtimeScoringOnly" <- function()
{
    form <- Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species
    irisLinMod <- rxLinMod(form, data = iris)
    ser <- rxSerializeModel(irisLinMod, realtimeScoringOnly=T)
    unser <- rxUnserializeModel(ser)
    checkEquals(unser, NULL)
}

"test.rxSerializeModel.Unsupported" <- function()
{
    infertRxGlm <- rxGlm(case ~ age + parity + education + spontaneous + induced,
                         family = binomial(), dropFirst = TRUE, data = infert)
    serMod <- rxSerializeModel(infertRxGlm)
    unserMod <- rxUnserializeModel(serMod)
    checkEquals(infertRxGlm, unserMod)
    
    checkException(rxSerializeModel(infertRxGlm, realtimeScoringOnly = T))
    
    metadata <- paste("Version:1.0, Date:", date())
    
    verify.rxSerializeModel(infertRxGlm)
    verify.rxSerializeModel.WithMetadata(infertRxGlm, metadata)
    
    claimsXdf <- file.path(rxGetOption("sampleDataDir"),"claims.xdf")
    claims.nb <- rxNaiveBayes(type ~ age + cost, data = claimsXdf)
    verify.rxSerializeModel(claims.nb)
    verify.rxSerializeModel.WithMetadata(claims.nb, metadata)
}


"test.rxSerializeModel.MicrosoftML" <- function()
{
    mml <- list()
    outModelBin <- charToRaw("dummy")
    mml$outModelBin <- outModelBin
    mml$otherParams <- "otherValue"
    class(mml) <- c("mlModel")
    
    verify.rxSerializeModel(mml)
    metadata <- paste("Version:1.0, Date:", date())
    verify.rxSerializeModel.WithMetadata(mml, metadata)
}

"test.rxSerializeModel.RevoScaleR.withTransformFunc" <- function()
{
    myDF <- data.frame(sex = c("Male", "Male", "Female", "Male"),
                       age = c(20, 20, 12, 15), score = 1.1:4.1, sport=c(1:3,2))
    
    transforms <- expression(list(
        revage = rev(age),
        division = factor(c("A","B","B","A")),
        sport = factor(sport, labels=c("tennis", "golf", "football"))))
    
    modelWithTransformFormula <- rxLinMod(score ~ sin(age) + sex, data = myDF)
    serMod <- rxSerializeModel(modelWithTransformFormula)
    
    modelWithTransformFn <- rxLinMod(score ~ revage + sex, data = myDF, transforms = transforms)
    serMod <- rxSerializeModel(modelWithTransformFn)
    
    modelWithTransform <- rxLinMod(score ~ sin(revage) + sex, data = myDF, transforms = transforms)
    serMod <- rxSerializeModel(modelWithTransform)
    
    options(warn=2) # Treat warnings to errors
    on.exit(options(warn=0))
    
    modelWithoutTransform <- rxLinMod(score ~ age + sex, data = myDF)
    serMod <- rxSerializeModel(modelWithoutTransform) # Verify no warning to sanity check
    
    checkException(rxSerializeModel(modelWithTransform))
    checkException(rxSerializeModel(modelWithTransformFormula))
    checkException(rxSerializeModel(modelWithTransformFn))
}

"verify.rxSerializeModel" <- function(model)
{
    modelCopy <- model
    serMod <- rxSerializeModel(model)
    checkEquals(model, modelCopy)
    
    unserMod <- rxUnserializeModel(serMod)
    checkEquals(model, unserMod)
}

"verify.rxSerializeModel.WithMetadata" <- function(model, metadata)
{
    serMod <- rxSerializeModel(model,charToRaw(metadata))
    unserMod <- rxUnserializeModel(serMod)
    checkEquals(model, unserMod$Model)
    checkEquals(metadata, rawToChar(unserMod$Metadata))
}

#               .----------.----------.
#      blob     | header   | payload  |
#               '----------'----------'
#               1------------5----------37--------41-----------45-----------61-----------65--------------69--------------72
#      header   | eyecatcher | checksum | version | total size | runtimeId  | model size | R model size  | meta data size |  
#               '------------'----------'---------'------------'------------'------------'---------------'----------------'
"appendFields" <- function(serializedModel, maxFieldId)
{
    # refer above diagram for the following constants
    modelSizeIndices <- c(61:64);
    todelSizeIndices <- c(41:44);
    checkSumIndices <- c(5:36);
    blobHeaderSize = 72
    
    oldModelSize <- readBin(serializedModel[modelSizeIndices[1]:modelSizeIndices[4]], what ='integer', n=4) 
    oldEndByteOfRTSModel = oldModelSize + blobHeaderSize
    
    #add 2 new uint32_t fields occupying 7 bytes each to the serialized Model
    #append these bytes before oldModelEnd - 2(old model's last 2 bytes are for BT_STOP_BASE(typeid = 01) and BT_STOP(typedid = 00) respectively )
    fudgedModel <- append(serializedModel, c(as.raw(05),as.raw(maxFieldId + 2),as.raw(00),as.raw(05),as.raw(00),as.raw(00),as.raw(00)), after=oldEndByteOfRTSModel - 2 )
    fudgedModel <- append(fudgedModel, c(as.raw(05),as.raw(maxFieldId + 1),as.raw(00),as.raw(04),as.raw(00),as.raw(00),as.raw(00)), after=oldEndByteOfRTSModel - 2 )
    
    #increase total size
    fudgedModel[todelSizeIndices[1]:todelSizeIndices[4]] = packBits(intToBits(length(fudgedModel)))
    
    #increase ModelSize
    # 1 byte for typeid, 2 bytes for id, 4 bytes for 
    fudgedModel[modelSizeIndices[1]:modelSizeIndices[4]] = packBits(intToBits(oldModelSize + (1 + 2 + 4)*2))
    
    #calculate the new checksum
    fudgedModelWithoutChecksum <- fudgedModel[c(-1:-checkSumIndices[32])]
    modifiedChecksum <- sha256(fudgedModelWithoutChecksum, key = NULL)
    fudgedModel[checkSumIndices[1]:checkSumIndices[32]] <- modifiedChecksum
    
    fudgedModel
}
