# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxPartition
#
############################################################################################

.setUp <- function()
{
}

.tearDown <- function()
{
}

# function to verify the total number of rows
".verifyRowCount" <- function(partList, expectedNumParts, expectedTotalRows)
{
  checkEquals(length(partList), expectedNumParts)
  
  oldReportProgress <- rxGetOption("reportProgress")
  rxOptions(reportProgress = 0)
  on.exit(rxOptions(reportProgress = oldReportProgress), add=TRUE)
  
  results <- lapply(partList, unlist)
  totalrows <- 0
  for (i in 1:length(results))
  {
    partDataSet <- rxImport(inData = results[[i]])
    nrowPart <- nrow(partDataSet)
    totalrows <- totalrows + nrowPart
  }
  checkEquals(totalrows, expectedTotalRows)
}


"test.rxPartition.XdfDataSource.partition.factor" <- function() {
    # Setup the input and output data paths.
    inFile <- "claims.xdf"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

    tmpPat <- "rxPartitionClaims"
    outFile <- tempfile(pattern = tmpPat)

    # Remove any artifacts from a pervious test.
    unlink(x = outFile, recursive = TRUE, force = TRUE)
    on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE) , add = TRUE)

    # create input and output Xdf data source
    inXdfDataSource <- RxXdfData(file = inFile)
    outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

    # Generate a partitioned xdf from the input xdf.
    varsToPartition <- c("car.age")
    partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, overwrite = TRUE)

    checkEquals(NROW(partDF$car.age), 4)
    checkEquals(levels(partDF$car.age), c("0-3","4-7", "8-9", "10+"))
}


"test.rxPartition.TxtDataSource" <- function() {

    # Setup the input and output data paths.
    inFile <- "claims.txt"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

    tmpPat <- "rxPartitionClaims"
    outFile <- tempfile(pattern = tmpPat)

    # Remove any artifacts from a pervious test.
    unlink(x = outFile, recursive = TRUE, force = TRUE)
    on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
    
    # 1. create input and output Xdf data source
    inTxtDataSource <- RxTextData(file = inFile, delimiter = ",")
    outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

    # Generate a partitioned xdf from the input xdf.
    varsToPartition <- c("car.age","type")
    partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
    .verifyRowCount(partDF$DataSource, 16, 128)
    
    # Different order of varsToPartition, verify the same set of partitions
    varsToPartition <- c("type", "car.age")
    partDF1 <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
    .verifyRowCount(partDF1$DataSource, 16, 128)
    
    checkTrue(identical(partDF[[1]], partDF1[[1]]))
    checkTrue(identical(partDF[[2]], partDF1[[2]]))
    
    # 2. input TextData source with ColInfo
    inTxtDataSource <- RxTextData(file = inFile, colInfo = list(RowNum = list(type = "string"), age = list(type = "factor"), type = list(type = "factor"), cost = list(type = "numeric")))

    # Generate a partitioned xdf.
    varsToPartition <- c("car.age","type")
    partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
    .verifyRowCount(partDF$DataSource, 16, 128)
    
    # verify data types
    for (i in 1:nrow(partDF))
    {
        df <- rxImport(partDF[[3]][[i]])
        checkTrue(is.character(df$RowNum))
        checkTrue(is.factor(df$age))
        checkTrue(is.character(df$car.age))
        checkTrue(is.factor(df$type))
        checkTrue(is.numeric(df$cost))
        checkTrue(is.numeric(df$number))
    }
    
    # 3. input TextData source with StringAsFactors
    inTxtDataSource <- RxTextData(file = inFile, stringsAsFactors = TRUE)
    
    # Generate a partitioned xdf.
    varsToPartition <- c("car.age","type")
    partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
    .verifyRowCount(partDF$DataSource, 16, 128)
    
    # verify data types
    for (i in 1:nrow(partDF))
    {
      df <- rxImport(partDF[[3]][[i]])
      checkTrue(is.integer(df$RowNum))
      checkTrue(is.factor(df$age))
      checkTrue(is.factor(df$car.age))
      checkTrue(is.factor(df$type))
      checkTrue(is.numeric(df$cost))
      checkTrue(is.numeric(df$number))
    }
    
    # 4. input TextData source with colClasses
    inTxtDataSource <- RxTextData(file = inFile, colClasses = c(RowNum = "character", age = "factor", car.age = "string", type = "factor", number = "integer"))
    
    # Generate a partitioned xdf.
    varsToPartition <- c("car.age","type")
    partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
    .verifyRowCount(partDF$DataSource, 16, 128)
    
    # verify data types
    for (i in 1:nrow(partDF))
    {
      df <- rxImport(partDF[[3]][[i]])
      checkTrue(is.character(df$RowNum))
      checkTrue(is.factor(df$age))
      checkTrue(is.character(df$car.age))
      checkTrue(is.factor(df$type))
      checkTrue(is.numeric(df$cost))
      checkTrue(is.integer(df$number))
    }
}


# Test createPartitionSet for RxXdfData
"test.RxXdfData.createPartitionSet" <- function() {

    # Setup the input and output data paths.
    inFile <- "claims.xdf"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

    # create Xdf data source objects
    xdfDataSource <- RxXdfData(file = inFile)
    xdfCompDataSource <- RxXdfData(file = inFile, createCompositeSet = TRUE)
    xdfPartDataSource <- RxXdfData(file = inFile, createPartitionSet = TRUE)

    # verify objects
    checkTrue(!is.null(xdfDataSource) && inherits(xdfDataSource, "RxXdfData"))
    checkTrue(!is.null(xdfCompDataSource) && inherits(xdfCompDataSource, "RxXdfData") && xdfCompDataSource@createCompositeSet)
    checkTrue(!is.null(xdfPartDataSource) && inherits(xdfPartDataSource, "RxXdfData") && xdfPartDataSource@createPartitionSet)

    # warning about both flags being set
    xdfCompDataSource <- RxXdfData(file = inFile, createCompositeSet = TRUE, createPartitionSet = TRUE)
    checkTrue(!is.null(xdfCompDataSource) && 
        inherits(xdfCompDataSource, "RxXdfData") && 
        xdfCompDataSource@createCompositeSet && 
        !xdfCompDataSource@createPartitionSet)
    
    # give error 
    partDF <- try(rxPartition(inData = xdfDataSource, outData = xdfCompDataSource, varsToPartition = "car.age", append = "none", overwrite = TRUE), silent = TRUE)
    checkTrue(inherits(partDF, "try-error"))
    
    # give error
    getPartDF <- try(rxGetPartitions(inData = xdfDataSource), silent = TRUE)
    checkTrue(inherits(getPartDF, "try-error"))
}


# Test with composite Xdf Data Source
"test.rxPartition.CompositeXdfDataSource" <- function()
{
    # Setup the input and output data paths.
    inFile <- "AirlineDemoSmallComposite"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

    tmpPat <- "rxPartitionAirline"
    outFile <- tempfile(pattern = tmpPat)

    # Remove any artifacts from a pervious test.
    unlink(x = outFile, recursive = TRUE, force = TRUE)
    on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)

    # create input and output Xdf data source
    inXdfDataSource <- RxXdfData(file = inFile, createCompositeSet = TRUE)
    outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

    # Generate a partitioned xdf
    varsToPartition <- c("DayOfWeek")
    partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)

    .verifyRowCount(partDF$DataSource, 7, 600000)
    
    # verify file size
    inXdfFileSize = sum(file.size(file.path(inFile, list.files(inFile, all.files = TRUE, recursive = TRUE)))) 
    outXdfFileSize = sum(file.size(file.path(outFile, list.files(outFile, all.files = TRUE, recursive = TRUE)))) 
    checkTrue(((outXdfFileSize - inXdfFileSize) * 100 / inXdfFileSize) < 15)
}


# Test Append 
"test.rxPartition.Append" <- function() 
{
  # Setup the input and output data paths.
  inFile <- "claims.txt"

  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
  
  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)
  
  # Remove any artifacts from a pervious test.
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
  
  # create input and output Xdf data source
  inTxtDataSource <- RxTextData(file = inFile, delimiter = ",", stringsAsFactors = TRUE)
  inDF <- rxImport(inData = inTxtDataSource)

  df1 <- inDF[1:50,]
  df2 <- inDF[51:nrow(inDF),]
  
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

  # create new partitions
  partDF <- rxPartition(inData = df1, outData = outXdfDataSource, varsToPartition = c("age"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 4, 50)

  # append to existing partitions
  partDF <- rxPartition(inData = df2, outData = outXdfDataSource, varsToPartition = c("age"))
  .verifyRowCount(partDF$DataSource, 8, 128)
}

# Test Append and Overwrite combinations
"test.rxPartition.Append.Overwrite" <- function() {
  
  # Setup the input and output data paths.
  inFile <- "claims.txt"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)
  
  # Remove any artifacts from a pervious test.
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
  
  # create input and output Xdf data source
  # inXdfDataSource <- RxXdfData(file = inFile)
  inTxtDataSource <- RxTextData(file = inFile, delimiter = ",")
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)
  
  varsToPartition <- c("car.age", "type")
  
  #
  # File doesn't exist
  #
  
  appendVec <- c("rows", "none")
  overwriteVec <- c(TRUE, FALSE)
  for (a in appendVec)
  {
    for (o in overwriteVec)
    {
      unlink(x = outFile, recursive = TRUE, force = TRUE)
      #print(paste0("append = ", a, "; overwrite = ", o))
      partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = a, overwrite = o)
      .verifyRowCount(partDF$DataSource, 16, 128)    
    }
  }
  
  #
  # File DOES exist
  #
  
  # append = none, overwrite = TRUE -> should recreate
  partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 16, 128)

  # append = none, overwrite = FALSE -> give error
  partDF <- try(rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = FALSE), silent = TRUE)
    
  # append = rows, overwrite = TRUE -> append
  partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "rows", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 16, 128*2)
  
  # append = rows, overwrite = FALSE -> append
  partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "rows", overwrite = FALSE)
  .verifyRowCount(partDF$DataSource, 16, 128*3)
}


# Test rxGetPartitions
"test.rxGetPartitions" <- function()
{
    # Setup the input and output data paths.
    inFile <- "claims.txt"
    inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)

    tmpPat <- "rxPartitionClaims"
    outFile <- tempfile(pattern = tmpPat)
    #outFile <- file.path("F:/temp/tmpPat")

    # Remove any artifacts from a pervious test.
    unlink(x = outFile, recursive = TRUE, force = TRUE)
    on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)

    # create input and output Xdf data source
    inTxtDataSource <- RxTextData(file = inFile, delimiter = ",")
    outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)

    varsToPartition <- c("car.age", "type")
    partDF <- rxPartition(inData = inTxtDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition)

    # create a Partitioned Xdf object and call rxGetPartitions
    outPartXdfDS <- RxXdfData(file = outFile)
    partDF1 <- rxGetPartitions(inData = outPartXdfDS)
    .verifyRowCount(partDF1$DataSource, 16, 128)
    
    # call rxGetPartitions with the path to a partitioned xdf
    partDF1 <- RevoScaleR:::rxGetPartitions(inData = outFile)
    .verifyRowCount(partDF1$DataSource, 16, 128)
    
    try(partDF1 <- rxGetPartitions(inData = inFile), silent = TRUE)
}

# test Append with factor columns
"test.rxPartition.Append.factor" <- function() 
{
    tmpPat <- "rxPart"
    outFile <- tempfile(pattern = tmpPat)

    # Remove any artifacts from a previous test.
    unlink(x = outFile, recursive = TRUE, force = TRUE)
    on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
    
    outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)
    
    df1 = data.frame(fcol=c("wed","thu"), icol=c(1,2))
    checkEquals(as.integer(df1[,1]), c(2,1))
    
    df2 = data.frame(fcol=c("wed"), icol=3)
    checkEquals(as.integer(df2[,1]), c(1))
    
    # create new partitions
    partDF <- rxPartition(inData = df1, outData = outXdfDataSource, varsToPartition = c("fcol"), append = "none", overwrite = TRUE)
    
    gi1 = rxGetInfo(partDF[[2]][[1]], getVarInfo = TRUE, numRows = 3)
    checkEquals(gi1$numVars, 2)
    checkEquals(NROW(gi1$varInfo$fcol$levels), 1)
    
    gi2 = rxGetInfo(partDF[[2]][[2]], getVarInfo = TRUE, numRows = 3)
    checkEquals(gi2$numVars, 2)
    checkEquals(NROW(gi1$varInfo$fcol$levels), 1)
    
    if(gi1$varInfo$fcol$levels[1] == "thu") {
        checkEquals(gi2$varInfo$fcol$levels, c("wed"))
        wedId = 2
    } else if(gi1$varInfo$fcol$levels[1] == "wed") {
        checkEquals(gi2$varInfo$fcol$levels, c("thu"))
        wedId = 1
    } else {
        checkEquals(TRUE, FALSE)
    }
    
    # append to existing partitions
    partDF <- rxPartition(inData = df2, outData = outXdfDataSource, varsToPartition = c("fcol"))
    
    gi = rxGetInfo(partDF[[2]][[wedId]], getVarInfo = TRUE, numRows = 3)
    checkEquals(gi$numVars, 2)
    checkEquals(gi$varInfo$fcol$levels, c("wed"))
    
    df3 = data.frame(fcol=c("sun", "fri", "fri", "wed", "thu"), icol=c(4, 5, 6, 7, 8))
    checkEquals(as.integer(df3[,1]), c(2,1,1,4,3))
    
    .verifyParition = function(part) {
        checkEquals(NROW(part$varInfo$fcol$levels), 1)
        if(part$varInfo$fcol$levels[1] == "wed") {
            checkEquals(gi$data, data.frame(fcol=c("wed","wed","wed"), icol=c(1,3,7)))
        } else if(part$varInfo$fcol$levels[1] == "thu") {
            checkEquals(gi$data, data.frame(fcol=c("thu","thu"), icol=c(2,8)))
        } else if(part$varInfo$fcol$levels[1] == "fri") {
            checkEquals(gi$data, data.frame(fcol=c("fri","fri"), icol=c(5,6)))
        } else if(part$varInfo$fcol$levels[1] == "sun") {
            checkEquals(gi$data, data.frame(fcol=c("sun"), icol=c(4)))
        } else {
            checkEquals(TRUE, FALSE)
        }
    }
    
    # append to existing partitions, adding new data
    partDF <- rxPartition(inData = df3, outData = outXdfDataSource, varsToPartition = c("fcol"))
    
    gi = rxGetInfo(partDF[[2]][[1]], getVarInfo = TRUE, numRows = 3)
    .verifyParition(gi)

    gi = rxGetInfo(partDF[[2]][[2]], getVarInfo = TRUE, numRows = 3)
    .verifyParition(gi)
    
    gi = rxGetInfo(partDF[[2]][[3]], getVarInfo = TRUE, numRows = 3)
    .verifyParition(gi)
    
    gi = rxGetInfo(partDF[[2]][[4]], getVarInfo = TRUE, numRows = 3)
    .verifyParition(gi)
}


"test.rxPartition.factor.NAs" <- function() 
{
  # Setup the input and output data paths.
  inFile <- file.path(rxGetOption("unitTestDataDir"), "tinyFactorNA.csv")
  
  tmpPat <- "rxPartitionFactorNA"
  outFile <- tempfile(pattern = tmpPat)
  
  # Remove any artifacts from a pervious test.
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
  
  # create input and output Xdf data source
  inTxtDataSource <- RxTextData(file = inFile, delimiter = ",", stringsAsFactors = TRUE)
  inDF <- rxImport(inData = inTxtDataSource)
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)
  
  # partition on MixedNA column (NA & <NA> from empty strings/missing values)
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("MixedNA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 4, 10)
  
  # partition on EmptyNA column ("")
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("EmptyNA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 3, 10)
  
  # partition on NANA column (NA from NA values)
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("NANA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 3, 10)
  
  # partition on BlankNA column (<NA> from missing values)
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("BlankNA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 3, 10)
  
  # partition on combined columns 
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("MixedNA", "NANA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 4, 10)
  
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("EmptyNA", "NANA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 3, 10)
  
  partDF <- rxPartition(inData = inDF, outData = outXdfDataSource, varsToPartition = c("BlankNA", "NANA"), append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 3, 10)
}

"test.rxPartition.datatypes" <- function() 
{
  # Setup the input and output data paths.
  inFile <- "claims.xdf"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
  
  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)
  
  # Remove any artifacts from a pervious test.
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE) , add = TRUE)
  
  # create input and output Xdf data source
  inXdfDataSource <- RxXdfData(file = inFile)
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)
  
  # int
  varsToPartition <- c("RowNum")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 128, 128)
  
  # varchar
  varsToPartition <- c("age")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 8, 128)
  
  # varchar
  varsToPartition <- c("type")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 4, 128)
  
  # number(8,2)
  varsToPartition <- c("number")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 80, 128)
  
  # number(8,2)
  varsToPartition <- c("cost")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 100, 128)
  
  # combined different data types
  varsToPartition <- c("age", "type")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 32, 128)
  
  varsToPartition <- c("number", "type")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 106, 128)
  
  varsToPartition <- c("RowNum", "age", "number", "type")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, append = "none", overwrite = TRUE)
  .verifyRowCount(partDF$DataSource, 128, 128)
}


"test.rxPartition.XdfDataSource.partition.varsToKeep" <- function() {
  inFile <- "claims.xdf"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
  varsToKeep = c("car.age", "number")
  inXdfDataSource <- RxXdfData(file = inFile, varsToKeep = varsToKeep)
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)
  varsToPartition <- c("car.age")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, overwrite = TRUE)
  checkEquals(NROW(partDF$car.age), 4)
  checkEquals(levels(partDF$car.age), c("0-3", "4-7", "8-9", "10+"))
  checkEquals(colnames(partDF[[2]][[1]]), varsToKeep)
  checkEquals(colnames(partDF[[2]][[2]]), varsToKeep)
  checkEquals(colnames(partDF[[2]][[3]]), varsToKeep)
  checkEquals(colnames(partDF[[2]][[4]]), varsToKeep)
}


"test.rxPartition.XdfDataSource.partition.varsToDrop" <- function() {
  inFile <- "claims.xdf"
  inFile <- file.path(dataPath = rxGetOption(opt = "sampleDataDir"), inFile)
  tmpPat <- "rxPartitionClaims"
  outFile <- tempfile(pattern = tmpPat)
  unlink(x = outFile, recursive = TRUE, force = TRUE)
  on.exit(unlink(x = outFile, recursive = TRUE, force = TRUE), add = TRUE)
  varsToDrop = c("age", "number")
  inXdfDataSource <- RxXdfData(file = inFile, varsToDrop = varsToDrop)
  allcols <- colnames(inXdfDataSource)
  outXdfDataSource <- RxXdfData(file = outFile, createPartitionSet = TRUE)
  varsToPartition <- c("car.age")
  partDF <- rxPartition(inData = inXdfDataSource, outData = outXdfDataSource, varsToPartition = varsToPartition, overwrite = TRUE)
  checkEquals(NROW(partDF$car.age), 4)
  checkEquals(levels(partDF$car.age), c("0-3", "4-7", "8-9", "10+"))
  correctCols = colnames(inXdfDataSource)
  checkEquals(colnames(partDF[[2]][[1]]), correctCols)
  checkEquals(colnames(partDF[[2]][[2]]), correctCols)
  checkEquals(colnames(partDF[[2]][[3]]), correctCols)
  checkEquals(colnames(partDF[[2]][[4]]), correctCols)
}
