# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxPredict
#
############################################################################################

# rUnit should create these objects just before defining and executing the tests in this file,
# then clean them up automatically after all tests in the file are finished.
unitTestDataDir <- rxGetOption("unitTestDataDir")
# Instead use R's tempdir()
testOutputDir <- file.path(tempdir(), "testOutput")
sampleDataDir <- rxGetOption("sampleDataDir")
require(MASS)

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

.setUp <- function()
{
  if (file.exists(testOutputDir))
  {
    unlink(testOutputDir,recursive=TRUE)
  }
  dir.create(testOutputDir)
}

.tearDown <- function()
{
  unlink(testOutputDir,recursive=TRUE)
}

test.rxPredict.data.iris <- function()
{
  # Fix for bug RA-4793
    myData <- iris

    form <- Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species
    linModObj <- rxLinMod(form, covCoef=TRUE, data=myData)
    myData$SepalLengthPred <- rxPredict(modelObject=linModObj, data=myData)
    
    lmObj <- lm(form, data=myData)
    outList <- rxPredict(modelObject=linModObj, data=myData, computeResiduals=TRUE,
          computeStdErrors=TRUE, interval="confidence", confLevel=0.975)
    checkEquals(names(outList)[1], "Sepal.Length_Pred")
    checkEquals(names(outList)[2], "Sepal.Length_StdErr")
  checkEquals(names(outList)[3], "Sepal.Length_Lower")
  checkEquals(names(outList)[4], "Sepal.Length_Upper")
  checkEquals(names(outList)[5], "Sepal.Length_Resid")
  
    checkTrue(all.equal(as.vector(lmObj$residuals),outList[[5]]))
    
    myNewData <- rxPredict(modelObject=linModObj, data=myData, outData=myData, computeResiduals=TRUE)
    checkTrue(all.equal(as.vector(lmObj$residuals), myNewData$Sepal.Length_Resid))
  
  ##
  ## Now check prediction standard errors and confidence limits
  ## 
  lmPred <- predict(object=lmObj, newdata=myData, se.fit=TRUE, 
            interval="confidence", level=0.975, type="response")
  checkEqualsNumeric(lmPred$se.fit, outList$Sepal.Length_StdErr)
  checkEqualsNumeric(lmPred$fit[ ,"lwr"], outList$Sepal.Length_Lower)
  checkEqualsNumeric(lmPred$fit[, "upr"], outList$Sepal.Length_Upper)
  rm(lmPred)
}

test.rxPredict.data.mortDefaultSmallXdf <- function()
{
  dataFile <- file.path(sampleDataDir, "mortDefaultSmall.xdf")
    outFile <- file.path(testOutputDir, "mortPredict.xdf")
    outFile2 <- file.path(testOutputDir, "mortDefaultSmallAppend.xdf")
    file.copy(from = dataFile, to = outFile2, overwrite = TRUE, recursive = FALSE) 
  myDF <- rxReadXdf(file = dataFile)
    
    # Compare first hundred residuals from rxLinMod with lm
  linModObj <- rxLinMod(creditScore ~ yearsEmploy, data = dataFile, covCoef=TRUE)
    rxPredict(modelObject = linModObj, data = dataFile, outData = outFile, computeResiduals = TRUE,
        computeStdErrors=TRUE, interval="prediction", overwrite=TRUE)  ## default confLevel=0.95
    residDF <- rxReadXdf(file = outFile)
    lmObj <- lm(creditScore ~ yearsEmploy, data = myDF)
    checkTrue(all.equal(as.vector(lmObj$residuals)[1:100], residDF$creditScore_Resid[1:100]))
    
  ## 
  ## Test prediction standard errors and prediction limits
  ## 
  lmPred <- predict(object=lmObj, newdata=myDF, se.fit=TRUE, interval="prediction", type="response")
  checkEqualsNumeric(lmPred$se.fit, residDF$creditScore_StdErr)
  checkEqualsNumeric(lmPred$fit[ ,"lwr"], residDF$creditScore_Lower)
  checkEqualsNumeric(lmPred$fit[, "upr"], residDF$creditScore_Upper)
  rm(lmPred)

  
    # Run rxPredict with various reporting arguments to make sure it runs
    rxPredict(modelObject = linModObj, data = dataFile, outData = outFile, 
              computeResiduals = TRUE, verbose = 1, overwrite = TRUE)
    rxPredict(modelObject = linModObj, data = dataFile, outData = outFile, 
              computeResiduals = TRUE, reportProgress = 1, overwrite = TRUE)
    rxPredict(modelObject = linModObj, data = dataFile, outData = outFile, 
              computeResiduals = TRUE, reportProgress = 2, overwrite = TRUE)
    
    # Append residuals to original file
    linModObj <- rxLinMod(creditScore ~ yearsEmploy, data = outFile2)
    rxPredict(modelObject = linModObj, data = outFile2, outData = outFile2, 
              computeResiduals = TRUE, overwrite = TRUE)
    colNamesLM <- rxGetVarNames(outFile2)
    checkTrue(colNamesLM[7] == "creditScore_Pred")
    checkTrue(colNamesLM[8] == "creditScore_Resid")
    
    # Test logit object  
    file.copy(from = dataFile, to = outFile2, overwrite = TRUE, recursive = FALSE)
    
    # Make sure copy overwrote file
    colNames <- names(rxGetVarInfo(outFile2))
    checkTrue(length(colNames) == 6)
    
    # Create logit model object and compute predictions
    logitModObj <- rxLogit(default ~ creditScore, data = outFile2, covCoef=TRUE)
    rxPredict(modelObject = logitModObj, data = outFile2, computeResiduals = TRUE,
        computeStdErrors=TRUE, interval="confidence")
    colNames <- names(rxGetVarInfo(outFile2))
    checkTrue(colNames[7] == "default_Pred")
    residDF <- rxReadXdf(file=outFile2)
    glmObj <- glm(default~creditScore, data=myDF, family = binomial())
    checkTrue(all.equal.numeric(as.vector(glmObj$fitted.values)[1:100], residDF$default_Pred[1:100], tolerance=.00001))
    checkTrue(all.equal.numeric(as.vector(residuals(glmObj, type="response"))[1:100], residDF$default_Resid[1:100], tolerance=.00001))
  
  ##
  ## Check prediction standard errors on the response scale; confidence bounds not checked
  ## 
  glmPred <- predict(object=glmObj, type="response", se.fit=TRUE)
  checkEqualsNumeric(glmPred$se.fit, residDF$default_StdErr, scale=1)
  rm(glmPred)
  
  if (file.exists(outFile)) file.remove(outFile)
  if (file.exists(outFile2)) file.remove(outFile2)
 } 

"test.rxPredict.args.modelObject.multipleInteractions" <- function()
{
# Check predictions with multiple interactions
    myData <- data.frame(
            xfac1 = factor(rep(c("a", "b", "c", "d"), times = 20)),
            xfac2 = factor(rep(c("x", "y", "z", "w"), each = 20)),
            xint = rep(1:8, times = 10))
    set.seed(8)
    myData$y <- (myData$xint*(myData$xfac1 == "a") + 2*myData$xint*(myData$xfac1 == "b") + 
                3*myData$xint*(myData$xfac1 == "c") +  myData$xint*(myData$xfac2 == "x") + 
                2*myData$xint*(myData$xfac2 == "y") + 3*myData$xint*(myData$xfac2 == "z") + 10*rnorm(80)) > 12
                
    logitModObj <- rxLogit(y ~ xfac1 + xfac2 + xint + xfac1:xint + xfac2:xint, data = myData)
    logitPred <- rxPredict(modelObject = logitModObj, data = myData)
    
    glmObj <- glm(y ~ xfac1 + xfac2 + xint + xfac1:xint + xfac2:xint, data = myData, family = binomial())
    
    checkEquals(as.vector(logitPred$y_Pred), as.vector(glmObj$fitted.values))
}

    
test.rxPredict.data.MissingsBlocks.test100 <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
    test100r5bStartMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bStartMiss")
    test100r5bEndMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bEndMiss")
    test77r1bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r1bNoMiss")
    test77r5bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r5bNoMiss")
    outFile100 <- file.path(testOutputDir, "testPredict100.xdf")
    outFile77 <- file.path(testOutputDir, "testPredict77.xdf")
    
    testDataFrame100 <- rxReadXdf(file=test100r1b)
    testDataFrame77 <- rxReadXdf(file=test77r1bNoMiss)
    testDataFrame100StartMiss <- rxReadXdf(file=test100r5bStartMiss)
    testDataFrame100EndMiss <- rxReadXdf(file=test100r5bEndMiss)
    
  # Use lm to compare data sets with and without missings
    lmObj100 <- lm(y~xnum1+xnum2miss+ xint1, data=testDataFrame100)
    lmObj77 <- lm(y~xnum1+xnum2miss+ xint1, data=testDataFrame77)
    lmObj100StartMiss <- lm(y~xnum1+xnum2miss+ xint1, data=testDataFrame100StartMiss)
    lmObj100EndMiss <- lm(y~xnum1+xnum2miss+ xint1, data=testDataFrame100EndMiss)
    
    # Compare rxLinMod residuals without missings with just one block
    linModObj77 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test77r1bNoMiss)
    rxPredict(modelObject=linModObj77, data=test77r1bNoMiss, outData=outFile77, computeResiduals=TRUE)
    residDF <- rxReadXdf(file=outFile77)
    checkTrue(all.equal(as.vector(lmObj77$residuals), as.vector(residDF$y_Resid)))
    file.remove(outFile77)
    rm(residDF)
    
    # Compare rxLinMod residuals without missings with 5 blocks
    linModObj77 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test77r5bNoMiss)
    rxPredict(modelObject=linModObj77, data=test77r5bNoMiss, outData=outFile77, computeResiduals=TRUE)
    residDF <- rxReadXdf(file=outFile77)
    checkTrue(all.equal(as.vector(lmObj77$residuals), as.vector(residDF$y_Resid)))
    file.remove(outFile77)
    rm(residDF)
    
    # Compare rxLinMod residuals:  1 block with missings
    linModObj1 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r1b)
    rxPredict(modelObject=linModObj1, data=test100r1b, outData=outFile100, computeResiduals=TRUE)
    residDF <- rxReadXdf(file=outFile100)
    # Remove missings from results since R does
    residDF <- residDF[which(!is.na(residDF$y_Resid)),]
    checkTrue(all.equal(as.vector(lmObj100$residuals), as.vector(residDF$y_Resid)))
    file.remove(outFile100)
    rm(residDF)
    
    # Compare rxLinMod:  5 blocks with missings, missings removed
    linModObj2 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5b)
    rxPredict(modelObject=linModObj2, data=test100r5b, outData=outFile77, 
    computeResiduals=TRUE, removeMissings=TRUE)
    residDF <- rxReadXdf(file=outFile77)
    checkTrue(all.equal(as.vector(lmObj77$residuals), as.vector(residDF$y_Resid)))
    file.remove(outFile77)
    rm(residDF)
 
    # Compare rxLinMod:  5 blocks with missings
    linModObj2 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5b)
    rxPredict(modelObject=linModObj2, data=test100r5b, outData=outFile100, computeResiduals=TRUE)
    residDF <- rxReadXdf(file=outFile100)
    checkTrue(length(as.vector(residDF$y_Resid)) == 100)
    # Remove missings since R does
    residDF <- residDF[which(!is.na(residDF$y_Resid)),]
    checkTrue(all.equal(as.vector(lmObj100$residuals), as.vector(residDF$y_Resid)))
    file.remove(outFile100)
    rm(residDF)
    
    # Compare rxLinMod: 5 blocks with missings first
    linModObj3 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5bStartMiss)
    rxPredict(modelObject=linModObj3, data=test100r5bStartMiss, outData=outFile100, computeResiduals=TRUE)
    residDF <- rxReadXdf(file=outFile100)
    # Remove missings since R does
    residDF <- residDF[which(!is.na(residDF$y_Resid)),]
    checkTrue(all.equal(as.vector(lmObj100StartMiss$residuals), as.vector(residDF$y_Resid)))
    file.remove(outFile100)
    rm(residDF)
    
    # Compare rxLinMod: 5 blocks with missings last
    linModObj4 <- rxLinMod(y~xnum1+xnum2miss+ xint1, data=test100r5bEndMiss)
    rxPredict(modelObject=linModObj4, data=test100r5bEndMiss, outData=outFile100, computeResiduals=TRUE)
    residDF <- rxReadXdf(file=outFile100)
    checkTrue(length(as.vector(residDF$y_Resid)) == 100)
    # Remove missings since R does
    residDF <- residDF[which(!is.na(residDF$y_Resid)),]
    checkTrue(all.equal(as.vector(lmObj100EndMiss$residuals), as.vector(residDF$y_Resid)))  
    file.remove(outFile100)
    rm(residDF)

}

"test.rxPredict.data.var.order" <- function()
{
  # testOutputDir <- getwd()
  testXdf1 <- file.path(testOutputDir, ".rxTestVarOrder1.xdf")		
  testXdf2 <- file.path(testOutputDir, ".rxTestVarOrder2.xdf")
  set.seed(50)
  income <- rep(c(1000,1500,2500,4000), each=5) + 100*rnorm(20)
  region <- factor(rep(c("Urban","Rural"), each=10))
  sex <- rep(c("Female", "Male"), each=5)
  sex <- factor(c(sex,sex))
  x1 <- rnorm(20)
  myData1 <- data.frame(income, x1, region, sex)
  rxDataStep(inData =myData1, outFile=testXdf1, overwrite=TRUE)
  myData2 <- data.frame(income, sex, region, x1)
  rxDataStep(inData =myData2, outFile=testXdf2, overwrite=TRUE)
  
  # LinMod with xdf, predictions with data frame
  linMod1 <- rxLinMod(income ~ region:sex, data = testXdf1)
  myDataPredict1 <- myData1
  myDataPredict1 <- rxPredict(linMod1, data=myDataPredict1, outData=myDataPredict1)
  myDataPredict2 <- myData2
  myDataPredict2 <- rxPredict(linMod1, data=myDataPredict2, outData=myDataPredict2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)
  
  # Linmod and predictions with data frame
  linMod1 <- rxLinMod(income~region:sex, data=myData1)
  myDataPredict1 <- myData1
  myDataPredict1 <- rxPredict(linMod1, data=myDataPredict1, outData=myDataPredict1)
  myDataPredict2 <- myData2
  myDataPredict2 <- rxPredict(linMod1, data=myDataPredict2, outData=myDataPredict2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)
  
  # Linmod and predictions with xdf file
  linMod1 <- rxLinMod(income ~ region:sex, data = testXdf1)
  rxPredict(linMod1, data = testXdf1, outData = testXdf1, overwrite = TRUE)
  rxPredict(linMod1, data = testXdf2, outData = testXdf2, overwrite = TRUE)
  myDataPredict1 <- rxReadXdf(testXdf1)
  myDataPredict2 <- rxReadXdf(testXdf2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)
  
  # Linmod and predictions with additional term
  linMod1 <- rxLinMod(income ~ x1 + region:sex, data = myData1)
  myDataPredict1 <- myData1
  myDataPredict1 <- rxPredict(linMod1, data=myDataPredict1, outData = myDataPredict1)
  myDataPredict2 <- myData2
  myDataPredict2 <- rxPredict(linMod1, data = myDataPredict2, outData = myDataPredict2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)	
  
  # Linmod and predictions with cube=TRUE additional term
  linMod1 <- rxLinMod(income~region:sex + x1, data=myData1, cube=TRUE)
  myDataPredict1 <- myData1
  myDataPredict1 <- rxPredict(linMod1, data=myDataPredict1, outData=myDataPredict1)
  myDataPredict2 <- myData2
  myDataPredict2 <- rxPredict(linMod1, data=myDataPredict2, outData=myDataPredict2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)	

  # Continous/factor interactions with data frame
  linMod1 <- rxLinMod(income~sex + x1:region, data=myData1, covCoef=TRUE)
  lm1 <- lm(income~sex + x1:region, data=myData1, contrasts = list(region = "contr.SAS", sex="contr.SAS"))
  linMod1Coef <- as.numeric(coef(linMod1))
  linMod1Coef <- linMod1Coef[!is.na(linMod1Coef)]
  lm1Coef <- as.numeric(coef(lm1))
  checkEquals(linMod1Coef, lm1Coef)
  myDataPredict1 <- myData1
  myDataPredict1 <- rxPredict(linMod1, data=myDataPredict1, outData=myDataPredict1)
  myDataPredict2 <- myData2
  myDataPredict2 <- rxPredict(linMod1, data=myDataPredict2, outData=myDataPredict2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)	
  
  ##
  ## Check prediction standard errors and intervals with continuous/factor interactions 
  ##
  
  # Fix for bug RA-4793
  
  rxPredData1SE <- rxPredict(linMod1, data=myDataPredict1, computeStdErrors=TRUE, interval="confidence",
          computeResiduals=FALSE)
  predData1SE   <- predict.lm(lm1, newdata=myData1, se.fit=TRUE, interval="confidence", type="response")
  checkEqualsNumeric(rxPredData1SE$income_StdErr, predData1SE$se.fit)
  checkEqualsNumeric(rxPredData1SE$income_Lower, predData1SE$fit[ ,"lwr"])
  checkEqualsNumeric(rxPredData1SE$income_Upper, predData1SE$fit[ ,"upr"])
  rm(rxPredDataSE, predData1SE)
    
  # Continuous/factor iteractions with xdf
  linMod1 <- rxLinMod(income ~ sex + x1:region, data=testXdf1)
  myDataPredict1 <- myData1
  myDataPredict1 <- rxPredict(linMod1, data = myDataPredict1, outData = myDataPredict1)
  myDataPredict2 <- myData2
  myDataPredict2 <- rxPredict(linMod1, data = myDataPredict2, outData = myDataPredict2)
  checkEquals(myDataPredict1$income_Pred, myDataPredict2$income_Pred)	
  
  if (file.exists(testXdf1)) file.remove(testXdf1)
  if (file.exists(testXdf2)) file.remove(testXdf2)
}

"test.rxPredict.args.checkFactorLevels" <- function()
{
    # copy the data source to a temporary directory and work from there so that
    # the original file is not ovwerwritten
  censusWorkersPkg <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
  censusWorkers <- file.path(tempdir(), "CensusWorkers.xdf")
    file.copy(censusWorkersPkg, censusWorkers, overwrite = TRUE)
   
  # Interacted factors, one mismatched
  linMod <- rxLinMod(incwage~sex:state, data=censusWorkers, cube=TRUE) 
  myData <- data.frame(sex=factor(c("Male", "Female", "Male", "Male")), incwage=c(1,1,1,1),
      state = factor(c("Connecticut", "Indiana", "Washington", "Indiana")))
    checkException(rxPredict(linMod, data=myData, outData=myData), silent = TRUE) 
  
  myPredict <- rxPredict(linMod, data=myData, outData=myData, checkFactorLevels=FALSE)
  linModCoef <- coef(linMod)
  # The first prediction (sex=Male, state=Connecticut) will be mismatched with the first coef
  checkEquals(linModCoef["sex=Female, state=Connecticut"], myPredict$incwage_Pred[1], checkNames=FALSE)
   
  rm(linMod)
  rm(myPredict)
  rm(linModCoef)
  
  # Switching order of mismatched factors
  linMod1 <- rxLinMod(incwage ~ state:sex, data = censusWorkers, cube = TRUE)
    checkException(rxPredict(linMod1, data = myData, outData = myData), silent = TRUE)

  myPredict <- rxPredict(linMod1, data=myData, outData=myData, checkFactorLevels=FALSE)
  linModCoef1 <- coef(linMod1)
  # The first prediction (state=Connecticut, sex=Male) will be mismatched with the first coef
  checkTrue(linModCoef1["state=Connecticut, sex=Male"] > myPredict$incwage_Pred[1])
  
  rm(myPredict)
  # Check that correctly matched factors work
  myData1 <- data.frame(sex=factor(c("Male", "Female", "Male", "Male"), levels=c("Male", "Female")), 
    incwage=c(1,1,1,1),	state = factor(c("Connecticut", "Indiana", "Washington", "Indiana"))) 
  testFile <- file.path(tempdir(), ".rxTestFactors.xdf")
  rxDataStep(inData = myData1, outFile = testFile, overwrite = TRUE)
  rxPredict(linMod1, data = testFile, outData = testFile, overwrite = TRUE)
  myPredict <- rxReadXdf(testFile)
  checkEquals(linModCoef1["state=Connecticut, sex=Male"], myPredict$incwage_Pred[1], checkNames = FALSE)
  checkEquals(linModCoef1["state=Indiana, sex=Female"], myPredict$incwage_Pred[2], checkNames = FALSE)
  checkEquals(linModCoef1["state=Washington, sex=Male"], myPredict$incwage_Pred[3], checkNames = FALSE)
  checkEquals(linModCoef1["state=Indiana, sex=Male"], myPredict$incwage_Pred[4], checkNames = FALSE)
  
  # Do the same with a data frame
  rm(myPredict)
  myPredict <- rxPredict(linMod1, data = myData1, outData = myData1)
  checkEquals(linModCoef1["state=Connecticut, sex=Male"], myPredict$incwage_Pred[1], checkNames = FALSE)
  checkEquals(linModCoef1["state=Indiana, sex=Female"], myPredict$incwage_Pred[2], checkNames = FALSE)
  checkEquals(linModCoef1["state=Washington, sex=Male"], myPredict$incwage_Pred[3], checkNames = FALSE)
  checkEquals(linModCoef1["state=Indiana, sex=Male"], myPredict$incwage_Pred[4], checkNames = FALSE)
  
  # Change the order of variables in the data frame
  myData2 <- data.frame(
      state = factor(c("Connecticut", "Indiana", "Washington", "Indiana")),
      sex = factor(c("Male", "Female", "Male", "Male"), levels = c("Male", "Female")), 
    incwage = c(1,1,1,1)) 
  myPredict <- rxPredict(linMod1, data=myData2, outData=myData2)
  checkEquals(linModCoef1["state=Connecticut, sex=Male"], myPredict$incwage_Pred[1], checkNames = FALSE)
  checkEquals(linModCoef1["state=Indiana, sex=Female"], myPredict$incwage_Pred[2], checkNames = FALSE)
  checkEquals(linModCoef1["state=Washington, sex=Male"], myPredict$incwage_Pred[3], checkNames = FALSE)
  checkEquals(linModCoef1["state=Indiana, sex=Male"], myPredict$incwage_Pred[4], checkNames = FALSE) 
  
  if (file.exists(testFile)) file.remove(testFile)
  if (file.exists(censusWorkers)) file.remove(censusWorkers)
}
  
"test.rxPredict.args.checkFactorLevels.additiveMainEffects" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
  #> rxGetVarInfo(censusWorkers)
  #Var 1: age, Age
       #Type: integer, Low/High: (20, 65)
  #Var 2: incwage, Wage and salary income
       #Type: integer, Low/High: (0, 354000)
  #Var 3: perwt, Type: integer, Low/High: (2, 168)
  #Var 4: sex, Sex
       #2 factor levels: Male Female
  #Var 5: wkswork1, Weeks worked last year
       #Type: integer, Low/High: (21, 52)
  #Var 6: state
       #3 factor levels: Connecticut Indiana Washington
  
  linMod <- rxLinMod(incwage ~ sex + state, data = censusWorkers)
  
  # only the first factor mismatched
  myData1 <- data.frame(
    sex = factor(c("Male", "Female", "Male", "Male"), levels = c("Female", "Male")), 
    incwage = c(1, 1, 1, 1),
      state = factor(c("Connecticut", "Indiana", "Washington", "Indiana"), levels = c("Connecticut", "Indiana", "Washington"))
    )
    checkException(rxPredict(linMod, data = myData1), silent = TRUE)
  
  # only the second factor mismatched
  myData2 <- data.frame(
    sex = factor(c("Male", "Female", "Male", "Male"), levels = c("Male", "Female")), 
    incwage = c(1, 1, 1, 1),
      state = factor(c("Connecticut", "Indiana", "Washington", "Indiana"), levels = c("Connecticut", "Washington", "Indiana"))
    )
    checkException(rxPredict(linMod, data = myData2), silent = TRUE)
}
  
###
# argument stress tests
###

"test.rxPredict.args.blocksPerRead" <- function()
{    
    XDF <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFileDefault <- tempfile( pattern = ".rxPredict1", fileext = ".xdf")
    outFileBlock1 <-  tempfile( pattern = ".rxPredict2", fileext = ".xdf")
    outFileBlock5 <-  tempfile( pattern = ".rxPredict3", fileext = ".xdf")
    file.copy(from=XDF, to=outFileDefault, overwrite = TRUE, recursive=FALSE) 
    file.copy(from=XDF, to=outFileBlock1, overwrite = TRUE, recursive=FALSE) 
    file.copy(from=XDF, to=outFileBlock5, overwrite = TRUE, recursive=FALSE) 

  form <- xnum1 ~ F(y) + xnum2miss
  zLinMod <- rxLinMod(form, data=XDF)
  
  rxPredict(zLinMod, data = XDF, blocksPerRead = -1L, outData = outFileDefault, overwrite = TRUE)
  rxPredict(zLinMod, data = XDF, blocksPerRead = 1L, outData = outFileBlock1, overwrite = TRUE)
  rxPredict(zLinMod, data = XDF, blocksPerRead = 5L, outData = outFileBlock5, overwrite = TRUE)
  zPredBlockDefault <- rxReadXdf(outFileDefault)
  zPredBlock1 <- rxReadXdf(outFileBlock1)
  zPredBlock5 <- rxReadXdf(outFileBlock5)
  
  checkEqualsNumeric(zPredBlockDefault, zPredBlock1)
  checkEqualsNumeric(zPredBlockDefault, zPredBlock5)
    
  if (file.exists(outFileDefault)) file.remove(outFileDefault)
  if (file.exists(outFileBlock1)) file.remove(outFileBlock1)
  if (file.exists(outFileBlock5)) file.remove(outFileBlock5)
}

"test.rxPredict.args.computeResiduals" <- function()
{
  # create data 
  form <- y ~ x
  set.seed(100)
  DF <- data.frame(y = runif(10), x = runif(10))
  
  # create linear regression model
  zLinModDF <- rxLinMod(form, data=DF)
  
  # create predictions
  predResidFalse <- rxPredict(zLinModDF, data = DF, outData = NULL, computeResiduals = FALSE)
  predResidTrue <- rxPredict(zLinModDF, data = DF, outData = NULL, computeResiduals = TRUE)
  
  # check output
  checkTrue(is.data.frame(predResidFalse))
  checkTrue(is.data.frame(predResidTrue))
  checkTrue("y_Resid" %in% names(predResidTrue))
}

"test.rxPredict.args.data" <- function()
{	
  # create data
  form <- y ~ x
  set.seed(10)
  DF <- data.frame(y=rnorm(10), x=rnorm(10))
  XDF <- tempfile( pattern = ".rxPredict4", fileext = ".xdf")
  rxDataStep(DF, XDF, overwrite=TRUE)
  
  # create models for data frame and XDF data sources
  zLinModDF <- rxLinMod(form, data=DF)
  zLinModXDF <- rxLinMod(form, data=XDF)
  
  # calculate and obtain predictions
  predDF <- rxPredict(zLinModDF, data=DF)
  rxPredict(zLinModXDF, data=XDF)
  predXDF <- rxReadXdf(XDF)
  
  # perform basic test on residuals
  checkEquals(predDF$y_Pred, predXDF$y_Pred)
  checkEqualsNumeric(sum(predDF$y_Pred - DF$y), 0.0)
  checkEqualsNumeric(sum(predXDF$y_Pred - DF$y), 0.0)
  file.remove(XDF)
}

"test.rxPredict.args.outData.differentFile" <- function()
{

  testFile <- tempfile(pattern = ".rxPred1TempTest", fileext = ".xdf") 
  testPredFile <- tempfile(pattern = ".rxPred1TempPred", fileext = ".xdf") 
  testPredBad <- tempfile(pattern = ".rxPred1TempBad", fileext = ".xdf") 
  rxDataStep(inData = infert, outFile = testFile, rowsPerRead = 100, overwrite = TRUE)
  rxDataStep(inData = infert, outFile = testPredBad, rowsPerRead = 120, overwrite = TRUE)
  
  # Estimate a model
  rxLogitOut1 <- rxLogit(case ~ spontaneous + induced, data=testFile )
  
  # Write results and model vars to new file
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
    outData = testPredFile, writeModelVars = TRUE,
    predVarNames = "casePred1")
  
  varInfo1 <- rxGetVarInfo(rxPredOut)
  checkEquals(names(varInfo1), c("casePred1", "case", "spontaneous", "induced"))
  
  # Estimate another model
  rxLogitOut2 <- rxLogit(case ~ spontaneous + induced + parity, data=testFile )
  
  # Write predictions to input file
  varInfoOrig <- rxGetVarInfo(testFile)
  rxPredOut2 <- rxPredict(modelObject = rxLogitOut2, data = testFile, 
    outData = testFile, predVarNames = "casePred2")
  
  varInfo2 <- rxGetVarInfo(rxPredOut2)
  checkEquals(names(varInfo2), c(names(varInfoOrig), "casePred2"))
  
  # Write predictions out to first output predictions file
  rxPredOut3 <- rxPredict(modelObject = rxLogitOut2, data = testFile, 
    outData = testPredFile, predVarNames = "casePred2")
  
  varInfo3 <- rxGetVarInfo(rxPredOut3)
  checkEquals(names(varInfo3), c("casePred1", "case", "spontaneous", "induced", "casePred2"))
  
    if (!rxIsExpressEdition())
    {
      # Try writing to an .xdf file with mis-matched blocks
      res <- try(
        rxPredict(modelObject = rxLogitOut2, data = testFile, 
          outData = testPredBad, predVarNames = "casePred2")
      )
      checkTrue(is(res, "try-error"))
    }
  
  # Should work if we set overwrite to TRUE
  rxPredOut4 <- rxPredict(modelObject = rxLogitOut2, data = testFile, 
      outData = testPredBad, predVarNames = "casePred2", overwrite = TRUE)
  varInfo4 <- rxGetVarInfo(rxPredOut4)
  checkEquals(names(varInfo4), "casePred2")
  
  file.remove(testPredFile)
  file.remove(testFile)
  file.remove(testPredBad)

}


"test.rxPredict.args.outData.multiblock" <- function()
{
  myDataXdf <- file.path(tempdir(), ".rxMyData.xdf")
  predXdf <- file.path(tempdir(), ".rxPred.xdf")
  set.seed(12)
  x <- 1:100
  y <- x + rnorm(100)/10000
  myData <- data.frame(y, x)
  lmObj <- lm(y~x)
  myData <- data.frame(y, x, lmObj$fitted.values)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = myData, outFile = myDataXdf, overwrite = TRUE)
    } else
    {
      rxDataStep(myData[1:25,], outFile = myDataXdf, overwrite=TRUE)
      rxDataStep(myData[26:50,], outFile = myDataXdf, append="rows")
      rxDataStep(myData[51:70,], outFile = myDataXdf, append="rows")
      rxDataStep(myData[71:100,], outFile = myDataXdf, append="rows")
    }
  linModObj <- rxLinMod(y~x, data=myDataXdf)
  
  # Compare lm output, writing predictions to same data set
  rxPredict(linModObj, data=myDataXdf, outData=myDataXdf)
  myDataNew <- rxDataStep(myDataXdf)
  checkEquals(myDataNew$lmObj.fitted.values, myDataNew$y_Pred, checkNames=FALSE)
  
  # Write to a different data set
  rxPredict(linModObj, data=myDataXdf, outData=predXdf)
  predData <- rxDataStep(predXdf)
  checkEquals(myDataNew$y_Pred, predData$y_Pred)
  
  file.remove(myDataXdf)
  file.remove(predXdf)
}

"test.rxPredict.args.modelObject" <- function()
{
    irisXDF <-  file.path(tempdir(), "iris.xdf")
    rxDataStep(iris, irisXDF, overwrite = TRUE)
    outXDF <- file.path(tempdir(), "temp.xdf")

    # check error in attempt to predict on multiple dependent variables ...
    # ... formula
    form <- cbind(Sepal.Length, Sepal.Width) ~ Petal.Length + Petal.Width + Species
    checkException(rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF), silent = TRUE)
    
    # ... formula string
    DF <- data.frame(x = 1:5, y = 5:1, z = 2:6) 
    model <- rxLinMod("x,y~z", data = DF) 
    checkException(rxPredict(model, data = DF), silent = TRUE)

  # create data source from indigenous 'iris' data frame
  # add a binary variable for logistic regression
  DF <- iris[1:20,]
  DF$BigSepal <- as.integer(DF$Sepal.Length > 5)

  # create models for linear and logistic regression
  zLinMod <- rxLinMod(Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width, data=DF)
  zLogit <- rxLogit(BigSepal ~ Sepal.Width + Petal.Length + Petal.Width, data=DF)
  
  # create predictions using linear and logistic regression models
  DF$SepalLengthPred <- rxPredict(zLinMod, data=DF)
  DF$BigSepalPred <- rxPredict(zLogit, data=DF)
  
  # perform basic test on residuals
  checkEqualsNumeric(sum(DF$SepalLengthPred - DF$Sepal.Length), 0.0)
  checkEqualsNumeric(sum(DF$BigSepalPred - DF$BigSepal), 0.0, tol = 1e-6)
    
    if (file.exists(irisXDF)) file.remove(irisXDF)
    if (file.exists(outXDF)) file.remove(outXDF)  
}

"test.rxPredict.args.outData" <- function()
{
  # file or existing data frame to store predictions; can be same
  # as the input file or NULL. If not NULL, must be an .xdf
  # file if 'data' is an .xdf file or a data frame if 'data' is
  # a data frame.
  
  # create data for local data frame and XDF data sources
  form <- y ~ x
  set.seed(100)
  DF <- data.frame(y = runif(10), x = runif(10))
  XDF <- tempfile(pattern = ".rxPredict", fileext = ".xdf")
  rxDataStep(DF, XDF, overwrite=TRUE)
    newDF <- data.frame(y = runif(10), x = runif(10))
  
  # create linear regression models for data frame and XDF data sources
  zLinModDF <- rxLinMod(form, data = DF)
  zLinModXDF <- rxLinMod(form, data = XDF)
  
  # illegal values
  checkException(rxPredict(zLinModDF, outData = ""), silent = TRUE)
  checkException(rxPredict(zLinModDF, outData = c("cat","dog")), silent = TRUE)
    #TODO: the following line is not throwing an error, should it?
  #checkException(rxPredict(zLinModDF, data = newDF, outData = XDF, computeResiduals = FALSE), silent = TRUE)

  # create predictions, using NULL for output data type
  predDFNULL <- rxPredict(zLinModDF, data = DF, outData = NULL)
  predXDFNULL <- rxPredict(zLinModXDF, data = XDF, outData = NULL)
  checkEquals(normalizePath(predXDFNULL@file), normalizePath(XDF))
  predXDFNULL <- rxReadXdf(XDF)
  checkEquals(predDFNULL$y_Pred, predXDFNULL$y_Pred)
  
  # perform basic tests on residuals
  checkEqualsNumeric(sum(predDFNULL$y_Pred - DF$y), 0.0)
  checkEqualsNumeric(sum(predXDFNULL$y_Pred - DF$y), 0.0)

  # create predictions, using data frame for output data type
  predDFDF <- rxPredict(zLinModDF, data=DF, outData=DF)
  checkEquals(predDFDF, data.frame(DF, y_Pred=predDFNULL))
  
  # create predictions, using XDF file for output data type
  rxPredict(zLinModDF, data = XDF, outData = XDF, overwrite = TRUE)
  predXDFXDF <- rxReadXdf(XDF)
  checkEquals(predXDFXDF, data.frame(DF, y_Pred=predDFNULL))

  # test for I/O inconsistencies 
  #checkTrue(is.null(rxPredict(zLinModDF, data=DF, outData=XDF)), msg="Caught mixed I/O data sources: data frame to XDF")
  #checkTrue(is.null(rxPredict(zLinModXDF, data=XDF, outData=DF)))
  
  # clean up
  if (file.exists(XDF)) file.remove(XDF)
}

"test.rxPredict.args.outData.text" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    linModObj <- rxLinMod(cost~number + type, data = claimsXdf)	
  outCsvFile <- tempfile( pattern = ".rxPredict1", fileext = ".csv")
    outXdfFile <- tempfile( pattern = ".rxPredictA1", fileext = ".xdf")

  outDS <- RxTextData(outCsvFile)
  rxPredict(linModObj, data = claimsXdf, outData = outDS)
    predCsvDF <- rxImport(outDS)
    
    rxPredict(linModObj, data = claimsXdf, outData = outXdfFile)
    predXdfDF <- rxReadXdf(outXdfFile)
    
    checkEquals(predCsvDF, predXdfDF, tolerance = 1e-6)
  if (file.exists(outCsvFile)) file.remove(outCsvFile)
    if (file.exists(outXdfFile)) file.remove(outXdfFile)
    
    # Test writeModelVars
    
    rxPredict(linModObj, data = claimsXdf, outData = outDS, writeModelVars = TRUE)
    predCsvDF <- rxImport(outDS, colInfo = list(type = list(type = "factor", 
        levels = c("A", "B", "C", "D"))))
    
    rxPredict(linModObj, data = claimsXdf, outData = outXdfFile, writeModelVars = TRUE)
    predXdfDF <- rxReadXdf(outXdfFile)
    
    checkEquals(predCsvDF, predXdfDF, tolerance = 1e-6)
  if (file.exists(outCsvFile)) file.remove(outCsvFile)
    if (file.exists(outXdfFile)) file.remove(outXdfFile)   
    
    # Can we pass extra variables through with transformVars?
    
    linModObj <- rxLinMod(cost~number + type, data = claimsXdf, transformVars = "RowNum")	
    rxPredict(linModObj, data = claimsXdf, outData = outDS, writeModelVars = TRUE)
    predCsvDF <- rxImport(outDS, colInfo = list(type = list(type = "factor", 
        levels = c("A", "B", "C", "D"))))
    
    checkEquals(predCsvDF$RowNum, as.integer(1:128))

  if (file.exists(outCsvFile)) file.remove(outCsvFile)
    
     
}


"test.rxPredict.args.overwrite" <- function()
{
    irisXDF <-  file.path(tempdir(), "iris.xdf")
    rxDataStep(iris, irisXDF, overwrite = TRUE)
    outXDF <- file.path(tempdir(), "temp.xdf")

    # check conflict of column name in existing output .xdf file
    rxDataStep(cbind(iris, list("log.Sepal.Length._Pred" = rnorm(NROW(iris)))), 
        outXDF, overwrite = TRUE)
    form <- log(Sepal.Length) ~ Petal.Length + Petal.Width + Species
    checkException(
        rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF), silent = TRUE)
    
    # overwrite should create new outFile; this should work
    z <- try(
        rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF, overwrite = TRUE))
    checkTrue(!inherits(z, "try-error"))
    
    rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF,
        predVarNames = "myPredName", residVarNames = "myResName", computeResiduals = TRUE)
    
    # This should fail because 'myResName' is already in the output data set
    checkException(rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF,
        predVarNames = "myNewPredName", residVarNames = "myResName", computeResiduals = TRUE), silent = TRUE)
   
    z <- try(rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF,
        predVarNames = "myNewPredName", residVarNames = "myResName", computeResiduals = TRUE, overwrite = TRUE))
    checkTrue(!inherits(z, "try-error"))

  ## append rows
    z <- try(rxPredict(rxLinMod(form, data = irisXDF), data = irisXDF, outData = outXDF, 
        predVarNames = "myNewPredName", residVarNames = "myResName", computeResiduals = TRUE, append = "rows"))
    dim.data <- dim(rxDataStep(irisXDF))
  dim.outData <- dim(rxDataStep(outXDF))
    checkEquals(dim.outData[1], dim.data[1] * 2)
  
  if (file.exists(irisXDF)) file.remove(irisXDF)
  if (file.exists(outXDF)) file.remove(outXDF)
}

"test.rxPredict.args.append.rows" <- function()
{
  dataXdfSingle <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    (dataXdfSingle.info <- rxGetInfo(dataXdfSingle, getVarInfo = TRUE, getBlockSizes = TRUE))
  dataXdfMulti <- file.path(rxGetOption("sampleDataDir"), "claims4blocks.xdf")
    (dataXdfMulti.info <- rxGetInfo(dataXdfMulti, getVarInfo = TRUE, getBlockSizes = TRUE))
  dataXdfComp <- file.path(rxGetOption("sampleDataDir"), "claimsXdf")
    (dataXdfComp.info <- rxGetInfo(dataXdfComp, getVarInfo = TRUE, getBlockSizes = TRUE))

    form <- cost ~ age + car.age + type + number
    model <- rxLinMod(form, data = dataXdfSingle)

  ####
  ## xdf to xdf
  outDataXdf <- file.path(tempdir(), "outData.xdf")
  on.exit(if(file.exists(outDataXdf)) unlink(outDataXdf, recursive = TRUE, force = TRUE), add = TRUE)
    outDataXdfDs <-  RxXdfData(outDataXdf, createCompositeSet = FALSE)
    rxPredict(model, data = dataXdfMulti, outData = outDataXdfDs, extraVarsToWrite = "RowNum", overwrite = TRUE)
  (outDataXdfDs.info <- rxGetInfo(outDataXdfDs, getVarInfo = TRUE, getBlockSizes = TRUE))
    checkEquals(outDataXdfDs.info[["numRows"]] * 1, dataXdfSingle.info[["numRows"]])

  rxPredict(model, data = dataXdfSingle, outData = outDataXdfDs, extraVarsToWrite = "RowNum", append = "rows")
  (outDataXdfDs2.info <- rxGetInfo(outDataXdfDs, getVarInfo = TRUE, getBlockSizes = TRUE))
    checkEquals(outDataXdfDs.info[["numRows"]] * 2, outDataXdfDs2.info[["numRows"]])
  
  rxPredict(model, data = dataXdfMulti, outData = outDataXdfDs, extraVarsToWrite = "RowNum", append = "rows")
  (outDataXdfDs3.info <- rxGetInfo(outDataXdfDs, getVarInfo = TRUE, getBlockSizes = TRUE))
    checkEquals(outDataXdfDs.info[["numRows"]] * 3, outDataXdfDs3.info[["numRows"]])
  
  ####
  ## cxdf to cxdf
  outDataXdfComp <- file.path(tempdir(), "outDataCompXdf")
  on.exit(if(file.exists(outDataXdfComp)) unlink(outDataXdfComp, recursive = TRUE, force = TRUE), add = TRUE)
    outDataXdfCompDs <-  RxXdfData(outDataXdfComp, createCompositeSet = TRUE, blocksPerCompositeFile = 3)
    rxPredict(model, data = dataXdfComp, outData = outDataXdfCompDs, extraVarsToWrite = "RowNum", overwrite = TRUE)
  (outDataXdfCompDs.info <- rxGetInfo(outDataXdfCompDs, getVarInfo = TRUE, getBlockSizes = TRUE))
    checkEquals(outDataXdfCompDs.info[["numRows"]] * 1, dataXdfComp.info[["numRows"]])

  rxPredict(model, data = dataXdfComp, outData = outDataXdfCompDs, extraVarsToWrite = "RowNum", append = "rows")
  (outDataXdfCompDs2.info <- rxGetInfo(outDataXdfCompDs, getVarInfo = TRUE, getBlockSizes = TRUE))
    checkEquals(outDataXdfCompDs.info[["numRows"]] * 2, outDataXdfCompDs2.info[["numRows"]])

  ####
  ## xdf to cxdf  #NOTE: mixing xdf and cxdf is not currently supported.
}

# FIXME: commented out on 12/7/2010
#"test.rxPredict.args.predNames" <- function()
#{
  ## character vector specifying name(s) to give to prediction
  ## results; length same as number of dependent variables.}
  #
  ## create data frame data source with a multitude of response variables
  #depVars <- letters
  #set.seed(100)
  #DF <- data.frame(lapply(seq_along(depVars), rnorm, n=10), runif(10))
  #depVarNames <- paste(depVars, "Var", sep="")
  #indVarNames <- "myIndVar"
  #names(DF) <- c(depVarNames, indVarNames)
  #
  #DEACTIVATED("Pending fix of bug RA-1547: predicted variables for multiple dependent variables not returned as a list")
  #
  ## define predicted variable names
  #defaultPredNames <- paste(depVarNames, "Pred", sep="_")
  #predNames <- paste("myPredicted-", depVarNames, sep="")
#
  ## calculate linear regression model and use it to predict the response variables:
  ## use default and assigned names for variables containing predicted values
  #formStr <- paste(paste(depVarNames, collapse=","), paste(indVarNames, collapse="+"), sep="~")
  #zLinMod <- rxLinMod(formStr, data=DF)
  ##outDFDefault <- outDFAssigned <- data.frame(foo=seq(NROW(DF)))
  #outDFDefault <- outDFAssigned <- DF
  #rxPredict(zLinMod, data=DF, predNames=NULL, outData=outDFDefault)
  #zPredictAssigned <- rxPredict(zLinMod, data=DF, predNames=predNames, outData=outDFAssigned)
#
  ## test the names of the returned prediction variables
  #checkTrue(all(defaultPredNames %in% names(outDFDefault)))
  #checkTrue(all(predNames %in% names(outDFAssigned)))
#}

"test.rxPredict.args.reportProgress" <- function()
{	
  # reportProgress argument stress tests
  form <- y ~ x
  set.seed(100)
  DF <- data.frame(y=runif(10), x=runif(10))
  zLinMod <- rxLinMod(form, data=DF)
  
  #reportProgress: logical, character string or integer: 
  #  0: no progress is reported. 
  #  1: the number of processed rows is printed and updated. 
  #  2: rows processed and timings are reported. 
  #  3: rows processed and all timings are reported. 

  variations <- list(0,1,2,3)	
  results <- printOutput <- vector("list", length=length(variations))
  
  for (i in seq_along(variations))
  {
    printOutput[[i]] <- capture.output(results[[i]] <- rxPredict(zLinMod, data=DF, reportProgress=variations[[i]]))
  }

  # check that all values are the same
  checkTrue(all(unlist(lapply(results[-1], 
    function(x,y){all.equal(x, y, check.attributes=TRUE)}, y=results[[1]]))))

  names(printOutput) <- as.character(variations)

  noResponseExpected <- c("0")
  equalLengthExpected <- c("2")
  equalLengthExpected1 <- c("1")
  
  checkTrue(length(unlist(printOutput[noResponseExpected])) == 0)
  
    # just rows
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1],length)) == 1))
  
  # all these outputs should be the same
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected],length)) == 1))
  
  # details gives us the goods
  checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=10, TotalRowsProcessed=10, ReadTime", 
    printOutput[["3"]][1])) > 0)
  
  # time
  checkTrue(length(grep("Rows Read: 10, Total Rows Processed: 10, Total Chunk Time:", printOutput[["2"]][1])) > 0)
}

"test.rxPredict.args.removeMissings" <- function()
{	
  # create data 
  form <- y ~ x
  set.seed(100)
  DF <- data.frame(y=runif(10), x=runif(10))
  
  # add some missing values
  DF$y[4] <- DF$x[7] <- NA
  
  # create linear regression model
  zLinModDF <- rxLinMod(form, data=DF)
  
  # create predictions
  predRemoveNAFalse <- rxPredict(zLinModDF, data=DF, outData=NULL, removeMissings=FALSE)
  predRemoveNATrue <- rxPredict(zLinModDF, data=DF, outData=NULL, removeMissings=TRUE)
  
  # check results
  checkTrue(all(which(is.na(predRemoveNAFalse)) %in% c(4,7)))
  checkTrue(all(!is.na(predRemoveNATrue)))
}

"test.rxPredict.args.verbose" <- function()
{	
  # create data 
  form <- y ~ x
  set.seed(100)
  DF <- data.frame(y=runif(10), x=runif(10))
  zLinMod <- rxLinMod(form, data=DF)
  
  zPredVerboseFalse <- capture.output(z <- rxPredict(zLinMod, data=DF, verbose=0))
  zPredVerboseTrue <- capture.output(z <- rxPredict(zLinMod, data=DF, verbose=1))
  checkTrue(length(zPredVerboseTrue) > length(zPredVerboseFalse))
}


"test.rxPredict.data.logit.factor" <- function()
{
  # RA-2605
  # Should get the same result with a two level factor as with a logical
    set.seed(12)
  test <- data.frame(Var1= rep(c(0,1), 20), Var2=runif(40), Var3=1:40, Var4=rep(c("M","F"),20),
                     Var5=rep(as.logical(c("TRUE","FALSE")), 20), Var6 = rep(c("M", "F", "A", "B"), 5))

  testLogit <- rxLogit(Var4~Var2 + Var3, data=test)
  testLogitPred <- rxPredict(testLogit, data=test, computeResiduals=TRUE)
  testLogit1 <- rxLogit(Var5~Var2 + Var3, data=test)
  testLogitPred1 <- rxPredict(testLogit1, data=test, computeResiduals=TRUE)
  checkEquals(testLogitPred$Var4_Pred, testLogitPred1$Var5_Pred)
  
}

"test.rxPredict.data.noDependentVar" <- function()
{    

    XDF <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile( pattern = ".rxPredict1", fileext = ".xdf")
    inFile <- tempfile( pattern = ".rxPredict2", fileext = ".xdf")

  form <- xnum1 ~ F(y) + xnum2miss
  zLinMod <- rxLinMod(form, data=XDF)
    
    myData <- rxReadXdf(XDF)
    myData$xnum1 <- NULL
    rxDataStep(myData, outFile = inFile)
    
    # No dependent variable in prediction data set
    rxPredict(zLinMod, data = inFile,  outData = outFile, writeModelVars = TRUE)
    varInfo1 <- rxGetVarInfo(outFile)
    checkEquals(names(varInfo1), c( "xnum1_Pred", "y", "xnum2miss"))
    file.remove(outFile)
    
    # Has dependent variable in prediction data set
    rxPredict(zLinMod, data = XDF,  outData = outFile, writeModelVars = TRUE)
    varInfo2 <- rxGetVarInfo(outFile)
    checkEquals(names(varInfo2), c("xnum1_Pred", "xnum1", "y", "xnum2miss"))
    
    if (file.exists(outFile)) file.remove(outFile)
    if (file.exists(inFile)) file.remove(inFile)
}


"test.rxPredict.args.writeModelVars" <- function()
{    

    XDF <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile( pattern = ".rxPredict1", fileext = ".xdf")

  form <- xnum1 ~ F(y) + xnum2miss
  zLinMod <- rxLinMod(form, data=XDF)
  origNumCoresToUse <- rxGetOption("numCoresToUse")
  
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = FALSE, overwrite = TRUE)
  varInfo1 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo1), c("xnum1_Pred"))
  fileInfo <- rxGetInfo(outFile)
  checkEquals(fileInfo$numRows, 100)
  if (file.exists( outFile )) file.remove( outFile )
  
  df1 <- rxReadXdf(file = XDF)
  
  # First try writeModelVars with data frame
  predDF <- rxPredict(zLinMod, data = df1, writeModelVars = TRUE)
  varInfoDF <- rxGetVarInfo(predDF)
  checkEquals(names(varInfoDF), c("xnum1_Pred", "xnum1", "y", "xnum2miss"))	
  
  varsToKeep <- c("xnum1", "y", "xnum2miss")
  df1 <- rxReadXdf(file = XDF, varsToKeep = varsToKeep )
  
  rxOptions(numCoresToUse = 1)
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = TRUE, overwrite = TRUE)
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo2), c("xnum1_Pred", "xnum1", "y", "xnum2miss"))
  df2 <- rxReadXdf(file = outFile, varsToKeep = varsToKeep )
  checkEquals(df1, df2)
  if (file.exists( outFile )) file.remove( outFile )
  rm(df2)
  
  rxOptions(numCoresToUse = 2)
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = TRUE, overwrite = TRUE)
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo2), c("xnum1_Pred", "xnum1", "y", "xnum2miss"))
  df2 <- rxReadXdf(file = outFile, varsToKeep = varsToKeep )
  checkEquals(df1, df2)
  if (file.exists( outFile )) file.remove( outFile )
  rm(df2)
  
  rxOptions(numCoresToUse = -1)
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = TRUE, overwrite = TRUE)
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo2), c("xnum1_Pred", "xnum1", "y", "xnum2miss"))
  df2 <- rxReadXdf(file = outFile, varsToKeep = varsToKeep )
  checkEquals(df1, df2)
  if (file.exists( outFile )) file.remove( outFile )
  rm(df2)
  
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = TRUE, overwrite = TRUE,
    blocksPerRead = 2)
  varInfo2 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo2), c("xnum1_Pred", "xnum1", "y", "xnum2miss"))
  df2 <- rxReadXdf(file = outFile, varsToKeep = varsToKeep )
  checkEquals(df1, df2)
  if (file.exists( outFile )) file.remove( outFile )
  rm(df2)
  rm(df1)
  rxOptions(numCoresToUse = origNumCoresToUse)
  
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = TRUE, computeResiduals=TRUE, overwrite = TRUE)
  varInfo3 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo3),  c("xnum1_Pred", "xnum1_Resid", "xnum1", "y", "xnum2miss"))
  if (file.exists( outFile )) file.remove( outFile )

  zLinMod <- rxLinMod(exp(xnum1) ~ F(y) + x2, transforms = list(x2 = xnum2miss + 1.0), data=XDF)
  rxPredict(zLinMod, data = XDF, outData = outFile, writeModelVars = TRUE, computeResiduals=TRUE, overwrite = TRUE)
  varInfo4 <- rxGetVarInfo(outFile)
  checkEquals(names(varInfo4),  c("exp.xnum1._Pred", "exp.xnum1._Resid", "xnum2miss", "xnum1", "y","x2", "exp.xnum1."))
  if (file.exists( outFile )) file.remove( outFile )

}

"test.rxPredict.args.se.fit.level" <- function()
{    
    myData <- iris
    linModObj <- rxLinMod(Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species, 
                          data = myData, covCoef = TRUE)
    myData[["SepalLengthPred"]] <- rxPredict(modelObject=linModObj, data = iris)$Sepal.Length_Pred

    # expect warnings
    oldWarn <- options(warn = 2L)
    on.exit(options(oldWarn), add = TRUE) # to be sure
    checkException(rxPredict(modelObject = linModObj, data = myData, computeResiduals = TRUE, se.fit = TRUE),
        silent = TRUE)
    checkException(rxPredict(modelObject = linModObj, data = myData, level = 0.8),
        silent = TRUE)
    checkException(rxPredict(modelObject = linModObj, data = myData, confLevel = 0.8, interval = "confidence", computeStdErrors = FALSE),
        silent = TRUE)
    checkException(rxPredict(modelObject = linModObj, data = myData, confLevel = 0.8, interval = "prediction", computeStdErrors = FALSE),
        silent = TRUE)
    options(oldWarn)

    # expect error
    checkException(rxPredict(modelObject = linModObj, data = myData, computeResiduals = TRUE, se.fit = TRUE, computeStdErrors = TRUE),
        silent = TRUE)
    checkException(rxPredict(modelObject = linModObj, data = myData, level = 0.95, confLevel = 0.8),
        silent = TRUE)

    # compare level and confLevel
    level <- 0.8
    z1 <- rxPredict(modelObject = linModObj, data = myData, level = level)
    z2 <- rxPredict(modelObject = linModObj, data = myData, confLevel = level)
    checkEquals(z1, z2)
    
    # check that the return type is a data frame when only predictions are returned
    z3 <- rxPredict(modelObject = linModObj, data = myData, confLevel = 0.8, interval = "none", computeStdErrors = FALSE)
    checkTrue(inherits(z3, "data.frame"))
    z4 <- rxPredict(modelObject = linModObj, data = myData, confLevel = 0.8, interval = "confidence", computeStdErrors = TRUE)
    checkTrue(inherits(z4, "data.frame"))
}

"test.rxPredict.rxLinMod.computeStdErrors.multiple.cores" <- function()
{
    # Define faux data frame    
    x    <- c(2,3,-1,4,1,NA,2,-3,2,0)
    y    <- c(-1,2,-3,2,0,1,-1,2,0,-1)
    z    <-  c(1,0,-1,2,0,1,2,-1,-1,0)
    sex  <- factor(c("M","F","M","M","F","F","M","M","M","F"))
    size <- factor(c("s","m", "m","s","b","m","b","s","b","m"))
    DF   <- data.frame(x = x, y = y, z = z, sex = sex, size = size)

    # Form linear regression model
    form <- y ~ x + size + z:sex
    lmObj <- rxLinMod(form, data = DF, covCoef = TRUE)

    # Record current number of cores and restore upon exit
    oldNumCores <- rxGetOption("numCoresToUse")
    on.exit(rxOptions(numCoresToUse = oldNumCores), add = TRUE)
     
    # Form common arg structure
    confArgs <- list(lmObj, data = DF, computeStdErrors = TRUE, interval = "confidence")
    predArgs <- list(lmObj, data = DF, computeStdErrors = TRUE, interval = "prediction")
    
    # Default Cores
    rxOptions(numCoresToUse = -1L)
    zConfDefaultCore <- do.call("rxPredict", confArgs)
    zPredDefaultCore <- do.call("rxPredict", predArgs)

    # Single Core
    rxOptions(numCoresToUse = 1L)
    zConfOneCore <- do.call("rxPredict", confArgs)
    zPredOneCore <- do.call("rxPredict", predArgs)
    
    # Two Cores
    rxOptions(numCoresToUse = 2L)
    zConfTwoCore <- do.call("rxPredict", confArgs)
    zPredTwoCore <- do.call("rxPredict", predArgs)

    # Four Cores
    rxOptions(numCoresToUse = 4L)
    zConfFourCore <- do.call("rxPredict", confArgs)
    zPredFourCore <- do.call("rxPredict", predArgs)
    
    # Define lm standard for comparison
    rxOptions(numCoresToUse = -1L)
    lmfit <- lm(form, DF)
    lmConf <- predict.lm(lmfit, se.fit = TRUE, interval = "confidence")
    lmPred <- predict.lm(lmfit, se.fit = TRUE, interval = "prediction")
    
    #####
    # Compare default core results to R's predict.lm output
  
  # fix for bug RA-4793
    
    # Confidence limits
    checkEquals(na.omit(data.frame(zConfDefaultCore[c("y_Lower", "y_Upper")])), 
                data.frame(lmConf[["fit"]][,c("lwr","upr")]), 
                check.attributes = FALSE)
            
    # Standard errors
    checkEquals(as.vector(na.omit(zConfDefaultCore[["y_StdErr"]])), lmConf[["se.fit"]])
  
    # Confidence limits
    checkEquals(na.omit(data.frame(zPredDefaultCore[c("y_Lower", "y_Upper")])), 
                data.frame(lmPred[["fit"]][,c("lwr","upr")]), 
                check.attributes = FALSE)
            
    # Standard errors
    checkEquals(as.vector(na.omit(zPredDefaultCore[["y_StdErr"]])), lmPred[["se.fit"]])
    
    #####
    # Compare results from different cores
    checkEquals(zConfDefaultCore, zConfOneCore)
    checkEquals(zConfDefaultCore, zConfTwoCore)
    checkEquals(zConfDefaultCore, zConfFourCore)
    checkEquals(zPredDefaultCore, zPredOneCore)
    checkEquals(zPredDefaultCore, zPredTwoCore)
    checkEquals(zPredDefaultCore, zPredFourCore)
}

"test.rxPredict.args.computeStdErrors.multiple.block.sizes" <- function()
{    
    # Fix for bug RA-4789
    # Define faux data frame    
    x    <- c(2,3,-1,4,1,NA,2,-3,2,0)
    y    <- c(-1,2,-3,2,0,1,-1,2,0,-1)
    z    <-  c(1,0,-1,2,0,1,2,-1,-1,0)
    sex  <- factor(c("M","F","M","M","F","F","M","M","M","F"))
    size <- factor(c("s","m", "m","s","b","m","b", "s","b","m"))
    DF   <- data.frame(x = x, y = y, z = z, sex = sex, size = size)

    XDFSingleBlock <- tempfile( pattern = ".rxPredictSingle", fileext = ".xdf")
    XDFMultipleBlock <- tempfile( pattern = ".rxPredictMult", fileext = ".xdf")
    

    rxDataStep(DF, XDFSingleBlock)
    checkEquals(rxGetInfo(XDFSingleBlock)[["numBlocks"]], 1L)
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDFMultipleBlock)
    } else
    {
        rxDataStep(DF[1:3,], XDFMultipleBlock)
        rxDataStep(DF[4:5,], XDFMultipleBlock, append = "rows")
        rxDataStep(DF[6,], XDFMultipleBlock, append = "rows")
        rxDataStep(DF[7:10,], XDFMultipleBlock, append = "rows")
        checkEquals(rxGetInfo(XDFMultipleBlock)[["numBlocks"]], 4L)
    }
    
    
    # Form linear regression model
    form <- y ~ x + size + z:sex
    lmObj <- rxLinMod(form, data = DF, covCoef = TRUE)

    # Form common arg structure
    confArgs <- list(lmObj, data = DF, computeStdErrors = TRUE, interval = "confidence")
    predArgs <- list(lmObj, data = DF, computeStdErrors = TRUE, interval = "prediction")
    
    # Calculate confidence/prediction intervals with single block XDF 
    xdfConfSingleBlock <- tempfile( pattern = ".rxPredictCFB", fileext = ".xdf")
    xdfPredSingleBlock <- tempfile( pattern = ".rxPredictPSB", fileext = ".xdf")
    zConfSingleBlock <- rxPredict(lmObj, data = XDFSingleBlock, outData = xdfConfSingleBlock, 
                                  computeStdErrors = TRUE, interval = "confidence")
    zPredSingleBlock <- rxPredict(lmObj, data = XDFSingleBlock, outData = xdfPredSingleBlock, 
                                  computeStdErrors = TRUE, interval = "prediction")
    
    # Calculate confidence/prediction intervals with multiple block XDF 
    xdfConfMultipleBlock <- tempfile( pattern = ".rxPredictCMB", fileext = ".xdf")
    xdfPredMultipleBlock <- tempfile( pattern = ".rxPredictPMB", fileext = ".xdf")
    zConfMultipleBlock <- rxPredict(lmObj, data = XDFMultipleBlock, outData = xdfConfMultipleBlock, 
                                    computeStdErrors = TRUE, interval = "confidence")
    zPredMultipleBlock <- rxPredict(lmObj, data = XDFMultipleBlock, outData = xdfPredMultipleBlock, 
                                    computeStdErrors = TRUE, interval = "prediction")
 
    # Compare the results
    zConfSingleBlockDF   <- rxDataStep(zConfSingleBlock)
    zConfMultipleBlockDF <- rxDataStep(zConfMultipleBlock)
    zPredSingleBlockDF   <- rxDataStep(zPredSingleBlock)
    zPredMultipleBlockDF <- rxDataStep(zPredMultipleBlock)
    
    checkEqualsNumeric(unlist(zConfSingleBlockDF[1L,, drop = TRUE]), c(1.2174, 1.215250, -2.156674, 4.591474), 
                       tol = 1e-6, check.attributes = FALSE)
    checkEquals(zConfSingleBlockDF, zConfMultipleBlockDF)
    checkEquals(zPredSingleBlockDF, zPredMultipleBlockDF)
    
    # Clean up
    invisible(sapply(c(XDFSingleBlock, XDFMultipleBlock, 
                       xdfConfSingleBlock, xdfPredSingleBlock, 
                       xdfConfMultipleBlock, xdfPredMultipleBlock), 
                     function(x) if (file.exists(x)) unlink(x)))
        
}

"test.rxPredict.args.computeStdErrors.medium.file.size" <- function()
{
    XDF <- file.path(system.file(package = "RevoScaleR"), "SampleData", "DJIAdaily.xdf")
    transforms <- expression(list(highClose = Close > (High + Low) / 2))
    form <- highClose ~ DayOfWeek
    zModel <- rxLogit(form, data = XDF, transforms = transforms, covCoef = TRUE)
  outData <- tempfile( pattern = ".rxPredictSE", fileext = ".xdf")
    zPredictXDF <- rxPredict(zModel, data = XDF, outData = outData, 
                             computeStdErrors = TRUE, interval = "confidence", writeModelVars = TRUE)
    info <- rxGetVarInfo(zPredictXDF)
    checkEquals(names(info), c("highClose_Pred", "highClose_StdErr", "highClose_Lower", "highClose_Upper", 
                               "Close", "High", "Low", "DayOfWeek", "highClose"))
    checkEquals(length(info), 9L)
    zPredict <- rxReadXdf(zPredictXDF, varsToKeep = "highClose_Pred")
    checkTrue(all(zPredict[["highClose_Pred"]] >= 0 & zPredict[["highClose_Pred"]] <= 1))
  file.remove(outData)
}

"test.rxPredict.rxLogit.computeStdErrors.multiple.cores" <- function()
{
    # Fix for bug RA-4789
    # Form logistic regression model
    form <- case ~ education + pooled.stratum
    logitObj <- rxLogit(form, data = infert, covCoef = TRUE)

    # Record current number of cores and restore upon exit
    oldNumCores <- rxGetOption("numCoresToUse")
    on.exit(rxOptions(numCoresToUse = oldNumCores), add = TRUE)
     
    # Form common arg structure
    confArgs <- list(logitObj, data = infert, computeStdErrors = TRUE, interval = "confidence")
    
    # Default Cores
    rxOptions(numCoresToUse = -1L)
    zConfDefaultCore <- do.call("rxPredict", confArgs)

    # Single Core
    rxOptions(numCoresToUse = 1L)
    zConfOneCore <- do.call("rxPredict", confArgs)
    
    # Two Cores
    rxOptions(numCoresToUse = 2L)
    zConfTwoCore <- do.call("rxPredict", confArgs)

    # Four Cores
    rxOptions(numCoresToUse = 4L)
    zConfFourCore <- do.call("rxPredict", confArgs)
    
    # Define lm standard for comparison
    rxOptions(numCoresToUse = -1L)
    logitfit <- glm(form, infert, family = binomial())
    logitPred <- predict.glm(logitfit, se.fit = TRUE, type = "response")
    
    #####
    # Compare default core results to R's predict.glm output
    checkEquals(as.vector(na.omit(zConfDefaultCore[["case_Pred"]])), as.vector(logitPred[["fit"]]))
    checkEquals(as.vector(na.omit(zConfDefaultCore[["case_StdErr"]])), as.vector(logitPred[["se.fit"]]), scale = 1) 
        
    #####
    # Compare results from different cores
    checkEquals(zConfDefaultCore, zConfOneCore)
    checkEquals(zConfDefaultCore, zConfTwoCore)
    checkEquals(zConfDefaultCore, zConfFourCore)
}

# Test the return value
"test.rxPredict.value" <- function()
{
    XDF <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
    outFile <- tempfile(pattern = ".rxTempPredOut", fileext = ".xdf")
  inFileNew <- tempfile(pattern = ".rxTempPredIn", fileext = ".xdf")
  inDataFrame <- rxReadXdf(XDF)
  
  if (file.exists(inFileNew)) file.remove(inFileNew)
  if (file.exists(outFile)) file.remove(outFile)
  
  file.copy(from = XDF, to = inFileNew, overwrite = TRUE)
  
  form <- xnum1 ~ F(y) + xnum2miss
  zLinMod <- rxLinMod(form, data=inFileNew)
  
  # Specify outfile: should return new XDF data source
  outDS1 <- rxPredict(zLinMod, data = XDF, outData = outFile)
  checkEquals(normalizePath(outDS1@file), normalizePath(outFile))
  file.remove(outFile)
    
  # Don't specify outData:  will append to original
  outDS2 <- rxPredict(zLinMod, data = inFileNew)
  checkEquals(normalizePath(outDS2@file), normalizePath(inFileNew))
  file.remove(inFileNew)
  
  # Don't specify outData with input as data frame
  outDF1 <- rxPredict(zLinMod, data = inDataFrame)
  checkTrue(is.data.frame(outDF1))
  checkEquals(ncol(outDF1), 1)
  
  outDF2 <- rxPredict(zLinMod, data = inDataFrame, computeResiduals = TRUE)
  checkTrue(is.data.frame(outDF2))
  checkEquals(ncol(outDF2), 2)
  
  # Specify outData same as inData data frame
  outDF3 <- rxPredict(zLinMod, data = inDataFrame, outData = inDataFrame)
  checkTrue(is.data.frame(outDF3))
  checkEquals(ncol(outDF3), 10)
  
  # Specify outfile with data frame input		
  res  <- rxPredict(zLinMod, data = inDataFrame, outData = outFile)
  outDF4 <- rxImport(outFile)
  checkEquals(ncol(outDF4), 1)
  
  file.remove(outFile)

}

"test.rxPredict.transforms.writeModelVars" <- function()
{ 
  # See RA-5912
  testXdf <- tempfile(pattern = ".rxTestPredict", fileext = ".xdf")
  predXdf <- tempfile(pattern = ".rxPredict", fileext = ".xdf")
      
  y <- rep(c(0,1), times = 40)
  x <- rep(1:4, times = 20)
  myData <- data.frame(x,y)
  
  # Create multi-block xdf file
  rxDataStep(inData = myData, outFile = testXdf, rowsPerRead = 20, overwrite = TRUE)
  
  # Logistic regression with a transform
  logitObj1 <- rxLogit(y1~x, data = testXdf, transforms = list(y1 = y))
  rxPredict(logitObj1, data = testXdf, outData =predXdf, 
    writeModelVars = TRUE, overwrite = TRUE)
  
  outData1 <- rxReadXdf(predXdf)
  checkEquals(outData1$x, myData$x)
  
  # Now without the transformation, just to be sure
  
  logitObj2 <- rxLogit(y~x, data = testXdf)
  rxPredict(logitObj2, data = testXdf, outData =predXdf, 
    writeModelVars = TRUE, overwrite = TRUE)
  
  outData1 <- rxReadXdf(predXdf)
  checkEquals(outData1$x, myData$x)
  
  # Now a transform with a single-block xdf
  rxDataStep(inData = myData, outFile = testXdf, overwrite = TRUE)
  
  # Logistic regression with a transform
  logitObj3 <- rxLogit(y1~x, data = testXdf, transforms = list(y1 = y))
  rxPredict(logitObj3, data = testXdf, outData =predXdf, 
    writeModelVars = TRUE, overwrite = TRUE)
  
  outData1 <- rxReadXdf(predXdf)
  checkEquals(outData1$x, myData$x)
  
  file.remove(testXdf)
  file.remove(predXdf)

}

"test.rxPredict.transforms.depVar" <- function()
{
    linModOut1 <- rxLinMod(Sepal.Length*2~Sepal.Width, data = iris)
    iris1 <- iris
    iris1$Sepal.Length <- NULL
    predOut0 <- rxPredict(linModOut1, data = iris)
    # Can we still predict without the dependent variable input?
    predOut1 <- rxPredict(linModOut1, data = iris1)
    checkEquals(predOut0, predOut1)
    
    # What about writeModelVars?
    predOutModelVars <- rxPredict(linModOut1, data = iris1, writeModelVars = TRUE)
    checkEquals(predOutModelVars$Sepal.Length...2_Pred, predOut0$Sepal.Length...2_Pred)
    checkEquals(predOutModelVars$Sepal.Width, iris1$Sepal.Width)
  
  predOutModelVarsOrig <- rxPredict(linModOut1, data = iris, writeModelVars = TRUE)
  checkEquals(names(predOutModelVarsOrig), c("Sepal.Length...2_Pred",
     "Sepal.Length", "Sepal.Width", "Sepal.Length...2"))
    
    # Make sure we error gracefully if a required variable is missing
    linModOut2 <- rxLinMod(Sepal.Length~log(Sepal.Width), data = iris)
    iris2 <- iris
    iris2$Sepal.Width <- NULL
    res <- try(
        predOutFail <- rxPredict(linModOut2, data = iris2)
    )
    checkTrue(is(res, "try-error"))    
   
    # We'd like to do this 
    #linModOut1 <- rxLinMod(Sepal.Length*2~log(Sepal.Width), data = iris)
    myTransform <- function(dataList)
    {
        print(.rxIsPrediction)
        print(.rxIsTestChunk)
        if (!.rxIsPrediction)
        {
            dataList$SepalLength2 <- dataList$Sepal.Length * 2
        }
        dataList$SepalWidthLog <- log(dataList$Sepal.Width)
        return( dataList )
    }
    linModOut3 <- rxLinMod(SepalLength2~SepalWidthLog, data = iris,
       transformFunc = myTransform, transformVars = c("Sepal.Length", "Sepal.Width"))
    predOutTransform <- rxPredict(linModOut3, data = iris)
    predOutTransformNoDep <- rxPredict(linModOut3, data = iris1)
    checkEquals(predOutTransform, predOutTransformNoDep)
    
    linModOut4 <- rxLinMod(Sepal.Length*2~log(Sepal.Width), data = iris)
    iris1 <- iris
    iris1$Sepal.Length <- NULL
    predOut4 <- rxPredict(linModOut4, data = iris)

    # Can we still predict without the dependent variable input?
    predOut5 <- rxPredict(linModOut4, data = iris1)
    checkEquals(predOut4, predOut5)
    
    # A combination of transforms and formula transforms
    linModOut6 <- rxLinMod(Sepal.Length*2~SepalWidthLog, data = iris,
      transforms = list(SepalWidthLog = log(Sepal.Width)))
    iris1 <- iris
    iris1$Sepal.Length <- NULL
    predOut6 <- rxPredict(linModOut6, data = iris)

    # Can we still predict without the dependent variable input?
    predOut7 <- rxPredict(linModOut6, data = iris1)
    checkEquals(predOut6, predOut7)
    
    

}
"test.rxPredict.formula.F.data" <- function()
{
  myData <- data.frame(x = c(1,2,3,4,1,2,3,4),
    ylogical = c(0,0,0,1,0,0,1,1),
    y = c(3,3,3,5,2,3,2,4),
    z = c(4,4,3,4,3,4,3,3),
    w = c(6,6,5,5,5,6,5,6))
  
  # Use a subset of the data for predictions
  myTestData <- myData[2:3,]	
  
  myLinMod <- rxLinMod(y~F(x, low = 1, high = 4), data = myData)
  # All predictions
  myPredData <- rxPredict(myLinMod, data = myData)
  # Predictions for subset of data
  myPredDataSub <- rxPredict(myLinMod, data = myTestData)
  checkEquals(myPredDataSub[1:2,], myPredData[2:3,])	
  
  # Now without low and high
  myLinMod1 <- rxLinMod(y~F(x), data = myData)

  # Use a subset of the data for predictions
  myPredDataSub1 <- rxPredict(myLinMod1, data = myTestData)
  checkEquals(myPredDataSub1[1:2,], myPredData[2:3,])	
  
  # Try a logit
  myLogitMod <- rxLogit(ylogical~F(x), data = myData)
  myPredLogit <- rxPredict(myLogitMod, data = myData)
  myPredLogitSub <- rxPredict(myLogitMod, data = myTestData)
  checkEquals(myPredLogitSub[1:2,], myPredLogit[2:3,])	
  
  # Try a glm
  myGlmMod <- rxGlm(ylogical~F(x), data = myData, family = binomial())
  myPredGlm <- rxPredict(myGlmMod, data = myData)
  myPredGlmSub <- rxPredict(myGlmMod, data = myTestData)
  checkEquals(myPredGlmSub[1:2,], myPredGlm[2:3,])	
  
  # Now with just low
  myLinMod2 <- rxLinMod(y~F(x, low = 1), data = myData)
  myPredDataSub2 <- rxPredict(myLinMod2, data = myTestData)
  checkEquals(myPredDataSub2[1:2,], myPredData[2:3,])
  
  # Try specifying exclude
  myLinMod3 <- rxLinMod(y~F(x, low = 1, exclude = FALSE), data = myData)
  myPredDataSub3 <- rxPredict(myLinMod3, data = myTestData)
  checkEquals(myPredDataSub3[1:2,], myPredData[2:3,])
  
  # How about interactions
  
  # See RA-6025
  myLinMod4 <- rxLinMod(y~F(x):z, data = myData)
  myPredData4 <- rxPredict(myLinMod4, data = myData)
  myPredDataSub4 <- rxPredict(myLinMod4, data = myTestData)
  checkEquals(myPredDataSub4[1:2,], myPredData4[2:3,])
  
  myLinMod5 <- rxLinMod(y~F(x):F(z), data = myData)
  myPredData5 <- rxPredict(myLinMod5, data = myData)
  myPredDataSub5 <- rxPredict(myLinMod5, data = myTestData)
  checkEquals(myPredDataSub5[1:2,], myPredData5[2:3,])
  
  # Try more than one interaction term	
  myLinMod6 <- rxLinMod(y~F(x):F(z):F(w), data = myData)
  myPredData6 <- rxPredict(myLinMod6, data = myData)
  myPredDataSub6 <- rxPredict(myLinMod6, data = myTestData)
  checkEquals(myPredDataSub6[1:2,], myPredData6[2:3,])
  
  # Try additional independent variables
  myLinMod7 <- rxLinMod(y~F(x):F(z) + w, data = myData)
  myPredData7 <- rxPredict(myLinMod7, data = myData)
  myPredDataSub7 <- rxPredict(myLinMod7, data = myTestData)
  checkEquals(myPredDataSub7[1:2,], myPredData7[2:3,])	
  
  myLinMod8 <- rxLinMod(y~x + F(x) + F(z), data = myData)
  myPredData8 <- rxPredict(myLinMod8, data = myData)
  myPredDataSub8 <- rxPredict(myLinMod8, data = myTestData)
  checkEquals(myPredDataSub8[1:2,], myPredData8[2:3,])	
  
}

"test.rxPredict.args.rxGlm.formula.offset" <- function()
{
    # Testing fix to RA-8552
    utils::data(anorexia, package = "MASS")
    inData <- anorexia
    glmOffset <- glm(Postwt ~  Treat + offset(Prewt), family = gaussian, 
        data = inData, contrast = list(Treat = "contr.SAS"))
    rxGlmOffsetFormula <- rxGlm(Postwt ~ Treat + offset(Prewt), family = gaussian, data = inData)
    outData <- rxPredict(rxGlmOffsetFormula, data = inData, outData = inData)
   
    # Now with .xdf file
    inFile <- tempfile(pattern = ".rxPredictOffset", fileext = ".xdf")
    rxDataStep(inData = inData, outFile = inFile, rowsPerRead = 20)
    rxGlmOffsetFormula <- rxGlm(Postwt ~ Treat + offset(Prewt), family = gaussian, data = inFile)
    outDS <- rxPredict(rxGlmOffsetFormula, data = inFile, outData = inFile)   
    outData2 <- rxReadXdf(outDS)
    checkEquals(outData, outData2) 
    file.remove(inFile)
}

"test.rxPredict.dataSource.SPSS" <- function()
{
  spssFile <- file.path(rxGetOption("sampleDataDir"),"claims.sav")
  spssDS <- RxSpssData(spssFile,
            colClasses = c("RowNum" = "integer", "age" = "factor",
                     "car_age" = "factor", "type" = "factor",
                     "cost" = "integer", "number" = "integer"),
             colInfo = list(  age = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34","35-39","40-49","50-59","60+")),
                      car.age = list(type = "factor", levels = c("0-3", "4-7", "8-9","10+")),
                      type = list(type = "factor", levels = c("A", "B", "C", "D")))
            )	

  # Simple model        					
  linModObj1 <- rxLinMod(cost ~number, data=spssDS)
    testOutFile <- tempfile(pattern = ".rxTempTest", fileext = ".xdf")  
  
  # Check that writeModelVars set to FALSE just produces predictions	
    predData1 <- rxPredict(linModObj1, data = spssDS, outData = testOutFile, writeModelVars = FALSE)  
  predVarInfo <- rxGetVarInfo(predData1)
  checkEquals(length(predVarInfo), 1)
    file.remove(testOutFile)
  
  # Compare results doing predictions directly with importing data first
  predData1 <- rxPredict(linModObj1, data = spssDS, outData = testOutFile, writeModelVars = TRUE)  
  predDF1 <- rxReadXdf(predData1)
    
  tempDF <- rxImport(spssDS,  overwrite = TRUE, reportProgress = 0)
  linModObj1a <- rxLinMod(cost ~number, data = tempDF)	
    predData1a <- rxPredict(linModObj1a, data = tempDF, writeModelVars = TRUE)
  
  checkEquals(predDF1, predData1a)
  file.remove(testOutFile)
  
  # Try a model with F
  linModObj2 <- rxLinMod(cost ~F(number), data=spssDS)
  predData2 <- rxPredict(linModObj2, data = spssDS, outData = testOutFile, writeModelVars = TRUE)  
  predDF2 <- rxReadXdf(predData2)
  linModObj2a <- rxLinMod(cost ~F(number), data = tempDF)	
    predData2a <- rxPredict(linModObj2a, data = tempDF, writeModelVars = TRUE)
  checkEquals(predDF2, predData2a)
  file.remove(testOutFile)
      
  # Try transforms
  linModObj3 <- rxLinMod(cost ~number2, data=spssDS, transforms = list(number2 = 2*number))
  predData3 <- rxPredict(linModObj3, data = spssDS, outData = testOutFile, writeModelVars = TRUE)  
  predDF3 <- rxReadXdf(predData3)
  linModObj3a <- rxLinMod(cost ~number2, data = tempDF, transforms = list(number2 = 2*number))	
    predData3a <- rxPredict(linModObj3a, data = tempDF, writeModelVars = TRUE)
    # Variables may be in different orders
  checkEquals(predDF3$number2, predData3a$number2)
  checkEquals(predDF3$cost_Pred, predData3a$cost_Pred)    
    file.remove(testOutFile)
    
    # Try a model with a factor and rowSelection
  linModObj4 <- rxLinMod(cost ~type, data=spssDS,  rowSelection =  car_age %in% c("0-3", "4-7","10+") )
  predData4 <- rxPredict(linModObj4, data = spssDS, outData = testOutFile, 
        writeModelVars = TRUE)  
  predDF4 <- rxReadXdf(predData4)
  linModObj4a <- rxLinMod(cost ~type, data = tempDF, rowSelection =  car_age %in% c("0-3", "4-7","10+"))	
    predData4a <- rxPredict(linModObj4a, data = tempDF, writeModelVars = TRUE)  
  checkEquals(predDF4$cost_Pred, predData4a$cost_Pred)	
    
    file.remove(testOutFile)	
   
}

"test.rxPredict.args.data.dataframe.onecol" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF <- rxReadXdf(claimsXdf, varsToKeep = c("cost", "number"))
    linModObj <- rxLinMod(cost~number, data = claimsDF)
  
  # Make a data frame with just one variable, excluding dependent var
    claimsDF1 <- claimsDF[,"number", drop = FALSE]
    predDF <- rxPredict(linModObj, data = claimsDF)
  predDF1 <- rxPredict(linModObj, data = claimsDF1)
  checkEquals(predDF, predDF1)
}

"test.rxPredict.args.model.transforms.dataframe" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
    claimsDF <- rxReadXdf(claimsXdf, varsToKeep = c("cost", "number"))
  linModObj1 <- rxLinMod(cost ~number2, data = claimsDF, transforms = list(number2 = 2*number))	
    predDF1 <- rxPredict(linModObj1, data = claimsDF)
  
  # Predictions should be the same as a simple model, since it's just a linear transformation
    linModObj2 <- rxLinMod(cost ~number, data = claimsDF)
  predDF2 <- rxPredict(linModObj2, data = claimsDF)
  checkEquals(predDF1, predDF2)
    
    predDF1a <- rxPredict(linModObj1, data = claimsDF, writeModelVars = TRUE)
    
    testOutFile <- tempfile(pattern = ".rxTempTest", fileext = ".xdf")  
    predDS1b <- rxPredict(linModObj1, data = claimsXdf, writeModelVars = TRUE, outData = testOutFile)
    predDF1b <- rxReadXdf(predDS1b)

    checkEquals(predDF1a, predDF1b)
    file.remove(testOutFile)    
    
}

"test.rxPredict.args.model.interactions" <- function()
{
    # See RA-6377
    x    <- c(2,3,-1,4,1,NA,2,-3,2,0)
    y    <- c(-1,2,-3,2,0,1,-1,2,0,-1)
    z    <-  c(1,0,-1,2,0,1,2,-1,-1,0)
    sex  <- factor(c("M","F","M","M","F","F","M","M","M","F"))
    size <- factor(c("s","m", "m","s","b","m","b","s","b","m"))
    DF   <- data.frame(x = x, y = y, z = z, sex = sex, size = size)

    linModObj1 <- rxLinMod(y ~ z*sex, data = DF, covCoef = TRUE)
  
    predDF1 <- rxPredict(linModObj1, data = DF,  computeStdErrors = TRUE, computeResiduals = TRUE,
        interval = "confidence") 
    checkEquals(ncol(predDF1), 5)
    
    linModObj2 <- rxLinMod(y ~ sex*size, data = DF, covCoef = TRUE)
  predDF2 <- rxPredict(linModObj2, data = DF,  computeStdErrors = TRUE, computeResiduals = TRUE,
        interval = "confidence") 
        
     checkEquals(ncol(predDF2), 5)       
 }

"test.rxPredict.args.data.RxTextData" <- function()
{
    testTxt <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")  
    textDS <- RxTextData(testTxt )
    linModObj <- rxLinMod(xnum1~xnum2, data = textDS)
    
    testOutFile <- tempfile(pattern = ".rxPred2TempTestOut", fileext = ".xdf")
    if (file.exists(testOutFile)) file.remove(testOutFile)
    # Try it twice, to make sure we can write to an existing file     
    rxPredict(linModObj, data = textDS, outData = testOutFile)
    rxPredict(linModObj, data = textDS, outData = testOutFile, overwrite = TRUE)
    
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf") 
    linModObj2 <- rxLinMod(xnum1~xnum2, data = testXdf)
    
    testOutFile2 <- tempfile(pattern = ".rxPred2TempTestOut2", fileext = ".xdf")
    if (file.exists(testOutFile2)) file.remove(testOutFile2)
    
    rxPredict(linModObj2, data = testXdf, outData = testOutFile2)
    
    outDF1 <- rxReadXdf(testOutFile)
    outDF2 <- rxReadXdf(testOutFile2)
    checkEquals(rxReadXdf(testOutFile), rxReadXdf(testOutFile2))
    file.remove(testOutFile)
    file.remove(testOutFile2)
}

"test.rxPredict.args.data.RxOdbcData" <- function()
{
    rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
      rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), 
                useFastRead = FALSE),
      silent = TRUE)
        
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
    # StatTran    
  odbcDS1 <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString,
                useFastRead = FALSE,
                colClasses = c("RowNum" = "integer", "age" = "character",
                 "car_age" = "character", "type" = "character",
                 "cost" = "integer", "number" = "integer"))
  linModObj1 = rxLinMod(cost~number, data=odbcDS1)
    # Native
  odbcDS2 <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString,
                useFastRead = TRUE,
                colClasses = c("RowNum" = "integer", "age" = "character",
                 "car_age" = "character", "type" = "character",
                 "cost" = "integer", "number" = "integer"))
  linModObj2 = rxLinMod(cost~number, data=odbcDS2)    
    
    
    testOutFile1 <- tempfile(pattern = ".rxPred3TempTestOut", fileext = ".xdf")
    if (file.exists(testOutFile1)) file.remove(testOutFile1)
    # Try it twice, to make sure we can write to an existing file     
    rxPredict(linModObj1, data = odbcDS1, outData = testOutFile1)
    rxPredict(linModObj1, data = odbcDS1, outData = testOutFile1, overwrite = TRUE)
    
    testOutFile2 <- tempfile(pattern = ".rxPred4TempTestOut", fileext = ".xdf")
    if (file.exists(testOutFile2)) file.remove(testOutFile2)
    # Try it twice, to make sure we can write to an existing file     
    rxPredict(linModObj2, data = odbcDS2, outData = testOutFile2)
    rxPredict(linModObj2, data = odbcDS2, outData = testOutFile2, overwrite = TRUE)    
    
    testXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf") 
    linModObj3 <- rxLinMod(cost~number, data = testXdf)
    
    testOutFile3 <- tempfile(pattern = ".rxPred5TempTestOut2", fileext = ".xdf")
    if (file.exists(testOutFile3)) file.remove(testOutFile3)
    
    rxPredict(linModObj3, data = testXdf, outData = testOutFile3)
    
    outDF1 <- rxReadXdf(testOutFile1)
    outDF2 <- rxReadXdf(testOutFile2)
    outDF3 <- rxReadXdf(testOutFile3)
    checkEquals(outDF1, outDF2)
    checkEquals(outDF1, outDF3)
    file.remove(testOutFile1)
    file.remove(testOutFile2)
    file.remove(testOutFile3)
}


"test.rxPredict.args.xdfCompressionLevel" <- function()
{
  testXdf <- tempfile(pattern = ".rxTest", fileext = ".xdf")
  predXdf0 <- tempfile(pattern = ".rxPredict", fileext = ".xdf")
  predXdf1 <- tempfile(pattern = ".rxPredict", fileext = ".xdf")
      
  y <- rep(c(0,1), times = 40)
  x <- rep(1:4, times = 20)
  myData <- data.frame(x,y)
  
  # Create multi-block xdf file: no compression
  rxDataStep(inData = myData, outFile = testXdf, rowsPerRead = 20, overwrite = TRUE)
  
  # Logistic regression with a transform
  logitObj1 <- rxLogit(y1~x, data = testXdf, transforms = list(y1 = y))
  
  rxPredict(logitObj1, data = testXdf, outData =predXdf0, 
    writeModelVars = TRUE, overwrite = TRUE, xdfCompressionLevel = 0)
  
  rxPredict(logitObj1, data = testXdf, outData =predXdf1, 
    writeModelVars = TRUE, overwrite = TRUE, xdfCompressionLevel = 1)
  
  compressedSize0 <- file.info(predXdf0)$size
  compressedSize1 <- file.info(predXdf1)$size
  checkTrue( compressedSize0 > compressedSize1)
  
  file.remove(testXdf)
  file.remove(predXdf0)
  file.remove(predXdf1)	
}

"test.rxPredict.args.extraVarsToWrite" <- function()
{

  testFile <- tempfile(pattern = ".rxPred1TempTest2", fileext = ".xdf") 

  testPredFile <- tempfile(pattern = ".rxPred1TempPred2", fileext = ".xdf") 

  rxDataStep(inData = infert, outFile = testFile, rowsPerRead = 100, overwrite = TRUE)
  
  # Estimate a model
  rxLogitOut1 <- rxLogit(case ~ spontaneous + induced, data=testFile )
  
  # Write results and model vars to new file
  rxPredOut <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
    outData = testPredFile, writeModelVars = TRUE,
    predVarNames = "casePred1", extraVarsToWrite = c("age"), overwrite = TRUE)
    
    varInfoIn <- rxGetVarInfo(testFile)
    varInfoOut <- rxGetVarInfo(rxPredOut)
    checkEquals(varInfoIn$age, varInfoOut$age)
    checkEquals(names(varInfoOut), c("casePred1", "age", "case", "spontaneous", "induced"))
    
    # Repeat, but without writing out the model vars
  rxPredOut1 <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
    outData = testPredFile, writeModelVars = FALSE,
    predVarNames = "casePred1", extraVarsToWrite = c("age"), overwrite = TRUE)
    
    varInfoOut1 <- rxGetVarInfo(rxPredOut1)
    checkEquals(varInfoIn$age, varInfoOut1$age)
    checkEquals(names(varInfoOut1), c("casePred1", "age"))
    
    
    # Write to another existing file 
    testOutFile <- tempfile(pattern = ".rxPred1TempTest3", fileext = ".xdf")
    keepVars <- c("education", "parity", "induced", "spontaneous", "stratum",
                    "pooled.stratum")    
    rxDataStep(inData = testFile, outFile = testOutFile, 
        varsToKeep = keepVars, overwrite = TRUE)
    rxPredOut3a <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
      outData = testOutFile, writeModelVars = TRUE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = FALSE)
    varInfoOut3a <- rxGetVarInfo(rxPredOut3a)
    checkEquals(varInfoIn$age, varInfoOut3a$age)
    checkEquals(names(varInfoOut3a), c(keepVars, "casePred1", "age", "case"))  
    
    # Repeat using writeModelVars = FALSE
    rxDataStep(inData = testFile, outFile = testOutFile, 
        varsToKeep = keepVars, overwrite = TRUE)
    rxPredOut3b <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
      outData = testOutFile, writeModelVars = FALSE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = FALSE)
    varInfoOut3b <- rxGetVarInfo(rxPredOut3b)
    checkEquals(varInfoIn$age, varInfoOut3b$age)
    checkEquals(names(varInfoOut3b), c(keepVars, "casePred1", "age"))     
            
    # Try data set without dependent variable
    infertNoCase <- infert
    infertNoCase$case <- NULL
    
    rxDataStep(inData = infertNoCase, outFile = testFile, rowsPerRead = 100, overwrite = TRUE)
    
  rxPredOut2 <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
    outData = testPredFile, writeModelVars = TRUE,
    predVarNames = "casePred1", extraVarsToWrite = c("age"), overwrite = TRUE)  
    
    varInfoOut2 <- rxGetVarInfo(rxPredOut2)
    checkEquals(varInfoIn$age, varInfoOut2$age)
    checkEquals(names(varInfoOut2), c("casePred1", "age", "spontaneous", "induced"))    
    
    # With writeModelVars off  
    
  rxPredOut3 <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
    outData = testPredFile, writeModelVars = FALSE,
    predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = TRUE)  
    
    varInfoOut3 <- rxGetVarInfo(rxPredOut3)
    checkEquals(varInfoIn$age, varInfoOut3$age)
    checkEquals(names(varInfoOut3), c("casePred1", "age", "induced"))      
    
    # Write to same file
    rxDataStep(inData = infert, outFile = testFile, rowsPerRead = 100, overwrite = TRUE)
  rxPredOut4 <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
      outData = testFile, writeModelVars = TRUE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = TRUE)  
    varInfoOut4 <- rxGetVarInfo(rxPredOut4)
    checkEquals(varInfoIn$age, varInfoOut4$age)
    checkEquals(names(varInfoOut4), c(names(varInfoIn), "casePred1"))     
    
    rxDataStep(inData = infert, outFile = testFile, rowsPerRead = 100, overwrite = TRUE)
  rxPredOut5 <- rxPredict(modelObject = rxLogitOut1, data = testFile, 
      outData = testFile, writeModelVars = FALSE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = TRUE)  
    varInfoOut5 <- rxGetVarInfo(rxPredOut5)
    checkEquals(varInfoIn$age, varInfoOut5$age)
    checkEquals(names(varInfoOut5), c(names(varInfoIn), "casePred1"))   
    
    unlink(testFile)
    unlink(testOutFile)
    unlink(testPredFile)
    
    # Use data frame
  rxPredOut6 <- rxPredict(modelObject = rxLogitOut1, data = infert, 
      writeModelVars = TRUE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = TRUE)   
    varInfoOut6 <- rxGetVarInfo(rxPredOut6)
    checkEquals(varInfoIn$age, varInfoOut6$age)
    checkEquals(names(varInfoOut6), c("casePred1", "age", "induced", "case", "spontaneous")) 
    
    # Data frame with writeModelVars set to FALSE
    rxPredOut7 <- rxPredict(modelObject = rxLogitOut1, data = infert, 
      writeModelVars = FALSE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = TRUE)   
    varInfoOut7 <- rxGetVarInfo(rxPredOut7)
    checkEquals(varInfoIn$age, varInfoOut7$age)
    checkEquals(names(varInfoOut7), c("casePred1", "age", "induced")) 
    
    # Usea a data frame as outData
    infertPred <- infert   
    infertPred$case <- NULL
    infertPred$age <- NULL

    rxPredOut8 <- rxPredict(modelObject = rxLogitOut1, data = infert, outData = infertPred,
      writeModelVars = FALSE,
      predVarNames = "casePred1", extraVarsToWrite = c("age", "induced"), overwrite = TRUE)   
    varInfoOut8 <- rxGetVarInfo(rxPredOut8)
    checkEquals(varInfoIn$age, varInfoOut8$age)
    # Induced should not appear twice; it is in infertPred already
    checkEquals(names(varInfoOut8), c(names(infertPred), "casePred1", "age"))         
    
}

"test.rxPredict.args.extraVarsToWrite.transforms" <- function()
{
    #DEACTIVATED("Pending fix of Bug 43217: rxPredict fails if extraVarsToWrite is a transformed variable.")

    form <- isVersi ~ Sepal.Length + Petal.Width
    model <- rxLogit(form, data = iris,
        transforms = list(isVersi = Species == "versicolor"))

    extraVars <- "isVersi"
    pred <- rxPredict(model, data = iris,
        extraVarsToWrite = extraVars
        )

    checkTrue(extraVars %in% colnames(pred))
}

"test.rxPredict.args.extraVarsToWrite.transforms.error" <- function() {

    #DEACTIVATED("Pending fix of Bug 43217: rxPredict fails if extraVarsToWrite is a transformed variable.")

    form <- isVersi ~ Sepal.Length + Petal.Width
    model <- rxLogit(form, data = iris,
        transforms = list(isVersi = Species == "versicolor"))

    extraVars <- "rand"
    checkException(pred <- rxPredict(model, data = iris, extraVarsToWrite = extraVars))
}