# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxQuantile
#
############################################################################################

"test.rxQuantile.data.integer" <- function()
{
  sampleDataDir <- rxGetOption("sampleDataDir")
  dataFile <- file.path( sampleDataDir, "CensusWorkers")

  ageQuantRx <- rxQuantile(data=dataFile, varName = "age", blocksPerRead = 2)
  ageData <- rxReadXdf(file=dataFile, varsToKeep = "age")
  ageQuant <- quantile(ageData$age)

  checkEquals(ageQuant, ageQuantRx)
  
  airlineXdf <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf") 
  airlineDF <- rxReadXdf(airlineXdf, varsToKeep = "ArrDelay") 
  arrDelayXdfQuantiles <- rxQuantile(varName = "ArrDelay", data = airlineXdf, probs = seq(0, 1, by = .1))
  arrDelayDFQuantiles <- quantile(airlineDF$ArrDelay, probs = seq(0, 1, by = .1), na.rm = TRUE, type = 4)
  
  checkEquals(arrDelayXdfQuantiles, arrDelayDFQuantiles)
  
  # Try uncompressed xdf file
    airlineXdfC <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmallUC.xdf") 
  arrDelayXdfQuantilesC <- rxQuantile(varName = "ArrDelay", data = airlineXdfC, probs = seq(0, 1, by = .1))
  checkEquals(arrDelayXdfQuantilesC, arrDelayDFQuantiles)		
 
}

"test.rxQuantile.data.singleValue" <- function()
{
  testData <- data.frame(zeros = rep(0, 100))
  quantValsRx <- rxQuantile(data = testData, probs = seq(from = 0, to = 1, by = .1), varName = "zeros")
  quantVals <- quantile(testData$zeros, probs = seq(from = 0, to = 1, by = .1), type = 4)
  checkEquals(quantValsRx, quantVals)
}

"test.rxQuantile.data.seq" <- function()
{
  testData <- data.frame(xTest = 1:100)
  quantValsRx <- rxQuantile(data = testData, probs = seq(from = 0, to = 1, by = .1), varName = "xTest")
  quantVals <- quantile(testData$xTest, probs = seq(from = 0, to = 1, by = .1), type = 4)
  checkEquals(quantValsRx, quantVals)
}

"test.rxQuantile.data.numeric" <- function()
{

  set.seed(4)
  testData <- data.frame(xTest=rnorm(100000))

  quantValsRx <- rxQuantile(data = testData, probs = seq(from = 0, to = 1, by = .1), varName = "xTest")

  quantVals <- quantile(testData$xTest, probs = seq(from = 0, to = 1, by = .1), type = 4)
  checkEquals(quantValsRx, quantVals, tolerance = .001)
}

"test.rxQuantile.args.multiple" <- function()
{

  set.seed(4)
  testData <- data.frame(xTest=rnorm(100000))

  quantValsRx <- rxQuantile(data = testData, probs = seq(from = 0, to = 1, by = .1), varName = "xTest")

  quantVals <- quantile(testData$xTest, probs = seq(from = 0, to = 1, by = .1), type = 4)
  checkEquals(quantValsRx, quantVals, tolerance = .001)
}

"test.rxQuantile.args.pweights.fweights" <- function()
{
  testData <- data.frame(xTest = 1:100)
  testData$myWeights <- rep(1:5, each = 20)
  fullData <- data.frame(xTest = c(1:20, rep(21:40, each = 2), rep(41:60, each = 3), rep(61:80, each = 4),
    rep(81:100, each = 5)))
  
  quantValsRx <- rxQuantile(data = testData, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xTest", fweights = "myWeights")
  
  quantValsRx2 <- rxQuantile(data = testData, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xTest", pweights = "myWeights")
    
  quantVals <- quantile(fullData$xTest, probs = seq(from = 0, to = 1, by = .1), type = 4)
  checkEquals(quantValsRx, quantVals)
  checkEquals(quantValsRx2, quantVals)
}

"test.rxQuantile.args.probs.outOfRange" <- function()
{
  set.seed(4)
  testData <- data.frame(xTest=rnorm(100))

  res <- try(rxQuantile(data = testData, probs = c(0, .5, 2), varName = "xTest"))
    checkTrue(is(res, "try-error"))
  
  res <- try(rxQuantile(data = testData, probs = c(0, .5, -.05), varName = "xTest"))
    checkTrue(is(res, "try-error"))
}

"test.rxQuantile.data.RxSpssData" <- function()
{
  spssFile <- file.path(rxGetOption("sampleDataDir"),"claims.sav")
  spssDS <- RxSpssData(spssFile,
            colClasses = c("RowNum" = "integer", "age" = "factor",
                     "cost" = "integer", "number" = "integer")
            )		
  quantValsRx <- rxQuantile(data = spssDS, probs = seq(from = 0, to = 1, by = .1), 
    varName = "cost")
  
  spssDF <- rxImport(inData = spssDS)
  quantVals <- quantile(spssDF$cost, probs = seq(from = 0, to = 1, by = .1), type = 1, 
    na.rm = TRUE)
  checkEquals(quantValsRx, quantVals, tolerance = .001)

}
"test.rxQuantile.args.maxIntegerBins" <- function()
{
  set.seed(101)
  testInt <- data.frame(xInt = as.integer(rnorm(100000) * 100))
  quantIntRx <- rxQuantile(data = testInt, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xInt", maxIntegerBins = 100)
  quantInt <- quantile(testInt$xInt, probs = seq(from = 0, to = 1, by = .1), type = 4)
  checkTrue(sum(quantIntRx == quantInt) < 10)
  quantIntRx2 <- rxQuantile(data = testInt, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xInt", maxIntegerBins = 1000)
  checkEquals(quantIntRx2, quantInt)
}

"test.rxQuantile.args.numNumericBreaks" <- function()
{
  set.seed(101)
  testNum <- data.frame(xNum = rnorm(100000))
  quantNumRx1 <- rxQuantile(data = testNum, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xNum", numericBins = TRUE, numNumericBreaks = 10)
  quantNum <- quantile(testNum$xNum, probs = seq(from = 0, to = 1, by = .1), type = 4)
  quantNum1 <- round(quantNum*1000)/1000
  checkTrue(sum(quantNumRx1 == quantNum1) < 10)
  quantNumRx2 <- rxQuantile(data = testNum, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xNum", numericBins = TRUE, numNumericBreaks = 10000)
  checkEquals(quantNumRx2, quantNum, tolerance = .001)
}

"test.rxQuantile.args.verbose" <- function()
{
  set.seed(101)
  testInt <- data.frame(xInt = as.integer(rnorm(100000) * 100))
  out1 <- capture.output( rxQuantile(data = testInt, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xInt",  verbose = 0))

  out2 <- capture.output( rxQuantile(data = testInt, probs = seq(from = 0, to = 1, by = .1), 
    varName = "xInt", verbose = 1))
  checkTrue(length(out1) < length(out2))
}


# Code below more appropriate for acceptance test
 
#revoDataDir <- "C:/Revolution/Data"
#airData <- file.path(revoDataDir, "AirlineData87to08/AirlineData87to08")
#rxGetVarInfo(airData)	
#system.time(
#delayQuant <- rxQuantile(data=airData, probs = seq(0, 1, 0.1), varName = "ArrDelay", blocksPerRead = 10)
#)
#
#delayData <- rxReadXdf(file=airData, varsToKeep = "ArrDelay")
#system.time(
#delayQuant1 <- quantile(delayData$ArrDelay, probs = seq(0, 1, 0.1), na.rm = TRUE)
#)
#
#delayQuant
#delayQuant1

#"test.rxRocCurve.data.numeric" <- function()
#{
  #model1 <- rxLogit(case ~ spontaneous+induced, data=infert)
  #predData <- rxPredict(model1, data = infert, predVarNames = "casePred", writeModelVars = TRUE)
  #
  #rocData <- rxRocCurve("case", "casePred", predData)
#}

  
  
  
    
    



  