# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxRng
#
############################################################################################

## NOTE:
## 1. seed set before or when calling rxRngNewStream() is transformed based on the old RNG
## 2. seed set after calling rxRngNewStream() is transformed by the new RNG.

.setUp <- function()
{
}

.tearDown <- function()
{
}

#######################################################################
## save/load

"test.rxRng.unif.save.load" <- function()
{

  ## save and load
  kinds <- c("R250", "MRG32K3A", #"MCG59", "WH", "MCG31", 
    "MT19937", "MT2203", 
    "SFMT19937",
    "SOBOL")
    #"NIEDERR") Dropping support pending fix for new MKL integration
  
  num <- 5
  for (kind in kinds) {
    print(kind)
    kind.old <- rxRngNewStream (kind = kind)
    a <- runif (num)
    saved <- rxRngGetStream ()
    a1 <- runif (num)
    res <- try(
    rxRngSetStream (saved)
    )
    a2 <- runif (num)
    rxRngDelStream(kind.old[1], kind.old[2])

    checkTrue( all (a1 == a2) )
  }

  ## save and load
  kind <- "MT2203"
  
  ## generate random numbers in one run
  kind.old <- rxRngNewStream (kind = kind, seed = 777)
  a <- runif (num + num)
  rxRngDelStream(kind.old[1], kind.old[2])
  
  ## generate random numbers in two runs
  kind.old <- rxRngNewStream (kind = kind, seed = 777)
  a1 <- runif (num)
  saved <- rxRngGetStream ()	
  
  tmp <- runif (100)
  
  rxRngSetStream (saved)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  ## check results
  checkTrue( all (a == c(a1, a2)) )
}

"test.rxRng.norm.save.load" <- function()
{

  ## save and load
  normalKinds <- c("BOXMULLER", "BOXMULLER2", "ICDF")
  
  num <- 5
  for (normalKind in normalKinds) {
    kind.old <- rxRngNewStream (normalKind = normalKind)
    a <- rnorm (num)
    saved <- rxRngGetStream ()
    a1 <- rnorm (num)
    rxRngSetStream (saved)
    a2 <- rnorm (num)
    rxRngDelStream(kind.old[1], kind.old[2])

    checkTrue( all (a1 == a2) )
  }

  ## save and load
  normalKind <- "ICDF"
  
  ## generate random numbers in one run
  kind.old <- rxRngNewStream (normalKind = normalKind, seed = 777)
  a <- rnorm (num + num)
  rxRngDelStream(kind.old[1], kind.old[2])
  
  ## generate random numbers in two runs
  kind.old <- rxRngNewStream (normalKind = normalKind, seed = 777)
  a1 <- rnorm (num)
  saved <- rxRngGetStream ()	
  
  tmp <- rnorm (100)
  
  rxRngSetStream (saved)
  a2 <- rnorm (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  ## check results
  checkTrue( all (a == c(a1, a2)) )
}

#######################################################################
## seed

"test.rxRng.unif.seed.out" <- function()
{

  kind <- "MT2203"
  num <- 5	
  
  ## set the same seed outside
  seed <- 123
  
  set.seed(seed)
  kind.old <- rxRngNewStream (kind = kind)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  set.seed(seed)
  kind.old <- rxRngNewStream (kind = kind)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )
  
  ## set different seed outside 
  seed1 <- 123
  seed2 <- 321
  
  set.seed(seed1)
  kind.old <- rxRngNewStream (kind = kind)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  set.seed(seed2)
  kind.old <- rxRngNewStream (kind = kind)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
}

"test.rxRng.unif.seed.in" <- function()
{
  
  kind <- "MT2203"
  num <- 5	
  
  ## set the same seed inside
  seed <- 123
  
  kind.old <- rxRngNewStream (kind = kind)
  set.seed(seed)
  a1 <- runif (num)
  set.seed(seed)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )

  ## set different seed inside
  seed1 <- 123
  seed2 <- 321
  
  kind.old <- rxRngNewStream (kind = kind)
  set.seed(seed1)
  a1 <- runif (num)
  set.seed(seed2)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
}

"test.rxRng.unif.seed.arg" <- function()
{
  
  kind <- "MT2203"
  num <- 5	
  
  ## set the same seed in arg
  seed <- 123
  
  kind.old <- rxRngNewStream (kind = kind, seed = seed)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  kind.old <- rxRngNewStream (kind = kind, seed = seed)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )
  
  ## set different seed in arg 
  seed1 <- 123
  seed2 <- 321
  
  kind.old <- rxRngNewStream (kind = kind, seed = seed1)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  kind.old <- rxRngNewStream (kind = kind, seed = seed2)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
}

"test.rxRng.unif.seed.argout" <- function()
{

  kind <- "MT2203"
  num <- 5	
  
  ## set different seed outside but same seed in arg
  seed1 <- 123
  seed2 <- 321
  seedarg <- 111
  
  set.seed(seed1)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  set.seed(seed2)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )
  
  ## set the same seed outside but different seed in arg 
  seed <- 111
  seedarg1 <- 123
  seedarg2 <- 321
  
  set.seed(seed)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg1)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  set.seed(seed)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg2)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
  
  ## set the same seed outside also the same seed in arg
  seed <- 123
  seedarg <- 321
  
  set.seed(seed)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  set.seed(seed)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )
  
  ## set different seed outside also different seed in arg 
  seed1 <- 111
  seed2 <- 222
  seedarg1 <- 123
  seedarg2 <- 321
  
  set.seed(seed1)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg1)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  set.seed(seed2)
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg2)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
}

"test.rxRng.unif.seed.argin" <- function()
{

  kind <- "MT2203"
  num <- 5	
  
  ## set different seed inside but same seed in arg
  seed1 <- 123
  seed2 <- 321
  seedarg <- 111
  
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  set.seed(seed1)	
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  set.seed(seed2)	
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
  
  ## set the same seed inside but different seed in arg 
  seed <- 111
  seedarg1 <- 123
  seedarg2 <- 321
  
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg1)
  set.seed(seed)	
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])


  kind.old <- rxRngNewStream (kind = kind, seed = seedarg2)
  set.seed(seed)	
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )
  
  ## set the same seed inside also the same seed in arg
  seed <- 123
  seedarg <- 321
  
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  set.seed(seed)	
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  kind.old <- rxRngNewStream (kind = kind, seed = seedarg)
  set.seed(seed)	
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( all (a1 == a2) )
  
  ## set different seed inside also different seed in arg 
  seed1 <- 111
  seed2 <- 222
  seedarg1 <- 123
  seedarg2 <- 321
  
  kind.old <- rxRngNewStream (kind = kind, seed = seedarg1)
  set.seed(seed1)	
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  kind.old <- rxRngNewStream (kind = kind, seed = seedarg2)
  set.seed(seed2)	
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  checkTrue( !all (a1 == a2) )
}

"test.rxRng.unif.seed.arginout" <- function()
{
  
  kind <- "MT2203"
  num <- 5	
  
  ## seed set inside is the same as seed set in arg
  seed <- 123

  kind.old <- rxRngNewStream (kind = kind, seed = seed)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])

  kind.old <- rxRngNewStream (kind = kind)
  set.seed(seed)	
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])
  
  checkTrue( all (a1 == a2) )
  
  ## seed set outside is different from seed set in arg
  seed <- 123

  kind.old <- rxRngNewStream (kind = kind, seed = seed)
  a1 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])
  
  set.seed(seed)	
  kind.old <- rxRngNewStream (kind = kind)
  a2 <- runif (num)
  rxRngDelStream(kind.old[1], kind.old[2])
  
  checkTrue( !all (a1 == a2) )
}
  
#######################################################################
## multiple random streams

"test.rxRng.unif.stream.process" <- function()
{
  if(RevoScaleR:::rxInDatabaseHostPlatform()) DEACTIVATED("Bug 72623: localpar fails in Azure sandbox")
  
  kind <- "MT2203"
  num <- 5
  maxStreams <- 6024
  seed <- 777
  
  ## local function
  prng <- function(kind, subStream, seed)
  {
    ## TODO: remove
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    
    kind.old <- rxRngNewStream (kind = kind, subStream = subStream, seed = seed)
    pid <- Sys.getpid()
    rn <- runif(5)
    oldstate <- rxRngDelStream()
    list(pid = pid, rn = rn, oldstate = oldstate)
  }

  ## different stream in the same process
  oldcc <- rxSetComputeContext("local")
  on.exit(rxSetComputeContext(oldcc), add = TRUE)		
  res1 <- rxExec(prng, kind = kind, seed = seed, subStream = rxElemArg(1:6))
  
  ## different stream in different process
  rxSetComputeContext("localpar")
  res2 <- rxExec(prng, kind = kind, seed = seed, subStream = rxElemArg(1:6))
  
  ## get rid of 'pid' and check results
  rn1 <- sapply(res1, function(x) x$rn)
  rn2 <- sapply(res2, function(x) x$rn)
  checkEqualsNumeric(rn1, rn2)
}

#######################################################################

test.rxRng.rxExec.RNGkind <- function()
{

  kinds <- c("auto", "MCG31", "R250", "MRG32K3A", "MCG59", "MT19937", "MT2203", "SFMT19937", "SOBOL")
     #, "NIEDERR") 
    #"WH")
  for (kind in kinds)
  {
    
    x <- rxExec(rnorm, 500, timesToRun=5, RNGkind=kind)
    x.df <- data.frame(x)
    corx <- cor(x.df)
    diag(corx) <- 0
    if (!(kind %in% c("SOBOL", "NIEDERR")))
    {
      checkTrue(!any(abs(corx)> 0.3))
    }
  }
}

test.rxRng.rxExec.RNGseed <- function()
{

  kinds <- c("auto", "MCG31", "R250", "MRG32K3A", "MCG59", "MT19937", "MT2203", "SFMT19937", "SOBOL")
  #"NIEDERR") 
    #"WH")
  for (kind in kinds)
  {
    
    seed <- sample(1:250, 1)
    x1 <- rxExec(rnorm, 500, timesToRun=5, RNGkind=kind, RNGseed=seed)
    x2 <- rxExec(rnorm, 500, timesToRun=5, RNGkind=kind, RNGseed=seed)
    checkEquals(x1,x2)
    x.df <- data.frame(x1)
    corx <- cor(x.df)
    diag(corx) <- 0
    if (!(kind %in% c("SOBOL", "NIEDERR")))
    {
      if (any(abs(corx)> 0.3)) {
        print(paste("Oops, maximum correlation was ", max(abs(corx)), sep=""))
        checkTrue(FALSE)
      } else {
        checkTrue(!any(abs(corx)> 0.3))
      }
    }
  }
}

#######################################################################
## tests to make sure that the RNGs based on MKL are platform-independent

#bigAnalytics RA-9927
#Suspected difference in parallel random number generator on MKL between Windows and Linux versions (as exhibited in rxDForest tests)

## MKL RNG Bug 6000037558: The random number generators in MKL/VSL are platform-dependent when the seed >= 2^31.
test.rxRng.cross.platform.seed <- function()
{
    "PRNG" <- function(kind = "MT2203", subStream = 0, seed = NULL, dRng = "runif", length = 1)
    { 
        rxRngNewStream(kind = kind, subStream = subStream, seed = seed) 
        rn <- do.call(dRng, list(length))
        rxRngDelStream()
    rn
    } 
  
    res <- rxExec(PRNG, kind = "MT2203", subStream = rxElemArg(c(0, 1)), 
    seed = rxElemArg(c(17, 123)), dRng = "runif", length = 5)
  
    #ans <- structure(list(
    #rxElem1 = c(0.124699725303799, 0.754309776239097, 0.349158101016656, 0.746507699834183, 0.884273183066398), 
    #rxElem2 = c(0.404556232970208, 0.750813415274024, 0.397173646138981, 0.12285250891, 0.334307834738865)
    #), .Names = c("rxElem1", "rxElem2"))
  
    ans <- structure(list(
        rxElem1 = c(0.381748995743692, 0.719091505743563, 0.644961138488725, 0.84546911646612, 0.573316463269293), 
        rxElem2 = c(0.404556232970208, 0.750813415274024, 0.397173646138981, 0.12285250891, 0.334307834738865)
        ), .Names = c("rxElem1", "rxElem2"))
    
    checkEquals(res, ans)
}

## MKL RNG Bug 6000037527: The random number generators in MKL/VSL are platform-dependent when a vector of random numbers are produced.
test.rxRng.cross.platform.vector <- function()
{
    n <- 2e5    
    myRngUnifInt <- if ("package:RevoScaleR" %in% search()) RevoScaleR:::rxRngUnifInt else rxRngUnifInt
  
    seed <- 0   #12735 ok #
    batchLength <- 1  # seed <- 0: < 16 same, >= 16 diff; seed <- 12735: -1 same
    r <- myRngUnifInt(n, min = 0, max = n, seed = seed, batchLength = batchLength)
    (r.sum <- sum(as.numeric(r)))
    checkIdentical(r.sum, 19990225232)
    
    #DEACTIVATED("MKL RNG Bug 6000037527 (Linux only)")  # need to update MKL to version 11.1.1
    
    seed <- 0   #12735 ok #
    batchLength <- -1  # failed only on Linux
    r <- myRngUnifInt(n, min = 0, max = n, seed = seed, batchLength = batchLength)
    (r.sum <- sum(as.numeric(r)))
    checkIdentical(r.sum, 19990225232)  # 19990225227 on Linux
    
    seed <- -1
    batchLength <- 1  # seed <- 0: < 16 same, >= 16 diff; seed <- 12735: -1 same
    r <- myRngUnifInt(n, min = 0, max = n, seed = seed, batchLength = batchLength)
    (r.sum <- sum(as.numeric(r)))
    checkIdentical(r.sum, 20009339747)  # 20009339747 on Linux
}

#Defect 7939722:rxKMeans fails only when rxDForest is run before the call
test.CxOpRandUniform.rxKMeans.after.rxDForest <- function()
{
    adsFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")

    model1 <- rxDForest(ArrDelay~CRSDepTime, data = adsFile,
        computeOobError = -1, nTree = 2, maxDepth = 2)
    model2 <- rxKmeans( ~ArrDelay + CRSDepTime, data = adsFile, numClusters = 2)
    
    checkTrue(TRUE)
}
