# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxSetVarInfo and rxSetVarInfoXdf
#
############################################################################################

"test.rxSetVarInfoXdf.args.varInfo.levels" <- function()
{
    # Specify name and location of output file
    outFile <-  ".rxTempFile.xdf"
    if (file.exists(outFile)) file.remove(outFile)
    # Create a sample data set
    # We will change factor levels
    set.seed(100)
    myData <- data.frame(y=rnorm(30), month=as.factor(c(0:11,as.integer(runif(18,0,12)))))
    # Convert new data to an XDF file
    rxDataStep(myData, outFile=outFile)
    myData1 <- rxReadXdf(file=outFile)

    # Get the varInfo of the new XDF file
    varInfo <- rxGetVarInfo(outFile)
  oldVarInfo <- varInfo

    # Set the labels for the month variable
    monthLabels <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
    varInfo$month$levels <- monthLabels

    # Write the modified varInfo back to the file
    outFileDS <- rxSetVarInfoXdf(varInfo, outFile)
  
    # Read the data and check to see if month is now a factor
    myData2 <- rxReadXdf(file=outFileDS)
    checkEquals(is.integer(myData2$month), FALSE)
    checkEquals(is.factor(myData2$month), TRUE)
    checkTrue(myData2$month[1] == "Jan")
    checkTrue(myData2$month[30] == monthLabels[as.integer(myData1$month[30])])
  
  file.remove(outFile)
}

"test.rxSetVarInfoXdf.args.varInfo.simple" <- function()
{
    testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <-  "rxTempTestCopyXdf.xdf"
  file.copy(testXdf, to=testCopyXdf, overwrite=TRUE)
    varInfo <- rxGetVarInfo(testCopyXdf)
    rxSetVarInfoXdf(varInfo, file=testCopyXdf)
    varInfo1 <- rxGetVarInfo(testCopyXdf)
    checkEquals(varInfo, varInfo1)
}

"test.rxSetVarInfoXdf.args.varInfo" <- function()
{
  # Specify name and location of test data file
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <-  "rxTempTestCopyXdf.xdf"
  file.copy(testXdf, to=testCopyXdf, overwrite=TRUE)
  
  # Read the entire varInfo 
  varInfo <- rxGetVarInfo(testCopyXdf)
  
    # Change levels of xfac2
  testLevels <- varInfo$xfac2$levels
  testLevels <- paste("test", testLevels, sep="") 
  varInfo$xfac2$levels <- testLevels
    rxSetVarInfoXdf(varInfo, testCopyXdf)
  
  # Read modified XDF file into data frame
  testDF <- rxReadXdf(testCopyXdf)
  # Compare the factor levels with the levels set
  checkEquals(levels(testDF$xfac2), testLevels)
  # Make sure we still have varInfo for all 9 variables
  varInfo <- rxGetVarInfo(testCopyXdf)
  checkEquals(length(varInfo), 9)
  
  # Cleanup
  file.remove(testCopyXdf)
}

"test.rxSetVarInfoXdf.args.varInfo.valueInfoCodes" <- function()
{
  # Specify name and location of test data file
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <-  "rxTempTestCopyXdf.xdf"
  file.copy(testXdf, to=testCopyXdf, overwrite=TRUE)
  
  # Read the entire varInfo 
  varInfo <- rxGetVarInfo(testCopyXdf)
  
    # Add value info
  valueInfoCodes = c("13", "14")
  valueInfoLabels = c("Not home", "Won't answer")
  varInfo$xint1$valueInfoCodes <- valueInfoCodes
  varInfo$xint1$valueInfoLabels <- valueInfoLabels
    rxSetVarInfoXdf(varInfo, testCopyXdf)
  
  # Read modified XDF file into data frame
  testDF <- rxReadXdf(testCopyXdf)
  # Compare the factor levels with the levels set
  checkEquals(attr(testDF$xint1, ".rxValueInfoCodes"), valueInfoCodes)
  checkEquals(attr(testDF$xint1, ".rxValueInfoLabels"), valueInfoLabels)
  
  # Make sure we still have varInfo for all 9 variables
  varInfo <- rxGetVarInfo(testCopyXdf)
  checkEquals(length(varInfo), 9)
  
  # Cleanup
  file.remove(testCopyXdf)
}

"test.rxSetVarInfoXdf.args.varsToKeep" <- function()
{
  # Specify name and location of test data file
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <- "rxTempCopyXdf.xdf"
  file.copy(testXdf, to=testCopyXdf, overwrite=TRUE)
  
  # Read the varInfo using varsToKeep
  varInfo <- rxGetVarInfo(testCopyXdf, varsToKeep=c("xnum2", "xfac2"))
  
    # Change levels of xfac2
  testLevels <- varInfo$xfac2$levels
  testLevels <- paste("test", testLevels, sep="") 
  varInfo$xfac2$levels <- testLevels
    rxSetVarInfoXdf(varInfo, testCopyXdf)
  
  # Read modified XDF file into data frame
  testDF <- rxReadXdf(testCopyXdf)
    
  # Compare the factor levels with the levels set
  checkEquals(levels(testDF$xfac2), testLevels)
  
    # Make sure we still have varInfo for all 9 variables
  varInfo <- rxGetVarInfo(testCopyXdf)
  checkEquals(length(varInfo), 9)
  
  # Cleanup
  file.remove(testCopyXdf)
}

"test.rxSetVarInfoXdf.args.varInfo.newName" <- function()
{
  
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <-  "rxTempTestCopyXdf.xdf"
  file.copy(testXdf, to=testCopyXdf, overwrite=TRUE)
  
  newBinaryName <- "yBinaryInteger"
  newMissingName <- "MissingXnum2"
    varList <- list(ybinint=list(newName=newBinaryName), list(position=6, newName=newMissingName))
  
  rxSetVarInfoXdf(varList, testCopyXdf)
  varInfo <- rxGetVarInfo(testCopyXdf)
  varNames <- names(varInfo)

  checkEquals(varNames[3], newBinaryName)
  checkEquals(varNames[6], newMissingName)
  
  # Cleanup
  file.remove(testCopyXdf)
}

"test.rxSetVarInfoXdf.args.varInfo.description" <- function()
{
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <-  "rxTempTestCopyXdf.xdf"
  file.copy(testXdf, to=testCopyXdf, overwrite=TRUE)
  
  yDescrip <- "My y variable"
  descrip4 <- "My 4th variable"
  newName4 <- "xNumber1"
  xfac2Descrip <- "Factor version of x2"
    varList <- list(
    y=list(description=yDescrip), 
    list(position=4, newName=newName4, description=descrip4),
    xfac2=list(description=xfac2Descrip))
  
  rxSetVarInfoXdf(varList, testCopyXdf)
  varInfo <- rxGetVarInfo(testCopyXdf)
  varNames <- names(varInfo)

  checkEquals(varNames[4], newName4)
  checkEquals(varInfo$y$description, yDescrip)
  checkEquals(varInfo[[4]]$description, descrip4)
  checkEquals(varInfo[[9]]$description, xfac2Descrip)
  # Cleanup
  file.remove(testCopyXdf)
}

"test.rxSetVarInfo.args.varInfo.description" <- function()
{
    # Test for data frame
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testDF <-  rxReadXdf(testXdf)
  
  yDescrip <- "My y variable"
  descrip4 <- "My 4th variable"
  newName4 <- "xNumber1"
  xfac2Descrip <- "Factor version of x2"
    varList <- list(
    y= list(description=yDescrip), 
    list(position=4, newName=newName4, description=descrip4),
    xfac2 = list(description=xfac2Descrip))
  
  testCopyDF <- rxSetVarInfo(varList, testDF)
  varInfo <- rxGetVarInfo(testCopyDF)
  varNames <- names(varInfo)

  checkEquals(varNames[4], newName4)
  checkEquals(varInfo$y$description, yDescrip)
  checkEquals(varInfo[[4]]$description, descrip4)
  checkEquals(varInfo[[9]]$description, xfac2Descrip)

}

"test.rxSetVarInfo.args.varInfo.levels" <- function()
{
    # Test for data frame
    # Create a sample data set
    # We will change factor levels
    set.seed(100)
    myData <- data.frame(y=rnorm(30), month=as.factor(c(0:11,as.integer(runif(18,0,12)))))

    # Get the varInfo for the data frame
    varInfo <- rxGetVarInfo( myData )
  oldVarInfo <- varInfo

    # Set the labels for the month variable
    monthLabels <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
    varInfo$month$levels <- monthLabels

    # Write the modified varInfo back to the file
    myData2 <- rxSetVarInfo(varInfo, myData)
  
    # Read the data and check to see if month is still a factor but with new labels
    checkEquals(is.integer(myData2$month), FALSE)
    checkEquals(is.factor(myData2$month), TRUE)
    checkTrue(myData2$month[1] == "Jan")
    checkTrue(myData2$month[30] == monthLabels[as.integer(myData$month[30])])
}

"test.rxSetVarInfo.value" <- function()
{
  
    set.seed(100)
    myData1 <- data.frame(y = rnorm(30),
                      month = factor(c(0:11, as.integer(runif(18, 0, 12)))))
                      
    # Set the labels for the month variable
    monthLabels <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
                     "Oct", "Nov", "Dec")
                 
    # Get the variable information for the data frame
    varInfo <- rxGetVarInfo(myData1)

    # Reset the names of the factor levels
    varInfo$month$levels <- monthLabels
    myData2 <- rxSetVarInfo( varInfo, myData1)
    
    # Redo, this time putting the data in an .xdf file
    # Specify name of output file
    tempFile <- file.path(tempdir(), "rxTempTestFile.xdf")

    # Convert the original data to an XDF file
    rxDataStep(myData1, outFile = tempFile, overwrite = TRUE)

    # Get the varInfo of the new XDF file
    varInfo <- rxGetVarInfo(tempFile)

    varInfo$month$levels <- monthLabels

    # Write the modified varInfo back to the file
    myDS <- rxSetVarInfo(varInfo, tempFile)
    
    myVarInfoDF <- rxGetVarInfo(myData2)
    myVarInfoDS <- rxGetVarInfo(myDS)

    checkEquals(myVarInfoDF$month$levels, myVarInfoDS$month$levels)
    checkEquals(myVarInfoDS$month$levels, monthLabels)
    
    file.remove( tempFile )
}

"test.rxSetVarInfo.data.DatePOSIXctOrdered" <- function()
{
  
  myFileXdf <- tempfile(pattern = ".rxSetVarInfo", fileext = ".xdf") 
  dates <- c("02/27/92", "02/27/92", "01/14/92", "02/28/92", "02/01/92",
                "03/27/92", "04/27/02", "06/14/03", "02/28/98", "02/01/04")
            
   datetimes <- c("1999-03-01 01:30:00", "2000-03-02 01:30:00", 
    "2001-03-03 01:30:00", "2002-03-04 01:30:03", "2003-03-27 01:30:00",
    "2004-03-27 01:30:00", "2005-03-27 01:30:00", "2006-03-14 12:30:00", 
    "2007-03-27 23:30:00", "2008-03-15 01:25:00")            

    myDF <- data.frame(
        x = 1:10, 
    yfac = factor(rep(1:2, times = 5), ordered = TRUE),
        d = as.Date(dates, "%m/%d/%y"),
        dt = as.POSIXct(strptime(datetimes, '%Y-%m-%d %H:%M:%S')),
        dtz = as.POSIXct(strptime(datetimes, '%Y-%m-%d %H:%M:%S', tz="EST"), tz="EST"))

  rxDataStep(myDF, myFileXdf, overwrite = TRUE)
    varInfoXdf <- rxGetVarInfo(myFileXdf)
  
  varInfoNew <- list(dt = list(newName = "MyNewDate")) 
  rxSetVarInfo(varInfoNew, myFileXdf)
  
  varInfoNew <- rxGetVarInfo(myFileXdf)
  checkEquals(names(varInfoNew)[4], "MyNewDate")
  names(varInfoXdf) <- names(varInfoNew)
    
  checkEquals(varInfoNew, varInfoXdf)
  file.remove(myFileXdf)
}

"test.rxSetVarInfo.args.data.dataSource" <- function()
{
  
  testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")
  testCopyXdf <-  "rxTempTestCopyXdf.xdf"
  filePath <- tempdir()
  testCopyXdfFullPath <- file.path(filePath, testCopyXdf)
  file.copy(testXdf, to=testCopyXdfFullPath, overwrite=TRUE)
  
  newBinaryName <- "yBinaryInteger"
  newMissingName <- "MissingXnum2"
    varList <- list(ybinint=list(newName=newBinaryName), list(position=6, newName=newMissingName))
  oldOptions <- rxOptions(dataPath = filePath)
  on.exit(rxOptions(dataPath = oldOptions$dataPath), add = TRUE)
  rxSetVarInfo(varList, data = RxXdfData(testCopyXdf))
  varInfo <- rxGetVarInfo(testCopyXdf)
  varNames <- names(varInfo)

  checkEquals(varNames[3], newBinaryName)
  checkEquals(varNames[6], newMissingName)
  
  # Cleanup
  file.remove(testCopyXdfFullPath)
}
