# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxSummary
#
############################################################################################

# rUnit should create these objects just before defining and executing the tests in this file,
# then clean them up automatically after all tests in the file are finished.
unitTestDataDir <- rxGetOption("unitTestDataDir")
# Instead use R's tempdir()
toutDir <- tempdir()
testOutputDir <- file.path(toutDir, "testOutput")
sampleDataDir <- rxGetOption("sampleDataDir")
require(MASS)

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

.setUp <- function()
{
  if (file.exists(testOutputDir))
  {
    unlink(testOutputDir,recursive=TRUE)
  }
  dir.create(testOutputDir)
}

.tearDown <- function()
{
  unlink(testOutputDir,recursive=TRUE)
}

test.rxSummary.Rcallback.xdf <- function()
{
  sampleDataDir <- rxGetOption("sampleDataDir")
    dataFile <- file.path(sampleDataDir, "CensusWorkers")
    
    xform <- function(x)
    {
      x$newage <- as.numeric(x$age) * 10
      return(x)
    }

    res <- try(rxSummary(~newage + sex, data=dataFile, transformFunc=xform, transformVars=c("age","sex")))
    checkTrue(!is(res, "try-error"))
}

test.rxSummary.Output <- function()
{
  long.shoots.df <- data.frame(
        Day = c(0,3,7,13,18,24,25,32,38,42,44,49,52,55,58,61,69,73,76,88,100,106),
      myweights = c(5,5,5,6,5,4,6,5,7,9,10,19,14,11,9,14,10,12,9,7,10,7),
        Stems.Avg=c(10.20,10.40,10.60,12.50,12.00,15.00,
          15.17,17.00,18.71,19.22,20.00,20.32,22.07,22.64,22.78,23.93,25.50,
            25.08,26.67,28.00,31.67,32.14))

  rxsum.out <- rxSummary(~ I(Day^2), long.shoots.df, transformVars="Day")
  gen.mean <- mean((long.shoots.df$Day)^2)
  gen.sd <- sd(long.shoots.df$Day^2)
  gen.min <- min(long.shoots.df$Day^2)
  gen.max <- max(long.shoots.df$Day^2)
  gen.numobs <- length(long.shoots.df$Day)
  rxsum.mean <- rxsum.out$sDataFrame[2]

  ### Remove names from object
  rxsum.mean <- unlist(rxsum.mean, use.names=FALSE)
  rxsum.sd <- rxsum.out$sDataFrame[3]
  rxsum.sd <- unlist(rxsum.sd, use.names=FALSE)
  rxsum.min <- rxsum.out$sDataFrame[4]
  rxsum.min <- unlist(rxsum.min, use.names=FALSE)
  rxsum.max <- rxsum.out$sDataFrame[5]
  rxsum.max <- unlist(rxsum.max, use.names=FALSE)
  rxsum.numobs <- rxsum.out$sDataFrame[6]
  rxsum.numobs <- unlist(rxsum.numobs, use.names=FALSE)
  checkTrue(all.equal(rxsum.mean,gen.mean))
  checkTrue(all.equal(rxsum.sd,gen.sd))
  checkTrue(all.equal(rxsum.min,gen.min))
  checkTrue(all.equal(rxsum.max,gen.max))
  checkTrue(all.equal(rxsum.numobs,gen.numobs))
}

###
# argument stress tests
###

"test.rxSummary.args.useSparseCube.mixed" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- ~ age:car.age:type:cost:number    #mixed continuous and discrete case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxSummary( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxSummary( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxSummary( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxSummary( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxSummary.args.useSparseCube.continuous" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- ~ cost:number    #pure continuous case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxSummary( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxSummary( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxSummary( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxSummary( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxSummary.args.useSparseCube.discrete" <- function()
{
    dataDir <- rxGetOption("sampleDataDir")
    myData <- file.path(dataDir, "claims4blocks.xdf") 
    #rxGetInfo(myData, getVarInfo = TRUE, getBlockSizes = TRUE)    
    myForm <- ~ age:car.age:type    #pure discrete case
    
    ###############################
    useSparseCubeF.removeZeroCountsT <- rxSummary( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = TRUE,
        )    
    useSparseCubeT.removeZeroCountsT <- rxSummary( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = TRUE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsT), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsT[domatch], useSparseCubeT.removeZeroCountsT[domatch])
    
    ###############################
    useSparseCubeF.removeZeroCountsF <- rxSummary( myForm, data = myData, 
        useSparseCube = FALSE, removeZeroCounts = FALSE,
        )    
    useSparseCubeT.removeZeroCountsF <- rxSummary( myForm, data = myData, 
        useSparseCube = TRUE, removeZeroCounts = FALSE,
        )

  dontmatch <- c("params", "call")
  domatch <- setdiff(names(useSparseCubeF.removeZeroCountsF), dontmatch)
    checkEquals(useSparseCubeF.removeZeroCountsF[domatch], useSparseCubeT.removeZeroCountsF[domatch])
}

"test.rxSummary.args.blocksPerRead" <- function()
{
    test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
    test100r1b <- file.path(rxGetOption("unitTestDataDir"), "test100r1b")
    test100r5bStartMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bStartMiss")
    test100r5bEndMiss <- file.path(rxGetOption("unitTestDataDir"), "test100r5bEndMiss")
    test77r1bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r1bNoMiss")
    test77r5bNoMiss <- file.path(rxGetOption("unitTestDataDir"), "test77r5bNoMiss")
    
    sumtest1 <- rxSummary(~y + xnum2miss, data=test100r5b)
    sumtest1DF <- sumtest1$sDataFrame
    
    # Try different blocks per read
    sumtest2 <- rxSummary(~y + xnum2miss, data=test100r5b, blocksPerRead=1)
    checkTrue(all.equal(sumtest2$sDataFrame, sumtest1DF))
        
        # Variations on data sources
        o2a <- capture.output(
            sumtest2a <- rxSummary(~y + xnum2miss, data=test100r5b, blocksPerRead=2)
            )
        
        xdfDS <- RxXdfData(test100r5b)
        o2b <- capture.output(
            sumtest2a <- rxSummary(~y + xnum2miss, data=xdfDS, blocksPerRead=2)
            )
        checkEquals(length(o2a), length(o2b))
        
        o2c <- capture.output(
            sumtest2a <- rxSummary(~y + xnum2miss, data=RxXdfData(test100r5b, blocksPerRead=2))
            )   
        checkEquals(length(o2a), length(o2c))
        o2d <- capture.output(
            sumtest2a <- rxSummary(~y + xnum2miss, data=RxXdfData(test100r5b, blocksPerRead=5), blocksPerRead = 2)
            )       
    checkEquals(length(o2a), length(o2d))
    
    defaultBlocksPerRead <- rxGetOption("blocksPerRead")
    rxOptions(blocksPerRead = 2)
    on.exit(rxOptions(blocksPerRead = defaultBlocksPerRead), add = TRUE)		
      o2e <- capture.output(

            sumtest2a <- rxSummary(~y + xnum2miss, data=test100r5b)

        ) 
      rxOptions(blocksPerRead = defaultBlocksPerRead)  	
    checkEquals(length(o2a), length(o2e))
        
    sumtest2 <- rxSummary(~y + xnum2miss, data=test100r5b, blocksPerRead=5)
    checkTrue(all.equal(sumtest2$sDataFrame, sumtest1DF))
    
    # Data set with just 1 block
    sumtest2 <- rxSummary(~y + xnum2miss, data=test100r1b, blocksPerRead=1)
    checkTrue(all.equal(sumtest2$sDataFrame, sumtest1DF))
    
    # Redo removing missing by observation
    sumtest1 <- rxSummary(~y + xnum2miss, data=test100r5b, byTerm=FALSE)
    sumtest1DF <- sumtest1$sDataFrame
    
    # Data set with missings removed
    sumtest2 <- rxSummary(~y + xnum2miss, data=test77r1bNoMiss , blocksPerRead=1)
    checkTrue(all.equal(sumtest2$sDataFrame[,1:6], sumtest1DF))
    
    sumtest2 <- rxSummary(~y + xnum2miss, data=test77r5bNoMiss , blocksPerRead=1)
    checkTrue(all.equal(sumtest2$sDataFrame[,1:6], sumtest1DF))
    
    # Datasets with missings at beginning and end
    
    sumtest2 <- rxSummary(~y + xnum2miss, data=test100r5bStartMiss , blocksPerRead=1, byTerm=FALSE)
    checkTrue(all.equal(sumtest2$sDataFrame, sumtest1DF))
    
    sumtest2 <- rxSummary(~y + xnum2miss, data=test100r5bEndMiss , blocksPerRead=1, byTerm=FALSE)
    checkTrue(all.equal(sumtest2$sDataFrame, sumtest1DF))
    
    sumtest2 <- rxSummary(~y + xnum2miss, data=test100r5bEndMiss , blocksPerRead=5, byTerm=FALSE)
    checkTrue(all.equal(sumtest2$sDataFrame, sumtest1DF))

}

# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxSummary.args.cppInterp" <- function()
#{
    #dataFile <- file.path(sampleDataDir, "CensusWorkers")
    #summaryRce.test <- rxSummary("@age+sex",data=dataFile)
    #summaryR.test <- rxSummary(~age+sex, data=dataFile)
    #checkTrue(all.equal(summaryRce.test$sDataFrame[,2:6], summaryR.test$sDataFrame[,2:6]))
  #
  #summaryRce.test <- rxSummary("@age+sex", data=dataFile, cppInterp=list(rowSelection="age>50"))
  #summaryRce2.test <- rxSummary(~F(age)+sex, data=dataFile, cppInterp=list(rowSelection="age>50"))
    #checkTrue(all.equal(summaryRce.test$sDataFrame[,2:6], summaryRce2.test$sDataFrame[,2:6]))
#}

"test.rxSummary.args.data" <- function()
{	
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  XDF <- tempfile( pattern = ".rxSum1", fileext = ".xdf")
  rxDataStep(DF, XDF)
  
  zSummaryDF <- rxSummary(form, data=DF)
  zSummaryXDF <- rxSummary(form, data=XDF)
  nocomp <- which(names(zSummaryDF) %in% c("call","params"))
  checkEquals(zSummaryDF[-nocomp], zSummaryXDF[-nocomp])
  file.remove(XDF)
}

"test.rxSummary.args.formula" <- function()
{	
  form <- ~ state
  formStr <- "state"
  set.seed(10)
  DF <- data.frame(state=sample(state.name, 5, replace=TRUE), division=letters[1:5])
  zSummaryFormula <- rxSummary(form, data=DF)
  zSummaryString <- rxSummary(formStr, data=DF)
  nocomp <- which(names(zSummaryString) %in% c("call","params","formula"))
  checkEquals(zSummaryFormula[-nocomp], zSummaryString[-nocomp])
  
  dataFile <- file.path(sampleDataDir, "CensusWorkers")
  rxSummaryResult <- rxSummary(~age+incwage+perwt+wkswork1, data=dataFile) 
  myData <- rxReadXdf(dataFile, varsToKeep = c("age", "incwage", "perwt", "wkswork1"))
  meanData <- sapply(myData, mean)
  sdData <- sapply(myData, sd)
  checkEquals(meanData, rxSummaryResult$sDataFrame$Mean, check.attributes=FALSE)
  checkEquals(sdData, rxSummaryResult$sDataFrame$StdDev, check.attributes=FALSE)
}

"test.rxDataStep.args.formula.emptyFactor" <- function()
{
  outFile <- tempfile( pattern = ".rxDFX1", fileext = ".xdf")
  
  myData <- data.frame(x = factor(rep(c(2:3), times = 3), levels = c(1:3)),
                       y = c(1,2,2,3,3,3))
  rxDataStep(inData = myData, outFile = outFile, overwrite = TRUE)
  
  # Make sure data frame and xdf return same results for empty factor
  sumDF <- rxSummary(~ x, myData, reportProgress = 0)
  sumXdf <- rxSummary(~ x, outFile, reportProgress = 0)
  checkEquals(sumDF$categorical, sumXdf$categorical)
  checkEquals(sumDF$categorical[[1]][1, "Counts"], 0)
  rm(sumDF, sumXdf)
  
  # Check to make sure F works
  sumDF <- rxSummary(~ F(y), myData, reportProgress = 0)
  sumXdf <- rxSummary(~ F(y), outFile, reportProgress = 0)
  checkEquals(sumDF$categorical, sumXdf$categorical)
  checkEquals(sumDF$categorical[[1]][1, "Counts"], 1)
  checkEquals(sumDF$categorical[[1]][2, "Counts"], 2)
  checkEquals(sumDF$categorical[[1]][3, "Counts"], 3)
  rm(sumDF, sumXdf)
  
  sumDF <- rxSummary(~ F(y, 2), myData, reportProgress = 0)
  sumXdf <- rxSummary(~ F(y, 2), outFile, reportProgress = 0)
  checkEquals(sumDF$categorical, sumXdf$categorical)
  checkEquals(sumDF$categorical[[1]][1, "Counts"], 2)
  checkEquals(sumDF$categorical[[1]][2, "Counts"], 3)
  
  file.remove(outFile)
}

"test.rxSummary.args.formula.single.term.interaction.conversion" <- function()
{
  DF <- as.data.frame(UCBAdmissions)
  
  z <- rxSummary(Freq ~ Admit, data = DF)
  z1 <- rxSummary(~Freq:Admit, data = DF)
  checkEquals(z$categorical, z1$categorical)
  checkEquals(z$sDataFrame, z1$sDataFrame)
  
  z <- rxSummary(Freq ~ Admit:Gender, data  =DF)
  z1 <- rxSummary(~Freq:Admit:Gender, data = DF)
  checkEquals(z$categorical, z1$categorical)
  checkEquals(z$sDataFrame, z1$sDataFrame)
  
  z <- rxSummary(~ Admit:Gender, data =DF)
  z1 <- rxSummary(Admit~Gender, data = DF)
  checkEquals(z$categorical, z1$categorical)
  checkEquals(z$sDataFrame, z1$sDataFrame)
}

"test.rxSummary.args.byTerm" <- function()
{
  testFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf")

  sumObj1 <- rxSummary(~., data=testFile)
  checkEquals(sumObj1$sDataFrame$ValidObs[1], 100)
  checkEquals(sumObj1$sDataFrame$ValidObs[6], 77)
  
  sumObj2 <- rxSummary(~., data=testFile, byTerm=FALSE)
  checkEquals(sumObj2$sDataFrame$ValidObs[1], 77)
  checkEquals(sumObj2$sDataFrame$ValidObs[6], 77)

}

"test.rxSummary.args.byTerm.factor" <- function()
{
  x0 <- c( 1, 3, 2, 4, 3, 1, 3, 4, 2, 2)
  xint0 <- as.integer(x0)
  xfac0 <- as.factor(xint0)

  x1 <- c( 1, 3, 2, 4, 3, NA, 3, 4, 2, 2)
  xint1 <- as.integer(x1)
  xfac1 <- as.factor(xint1)
  
  x2 <- c( 1, 3, NA, 4, 3, NA, 3, 4, 2, 2)
  xint2 <- as.integer(x2)
  xfac2 <- as.factor(xint2)
  
  testData <- data.frame(x0, xint0, xfac0, x1, xint1, xfac1, x2, xint2, xfac2)
  
  sumObj1 <- rxSummary(~., data=testData, byTerm = TRUE)
  out1 <- capture.output( sumObj1 )
  checkEquals(out1[8], " Name  Mean     StdDev   Min Max ValidObs MissingObs" )
  checkEquals(out1[12]," xint1 2.666667 1.000000 1   4    9       1         " )
  checkEquals(out1[30], "Number of missing observations: 1" )
  
  sumObj2 <- rxSummary(~., data=testData, byTerm=FALSE)
  out2 <- capture.output( sumObj2 )
  checkEquals(out2[7], "Number of missing observations: 2 " )
  checkEquals(out2[9],  " Name  Mean StdDev   Min Max" )
  checkEquals(out2[17], "Category Counts for xfac0" )
  
  sumObj3 <- rxSummary(~xint1 + F(xint2), data=testData, byTerm = TRUE)
  out3 <- capture.output( sumObj3 )
  checkEquals(out3[14],  "Number of missing observations: 2" )
  
  sumObj4 <- rxSummary(~xint1 + F(xint2), data=testData, byTerm = FALSE)
  out4 <- capture.output( sumObj4 )
  checkEquals(out4[7], "Number of missing observations: 2 " )
  checkEquals(out4[9], " Name  Mean StdDev   Min Max" )
  
  # Just factors
  sumObj5 <- rxSummary(~xfac1 + xfac2, data=testData, byTerm = TRUE)
  out5 <- capture.output( sumObj5 )
  checkEquals(out5[23],  "Number of missing observations: 2" )
  
  sumObj6 <- rxSummary(~xfac1 + xfac2, data=testData, byTerm = FALSE)
  out6 <- capture.output( sumObj6 )
  checkEquals(out6[7], "Number of missing observations: 2 " )
  checkEquals(out6[10], "Category Counts for xfac1"    )

}

"test.rxSummary.args.fweights" <- function()
{
  # create frequency weighting vector whose length is equal to the
  # number of rows inour test data frame. select weight randomly from 
  # the set of integers on the interval [1,4]
  set.seed(10)
  DF <- infert
  DF$fwts <- sample(1:4, nrow(DF), replace=TRUE)
  
  # create new data frame where the rows are replicated explicitly according
  # to the number frequency weighting vector
  DF2 <- as.data.frame(lapply(DF, rep, times=DF$fwts))
  DF2$fwts <- NULL
  
  # define common formula	
  form <- ~ age + parity + education + spontaneous + induced
  
  # calculate logistic regression coefficients for the two cases and compare
  z1 <- rxSummary(form, data=DF, fweights = "fwts")
  z2 <- rxSummary(form, data=DF2)
  nocomp <- which(names(z1) %in% c("call","params","nobs.valid"))
  checkEquals(z1[-nocomp], z2[-nocomp], check.attributes=FALSE)
}

"test.rxSummary.args.fweights.missings" <- function()
{
  # Create frequency weighting vector whose length is equal to the
  # number of rows inour test data frame. select weight randomly from 
  # the set of integers on the interval [1,4]
  set.seed(10)
  DF <- infert
  DF$induced[c(5, 10, 15, 20, 25, 30, 200, 244)] <- NA
  DF$fwts <- sample(1:4, nrow(DF), replace=TRUE)
  
  # create new data frame where the rows are replicated explicitly according
  # to the number frequency weighting vector
  DF2 <- as.data.frame(lapply(DF, rep, times=DF$fwts))
  DF2$fwts <- NULL
  
  # define common formula	
  form <- ~ age + parity + education + spontaneous + induced
  
  # calculate logistic regression coefficients for the two cases and compare
  z1 <- rxSummary(form, data=DF, fweights="fwts")
  z2 <- rxSummary(form, data=DF2)
  nocomp <- which(names(z1) %in% c("call","params","nobs.valid"))
  checkEquals(z1[-nocomp], z2[-nocomp], check.attributes=FALSE)
}

"test.rxSummary.args.pweights" <- function()
{
  fwts <- 1:4
  sex <- c("Male","Male","Female", "Male")
  age <- c(20,20,12,15)
  score <- (1:4)+0.1
  
  myDF1 <- data.frame(sex=sex, age=age, score=score, pWeights=fwts)
  myDF2 <- data.frame(sex=rep(sex,fwts), age=rep(age,fwts), score=rep(score,fwts))
  
  z1 <- rxSummary(~age + sex + score, data=myDF1, pweights="pWeights")
  z2 <- rxSummary(~age + sex + score, data=myDF2)
  pw <- "pWeights"
  z3 <- rxSummary(~age + sex + score, data = myDF1, fweights = pw)
  
  z1Data <- z1$sDataFrame
  z2Data <- z2$sDataFrame
  z3Data <- z3$sDataFrame
  checkTrue(all.equal(z1Data$Mean,z2Data$Mean))
  
  # TODO: The StdDev's are not the same for pweights and fweights (RA-1286)
  #checkTrue(all.equal(z1Data$StdDev,z2Data$StdDev))
  checkTrue(all.equal(z1Data$SumOfWeights,z2Data$ValidObs)) 
  
  # TODO: The StdDev's are not the same for pweights and fweights (RA-1286)
  #pweights and fweights give same results 
  #comparables <- setdiff(names(z1),c("params","call"))
  #checkTrue(all.equal(z3[comparables],z1[comparables]))
}

"test.rxSummary.args.reportProgress" <- function()
{	
  # reportProgress argument stress tests
  form <- ~ division : state
  set.seed(10)
  N <- 5
  DF <- data.frame(
    division=sample(letters, N, replace=TRUE),
    state=sample(state.name, N, replace=TRUE))
  
  #reportProgress: logical, character string or integer: 
  # 0: no progress is reported. 
  # 1: the number of processed rows is printed and updated. 
  # 2: rows processed and timings are reported. 
  # 3: rows processed and all timings are reported. \
  
  variations <- list(0,1,2,3)	
  results <- printOutput <- vector("list", length=length(variations))
  
  for (i in seq_along(variations))
  {
    printOutput[[i]] <- capture.output(results[[i]] <- rxSummary(form=form, data=DF, reportProgress=variations[[i]]))
    results[[i]]$params <- NULL
  }
  
  # check that all values are the same
  checkTrue(all(unlist(lapply(results[-1], 
    function(x,y){all.equal(x, y, check.attributes=FALSE)}, y=results[[1]]))))
  
  names(printOutput) <- as.character(variations)
  
  noResponseExpected <- c("0")
  equalLengthExpected <- c("2")
  equalLengthExpected1 <- c("1")	
  
  checkTrue(length(unlist(printOutput[noResponseExpected])) == 0)
  
  # just rows
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected1],length)) == 1))
  
  # all these outputs should be the same
  checkTrue(all(unlist(lapply(printOutput[equalLengthExpected],length)) == 2))
  
  # details gives us the goods
  checkTrue(length(grep("ReadNum=1, CurrentBlockNum=1, CurrentNumRows=5, TotalRowsProcessed=5, ReadTime", 
    printOutput[["3"]][1])) > 0)
  
  # time
  checkTrue(length(grep("^Computation time:", printOutput[["2"]][2])) > 0)
}

# FIXME: COMMENTED OUT FOR cppInterp ON LINUX 12/7/2010
#"test.rxSummary.args.rowSelection.cppInterp" <- function()
#{	
  #DF <- as.data.frame(UCBAdmissions)
  #DF1 <- subset(DF, subset = Gender=='Male' & Freq > 200)
  #zA <- rxSummary(~ Gender + Admit, cppInterp=list(rowSelection="Gender=='Male' && Freq > 200"), data=DF)
  #zB <- rxSummary(~ Gender + Admit, data= DF1)
  #nocomp <- which(names(zA) %in% c("call","params"))
  #checkEquals(zA[-nocomp], zB[-nocomp])
  #
  #testXdf <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")  
  #summaryR1.test <- rxSummary(~y + xint1, data=testXdf, rowSelection = y > 0)
  #summaryCpp.test <- rxSummary("y+xint1", data=testXdf, cppInterp=list(rowSelection="y>0"))
  #checkTrue(all.equal(summaryR1.test$sDataFrame[,2:6], summaryCpp.test$sDataFrame[,2:6]))
#}

"test.rxSummary.args.transformFunc" <- function()
{	
  set.seed(10)
  DF <- data.frame(score=(1:5)*0.1,
    division=c("a","a","z","t","l"), 
    state=sample(datasets::state.name, 5, replace=TRUE))
  xform <- function(dataList)
  {
    set.seed(10)
    dataList$STATE <- factor(sample(datasets::state.name, 5, replace=TRUE))
    dataList
  }
  
  z1 <- rxSummary(~ division : STATE, data=DF, transformFunc=xform)
  z2 <- rxSummary(~ division : state, data=DF)
  z1$sDataFrame$Name <- tolower(z1$sDataFrame$Name)
  nocomp <- which(names(z1) %in% c("call","params","formula"))
  checkEquals(z1[-nocomp], z2[-nocomp], check.attributes=FALSE)
}

"test.rxSummary.args.transformFunc.negative" <- function(){
  xform <- function(dataList)
  {
    ret = data.frame(dataList)
    ret = ret[ret$age < 40,,drop=FALSE]
    ret    
  }
  
  infert_mod = tryCatch({
    rxSummary(formula="parity+age", data=infert, transformFunc=xform, transformVars="age")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxSummary"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxSummary"))
    }
  })
}

"test.rxSummary.args.summaryStats" <- function()
{
  censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
  
  sumOut <- rxSummary(~age + age:sex, data = censusWorkers, blocksPerRead = 3)
  defaultStats <- c("Mean", "StdDev", "Min", "Max", "ValidObs", "MissingObs")
  defaultCatStats <- c("Means", "StdDev", "Min", "Max", "ValidObs", "MissingObs")
    
  checkEquals(names(sumOut$sDataFrame)[2:7], defaultStats)
  checkEquals(names(sumOut$categorical[[1]])[3:7], defaultCatStats[1:5])
  allStats <- c("Mean", "StdDev", "Min", "Max", "Sum", "ValidObs", "MissingObs")
  catStats <- c("Means", "StdDev", "Min", "Max", "Sum", "ValidObs", "MissingObs")
  sumOutAll <- rxSummary(~age + age:sex, data = censusWorkers, blocksPerRead = 3,
    summaryStats = allStats)
  checkEquals(names(sumOutAll$sDataFrame)[2:8], allStats)
  checkEquals(names(sumOutAll$categorical[[1]])[3:8], catStats[1:6])
  
  # Just sums
  sumOutSums <- rxSummary(~age + age:sex, data = censusWorkers, blocksPerRead = 3,
    summaryStats = c("Sum"))
  
    oldWarn <- options(warn=2)
    on.exit(options(oldWarn), add = TRUE)	
    # Make sure print method works
  res <- try(
    print(sumOutSums)
    )
    checkTrue(!is(res, "try-error"))
  
}

"test.rxSummary.args.byGroupOutFile" <- function()
{
  censusWorkers <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
  sumOutFile <-  tempfile(pattern = ".rxTempSumOut", fileext = ".xdf")
  
  if (file.exists(sumOutFile)) file.remove(sumOutFile)
  
  sumOut <- rxSummary(~incwage:F(age) + F(age) + wkswork1:F(age), data = censusWorkers, blocksPerRead = 3,
    byGroupOutFile = sumOutFile, rowsPerBlock = 10, summaryStats = c("Mean", "Min", "ValidObs"))
  
  fileInfo <- rxGetInfo(sumOutFile)
  checkEquals(fileInfo$numBlocks, 5 )
  checkEquals(fileInfo$numVars, 8 )
    
  sumOutXdf <- rxReadXdf(sumOutFile)
  
  sumOutDF <- rxSummary(~incwage:F(age) + F(age) + wkswork1:F(age), data = censusWorkers, blocksPerRead = 3,
     summaryStats = c("Mean", "Min", "ValidObs"))
  namesDF1 <- names(sumOutDF$categorical[[1]])
  namesDF1[namesDF1 %in% "Means"] <- "Mean"
  numN1 <- length(namesDF1)
  namesDF1[3:numN1] <- paste("incwage_", namesDF1[3:numN1], sep = "")
  namesDF2 <- names(sumOutDF$categorical[[2]])
  numN2 <- length(namesDF2)
  namesDF3 <- names(sumOutDF$categorical[[3]])
  namesDF3[namesDF3 %in% "Means"] <- "Mean"
  numN3 <- length(namesDF3)
  namesDF3[3:numN1] <- paste("wkswork1_", namesDF3[3:numN1], sep = "")
  allNames <- c(namesDF1[2:numN1], namesDF2[2:numN2], namesDF3[3:numN3])
  
  allDF <- cbind(sumOutDF$categorical[[1]][,2:numN1], sumOutDF$categorical[[2]][,2:numN2], sumOutDF$categorical[[3]][,3:numN3])
  names(allDF) <- allNames
  
  checkEquals(allDF, sumOutXdf)
  
  file.remove(sumOutFile)
  
  # Test for RxXdfData objects
  sumOutFile1 <-  tempfile(pattern = ".rxTempSumOut1", fileext = ".xdf")
  sumOutFile2 <-  tempfile(pattern = ".rxTempSumOut2", fileext = ".xdf")
  
  if (file.exists(sumOutFile1)) file.remove(sumOutFile1)
  if (file.exists(sumOutFile2)) file.remove(sumOutFile2)
  
  myXdfData = RxXdfData(sumOutFile1)
  sumOut <- rxSummary(~incwage:F(age) + F(age), data = censusWorkers,
    byGroupOutFile = myXdfData)
  varInfo <- rxGetVarInfo(myXdfData)
  checkEquals(names(varInfo), c("F_age", "incwage_Mean", "incwage_StdDev",
    "incwage_Min", "incwage_Max", "incwage_ValidObs", "Counts"))
  
  file.remove(sumOutFile1)
  
  sumOutFileObjs <- list(RxXdfData(sumOutFile1), RxXdfData(sumOutFile2))
  
  sumOut2 <- rxSummary(~incwage:F(age) + wkswork1:state, data = censusWorkers, blocksPerRead = 3,
    byGroupOutFile = sumOutFileObjs, rowsPerBlock = 10, summaryStats = c("StdDev"))
  
  varInfo1 <- rxGetVarInfo(sumOutFileObjs[[1]]@file)
  varInfo2 <- rxGetVarInfo(sumOutFileObjs[[2]]@file)
  checkEquals(names(varInfo1), c("F_age", "incwage_StdDev"))
  checkEquals(names(varInfo2), c("state", "wkswork1_StdDev"))

  file.remove(sumOutFile1)
  file.remove(sumOutFile2)
  
  # Test for creation of file names
  sumOutFile1 <-  file.path(tempdir(), ".rxTempSumMult.xdf")
  x <- rnorm(100)
  xfac1 <- factor(rep(1:5, times = 20), levels = 0:6)
  xfac2 <- factor(rep(1:5, each = 20), levels = 0:6)
  myData <- data.frame(x, xfac1, xfac2)
  sumOut <- rxSummary(~ x + xfac1 + xfac2 + x:xfac1:xfac2, data = myData, removeZeroCounts = TRUE,
     byGroupOutFile = sumOutFile1)
    checkTrue(grepl(".rxTempSumMult", sumOut$categorical[[1]]@file))
  file.remove(sumOut$categorical[[1]]@file)
    checkTrue(grepl(".rxTempSumMult_2", sumOut$categorical[[2]]@file))
  file.remove(sumOut$categorical[[2]]@file)
    checkTrue(grepl(".rxTempSumMult_3", sumOut$categorical[[3]]@file))
  file.remove(sumOut$categorical[[3]]@file)			
}	
    
"test.rxSummary.args.transformVars" <- function()
{	
  # reportProgress argument stress tests
  set.seed(10)
  DF <- data.frame(score=(1:5)*0.1,
    division=c("a","a","z","t","l"), 
    state=factor(sample(state.name, 5, replace=TRUE)))
  xform <- function(dataList)
  {
    dataList$stateCopy <- dataList$state
    dataList
  }
  
  z1 <- rxSummary(~ division : stateCopy, data=DF, transformFunc=xform, transformVars="state")
  z2 <- rxSummary(~ division : state, data=DF)
  z1$sDataFrame$Name <- z2$sDataFrame$Name <- NULL
  
  nocomp <- which(names(z1) %in% c("call","params","formula"))
  checkEquals(z1[-nocomp], z2[-nocomp], check.attributes=FALSE)
  
  rDataFrame <- data.frame(a=c("one","two","three"), b=c(1,2,3))
  xForm <- function(x)
  {
    x$c <- x$b * 10
    return(x)
  }
  rxSummaryResult <- rxSummary(~c, data=rDataFrame, transformFunc=xForm, transformVars=c("b"))
  compResults <- dget(file.path(rxGetOption("unitTestDataDir"),"test.rxSummary.transformation.1.comp"))
  checkEquals(compResults, rxSummaryResult$sDataFrame[,1:6])
}

"test.rxSummary.args.verbose" <- function()
{
  # Fix for bug RA-1548: this test was causing subsequent RUnit test failures involving capture.output()
  form <- ~ division : state
  set.seed(10)
  DF <- data.frame(division=sample(letters, 5, replace=TRUE), state=sample(state.name, 5, replace=TRUE))
  
  zSummaryVerboseFalse <- capture.output(z <- rxSummary(form, data=DF, verbose=0))
  zSummaryVerboseTrue <- capture.output(z <- rxSummary(form, data=DF, verbose=1))
  checkTrue(length(zSummaryVerboseTrue) > length(zSummaryVerboseFalse))
}

"test.rxSummary.data.factor.factor" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b")
  df <- rxDataStep(test100r5b)
  #add another factor to this data set
  set.seed(33)
  animal <- sample(c("dog","cat","bird","extra"),100,replace=TRUE)
  nalist <-(sample(c(1:100),20,FALSE))
  animal[nalist] <- NA
  animal <- as.factor(animal)
  df <- data.frame(df,animal)
  
    factorXdf <- tempfile(pattern = ".rxTempFactor", fileext = ".xdf")
  # create xdf with 3 unequal blocks
  if(file.exists(factorXdf)) file.remove(factorXdf)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = df, outFile = factorXdf)
    } else
    {
      rxDataStep(df[1:33,],factorXdf)
      rxDataStep(df[34:56,],factorXdf,append= "rows")
      rxDataStep(df[57:100,],factorXdf,append= "rows")
    }
    
  #animal
  sum.animal <- rxSummary(~animal, data = factorXdf)
  num.animal <- length(df$animal[!is.na(df$animal)])

  # Mean should not be reported for categorical data
  checkTrue(is.na(sum.animal$sDataFrame$Mean))
  checkEquals(sum.animal$sDataFrame$ValidObs, num.animal)
  
  num.animal.bird <- length(df$animal[df$animal == "bird" & is.na(df$animal)==FALSE])
  num.animal.cat <- length(df$animal[df$animal == "cat" & is.na(df$animal)==FALSE])
  num.animal.dog <- length(df$animal[df$animal == "dog" & is.na(df$animal)==FALSE])
  num.animal.extra <- length(df$animal[df$animal == "extra" & is.na(df$animal)==FALSE])
  
  ### check counts of each factor level
  checkEquals(as.vector(unlist(sum.animal$categorical[[1]]["Counts"])),c(num.animal.bird,num.animal.cat,num.animal.dog,num.animal.extra),
    check.attributes=FALSE, check.names=FALSE)
  
  #animal + zfac
  mean.zfac <- mean(na.omit(as.numeric(df$zfac)-1))
  num.zfac <- length(df$zfac[!is.na(df$zfac)])
  sum.animalzfac <- rxSummary(~animal + zfac, data=factorXdf)
  
  checkTrue(is.na(sum.animalzfac$sDataFrame$Mean[1]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs[1],num.animal)
  checkTrue(is.na(sum.animalzfac$sDataFrame$Mean[2]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs[2],num.zfac)
  
  num.animal.bird <- length(df$animal[df$animal == "bird" & is.na(df$animal)==FALSE])
  num.animal.cat <- length(df$animal[df$animal == "cat" & is.na(df$animal)==FALSE])
  num.animal.dog <- length(df$animal[df$animal == "dog" & is.na(df$animal)==FALSE])
  num.animal.extra <- length(df$animal[df$animal == "extra" & is.na(df$animal)==FALSE])
  num.zfac.Washington <- length(df$zfac[df$zfac == "Washington" & is.na(df$zfac)==FALSE])
  num.zfac.Alaska <- length(df$zfac[df$zfac == "Alaska" & is.na(df$zfac)==FALSE])
  num.zfac.NorthCarolina <- length(df$zfac[df$zfac == "North Carolina" & is.na(df$zfac)==FALSE])

  ### check counts of each factor level
  checkEquals(as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"])),c(num.animal.bird,num.animal.cat,num.animal.dog,num.animal.extra),
    check.attributes=FALSE, check.names=FALSE)
  checkEquals(as.vector(unlist(sum.animalzfac$categorical[[2]]["Counts"])),c(num.zfac.Washington,num.zfac.Alaska,num.zfac.NorthCarolina),
    check.attributes=FALSE, check.names=FALSE)	
  
  # with byTerm=FALSE
  sum.animalzfac <- rxSummary(~animal+zfac, data=factorXdf, byTerm=FALSE)
  nobs <- nrow(na.omit(df[,c("animal","zfac")]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs,c(nobs,nobs))
  sum.counts <- sum(as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"])))
  
  #animal:zfac
  sum.animalzfac <- rxSummary(~animal:zfac, data=factorXdf)
  
  # RA-3265 - rxSummary with Interacting Factors with Missings
  nobs <- nrow(na.omit(df[,c("animal","zfac")]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs, nobs)
  
  ### check a few counts 
  nobs.birdWash <- nrow(na.omit(df[df$animal=="bird" & df$zfac=="Washington",c("animal","zfac")]))
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[1], nobs.birdWash)
  
  nobs.catWash <- nrow(na.omit(df[df$animal=="cat" & df$zfac=="Washington",c("animal","zfac")]))
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[2] ,nobs.catWash)
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[3],
    nrow(na.omit(df[df$animal=="dog" & df$zfac=="Washington",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[4],
    nrow(na.omit(df[df$animal=="extra" & df$zfac=="Washington",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[5],
    nrow(na.omit(df[df$animal=="bird" & df$zfac=="Alaska",c("animal","zfac")])))
    
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[6],
    nrow(na.omit(df[df$animal=="cat" & df$zfac=="Alaska",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[7],
    nrow(na.omit(df[df$animal=="dog" & df$zfac=="Alaska",c("animal","zfac")])))
    
  nobs.catNC <- nrow(na.omit(df[df$animal=="cat" & df$zfac=="North Carolina",c("animal","zfac")]))
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[10]   ,nobs.catNC)
  
  file.remove(factorXdf)
}	

"test.rxSummary.data.factor.factor.emptyLevel" <- function()
{
  ### check case of levels with no observations
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b")
  df <- rxDataStep(inData = test100r5b)
    
  #add another factor to this data set
  set.seed(33)
  animal <- sample(c("dog","cat","bird"),100,replace=TRUE)
  nalist <-(sample(c(1:100),20,FALSE))
  animal[nalist] <- NA
  animal <- factor(animal, levels=c("dog","cat","bird","extra"))
  df <- data.frame(df,animal)
    
  factorXdf <- tempfile(pattern = ".rxTempFactor", fileext = ".xdf")
  # create xdf with 3 unequal blocks
  if(file.exists(factorXdf))	file.remove(factorXdf)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = df, outFile = factorXdf)
    } else
    {
      rxDataStep(df[1:33,],factorXdf)
      rxDataStep(df[34:56,],factorXdf, append= "rows")
      rxDataStep(df[57:100,],factorXdf, append= "rows")
    }
    
  
  #animal
  sum.animal <- rxSummary(~animal, data=factorXdf)
  num.animal <- length(df$animal[!is.na(df$animal)])

  checkTrue(is.na(sum.animal$sDataFrame$Mean))
  checkEquals(sum.animal$sDataFrame$ValidObs,num.animal)
  
  num.animal.bird <- length(df$animal[df$animal == "bird" & is.na(df$animal)==FALSE])
  num.animal.cat <- length(df$animal[df$animal == "cat" & is.na(df$animal)==FALSE])
  num.animal.dog <- length(df$animal[df$animal == "dog" & is.na(df$animal)==FALSE])
  num.animal.extra <- length(df$animal[df$animal == "extra" & is.na(df$animal)==FALSE])

  ### check counts of each factor level
  checkEquals(as.vector(unlist(sum.animal$categorical[[1]]["Counts"])),c(num.animal.dog,num.animal.cat,num.animal.bird,num.animal.extra),
    check.attributes=FALSE, check.names=FALSE)

  #animal + zfac
  num.zfac <- length(df$zfac[!is.na(df$zfac)])
  sum.animalzfac <- rxSummary(~animal + zfac, data=factorXdf)
  
  checkTrue(is.na(sum.animalzfac$sDataFrame$Mean[1]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs[1],num.animal)
  checkTrue(is.na(sum.animalzfac$sDataFrame$Mean[2]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs[2],num.zfac)
  
  num.animal.bird <- length(df$animal[df$animal == "bird" & is.na(df$animal)==FALSE])
  num.animal.cat <- length(df$animal[df$animal == "cat" & is.na(df$animal)==FALSE])
  num.animal.dog <- length(df$animal[df$animal == "dog" & is.na(df$animal)==FALSE])
  num.animal.extra <- length(df$animal[df$animal == "extra" & is.na(df$animal)==FALSE])
  num.zfac.Washington <- length(df$zfac[df$zfac == "Washington" & is.na(df$zfac)==FALSE])
  num.zfac.Alaska <- length(df$zfac[df$zfac == "Alaska" & is.na(df$zfac)==FALSE])
  num.zfac.NorthCarolina <- length(df$zfac[df$zfac == "North Carolina" & is.na(df$zfac)==FALSE])

  ### check counts of each factor level
  checkEquals(as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"])),c(num.animal.dog,num.animal.cat,num.animal.bird,num.animal.extra),
    check.attributes=FALSE, check.names=FALSE)
  checkEquals(as.vector(unlist(sum.animalzfac$categorical[[2]]["Counts"])),c(num.zfac.Washington,num.zfac.Alaska,num.zfac.NorthCarolina),
    check.attributes=FALSE, check.names=FALSE)	
  
  # with byTerm=FALSE
  sum.animalzfac <- rxSummary(~animal + zfac, data=factorXdf, byTerm=FALSE)
  nobs <- nrow(na.omit(df[,c("animal","zfac")]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs,c(nobs,nobs))
  
  #animal:zfac
  sum.animalzfac <- rxSummary(~animal:zfac, data=factorXdf)
  
  #RA-3265 - rxSummary with Interacting Factors with Missings
  
  nobs <- nrow(na.omit(df[,c("animal","zfac")]))
  checkEquals(sum.animalzfac$sDataFrame$ValidObs, nobs)
  
  ### check a few counts 
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[2],
    nrow(na.omit(df[df$animal=="cat" & df$zfac=="Washington",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[1],
    nrow(na.omit(df[df$animal=="dog" & df$zfac=="Washington",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[4],
    nrow(na.omit(df[df$animal=="extra" & df$zfac=="Washington",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[7],
    nrow(na.omit(df[df$animal=="bird" & df$zfac=="Alaska",c("animal","zfac")])))
    
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[6],
    nrow(na.omit(df[df$animal=="cat" & df$zfac=="Alaska",c("animal","zfac")])))
  
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[5],
    nrow(na.omit(df[df$animal=="dog" & df$zfac=="Alaska",c("animal","zfac")])))
    
  nobs.catNC <- nrow(na.omit(df[df$animal=="cat" & df$zfac=="North Carolina",c("animal","zfac")]))
  checkEquals( as.vector(unlist(sum.animalzfac$categorical[[1]]["Counts"]))[10]   ,nobs.catNC)
  
  file.remove(factorXdf)	
}	
  
"test.rxSummary.data.factors.F" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b")
  df <- rxDataStep(test100r5b)
  #### add missing to xint1
  set.seed(33)
  df$xint1[sample(1:100,20)] <- NA
  
    factorXdf <- tempfile(pattern = ".rxTempFactor", fileext = ".xdf")
  ### create xdf with 3 unequal blocks
  if(file.exists(factorXdf)) 	file.remove(factorXdf)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = df, outFile = factorXdf, overwrite = TRUE)
    } else
    {
      rxDataStep(df[1:33,],factorXdf, overwrite = TRUE)
      rxDataStep(df[34:56,],factorXdf,append="rows")
      rxDataStep(df[57:100,],factorXdf,append="rows")
    }
    
  
  sum.xintxnum2miss <- rxSummary(~F(xint1, low=5,high=12,exclude=TRUE) + xnum2miss , data=factorXdf)
  sum.xint <-  rxSummary(~F(xint1, low=5,high=12,exclude=TRUE), data=factorXdf)
  sum.xnum2miss <- rxSummary(~ xnum2miss , data=factorXdf)
  
  checkEquals(sum.xintxnum2miss$sDataFrame$ValidObs[1], sum.xint$sDataFrame$ValidObs)
  checkEquals(sum.xintxnum2miss$sDataFrame$ValidObs[2], sum.xnum2miss$sDataFrame$ValidObs)
  checkEquals(sum.xintxnum2miss$sDataFrame[1,], sum.xint$sDataFrame,check.attributes=FALSE)
  checkEquals(sum.xintxnum2miss$sDataFrame[2,], sum.xnum2miss$sDataFrame,check.attributes=FALSE)
  
  df2 <- df[df$xint1 >= 5 & df$xint1 <= 12,c("xint1","xnum2miss")]
  mean.xint1 <- mean(na.omit(df2$xint1))
  mean.xnum2miss <- mean(na.omit(df$xnum2miss))
  num.xint1 <- length(na.omit(df2$xint1))
  num.xnum2miss <- length(na.omit(df$xnum2miss))
  checkTrue(is.na(sum.xintxnum2miss$sDataFrame$Mean[1]))
  checkEquals(sum.xintxnum2miss$sDataFrame$Mean[2],c(mean.xnum2miss))
  checkEquals(sum.xintxnum2miss$sDataFrame$ValidObs,c(num.xint1,num.xnum2miss))
  
  # With exclude=FALSE - we should get same valid obs of xinit1 same as xinit1 without using F
  sum.xintxnum2miss <- rxSummary(~F(xint1, low=5,high=12,exclude=FALSE) + xnum2miss , data=factorXdf)
  num.xint <- length(na.omit(df$xint1))
  checkTrue(is.na(sum.xintxnum2miss$sDataFrame$Mean[1]))
  checkEquals(sum.xintxnum2miss$sDataFrame$ValidObs[1],num.xint)
  checkEquals(sum.xintxnum2miss$sDataFrame$Mean[2], mean.xnum2miss)
  
  file.remove(factorXdf)
  
}	

"test.rxSummary.data.factor.continuous" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "richTest100r5b")
  df <- rxDataStep(test100r5b)
  
  #zfac1
  sum.zfac1 <- rxSummary(~zfac, data=test100r5b)
  nobs.wa <- length(df$zfac[df$zfac=="Washington" & is.na(df$zfac) == FALSE])
  nobs.nc <- length(df$zfac[df$zfac=="North Carolina" & is.na(df$zfac) == FALSE])
  nobs.ak <- length(df$zfac[df$zfac=="Alaska" & is.na(df$zfac) == FALSE])
  checkEquals(unlist(sum.zfac1$categorical)[4:6],c(nobs.wa,nobs.ak,nobs.nc),check.names=FALSE,check.attributes=FALSE)
  
  #xnum1
  sum.xnum1 <- rxSummary(~xnum1, data=test100r5b)
  mean.xnum1 <- mean(df$xnum1)
  checkEquals(sum.xnum1$sDataFrame$Mean,mean.xnum1)
  checkEquals(length(df$xnum1),sum.xnum1$nobs.valid)
  
  #xnum1 + zfac 
  sum.zfacxnum1 <- rxSummary(~zfac + xnum1, data=test100r5b)	
  checkEquals(sum.zfacxnum1$sDataFrame$Mean[2],mean.xnum1)
  
  #xnum1:zfac
  sum.zfacxnum1 <- rxSummary(~zfac:xnum1 + xnum1, data=test100r5b)	
  mean.zfacxnum1.wa <- mean(df$xnum1[df$zfac=="Washington" & is.na(df$zfac) == FALSE])
  mean.zfacxnum1.ak <- mean(df$xnum1[df$zfac=="Alaska" & is.na(df$zfac) == FALSE])
  mean.zfacxnum1.nc <- mean(df$xnum1[df$zfac=="North Carolina" & is.na(df$zfac) == FALSE])

  cat <- unlist(sum.zfacxnum1$categorical)
  checkEquals(cat["Means1"],mean.zfacxnum1.wa,check.attributes=FALSE, check.names=FALSE)
  checkEquals(cat["Means2"],mean.zfacxnum1.ak,check.attributes=FALSE, check.names=FALSE)	
  checkEquals(cat["Means3"],mean.zfacxnum1.nc,check.attributes=FALSE, check.names=FALSE)	
  
  checkEquals(length(df$xnum1[df$zfac=="Washington" & is.na(df$zfac) == FALSE]), cat["ValidObs1"],check.attributes=FALSE, check.names=FALSE)
  checkEquals(length(df$xnum1[df$zfac=="Alaska" & is.na(df$zfac) == FALSE]), cat["ValidObs2"],check.attributes=FALSE, check.names=FALSE)
  checkEquals(length(df$xnum1[df$zfac=="North Carolina" & is.na(df$zfac) == FALSE]), cat["ValidObs3"],check.attributes=FALSE, check.names=FALSE)

}

"test.rxSummary.data.interact.cont.factor" <- function()
{
  test100r5b <- file.path(rxGetOption("unitTestDataDir"), "test100r5b")
  
  sumtest <- rxSummary(~y:ylogical+ xnum2miss:ylogical, data=test100r5b)
  sumtestDF <- sumtest$sDataFrame

  checkEquals(sumtest$nobs.valid,100)
  checkTrue(is.na(sumtest$nobs.missing))
  checkEquals(sumtestDF$ValidObs,c(100,77))
  df <- rxDataStep(test100r5b)
  
  #y:ylogical
  nobsTrue <- length(df$y[df$ylogical == TRUE])
  nobsFalse <- length(df$y[df$ylogical == FALSE])
  sumTrue <- mean(df$y[df$ylogical == TRUE]) 
  sumFalse <- mean(df$y[df$ylogical == FALSE])
  sum1 <- unlist(rxSummary(~ylogical:y, data=test100r5b)$categorical)
  checkEquals(sum1[13], nobsTrue, check.attributes=FALSE)
  checkEquals(sum1[14], nobsFalse, check.attributes=FALSE)

  checkEquals(sumTrue, sum1["Means1"], checkAttributes=FALSE, checkNames=FALSE)
  checkEquals(sumFalse, sum1["Means2"], checkAttributes=FALSE, checkNames=FALSE)
  
  #xnum2miss:ylogical
  nobsTrue <- length(is.na(df$xnum2miss[df$ylogical == TRUE & !is.na(df$xnum2miss) ]))
  nobsFalse <- length(df$xnum2miss[df$ylogical == FALSE  & !is.na(df$xnum2miss) ])
  sumTrue <- mean(df$xnum2miss[df$ylogical == TRUE & !is.na(df$xnum2miss) ]) 
  sumFalse <- mean(df$xnum2miss[df$ylogical == FALSE & !is.na(df$xnum2miss) ]) 
  sum1 <- unlist(rxSummary(~xnum2miss:ylogical, data=test100r5b)$categorical)
  checkEquals(sum1[13], nobsTrue, check.attributes = FALSE)
  checkEquals(sum1[14], nobsFalse, check.attributes = FALSE)

  checkEquals(sumTrue, sum1["Means1"], checkAttributes = FALSE, checkNames = FALSE)
  checkEquals(sumFalse, sum1["Means2"], checkAttributes = FALSE, checkNames = FALSE)

  df2 <- rxGetInfo(test100r5b,numRows=100)
  checkEquals(df,df2$data)
}

"test.rxSummary.args.removeZeroCounts" <- function()
{
  a <- as.character(1:10) 
  a[c(1,3,5)] <- NA 
  b <- letters[1:10] 
  b[c(2,4,5)] <- NA 
  df <- data.frame(a = factor(a,levels=a), b = factor(b,levels=b))
  
  sum.ab.withZeros <- rxSummary(~a:b, data = df) 
  sum.ab.withoutZeros <- rxSummary(~a:b, data = df, removeZeroCounts = TRUE) 
  
  countsWithZeros <- as.data.frame(sum.ab.withZeros$categorical)	
  countsWithoutZeros <- as.data.frame(sum.ab.withoutZeros$categorical)
  abInteract <- df$a:df$b
  # Missings are put in last category summary
  abInteractSum <- summary(abInteract)[1:49]
  abInteractSumNoZeros <- abInteractSum[abInteractSum > 0]
  
  checkEquals(sum(countsWithZeros$Counts), sum(countsWithoutZeros$Counts))
  checkEquals(countsWithZeros$Counts, abInteractSum, checkNames=FALSE)
  checkEquals(nrow(countsWithZeros), length(levels(abInteract)))	
  checkEquals(countsWithoutZeros$Counts, abInteractSumNoZeros, checkNames=FALSE)
  
  # Summary stats by category: single term
  x <- 1:10
  y <- factor(rep(1:2, times = 5), levels = 1:4)
  testData <- data.frame(x, y)
  sumOut <- rxSummary(~x:y, data = testData)
  checkEquals(nrow(sumOut$categorical[[1]]), 4)
  sumOutZeroCounts <- rxSummary(~x:y, data = testData, removeZeroCounts = TRUE)
  checkEquals(nrow(sumOutZeroCounts$categorical[[1]]), 2)
  
  # Summary stats by category: multiple terms
  y1 <- rep(c(1,4), times = 5)
  testData <- data.frame(x, y, y1)
  sumOut <- rxSummary(~x:y + x:F(y), data = testData)
  checkEquals(nrow(sumOut$categorical[[1]]), 4)
  checkEquals(nrow(sumOut$categorical[[2]]), 4)
  sumOutZeroCounts <- rxSummary(~x:y + x:F(y), data = testData, removeZeroCounts = TRUE)
  checkEquals(nrow(sumOutZeroCounts$categorical[[1]]), 2)	
  checkEquals(nrow(sumOutZeroCounts$categorical[[2]]), 2)	
  
}

"test.rxSummary.args.formula.F" <- function()
{
    # Unit test for #RA-4635
    origNumCoresToUse <- rxGetOption("numCoresToUse")
    on.exit(rxOptions(numCoresToUse = origNumCoresToUse), add = TRUE)
    
    for( lowVal in -1:1)
    {
        for (numCoresToUse in 1:4)
        {
            rxOptions(numCoresToUse = numCoresToUse)
            myData <- data.frame(y = lowVal: (lowVal + 2))
            if (numCoresToUse > 1)
            {
                lastResults <- theseResults
            }
            
            theseResults <- rxResultsDF(rxSummary(~F(y), myData), "counts")
            if (numCoresToUse > 1)
            {
                checkEquals(lastResults, theseResults)
            }
        }
    }
    rxOptions(numCoresToUse = origNumCoresToUse)
}


"test.rxSummary.method.rxResultsDF" <- function()
{

  x0 <- c( 1, 3, 2, 4, 3, 1, 3, 4, 2, 2)
  xint0 <- as.integer(x0)
  xfac0 <- as.factor(xint0)

  x1 <- c( 1, 3, 2, 4, 3, NA, 3, 4, 2, 2)
  xint1 <- as.integer(x1)
  xfac1 <- as.factor(xint1)
  
  x2 <- c( 1, 3, NA, 4, 3, NA, 3, 4, 2, 2)
  xint2 <- as.integer(x2)
  xfac2 <- as.factor(xint2)
  
  testData <- data.frame(x0, xint0, xfac0, x1, xint1, xfac1, x2, xint2, xfac2)
  
  sumObj1 <- rxSummary(~., data=testData)
  
  statsResults <- rxResultsDF( sumObj1 )
  checkEquals(statsResults[4,"ValidObs"], 9)
  countsObj1 <- rxResultsDF( sumObj1, "counts", 1 )
  checkEquals(attr(countsObj1, "validObs"), 10)
  checkEquals(attr(countsObj1, "missingObs"), 0)
  countsObj2 <- rxResultsDF( sumObj1, "counts", "xfac2" )
  checkEquals(attr(countsObj2, "validObs"), 8)
  checkEquals(attr(countsObj2, "missingObs"), 2)
  
}

"test.rxSummary.model.depVar" <- function()
{
  
  x0 <- c( 1, 3, 2, 4, 3, 1, 3, 4, 2, 2)

  x1 <- c( 1, 3, 2, 4, 3, NA, 3, 4, 2, 2)
  xint1 <- as.integer(x1)
  xfac1 <- as.factor(xint1)
  
  x2 <- c( 1, 3, NA, 4, 3, NA, 3, 4, 2, 2)
  xint2 <- as.integer(x2)
  xfac2 <- as.factor(xint2)
  
  testData <- data.frame(x0, xfac1, xfac2)
  sumOut <- rxSummary( x0~xfac1+xfac2, data=testData)
  checkEquals(length(sumOut$categorical), 2)
}

"test.rxSummary.dataSource.RxTextData" <- function()
{
    textFile <- file.path(rxGetOption("sampleDataDir"),"claims.txt")
    textDS <- RxTextData(textFile, stringsAsFactors = TRUE, rowsPerRead =10)
    
    sumObj <- rxSummary(~., data = textDS)
    ageLevels <- factor(1:8, labels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
    ageCounts <- sumObj$categorical[[1]]
    checkEquals(ageCounts$age, ageLevels)
    typeLevels <- factor(1:4, labels = c("A", "B", "C", "D"))
    typeCounts <- sumObj$categorical[[3]]
    checkEquals(typeCounts$type, typeLevels)

}


#####  SPSS datasource 
"test.rxSummary.dataSource.SPSS" <- function()
{
  spssFile <- file.path(rxGetOption("sampleDataDir"),"claims.sav")
  spssDS <- RxSpssData(spssFile,
            colClasses = c("RowNum" = "integer", "age" = "factor",
                     "car_age" = "factor", "type" = "factor",
                     "cost" = "integer", "number" = "integer"),
             colInfo = list(  age = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34","35-39","40-49","50-59","60+")),
                      car.age = list(type = "factor", levels = c("0-3", "4-7", "8-9","10+")),
                      type = list(type = "factor", levels = c("A", "B", "C", "D")))
            )		
  result <- rxSummary(cost ~number, data=spssDS)	
  
    testFile <- tempfile(pattern = ".rxTempSum1", fileext = ".xdf")     
  rxImport(spssDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxSummary(cost ~number, testFile)	
  
    checkEquals(result$coefficients, result2$coefficients)
    file.remove(testFile)
}

#### ODBC datasource
"test.rxSummary.dataSource.ODBC.ST" <- function()
{	
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
      rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), 
                      "claims.sqlite"), useFastRead = FALSE),
      silent = TRUE)
        
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
  odbcDS <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                useFastRead = FALSE,
                connectionString = conString)
            # TODO: look a specifying colClasses
                #colClasses = c("RowNum" = "integer", "age" = "character",
                 #"car_age" = "character", "type" = "character",
                 #"cost" = "integer", "number" = "integer"))
  result = rxSummary(cost~number, data=odbcDS)
    
    testFile <- tempfile(pattern = ".rxTempSum2", fileext = ".xdf")         
  rxImport(odbcDS, testFile, overwrite = TRUE, reportProgress = 0)
  result2 <- rxSummary(cost~number, testFile)	
  
    checkEquals(rxResultsDF(result), rxResultsDF(result2))
    file.remove(testFile)
}

"test.rxSummary.dataSource.ODBC.colClasses" <- function()
{	
    # RA-4729 - rxSummary with interacted integers
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
      rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), 
                      "claims.sqlite")),
      silent = TRUE)
        
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
        
    
  odbcDS1 <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString,
                colClasses = c(cost = "integer",  number = "integer"))
            
  odbcDS2 <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                connectionString = conString)        
                        
  result1a = rxSummary(cost~number, data=odbcDS1)
    
  myData1 <- rxImport(odbcDS1, reportProgress = 0)
  result1b <- rxSummary(cost~number, myData1)	
  
    checkEquals(rxResultsDF(result1a), rxResultsDF(result1b))
    
    result2a = rxSummary(cost~number, data=odbcDS2)
    
  myData2 <- rxImport(odbcDS2, reportProgress = 0)
  result2b <- rxSummary(cost~number, myData2)
  
    checkEquals(rxResultsDF(result2a), rxResultsDF(result2b))    
}

"test.rxSummary.data.integer.interaction" <- function()
{	
    # RA-4729 - rxSummary with interacted integers
   
    x <- 1:10
    x[3] <- NA 
    myData1 <- data.frame(x = as.integer(x), y = as.integer(rep(1:5,2)))
    result1 <- rxSummary(~x:y, myData1) 
    result1
    
    myData2 <- data.frame(x = as.numeric(x), y = as.numeric(rep(1:5,2)))
    
    myData3 <- data.frame(z = myData1$x * myData1$y)

    result1 <- rxSummary(~x:y, myData1)
    result2 <- rxSummary(~x:y, myData2)
    result3 <- rxSummary(~z, myData3)
  
    checkEquals(rxResultsDF(result1), rxResultsDF(result2))    
}

"test.rxSummary.formula.multFactorInteractions" <- function()
{
    testDataFrame <- data.frame(
        xfac1 = factor(rep(c("a", "b", "c"), times = 4)),
        xfac2 = factor(rep(c("x", "y", "z", "w"), each = 3)),
        int1 = as.integer(rep(1:2, times = 6))
    )

    sumOut <- rxSummary(~xfac1:F(int1) + xfac2:F(int1), data = testDataFrame)
  names1 <- names(sumOut$categorical[[1]])
  names2 <- names(sumOut$categorical[[2]])
  checkEquals(names1[2], names2[2])		
  
}
"test.rxSummary.args.data.colInfo.newNames" <- function()
{
    inData <- file.path(rxGetOption("unitTestDataDir"), "test100r.txt")

    testColInfo <- list(
      y = list(newName = "newY"),
      ylogical = list(newName = "newYlogical"))

    testDS <- RxTextData(file = inData, colInfo = testColInfo)
    sumOut1 <- rxSummary(~newY, data = testDS)
    
    inDF <- rxImport(inData)
    sumOut2 <- rxSummary(~y, data = inDF)
    checkEquals(sumOut1$sDataFrame$Mean, sumOut2$sDataFrame$Mean)
}

"test.rxSummary.data.Date" <- function()
{
    # Check printing of date data (RA-10850)
    dateTimeVar <- as.POSIXct("2014/01/01 00:00:00") + (0:9)*60*60*24 
    justDate <- c("1jan2013", "2jan2013", "31mar2013", "30jul2013", "1jan2014",
                  "5jan2014", "8jan2014", "31aug2014", "15sep2014", "12dec2014")
    dateVar <- as.Date(justDate, "%d%b%Y")
    x <- 1:10
    dateTimeDF  <- data.frame(dateTimeVar = dateTimeVar, dateVar = dateVar, x = x) 
    
    out1 <- capture.output(rxSummary(~., data = dateTimeDF, reportProgress = 0))
    checkTrue(grepl(as.character(min(dateTimeVar)), out1[9]))
    checkTrue(grepl(as.character(min(dateVar)), out1[10])) 
    checkTrue(grepl(as.character(min(x)), out1[11]))
 }     

"test.rxSummary.punctuations" <- function()
{
    # Bug 17450: mxFastLinear fails with text featurizer
    iris1 <- iris
    punct1 <- ">"   #pipe error (now fixed)
    #punct1 <- c("\"", "'", ":", "^", "~")  #formula error
    #punct1 <- c("+", "-")                  #ignored
    #punct1 <- c("*")                       #wrong
    iris1[punct1] <- iris1$Petal.Width
    summ1 <- rxSummary(~., data = iris1)
    checkTrue(punct1 %in% summ1$sDataFrame$Name)
    
    iris2 <- iris
    punct2 <- c(    #ok
        "!", "#", "$", "%", "&", "(", ")", ",", ".", "/", ";", "<", 
        "=", "?", "@", "[", "\\", "]", "_", "`", "{", "|", "}")
    iris2[punct2] <- iris2$Petal.Width
    summ2 <- rxSummary(~., data = iris2)
    checkTrue(all(punct2 %in% summ2$sDataFrame$Name))
}     

"test.rxSummary.longTagName" <- function()
{
    # Bug 21712: Customer Issue: Change in mtCatHash
    
    iris1 <- iris
    tagname <- sapply(mapply(rep_len, x = letters[1:5], length = 470:479), paste, collapse = "")
    iris1[tagname] <- iris1$Petal.Width
    summ1 <- rxSummary(~., data = iris1)

    checkTrue(all(!duplicated(summ1$sDataFrame$Name)))
    
    ## verify the truncated name format to avoid unintended regression
    bLongTagName <- sapply(summ1$sDataFrame$Name, nchar) > 471
    longTagNames <- summ1$sDataFrame$Name[bLongTagName]
    checkTrue(length(grep("[[:alnum:]]+...[[:alnum:]]+", longTagNames)) == length(longTagNames))
}     
