# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for transforms 
#
############################################################################################

".rxLinkTransformComponents" <- function()
{
    if (exists("rxLinkTransformComponents", globalenv()))
        get("rxLinkTransformComponents", globalenv())
    else
        RevoScaleR:::rxLinkTransformComponents
}

## Basic tests
"test.rxLinkTransformComponents.empty" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    empty <- rxLinkTransformComponents()
    checkIdentical(typeof(empty) , "list")
    checkIdentical(names(empty),
                   c("evalEnv", "transformString", "transformVars",
          "transformVarsNoRowSelection", "hasRowSelection"))
    checkIdentical(typeof(empty[["evalEnv"]]), "environment")
    checkIdentical(typeof(empty[["transformString"]]), "NULL")
    checkIdentical(typeof(empty[["transformVars"]]), "character")
    checkIdentical(typeof(empty[["hasRowSelection"]]), "logical")
}

"test.rxLinkTransformComponents.transforms.bad" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    checkException(rxLinkTransformComponents(transforms = "list(log(age))"),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(transforms = list(log(age))),
                   silent = TRUE)
}

"test.rxLinkTransformComponents.transforms.single.new" <- function()
{
    df <- infert
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    for (expr in list(quote(list(logage = log(age))),
                      expression(list(logage = log(age)))))
    {
        single <- rxLinkTransformComponents(transforms = expr)
        checkTrue(exists(".rxTransformsImpliedFunc", single[["evalEnv"]]))
        transformsFunc <- get(".rxTransformsImpliedFunc", single[["evalEnv"]])
        checkIdentical(single[["transformVars"]], "age")
        selected <- df[single[["transformVars"]]]
        checkTrue(is.function(transformsFunc))
        checkIdentical(transformsFunc(selected),
                       c(selected, list(logage = log(df$age))))
        checkIdentical(single[["transformString"]],
                       "rxSafeTry(.rxTransformsImpliedFunc(..revoTransDf), silent = FALSE)")
    }
}

"test.rxLinkTransformComponents.transforms.single.removal" <- function()
{
    df <- infert
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    for (expr in list(quote(list(age = NULL)), expression(list(age = NULL))))
    {
        single <- rxLinkTransformComponents(transforms = expr)
        checkTrue(exists(".rxTransformsImpliedFunc", single[["evalEnv"]]))
        transformsFunc <- get(".rxTransformsImpliedFunc", single[["evalEnv"]])
        checkIdentical(single[["transformVars"]], character(0))
        selected <- df[single[["transformVars"]]]
        checkTrue(is.function(transformsFunc))
        checkIdentical(transformsFunc(selected), df[integer(0)])
        checkIdentical(single[["transformString"]],
                       "rxSafeTry(.rxTransformsImpliedFunc(..revoTransDf), silent = FALSE)")
    }
}

"test.rxLinkTransformComponents.transforms.multiple.chain" <- function()
{
    df <- infert
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    for (expr in list(quote(list(logage = log(age), logagePlus1 = logage + 1)),
                      expression(list(logage = log(age),
                                      logagePlus1 = logage + 1))))
    {
        multiple <- rxLinkTransformComponents(transforms = expr)
        checkTrue(exists(".rxTransformsImpliedFunc", multiple[["evalEnv"]]))
        transformsFunc <- get(".rxTransformsImpliedFunc", multiple[["evalEnv"]])
        checkIdentical(multiple[["transformVars"]], c("age", "logage"))
        selected <- df["age"]
        checkTrue(is.function(transformsFunc))
        checkIdentical(transformsFunc(selected),
                       c(selected,
                         list(logage = log(df$age),
                              logagePlus1 = log(df$age) + 1)))
        checkIdentical(multiple[["transformString"]],
                       "rxSafeTry(.rxTransformsImpliedFunc(..revoTransDf), silent = FALSE)")
    }
}

"test.rxLinkTransformComponents.transforms.multiple.vars" <- function()
{
    df <- infert
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    for (expr in list(quote(list(logage = log(age), case = NULL,
                                 logagePlus1 = logage + 1, age = NULL)),
                      expression(list(logage = log(age), case = NULL,
                                      logagePlus1 = logage + 1, age = NULL))))
    {
        multiple <- rxLinkTransformComponents(transforms = expr)
        checkTrue(exists(".rxTransformsImpliedFunc", multiple[["evalEnv"]]))
        transformsFunc <- get(".rxTransformsImpliedFunc", multiple[["evalEnv"]])
        checkIdentical(multiple[["transformVars"]], c("age", "logage"))
        selected <- df["age"]
        checkTrue(is.function(transformsFunc))
        checkIdentical(transformsFunc(selected),
                       list(logage = log(df$age),
                            logagePlus1 = log(df$age) + 1))
        checkIdentical(multiple[["transformString"]],
                       "rxSafeTry(.rxTransformsImpliedFunc(..revoTransDf), silent = FALSE)")
    }
}

"test.rxLinkTransformComponents.formulaTransforms.bad" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    checkException(rxLinkTransformComponents(formulaTransforms = "list(log(age))"),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(formulaTransforms =
                                             quote(list(log(age)))),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(formulaTransforms =
                                             expression(list(log(age)))),
                   silent = TRUE)
}

"test.rxLinkTransformComponents.formulaTransforms.simple" <- function()
{
    df <- infert
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    for (expr in list(quote(log(age^2)), expression(log(age^2))))
    {
        simple <-
          rxLinkTransformComponents(formulaTransforms = list(log.age.2. = expr))
        checkTrue(exists(".rxFormulaFunc", simple[["evalEnv"]]))
        formulaFunc <- get(".rxFormulaFunc", simple[["evalEnv"]])
        checkIdentical(simple[["transformVars"]], "age")
        selected <- df[simple[["transformVars"]]]
        checkTrue(is.function(formulaFunc))
        checkIdentical(formulaFunc(selected),
                       c(selected, list(log.age.2. = log(df$age^2))))
        checkIdentical(simple[["transformString"]],
                       "rxSafeTry(.rxFormulaFunc(..revoTransDf), silent = FALSE)")
    }
}

"test.rxLinkTransformComponents.transformFunc.bad" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    checkException(rxLinkTransformComponents(transformFunc = "list(log(age))"),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(transformFunc = list(log(age))),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(transformFunc =
                                             list(logage = log(age))),
                   silent = TRUE)
}

"test.rxLinkTransformComponents.transformFunc.good" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    func <- function(dataList) {data$logage <- log(data$age); dataList}
    good <- rxLinkTransformComponents(transformFunc = func)
    checkTrue(exists(".rxTransformFunc", good[["evalEnv"]]))
    transformFunc <- get(".rxTransformFunc", good[["evalEnv"]])
    checkEquals(func, transformFunc)
    checkIdentical(good[["transformString"]],
                   "rxSafeTry(.rxTransformFunc(..revoTransDf), silent = FALSE)")
}

"test.rxLinkTransformComponents.composition" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    func <- function(dataList) {data$logage <- log(data$age); dataList}

    composite123 <-
      rxLinkTransformComponents(transforms = quote(list(logage = log(age))),
                                transformFunc = func,
                                formulaTransforms =
                                list(log.age. = expression(log(age))))
    checkTrue(exists(".rxTransformsImpliedFunc", composite123[["evalEnv"]]))
    checkTrue(exists(".rxTransformFunc", composite123[["evalEnv"]]))
    checkTrue(exists(".rxFormulaFunc", composite123[["evalEnv"]]))
    checkIdentical(composite123[["transformString"]],
      "rxSafeTry(.rxFormulaFunc(.rxTransformFunc(.rxTransformsImpliedFunc(..revoTransDf))), silent = FALSE)")
}

"test.rxLinkTransformComponents.transformObjects.bad" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    checkException(rxLinkTransformComponents(transformObjects = letters),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(transformObjects = new.env()),
                   silent = TRUE)
    checkException(rxLinkTransformComponents(transformObjects = list(1, 2, 3)),
                   silent = TRUE)
}

"test.rxLinkTransformComponents.transformObjects.good" <- function()
{
    rxLinkTransformComponents <- .rxLinkTransformComponents()
    checkTrue(is.list(rxLinkTransformComponents(transformObjects = list())))
    checkEquals(get("a",
                    rxLinkTransformComponents(transformObjects = list(a = 1L))$evalEnv),
                1L)
}

## Advanced tests

"test.rxTransform.CallingFrame" <- function()
{
  # testing bug: RA-1422
  # Check calling frame of transformFunc argument

  # Input file
  airlineFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.xdf")

  # CrossTabs
  checkException(rxCrossTabs(blocksPerRead ~ F(cube), airlineFile,
                             transformFunc = function(dataList)
                             list(blocksPerRead = blocksPerRead, cube = cube),
                             transformVars = "ArrDelay",
                             reportProgress = 0), silent = TRUE)

  # Cube
  checkException(rxCube(blocksPerRead ~ F(cube), airlineFile,
                        transformFunc = function(dataList)
                        list(blocksPerRead = blocksPerRead, cube = cube),
                        transformVars = "ArrDelay",
                        reportProgress = 0), silent = TRUE)

  # DataStep
  checkException(rxDataStep(airlineFile, tempfile(pattern = ".rxTransCheck"),
                               transformFunc = function(dataList)
                               verboseFlags, reportProgress = 0),
                   silent = TRUE)

  # Linear Model
  checkException(rxLinMod(y ~ x, airlineFile,
                          transformFunc = function(dataList){
                              n <- length(dataList[[1]])
                              list(x = stats::rnorm(n, mean = blocksPerRead),
                                   y = stats::rnorm(n, mean = blocksPerRead))
                          }, transformVars = "ArrDelay", reportProgress = 0),
                   silent = TRUE)
  
  # Logistic Regression
  checkException(rxLogit(y ~ x, airlineFile,
                         transformFunc = function(dataList) {
                             n <- length(dataList[[1]])
                             blocksPerRead
                             x <- stats::rbinom(n, 1, 0.1)
                             y <- stats::rbinom(n, 1, ifelse(x == 0L, 0.5, 0.95))
                             list(x = x, y = y)
                         }, transformVars = "ArrDelay", reportProgress = 0,
                         coeffTolerance = 1e-8,
                         objectiveFunctionTolerance = 1e-8),
                   silent = TRUE)

  # Univariate Summary
  checkException(rxSummary(~ blocksPerRead + verbose, airlineFile,
                           transformFunc = function(dataList)
                           list(blocksPerRead = blocksPerRead,
                                verbose = verbose),
                           transformVars = "ArrDelay",
                           reportProgress = 0),
                   silent = TRUE)
  
  # Text To Xdf
  airlineFile <- file.path(rxGetOption("sampleDataDir"), "AirlineDemoSmall.csv")
  checkException(rxTextToXdf(airlineCsv, tempfile(pattern = ".rxTransCheck2"),
                             transformFunc = function(dataList)
                             list(rowsToSkip = rowsToSkip,
                                    rowsToSniff = rowsToSniff),
                             transformVars = "ArrDelay", reportProgress = 0),
                   silent = TRUE)
}

"test.rxTransform.rowSelection.substitute.fallthrough" <- function()
{
    # tests whether a function that passes rowSelection as
    # a substituted variable in the primary caller gets resolved
    # correctly in the target analytical function after being passed
    # through interediate function(s)

    DF <- as.data.frame(UCBAdmissions)
    DF1 <- subset(DF, subset = Gender=='Male' & Freq > 200)
    form <- Freq ~ Gender : Admit
    zA <- rxCube(form, rowSelection=Gender=='Male' & Freq > 200, data=DF,
        returnDataFrame=TRUE, reportProgress=0)
    zB <- rxCube(form, data=DF1, returnDataFrame=TRUE, reportProgress=0)

    f <- function(form, data, rowSelection=NULL, ...)
    {
       g(form, data=data, rowSelection=substitute(rowSelection))
    }
    g <- function(form, ...) h(form, ...)
    h <- function(form, ...) rxCube(form, ..., reportProgress=0)

    zC <- f(Freq ~ Gender : Admit, rowSelection=Gender=='Male' & Freq > 200,
        data=DF, returnDataFrame=TRUE)
    checkEquals(zA, zB, check.attributes=FALSE)
    checkEquals(zB, zC, check.attributes=FALSE)
}

"test.rxTransform.environment" <- function()
{
    # Create local data source
    DF <- as.data.frame(UCBAdmissions)

    # Create transforms variables.
    # New variables: Team, FreqDouble
    # Deleted variables: Dept, Freq
    transforms <- quote(list(
        Team=factor(paste("Team", Dept)),
        Dept=NULL,
        FreqDouble=Freq * 2,
        Freq=NULL))

    # Create transformFunc variables.
    # Restore Freq and Dept variables, giving them new names for comparison.
    # Explicitly assign row selection variable .rxRowSelection
    xform <- function(dataList)
    {
        dataList$OldFreq <- dataList$FreqDouble / 2
        dataList$OldDept <- factor(gsub("^Team ","",dataList$Team))
        dataList$.rxRowSelection <- dataList$Gender=='Female' & dataList$OldFreq > 200
        return(dataList)
    }

    # Define rowSelection variable that specifies Male, not Female
    rowSelection <- quote(Gender=='Male' & Freq > 200)
    rowSelection2 <- quote(Gender=='Male' & OldFreq > 200)

    # perform rxCube on original data source without transformation
    # and one with transformation. Alter the results so that they
    # should be comparable and, indeed, compare away.
    zCubeOriginal <- as.data.frame(rxCube(Freq ~ Gender : Dept, DF,
        reportProgress=0))
    zCubeOriginal$Team <- factor(paste("Team", zCubeOriginal$Dept))
    zCubeOriginal$Dept <- NULL

    zTransform <- as.data.frame(rxCube(FreqDouble/2 ~ Gender : Team, data=DF,
        transforms=transforms, reportProgress=0))
    nms <- gsub("FreqDouble.2", "Freq", names(zTransform))
    names(zTransform) <- nms
    checkEquals(zCubeOriginal[nms], zTransform)

    # Run both the transforms and transformFunc as well as an original.
    # Adapt the output to be comparable and compare.
    zCubeOriginal2 <- as.data.frame(rxCube(Freq ~ Gender : Dept, data=DF,
        rowSelection=Gender=='Female' & Freq > 200, removeZeroCounts=TRUE,
        reportProgress=0))
    zTransform2 <- as.data.frame(rxCube(OldFreq ~ Gender : OldDept, data=DF, 
        transforms=transforms, transformFunc=xform, transformVars="Gender", 
        removeZeroCounts=TRUE, reportProgress=0))
    nms <- gsub("^Old(.*)","\\1", names(zTransform2))
    names(zTransform2) <- nms
    checkEquals(zCubeOriginal2[nms], zTransform2)
    checkTrue(unique(zTransform2$Gender) == "Female")

    # Run both the transforms, transformFunc, and rowSelection as well as an original.
    # Adapt the output to be comparable and compare.
    zCubeOriginal3 <- as.data.frame(rxCube(Freq ~ Gender : Dept, data=DF,
        rowSelection=rowSelection, removeZeroCounts=TRUE, reportProgress=0))
    zTransform3 <- as.data.frame(rxCube(OldFreq ~ Gender : OldDept, data=DF,
        transforms=transforms, transformFunc=xform, transformVars="Gender",
        rowSelection=rowSelection2, removeZeroCounts=TRUE, reportProgress=0))
    nms <- gsub("^Old(.*)","\\1", names(zTransform3))
    names(zTransform3) <- nms
    checkEquals(zCubeOriginal3[nms], zTransform3)
    checkTrue(unique(zTransform3$Gender) == "Male")
}
"test.rxTransform.transformObjects.packed.function" <- function()
{
    # define local function that we wish to pack into the
    # environment we ship off to exastat and use it for defining
    # a new variable in a transform
    hmsToSeconds <- function(hmsStrings)
    {
        # Time data in "H:M:S" format converted to seconds
        sapply(strsplit(hmsStrings, ":"), function(x){sum(as.numeric(x) * c(3600, 60, 1))})
    }

    # create some faux race data
    set.seed(100)
    N <- 100
    H <- sample.int(5, N, replace = TRUE)
    M <- sample.int(60, N, replace = TRUE)
    S <- sample.int(60, N, replace = TRUE)
    HMS <- paste(H, M, S, sep=":")
    division <- factor(sample(LETTERS[1:4], N, replace = TRUE))
    sex <- factor(sample(c("Male","Female"), N, replace = TRUE))
    DF <- data.frame(HMS = HMS, Division = division, Sex = sex, stringsAsFactors = FALSE)

    # now for some fun. perform a cube on the data, converting
    # the HMS string data to seconds along the way
    z <- rxCube(Minutes ~ Division : Sex, data = DF, 
        transforms = list(Minutes = foo(HMS) / 60),
        transformObjects = list(foo = hmsToSeconds))

    # validate one of the results
    AFemaleMean <- mean(hmsToSeconds(subset(DF, Division == "A" & Sex == "Female")[["HMS"]]) / 60)
    AFemaleMeanCube <- subset(z, Division == "A" & Sex == "Female")[["Minutes"]]
    checkEqualsNumeric(AFemaleMean, AFemaleMeanCube)
}

"test.rxTransform.rxSet.rxGet.rxModify.rxTransformEnvir.functor" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf") 

    "myFunction" <- function() 
    { 
        count1 <- 100L
        
        "myTestFun" <- function(dataList) 
        { 
            if (!.rxIsTestChunk)
            {
                # ladder update: this only updates local count1
                # Chunk 0: 100
                # Chunk 1: 101
                # Chunk 2: 102
                # Chunk 3: 103
                # Chunk 4: 104
                # Chunk 5: 105         
                count1 <<- count1 + 1L
                print(count1)
                
                # targeted update: increment count2
                .rxSet("count2", .rxGet("count2") + 1L)
                
                # targeted update with .rxModify: increment count3
                .rxModify("count3", 1L, FUN = "+")
                
                # targeted update with .rxModify: multiply current count4 by twice its value
                .rxModify("count4", 2L, FUN = "*")
                
                # targeted update with .rxModify: permute second term of series, add nothing
                # Chunk 0: 1 2 3 4 5
                # Chunk 1: 1 3 4 5 2
                # Chunk 2: 1 4 5 2 3
                # Chunk 3: 1 5 2 3 4
                # Chunk 4: 1 2 3 4 5
                # Chunk 5: 1 3 4 5 2
                .rxModify("series", n = 2L, bias = 0L, FUN = "permute")
            }

            return(dataList) 
        }

        myEnv <- new.env(hash = TRUE, parent = baseenv())
        assign("permute", function(x, n = 1L, bias = 10) c(x[-n], x[n]) + bias, envir = myEnv)
        rxDataStep( inData = inFile, transformFunc = myTestFun, 
                    transformObjects = list(count1 = 0L, count2 = 0L, count3 = 0L, count4 = 1L, series = 1:5),
                    transformEnv = myEnv)
                    
        myEnv
    }

    # Obtain the transform environment containing the results
    transformEnvir <- myFunction()
    
    # Check results
    checkEquals(transformEnvir[["count1"]], 0L)
    if (rxIsExpressEdition())
    {
        checkEquals(transformEnvir[["count2"]], 1L)
        checkEquals(transformEnvir[["count3"]], 1L)
        checkEquals(transformEnvir[["count4"]], 2L)
    }
    else {
        checkEquals(transformEnvir[["count2"]], 5L)
        checkEquals(transformEnvir[["count3"]], 5L)
        checkEquals(transformEnvir[["count4"]], 32L)
    }
    checkTrue(is.function(transformEnvir[["permute"]]))
    checkEquals(transformEnvir[["series"]], c(1L, 3L, 4L, 5L, 2L))

    
}

"test.rxTransform.rxSet.rxGet.rxModify.rxTransformEnvir.direct" <- function()
{
    inFile <- file.path(rxGetOption("unitTestDataDir"), "test100r5b.xdf") 
    count1 <- 100L
    
    "myTestFun" <- function(dataList) 
    { 
        if (!.rxIsTestChunk)
        {
            # ladder update: this only updates local count1
            # Chunk 0: 100
            # Chunk 1: 101
            # Chunk 2: 102
            # Chunk 3: 103
            # Chunk 4: 104
            # Chunk 5: 105         
            count1 <<- count1 + 1L
            print(count1)
            
            # targeted update: increment count2
            .rxSet("count2", .rxGet("count2") + 1L)
            
            # targeted update with .rxModify: increment count3
            .rxModify("count3", 1L, FUN = "+")
            
            # targeted update with .rxModify: multiply current count4 by twice its value
            .rxModify("count4", 2L, FUN = "*")
            
            # targeted update with .rxModify: permute second term of series, add nothing
            # Chunk 0: 1 2 3 4 5
            # Chunk 1: 1 3 4 5 2
            # Chunk 2: 1 4 5 2 3
            # Chunk 3: 1 5 2 3 4
            # Chunk 4: 1 2 3 4 5
            # Chunk 5: 1 3 4 5 2
            .rxModify("series", n = 2L, bias = 0L, FUN = "permute")
        }

        return(dataList) 
    }

    myEnv <- new.env(hash = TRUE, parent = baseenv())
    assign("permute", function(x, n = 1L, bias = 10) c(x[-n], x[n]) + bias, envir = myEnv)
    rxDataStep( inData = inFile, transformFunc = myTestFun, 
                transformObjects = list(count1 = 0L, count2 = 0L, count3 = 0L, count4 = 1L, series = 1:5),
                transformEnv = myEnv)
    
    # Check results


    if (rxIsExpressEdition())
    {
        checkEquals(myEnv[["count1"]], 0L)  
        checkEquals(myEnv[["count2"]], 1L)
        checkEquals(myEnv[["count3"]], 1L)
        checkEquals(myEnv[["count4"]], 2L)
    }
    else {  
        checkEquals(myEnv[["count1"]], 0L)  
        checkEquals(myEnv[["count2"]], 5L)
        checkEquals(myEnv[["count3"]], 5L)
        checkEquals(myEnv[["count4"]], 32L)
    }
    checkTrue(is.function(myEnv[["permute"]]))
    checkEquals(myEnv[["series"]], c(1L, 3L, 4L, 5L, 2L))
}
