# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for variable independence tests
#
############################################################################################


mtcarsDF <- rxFactors(datasets::mtcars, factorInfo = c("gear", "cyl", "vs"), varsToKeep = c("gear", "cyl", "vs"), sortLevels = TRUE)
gearCylXtabs <- rxCrossTabs(~ gear : cyl, data = mtcarsDF, returnXtabs = TRUE)
gearVsXtabs <- rxCrossTabs(~ gear : vs, data = mtcarsDF, returnXtabs = TRUE)

###
# Exported Functions
###

"test.rxChiSquaredTest" <- function()
{
  cst1 <- rxChiSquaredTest(gearCylXtabs)
  cst2 <- chisq.test(gearCylXtabs)
  checkEqualsNumeric(cst1[1L, 1L], cst2[["statistic"]])
  checkEqualsNumeric(cst1[1L, 2L], cst2[["parameter"]])
  checkEqualsNumeric(cst1[1L, 3L], cst2[["p.value"]])
}

"test.rxFisherTest" <- function()
{
  ft1 <- rxFisherTest(gearVsXtabs)
  ft2 <- fisher.test(gearVsXtabs)
  checkEqualsNumeric(ft1[1L, 1L], ft2[["p.value"]])
}

"test.rxKendallCor" <- function()
{
  kendallB <- rxKendallCor(gearCylXtabs, type = "b")
  corB <- cor(mtcars[["gear"]], mtcars[["cyl"]], method = "kendall")
  checkEqualsNumeric(kendallB[1L, 1L], corB)
  
  # TODO: need tests for a and c and permutation test
  #kendallA <- rxKendallCor(gearCylXtabs, type = "a")
  #kendallC <- rxKendallCor(gearCylXtabs, type = "c")
}

"test.rxMultiTest" <- function()
{
  data(mtcars)
  t1 <- t.test(mpg ~ vs, data = mtcars)
  t2 <- t.test(hp ~ vs, data = mtcars)
  tsts <- list(t1, t2)
  mt <- rxMultiTest(tsts)
  checkTrue(all(dim(mt) == c(2,8)))
  checkTrue(all(mt[1,1:2] == t1$estimate))
  checkTrue(mt[1,8] == t1$p.value)
  checkTrue(all(mt[2,1:2] == t2$estimate))
  checkTrue(mt[2,8] == t2$p.value)
}

"test.rxPairwiseCrossTab" <- function()
{
  tsts <- rxPairwiseCrossTab(gearCylXtabs, FUN = rxRiskRatio, pooled = TRUE)
  x1 <- gearCylXtabs
  x1 <- rbind(x1[1L,], colSums(x1[2:3,]))
  x1 <- cbind(x1[, 1L], rowSums(x1[,2:3]))
  rr <- rxRiskRatio(x1[2:1, 2:1])
  checkEqualsNumeric(rr$estimate, tsts[[1L]][[1L]][[1L]]$estimate)
  checkEqualsNumeric(rr$conf.int, tsts[[1L]][[1L]][[1L]]$ci)
  checkEqualsNumeric(rr$p.value, tsts[[1L]][[1L]][[1L]]$p.value)
}

"test.rxRiskRatio" <- function()
{
  if (require(epitools))
  {
    epirr <- epitools::riskratio(gearVsXtabs)
    for(i in 2:nrow(gearVsXtabs))
    {
      rr <- rxRiskRatio(gearVsXtabs[c(1L, i),])
      checkEqualsNumeric(rr$estimate,epirr$measure[i, 1L])
      checkEqualsNumeric(rr$conf.int, epirr$measure[i, 2:3])
    }
  }
}

"test.rxOddsRatio" <- function()
{
  if(require(epitools))
  {
    epior <- epitools::oddsratio(gearVsXtabs, method = "wald")
    for(i in 2:nrow(gearVsXtabs))
    {
      or <- rxOddsRatio(gearVsXtabs[c(1L, i),])
      checkEqualsNumeric(or$estimate, epior$measure[i, 1L])
      checkEqualsNumeric(or$conf.int, epior$measure[i, 2:3])
    }
  }
}

###
# Non-exported Functions
###

#"test.flatten3d" <- function()
#{
  #xt <- xtabs(~cyl + vs + gear + am, data=mtcars)
  #fxt <- flatten3d(xt)
  #for (i in seq(xt))
    #{
        #checkEquals(xt[i], fxt[i])
    #}
    #
  #checkTrue(length(dim(fxt))==3)
#}

#"test.xapply" <- function()
#{
  #if(require(ggplot2))
  #{
      #data(diamonds)
      #x <- xtabs( ~color + cut + clarity, data=diamonds)
      #res <- xapply(x, sum)
      #for(i in dim(x)[3])
      #{
        #checkTrue(res[[i]] == sum(x[,,i]))
      #}
    #}
#}

###
# UNSUPPORTED
###

#"test.kruskalWallisTest" <- function()
#{
  #x <- xtabs( ~gear + vs,data=mtcars)
  #kw1 <- kruskalWallisTest(x,ordinalCol=FALSE)
  #kw2 <- kruskal_test( gear ~ as.factor(vs),data=mtcars)
  ##checkTrue(ft1[1,1] == ft2$p.value)
  ##found inconsistancy in result. e-mailing package author
#}
#
#"test.spearmanCor" <- function()
#{
  #data(mtcars)
  #x <- xtabs( ~gear + cyl,data=mtcars)
  #sc <-spearmanCor(x)
  #val <- cor(mtcars$gear,mtcars$cyl,method="spearman")
  #checkEqualsNumeric(sc[1,1] , val)
  #
  #df <- as.data.frame.table(x)
  #spearman_test(as.numeric(df[[1]]) ~ as.numeric(df[[2]]),weights=~df[[3]])
  #spearman_test(gear ~ cyl,data=mtcars)
  ##found inconsistancy in result. e-mailing package author
#}

