# Copyright (c) Microsoft Corporation.  All rights reserved.
#
# FUNCTIONS
#    Unit tests for rxDTree
#
############################################################################################

## NOTE: currently a fixed pattern is repeated to generate the cross-validation fold indices,
## so depending on the number of data blocks and threads, the CV results might be different,
## i.e., the list component cptable[, c("xerror", "xstd")] of the fitted model.
## * if the input is a data.frame, each thread processes roughly equal number of rows
## * if the input is a xdf file, 
##      . if the xdf file has only 1 block, each thread processes roughly equal number of rows
##      . if the xdf file has multi-blocks, 1 thread is responsible for dealing out the blocks
##          and the rest threads each processes roughly equal number of rows

".rxSqliteConnectionStringFromFile" <- function()
{
    if (exists("rxSqliteConnectionStringFromFile", globalenv()))
        get("rxSqliteConnectionStringFromFile", globalenv())
    else
        RevoScaleR:::rxSqliteConnectionStringFromFile
}

.setUp <- function()
{
    #require(rpart)
  #THIS MUST ALWAYS BE SET TO FALSE ON CHECKIN
  .check.cptable <<- FALSE
    
    .deactivate.rxDTree.tests <<- FALSE
    #.maxRowsInMemory <<- 1e6
    
    rxOptions(reportProgress = 0) # by default, don't report progress
}

.tearDown <- function()
{
  if(exists(".maxRowsInMemory", where = ".GlobalEnv")) rm(".maxRowsInMemory", pos = ".GlobalEnv")
    
  rxOptions(TRUE)
}

## rpart
    #rpart(formula, data, weights, subset, na.action = na.rpart, method,
        #model = FALSE, x = FALSE, y = TRUE, parms, control, cost, ...)
        
## rpart.control
    #rpart.control(minsplit = 20, minbucket = round(minsplit/3), cp = 0.01, 
        #maxcompete = 4, maxsurrogate = 5, usesurrogate = 2, xval = 10,
        #surrogatestyle = 0, maxdepth = 30, ...)
        
## rpart.object
    # frame, splits, csplits, cptable, where, 
    # method, parms, control, na.action, 
    # call, terms, x, y, ordered, functions, 

#######################################################################
## classification

###################################
## arguments

"test.rxDTree.classification.args" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        #reportProgress = 1, verbose = 2
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    checkEquals(rxTreeDepth(fit.rpart),  rxTreeDepth(fit.dtree))
    checkEquals(rxLeafSize(fit.rpart),  rxLeafSize(fit.dtree))
        
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)
  
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))

    checkException(rxDTree(formula = form, data = data, maxDepth = 64))
}
 
"test.rxDTree.classification.args.rowSelection" <- function()
{
    
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where, checkNames = FALSE)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)

    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)
  
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.noweight.rowSelection" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, subset = eval(sub), #weights = kyphosis.with.weights$Age,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, rowSelection = sub, #pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where, checkNames = FALSE)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.weight.rowSelection" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response, 4 is ok but not 9

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, subset = eval(sub), weights = kyphosis.with.weights$Age,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, rowSelection = sub, pweights = "Age", #
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where, checkNames = FALSE)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.xdf.rowSelection" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    # xdf file with only 1 block
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    }
    
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age",  rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        #reportProgress = 2, verbose = 2,
        numThreads = 3)
    
    ## cleanup
    file.remove(XDF)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.args.transformFunc" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
    
    bogusTransform <- function(dataList) 
    { 
        dataList$newVar <-dataList$Age*2 
        #print("newVar") 
        #print(dataList$newVar) 
        return(dataList) 
    } 

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    
    res <- try(
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        transformFunc = bogusTransform),
    silent = TRUE)
    
    # Check for error when transformVars is not specified	
  checkTrue(is(res,"try-error"))
  #checkTrue(grepl("Make sure that variables used in the transformFunc are specified in transformVars", attr(res, "condition")$message))
}

"test.rxDTree.classification.args.transformVars" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
    
    bogusTransform <- function(dataList) 
    { 
        dataList$newVar <-dataList$Age*2 
        #print("newVar") 
        #print(dataList$newVar) 
        return(dataList) 
    } 

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        transformFunc = bogusTransform, transformVars = c("Age"))
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)

    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTrees.transformFunc.negative" <- function(){
  
  xform <- function(dataList)
  {
    ret = data.frame(dataList)
    ret = ret[ret$age < 40,,drop=FALSE]
    ret    
  }
  
  infert_mod = tryCatch({
    rxDTree(case ~ parity + education + spontaneous + induced, 
             data=infert, transformFunc=xform, transformVars="age")
    stop(paste("Subsetting data with the transform function did not give correct error message in", "rxDTree"))
  }, error = function(e){
    if (grepl("transform function does rows selections", e$message) == FALSE){
      print(e$message)
      stop(paste("Error message is not informative with why transform function fails in", "rxDTree"))
    }
  })
  
}

rtsCheck <- function(regular.model, binary.model, data, type)
{
    pred.regular <- rxPredict(regular.model, data = data, type = type)
    oo <- capture.output(pred.rts <- rxPredict(binary.model, data = data, type = type))
    
    # Verify we went through the native scoring path    
    checkEquals(1, sum(grepl("Native prediction succeeded", oo)), paste0("Verification that native scoring was used [type = ", type, "]."))
    # Rename the columns - the name are different
    colnames(pred.regular) <- paste0("col", 1:NCOL(pred.regular))
    colnames(pred.rts) <- paste0("col", 1:NCOL(pred.rts))
    # Verify the predictions are the same
    checkEquals(pred.regular, pred.rts, paste0("RTS results verification [type = ", type, "]."))
}

#bigAnalytics RA-11082
#rxDTree doesn't know how to deal with transformations in formula
"test.rxDTree.classification.args.transformVars.numeric" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## set up arguments
    form        <- Kyphosis ~ Number + Start + Age2
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    
    #range(data$Age * 2) #[1]   2 412
    myTransform <- function(dataList) 
    { 
        dataList$Age2 <-dataList$Age*2 
        attr(dataList$Age2, ".rxSetLowHigh") <- c(2, 412) # to set the overall low/high values for this newly created variable
        return(dataList) 
    } 

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = cbind(data, Age2 = data$Age * 2), weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        transformFunc = myTransform, transformVars = c("Age"))
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)

    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    indexcol <- grep("index", colnames(fit.rpart$split))    # an index different by 0.5
    checkEquals(fit.rpart$splits[, -c(countcol, indexcol)], fit.dtree$splits[, -c(countcol, indexcol)])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.pred <- predict(fit.rpart, newdata = cbind(data, Age2 = data$Age * 2))
    
    outData <- tempfile( pattern = ".rxDTree", fileext = ".xdf")
    outFile <- tempfile( pattern = ".rxDTree", fileext = ".xdf")
    inData <- rxImport(data, outFile)
    rxPredict(fit.dtree, data = inData, outData = outData, writeModelVars = FALSE, overwrite = TRUE)
    fit.dtree.pred <- rxReadXdf(outData)
    
    checkEqualsNumeric(fit.rpart.pred,  as.matrix(fit.dtree.pred))
}

#######################################################################
## variables with only missing values
"test.rxDTree.classification.onlymissing.logical" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
  # Change in behavior of high/lows for logicals (fix for RA-7064)
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis

    ## introduce all missings
    kyphosis.with.weights$Number     <- as.logical(NA)   # missing all of one predictor, logical
    
    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)   # a variable removed due to all missing
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)

    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    #checkEquals(fit.rpart$ordered,  fit.dtree$ordered) # a variable removed due to all missing
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.onlymissing.integer" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis

    ## introduce all missings
    kyphosis.with.weights$Number     <- as.integer(NA)   # as.character, as.double, as.factor, as.ordered
    
    ## set up arguments
    form        <- Kyphosis ~ Number + Start
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## We are getting the same split vars and values as rpart
    ## but other attributes do not match the results of rpart
    ## because we remove variables with only missing values
    ## completely for further analysis but rpart does not.
    
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)   # a variable removed due to all missing
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)

    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    #checkEquals(fit.rpart$ordered,  fit.dtree$ordered) # a variable removed due to all missing
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## missing value handling

##
"test.rxDTree.classification.missing" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.missing.args.removeMissings" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, na.action = na.omit,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", removeMissings = TRUE,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## categorical predictors

## unordered factor
"test.rxDTree.classification.factor" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.classification.ordered" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## cross validation for classification

## unordered factor
"test.rxDTree.classification.factor.crossval" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        #reportProgress = 0, verbose = 2
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

#bigAnalytics RA-10112
#rxDTree with a loss matrix produces a plotcp plot very different from that produced by a comparable rpart model
"test.rxDTree.classification.factor.crossval.cptable" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    numThreads = 1)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    #if(exists(".check.cptable") && .check.cptable) 
  checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    #xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    #checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.classification.ordered.crossval" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0         
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## cross validation for classification without weight

## unordered factor
"test.rxDTree.classification.factor.crossval.noweight" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, #weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, #pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.classification.ordered.crossval.noweight" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0         
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, #weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, #pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    ##TODO: investigate the minor difference for index col by setting indexcol = NULL
    indexcol <- grep("index", colnames(fit.rpart$split))    # NULL 
    countcol <- grep("count", colnames(fit.rpart$split))
    countcol <- c(countcol, indexcol)
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## cross validation for classification for multi-block xdf file

## unordered factor
"test.rxDTree.classification.factor.crossval.xdf" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    # xdf file with only 1 block
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    { 
        rxDataStep(DF[1:40,], XDF, overwrite = TRUE)
        rxDataStep(DF[41:81,], XDF, append = "rows")
    }
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        numThreads = 3)
    
    ## cleanup
    file.remove(XDF)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.classification.ordered.crossval.xdf" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(DF[1:40,], XDF, overwrite = TRUE)
        rxDataStep(DF[41:81,], XDF, append = "rows")
    }
    
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0         
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        numThreads = 3)
    
    ## cleanup
    file.remove(XDF)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## an obs is missing response

## an obs is missing weight

## an obs is missing some predictors

## an obs is missing all predictors with weights not specified

## an obs is missing all predictors with weights specified

#######################################################################
## regression

###################################
## missing value handling

##
"test.rxDTree.regression.missing" <- function()
{
    if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
  
    #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        #reportProgress = 1, verbose = 2
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.regression.missing.args.removeMissings" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, na.action = na.omit,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", removeMissings = TRUE,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    #checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol]) // 0.5 diff in index
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## categorical predictors

## unordered factor
"test.rxDTree.regression.factor" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.regression.ordered" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## cross validation for regression

## unordered factor
"test.rxDTree.regression.factor.crossval" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.regression.ordered.crossval" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## cross validation for regression without weight

## unordered factor
"test.rxDTree.regression.factor.crossval.noweight" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, #weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, #pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.regression.ordered.crossval.noweight" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, #weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, #pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

###################################
## cross validation for regression for multi-block xdf file

## unordered factor
"test.rxDTree.regression.factor.crossval.xdf" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree2", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    }
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        numThreads = 3)
    
    ## cleanup
    file.remove(XDF)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

## ordered factor
"test.rxDTree.regression.ordered.crossval.xdf" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce ordered factors
    kyphosis.with.weights$Number <- as.ordered(kyphosis$Number)
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree3", fileext = ".xdf")
    if (file.exists(XDF)) file.remove(XDF)
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    }
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 2
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        numThreads = 3)
    
    ## cleanup
    file.remove(XDF)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)    # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

#######################################################################
## rxPredict.rxDTree
"test.rxDTree.classification.missing.predict" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.pred <- predict(fit.rpart, newdata = data)
    fit.dtree.pred <- rxPredict(fit.dtree, data = data)
    checkEqualsNumeric(fit.rpart.pred,  as.matrix(fit.dtree.pred))
    
    ##############
    ## Real-Time scoring
    fit.binary <- fit.dtree
    fit.binary$binary <- rxSerializeModel(fit.binary)
    
    for(tt in c("vector", "matrix", "prob", "class"))
    {
        rtsCheck(fit.dtree, fit.binary, data, type = tt)
    }
}

"test.rxDTree.classification.missing.predict.compression" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
  
    DF <- kyphosis.with.weights
    
    ## create a multi-block uncompressed xdf input file
    XDF.in.uncompressed <- tempfile(pattern = ".rxDTree4.in.uncompressed", fileext = ".xdf")
  on.exit(file.remove(XDF.in.uncompressed), add = TRUE)
    if (file.exists(XDF.in.uncompressed)) file.remove(XDF.in.uncompressed)
    
    if (rxIsExpressEdition())
    {
        rxDataStep( inData = DF, outFile = XDF.in.uncompressed, overwrite=TRUE, xdfCompressionLevel = 0)
    } else
    {
        rxDataStep(DF[1:40,], XDF.in.uncompressed, overwrite = TRUE, xdfCompressionLevel = 0)
        rxDataStep(DF[41:81,], XDF.in.uncompressed, append = "rows", xdfCompressionLevel = 0)
    }
    
    #rxGetInfo(XDF.in.uncompressed, getVarInfo = TRUE, getBlockSizes = TRUE, verbose = 1)
    DF.in.uncompressed <- rxDataStep(XDF.in.uncompressed)
  
    ## create a multi-block compressed xdf input file
    XDF.in.compressed <- tempfile(pattern = ".rxDTree4.in.compressed", fileext = ".xdf")
  on.exit(file.remove(XDF.in.compressed), add = TRUE)
    if (file.exists(XDF.in.compressed)) file.remove(XDF.in.compressed)
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF.in.compressed, overwrite = TRUE, xdfCompressionLevel = 1)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF.in.compressed, overwrite = TRUE, xdfCompressionLevel = 1)
        rxDataStep(inData = DF[41:81,], outFile = XDF.in.compressed, append = "rows", xdfCompressionLevel = 1)
    }
    #rxDataFrameToXdf(DF[1:40,], XDF.in.compressed, overwrite = TRUE, xdfCompressionLevel = 1)
    #rxDataFrameToXdf(DF[41:81,], XDF.in.compressed, append = "rows", xdfCompressionLevel = 1)
    #rxGetInfo(XDF.in.compressed, getVarInfo = TRUE, getBlockSizes = TRUE, verbose = 1)
    DF.in.compressed <- rxDataStep(XDF.in.compressed)
  
  ## check results
  checkEquals(DF.in.uncompressed, DF.in.compressed)
  checkTrue( file.info( XDF.in.uncompressed )$size > file.info( XDF.in.compressed )$size )
  
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF.in.compressed, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.pred <- predict(fit.rpart, newdata = data)
    fit.dtree.pred <- rxPredict(fit.dtree, data = data)
    checkEqualsNumeric(fit.rpart.pred,  as.matrix(fit.dtree.pred))
  
  ## compression prediction vs. uncompressed prediction
  
  ## do prediction
  XDF.out.uncompressed <- tempfile(pattern = ".rxDTree4.out.uncompressed", fileext = ".xdf")
  on.exit(file.remove(XDF.out.uncompressed), add = TRUE)
    fit.dtree.pred.0 <- rxPredict(fit.dtree, data = XDF.in.compressed, outData = XDF.out.uncompressed, 
    xdfCompressionLevel = 0)
  XDF.out.compressed <- tempfile(pattern = ".rxDTree4.out.compressed", fileext = ".xdf")
  on.exit(file.remove(XDF.out.compressed), add = TRUE)
    fit.dtree.pred.1 <- rxPredict(fit.dtree, data = XDF.in.compressed, outData = XDF.out.compressed, 
    xdfCompressionLevel = 1)
  
  ## check results
  DF.out.uncompressed <- rxDataStep(XDF.out.uncompressed)
  DF.out.compressed <- rxDataStep(XDF.out.compressed)
  checkEquals(DF.out.uncompressed, DF.out.compressed)
  
  checkTrue( file.info( XDF.out.uncompressed )$size > file.info( XDF.out.compressed )$size )
}

"test.rxDTree.classification.predict.writeModelVars.extraVarsToWrite" <- function()
{
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
    kyphosis.with.weights$ExtraVar <- 1:nrow(kyphosis.with.weights)
    kyphosis.with.weights$ExtraVar2 <- 1:nrow(kyphosis.with.weights)

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree5a", fileext = ".xdf")
    on.exit(file.remove(XDF), add = TRUE)
    if (file.exists(XDF)) file.remove(XDF)

    rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
    rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    
    XDF.out <- tempfile( pattern = ".rxDTree5aOut", fileext = ".xdf")
    XDF.out.extra <- tempfile( pattern = ".rxDTree5bOut", fileext = ".xdf")
    on.exit(file.remove(XDF.out), add = TRUE)
    on.exit(file.remove(XDF.out.extra), add = TRUE)
    if (file.exists(XDF.out)) file.remove(XDF.out)
    if (file.exists(XDF.out.extra)) file.remove(XDF.out.extra)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    method      <- "class"
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))

   fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", # outFile = XDF.out, writeModelVars = TRUE, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    rxPredict(modelObject = fit.dtree, data = XDF, outData = XDF.out, writeModelVars = TRUE, 
        overwrite = TRUE)
    varInfoWriteModelVars <- rxGetVarInfo(XDF.out)
    
    rxPredict(modelObject = fit.dtree, data = XDF, outData = XDF.out, writeModelVars = TRUE, 
        extraVarsToWrite = "ExtraVar", overwrite = TRUE)    
    
    varInfoExtra  <- rxGetVarInfo(XDF.out)
       
    DF.out <- rxPredict(modelObject = fit.dtree, data = DF, writeModelVars = TRUE, 
        extraVarsToWrite = "ExtraVar", overwrite = TRUE)    
    (varInfoExtraDF  <- rxGetVarInfo(DF.out))
    baseModelVars <- fit.dtree$params$readVars
    checkTrue(all(c(baseModelVars, "ExtraVar") %in% names(varInfoExtraDF)))
    checkEquals(varInfoExtraDF$ExtraVar$low, 1)
    checkEquals(varInfoExtraDF$ExtraVar$high, 81)

    DF.out <- rxPredict(modelObject = fit.dtree, data = DF, writeModelVars = FALSE, 
        extraVarsToWrite = "ExtraVar", overwrite = TRUE)    
    (varInfoExtraDF  <- rxGetVarInfo(DF.out))
    baseModelVars <- fit.dtree$params$readVars
    checkTrue(all("ExtraVar" %in% names(varInfoExtraDF)))   
    checkTrue(!any(baseModelVars %in% names(varInfoExtraDF)))
    checkEquals(varInfoExtraDF$ExtraVar$low, 1)
    checkEquals(varInfoExtraDF$ExtraVar$high, 81)

    DF.out <- rxPredict(modelObject = fit.dtree, data = DF, writeModelVars = TRUE, 
        overwrite = TRUE)    
    (varInfoExtraDF  <- rxGetVarInfo(DF.out))
    baseModelVars <- fit.dtree$params$readVars
    checkTrue(all(baseModelVars %in% names(varInfoExtraDF)))   
    checkTrue(!any("ExtraVar" %in% names(varInfoExtraDF)))

    DF.out <- rxPredict(modelObject = fit.dtree, data = DF, writeModelVars = FALSE, 
        overwrite = TRUE)    
    (varInfoExtraDF  <- rxGetVarInfo(DF.out))
    baseModelVars <- fit.dtree$params$readVars
    checkTrue(!any(c(baseModelVars, "ExtraVar") %in% names(varInfoExtraDF)))
    
    rxPredict(modelObject = fit.dtree, data = XDF, outData = XDF.out, writeModelVars = FALSE, 
        extraVarsToWrite = "ExtraVar", overwrite = TRUE) 
    
    varInfoExtraOnly  <- rxGetVarInfo(XDF.out)
    
    rxPredict(modelObject = fit.dtree, data = XDF, outData = XDF.out, writeModelVars = TRUE, 
        computeResiduals = TRUE, overwrite = TRUE) 
    varInfoResid <- rxGetVarInfo(XDF.out)
         
    checkEquals(length(varInfoWriteModelVars)+1, length(varInfoExtra))
    
    checkEquals(varInfoWriteModelVars$absent_prob, varInfoExtra$absent_prob)
    checkEquals(varInfoExtra$ExtraVar$low, 1)
    checkEquals(varInfoExtra$ExtraVar$high, 81)
    
    checkEquals(length(varInfoExtraOnly), 3)
    
    checkEquals(varInfoWriteModelVars$absent_prob, varInfoExtraOnly$absent_prob)
    checkEquals(varInfoExtraOnly$ExtraVar$low, 1)
    checkEquals(varInfoExtraOnly$ExtraVar$high, 81)
    
    checkEquals(varInfoResid$Kyphosis_Resid$low, 0)
    checkEquals(varInfoResid$Kyphosis_Resid$high, 3)

  ## append rows
  dim.XDF.out1 <- dim(rxDataStep(XDF.out))
    rxPredict(modelObject = fit.dtree, data = XDF, outData = XDF.out, writeModelVars = TRUE, 
        computeResiduals = TRUE, append = "rows") 
  dim.XDF.out2 <- dim(rxDataStep(XDF.out))
  checkEquals(dim.XDF.out2[1], dim.XDF.out1[1] * 2)
}

"test.rxDTree.regression.missing.predict" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.pred <- predict(fit.rpart, newdata = data)
    fit.dtree.pred <- rxPredict(fit.dtree, data = data)
    checkEqualsNumeric(fit.rpart.pred,  as.matrix(fit.dtree.pred))

    ##############
    ## Real-Time scoring
    fit.binary <- fit.dtree
    fit.binary$binary <- rxSerializeModel(fit.binary)
    for(tt in c("vector", "matrix"))
    {
        rtsCheck(fit.dtree, fit.binary, data, type = tt)
    }
}

"test.rxDTree.regression.missing.predict.compression" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    DF <- kyphosis.with.weights
    
    ## create a multi-block uncompressed xdf input file
    XDF.in.uncompressed <- tempfile(pattern = ".rxDTree4.in.uncompressed", fileext = ".xdf")
  on.exit(file.remove(XDF.in.uncompressed), add = TRUE)
    if (file.exists(XDF.in.uncompressed)) file.remove(XDF.in.uncompressed)
    
    if (rxIsExpressEdition())
    {
         rxDataStep(DF, XDF.in.uncompressed, overwrite = TRUE, xdfCompressionLevel = 0)
    } else
    {
        rxDataStep(DF[1:40,], XDF.in.uncompressed, overwrite = TRUE, xdfCompressionLevel = 0)
        rxDataStep(DF[41:81,], XDF.in.uncompressed, append = "rows", xdfCompressionLevel = 0)
    }
    
    #rxGetInfo(XDF.in.uncompressed, getVarInfo = TRUE, getBlockSizes = TRUE, verbose = 1)
    DF.in.uncompressed <- rxDataStep(XDF.in.uncompressed)
  
    ## create a multi-block compressed xdf input file
    XDF.in.compressed <- tempfile(pattern = ".rxDTree4.in.compressed", fileext = ".xdf")
  on.exit(file.remove(XDF.in.compressed), add = TRUE)
    if (file.exists(XDF.in.compressed)) file.remove(XDF.in.compressed)
    
    if (rxIsExpressEdition())
    {
        rxDataStep(DF, XDF.in.compressed, overwrite = TRUE, xdfCompressionLevel = 1)
    } else
    {
        rxDataStep(DF[1:40,], XDF.in.compressed, overwrite = TRUE, xdfCompressionLevel = 1)
        rxDataStep(DF[41:81,], XDF.in.compressed, append = "rows", xdfCompressionLevel = 1)
    }
    
    #rxGetInfo(XDF.in.compressed, getVarInfo = TRUE, getBlockSizes = TRUE, verbose = 1)
    DF.in.compressed <- rxDataStep(XDF.in.compressed)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF.in.compressed, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.pred <- predict(fit.rpart, newdata = data)
    fit.dtree.pred <- rxPredict(fit.dtree, data = data)
    checkEqualsNumeric(fit.rpart.pred,  as.matrix(fit.dtree.pred))
  
  ## compression prediction vs. uncompressed prediction
  
  ## do prediction
  XDF.out.uncompressed <- tempfile(pattern = ".rxDTree4.out.uncompressed", fileext = ".xdf")
  on.exit(file.remove(XDF.out.uncompressed), add = TRUE)
    fit.dtree.pred.0 <- rxPredict(fit.dtree, data = XDF.in.compressed, outData = XDF.out.uncompressed, 
    xdfCompressionLevel = 0)
  XDF.out.compressed <- tempfile(pattern = ".rxDTree4.out.compressed", fileext = ".xdf")
  on.exit(file.remove(XDF.out.compressed), add = TRUE)
    fit.dtree.pred.1 <- rxPredict(fit.dtree, data = XDF.in.compressed, outData = XDF.out.compressed, 
    xdfCompressionLevel = 1)
  
  ## check results
  DF.out.uncompressed <- rxDataStep(XDF.out.uncompressed)
  DF.out.compressed <- rxDataStep(XDF.out.compressed)
  checkEquals(DF.out.uncompressed, DF.out.compressed)
  
  checkTrue( file.info( XDF.out.uncompressed )$size > file.info( XDF.out.compressed )$size )
}

#######################################################################
## residuals
"test.rxDTree.classification.missing.residuals" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.resid <- residuals(fit.rpart)
    fit.dtree.resid <- rxPredict(fit.dtree, data = data, computeResiduals = TRUE)
    # NOTE: rpart removes the observations with missing response
    fit.dtree.resid <- fit.dtree.resid[[ncol(fit.dtree.resid)]]
    fit.dtree.resid <- fit.dtree.resid[!is.na(fit.dtree.resid)]
    checkEqualsNumeric(fit.rpart.resid,  fit.dtree.resid)
}

"test.rxDTree.regression.missing.residuals" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
        
    kyphosis.with.weights$Kyphosis <- as.integer(kyphosis$Kyphosis)
    
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Age[9]        <- NA   # missing response

    ## set up arguments
    form        <- Age ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "anova"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
        
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Kyphosis, 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Kyphosis", 
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    checkEquals(fit.rpart$where,    fit.dtree$where)
    checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    checkEquals(fit.rpart$control,  fit.dtree$control)
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
    
    ## predict
    fit.rpart.resid <- residuals(fit.rpart)
    fit.dtree.resid <- rxPredict(fit.dtree, data = data, computeResiduals = TRUE)
    # NOTE: rpart removes the observations with missing response
    fit.dtree.resid <- fit.dtree.resid[[ncol(fit.dtree.resid)]]
    fit.dtree.resid <- fit.dtree.resid[!is.na(fit.dtree.resid)]
    checkEqualsNumeric(fit.rpart.resid,  fit.dtree.resid)
}

#######################################################################
## outFile
"test.rxDTree.classification.xdf.outFile" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree5", fileext = ".xdf")
    on.exit(file.remove(XDF), add = TRUE)
    if (file.exists(XDF)) file.remove(XDF)
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    # xdf file with only 1 block
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    }
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    XDF.out <- tempfile( pattern = ".rxDTree5Out", fileext = ".xdf")
    on.exit(file.remove(XDF.out), add = TRUE)
    if (file.exists(XDF.out)) file.remove(XDF.out)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    #sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, #subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", outFile = XDF.out, writeModelVars = FALSE, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  fit.dtree$where <- rxDataStep(XDF.out)$.rxNode
    checkEquals(fit.rpart$where,    fit.dtree$where[!is.na(fit.dtree$where)], check.attributes = FALSE)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.xdf.outFile.writeModelVars" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree5", fileext = ".xdf")
    on.exit(file.remove(XDF), add = TRUE)
    if (file.exists(XDF)) file.remove(XDF)
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    # xdf file with only 1 block
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    }
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    XDF.out <- tempfile( pattern = ".rxDTree5Out", fileext = ".xdf")
    on.exit(file.remove(XDF.out), add = TRUE)
    if (file.exists(XDF.out)) file.remove(XDF.out)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    #sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, #subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", outFile = XDF.out, writeModelVars = TRUE, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  fit.dtree$where <- rxDataStep(XDF.out)$.rxNode
    checkEquals(fit.rpart$where,    fit.dtree$where[!is.na(fit.dtree$where)], check.attributes = FALSE)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.xdf.outFile.writeModelVars.extraVarsToWrite" <- function()
{
    #require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
    kyphosis.with.weights$ExtraVar <- 1:nrow(kyphosis.with.weights)
    kyphosis.with.weights$ExtraVar2 <- 1:nrow(kyphosis.with.weights)

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree5a", fileext = ".xdf")
    on.exit(file.remove(XDF), add = TRUE)
    if (file.exists(XDF)) file.remove(XDF)

    rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
    rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    
    XDF.out <- tempfile( pattern = ".rxDTree5aOut", fileext = ".xdf")
    XDF.out.extra <- tempfile( pattern = ".rxDTree5bOut", fileext = ".xdf")
    on.exit(file.remove(XDF.out), add = TRUE)
    on.exit(file.remove(XDF.out.extra), add = TRUE)
    if (file.exists(XDF.out)) file.remove(XDF.out)
    if (file.exists(XDF.out.extra)) file.remove(XDF.out.extra)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    method      <- "class"
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))

   fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", outFile = XDF.out, writeModelVars = TRUE, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    varInfo <- rxGetVarInfo(XDF.out)
    
    fitDtreeExtra <- rxDTree(
         formula = form, data = XDF, pweights = "Age", outFile = XDF.out.extra, 
         writeModelVars = TRUE, extraVarsToWrite = "ExtraVar",  
         method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
         maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    varInfoExtra  <- rxGetVarInfo(XDF.out.extra)
    checkEquals(length(varInfo)+1, length(varInfoExtra))
    
    checkEquals(varInfo$.rxNode, varInfoExtra$.rxNode)
    checkEquals(varInfoExtra$ExtraVar$low, 1)
    checkEquals(varInfoExtra$ExtraVar$high, 81)
    
    
    if (file.exists(XDF.out.extra)) file.remove(XDF.out.extra)
    fitDtreeExtraNoModel <- rxDTree(
         formula = form, data = XDF, pweights = "Age", outFile = XDF.out.extra, 
         writeModelVars = FALSE, extraVarsToWrite = "ExtraVar", overwrite = TRUE, 
         method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
         maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )    
        
    varInfoExtraNoModel <- rxGetVarInfo(XDF.out.extra)
    
    checkEquals(length(varInfoExtraNoModel), 2)
    
    checkEquals(varInfo$.rxNode, varInfoExtraNoModel$.rxNode)
    checkEquals(varInfoExtraNoModel$ExtraVar$low, 1)
    checkEquals(varInfoExtraNoModel$ExtraVar$high, 81)
    
}

"test.rxDTree.classification.xdf.outFile.writeModelVars.xdfCompressionLevel" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response

    ## create a multi-block xdf file
    DF <- kyphosis.with.weights
    XDF <- tempfile( pattern = ".rxDTree5", fileext = ".xdf")
    on.exit(file.remove(XDF), add = TRUE)
    if (file.exists(XDF)) file.remove(XDF)
    #rxDataFrameToXdf(DF, XDF, overwrite = TRUE)    # xdf file with only 1 block
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = DF, outFile = XDF, overwrite = TRUE)
    } else
    {
        rxDataStep(inData = DF[1:40,], outFile = XDF, overwrite = TRUE)
        rxDataStep(inData = DF[41:81,], outFile = XDF, append = "rows")
    }
    #rxGetInfo(XDF, getVarInfo = TRUE, getBlockSizes = TRUE)
    #DF.my <- rxDataStep(XDF)
    
    XDF.out.uncompressed <- tempfile( pattern = ".rxDTree5Out", fileext = ".xdf")
    on.exit(file.remove(XDF.out.uncompressed), add = TRUE)
    if (file.exists(XDF.out.uncompressed)) file.remove(XDF.out.uncompressed)
  
    XDF.out.compressed <- tempfile( pattern = ".rxDTree5OutCompressed", fileext = ".xdf")
    on.exit(file.remove(XDF.out.compressed), add = TRUE)
    if (file.exists(XDF.out.compressed)) file.remove(XDF.out.compressed)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    #sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, #subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = XDF, pweights = "Age", outFile = XDF.out.uncompressed, writeModelVars = TRUE, #rowSelection = sub,
        xdfCompressionLevel = 0, #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    fit.dtree.compressed <- rxDTree(
        formula = form, data = XDF, pweights = "Age", outFile = XDF.out.compressed, writeModelVars = TRUE, 
        xdfCompressionLevel = 1, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## check compression
  DF.out.uncompressed <- rxDataStep(XDF.out.uncompressed)
  DF.out.compressed <- rxDataStep(XDF.out.compressed)
  checkEquals(DF.out.uncompressed, DF.out.compressed)
  checkTrue( file.info( XDF.out.uncompressed )$size > file.info( XDF.out.compressed )$size )
  
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
  fit.dtree$where <- rxDataStep(XDF.out.uncompressed)$.rxNode
    checkEquals(fit.rpart$where,    fit.dtree$where[!is.na(fit.dtree$where)], check.attributes = FALSE)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.df.outFile" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
    
    XDF.out <- tempfile( pattern = ".rxDTree5Out", fileext = ".xdf")
    on.exit(file.remove(XDF.out), add = TRUE)
    if (file.exists(XDF.out)) file.remove(XDF.out)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    #sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, #subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", outFile = XDF.out, writeModelVars = FALSE, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100, 
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        overwrite = TRUE)
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  fit.dtree$where <- rxDataStep(XDF.out)$.rxNode
    checkEquals(fit.rpart$where,    fit.dtree$where[!is.na(fit.dtree$where)], check.attributes = FALSE)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.df.outFile.writeModelVars" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
    
    XDF.out <- tempfile( pattern = ".rxDTree5Out", fileext = ".xdf")
    on.exit(file.remove(XDF.out), add = TRUE)
    if (file.exists(XDF.out)) file.remove(XDF.out)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    #sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, #subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", outFile = XDF.out, writeModelVars = TRUE, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  fit.dtree$where <- rxDataStep(XDF.out)$.rxNode
    checkEquals(fit.rpart$where,    fit.dtree$where[!is.na(fit.dtree$where)], check.attributes = FALSE)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.df.outFile.writeModelVars.xdfCompressionLevel" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    require(rpart)
  
    ## make local data
    kyphosis.with.weights <- kyphosis
    
    ## introduce unordered factors
    kyphosis.with.weights$Number <- as.factor(kyphosis$Number)
        
    ## introduce missings
    kyphosis.with.weights$Number[1]     <- NA   # missing one predictor
    kyphosis.with.weights$Start[3]      <- NA   # missing one predictor
    kyphosis.with.weights$Number[5]     <- NA   # missing one predictor
    kyphosis.with.weights$Number[7]     <- NA   # missing both predictors
    kyphosis.with.weights$Start[7]      <- NA   # 
    kyphosis.with.weights$Kyphosis[9]   <- NA   # missing response
    
    XDF.out.uncompressed <- tempfile( pattern = ".rxDTree5Out", fileext = ".xdf")
    on.exit(file.remove(XDF.out.uncompressed), add = TRUE)
    if (file.exists(XDF.out.uncompressed)) file.remove(XDF.out.uncompressed)
  
    XDF.out.compressed <- tempfile( pattern = ".rxDTree5OutCompressed", fileext = ".xdf")
    on.exit(file.remove(XDF.out.compressed), add = TRUE)
    if (file.exists(XDF.out.compressed)) file.remove(XDF.out.compressed)
  
    ## set up arguments
    form        <- Kyphosis ~ Number + Start    # one response and two predictors
    data        <- kyphosis.with.weights
    #weights    <- 
    #subset     <- 
    #na.action  <- na.rpart
    method      <- "class"
    #model      <- FALSE
    #x          <- FALSE
    #y          <- TRUE
    parms       <- list(prior = c(0.8, 0.2), loss = c(0, 2, 3, 0), split = "gini")
    xval        <- 0
    control.dtree   <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data)) else 0
    cost        <- 1 + seq(length(attr(terms(form), "term.labels")))
    #sub         <- expression(Start > 3)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = data, weights = kyphosis.with.weights$Age, #subset = eval(sub),
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.rpart, cost = cost)
    fit.dtree <- rxDTree(
        formula = form, data = data, pweights = "Age", outFile = XDF.out.uncompressed, writeModelVars = TRUE, #rowSelection = sub,
        xdfCompressionLevel = 0, #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    fit.dtree.compressed <- rxDTree(
        formula = form, data = data, pweights = "Age", outFile = XDF.out.compressed, writeModelVars = TRUE, 
        xdfCompressionLevel = 1, #rowSelection = sub,
        #subset, na.action = na.rpart, model = FALSE, x = FALSE, y = TRUE,
        method = method, parms = parms, control = control.dtree, cost = cost, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## check compression
  DF.out.uncompressed <- rxDataStep(XDF.out.uncompressed)
  DF.out.compressed <- rxDataStep(XDF.out.compressed)
  checkEquals(DF.out.uncompressed, DF.out.compressed)
  checkTrue( file.info( XDF.out.uncompressed )$size > file.info( XDF.out.compressed )$size )
  
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  fit.dtree$where <- rxDataStep(XDF.out.uncompressed)$.rxNode
    checkEquals(fit.rpart$where,    fit.dtree$where[!is.na(fit.dtree$where)], check.attributes = FALSE)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

#######################################################################
## bug fixes

#bigAnalytics RA-9803
#rxDTree with outFile specified throws an error when rxOptions() outDataPath is set
"test.rxDTree.RA-9803.outDataPath.outFile" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))

  tmpDir <- tempdir()
  outFile <- "rxDTreeOutFile.xdf"
  tmpFile <- file.path(tmpDir, outFile)
  if (file.exists(tmpFile)) file.remove(tmpFile)
  on.exit(if (file.exists(tmpFile)) file.remove(tmpFile), add = TRUE)

  outDataPath.old <- rxGetOption("outDataPath")
  rxOptions(outDataPath = c(tmpDir, outDataPath.old))
  on.exit(rxOptions(outDataPath = outDataPath.old), add = TRUE)

  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width
  iris.dtree <- rxDTree(form, data = iris, outFile = outFile,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )

  # The following indicate that the outFile gets written to the proper place 
  checkTrue(file.exists(tmpFile))
}

#bigAnalytics RA-9718
#rxDTree is overwriting rather than appending to the contents of its outFile
"test.rxDTree.outFile.bSameInandOutFile" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  
  on.exit(if(file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- data.xdf

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
    )
  
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.	
  where <- ".rxNode"
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), where) %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

#bigAnalytics RA-10414
#rxDForest example with transform produces an NA in the oob.err column for the 3rd tree
#NOTE: the real cause is that we did not do cross validation when there is no split.
"test.rxDTree.nosplit" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)

    ## set up arguments
  ## NOTE: make minsplit big enough so that the tree does not split at all.
  data.df <- iris
    xval <- 2
    control.dtree <- rpart.control(
        minsplit = 500, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = xval)      
    control.rpart   <- control.dtree
    control.rpart$xval <- if(xval!=0) rep(1:xval, length.out = nrow(data.df)) else 0
  
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  fit.rpart <- rpart(form, data = data.df, control = control.rpart
    )
  
  fit.dtree <- rxDTree(form, data = data.df, control = control.dtree,
    numThreads = 1,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

    #Bug 24410: incorrect value of DTree$valid.obs for rxDTree
    checkEquals(fit.dtree$valid.obs, nrow(data.df))
    checkEquals(fit.dtree$missing.obs, 0)

  ## check results
    checkEquals(fit.rpart$frame,    fit.dtree$frame, check.attributes = FALSE)
    checkEquals(fit.rpart$where,    fit.dtree$where, checkNames = FALSE)
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
  ## NOTE: fit.dtree$variable.importance is more consistent than fit.rpart$variable.importance
    #checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

#Bug 24410: incorrect value of DTree$valid.obs for rxDTree
"test.rxDTree.nosplit.xdf" <- function()
{
    form <- age ~ cost + number
    data.xdf <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    
    fit.dtree <- rxDTree(form, data = data.xdf,
            minSplit = 5, maxDepth = 2, xVal = 0, minbucket = 258,
            allowDiskWrite = TRUE,
            maxNumBins = 200, reportProgress = 2, verbose = 0)
        
    checkEquals(fit.dtree$valid.obs, nrow(data.xdf))
    checkEquals(fit.dtree$missing.obs, 0)
}

"test.rxDTree.outFile.nonempty" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  rxDataStep( data.xdf, outFile)
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
    )
  
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  where <- ".rxNode"
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), where) %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

#bigAnalytics RA-9912
#rxDTree error: When overwriting an existing column in an existing block in a DataFile, then number of rows cannot be increased
"test.rxDTree.outFile.nonempty.blocksPerRead.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
    
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  rxDataStep( data.xdf, outFile)
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control)
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    blocksPerRead = 2, writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  
  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  where <- ".rxNode"
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), where) %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

"test.rxDTree.outFile.empty" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
    )
  
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  where <- ".rxNode"
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(where %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

#bigAnalytics RA-10012
#rxDForest is ignoring "writeModelVars = TRUE" and not writing the model variables to its output file
"test.rxDTree.outFile.bSameInandOutFile.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if(file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- data.xdf

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
    )
  
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.	
  where <- ".rxNode"
  modelVars <- as.character(attr(terms(form), "variables"))[-1]
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), where, modelVars) %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

"test.rxDTree.outFile.nonempty.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
    
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  varsToKeep <- c("Sepal.Length", "Sepal.Width")
  rxDataStep( data.xdf, outFile, varsToKeep = varsToKeep)
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control)
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  
  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  where <- ".rxNode"
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(names(data.df), where, varsToKeep) %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

"test.rxDTree.outFile.empty.writeModelVars" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
    require(rpart)
  
  ## create input data
  data.df <- iris
  data.xdf <- tempfile("iris", , ".xdf") 
  
    if (rxIsExpressEdition())
    {
        rxDataStep(inData = data.df, outFile = data.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(data.df[1:100, ], data.xdf, overwrite = TRUE) 
      rxDataStep(data.df[101:150, ], data.xdf, append = "rows", overwrite = TRUE)
    }
  on.exit(if (file.exists(data.xdf)) file.remove(data.xdf), add = TRUE)
  
  ## create outFile
  outFile <- tempfile("rxDTree", , ".xdf")
  if (file.exists(outFile)) file.remove(outFile) 
  on.exit(if (file.exists(outFile)) file.remove(outFile), add = TRUE)

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

  ## fit the model
  iris.rpart <- rpart(form, data = data.df, control = control,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
    )
  
  iris.dtree <- rxDTree(form, data = data.xdf, control = control, outFile = outFile,
    writeModelVars = TRUE,
    #allowDiskWrite = TRUE, useWriteDataSource = 0,		# hidden args
    #reportProgress = 2, verbose = 3,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )

  ## check results
  ##NOTE: if useWriteDataSource > 0, 2 more cols (i.e., "___nodeid", ".rxRowSelection") will be generated.
  where <- ".rxNode"
  modelVars <- as.character(attr(terms(form), "variables"))[-1]
  outFile.df <- rxDataStep(outFile)
  checkTrue(all(c(where, modelVars) %in% names(outFile.df)))
  checkEquals(outFile.df[[where]], unname(iris.rpart$where))
}

#bigAnalytics RA-10017
#rxVarImpPlot( rxDForestObject ) does not show the variable names in the plot
"test.rxDTree.importance" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  
  ## create input data
  data.df <- iris

    ## set up arguments
  form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width	#classification
  #form <- Sepal.Length ~ Species + Sepal.Width + Petal.Length + Petal.Width	#regression

  ## fit the model
  iris.dtree <- rxDTree(form, data = data.df, maxDepth = 10,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )

  #library(rpart)
  #iris.rpart <- rpart(form, data = data.df, maxdepth = 10)
  #iris.importance.rpart <- iris.rpart$variable.importance
  
  ## check results
  iris.importance <- iris.dtree$variable.importance
  checkTrue(all(names( iris.importance ) %in% names(data.df)))
}

#bigAnalytics RA-9425
#rxDForest crashes when using removeMissings = TRUE and a subset of the Census5PCT2000.xdf data set
"test.rxDTree.regression.integerVar.bigRange" <- function()
{
  Census5PCT2000Sub <- 
    structure(
      list(
        inctot = c(43500L, 12100L, 9000L, 50000L, 4000L,
          12500L, 12000L, 85100L, 32100L, 40000L, 16000L, 28600L, 29700L,
          13600L, 15700L, 359000L, 15000L, 5300L, 1100L, 4150L, 9200L,
          6000L, 23800L, 18000L, 1200L, 22800L, 1820L, 261000L, 40600L,
          5000L, 2500L, 30000L, 30000L), 
        hhincome = c(64600L, 64600L, 64600L,
          54000L, 54000L, 46700L, 83800L, 117200L, 117200L, 59000L, 59000L,
          53600L, 71900L, 71900L, 71900L, 374000L, 374000L, 57400L, 57400L,
          11050L, 25200L, 25200L, 41800L, 41800L, 6700L, 64800L, 28820L,
          261000L, 85600L, 85600L, 34000L, 34000L, 30000L)
      ), 
      .Names = c("inctot", "hhincome"), 
      class = "data.frame", 
      row.names = c(138L, 139L,
        140L, 173L, 174L, 254L, 349L, 363L, 364L, 377L, 378L, 422L, 503L,
        504L, 509L, 540L, 541L, 552L, 553L, 570L, 578L, 579L, 615L, 616L,
        689L, 716L, 751L, 812L, 911L, 912L, 967L, 968L, 994L)
    )
  
  form <- inctot ~  hhincome
  Census.dtree <- rxDTree( form, data = Census5PCT2000Sub,
    minBucket = 5, maxDepth = 2, numThreads = 1,
    removeMissings = TRUE, useSparseCube = FALSE,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  
  ## just to make sure rxDTree runs OK
  checkTrue(TRUE)
}

#bigAnalytics RA-9688
#rxDTree with useSparseCube = TRUE gives different results with numCoresToUse = 4 than it does with 1, 2, or 3 cores
"test.rxDTree.classification.useSparseCube.emptyCube" <- function()
{
  #source(file.path(Sys.getenv("RXSVNROOT"), "revoAnalytics\\init.R"))
  require(rpart)
  
  data(fgl, package = "MASS")
  fgl.xdf <- tempfile(fileext = ".xdf")
  on.exit(if(file.exists(fgl.xdf)) file.remove(fgl.xdf), add = TRUE)

  ## 3-block xdf
    if (rxIsExpressEdition())
    {
        rxDataStep( inData = fgl, outFile = fgl.xdf, overwrite = TRUE)
    } else
    {
      rxDataStep(fgl[1:50,], fgl.xdf, overwrite = TRUE) 
      rxDataStep(fgl[51:150,], fgl.xdf, append="rows", overwrite = TRUE)
      rxDataStep(fgl[151:214,], fgl.xdf, append="rows", overwrite = TRUE) 
    }
    
  
  fgl.form <- type ~ Mg + Al + Ba	
  control <- rpart.control(minsplit = 75, maxcompete = 0, maxsurrogate = 0, maxdepth = 4, cp = 0, xval = 0)
  
  fgl.rpart <- rpart(fgl.form, data=fgl, control = control) 
  fgl.dtree <- rxDTree(fgl.form, data=fgl.xdf, control = control, 
    useSparseCube = TRUE, blocksPerRead = 1,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        ) 

  checkEquals(fgl.rpart$frame, fgl.dtree$frame, tolerance = 0.01) # node8 -> {node16, node17} was wrong
}

#bigAnalytics RA-9709
#rxPredict fails using tree pruned with prune.rxDTree: Does not find variable in the dataset
"test.rxDTree.classification.formula.string" <- function()
{
  ## string (as formula)
  form.str <- "Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width"
  dtree.iris.str <- rxDTree(form.str, iris,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  dtree.pred.str <- rxPredict(dtree.iris.str, data = iris, type = "class")	
  
  ## formula
  form.form <- formula(form.str)
  dtree.iris.form <- rxDTree(form.form, iris,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  dtree.pred.form <- rxPredict(dtree.iris.form, data = iris, type = "class")
  
  ## compare
  checkEquals(dtree.pred.str, dtree.pred.form)
}

"test.rxDTree.classification.nonxdf.sas" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    #RA-6807: rxDTree does not work wtih non-xdf data source
    require(rpart)
  
    ## make local data
    claimsSas   <- RxSasData(file = file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat"), 
        stringsAsFactors = TRUE)
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
        
    ## set up arguments
    form        <- age ~ cost + number
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsSas, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)	# the complexity of node 4 is different
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.regression.nonxdf.sas" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
    #RA-6807: rxDTree does not work wtih non-xdf data source
    require(rpart)
  
    ## make local data
    claimsSas   <- RxSasData(file = file.path(rxGetOption("sampleDataDir"), "claims.sas7bdat"), 
        stringsAsFactors = TRUE)
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
        
    ## set up arguments
    form        <- cost ~ age + number
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 10, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsSas, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)    # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])
    
    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.nonxdf.odbc.useFastRead.TRUE" <- function()
{
  #DEACTIVATED("Pending fix for RA-8122")
    require(rpart)
  
    ## make local data
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"),
                        useFastRead = TRUE),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  # Temporary conString
    #conString <- paste("Driver=SQLite3 ODBC Driver;Database=", file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), sep = "")
    claimsColInfo <- list( "type" =
                        list(type = "factor",
                            levels = c("A", "B", "C", "D")),	
              "age" =
            list(type = "factor",
              levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
            )		
          
  claimsOdbc <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                useFastRead = TRUE,
                connectionString = conString,
                colInfo = claimsColInfo,
                rowsPerRead = 32)
                            
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
  claimsOdbcDF <- rxImport(claimsOdbc)
  res <- try(checkEquals(nrow(claimsOdbcDF), 128))
  if (inherits(res, "try-error"))
  {
    message("Incorrect number of rows returned. Verify that you have a compatible SQLite ODBC driver (!=.92).")
  }
    ## set up arguments
    form        <- age ~ cost + number
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsOdbc, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)	# the complexity of node 4 is different
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
  
  # Redo with numcores set to 1
  numCoresOld <- rxGetOption("numCoresToUse")
  rxOptions(numCoresToUse=1)
  fit.dtree <- rxDTree(
        formula = form, data = claimsOdbc, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
  
  rxOptions(numCoresToUse = numCoresOld)
}

"test.rxDTree.classification.nonxdf.odbc.useFastRead.FALSE" <- function()
{
  #DEACTIVATED("Pending fix for RA-8122")
    require(rpart)
  
    ## make local data
  rxSqliteConnectionStringFromFile <- .rxSqliteConnectionStringFromFile()
  conString <- try(
          rxSqliteConnectionStringFromFile(file.path(rxGetOption("sampleDataDir"), "claims.sqlite"),
                        useFastRead = FALSE),
          silent = TRUE)
    if (inherits(conString, "try-error"))
        DEACTIVATED(strsplit(conString, "\n")[[1]][2])
    
  # Temporary conString
    # conString <- paste("Driver=SQLite3 ODBC Driver;Database=", file.path(rxGetOption("sampleDataDir"), "claims.sqlite"), sep = "")
    claimsColInfo <- list( "type" =
                        list(type = "factor",
                            levels = c("A", "B", "C", "D")),	
              "age" =
            list(type = "factor",
              levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
            )		
          
  claimsOdbc <- RxOdbcData(
        sqlQuery = "SELECT * FROM claims",
                useFastRead = FALSE,
                connectionString = conString,
                colInfo = claimsColInfo,
                rowsPerRead = 32)
                            
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
  claimsOdbcDF <- rxImport(claimsOdbc)
  res <- try(checkEquals(nrow(claimsOdbcDF), 128))
  if (inherits(res, "try-error"))
  {
    message("Incorrect number of rows returned. Verify that you have a compatible SQLite ODBC driver (!=.92).")
  }
    ## set up arguments
    form        <- age ~ cost + number
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsOdbc, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)	# the complexity of node 4 is different
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.nonxdf.txt.useFastRead.TRUE" <- function()
{
  #DEACTIVATED("Pending fix for RA-9298")
    require(rpart)
  
    ## make local data
    claimsColInfo <- list( "type" =
                        list(type = "factor",
                            levels = c("A", "B", "C", "D")),	
              "age" =
            list(type = "factor",
              levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
            )		

  claimsText <- RxTextData(
        file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
                useFastRead = TRUE,
                colInfo = claimsColInfo,
                rowsPerRead = 32)
                            
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))

    ## set up arguments
    form        <- age ~ cost + number
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsText, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
    )
  
    ## check results
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)	# the complexity of node 4 is different
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.nonxdf.txt.useFastRead.FALSE" <- function()
{
  #DEACTIVATED("Pending fix for RA-9298")
    require(rpart)
  
    ## make local data
    claimsColInfo <- list( "type" =
                        list(type = "factor",
                            levels = c("A", "B", "C", "D")),	
              "age" =
            list(type = "factor",
              levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
            )		

  claimsText <- RxTextData(
        file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
                useFastRead = FALSE,
                colInfo = claimsColInfo,
                rowsPerRead = 32)
                            
    claimsDF    <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))

    ## set up arguments
    form        <- age ~ cost + number
    control     <- rpart.control(
                    minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
                    maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsText, control = control, maxNumBins = 100,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    #checkEquals(fit.rpart$frame,    fit.dtree$frame)	# the complexity of node 4 is different
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    #checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.classification.nonxdf.txt.newLevels" <- function()
{
  #DEACTIVATED("Pending fix for RA-9298")
  #library(RevoScaleR)
    require(rpart)
  
    ## data
  newLevels <- c("a", "b", "c", "d")
    claimsColInfo <- list( 
    "type" = list(
      type = "factor",
      levels = c("A", "B", "C", "D"),
      newLevels = newLevels
      )
    )		

  claimsText <- RxTextData(
    file = file.path(rxGetOption("sampleDataDir"), "claims.txt"),
        colInfo = claimsColInfo)
                            
    claimsDF <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
  levels(claimsDF$type) <- newLevels

    ## set up arguments
    form <- cost ~ type
  
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)
    
    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree <- rxDTree(
        formula = form, data = claimsText, control = control, maxNumBins = 100,
    #allowDiskWrite = FALSE,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
    
    ## check results
    checkEquals(fit.rpart$frame,    fit.dtree$frame)
    #checkEquals(fit.rpart$where,    fit.dtree$where)   # fit.dtree$where is NULL if data is a XDF file
    
    # cross validation results might be different depending on 
    # the number of cores and the number of blocks in the data
    if(exists(".check.cptable") && .check.cptable) checkEquals(fit.rpart$cptable,  fit.dtree$cptable)
    xcol <- colnames(fit.rpart$cptable) %in% c("xerror", "xstd")
    checkEquals(fit.rpart$cptable[, !xcol],  fit.dtree$cptable[, !xcol])
    
    # "count" for surrogate splits are not updated for now
    countcol <- grep("count", colnames(fit.rpart$split))
    checkEquals(fit.rpart$splits[, -countcol],   fit.dtree$splits[, -countcol])
    
    checkEquals(fit.rpart$csplit,   fit.dtree$csplit)
    checkEquals(fit.rpart$method,   fit.dtree$method)
    checkEquals(fit.rpart$parms,    fit.dtree$parms)
    
    # "xval" for control are different for now
    checkEquals(fit.rpart$control,  fit.dtree$control)    
    xvalelem <- grep("xval", names(fit.rpart$control))
    checkEquals(fit.rpart$control[-xvalelem],   fit.dtree$control[-xvalelem])

    checkEquals(fit.rpart$ordered,  fit.dtree$ordered)
  
    checkEquals(fit.rpart$variable.importance,  fit.dtree$variable.importance)

    checkEquals(attr(fit.rpart, "xlevels"),  attr(fit.dtree, "xlevels"))
    checkEquals(attr(fit.rpart, "ylevels"),  attr(fit.dtree, "ylevels"))
}

"test.rxDTree.regression.nonxdf.txt.useFastRead.TRUE" <- function()
{
    #DEACTIVATED("Bug 24854: [Local cc + DTree] rxDTree fails on RxTextData in local compute context")

  #library(RevoScaleR)
    require(rpart)

    ## make local data
    useFastRead <- TRUE
    yvar <- "cost"
    xvars <- c("number", "age")
    sub <- c(yvar, xvars)

    claimsColInfo <- list(
        "type" = list(type = "factor", levels = c("A", "B", "C", "D")),
        "age" = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
        )

    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(file = claimsTxt, useFastRead = useFastRead, colInfo = claimsColInfo, rowsPerRead = 32)

    claimsDF <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))

    claimsTxtSub <- tempfile(pattern = "claimsTxtSub", fileext = ".txt")
    on.exit(if (file.exists(claimsTxtSub)) file.remove(claimsTxtSub), add = TRUE)
    claimsTxtSubDS <- RxTextData(file = claimsTxtSub, useFastRead = useFastRead, colInfo = claimsColInfo, rowsPerRead = 32)
    rxDataStep(claimsDF[sub], claimsTxtSubDS, overwrite = TRUE)

    claimsXdfSub <- tempfile(pattern = "claimsXdfSub", fileext = ".xdf")
    on.exit(if (file.exists(claimsXdfSub)) file.remove(claimsXdfSub), add = TRUE)
    claimsXdfSubDs <- RxXdfData(file = claimsXdfSub)
    rxDataStep(claimsDF[sub], claimsXdfSubDs, overwrite = TRUE)

    ## set up arguments
    form <- paste(yvar, paste(xvars, collapse = " + "), sep = " ~ ")
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree.txt <- rxDTree(
        formula = form, data = claimsTxtSubDS, control = control
        #, reportProgress = 2, verbose = 3
        )
    fit.dtree.xdf <- rxDTree(
        formula = form, data = claimsXdfSubDs, control = control
        #, reportProgress = 2, verbose = 3
        )

    ## compare the tree built on txt and xdf file respectively
    dontmatch <- c("params", "call")
    domatch <- setdiff(names(fit.dtree.xdf), dontmatch)
    #sapply(domatch, function(nm) all.equal(fit.dtree.xdf[[nm]], fit.dtree.txt[[nm]]))
    sapply(domatch, function(nm) checkIdentical(fit.dtree.xdf[[nm]], fit.dtree.txt[[nm]]))
}

"test.rxDTree.regression.nonxdf.txt.useFastRead.FALSE" <- function()
{
  #library(RevoScaleR)
    require(rpart)

    ## make local data
    useFastRead <- FALSE
    yvar <- "cost"
    xvars <- c("number", "age")
    sub <- c(yvar, xvars, "type")   #add an unused var

    claimsColInfo <- list(
        "type" = list(type = "factor", levels = c("A", "B", "C", "D")),
        "age" = list(type = "factor", levels = c("17-20", "21-24", "25-29", "30-34", "35-39", "40-49", "50-59", "60+"))
        )

    claimsTxt <- file.path(rxGetOption("sampleDataDir"), "claims.txt")
    claimsTxtDs <- RxTextData(file = claimsTxt, useFastRead = useFastRead, colInfo = claimsColInfo, rowsPerRead = 32)

    claimsDF <- rxReadXdf(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))

    claimsTxtSub <- tempfile(pattern = "claimsTxtSub", fileext = ".txt")
    on.exit(if (file.exists(claimsTxtSub)) file.remove(claimsTxtSub), add = TRUE)
    claimsTxtSubDS <- RxTextData(file = claimsTxtSub, useFastRead = TRUE, colInfo = claimsColInfo, rowsPerRead = 32) #'useFastRead' must be TRUE in order to write to a text data source.
    rxDataStep(claimsDF[sub], claimsTxtSubDS, overwrite = TRUE) 
    claimsTxtSubDS@useFastRead <- useFastRead #change useFastRead

    claimsXdfSub <- tempfile(pattern = "claimsXdfSub", fileext = ".xdf")
    on.exit(if (file.exists(claimsXdfSub)) file.remove(claimsXdfSub), add = TRUE)
    claimsXdfSubDs <- RxXdfData(file = claimsXdfSub)
    rxDataStep(claimsDF[sub], claimsXdfSubDs, overwrite = TRUE)

    ## set up arguments
    form <- paste(yvar, paste(xvars, collapse = " + "), sep = " ~ ")
    control <- rpart.control(
        minsplit = 5, minbucket = 2, cp = 0.01, maxdepth = 4, maxcompete = 4,
        maxsurrogate = 5, usesurrogate = 2, surrogatestyle = 0, xval = 0)

    ## build the models
    fit.rpart <- rpart(
        formula = form, data = claimsDF, control = control)
    fit.dtree.txt <- rxDTree(
        formula = form, data = claimsTxtSubDS, control = control
        #, reportProgress = 2, verbose = 3
        )
    fit.dtree.xdf <- rxDTree(
        formula = form, data = claimsXdfSubDs, control = control
        #, reportProgress = 2, verbose = 3
        )

    ## compare the tree built on txt and xdf file respectively
    dontmatch <- c("params", "call")
    domatch <- setdiff(names(fit.dtree.xdf), dontmatch)
    #sapply(domatch, function(nm) all.equal(fit.dtree.xdf[[nm]], fit.dtree.txt[[nm]]))
    sapply(domatch, function(nm) checkIdentical(fit.dtree.xdf[[nm]], fit.dtree.txt[[nm]]))
}

"test.rxDTree.rxPredict.regression.missingmodelObjArg" <- function()
{
  checkException(rxPredict())
}

"test.rxDTree.rxPredict.regression.missingDataArg" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
  myIris <- iris
  myIris[1:5,]
  form <- Sepal.Length ~ Sepal.Width + Petal.Length + Petal.Width + Species
  irisLinMod <- rxLinMod(form, data = myIris)
  checkException(myIrisPred <- rxPredict(modelObject = irisLinMod))
  
  data( "car.test.frame", package = "rpart" )
  z.auto <- rxDTree( Mileage ~ Weight, car.test.frame,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  checkException(rxPredict( z.auto ) )
}

"test.rxDTree.minSplitDefault.arg" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
  require("rpart")
  kyphTree <- rxDTree(Kyphosis ~ Age + Start + Number, data=kyphosis,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  checkTrue(all(kyphTree$frame$n >= 7))
  mtcarTree <- rxDTree(mpg ~ disp, data=mtcars,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  checkTrue(all(mtcarTree$frame$n >= 7))
}

"test.rxDTree.rxAddInheritance.dispatch" <- function()
{
  if(exists(".deactivate.rxDTree.tests") && .deactivate.rxDTree.tests) DEACTIVATED("Change in behavior for rpart")
  require("rpart")
  kyphTree <- rxDTree(Kyphosis ~ Age + Start + Number, data=kyphosis,
        maxRowsInMemory = if(exists(".maxRowsInMemory")) .maxRowsInMemory else -1,
        )
  kyphRpart <- rxAddInheritance(kyphTree)
  res1 <- try(printcp(kyphRpart), silent=TRUE)
  checkTrue(!is(res1,"try-error"))
  res2 <- try(plot(kyphRpart), silent=TRUE)
  checkTrue(!is(res2, "try-error"))
}

# Real-time scoring
"test.rxDTree.rts" <- function()
{
    ids <- file.path(rxGetOption("sampleDataDir"), "CensusWorkers.xdf")
    colInfo = list(sex = list(type="factor", levels=c("Female", "Male")))
    censusWorkers = rxImport(ids)
 
    # pick the data to score on randomly   
    testIndices <- sample(1:NROW(censusWorkers), 13)
    testData <- censusWorkers[testIndices,]
    
    # Exclude testData from trainData
    trainData <- censusWorkers[setdiff(1:NROW(censusWorkers), testIndices),]
    # Reduce the size of trainData to speed things up
    trainData <- trainData[sample(1:NROW(trainData), 10000),]
    
    formula = incwage ~ age + sex + wkswork1
    
    # Train the model
    model <- rxDTree(formula, pweights = "perwt", maxDepth = 3, minBucket = 30, data = trainData)
    
    # Prdict
    pred1 <- rxPredict(model, testData)
    
    # Obtain the binary model
    model$binary <- rxSerializeModel(model)
    
    # Convert factors to strings. Real-time scoring takes care of the factorization.
    factorColumns <- sapply(testData, is.factor)
    testData[factorColumns] <- lapply(testData[factorColumns], as.character)
    
    # Predict using the binary model
    output <- capture.output(pred2 <- rxPredict(model, testData))

    # Verify we went through the native scoring path    
    checkEquals(1, sum(grepl("Native prediction succeeded", output)), "Check default rxPredict with binary model gives native prediction in output.")
    
    checkIdentical(pred1, pred2)
}

"test.rxPredict.rxDTree.parameters" <- function()
{
    predVarNames <- "mtCars_out"
    checkEquals(rxGetVarNames(rxPredict(rxDTree(am ~ cyl + mpg, data = mtcars), mtcars, predVarNames = predVarNames)), predVarNames)
}

"test.rxDTree.fileSystem" <- function()
{
    #library(RevoScaleR)
    set.seed(1234)

    "run_trees" <- function(form, data)
    {
        dtree <- rxDTree(form, data)
        dforest <- rxDForest(form, data, nTree = 3)
        btrees <- rxBTrees(form, data, nTree = 3,
            lossFunction = "multinomial", learningRate = 0.1)
    }

    ## data.frame
    data <- iris
    form <- Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width
  
    rxOptions(fileSystem = RxNativeFileSystem())
    run_trees(form, data)

    rxOptions(fileSystem = RxHdfsFileSystem())
    run_trees(form, data)

    ## xdf
    data <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "claims.xdf"), fileSystem = RxNativeFileSystem())
    form <- age ~ type + cost + number

    rxOptions(fileSystem = RxNativeFileSystem())
    run_trees(form, data)

    rxOptions(fileSystem = RxHdfsFileSystem())
    run_trees(form, data)

    checkTrue(TRUE)
}

"test.rxDTree.dataframe.transform.formula" <- function()
{
    #library(RevoScaleR)
    #rxOptions(numCoresToUse = 1)

    data_path = file.path(rxGetOption("sampleDataDir"), "kyphosis.xdf")
    data_ds = RxXdfData(data_path)
    data_df = rxImport(data_ds)

    transform_func <- function(data) data
    real_form <- Kyphosis ~ Age + Start
    stri_form <- deparse(real_form)

    dtree_rform <- rxDTree(real_form, data_df, transformFunc=transform_func)
    dtree_sform <- rxDTree(stri_form, data_df, transformFunc=transform_func) #failed
    checkEquals(dtree_rform$frame, dtree_sform$frame)

    seed <- 1234
    nTree <- 3
    maxDepth <- 2

    dforest_rform <- rxDForest(real_form, data_df, transformFunc=transform_func, 
        nTree = nTree, maxDepth = maxDepth, seed = seed)
    dforest_sform <- rxDForest(stri_form, data_df, transformFunc=transform_func, 
        nTree = nTree, maxDepth = maxDepth, seed = seed) #failed
    checkEquals(dforest_rform$forest[[1]]$frame, dforest_sform$forest[[1]]$frame)

    btrees_rform <- rxBTrees(real_form, data_df, transformFunc=transform_func, 
        nTree = nTree, maxDepth = maxDepth, seed = seed)
    btrees_sform <- rxBTrees(stri_form, data_df, transformFunc=transform_func, 
        nTree = nTree, maxDepth = maxDepth, seed = seed) #failed
    checkEquals(btrees_rform$forest[[1]]$frame, btrees_sform$forest[[1]]$frame)
}
